<?php

namespace Spatie\FlareClient\Enums;

class MessageLevels
{
    const INFO = 'info';
    const DEBUG = 'debug';
    const WARNING = 'warning';
    const ERROR = 'error';
    const CRITICAL = 'critical';
}
