<?php

// autoload_classmap.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'App\\Console\\Commands\\AutoApprovePaymentCommand' => $baseDir . '/app/Console/Commands/AutoApprovePaymentCommand.php',
    'App\\Console\\Commands\\MarkRewardExpiredCommand' => $baseDir . '/app/Console/Commands/MarkRewardExpiredCommand.php',
    'App\\Console\\Commands\\TestCommand' => $baseDir . '/app/Console/Commands/TestCommand.php',
    'App\\Console\\Kernel' => $baseDir . '/app/Console/Kernel.php',
    'App\\Constants\\FileInfo' => $baseDir . '/app/Constants/FileInfo.php',
    'App\\Constants\\Status' => $baseDir . '/app/Constants/Status.php',
    'App\\Exceptions\\Handler' => $baseDir . '/app/Exceptions/Handler.php',
    'App\\Exports\\ActiveMembersExport' => $baseDir . '/app/Exports/ActiveMembersExport.php',
    'App\\Exports\\AdminRewardExport' => $baseDir . '/app/Exports/AdminRewardExport.php',
    'App\\Exports\\CouponExport' => $baseDir . '/app/Exports/CouponExport.php',
    'App\\Exports\\DebtFreeExport' => $baseDir . '/app/Exports/DebtFreeExport.php',
    'App\\Exports\\DebtNotFreeExport' => $baseDir . '/app/Exports/DebtNotFreeExport.php',
    'App\\Exports\\DonationListExport' => $baseDir . '/app/Exports/DonationListExport.php',
    'App\\Exports\\ExportTransactionReport' => $baseDir . '/app/Exports/ExportTransactionReport.php',
    'App\\Exports\\InactiveMemberExport' => $baseDir . '/app/Exports/InactiveMemberExport.php',
    'App\\Exports\\MyRewardsExport' => $baseDir . '/app/Exports/MyRewardsExport.php',
    'App\\Exports\\NetworkExport' => $baseDir . '/app/Exports/NetworkExport.php',
    'App\\Exports\\PartiallyActiveMembersExport' => $baseDir . '/app/Exports/PartiallyActiveMembersExport.php',
    'App\\Exports\\RewardsExport' => $baseDir . '/app/Exports/RewardsExport.php',
    'App\\Exports\\TransactionsExport' => $baseDir . '/app/Exports/TransactionsExport.php',
    'App\\Exports\\UserAppDownloadExport' => $baseDir . '/app/Exports/UserAppDownloadExport.php',
    'App\\Exports\\UserExport' => $baseDir . '/app/Exports/UserExport.php',
    'App\\Http\\Controllers\\Admin\\AdminController' => $baseDir . '/app/Http/Controllers/Admin/AdminController.php',
    'App\\Http\\Controllers\\Admin\\Auth\\ForgotPasswordController' => $baseDir . '/app/Http/Controllers/Admin/Auth/ForgotPasswordController.php',
    'App\\Http\\Controllers\\Admin\\Auth\\LoginController' => $baseDir . '/app/Http/Controllers/Admin/Auth/LoginController.php',
    'App\\Http\\Controllers\\Admin\\Auth\\ResetPasswordController' => $baseDir . '/app/Http/Controllers/Admin/Auth/ResetPasswordController.php',
    'App\\Http\\Controllers\\Admin\\AutomaticGatewayController' => $baseDir . '/app/Http/Controllers/Admin/AutomaticGatewayController.php',
    'App\\Http\\Controllers\\Admin\\CouponController' => $baseDir . '/app/Http/Controllers/Admin/CouponController.php',
    'App\\Http\\Controllers\\Admin\\DepositController' => $baseDir . '/app/Http/Controllers/Admin/DepositController.php',
    'App\\Http\\Controllers\\Admin\\ExtensionController' => $baseDir . '/app/Http/Controllers/Admin/ExtensionController.php',
    'App\\Http\\Controllers\\Admin\\FrontendController' => $baseDir . '/app/Http/Controllers/Admin/FrontendController.php',
    'App\\Http\\Controllers\\Admin\\GeneralSettingController' => $baseDir . '/app/Http/Controllers/Admin/GeneralSettingController.php',
    'App\\Http\\Controllers\\Admin\\KycController' => $baseDir . '/app/Http/Controllers/Admin/KycController.php',
    'App\\Http\\Controllers\\Admin\\LanguageController' => $baseDir . '/app/Http/Controllers/Admin/LanguageController.php',
    'App\\Http\\Controllers\\Admin\\LevelController' => $baseDir . '/app/Http/Controllers/Admin/LevelController.php',
    'App\\Http\\Controllers\\Admin\\ManageUsersController' => $baseDir . '/app/Http/Controllers/Admin/ManageUsersController.php',
    'App\\Http\\Controllers\\Admin\\ManualGatewayController' => $baseDir . '/app/Http/Controllers/Admin/ManualGatewayController.php',
    'App\\Http\\Controllers\\Admin\\NoticeBoardController' => $baseDir . '/app/Http/Controllers/Admin/NoticeBoardController.php',
    'App\\Http\\Controllers\\Admin\\NotificationController' => $baseDir . '/app/Http/Controllers/Admin/NotificationController.php',
    'App\\Http\\Controllers\\Admin\\PageBuilderController' => $baseDir . '/app/Http/Controllers/Admin/PageBuilderController.php',
    'App\\Http\\Controllers\\Admin\\PinController' => $baseDir . '/app/Http/Controllers/Admin/PinController.php',
    'App\\Http\\Controllers\\Admin\\PlanController' => $baseDir . '/app/Http/Controllers/Admin/PlanController.php',
    'App\\Http\\Controllers\\Admin\\ReportController' => $baseDir . '/app/Http/Controllers/Admin/ReportController.php',
    'App\\Http\\Controllers\\Admin\\RewardController' => $baseDir . '/app/Http/Controllers/Admin/RewardController.php',
    'App\\Http\\Controllers\\Admin\\SocialMediaController' => $baseDir . '/app/Http/Controllers/Admin/SocialMediaController.php',
    'App\\Http\\Controllers\\Admin\\SpatiePermissionController' => $baseDir . '/app/Http/Controllers/Admin/SpatiePermissionController.php',
    'App\\Http\\Controllers\\Admin\\SubscriberController' => $baseDir . '/app/Http/Controllers/Admin/SubscriberController.php',
    'App\\Http\\Controllers\\Admin\\SupportTicketController' => $baseDir . '/app/Http/Controllers/Admin/SupportTicketController.php',
    'App\\Http\\Controllers\\Admin\\SystemController' => $baseDir . '/app/Http/Controllers/Admin/SystemController.php',
    'App\\Http\\Controllers\\Admin\\WithdrawMethodController' => $baseDir . '/app/Http/Controllers/Admin/WithdrawMethodController.php',
    'App\\Http\\Controllers\\Admin\\WithdrawalController' => $baseDir . '/app/Http/Controllers/Admin/WithdrawalController.php',
    'App\\Http\\Controllers\\Controller' => $baseDir . '/app/Http/Controllers/Controller.php',
    'App\\Http\\Controllers\\Gateway\\Authorize\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Authorize/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Blockchain\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Blockchain/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Cashmaal\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Cashmaal/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\CoinbaseCommerce\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/CoinbaseCommerce/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Coingate\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Coingate/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\CoinpaymentsFiat\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/CoinpaymentsFiat/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Coinpayments\\CoinPaymentHosted' => $baseDir . '/app/Http/Controllers/Gateway/Coinpayments/CoinPaymentHosted.php',
    'App\\Http\\Controllers\\Gateway\\Coinpayments\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Coinpayments/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Flutterwave\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Flutterwave/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Instamojo\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Instamojo/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\MercadoPago\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/MercadoPago/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Mollie\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Mollie/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\NMI\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/NMI/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Payeer\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Payeer/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\PaymentController' => $baseDir . '/app/Http/Controllers/Gateway/PaymentController.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\AccessToken' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/AccessToken.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\AccessTokenRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/AccessTokenRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\AuthorizationInjector' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/AuthorizationInjector.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\FPTIInstrumentationInjector' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/FPTIInstrumentationInjector.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\GzipInjector' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/GzipInjector.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\PayPalEnvironment' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/PayPalEnvironment.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\PayPalHttpClient' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/PayPalHttpClient.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\ProductionEnvironment' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/ProductionEnvironment.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\RefreshTokenRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/RefreshTokenRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\SandboxEnvironment' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/SandboxEnvironment.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\UserAgent' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/UserAgent.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Core\\Version' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Core/Version.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Orders\\OrdersAuthorizeRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Orders/OrdersAuthorizeRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Orders\\OrdersCaptureRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Orders/OrdersCaptureRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Orders\\OrdersCreateRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Orders/OrdersCreateRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Orders\\OrdersGetRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Orders/OrdersGetRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Orders\\OrdersPatchRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Orders/OrdersPatchRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Orders\\OrdersValidateRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Orders/OrdersValidateRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Curl' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Curl.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Encoder' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Encoder.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Environment' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Environment.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\HttpClient' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/HttpClient.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\HttpException' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/HttpException.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\HttpRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/HttpRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\HttpResponse' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/HttpResponse.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\IOException' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/IOException.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Injector' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Injector.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Serializer' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Serializer.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Serializer\\Form' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Serializer/Form.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Serializer\\FormPart' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Serializer/FormPart.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Serializer\\Json' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Serializer/Json.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Serializer\\Multipart' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Serializer/Multipart.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\PayPalHttp\\Serializer\\Text' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/PayPalHttp/Serializer/Text.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Payments\\AuthorizationsCaptureRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Payments/AuthorizationsCaptureRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Payments\\AuthorizationsGetRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Payments/AuthorizationsGetRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Payments\\AuthorizationsReauthorizeRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Payments/AuthorizationsReauthorizeRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Payments\\AuthorizationsVoidRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Payments/AuthorizationsVoidRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Payments\\CapturesGetRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Payments/CapturesGetRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Payments\\CapturesRefundRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Payments/CapturesRefundRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\Payments\\RefundsGetRequest' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/Payments/RefundsGetRequest.php',
    'App\\Http\\Controllers\\Gateway\\PaypalSdk\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/PaypalSdk/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Paypal\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Paypal/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Paystack\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Paystack/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Paytm\\PayTM' => $baseDir . '/app/Http/Controllers/Gateway/Paytm/PayTM.php',
    'App\\Http\\Controllers\\Gateway\\Paytm\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Paytm/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\PerfectMoney\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/PerfectMoney/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Razorpay\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Razorpay/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Skrill\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Skrill/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\StripeJs\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/StripeJs/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\StripeV3\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/StripeV3/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Stripe\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Stripe/ProcessController.php',
    'App\\Http\\Controllers\\Gateway\\Voguepay\\ProcessController' => $baseDir . '/app/Http/Controllers/Gateway/Voguepay/ProcessController.php',
    'App\\Http\\Controllers\\PlanController' => $baseDir . '/app/Http/Controllers/PlanController.php',
    'App\\Http\\Controllers\\SiteController' => $baseDir . '/app/Http/Controllers/SiteController.php',
    'App\\Http\\Controllers\\TicketController' => $baseDir . '/app/Http/Controllers/TicketController.php',
    'App\\Http\\Controllers\\User\\Auth\\ForgotPasswordController' => $baseDir . '/app/Http/Controllers/User/Auth/ForgotPasswordController.php',
    'App\\Http\\Controllers\\User\\Auth\\LoginController' => $baseDir . '/app/Http/Controllers/User/Auth/LoginController.php',
    'App\\Http\\Controllers\\User\\Auth\\RegisterController' => $baseDir . '/app/Http/Controllers/User/Auth/RegisterController.php',
    'App\\Http\\Controllers\\User\\Auth\\ResetPasswordController' => $baseDir . '/app/Http/Controllers/User/Auth/ResetPasswordController.php',
    'App\\Http\\Controllers\\User\\AuthorizationController' => $baseDir . '/app/Http/Controllers/User/AuthorizationController.php',
    'App\\Http\\Controllers\\User\\EpinController' => $baseDir . '/app/Http/Controllers/User/EpinController.php',
    'App\\Http\\Controllers\\User\\PaymentsController' => $baseDir . '/app/Http/Controllers/User/PaymentsController.php',
    'App\\Http\\Controllers\\User\\ProfileController' => $baseDir . '/app/Http/Controllers/User/ProfileController.php',
    'App\\Http\\Controllers\\User\\UserController' => $baseDir . '/app/Http/Controllers/User/UserController.php',
    'App\\Http\\Controllers\\User\\WithdrawController' => $baseDir . '/app/Http/Controllers/User/WithdrawController.php',
    'App\\Http\\Kernel' => $baseDir . '/app/Http/Kernel.php',
    'App\\Http\\Middleware\\AdminActiveMiddleware' => $baseDir . '/app/Http/Middleware/AdminActiveMiddleware.php',
    'App\\Http\\Middleware\\AllowRegistration' => $baseDir . '/app/Http/Middleware/AllowRegistration.php',
    'App\\Http\\Middleware\\Authenticate' => $baseDir . '/app/Http/Middleware/Authenticate.php',
    'App\\Http\\Middleware\\CheckForMaintenanceMode' => $baseDir . '/app/Http/Middleware/CheckForMaintenanceMode.php',
    'App\\Http\\Middleware\\CheckStatus' => $baseDir . '/app/Http/Middleware/CheckStatus.php',
    'App\\Http\\Middleware\\Demo' => $baseDir . '/app/Http/Middleware/Demo.php',
    'App\\Http\\Middleware\\EncryptCookies' => $baseDir . '/app/Http/Middleware/EncryptCookies.php',
    'App\\Http\\Middleware\\KycMiddleware' => $baseDir . '/app/Http/Middleware/KycMiddleware.php',
    'App\\Http\\Middleware\\LanguageMiddleware' => $baseDir . '/app/Http/Middleware/LanguageMiddleware.php',
    'App\\Http\\Middleware\\MaintenanceMode' => $baseDir . '/app/Http/Middleware/MaintenanceMode.php',
    'App\\Http\\Middleware\\PreventRequestsDuringMaintenance' => $baseDir . '/app/Http/Middleware/PreventRequestsDuringMaintenance.php',
    'App\\Http\\Middleware\\RedirectIfAdmin' => $baseDir . '/app/Http/Middleware/RedirectIfAdmin.php',
    'App\\Http\\Middleware\\RedirectIfAuthenticated' => $baseDir . '/app/Http/Middleware/RedirectIfAuthenticated.php',
    'App\\Http\\Middleware\\RedirectIfNotAdmin' => $baseDir . '/app/Http/Middleware/RedirectIfNotAdmin.php',
    'App\\Http\\Middleware\\RegistrationStep' => $baseDir . '/app/Http/Middleware/RegistrationStep.php',
    'App\\Http\\Middleware\\TrimStrings' => $baseDir . '/app/Http/Middleware/TrimStrings.php',
    'App\\Http\\Middleware\\TrustHosts' => $baseDir . '/app/Http/Middleware/TrustHosts.php',
    'App\\Http\\Middleware\\TrustProxies' => $baseDir . '/app/Http/Middleware/TrustProxies.php',
    'App\\Http\\Middleware\\VerifyCsrfToken' => $baseDir . '/app/Http/Middleware/VerifyCsrfToken.php',
    'App\\Lib\\Captcha' => $baseDir . '/app/Lib/Captcha.php',
    'App\\Lib\\ClientInfo' => $baseDir . '/app/Lib/ClientInfo.php',
    'App\\Lib\\CurlRequest' => $baseDir . '/app/Lib/CurlRequest.php',
    'App\\Lib\\FileManager' => $baseDir . '/app/Lib/FileManager.php',
    'App\\Lib\\FormProcessor' => $baseDir . '/app/Lib/FormProcessor.php',
    'App\\Lib\\GoogleAuthenticator' => $baseDir . '/app/Lib/GoogleAuthenticator.php',
    'App\\Lib\\Matrix' => $baseDir . '/app/Lib/Matrix.php',
    'App\\Lib\\RewardLib' => $baseDir . '/app/Lib/RewardLib.php',
    'App\\Lib\\Sms' => $baseDir . '/app/Lib/Sms.php',
    'App\\Lib\\UserNetwork' => $baseDir . '/app/Lib/UserNetwork.php',
    'App\\Models\\Admin' => $baseDir . '/app/Models/Admin.php',
    'App\\Models\\AdminNoteList' => $baseDir . '/app/Models/AdminNoteList.php',
    'App\\Models\\AdminNotification' => $baseDir . '/app/Models/AdminNotification.php',
    'App\\Models\\AdminPasswordReset' => $baseDir . '/app/Models/AdminPasswordReset.php',
    'App\\Models\\BankDetail' => $baseDir . '/app/Models/BankDetail.php',
    'App\\Models\\Banner' => $baseDir . '/app/Models/Banner.php',
    'App\\Models\\Commission' => $baseDir . '/app/Models/Commission.php',
    'App\\Models\\Coupon' => $baseDir . '/app/Models/Coupon.php',
    'App\\Models\\Deposit' => $baseDir . '/app/Models/Deposit.php',
    'App\\Models\\Extension' => $baseDir . '/app/Models/Extension.php',
    'App\\Models\\Form' => $baseDir . '/app/Models/Form.php',
    'App\\Models\\Frontend' => $baseDir . '/app/Models/Frontend.php',
    'App\\Models\\Gateway' => $baseDir . '/app/Models/Gateway.php',
    'App\\Models\\GatewayCurrency' => $baseDir . '/app/Models/GatewayCurrency.php',
    'App\\Models\\GeneralSetting' => $baseDir . '/app/Models/GeneralSetting.php',
    'App\\Models\\Language' => $baseDir . '/app/Models/Language.php',
    'App\\Models\\Level' => $baseDir . '/app/Models/Level.php',
    'App\\Models\\NoticeBoard' => $baseDir . '/app/Models/NoticeBoard.php',
    'App\\Models\\NotificationLog' => $baseDir . '/app/Models/NotificationLog.php',
    'App\\Models\\NotificationTemplate' => $baseDir . '/app/Models/NotificationTemplate.php',
    'App\\Models\\Page' => $baseDir . '/app/Models/Page.php',
    'App\\Models\\PasswordReset' => $baseDir . '/app/Models/PasswordReset.php',
    'App\\Models\\Payment' => $baseDir . '/app/Models/Payment.php',
    'App\\Models\\Permission' => $baseDir . '/app/Models/Permission.php',
    'App\\Models\\Pin' => $baseDir . '/app/Models/Pin.php',
    'App\\Models\\Plan' => $baseDir . '/app/Models/Plan.php',
    'App\\Models\\PlanPurchase' => $baseDir . '/app/Models/PlanPurchase.php',
    'App\\Models\\Reward' => $baseDir . '/app/Models/Reward.php',
    'App\\Models\\Role' => $baseDir . '/app/Models/Role.php',
    'App\\Models\\SocialMedia' => $baseDir . '/app/Models/SocialMedia.php',
    'App\\Models\\Subscriber' => $baseDir . '/app/Models/Subscriber.php',
    'App\\Models\\SupportAttachment' => $baseDir . '/app/Models/SupportAttachment.php',
    'App\\Models\\SupportMessage' => $baseDir . '/app/Models/SupportMessage.php',
    'App\\Models\\SupportTicket' => $baseDir . '/app/Models/SupportTicket.php',
    'App\\Models\\Transaction' => $baseDir . '/app/Models/Transaction.php',
    'App\\Models\\User' => $baseDir . '/app/Models/User.php',
    'App\\Models\\UserLogin' => $baseDir . '/app/Models/UserLogin.php',
    'App\\Models\\UserNotification' => $baseDir . '/app/Models/UserNotification.php',
    'App\\Models\\WithdrawMethod' => $baseDir . '/app/Models/WithdrawMethod.php',
    'App\\Models\\Withdrawal' => $baseDir . '/app/Models/Withdrawal.php',
    'App\\Notify\\Email' => $baseDir . '/app/Notify/Email.php',
    'App\\Notify\\Notifiable' => $baseDir . '/app/Notify/Notifiable.php',
    'App\\Notify\\Notify' => $baseDir . '/app/Notify/Notify.php',
    'App\\Notify\\NotifyProcess' => $baseDir . '/app/Notify/NotifyProcess.php',
    'App\\Notify\\Sms' => $baseDir . '/app/Notify/Sms.php',
    'App\\Notify\\SmsGateway' => $baseDir . '/app/Notify/SmsGateway.php',
    'App\\Observers\\ModelTimezoneObserver' => $baseDir . '/app/Observers/ModelTimezoneObserver.php',
    'App\\Providers\\AppServiceProvider' => $baseDir . '/app/Providers/AppServiceProvider.php',
    'App\\Providers\\AuthServiceProvider' => $baseDir . '/app/Providers/AuthServiceProvider.php',
    'App\\Providers\\BroadcastServiceProvider' => $baseDir . '/app/Providers/BroadcastServiceProvider.php',
    'App\\Providers\\EventServiceProvider' => $baseDir . '/app/Providers/EventServiceProvider.php',
    'App\\Providers\\RouteServiceProvider' => $baseDir . '/app/Providers/RouteServiceProvider.php',
    'App\\Rules\\FileTypeValidate' => $baseDir . '/app/Rules/FileTypeValidate.php',
    'App\\Traits\\GlobalStatus' => $baseDir . '/app/Traits/GlobalStatus.php',
    'App\\Traits\\Searchable' => $baseDir . '/app/Traits/Searchable.php',
    'App\\Traits\\SupportTicketManager' => $baseDir . '/app/Traits/SupportTicketManager.php',
    'App\\View\\Components\\Captcha' => $baseDir . '/app/View/Components/Captcha.php',
    'App\\View\\Components\\ConfirmationModal' => $baseDir . '/app/View/Components/ConfirmationModal.php',
    'App\\View\\Components\\FormGenerator' => $baseDir . '/app/View/Components/FormGenerator.php',
    'App\\View\\Components\\FormModal' => $baseDir . '/app/View/Components/FormModal.php',
    'App\\View\\Components\\ViserForm' => $baseDir . '/app/View/Components/ViserForm.php',
    'Attribute' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/Attribute.php',
    'Barryvdh\\Debugbar\\Console\\ClearCommand' => $vendorDir . '/barryvdh/laravel-debugbar/src/Console/ClearCommand.php',
    'Barryvdh\\Debugbar\\Controllers\\AssetController' => $vendorDir . '/barryvdh/laravel-debugbar/src/Controllers/AssetController.php',
    'Barryvdh\\Debugbar\\Controllers\\BaseController' => $vendorDir . '/barryvdh/laravel-debugbar/src/Controllers/BaseController.php',
    'Barryvdh\\Debugbar\\Controllers\\CacheController' => $vendorDir . '/barryvdh/laravel-debugbar/src/Controllers/CacheController.php',
    'Barryvdh\\Debugbar\\Controllers\\OpenHandlerController' => $vendorDir . '/barryvdh/laravel-debugbar/src/Controllers/OpenHandlerController.php',
    'Barryvdh\\Debugbar\\Controllers\\QueriesController' => $vendorDir . '/barryvdh/laravel-debugbar/src/Controllers/QueriesController.php',
    'Barryvdh\\Debugbar\\Controllers\\TelescopeController' => $vendorDir . '/barryvdh/laravel-debugbar/src/Controllers/TelescopeController.php',
    'Barryvdh\\Debugbar\\DataCollector\\CacheCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/CacheCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\EventCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/EventCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\FilesCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/FilesCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\GateCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/GateCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\JobsCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/JobsCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\LaravelCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/LaravelCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\LivewireCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/LivewireCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\LogsCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/LogsCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\ModelsCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/ModelsCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\MultiAuthCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/MultiAuthCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\PennantCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/PennantCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\QueryCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/QueryCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\RequestCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/RequestCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\RouteCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/RouteCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\SessionCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/SessionCollector.php',
    'Barryvdh\\Debugbar\\DataCollector\\ViewCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataCollector/ViewCollector.php',
    'Barryvdh\\Debugbar\\DataFormatter\\QueryFormatter' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataFormatter/QueryFormatter.php',
    'Barryvdh\\Debugbar\\DataFormatter\\SimpleFormatter' => $vendorDir . '/barryvdh/laravel-debugbar/src/DataFormatter/SimpleFormatter.php',
    'Barryvdh\\Debugbar\\DebugbarViewEngine' => $vendorDir . '/barryvdh/laravel-debugbar/src/DebugbarViewEngine.php',
    'Barryvdh\\Debugbar\\Facade' => $vendorDir . '/barryvdh/laravel-debugbar/src/Facade.php',
    'Barryvdh\\Debugbar\\Facades\\Debugbar' => $vendorDir . '/barryvdh/laravel-debugbar/src/Facades/Debugbar.php',
    'Barryvdh\\Debugbar\\JavascriptRenderer' => $vendorDir . '/barryvdh/laravel-debugbar/src/JavascriptRenderer.php',
    'Barryvdh\\Debugbar\\LaravelDebugbar' => $vendorDir . '/barryvdh/laravel-debugbar/src/LaravelDebugbar.php',
    'Barryvdh\\Debugbar\\LumenServiceProvider' => $vendorDir . '/barryvdh/laravel-debugbar/src/LumenServiceProvider.php',
    'Barryvdh\\Debugbar\\Middleware\\DebugbarEnabled' => $vendorDir . '/barryvdh/laravel-debugbar/src/Middleware/DebugbarEnabled.php',
    'Barryvdh\\Debugbar\\Middleware\\InjectDebugbar' => $vendorDir . '/barryvdh/laravel-debugbar/src/Middleware/InjectDebugbar.php',
    'Barryvdh\\Debugbar\\ServiceProvider' => $vendorDir . '/barryvdh/laravel-debugbar/src/ServiceProvider.php',
    'Barryvdh\\Debugbar\\Storage\\FilesystemStorage' => $vendorDir . '/barryvdh/laravel-debugbar/src/Storage/FilesystemStorage.php',
    'Barryvdh\\Debugbar\\Storage\\SocketStorage' => $vendorDir . '/barryvdh/laravel-debugbar/src/Storage/SocketStorage.php',
    'Barryvdh\\Debugbar\\Support\\Clockwork\\ClockworkCollector' => $vendorDir . '/barryvdh/laravel-debugbar/src/Support/Clockwork/ClockworkCollector.php',
    'Barryvdh\\Debugbar\\Support\\Clockwork\\Converter' => $vendorDir . '/barryvdh/laravel-debugbar/src/Support/Clockwork/Converter.php',
    'Barryvdh\\Debugbar\\Support\\Explain' => $vendorDir . '/barryvdh/laravel-debugbar/src/Support/Explain.php',
    'Barryvdh\\Debugbar\\SymfonyHttpDriver' => $vendorDir . '/barryvdh/laravel-debugbar/src/SymfonyHttpDriver.php',
    'Barryvdh\\Debugbar\\Twig\\Extension\\Debug' => $vendorDir . '/barryvdh/laravel-debugbar/src/Twig/Extension/Debug.php',
    'Barryvdh\\Debugbar\\Twig\\Extension\\Dump' => $vendorDir . '/barryvdh/laravel-debugbar/src/Twig/Extension/Dump.php',
    'Barryvdh\\Debugbar\\Twig\\Extension\\Stopwatch' => $vendorDir . '/barryvdh/laravel-debugbar/src/Twig/Extension/Stopwatch.php',
    'Barryvdh\\DomPDF\\Facade\\Pdf' => $vendorDir . '/barryvdh/laravel-dompdf/src/Facade/Pdf.php',
    'Barryvdh\\DomPDF\\PDF' => $vendorDir . '/barryvdh/laravel-dompdf/src/PDF.php',
    'Barryvdh\\DomPDF\\ServiceProvider' => $vendorDir . '/barryvdh/laravel-dompdf/src/ServiceProvider.php',
    'BaseSendGridClientInterface' => $vendorDir . '/sendgrid/sendgrid/lib/BaseSendGridClientInterface.php',
    'BeyondCode\\QueryDetector\\Events\\QueryDetected' => $vendorDir . '/beyondcode/laravel-query-detector/src/Events/QueryDetected.php',
    'BeyondCode\\QueryDetector\\LumenQueryDetectorServiceProvider' => $vendorDir . '/beyondcode/laravel-query-detector/src/LumenQueryDetectorServiceProvider.php',
    'BeyondCode\\QueryDetector\\Outputs\\Alert' => $vendorDir . '/beyondcode/laravel-query-detector/src/Outputs/Alert.php',
    'BeyondCode\\QueryDetector\\Outputs\\Clockwork' => $vendorDir . '/beyondcode/laravel-query-detector/src/Outputs/Clockwork.php',
    'BeyondCode\\QueryDetector\\Outputs\\Console' => $vendorDir . '/beyondcode/laravel-query-detector/src/Outputs/Console.php',
    'BeyondCode\\QueryDetector\\Outputs\\Debugbar' => $vendorDir . '/beyondcode/laravel-query-detector/src/Outputs/Debugbar.php',
    'BeyondCode\\QueryDetector\\Outputs\\Json' => $vendorDir . '/beyondcode/laravel-query-detector/src/Outputs/Json.php',
    'BeyondCode\\QueryDetector\\Outputs\\Log' => $vendorDir . '/beyondcode/laravel-query-detector/src/Outputs/Log.php',
    'BeyondCode\\QueryDetector\\Outputs\\Output' => $vendorDir . '/beyondcode/laravel-query-detector/src/Outputs/Output.php',
    'BeyondCode\\QueryDetector\\QueryDetector' => $vendorDir . '/beyondcode/laravel-query-detector/src/QueryDetector.php',
    'BeyondCode\\QueryDetector\\QueryDetectorMiddleware' => $vendorDir . '/beyondcode/laravel-query-detector/src/QueryDetectorMiddleware.php',
    'BeyondCode\\QueryDetector\\QueryDetectorServiceProvider' => $vendorDir . '/beyondcode/laravel-query-detector/src/QueryDetectorServiceProvider.php',
    'Brick\\Math\\BigDecimal' => $vendorDir . '/brick/math/src/BigDecimal.php',
    'Brick\\Math\\BigInteger' => $vendorDir . '/brick/math/src/BigInteger.php',
    'Brick\\Math\\BigNumber' => $vendorDir . '/brick/math/src/BigNumber.php',
    'Brick\\Math\\BigRational' => $vendorDir . '/brick/math/src/BigRational.php',
    'Brick\\Math\\Exception\\DivisionByZeroException' => $vendorDir . '/brick/math/src/Exception/DivisionByZeroException.php',
    'Brick\\Math\\Exception\\IntegerOverflowException' => $vendorDir . '/brick/math/src/Exception/IntegerOverflowException.php',
    'Brick\\Math\\Exception\\MathException' => $vendorDir . '/brick/math/src/Exception/MathException.php',
    'Brick\\Math\\Exception\\NegativeNumberException' => $vendorDir . '/brick/math/src/Exception/NegativeNumberException.php',
    'Brick\\Math\\Exception\\NumberFormatException' => $vendorDir . '/brick/math/src/Exception/NumberFormatException.php',
    'Brick\\Math\\Exception\\RoundingNecessaryException' => $vendorDir . '/brick/math/src/Exception/RoundingNecessaryException.php',
    'Brick\\Math\\Internal\\Calculator' => $vendorDir . '/brick/math/src/Internal/Calculator.php',
    'Brick\\Math\\Internal\\Calculator\\BcMathCalculator' => $vendorDir . '/brick/math/src/Internal/Calculator/BcMathCalculator.php',
    'Brick\\Math\\Internal\\Calculator\\GmpCalculator' => $vendorDir . '/brick/math/src/Internal/Calculator/GmpCalculator.php',
    'Brick\\Math\\Internal\\Calculator\\NativeCalculator' => $vendorDir . '/brick/math/src/Internal/Calculator/NativeCalculator.php',
    'Brick\\Math\\RoundingMode' => $vendorDir . '/brick/math/src/RoundingMode.php',
    'Carbon\\AbstractTranslator' => $vendorDir . '/nesbot/carbon/src/Carbon/AbstractTranslator.php',
    'Carbon\\Carbon' => $vendorDir . '/nesbot/carbon/src/Carbon/Carbon.php',
    'Carbon\\CarbonConverterInterface' => $vendorDir . '/nesbot/carbon/src/Carbon/CarbonConverterInterface.php',
    'Carbon\\CarbonImmutable' => $vendorDir . '/nesbot/carbon/src/Carbon/CarbonImmutable.php',
    'Carbon\\CarbonInterface' => $vendorDir . '/nesbot/carbon/src/Carbon/CarbonInterface.php',
    'Carbon\\CarbonInterval' => $vendorDir . '/nesbot/carbon/src/Carbon/CarbonInterval.php',
    'Carbon\\CarbonPeriod' => $vendorDir . '/nesbot/carbon/src/Carbon/CarbonPeriod.php',
    'Carbon\\CarbonPeriodImmutable' => $vendorDir . '/nesbot/carbon/src/Carbon/CarbonPeriodImmutable.php',
    'Carbon\\CarbonTimeZone' => $vendorDir . '/nesbot/carbon/src/Carbon/CarbonTimeZone.php',
    'Carbon\\Cli\\Invoker' => $vendorDir . '/nesbot/carbon/src/Carbon/Cli/Invoker.php',
    'Carbon\\Doctrine\\CarbonDoctrineType' => $vendorDir . '/carbonphp/carbon-doctrine-types/src/Carbon/Doctrine/CarbonDoctrineType.php',
    'Carbon\\Doctrine\\CarbonImmutableType' => $vendorDir . '/carbonphp/carbon-doctrine-types/src/Carbon/Doctrine/CarbonImmutableType.php',
    'Carbon\\Doctrine\\CarbonType' => $vendorDir . '/carbonphp/carbon-doctrine-types/src/Carbon/Doctrine/CarbonType.php',
    'Carbon\\Doctrine\\CarbonTypeConverter' => $vendorDir . '/carbonphp/carbon-doctrine-types/src/Carbon/Doctrine/CarbonTypeConverter.php',
    'Carbon\\Doctrine\\DateTimeDefaultPrecision' => $vendorDir . '/carbonphp/carbon-doctrine-types/src/Carbon/Doctrine/DateTimeDefaultPrecision.php',
    'Carbon\\Doctrine\\DateTimeImmutableType' => $vendorDir . '/carbonphp/carbon-doctrine-types/src/Carbon/Doctrine/DateTimeImmutableType.php',
    'Carbon\\Doctrine\\DateTimeType' => $vendorDir . '/carbonphp/carbon-doctrine-types/src/Carbon/Doctrine/DateTimeType.php',
    'Carbon\\Exceptions\\BadComparisonUnitException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/BadComparisonUnitException.php',
    'Carbon\\Exceptions\\BadFluentConstructorException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/BadFluentConstructorException.php',
    'Carbon\\Exceptions\\BadFluentSetterException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/BadFluentSetterException.php',
    'Carbon\\Exceptions\\BadMethodCallException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/BadMethodCallException.php',
    'Carbon\\Exceptions\\EndLessPeriodException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/EndLessPeriodException.php',
    'Carbon\\Exceptions\\Exception' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/Exception.php',
    'Carbon\\Exceptions\\ImmutableException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/ImmutableException.php',
    'Carbon\\Exceptions\\InvalidArgumentException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/InvalidArgumentException.php',
    'Carbon\\Exceptions\\InvalidCastException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/InvalidCastException.php',
    'Carbon\\Exceptions\\InvalidDateException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/InvalidDateException.php',
    'Carbon\\Exceptions\\InvalidFormatException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/InvalidFormatException.php',
    'Carbon\\Exceptions\\InvalidIntervalException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/InvalidIntervalException.php',
    'Carbon\\Exceptions\\InvalidPeriodDateException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/InvalidPeriodDateException.php',
    'Carbon\\Exceptions\\InvalidPeriodParameterException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/InvalidPeriodParameterException.php',
    'Carbon\\Exceptions\\InvalidTimeZoneException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/InvalidTimeZoneException.php',
    'Carbon\\Exceptions\\InvalidTypeException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/InvalidTypeException.php',
    'Carbon\\Exceptions\\NotACarbonClassException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/NotACarbonClassException.php',
    'Carbon\\Exceptions\\NotAPeriodException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/NotAPeriodException.php',
    'Carbon\\Exceptions\\NotLocaleAwareException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/NotLocaleAwareException.php',
    'Carbon\\Exceptions\\OutOfRangeException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/OutOfRangeException.php',
    'Carbon\\Exceptions\\ParseErrorException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/ParseErrorException.php',
    'Carbon\\Exceptions\\RuntimeException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/RuntimeException.php',
    'Carbon\\Exceptions\\UnitException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/UnitException.php',
    'Carbon\\Exceptions\\UnitNotConfiguredException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/UnitNotConfiguredException.php',
    'Carbon\\Exceptions\\UnknownGetterException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/UnknownGetterException.php',
    'Carbon\\Exceptions\\UnknownMethodException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/UnknownMethodException.php',
    'Carbon\\Exceptions\\UnknownSetterException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/UnknownSetterException.php',
    'Carbon\\Exceptions\\UnknownUnitException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/UnknownUnitException.php',
    'Carbon\\Exceptions\\UnreachableException' => $vendorDir . '/nesbot/carbon/src/Carbon/Exceptions/UnreachableException.php',
    'Carbon\\Factory' => $vendorDir . '/nesbot/carbon/src/Carbon/Factory.php',
    'Carbon\\FactoryImmutable' => $vendorDir . '/nesbot/carbon/src/Carbon/FactoryImmutable.php',
    'Carbon\\Language' => $vendorDir . '/nesbot/carbon/src/Carbon/Language.php',
    'Carbon\\Laravel\\ServiceProvider' => $vendorDir . '/nesbot/carbon/src/Carbon/Laravel/ServiceProvider.php',
    'Carbon\\MessageFormatter\\MessageFormatterMapper' => $vendorDir . '/nesbot/carbon/src/Carbon/MessageFormatter/MessageFormatterMapper.php',
    'Carbon\\PHPStan\\AbstractMacro' => $vendorDir . '/nesbot/carbon/src/Carbon/PHPStan/AbstractMacro.php',
    'Carbon\\PHPStan\\Macro' => $vendorDir . '/nesbot/carbon/src/Carbon/PHPStan/Macro.php',
    'Carbon\\PHPStan\\MacroExtension' => $vendorDir . '/nesbot/carbon/src/Carbon/PHPStan/MacroExtension.php',
    'Carbon\\PHPStan\\MacroScanner' => $vendorDir . '/nesbot/carbon/src/Carbon/PHPStan/MacroScanner.php',
    'Carbon\\Traits\\Boundaries' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Boundaries.php',
    'Carbon\\Traits\\Cast' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Cast.php',
    'Carbon\\Traits\\Comparison' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Comparison.php',
    'Carbon\\Traits\\Converter' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Converter.php',
    'Carbon\\Traits\\Creator' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Creator.php',
    'Carbon\\Traits\\Date' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Date.php',
    'Carbon\\Traits\\DeprecatedProperties' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/DeprecatedProperties.php',
    'Carbon\\Traits\\Difference' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Difference.php',
    'Carbon\\Traits\\IntervalRounding' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/IntervalRounding.php',
    'Carbon\\Traits\\IntervalStep' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/IntervalStep.php',
    'Carbon\\Traits\\Localization' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Localization.php',
    'Carbon\\Traits\\Macro' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Macro.php',
    'Carbon\\Traits\\MagicParameter' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/MagicParameter.php',
    'Carbon\\Traits\\Mixin' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Mixin.php',
    'Carbon\\Traits\\Modifiers' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Modifiers.php',
    'Carbon\\Traits\\Mutability' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Mutability.php',
    'Carbon\\Traits\\ObjectInitialisation' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/ObjectInitialisation.php',
    'Carbon\\Traits\\Options' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Options.php',
    'Carbon\\Traits\\Rounding' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Rounding.php',
    'Carbon\\Traits\\Serialization' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Serialization.php',
    'Carbon\\Traits\\Test' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Test.php',
    'Carbon\\Traits\\Timestamp' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Timestamp.php',
    'Carbon\\Traits\\ToStringFormat' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/ToStringFormat.php',
    'Carbon\\Traits\\Units' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Units.php',
    'Carbon\\Traits\\Week' => $vendorDir . '/nesbot/carbon/src/Carbon/Traits/Week.php',
    'Carbon\\Translator' => $vendorDir . '/nesbot/carbon/src/Carbon/Translator.php',
    'Carbon\\TranslatorImmutable' => $vendorDir . '/nesbot/carbon/src/Carbon/TranslatorImmutable.php',
    'Carbon\\TranslatorStrongTypeInterface' => $vendorDir . '/nesbot/carbon/src/Carbon/TranslatorStrongTypeInterface.php',
    'CoinGate\\APIError\\APIError' => $vendorDir . '/coingate/coingate-php/lib/APIError/APIError.php',
    'CoinGate\\APIError\\AccountBlocked' => $vendorDir . '/coingate/coingate-php/lib/APIError/AccountBlocked.php',
    'CoinGate\\APIError\\BadAuthToken' => $vendorDir . '/coingate/coingate-php/lib/APIError/BadAuthToken.php',
    'CoinGate\\APIError\\BadCredentials' => $vendorDir . '/coingate/coingate-php/lib/APIError/BadCredentials.php',
    'CoinGate\\APIError\\BadEnvironment' => $vendorDir . '/coingate/coingate-php/lib/APIError/BadEnvironment.php',
    'CoinGate\\APIError\\BadRequest' => $vendorDir . '/coingate/coingate-php/lib/APIError/BadRequest.php',
    'CoinGate\\APIError\\CredentialsMissing' => $vendorDir . '/coingate/coingate-php/lib/APIError/CredentialsMissing.php',
    'CoinGate\\APIError\\InternalServerError' => $vendorDir . '/coingate/coingate-php/lib/APIError/InternalServerError.php',
    'CoinGate\\APIError\\IpAddressIsNotAllowed' => $vendorDir . '/coingate/coingate-php/lib/APIError/IpAddressIsNotAllowed.php',
    'CoinGate\\APIError\\NotFound' => $vendorDir . '/coingate/coingate-php/lib/APIError/NotFound.php',
    'CoinGate\\APIError\\OrderIsNotValid' => $vendorDir . '/coingate/coingate-php/lib/APIError/OrderIsNotValid.php',
    'CoinGate\\APIError\\OrderNotFound' => $vendorDir . '/coingate/coingate-php/lib/APIError/OrderNotFound.php',
    'CoinGate\\APIError\\PageNotFound' => $vendorDir . '/coingate/coingate-php/lib/APIError/PageNotFound.php',
    'CoinGate\\APIError\\RateLimitException' => $vendorDir . '/coingate/coingate-php/lib/APIError/RateLimitException.php',
    'CoinGate\\APIError\\RecordNotFound' => $vendorDir . '/coingate/coingate-php/lib/APIError/RecordNotFound.php',
    'CoinGate\\APIError\\Unauthorized' => $vendorDir . '/coingate/coingate-php/lib/APIError/Unauthorized.php',
    'CoinGate\\APIError\\UnprocessableEntity' => $vendorDir . '/coingate/coingate-php/lib/APIError/UnprocessableEntity.php',
    'CoinGate\\CoinGate' => $vendorDir . '/coingate/coingate-php/lib/CoinGate.php',
    'CoinGate\\Exception' => $vendorDir . '/coingate/coingate-php/lib/Exception.php',
    'CoinGate\\Merchant' => $vendorDir . '/coingate/coingate-php/lib/Merchant.php',
    'CoinGate\\Merchant\\Order' => $vendorDir . '/coingate/coingate-php/lib/Merchant/Order.php',
    'Complex\\Complex' => $vendorDir . '/markbaker/complex/classes/src/Complex.php',
    'Complex\\Exception' => $vendorDir . '/markbaker/complex/classes/src/Exception.php',
    'Complex\\Functions' => $vendorDir . '/markbaker/complex/classes/src/Functions.php',
    'Complex\\Operations' => $vendorDir . '/markbaker/complex/classes/src/Operations.php',
    'Composer\\CaBundle\\CaBundle' => $vendorDir . '/composer/ca-bundle/src/CaBundle.php',
    'Composer\\InstalledVersions' => $vendorDir . '/composer/InstalledVersions.php',
    'Composer\\Semver\\Comparator' => $vendorDir . '/composer/semver/src/Comparator.php',
    'Composer\\Semver\\CompilingMatcher' => $vendorDir . '/composer/semver/src/CompilingMatcher.php',
    'Composer\\Semver\\Constraint\\Bound' => $vendorDir . '/composer/semver/src/Constraint/Bound.php',
    'Composer\\Semver\\Constraint\\Constraint' => $vendorDir . '/composer/semver/src/Constraint/Constraint.php',
    'Composer\\Semver\\Constraint\\ConstraintInterface' => $vendorDir . '/composer/semver/src/Constraint/ConstraintInterface.php',
    'Composer\\Semver\\Constraint\\MatchAllConstraint' => $vendorDir . '/composer/semver/src/Constraint/MatchAllConstraint.php',
    'Composer\\Semver\\Constraint\\MatchNoneConstraint' => $vendorDir . '/composer/semver/src/Constraint/MatchNoneConstraint.php',
    'Composer\\Semver\\Constraint\\MultiConstraint' => $vendorDir . '/composer/semver/src/Constraint/MultiConstraint.php',
    'Composer\\Semver\\Interval' => $vendorDir . '/composer/semver/src/Interval.php',
    'Composer\\Semver\\Intervals' => $vendorDir . '/composer/semver/src/Intervals.php',
    'Composer\\Semver\\Semver' => $vendorDir . '/composer/semver/src/Semver.php',
    'Composer\\Semver\\VersionParser' => $vendorDir . '/composer/semver/src/VersionParser.php',
    'Cron\\AbstractField' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/AbstractField.php',
    'Cron\\CronExpression' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/CronExpression.php',
    'Cron\\DayOfMonthField' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/DayOfMonthField.php',
    'Cron\\DayOfWeekField' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/DayOfWeekField.php',
    'Cron\\FieldFactory' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/FieldFactory.php',
    'Cron\\FieldFactoryInterface' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/FieldFactoryInterface.php',
    'Cron\\FieldInterface' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/FieldInterface.php',
    'Cron\\HoursField' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/HoursField.php',
    'Cron\\MinutesField' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/MinutesField.php',
    'Cron\\MonthField' => $vendorDir . '/dragonmantank/cron-expression/src/Cron/MonthField.php',
    'Database\\Factories\\UserFactory' => $baseDir . '/database/factories/UserFactory.php',
    'Database\\Seeders\\DatabaseSeeder' => $baseDir . '/database/seeders/DatabaseSeeder.php',
    'Database\\Seeders\\PermissionSeeder' => $baseDir . '/database/seeders/PermissionSeeder.php',
    'DateError' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateError.php',
    'DateException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateException.php',
    'DateInvalidOperationException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateInvalidOperationException.php',
    'DateInvalidTimeZoneException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateInvalidTimeZoneException.php',
    'DateMalformedIntervalStringException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateMalformedIntervalStringException.php',
    'DateMalformedPeriodStringException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateMalformedPeriodStringException.php',
    'DateMalformedStringException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateMalformedStringException.php',
    'DateObjectError' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateObjectError.php',
    'DateRangeError' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateRangeError.php',
    'DebugBar\\Bridge\\CacheCacheCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/CacheCacheCollector.php',
    'DebugBar\\Bridge\\DoctrineCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/DoctrineCollector.php',
    'DebugBar\\Bridge\\MonologCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/MonologCollector.php',
    'DebugBar\\Bridge\\NamespacedTwigProfileCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/NamespacedTwigProfileCollector.php',
    'DebugBar\\Bridge\\Propel2Collector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Propel2Collector.php',
    'DebugBar\\Bridge\\PropelCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/PropelCollector.php',
    'DebugBar\\Bridge\\SlimCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/SlimCollector.php',
    'DebugBar\\Bridge\\SwiftMailer\\SwiftLogCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/SwiftMailer/SwiftLogCollector.php',
    'DebugBar\\Bridge\\SwiftMailer\\SwiftMailCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/SwiftMailer/SwiftMailCollector.php',
    'DebugBar\\Bridge\\Symfony\\SymfonyMailCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Symfony/SymfonyMailCollector.php',
    'DebugBar\\Bridge\\TwigProfileCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/TwigProfileCollector.php',
    'DebugBar\\Bridge\\Twig\\DebugTwigExtension' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Twig/DebugTwigExtension.php',
    'DebugBar\\Bridge\\Twig\\DumpTwigExtension' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Twig/DumpTwigExtension.php',
    'DebugBar\\Bridge\\Twig\\MeasureTwigExtension' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Twig/MeasureTwigExtension.php',
    'DebugBar\\Bridge\\Twig\\MeasureTwigNode' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Twig/MeasureTwigNode.php',
    'DebugBar\\Bridge\\Twig\\MeasureTwigTokenParser' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Twig/MeasureTwigTokenParser.php',
    'DebugBar\\Bridge\\Twig\\TimeableTwigExtensionProfiler' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Twig/TimeableTwigExtensionProfiler.php',
    'DebugBar\\Bridge\\Twig\\TraceableTwigEnvironment' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Twig/TraceableTwigEnvironment.php',
    'DebugBar\\Bridge\\Twig\\TraceableTwigTemplate' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Twig/TraceableTwigTemplate.php',
    'DebugBar\\Bridge\\Twig\\TwigCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Bridge/Twig/TwigCollector.php',
    'DebugBar\\DataCollector\\AggregatedCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/AggregatedCollector.php',
    'DebugBar\\DataCollector\\AssetProvider' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/AssetProvider.php',
    'DebugBar\\DataCollector\\ConfigCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/ConfigCollector.php',
    'DebugBar\\DataCollector\\DataCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/DataCollector.php',
    'DebugBar\\DataCollector\\DataCollectorInterface' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/DataCollectorInterface.php',
    'DebugBar\\DataCollector\\ExceptionsCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/ExceptionsCollector.php',
    'DebugBar\\DataCollector\\LocalizationCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/LocalizationCollector.php',
    'DebugBar\\DataCollector\\MemoryCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/MemoryCollector.php',
    'DebugBar\\DataCollector\\MessagesAggregateInterface' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/MessagesAggregateInterface.php',
    'DebugBar\\DataCollector\\MessagesCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/MessagesCollector.php',
    'DebugBar\\DataCollector\\ObjectCountCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/ObjectCountCollector.php',
    'DebugBar\\DataCollector\\PDO\\PDOCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/PDO/PDOCollector.php',
    'DebugBar\\DataCollector\\PDO\\TraceablePDO' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/PDO/TraceablePDO.php',
    'DebugBar\\DataCollector\\PDO\\TraceablePDOStatement' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/PDO/TraceablePDOStatement.php',
    'DebugBar\\DataCollector\\PDO\\TracedStatement' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/PDO/TracedStatement.php',
    'DebugBar\\DataCollector\\PhpInfoCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/PhpInfoCollector.php',
    'DebugBar\\DataCollector\\Renderable' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/Renderable.php',
    'DebugBar\\DataCollector\\RequestDataCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/RequestDataCollector.php',
    'DebugBar\\DataCollector\\TimeDataCollector' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataCollector/TimeDataCollector.php',
    'DebugBar\\DataFormatter\\DataFormatter' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataFormatter/DataFormatter.php',
    'DebugBar\\DataFormatter\\DataFormatterInterface' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataFormatter/DataFormatterInterface.php',
    'DebugBar\\DataFormatter\\DebugBarVarDumper' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataFormatter/DebugBarVarDumper.php',
    'DebugBar\\DataFormatter\\HasDataFormatter' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataFormatter/HasDataFormatter.php',
    'DebugBar\\DataFormatter\\HasXdebugLinks' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataFormatter/HasXdebugLinks.php',
    'DebugBar\\DataFormatter\\VarDumper\\DebugBarHtmlDumper' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DataFormatter/VarDumper/DebugBarHtmlDumper.php',
    'DebugBar\\DebugBar' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DebugBar.php',
    'DebugBar\\DebugBarException' => $vendorDir . '/maximebf/debugbar/src/DebugBar/DebugBarException.php',
    'DebugBar\\HttpDriverInterface' => $vendorDir . '/maximebf/debugbar/src/DebugBar/HttpDriverInterface.php',
    'DebugBar\\JavascriptRenderer' => $vendorDir . '/maximebf/debugbar/src/DebugBar/JavascriptRenderer.php',
    'DebugBar\\OpenHandler' => $vendorDir . '/maximebf/debugbar/src/DebugBar/OpenHandler.php',
    'DebugBar\\PhpHttpDriver' => $vendorDir . '/maximebf/debugbar/src/DebugBar/PhpHttpDriver.php',
    'DebugBar\\RequestIdGenerator' => $vendorDir . '/maximebf/debugbar/src/DebugBar/RequestIdGenerator.php',
    'DebugBar\\RequestIdGeneratorInterface' => $vendorDir . '/maximebf/debugbar/src/DebugBar/RequestIdGeneratorInterface.php',
    'DebugBar\\StandardDebugBar' => $vendorDir . '/maximebf/debugbar/src/DebugBar/StandardDebugBar.php',
    'DebugBar\\Storage\\FileStorage' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Storage/FileStorage.php',
    'DebugBar\\Storage\\MemcachedStorage' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Storage/MemcachedStorage.php',
    'DebugBar\\Storage\\PdoStorage' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Storage/PdoStorage.php',
    'DebugBar\\Storage\\RedisStorage' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Storage/RedisStorage.php',
    'DebugBar\\Storage\\StorageInterface' => $vendorDir . '/maximebf/debugbar/src/DebugBar/Storage/StorageInterface.php',
    'DeepCopy\\DeepCopy' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/DeepCopy.php',
    'DeepCopy\\Exception\\CloneException' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Exception/CloneException.php',
    'DeepCopy\\Exception\\PropertyException' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Exception/PropertyException.php',
    'DeepCopy\\Filter\\ChainableFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Filter/ChainableFilter.php',
    'DeepCopy\\Filter\\Doctrine\\DoctrineCollectionFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Filter/Doctrine/DoctrineCollectionFilter.php',
    'DeepCopy\\Filter\\Doctrine\\DoctrineEmptyCollectionFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Filter/Doctrine/DoctrineEmptyCollectionFilter.php',
    'DeepCopy\\Filter\\Doctrine\\DoctrineProxyFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Filter/Doctrine/DoctrineProxyFilter.php',
    'DeepCopy\\Filter\\Filter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Filter/Filter.php',
    'DeepCopy\\Filter\\KeepFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Filter/KeepFilter.php',
    'DeepCopy\\Filter\\ReplaceFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Filter/ReplaceFilter.php',
    'DeepCopy\\Filter\\SetNullFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Filter/SetNullFilter.php',
    'DeepCopy\\Matcher\\Doctrine\\DoctrineProxyMatcher' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Matcher/Doctrine/DoctrineProxyMatcher.php',
    'DeepCopy\\Matcher\\Matcher' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Matcher/Matcher.php',
    'DeepCopy\\Matcher\\PropertyMatcher' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Matcher/PropertyMatcher.php',
    'DeepCopy\\Matcher\\PropertyNameMatcher' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Matcher/PropertyNameMatcher.php',
    'DeepCopy\\Matcher\\PropertyTypeMatcher' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Matcher/PropertyTypeMatcher.php',
    'DeepCopy\\Reflection\\ReflectionHelper' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/Reflection/ReflectionHelper.php',
    'DeepCopy\\TypeFilter\\Date\\DateIntervalFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/TypeFilter/Date/DateIntervalFilter.php',
    'DeepCopy\\TypeFilter\\Date\\DatePeriodFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/TypeFilter/Date/DatePeriodFilter.php',
    'DeepCopy\\TypeFilter\\ReplaceFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/TypeFilter/ReplaceFilter.php',
    'DeepCopy\\TypeFilter\\ShallowCopyFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/TypeFilter/ShallowCopyFilter.php',
    'DeepCopy\\TypeFilter\\Spl\\ArrayObjectFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/TypeFilter/Spl/ArrayObjectFilter.php',
    'DeepCopy\\TypeFilter\\Spl\\SplDoublyLinkedList' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/TypeFilter/Spl/SplDoublyLinkedList.php',
    'DeepCopy\\TypeFilter\\Spl\\SplDoublyLinkedListFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/TypeFilter/Spl/SplDoublyLinkedListFilter.php',
    'DeepCopy\\TypeFilter\\TypeFilter' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/TypeFilter/TypeFilter.php',
    'DeepCopy\\TypeMatcher\\TypeMatcher' => $vendorDir . '/myclabs/deep-copy/src/DeepCopy/TypeMatcher/TypeMatcher.php',
    'Dflydev\\DotAccessData\\Data' => $vendorDir . '/dflydev/dot-access-data/src/Data.php',
    'Dflydev\\DotAccessData\\DataInterface' => $vendorDir . '/dflydev/dot-access-data/src/DataInterface.php',
    'Dflydev\\DotAccessData\\Exception\\DataException' => $vendorDir . '/dflydev/dot-access-data/src/Exception/DataException.php',
    'Dflydev\\DotAccessData\\Exception\\InvalidPathException' => $vendorDir . '/dflydev/dot-access-data/src/Exception/InvalidPathException.php',
    'Dflydev\\DotAccessData\\Exception\\MissingPathException' => $vendorDir . '/dflydev/dot-access-data/src/Exception/MissingPathException.php',
    'Dflydev\\DotAccessData\\Util' => $vendorDir . '/dflydev/dot-access-data/src/Util.php',
    'Doctrine\\Common\\Cache\\Cache' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/Cache.php',
    'Doctrine\\Common\\Cache\\CacheProvider' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/CacheProvider.php',
    'Doctrine\\Common\\Cache\\ClearableCache' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/ClearableCache.php',
    'Doctrine\\Common\\Cache\\FlushableCache' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/FlushableCache.php',
    'Doctrine\\Common\\Cache\\MultiDeleteCache' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/MultiDeleteCache.php',
    'Doctrine\\Common\\Cache\\MultiGetCache' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/MultiGetCache.php',
    'Doctrine\\Common\\Cache\\MultiOperationCache' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/MultiOperationCache.php',
    'Doctrine\\Common\\Cache\\MultiPutCache' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/MultiPutCache.php',
    'Doctrine\\Common\\Cache\\Psr6\\CacheAdapter' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/Psr6/CacheAdapter.php',
    'Doctrine\\Common\\Cache\\Psr6\\CacheItem' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/Psr6/CacheItem.php',
    'Doctrine\\Common\\Cache\\Psr6\\DoctrineProvider' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/Psr6/DoctrineProvider.php',
    'Doctrine\\Common\\Cache\\Psr6\\InvalidArgument' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/Psr6/InvalidArgument.php',
    'Doctrine\\Common\\Cache\\Psr6\\TypedCacheItem' => $vendorDir . '/doctrine/cache/lib/Doctrine/Common/Cache/Psr6/TypedCacheItem.php',
    'Doctrine\\Common\\EventArgs' => $vendorDir . '/doctrine/event-manager/src/EventArgs.php',
    'Doctrine\\Common\\EventManager' => $vendorDir . '/doctrine/event-manager/src/EventManager.php',
    'Doctrine\\Common\\EventSubscriber' => $vendorDir . '/doctrine/event-manager/src/EventSubscriber.php',
    'Doctrine\\Common\\Lexer\\AbstractLexer' => $vendorDir . '/doctrine/lexer/src/AbstractLexer.php',
    'Doctrine\\Common\\Lexer\\Token' => $vendorDir . '/doctrine/lexer/src/Token.php',
    'Doctrine\\DBAL\\ArrayParameterType' => $vendorDir . '/doctrine/dbal/src/ArrayParameterType.php',
    'Doctrine\\DBAL\\ArrayParameters\\Exception' => $vendorDir . '/doctrine/dbal/src/ArrayParameters/Exception.php',
    'Doctrine\\DBAL\\ArrayParameters\\Exception\\MissingNamedParameter' => $vendorDir . '/doctrine/dbal/src/ArrayParameters/Exception/MissingNamedParameter.php',
    'Doctrine\\DBAL\\ArrayParameters\\Exception\\MissingPositionalParameter' => $vendorDir . '/doctrine/dbal/src/ArrayParameters/Exception/MissingPositionalParameter.php',
    'Doctrine\\DBAL\\Cache\\ArrayResult' => $vendorDir . '/doctrine/dbal/src/Cache/ArrayResult.php',
    'Doctrine\\DBAL\\Cache\\CacheException' => $vendorDir . '/doctrine/dbal/src/Cache/CacheException.php',
    'Doctrine\\DBAL\\Cache\\QueryCacheProfile' => $vendorDir . '/doctrine/dbal/src/Cache/QueryCacheProfile.php',
    'Doctrine\\DBAL\\ColumnCase' => $vendorDir . '/doctrine/dbal/src/ColumnCase.php',
    'Doctrine\\DBAL\\Configuration' => $vendorDir . '/doctrine/dbal/src/Configuration.php',
    'Doctrine\\DBAL\\Connection' => $vendorDir . '/doctrine/dbal/src/Connection.php',
    'Doctrine\\DBAL\\ConnectionException' => $vendorDir . '/doctrine/dbal/src/ConnectionException.php',
    'Doctrine\\DBAL\\Connections\\PrimaryReadReplicaConnection' => $vendorDir . '/doctrine/dbal/src/Connections/PrimaryReadReplicaConnection.php',
    'Doctrine\\DBAL\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver.php',
    'Doctrine\\DBAL\\DriverManager' => $vendorDir . '/doctrine/dbal/src/DriverManager.php',
    'Doctrine\\DBAL\\Driver\\API\\ExceptionConverter' => $vendorDir . '/doctrine/dbal/src/Driver/API/ExceptionConverter.php',
    'Doctrine\\DBAL\\Driver\\API\\IBMDB2\\ExceptionConverter' => $vendorDir . '/doctrine/dbal/src/Driver/API/IBMDB2/ExceptionConverter.php',
    'Doctrine\\DBAL\\Driver\\API\\MySQL\\ExceptionConverter' => $vendorDir . '/doctrine/dbal/src/Driver/API/MySQL/ExceptionConverter.php',
    'Doctrine\\DBAL\\Driver\\API\\OCI\\ExceptionConverter' => $vendorDir . '/doctrine/dbal/src/Driver/API/OCI/ExceptionConverter.php',
    'Doctrine\\DBAL\\Driver\\API\\PostgreSQL\\ExceptionConverter' => $vendorDir . '/doctrine/dbal/src/Driver/API/PostgreSQL/ExceptionConverter.php',
    'Doctrine\\DBAL\\Driver\\API\\SQLSrv\\ExceptionConverter' => $vendorDir . '/doctrine/dbal/src/Driver/API/SQLSrv/ExceptionConverter.php',
    'Doctrine\\DBAL\\Driver\\API\\SQLite\\ExceptionConverter' => $vendorDir . '/doctrine/dbal/src/Driver/API/SQLite/ExceptionConverter.php',
    'Doctrine\\DBAL\\Driver\\API\\SQLite\\UserDefinedFunctions' => $vendorDir . '/doctrine/dbal/src/Driver/API/SQLite/UserDefinedFunctions.php',
    'Doctrine\\DBAL\\Driver\\AbstractDB2Driver' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractDB2Driver.php',
    'Doctrine\\DBAL\\Driver\\AbstractException' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractException.php',
    'Doctrine\\DBAL\\Driver\\AbstractMySQLDriver' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractMySQLDriver.php',
    'Doctrine\\DBAL\\Driver\\AbstractOracleDriver' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractOracleDriver.php',
    'Doctrine\\DBAL\\Driver\\AbstractOracleDriver\\EasyConnectString' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractOracleDriver/EasyConnectString.php',
    'Doctrine\\DBAL\\Driver\\AbstractPostgreSQLDriver' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractPostgreSQLDriver.php',
    'Doctrine\\DBAL\\Driver\\AbstractSQLServerDriver' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractSQLServerDriver.php',
    'Doctrine\\DBAL\\Driver\\AbstractSQLServerDriver\\Exception\\PortWithoutHost' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractSQLServerDriver/Exception/PortWithoutHost.php',
    'Doctrine\\DBAL\\Driver\\AbstractSQLiteDriver' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractSQLiteDriver.php',
    'Doctrine\\DBAL\\Driver\\AbstractSQLiteDriver\\Middleware\\EnableForeignKeys' => $vendorDir . '/doctrine/dbal/src/Driver/AbstractSQLiteDriver/Middleware/EnableForeignKeys.php',
    'Doctrine\\DBAL\\Driver\\Connection' => $vendorDir . '/doctrine/dbal/src/Driver/Connection.php',
    'Doctrine\\DBAL\\Driver\\Exception' => $vendorDir . '/doctrine/dbal/src/Driver/Exception.php',
    'Doctrine\\DBAL\\Driver\\Exception\\UnknownParameterType' => $vendorDir . '/doctrine/dbal/src/Driver/Exception/UnknownParameterType.php',
    'Doctrine\\DBAL\\Driver\\FetchUtils' => $vendorDir . '/doctrine/dbal/src/Driver/FetchUtils.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Connection' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Connection.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\DataSourceName' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/DataSourceName.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Driver.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Exception\\CannotCopyStreamToStream' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Exception/CannotCopyStreamToStream.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Exception\\CannotCreateTemporaryFile' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Exception/CannotCreateTemporaryFile.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Exception\\ConnectionError' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Exception/ConnectionError.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Exception\\ConnectionFailed' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Exception/ConnectionFailed.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Exception\\Factory' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Exception/Factory.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Exception\\PrepareFailed' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Exception/PrepareFailed.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Exception\\StatementError' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Exception/StatementError.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Result' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Result.php',
    'Doctrine\\DBAL\\Driver\\IBMDB2\\Statement' => $vendorDir . '/doctrine/dbal/src/Driver/IBMDB2/Statement.php',
    'Doctrine\\DBAL\\Driver\\Middleware' => $vendorDir . '/doctrine/dbal/src/Driver/Middleware.php',
    'Doctrine\\DBAL\\Driver\\Middleware\\AbstractConnectionMiddleware' => $vendorDir . '/doctrine/dbal/src/Driver/Middleware/AbstractConnectionMiddleware.php',
    'Doctrine\\DBAL\\Driver\\Middleware\\AbstractDriverMiddleware' => $vendorDir . '/doctrine/dbal/src/Driver/Middleware/AbstractDriverMiddleware.php',
    'Doctrine\\DBAL\\Driver\\Middleware\\AbstractResultMiddleware' => $vendorDir . '/doctrine/dbal/src/Driver/Middleware/AbstractResultMiddleware.php',
    'Doctrine\\DBAL\\Driver\\Middleware\\AbstractStatementMiddleware' => $vendorDir . '/doctrine/dbal/src/Driver/Middleware/AbstractStatementMiddleware.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Connection' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Connection.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Driver.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Exception\\ConnectionError' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Exception/ConnectionError.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Exception\\ConnectionFailed' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Exception/ConnectionFailed.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Exception\\FailedReadingStreamOffset' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Exception/FailedReadingStreamOffset.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Exception\\HostRequired' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Exception/HostRequired.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Exception\\InvalidCharset' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Exception/InvalidCharset.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Exception\\InvalidOption' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Exception/InvalidOption.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Exception\\NonStreamResourceUsedAsLargeObject' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Exception/NonStreamResourceUsedAsLargeObject.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Exception\\StatementError' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Exception/StatementError.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Initializer' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Initializer.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Initializer\\Charset' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Initializer/Charset.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Initializer\\Options' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Initializer/Options.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Initializer\\Secure' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Initializer/Secure.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Result' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Result.php',
    'Doctrine\\DBAL\\Driver\\Mysqli\\Statement' => $vendorDir . '/doctrine/dbal/src/Driver/Mysqli/Statement.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Connection' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Connection.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\ConvertPositionalToNamedPlaceholders' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/ConvertPositionalToNamedPlaceholders.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Driver.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Exception\\ConnectionFailed' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Exception/ConnectionFailed.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Exception\\Error' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Exception/Error.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Exception\\InvalidConfiguration' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Exception/InvalidConfiguration.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Exception\\NonTerminatedStringLiteral' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Exception/NonTerminatedStringLiteral.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Exception\\SequenceDoesNotExist' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Exception/SequenceDoesNotExist.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Exception\\UnknownParameterIndex' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Exception/UnknownParameterIndex.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\ExecutionMode' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/ExecutionMode.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Middleware\\InitializeSession' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Middleware/InitializeSession.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Result' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Result.php',
    'Doctrine\\DBAL\\Driver\\OCI8\\Statement' => $vendorDir . '/doctrine/dbal/src/Driver/OCI8/Statement.php',
    'Doctrine\\DBAL\\Driver\\PDO\\Connection' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/Connection.php',
    'Doctrine\\DBAL\\Driver\\PDO\\Exception' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/Exception.php',
    'Doctrine\\DBAL\\Driver\\PDO\\MySQL\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/MySQL/Driver.php',
    'Doctrine\\DBAL\\Driver\\PDO\\OCI\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/OCI/Driver.php',
    'Doctrine\\DBAL\\Driver\\PDO\\PDOException' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/PDOException.php',
    'Doctrine\\DBAL\\Driver\\PDO\\ParameterTypeMap' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/ParameterTypeMap.php',
    'Doctrine\\DBAL\\Driver\\PDO\\PgSQL\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/PgSQL/Driver.php',
    'Doctrine\\DBAL\\Driver\\PDO\\Result' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/Result.php',
    'Doctrine\\DBAL\\Driver\\PDO\\SQLSrv\\Connection' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/SQLSrv/Connection.php',
    'Doctrine\\DBAL\\Driver\\PDO\\SQLSrv\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/SQLSrv/Driver.php',
    'Doctrine\\DBAL\\Driver\\PDO\\SQLSrv\\Statement' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/SQLSrv/Statement.php',
    'Doctrine\\DBAL\\Driver\\PDO\\SQLite\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/SQLite/Driver.php',
    'Doctrine\\DBAL\\Driver\\PDO\\Statement' => $vendorDir . '/doctrine/dbal/src/Driver/PDO/Statement.php',
    'Doctrine\\DBAL\\Driver\\PgSQL\\Connection' => $vendorDir . '/doctrine/dbal/src/Driver/PgSQL/Connection.php',
    'Doctrine\\DBAL\\Driver\\PgSQL\\ConvertParameters' => $vendorDir . '/doctrine/dbal/src/Driver/PgSQL/ConvertParameters.php',
    'Doctrine\\DBAL\\Driver\\PgSQL\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/PgSQL/Driver.php',
    'Doctrine\\DBAL\\Driver\\PgSQL\\Exception' => $vendorDir . '/doctrine/dbal/src/Driver/PgSQL/Exception.php',
    'Doctrine\\DBAL\\Driver\\PgSQL\\Exception\\UnexpectedValue' => $vendorDir . '/doctrine/dbal/src/Driver/PgSQL/Exception/UnexpectedValue.php',
    'Doctrine\\DBAL\\Driver\\PgSQL\\Exception\\UnknownParameter' => $vendorDir . '/doctrine/dbal/src/Driver/PgSQL/Exception/UnknownParameter.php',
    'Doctrine\\DBAL\\Driver\\PgSQL\\Result' => $vendorDir . '/doctrine/dbal/src/Driver/PgSQL/Result.php',
    'Doctrine\\DBAL\\Driver\\PgSQL\\Statement' => $vendorDir . '/doctrine/dbal/src/Driver/PgSQL/Statement.php',
    'Doctrine\\DBAL\\Driver\\Result' => $vendorDir . '/doctrine/dbal/src/Driver/Result.php',
    'Doctrine\\DBAL\\Driver\\SQLSrv\\Connection' => $vendorDir . '/doctrine/dbal/src/Driver/SQLSrv/Connection.php',
    'Doctrine\\DBAL\\Driver\\SQLSrv\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/SQLSrv/Driver.php',
    'Doctrine\\DBAL\\Driver\\SQLSrv\\Exception\\Error' => $vendorDir . '/doctrine/dbal/src/Driver/SQLSrv/Exception/Error.php',
    'Doctrine\\DBAL\\Driver\\SQLSrv\\Result' => $vendorDir . '/doctrine/dbal/src/Driver/SQLSrv/Result.php',
    'Doctrine\\DBAL\\Driver\\SQLSrv\\Statement' => $vendorDir . '/doctrine/dbal/src/Driver/SQLSrv/Statement.php',
    'Doctrine\\DBAL\\Driver\\SQLite3\\Connection' => $vendorDir . '/doctrine/dbal/src/Driver/SQLite3/Connection.php',
    'Doctrine\\DBAL\\Driver\\SQLite3\\Driver' => $vendorDir . '/doctrine/dbal/src/Driver/SQLite3/Driver.php',
    'Doctrine\\DBAL\\Driver\\SQLite3\\Exception' => $vendorDir . '/doctrine/dbal/src/Driver/SQLite3/Exception.php',
    'Doctrine\\DBAL\\Driver\\SQLite3\\Result' => $vendorDir . '/doctrine/dbal/src/Driver/SQLite3/Result.php',
    'Doctrine\\DBAL\\Driver\\SQLite3\\Statement' => $vendorDir . '/doctrine/dbal/src/Driver/SQLite3/Statement.php',
    'Doctrine\\DBAL\\Driver\\ServerInfoAwareConnection' => $vendorDir . '/doctrine/dbal/src/Driver/ServerInfoAwareConnection.php',
    'Doctrine\\DBAL\\Driver\\Statement' => $vendorDir . '/doctrine/dbal/src/Driver/Statement.php',
    'Doctrine\\DBAL\\Event\\ConnectionEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/ConnectionEventArgs.php',
    'Doctrine\\DBAL\\Event\\Listeners\\OracleSessionInit' => $vendorDir . '/doctrine/dbal/src/Event/Listeners/OracleSessionInit.php',
    'Doctrine\\DBAL\\Event\\Listeners\\SQLSessionInit' => $vendorDir . '/doctrine/dbal/src/Event/Listeners/SQLSessionInit.php',
    'Doctrine\\DBAL\\Event\\Listeners\\SQLiteSessionInit' => $vendorDir . '/doctrine/dbal/src/Event/Listeners/SQLiteSessionInit.php',
    'Doctrine\\DBAL\\Event\\SchemaAlterTableAddColumnEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaAlterTableAddColumnEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaAlterTableChangeColumnEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaAlterTableChangeColumnEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaAlterTableEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaAlterTableEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaAlterTableRemoveColumnEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaAlterTableRemoveColumnEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaAlterTableRenameColumnEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaAlterTableRenameColumnEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaColumnDefinitionEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaColumnDefinitionEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaCreateTableColumnEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaCreateTableColumnEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaCreateTableEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaCreateTableEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaDropTableEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaDropTableEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaEventArgs.php',
    'Doctrine\\DBAL\\Event\\SchemaIndexDefinitionEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/SchemaIndexDefinitionEventArgs.php',
    'Doctrine\\DBAL\\Event\\TransactionBeginEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/TransactionBeginEventArgs.php',
    'Doctrine\\DBAL\\Event\\TransactionCommitEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/TransactionCommitEventArgs.php',
    'Doctrine\\DBAL\\Event\\TransactionEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/TransactionEventArgs.php',
    'Doctrine\\DBAL\\Event\\TransactionRollBackEventArgs' => $vendorDir . '/doctrine/dbal/src/Event/TransactionRollBackEventArgs.php',
    'Doctrine\\DBAL\\Events' => $vendorDir . '/doctrine/dbal/src/Events.php',
    'Doctrine\\DBAL\\Exception' => $vendorDir . '/doctrine/dbal/src/Exception.php',
    'Doctrine\\DBAL\\Exception\\ConnectionException' => $vendorDir . '/doctrine/dbal/src/Exception/ConnectionException.php',
    'Doctrine\\DBAL\\Exception\\ConnectionLost' => $vendorDir . '/doctrine/dbal/src/Exception/ConnectionLost.php',
    'Doctrine\\DBAL\\Exception\\ConstraintViolationException' => $vendorDir . '/doctrine/dbal/src/Exception/ConstraintViolationException.php',
    'Doctrine\\DBAL\\Exception\\DatabaseDoesNotExist' => $vendorDir . '/doctrine/dbal/src/Exception/DatabaseDoesNotExist.php',
    'Doctrine\\DBAL\\Exception\\DatabaseObjectExistsException' => $vendorDir . '/doctrine/dbal/src/Exception/DatabaseObjectExistsException.php',
    'Doctrine\\DBAL\\Exception\\DatabaseObjectNotFoundException' => $vendorDir . '/doctrine/dbal/src/Exception/DatabaseObjectNotFoundException.php',
    'Doctrine\\DBAL\\Exception\\DatabaseRequired' => $vendorDir . '/doctrine/dbal/src/Exception/DatabaseRequired.php',
    'Doctrine\\DBAL\\Exception\\DeadlockException' => $vendorDir . '/doctrine/dbal/src/Exception/DeadlockException.php',
    'Doctrine\\DBAL\\Exception\\DriverException' => $vendorDir . '/doctrine/dbal/src/Exception/DriverException.php',
    'Doctrine\\DBAL\\Exception\\ForeignKeyConstraintViolationException' => $vendorDir . '/doctrine/dbal/src/Exception/ForeignKeyConstraintViolationException.php',
    'Doctrine\\DBAL\\Exception\\InvalidArgumentException' => $vendorDir . '/doctrine/dbal/src/Exception/InvalidArgumentException.php',
    'Doctrine\\DBAL\\Exception\\InvalidFieldNameException' => $vendorDir . '/doctrine/dbal/src/Exception/InvalidFieldNameException.php',
    'Doctrine\\DBAL\\Exception\\InvalidLockMode' => $vendorDir . '/doctrine/dbal/src/Exception/InvalidLockMode.php',
    'Doctrine\\DBAL\\Exception\\LockWaitTimeoutException' => $vendorDir . '/doctrine/dbal/src/Exception/LockWaitTimeoutException.php',
    'Doctrine\\DBAL\\Exception\\MalformedDsnException' => $vendorDir . '/doctrine/dbal/src/Exception/MalformedDsnException.php',
    'Doctrine\\DBAL\\Exception\\NoKeyValue' => $vendorDir . '/doctrine/dbal/src/Exception/NoKeyValue.php',
    'Doctrine\\DBAL\\Exception\\NonUniqueFieldNameException' => $vendorDir . '/doctrine/dbal/src/Exception/NonUniqueFieldNameException.php',
    'Doctrine\\DBAL\\Exception\\NotNullConstraintViolationException' => $vendorDir . '/doctrine/dbal/src/Exception/NotNullConstraintViolationException.php',
    'Doctrine\\DBAL\\Exception\\ReadOnlyException' => $vendorDir . '/doctrine/dbal/src/Exception/ReadOnlyException.php',
    'Doctrine\\DBAL\\Exception\\RetryableException' => $vendorDir . '/doctrine/dbal/src/Exception/RetryableException.php',
    'Doctrine\\DBAL\\Exception\\SchemaDoesNotExist' => $vendorDir . '/doctrine/dbal/src/Exception/SchemaDoesNotExist.php',
    'Doctrine\\DBAL\\Exception\\ServerException' => $vendorDir . '/doctrine/dbal/src/Exception/ServerException.php',
    'Doctrine\\DBAL\\Exception\\SyntaxErrorException' => $vendorDir . '/doctrine/dbal/src/Exception/SyntaxErrorException.php',
    'Doctrine\\DBAL\\Exception\\TableExistsException' => $vendorDir . '/doctrine/dbal/src/Exception/TableExistsException.php',
    'Doctrine\\DBAL\\Exception\\TableNotFoundException' => $vendorDir . '/doctrine/dbal/src/Exception/TableNotFoundException.php',
    'Doctrine\\DBAL\\Exception\\UniqueConstraintViolationException' => $vendorDir . '/doctrine/dbal/src/Exception/UniqueConstraintViolationException.php',
    'Doctrine\\DBAL\\ExpandArrayParameters' => $vendorDir . '/doctrine/dbal/src/ExpandArrayParameters.php',
    'Doctrine\\DBAL\\FetchMode' => $vendorDir . '/doctrine/dbal/src/FetchMode.php',
    'Doctrine\\DBAL\\Id\\TableGenerator' => $vendorDir . '/doctrine/dbal/src/Id/TableGenerator.php',
    'Doctrine\\DBAL\\Id\\TableGeneratorSchemaVisitor' => $vendorDir . '/doctrine/dbal/src/Id/TableGeneratorSchemaVisitor.php',
    'Doctrine\\DBAL\\LockMode' => $vendorDir . '/doctrine/dbal/src/LockMode.php',
    'Doctrine\\DBAL\\Logging\\Connection' => $vendorDir . '/doctrine/dbal/src/Logging/Connection.php',
    'Doctrine\\DBAL\\Logging\\DebugStack' => $vendorDir . '/doctrine/dbal/src/Logging/DebugStack.php',
    'Doctrine\\DBAL\\Logging\\Driver' => $vendorDir . '/doctrine/dbal/src/Logging/Driver.php',
    'Doctrine\\DBAL\\Logging\\LoggerChain' => $vendorDir . '/doctrine/dbal/src/Logging/LoggerChain.php',
    'Doctrine\\DBAL\\Logging\\Middleware' => $vendorDir . '/doctrine/dbal/src/Logging/Middleware.php',
    'Doctrine\\DBAL\\Logging\\SQLLogger' => $vendorDir . '/doctrine/dbal/src/Logging/SQLLogger.php',
    'Doctrine\\DBAL\\Logging\\Statement' => $vendorDir . '/doctrine/dbal/src/Logging/Statement.php',
    'Doctrine\\DBAL\\ParameterType' => $vendorDir . '/doctrine/dbal/src/ParameterType.php',
    'Doctrine\\DBAL\\Platforms\\AbstractMySQLPlatform' => $vendorDir . '/doctrine/dbal/src/Platforms/AbstractMySQLPlatform.php',
    'Doctrine\\DBAL\\Platforms\\AbstractPlatform' => $vendorDir . '/doctrine/dbal/src/Platforms/AbstractPlatform.php',
    'Doctrine\\DBAL\\Platforms\\DB2111Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/DB2111Platform.php',
    'Doctrine\\DBAL\\Platforms\\DB2Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/DB2Platform.php',
    'Doctrine\\DBAL\\Platforms\\DateIntervalUnit' => $vendorDir . '/doctrine/dbal/src/Platforms/DateIntervalUnit.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\DB2Keywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/DB2Keywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\KeywordList' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/KeywordList.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\MariaDBKeywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/MariaDBKeywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\MariaDb102Keywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/MariaDb102Keywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\MySQL57Keywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/MySQL57Keywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\MySQL80Keywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/MySQL80Keywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\MySQL84Keywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/MySQL84Keywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\MySQLKeywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/MySQLKeywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\OracleKeywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/OracleKeywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\PostgreSQL100Keywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/PostgreSQL100Keywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\PostgreSQL94Keywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/PostgreSQL94Keywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\PostgreSQLKeywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/PostgreSQLKeywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\ReservedKeywordsValidator' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/ReservedKeywordsValidator.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\SQLServer2012Keywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/SQLServer2012Keywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\SQLServerKeywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/SQLServerKeywords.php',
    'Doctrine\\DBAL\\Platforms\\Keywords\\SQLiteKeywords' => $vendorDir . '/doctrine/dbal/src/Platforms/Keywords/SQLiteKeywords.php',
    'Doctrine\\DBAL\\Platforms\\MariaDBPlatform' => $vendorDir . '/doctrine/dbal/src/Platforms/MariaDBPlatform.php',
    'Doctrine\\DBAL\\Platforms\\MariaDb1010Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/MariaDb1010Platform.php',
    'Doctrine\\DBAL\\Platforms\\MariaDb1027Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/MariaDb1027Platform.php',
    'Doctrine\\DBAL\\Platforms\\MariaDb1043Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/MariaDb1043Platform.php',
    'Doctrine\\DBAL\\Platforms\\MariaDb1052Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/MariaDb1052Platform.php',
    'Doctrine\\DBAL\\Platforms\\MariaDb1060Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/MariaDb1060Platform.php',
    'Doctrine\\DBAL\\Platforms\\MySQL57Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/MySQL57Platform.php',
    'Doctrine\\DBAL\\Platforms\\MySQL80Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/MySQL80Platform.php',
    'Doctrine\\DBAL\\Platforms\\MySQL84Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/MySQL84Platform.php',
    'Doctrine\\DBAL\\Platforms\\MySQLPlatform' => $vendorDir . '/doctrine/dbal/src/Platforms/MySQLPlatform.php',
    'Doctrine\\DBAL\\Platforms\\MySQL\\CollationMetadataProvider' => $vendorDir . '/doctrine/dbal/src/Platforms/MySQL/CollationMetadataProvider.php',
    'Doctrine\\DBAL\\Platforms\\MySQL\\CollationMetadataProvider\\CachingCollationMetadataProvider' => $vendorDir . '/doctrine/dbal/src/Platforms/MySQL/CollationMetadataProvider/CachingCollationMetadataProvider.php',
    'Doctrine\\DBAL\\Platforms\\MySQL\\CollationMetadataProvider\\ConnectionCollationMetadataProvider' => $vendorDir . '/doctrine/dbal/src/Platforms/MySQL/CollationMetadataProvider/ConnectionCollationMetadataProvider.php',
    'Doctrine\\DBAL\\Platforms\\MySQL\\Comparator' => $vendorDir . '/doctrine/dbal/src/Platforms/MySQL/Comparator.php',
    'Doctrine\\DBAL\\Platforms\\OraclePlatform' => $vendorDir . '/doctrine/dbal/src/Platforms/OraclePlatform.php',
    'Doctrine\\DBAL\\Platforms\\PostgreSQL100Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/PostgreSQL100Platform.php',
    'Doctrine\\DBAL\\Platforms\\PostgreSQL120Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/PostgreSQL120Platform.php',
    'Doctrine\\DBAL\\Platforms\\PostgreSQL94Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/PostgreSQL94Platform.php',
    'Doctrine\\DBAL\\Platforms\\PostgreSQLPlatform' => $vendorDir . '/doctrine/dbal/src/Platforms/PostgreSQLPlatform.php',
    'Doctrine\\DBAL\\Platforms\\SQLServer2012Platform' => $vendorDir . '/doctrine/dbal/src/Platforms/SQLServer2012Platform.php',
    'Doctrine\\DBAL\\Platforms\\SQLServerPlatform' => $vendorDir . '/doctrine/dbal/src/Platforms/SQLServerPlatform.php',
    'Doctrine\\DBAL\\Platforms\\SQLServer\\Comparator' => $vendorDir . '/doctrine/dbal/src/Platforms/SQLServer/Comparator.php',
    'Doctrine\\DBAL\\Platforms\\SQLServer\\SQL\\Builder\\SQLServerSelectSQLBuilder' => $vendorDir . '/doctrine/dbal/src/Platforms/SQLServer/SQL/Builder/SQLServerSelectSQLBuilder.php',
    'Doctrine\\DBAL\\Platforms\\SQLite\\Comparator' => $vendorDir . '/doctrine/dbal/src/Platforms/SQLite/Comparator.php',
    'Doctrine\\DBAL\\Platforms\\SqlitePlatform' => $vendorDir . '/doctrine/dbal/src/Platforms/SqlitePlatform.php',
    'Doctrine\\DBAL\\Platforms\\TrimMode' => $vendorDir . '/doctrine/dbal/src/Platforms/TrimMode.php',
    'Doctrine\\DBAL\\Portability\\Connection' => $vendorDir . '/doctrine/dbal/src/Portability/Connection.php',
    'Doctrine\\DBAL\\Portability\\Converter' => $vendorDir . '/doctrine/dbal/src/Portability/Converter.php',
    'Doctrine\\DBAL\\Portability\\Driver' => $vendorDir . '/doctrine/dbal/src/Portability/Driver.php',
    'Doctrine\\DBAL\\Portability\\Middleware' => $vendorDir . '/doctrine/dbal/src/Portability/Middleware.php',
    'Doctrine\\DBAL\\Portability\\OptimizeFlags' => $vendorDir . '/doctrine/dbal/src/Portability/OptimizeFlags.php',
    'Doctrine\\DBAL\\Portability\\Result' => $vendorDir . '/doctrine/dbal/src/Portability/Result.php',
    'Doctrine\\DBAL\\Portability\\Statement' => $vendorDir . '/doctrine/dbal/src/Portability/Statement.php',
    'Doctrine\\DBAL\\Query' => $vendorDir . '/doctrine/dbal/src/Query.php',
    'Doctrine\\DBAL\\Query\\Expression\\CompositeExpression' => $vendorDir . '/doctrine/dbal/src/Query/Expression/CompositeExpression.php',
    'Doctrine\\DBAL\\Query\\Expression\\ExpressionBuilder' => $vendorDir . '/doctrine/dbal/src/Query/Expression/ExpressionBuilder.php',
    'Doctrine\\DBAL\\Query\\ForUpdate' => $vendorDir . '/doctrine/dbal/src/Query/ForUpdate.php',
    'Doctrine\\DBAL\\Query\\ForUpdate\\ConflictResolutionMode' => $vendorDir . '/doctrine/dbal/src/Query/ForUpdate/ConflictResolutionMode.php',
    'Doctrine\\DBAL\\Query\\Limit' => $vendorDir . '/doctrine/dbal/src/Query/Limit.php',
    'Doctrine\\DBAL\\Query\\QueryBuilder' => $vendorDir . '/doctrine/dbal/src/Query/QueryBuilder.php',
    'Doctrine\\DBAL\\Query\\QueryException' => $vendorDir . '/doctrine/dbal/src/Query/QueryException.php',
    'Doctrine\\DBAL\\Query\\SelectQuery' => $vendorDir . '/doctrine/dbal/src/Query/SelectQuery.php',
    'Doctrine\\DBAL\\Result' => $vendorDir . '/doctrine/dbal/src/Result.php',
    'Doctrine\\DBAL\\SQL\\Builder\\CreateSchemaObjectsSQLBuilder' => $vendorDir . '/doctrine/dbal/src/SQL/Builder/CreateSchemaObjectsSQLBuilder.php',
    'Doctrine\\DBAL\\SQL\\Builder\\DefaultSelectSQLBuilder' => $vendorDir . '/doctrine/dbal/src/SQL/Builder/DefaultSelectSQLBuilder.php',
    'Doctrine\\DBAL\\SQL\\Builder\\DropSchemaObjectsSQLBuilder' => $vendorDir . '/doctrine/dbal/src/SQL/Builder/DropSchemaObjectsSQLBuilder.php',
    'Doctrine\\DBAL\\SQL\\Builder\\SelectSQLBuilder' => $vendorDir . '/doctrine/dbal/src/SQL/Builder/SelectSQLBuilder.php',
    'Doctrine\\DBAL\\SQL\\Parser' => $vendorDir . '/doctrine/dbal/src/SQL/Parser.php',
    'Doctrine\\DBAL\\SQL\\Parser\\Exception' => $vendorDir . '/doctrine/dbal/src/SQL/Parser/Exception.php',
    'Doctrine\\DBAL\\SQL\\Parser\\Exception\\RegularExpressionError' => $vendorDir . '/doctrine/dbal/src/SQL/Parser/Exception/RegularExpressionError.php',
    'Doctrine\\DBAL\\SQL\\Parser\\Visitor' => $vendorDir . '/doctrine/dbal/src/SQL/Parser/Visitor.php',
    'Doctrine\\DBAL\\Schema\\AbstractAsset' => $vendorDir . '/doctrine/dbal/src/Schema/AbstractAsset.php',
    'Doctrine\\DBAL\\Schema\\AbstractSchemaManager' => $vendorDir . '/doctrine/dbal/src/Schema/AbstractSchemaManager.php',
    'Doctrine\\DBAL\\Schema\\Column' => $vendorDir . '/doctrine/dbal/src/Schema/Column.php',
    'Doctrine\\DBAL\\Schema\\ColumnDiff' => $vendorDir . '/doctrine/dbal/src/Schema/ColumnDiff.php',
    'Doctrine\\DBAL\\Schema\\Comparator' => $vendorDir . '/doctrine/dbal/src/Schema/Comparator.php',
    'Doctrine\\DBAL\\Schema\\Constraint' => $vendorDir . '/doctrine/dbal/src/Schema/Constraint.php',
    'Doctrine\\DBAL\\Schema\\DB2SchemaManager' => $vendorDir . '/doctrine/dbal/src/Schema/DB2SchemaManager.php',
    'Doctrine\\DBAL\\Schema\\DefaultSchemaManagerFactory' => $vendorDir . '/doctrine/dbal/src/Schema/DefaultSchemaManagerFactory.php',
    'Doctrine\\DBAL\\Schema\\Exception\\ColumnAlreadyExists' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/ColumnAlreadyExists.php',
    'Doctrine\\DBAL\\Schema\\Exception\\ColumnDoesNotExist' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/ColumnDoesNotExist.php',
    'Doctrine\\DBAL\\Schema\\Exception\\ForeignKeyDoesNotExist' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/ForeignKeyDoesNotExist.php',
    'Doctrine\\DBAL\\Schema\\Exception\\IndexAlreadyExists' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/IndexAlreadyExists.php',
    'Doctrine\\DBAL\\Schema\\Exception\\IndexDoesNotExist' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/IndexDoesNotExist.php',
    'Doctrine\\DBAL\\Schema\\Exception\\IndexNameInvalid' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/IndexNameInvalid.php',
    'Doctrine\\DBAL\\Schema\\Exception\\InvalidTableName' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/InvalidTableName.php',
    'Doctrine\\DBAL\\Schema\\Exception\\NamedForeignKeyRequired' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/NamedForeignKeyRequired.php',
    'Doctrine\\DBAL\\Schema\\Exception\\NamespaceAlreadyExists' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/NamespaceAlreadyExists.php',
    'Doctrine\\DBAL\\Schema\\Exception\\SequenceAlreadyExists' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/SequenceAlreadyExists.php',
    'Doctrine\\DBAL\\Schema\\Exception\\SequenceDoesNotExist' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/SequenceDoesNotExist.php',
    'Doctrine\\DBAL\\Schema\\Exception\\TableAlreadyExists' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/TableAlreadyExists.php',
    'Doctrine\\DBAL\\Schema\\Exception\\TableDoesNotExist' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/TableDoesNotExist.php',
    'Doctrine\\DBAL\\Schema\\Exception\\UniqueConstraintDoesNotExist' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/UniqueConstraintDoesNotExist.php',
    'Doctrine\\DBAL\\Schema\\Exception\\UnknownColumnOption' => $vendorDir . '/doctrine/dbal/src/Schema/Exception/UnknownColumnOption.php',
    'Doctrine\\DBAL\\Schema\\ForeignKeyConstraint' => $vendorDir . '/doctrine/dbal/src/Schema/ForeignKeyConstraint.php',
    'Doctrine\\DBAL\\Schema\\Identifier' => $vendorDir . '/doctrine/dbal/src/Schema/Identifier.php',
    'Doctrine\\DBAL\\Schema\\Index' => $vendorDir . '/doctrine/dbal/src/Schema/Index.php',
    'Doctrine\\DBAL\\Schema\\LegacySchemaManagerFactory' => $vendorDir . '/doctrine/dbal/src/Schema/LegacySchemaManagerFactory.php',
    'Doctrine\\DBAL\\Schema\\MySQLSchemaManager' => $vendorDir . '/doctrine/dbal/src/Schema/MySQLSchemaManager.php',
    'Doctrine\\DBAL\\Schema\\OracleSchemaManager' => $vendorDir . '/doctrine/dbal/src/Schema/OracleSchemaManager.php',
    'Doctrine\\DBAL\\Schema\\PostgreSQLSchemaManager' => $vendorDir . '/doctrine/dbal/src/Schema/PostgreSQLSchemaManager.php',
    'Doctrine\\DBAL\\Schema\\SQLServerSchemaManager' => $vendorDir . '/doctrine/dbal/src/Schema/SQLServerSchemaManager.php',
    'Doctrine\\DBAL\\Schema\\Schema' => $vendorDir . '/doctrine/dbal/src/Schema/Schema.php',
    'Doctrine\\DBAL\\Schema\\SchemaConfig' => $vendorDir . '/doctrine/dbal/src/Schema/SchemaConfig.php',
    'Doctrine\\DBAL\\Schema\\SchemaDiff' => $vendorDir . '/doctrine/dbal/src/Schema/SchemaDiff.php',
    'Doctrine\\DBAL\\Schema\\SchemaException' => $vendorDir . '/doctrine/dbal/src/Schema/SchemaException.php',
    'Doctrine\\DBAL\\Schema\\SchemaManagerFactory' => $vendorDir . '/doctrine/dbal/src/Schema/SchemaManagerFactory.php',
    'Doctrine\\DBAL\\Schema\\Sequence' => $vendorDir . '/doctrine/dbal/src/Schema/Sequence.php',
    'Doctrine\\DBAL\\Schema\\SqliteSchemaManager' => $vendorDir . '/doctrine/dbal/src/Schema/SqliteSchemaManager.php',
    'Doctrine\\DBAL\\Schema\\Table' => $vendorDir . '/doctrine/dbal/src/Schema/Table.php',
    'Doctrine\\DBAL\\Schema\\TableDiff' => $vendorDir . '/doctrine/dbal/src/Schema/TableDiff.php',
    'Doctrine\\DBAL\\Schema\\UniqueConstraint' => $vendorDir . '/doctrine/dbal/src/Schema/UniqueConstraint.php',
    'Doctrine\\DBAL\\Schema\\View' => $vendorDir . '/doctrine/dbal/src/Schema/View.php',
    'Doctrine\\DBAL\\Schema\\Visitor\\AbstractVisitor' => $vendorDir . '/doctrine/dbal/src/Schema/Visitor/AbstractVisitor.php',
    'Doctrine\\DBAL\\Schema\\Visitor\\CreateSchemaSqlCollector' => $vendorDir . '/doctrine/dbal/src/Schema/Visitor/CreateSchemaSqlCollector.php',
    'Doctrine\\DBAL\\Schema\\Visitor\\DropSchemaSqlCollector' => $vendorDir . '/doctrine/dbal/src/Schema/Visitor/DropSchemaSqlCollector.php',
    'Doctrine\\DBAL\\Schema\\Visitor\\Graphviz' => $vendorDir . '/doctrine/dbal/src/Schema/Visitor/Graphviz.php',
    'Doctrine\\DBAL\\Schema\\Visitor\\NamespaceVisitor' => $vendorDir . '/doctrine/dbal/src/Schema/Visitor/NamespaceVisitor.php',
    'Doctrine\\DBAL\\Schema\\Visitor\\RemoveNamespacedAssets' => $vendorDir . '/doctrine/dbal/src/Schema/Visitor/RemoveNamespacedAssets.php',
    'Doctrine\\DBAL\\Schema\\Visitor\\Visitor' => $vendorDir . '/doctrine/dbal/src/Schema/Visitor/Visitor.php',
    'Doctrine\\DBAL\\Statement' => $vendorDir . '/doctrine/dbal/src/Statement.php',
    'Doctrine\\DBAL\\Tools\\Console\\Command\\CommandCompatibility' => $vendorDir . '/doctrine/dbal/src/Tools/Console/Command/CommandCompatibility.php',
    'Doctrine\\DBAL\\Tools\\Console\\Command\\ReservedWordsCommand' => $vendorDir . '/doctrine/dbal/src/Tools/Console/Command/ReservedWordsCommand.php',
    'Doctrine\\DBAL\\Tools\\Console\\Command\\RunSqlCommand' => $vendorDir . '/doctrine/dbal/src/Tools/Console/Command/RunSqlCommand.php',
    'Doctrine\\DBAL\\Tools\\Console\\ConnectionNotFound' => $vendorDir . '/doctrine/dbal/src/Tools/Console/ConnectionNotFound.php',
    'Doctrine\\DBAL\\Tools\\Console\\ConnectionProvider' => $vendorDir . '/doctrine/dbal/src/Tools/Console/ConnectionProvider.php',
    'Doctrine\\DBAL\\Tools\\Console\\ConnectionProvider\\SingleConnectionProvider' => $vendorDir . '/doctrine/dbal/src/Tools/Console/ConnectionProvider/SingleConnectionProvider.php',
    'Doctrine\\DBAL\\Tools\\Console\\ConsoleRunner' => $vendorDir . '/doctrine/dbal/src/Tools/Console/ConsoleRunner.php',
    'Doctrine\\DBAL\\Tools\\DsnParser' => $vendorDir . '/doctrine/dbal/src/Tools/DsnParser.php',
    'Doctrine\\DBAL\\TransactionIsolationLevel' => $vendorDir . '/doctrine/dbal/src/TransactionIsolationLevel.php',
    'Doctrine\\DBAL\\Types\\ArrayType' => $vendorDir . '/doctrine/dbal/src/Types/ArrayType.php',
    'Doctrine\\DBAL\\Types\\AsciiStringType' => $vendorDir . '/doctrine/dbal/src/Types/AsciiStringType.php',
    'Doctrine\\DBAL\\Types\\BigIntType' => $vendorDir . '/doctrine/dbal/src/Types/BigIntType.php',
    'Doctrine\\DBAL\\Types\\BinaryType' => $vendorDir . '/doctrine/dbal/src/Types/BinaryType.php',
    'Doctrine\\DBAL\\Types\\BlobType' => $vendorDir . '/doctrine/dbal/src/Types/BlobType.php',
    'Doctrine\\DBAL\\Types\\BooleanType' => $vendorDir . '/doctrine/dbal/src/Types/BooleanType.php',
    'Doctrine\\DBAL\\Types\\ConversionException' => $vendorDir . '/doctrine/dbal/src/Types/ConversionException.php',
    'Doctrine\\DBAL\\Types\\DateImmutableType' => $vendorDir . '/doctrine/dbal/src/Types/DateImmutableType.php',
    'Doctrine\\DBAL\\Types\\DateIntervalType' => $vendorDir . '/doctrine/dbal/src/Types/DateIntervalType.php',
    'Doctrine\\DBAL\\Types\\DateTimeImmutableType' => $vendorDir . '/doctrine/dbal/src/Types/DateTimeImmutableType.php',
    'Doctrine\\DBAL\\Types\\DateTimeType' => $vendorDir . '/doctrine/dbal/src/Types/DateTimeType.php',
    'Doctrine\\DBAL\\Types\\DateTimeTzImmutableType' => $vendorDir . '/doctrine/dbal/src/Types/DateTimeTzImmutableType.php',
    'Doctrine\\DBAL\\Types\\DateTimeTzType' => $vendorDir . '/doctrine/dbal/src/Types/DateTimeTzType.php',
    'Doctrine\\DBAL\\Types\\DateType' => $vendorDir . '/doctrine/dbal/src/Types/DateType.php',
    'Doctrine\\DBAL\\Types\\DecimalType' => $vendorDir . '/doctrine/dbal/src/Types/DecimalType.php',
    'Doctrine\\DBAL\\Types\\FloatType' => $vendorDir . '/doctrine/dbal/src/Types/FloatType.php',
    'Doctrine\\DBAL\\Types\\GuidType' => $vendorDir . '/doctrine/dbal/src/Types/GuidType.php',
    'Doctrine\\DBAL\\Types\\IntegerType' => $vendorDir . '/doctrine/dbal/src/Types/IntegerType.php',
    'Doctrine\\DBAL\\Types\\JsonType' => $vendorDir . '/doctrine/dbal/src/Types/JsonType.php',
    'Doctrine\\DBAL\\Types\\ObjectType' => $vendorDir . '/doctrine/dbal/src/Types/ObjectType.php',
    'Doctrine\\DBAL\\Types\\PhpDateTimeMappingType' => $vendorDir . '/doctrine/dbal/src/Types/PhpDateTimeMappingType.php',
    'Doctrine\\DBAL\\Types\\PhpIntegerMappingType' => $vendorDir . '/doctrine/dbal/src/Types/PhpIntegerMappingType.php',
    'Doctrine\\DBAL\\Types\\SimpleArrayType' => $vendorDir . '/doctrine/dbal/src/Types/SimpleArrayType.php',
    'Doctrine\\DBAL\\Types\\SmallIntType' => $vendorDir . '/doctrine/dbal/src/Types/SmallIntType.php',
    'Doctrine\\DBAL\\Types\\StringType' => $vendorDir . '/doctrine/dbal/src/Types/StringType.php',
    'Doctrine\\DBAL\\Types\\TextType' => $vendorDir . '/doctrine/dbal/src/Types/TextType.php',
    'Doctrine\\DBAL\\Types\\TimeImmutableType' => $vendorDir . '/doctrine/dbal/src/Types/TimeImmutableType.php',
    'Doctrine\\DBAL\\Types\\TimeType' => $vendorDir . '/doctrine/dbal/src/Types/TimeType.php',
    'Doctrine\\DBAL\\Types\\Type' => $vendorDir . '/doctrine/dbal/src/Types/Type.php',
    'Doctrine\\DBAL\\Types\\TypeRegistry' => $vendorDir . '/doctrine/dbal/src/Types/TypeRegistry.php',
    'Doctrine\\DBAL\\Types\\Types' => $vendorDir . '/doctrine/dbal/src/Types/Types.php',
    'Doctrine\\DBAL\\Types\\VarDateTimeImmutableType' => $vendorDir . '/doctrine/dbal/src/Types/VarDateTimeImmutableType.php',
    'Doctrine\\DBAL\\Types\\VarDateTimeType' => $vendorDir . '/doctrine/dbal/src/Types/VarDateTimeType.php',
    'Doctrine\\DBAL\\VersionAwarePlatformDriver' => $vendorDir . '/doctrine/dbal/src/VersionAwarePlatformDriver.php',
    'Doctrine\\Deprecations\\Deprecation' => $vendorDir . '/doctrine/deprecations/src/Deprecation.php',
    'Doctrine\\Deprecations\\PHPUnit\\VerifyDeprecations' => $vendorDir . '/doctrine/deprecations/src/PHPUnit/VerifyDeprecations.php',
    'Doctrine\\Inflector\\CachedWordInflector' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/CachedWordInflector.php',
    'Doctrine\\Inflector\\GenericLanguageInflectorFactory' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/GenericLanguageInflectorFactory.php',
    'Doctrine\\Inflector\\Inflector' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Inflector.php',
    'Doctrine\\Inflector\\InflectorFactory' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/InflectorFactory.php',
    'Doctrine\\Inflector\\Language' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Language.php',
    'Doctrine\\Inflector\\LanguageInflectorFactory' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/LanguageInflectorFactory.php',
    'Doctrine\\Inflector\\NoopWordInflector' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/NoopWordInflector.php',
    'Doctrine\\Inflector\\Rules\\English\\Inflectible' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/English/Inflectible.php',
    'Doctrine\\Inflector\\Rules\\English\\InflectorFactory' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/English/InflectorFactory.php',
    'Doctrine\\Inflector\\Rules\\English\\Rules' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/English/Rules.php',
    'Doctrine\\Inflector\\Rules\\English\\Uninflected' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/English/Uninflected.php',
    'Doctrine\\Inflector\\Rules\\French\\Inflectible' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/French/Inflectible.php',
    'Doctrine\\Inflector\\Rules\\French\\InflectorFactory' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/French/InflectorFactory.php',
    'Doctrine\\Inflector\\Rules\\French\\Rules' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/French/Rules.php',
    'Doctrine\\Inflector\\Rules\\French\\Uninflected' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/French/Uninflected.php',
    'Doctrine\\Inflector\\Rules\\NorwegianBokmal\\Inflectible' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/NorwegianBokmal/Inflectible.php',
    'Doctrine\\Inflector\\Rules\\NorwegianBokmal\\InflectorFactory' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/NorwegianBokmal/InflectorFactory.php',
    'Doctrine\\Inflector\\Rules\\NorwegianBokmal\\Rules' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/NorwegianBokmal/Rules.php',
    'Doctrine\\Inflector\\Rules\\NorwegianBokmal\\Uninflected' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/NorwegianBokmal/Uninflected.php',
    'Doctrine\\Inflector\\Rules\\Pattern' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Pattern.php',
    'Doctrine\\Inflector\\Rules\\Patterns' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Patterns.php',
    'Doctrine\\Inflector\\Rules\\Portuguese\\Inflectible' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Portuguese/Inflectible.php',
    'Doctrine\\Inflector\\Rules\\Portuguese\\InflectorFactory' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Portuguese/InflectorFactory.php',
    'Doctrine\\Inflector\\Rules\\Portuguese\\Rules' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Portuguese/Rules.php',
    'Doctrine\\Inflector\\Rules\\Portuguese\\Uninflected' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Portuguese/Uninflected.php',
    'Doctrine\\Inflector\\Rules\\Ruleset' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Ruleset.php',
    'Doctrine\\Inflector\\Rules\\Spanish\\Inflectible' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Spanish/Inflectible.php',
    'Doctrine\\Inflector\\Rules\\Spanish\\InflectorFactory' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Spanish/InflectorFactory.php',
    'Doctrine\\Inflector\\Rules\\Spanish\\Rules' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Spanish/Rules.php',
    'Doctrine\\Inflector\\Rules\\Spanish\\Uninflected' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Spanish/Uninflected.php',
    'Doctrine\\Inflector\\Rules\\Substitution' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Substitution.php',
    'Doctrine\\Inflector\\Rules\\Substitutions' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Substitutions.php',
    'Doctrine\\Inflector\\Rules\\Transformation' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Transformation.php',
    'Doctrine\\Inflector\\Rules\\Transformations' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Transformations.php',
    'Doctrine\\Inflector\\Rules\\Turkish\\Inflectible' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Turkish/Inflectible.php',
    'Doctrine\\Inflector\\Rules\\Turkish\\InflectorFactory' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Turkish/InflectorFactory.php',
    'Doctrine\\Inflector\\Rules\\Turkish\\Rules' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Turkish/Rules.php',
    'Doctrine\\Inflector\\Rules\\Turkish\\Uninflected' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Turkish/Uninflected.php',
    'Doctrine\\Inflector\\Rules\\Word' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/Rules/Word.php',
    'Doctrine\\Inflector\\RulesetInflector' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/RulesetInflector.php',
    'Doctrine\\Inflector\\WordInflector' => $vendorDir . '/doctrine/inflector/lib/Doctrine/Inflector/WordInflector.php',
    'Dompdf\\Adapter\\CPDF' => $vendorDir . '/dompdf/dompdf/src/Adapter/CPDF.php',
    'Dompdf\\Adapter\\GD' => $vendorDir . '/dompdf/dompdf/src/Adapter/GD.php',
    'Dompdf\\Adapter\\PDFLib' => $vendorDir . '/dompdf/dompdf/src/Adapter/PDFLib.php',
    'Dompdf\\Canvas' => $vendorDir . '/dompdf/dompdf/src/Canvas.php',
    'Dompdf\\CanvasFactory' => $vendorDir . '/dompdf/dompdf/src/CanvasFactory.php',
    'Dompdf\\Cellmap' => $vendorDir . '/dompdf/dompdf/src/Cellmap.php',
    'Dompdf\\Cpdf' => $vendorDir . '/dompdf/dompdf/lib/Cpdf.php',
    'Dompdf\\Css\\AttributeTranslator' => $vendorDir . '/dompdf/dompdf/src/Css/AttributeTranslator.php',
    'Dompdf\\Css\\Color' => $vendorDir . '/dompdf/dompdf/src/Css/Color.php',
    'Dompdf\\Css\\Content\\Attr' => $vendorDir . '/dompdf/dompdf/src/Css/Content/Attr.php',
    'Dompdf\\Css\\Content\\CloseQuote' => $vendorDir . '/dompdf/dompdf/src/Css/Content/CloseQuote.php',
    'Dompdf\\Css\\Content\\ContentPart' => $vendorDir . '/dompdf/dompdf/src/Css/Content/ContentPart.php',
    'Dompdf\\Css\\Content\\Counter' => $vendorDir . '/dompdf/dompdf/src/Css/Content/Counter.php',
    'Dompdf\\Css\\Content\\Counters' => $vendorDir . '/dompdf/dompdf/src/Css/Content/Counters.php',
    'Dompdf\\Css\\Content\\NoCloseQuote' => $vendorDir . '/dompdf/dompdf/src/Css/Content/NoCloseQuote.php',
    'Dompdf\\Css\\Content\\NoOpenQuote' => $vendorDir . '/dompdf/dompdf/src/Css/Content/NoOpenQuote.php',
    'Dompdf\\Css\\Content\\OpenQuote' => $vendorDir . '/dompdf/dompdf/src/Css/Content/OpenQuote.php',
    'Dompdf\\Css\\Content\\StringPart' => $vendorDir . '/dompdf/dompdf/src/Css/Content/StringPart.php',
    'Dompdf\\Css\\Content\\Url' => $vendorDir . '/dompdf/dompdf/src/Css/Content/Url.php',
    'Dompdf\\Css\\Style' => $vendorDir . '/dompdf/dompdf/src/Css/Style.php',
    'Dompdf\\Css\\Stylesheet' => $vendorDir . '/dompdf/dompdf/src/Css/Stylesheet.php',
    'Dompdf\\Dompdf' => $vendorDir . '/dompdf/dompdf/src/Dompdf.php',
    'Dompdf\\Exception' => $vendorDir . '/dompdf/dompdf/src/Exception.php',
    'Dompdf\\Exception\\ImageException' => $vendorDir . '/dompdf/dompdf/src/Exception/ImageException.php',
    'Dompdf\\FontMetrics' => $vendorDir . '/dompdf/dompdf/src/FontMetrics.php',
    'Dompdf\\Frame' => $vendorDir . '/dompdf/dompdf/src/Frame.php',
    'Dompdf\\FrameDecorator\\AbstractFrameDecorator' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/AbstractFrameDecorator.php',
    'Dompdf\\FrameDecorator\\Block' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/Block.php',
    'Dompdf\\FrameDecorator\\Image' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/Image.php',
    'Dompdf\\FrameDecorator\\Inline' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/Inline.php',
    'Dompdf\\FrameDecorator\\ListBullet' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/ListBullet.php',
    'Dompdf\\FrameDecorator\\ListBulletImage' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/ListBulletImage.php',
    'Dompdf\\FrameDecorator\\NullFrameDecorator' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/NullFrameDecorator.php',
    'Dompdf\\FrameDecorator\\Page' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/Page.php',
    'Dompdf\\FrameDecorator\\Table' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/Table.php',
    'Dompdf\\FrameDecorator\\TableCell' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/TableCell.php',
    'Dompdf\\FrameDecorator\\TableRow' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/TableRow.php',
    'Dompdf\\FrameDecorator\\TableRowGroup' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/TableRowGroup.php',
    'Dompdf\\FrameDecorator\\Text' => $vendorDir . '/dompdf/dompdf/src/FrameDecorator/Text.php',
    'Dompdf\\FrameReflower\\AbstractFrameReflower' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/AbstractFrameReflower.php',
    'Dompdf\\FrameReflower\\Block' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/Block.php',
    'Dompdf\\FrameReflower\\Image' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/Image.php',
    'Dompdf\\FrameReflower\\Inline' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/Inline.php',
    'Dompdf\\FrameReflower\\ListBullet' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/ListBullet.php',
    'Dompdf\\FrameReflower\\NullFrameReflower' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/NullFrameReflower.php',
    'Dompdf\\FrameReflower\\Page' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/Page.php',
    'Dompdf\\FrameReflower\\Table' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/Table.php',
    'Dompdf\\FrameReflower\\TableCell' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/TableCell.php',
    'Dompdf\\FrameReflower\\TableRow' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/TableRow.php',
    'Dompdf\\FrameReflower\\TableRowGroup' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/TableRowGroup.php',
    'Dompdf\\FrameReflower\\Text' => $vendorDir . '/dompdf/dompdf/src/FrameReflower/Text.php',
    'Dompdf\\Frame\\Factory' => $vendorDir . '/dompdf/dompdf/src/Frame/Factory.php',
    'Dompdf\\Frame\\FrameListIterator' => $vendorDir . '/dompdf/dompdf/src/Frame/FrameListIterator.php',
    'Dompdf\\Frame\\FrameTree' => $vendorDir . '/dompdf/dompdf/src/Frame/FrameTree.php',
    'Dompdf\\Frame\\FrameTreeIterator' => $vendorDir . '/dompdf/dompdf/src/Frame/FrameTreeIterator.php',
    'Dompdf\\Helpers' => $vendorDir . '/dompdf/dompdf/src/Helpers.php',
    'Dompdf\\Image\\Cache' => $vendorDir . '/dompdf/dompdf/src/Image/Cache.php',
    'Dompdf\\JavascriptEmbedder' => $vendorDir . '/dompdf/dompdf/src/JavascriptEmbedder.php',
    'Dompdf\\LineBox' => $vendorDir . '/dompdf/dompdf/src/LineBox.php',
    'Dompdf\\Options' => $vendorDir . '/dompdf/dompdf/src/Options.php',
    'Dompdf\\PhpEvaluator' => $vendorDir . '/dompdf/dompdf/src/PhpEvaluator.php',
    'Dompdf\\Positioner\\Absolute' => $vendorDir . '/dompdf/dompdf/src/Positioner/Absolute.php',
    'Dompdf\\Positioner\\AbstractPositioner' => $vendorDir . '/dompdf/dompdf/src/Positioner/AbstractPositioner.php',
    'Dompdf\\Positioner\\Block' => $vendorDir . '/dompdf/dompdf/src/Positioner/Block.php',
    'Dompdf\\Positioner\\Fixed' => $vendorDir . '/dompdf/dompdf/src/Positioner/Fixed.php',
    'Dompdf\\Positioner\\Inline' => $vendorDir . '/dompdf/dompdf/src/Positioner/Inline.php',
    'Dompdf\\Positioner\\ListBullet' => $vendorDir . '/dompdf/dompdf/src/Positioner/ListBullet.php',
    'Dompdf\\Positioner\\NullPositioner' => $vendorDir . '/dompdf/dompdf/src/Positioner/NullPositioner.php',
    'Dompdf\\Positioner\\TableCell' => $vendorDir . '/dompdf/dompdf/src/Positioner/TableCell.php',
    'Dompdf\\Positioner\\TableRow' => $vendorDir . '/dompdf/dompdf/src/Positioner/TableRow.php',
    'Dompdf\\Renderer' => $vendorDir . '/dompdf/dompdf/src/Renderer.php',
    'Dompdf\\Renderer\\AbstractRenderer' => $vendorDir . '/dompdf/dompdf/src/Renderer/AbstractRenderer.php',
    'Dompdf\\Renderer\\Block' => $vendorDir . '/dompdf/dompdf/src/Renderer/Block.php',
    'Dompdf\\Renderer\\Image' => $vendorDir . '/dompdf/dompdf/src/Renderer/Image.php',
    'Dompdf\\Renderer\\Inline' => $vendorDir . '/dompdf/dompdf/src/Renderer/Inline.php',
    'Dompdf\\Renderer\\ListBullet' => $vendorDir . '/dompdf/dompdf/src/Renderer/ListBullet.php',
    'Dompdf\\Renderer\\TableCell' => $vendorDir . '/dompdf/dompdf/src/Renderer/TableCell.php',
    'Dompdf\\Renderer\\TableRow' => $vendorDir . '/dompdf/dompdf/src/Renderer/TableRow.php',
    'Dompdf\\Renderer\\TableRowGroup' => $vendorDir . '/dompdf/dompdf/src/Renderer/TableRowGroup.php',
    'Dompdf\\Renderer\\Text' => $vendorDir . '/dompdf/dompdf/src/Renderer/Text.php',
    'Dotenv\\Dotenv' => $vendorDir . '/vlucas/phpdotenv/src/Dotenv.php',
    'Dotenv\\Exception\\ExceptionInterface' => $vendorDir . '/vlucas/phpdotenv/src/Exception/ExceptionInterface.php',
    'Dotenv\\Exception\\InvalidEncodingException' => $vendorDir . '/vlucas/phpdotenv/src/Exception/InvalidEncodingException.php',
    'Dotenv\\Exception\\InvalidFileException' => $vendorDir . '/vlucas/phpdotenv/src/Exception/InvalidFileException.php',
    'Dotenv\\Exception\\InvalidPathException' => $vendorDir . '/vlucas/phpdotenv/src/Exception/InvalidPathException.php',
    'Dotenv\\Exception\\ValidationException' => $vendorDir . '/vlucas/phpdotenv/src/Exception/ValidationException.php',
    'Dotenv\\Loader\\Loader' => $vendorDir . '/vlucas/phpdotenv/src/Loader/Loader.php',
    'Dotenv\\Loader\\LoaderInterface' => $vendorDir . '/vlucas/phpdotenv/src/Loader/LoaderInterface.php',
    'Dotenv\\Loader\\Resolver' => $vendorDir . '/vlucas/phpdotenv/src/Loader/Resolver.php',
    'Dotenv\\Parser\\Entry' => $vendorDir . '/vlucas/phpdotenv/src/Parser/Entry.php',
    'Dotenv\\Parser\\EntryParser' => $vendorDir . '/vlucas/phpdotenv/src/Parser/EntryParser.php',
    'Dotenv\\Parser\\Lexer' => $vendorDir . '/vlucas/phpdotenv/src/Parser/Lexer.php',
    'Dotenv\\Parser\\Lines' => $vendorDir . '/vlucas/phpdotenv/src/Parser/Lines.php',
    'Dotenv\\Parser\\Parser' => $vendorDir . '/vlucas/phpdotenv/src/Parser/Parser.php',
    'Dotenv\\Parser\\ParserInterface' => $vendorDir . '/vlucas/phpdotenv/src/Parser/ParserInterface.php',
    'Dotenv\\Parser\\Value' => $vendorDir . '/vlucas/phpdotenv/src/Parser/Value.php',
    'Dotenv\\Repository\\AdapterRepository' => $vendorDir . '/vlucas/phpdotenv/src/Repository/AdapterRepository.php',
    'Dotenv\\Repository\\Adapter\\AdapterInterface' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/AdapterInterface.php',
    'Dotenv\\Repository\\Adapter\\ApacheAdapter' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/ApacheAdapter.php',
    'Dotenv\\Repository\\Adapter\\ArrayAdapter' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/ArrayAdapter.php',
    'Dotenv\\Repository\\Adapter\\EnvConstAdapter' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/EnvConstAdapter.php',
    'Dotenv\\Repository\\Adapter\\GuardedWriter' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/GuardedWriter.php',
    'Dotenv\\Repository\\Adapter\\ImmutableWriter' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/ImmutableWriter.php',
    'Dotenv\\Repository\\Adapter\\MultiReader' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/MultiReader.php',
    'Dotenv\\Repository\\Adapter\\MultiWriter' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/MultiWriter.php',
    'Dotenv\\Repository\\Adapter\\PutenvAdapter' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/PutenvAdapter.php',
    'Dotenv\\Repository\\Adapter\\ReaderInterface' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/ReaderInterface.php',
    'Dotenv\\Repository\\Adapter\\ReplacingWriter' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/ReplacingWriter.php',
    'Dotenv\\Repository\\Adapter\\ServerConstAdapter' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/ServerConstAdapter.php',
    'Dotenv\\Repository\\Adapter\\WriterInterface' => $vendorDir . '/vlucas/phpdotenv/src/Repository/Adapter/WriterInterface.php',
    'Dotenv\\Repository\\RepositoryBuilder' => $vendorDir . '/vlucas/phpdotenv/src/Repository/RepositoryBuilder.php',
    'Dotenv\\Repository\\RepositoryInterface' => $vendorDir . '/vlucas/phpdotenv/src/Repository/RepositoryInterface.php',
    'Dotenv\\Store\\FileStore' => $vendorDir . '/vlucas/phpdotenv/src/Store/FileStore.php',
    'Dotenv\\Store\\File\\Paths' => $vendorDir . '/vlucas/phpdotenv/src/Store/File/Paths.php',
    'Dotenv\\Store\\File\\Reader' => $vendorDir . '/vlucas/phpdotenv/src/Store/File/Reader.php',
    'Dotenv\\Store\\StoreBuilder' => $vendorDir . '/vlucas/phpdotenv/src/Store/StoreBuilder.php',
    'Dotenv\\Store\\StoreInterface' => $vendorDir . '/vlucas/phpdotenv/src/Store/StoreInterface.php',
    'Dotenv\\Store\\StringStore' => $vendorDir . '/vlucas/phpdotenv/src/Store/StringStore.php',
    'Dotenv\\Util\\Regex' => $vendorDir . '/vlucas/phpdotenv/src/Util/Regex.php',
    'Dotenv\\Util\\Str' => $vendorDir . '/vlucas/phpdotenv/src/Util/Str.php',
    'Dotenv\\Validator' => $vendorDir . '/vlucas/phpdotenv/src/Validator.php',
    'Egulias\\EmailValidator\\EmailLexer' => $vendorDir . '/egulias/email-validator/src/EmailLexer.php',
    'Egulias\\EmailValidator\\EmailParser' => $vendorDir . '/egulias/email-validator/src/EmailParser.php',
    'Egulias\\EmailValidator\\EmailValidator' => $vendorDir . '/egulias/email-validator/src/EmailValidator.php',
    'Egulias\\EmailValidator\\MessageIDParser' => $vendorDir . '/egulias/email-validator/src/MessageIDParser.php',
    'Egulias\\EmailValidator\\Parser' => $vendorDir . '/egulias/email-validator/src/Parser.php',
    'Egulias\\EmailValidator\\Parser\\Comment' => $vendorDir . '/egulias/email-validator/src/Parser/Comment.php',
    'Egulias\\EmailValidator\\Parser\\CommentStrategy\\CommentStrategy' => $vendorDir . '/egulias/email-validator/src/Parser/CommentStrategy/CommentStrategy.php',
    'Egulias\\EmailValidator\\Parser\\CommentStrategy\\DomainComment' => $vendorDir . '/egulias/email-validator/src/Parser/CommentStrategy/DomainComment.php',
    'Egulias\\EmailValidator\\Parser\\CommentStrategy\\LocalComment' => $vendorDir . '/egulias/email-validator/src/Parser/CommentStrategy/LocalComment.php',
    'Egulias\\EmailValidator\\Parser\\DomainLiteral' => $vendorDir . '/egulias/email-validator/src/Parser/DomainLiteral.php',
    'Egulias\\EmailValidator\\Parser\\DomainPart' => $vendorDir . '/egulias/email-validator/src/Parser/DomainPart.php',
    'Egulias\\EmailValidator\\Parser\\DoubleQuote' => $vendorDir . '/egulias/email-validator/src/Parser/DoubleQuote.php',
    'Egulias\\EmailValidator\\Parser\\FoldingWhiteSpace' => $vendorDir . '/egulias/email-validator/src/Parser/FoldingWhiteSpace.php',
    'Egulias\\EmailValidator\\Parser\\IDLeftPart' => $vendorDir . '/egulias/email-validator/src/Parser/IDLeftPart.php',
    'Egulias\\EmailValidator\\Parser\\IDRightPart' => $vendorDir . '/egulias/email-validator/src/Parser/IDRightPart.php',
    'Egulias\\EmailValidator\\Parser\\LocalPart' => $vendorDir . '/egulias/email-validator/src/Parser/LocalPart.php',
    'Egulias\\EmailValidator\\Parser\\PartParser' => $vendorDir . '/egulias/email-validator/src/Parser/PartParser.php',
    'Egulias\\EmailValidator\\Result\\InvalidEmail' => $vendorDir . '/egulias/email-validator/src/Result/InvalidEmail.php',
    'Egulias\\EmailValidator\\Result\\MultipleErrors' => $vendorDir . '/egulias/email-validator/src/Result/MultipleErrors.php',
    'Egulias\\EmailValidator\\Result\\Reason\\AtextAfterCFWS' => $vendorDir . '/egulias/email-validator/src/Result/Reason/AtextAfterCFWS.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CRLFAtTheEnd' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CRLFAtTheEnd.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CRLFX2' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CRLFX2.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CRNoLF' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CRNoLF.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CharNotAllowed' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CharNotAllowed.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CommaInDomain' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CommaInDomain.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CommentsInIDRight' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CommentsInIDRight.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ConsecutiveAt' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ConsecutiveAt.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ConsecutiveDot' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ConsecutiveDot.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DetailedReason' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DetailedReason.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DomainAcceptsNoMail' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DomainAcceptsNoMail.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DomainHyphened' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DomainHyphened.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DomainTooLong' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DomainTooLong.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DotAtEnd' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DotAtEnd.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DotAtStart' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DotAtStart.php',
    'Egulias\\EmailValidator\\Result\\Reason\\EmptyReason' => $vendorDir . '/egulias/email-validator/src/Result/Reason/EmptyReason.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExceptionFound' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExceptionFound.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExpectingATEXT' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExpectingATEXT.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExpectingCTEXT' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExpectingCTEXT.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExpectingDTEXT' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExpectingDTEXT.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExpectingDomainLiteralClose' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExpectingDomainLiteralClose.php',
    'Egulias\\EmailValidator\\Result\\Reason\\LabelTooLong' => $vendorDir . '/egulias/email-validator/src/Result/Reason/LabelTooLong.php',
    'Egulias\\EmailValidator\\Result\\Reason\\LocalOrReservedDomain' => $vendorDir . '/egulias/email-validator/src/Result/Reason/LocalOrReservedDomain.php',
    'Egulias\\EmailValidator\\Result\\Reason\\NoDNSRecord' => $vendorDir . '/egulias/email-validator/src/Result/Reason/NoDNSRecord.php',
    'Egulias\\EmailValidator\\Result\\Reason\\NoDomainPart' => $vendorDir . '/egulias/email-validator/src/Result/Reason/NoDomainPart.php',
    'Egulias\\EmailValidator\\Result\\Reason\\NoLocalPart' => $vendorDir . '/egulias/email-validator/src/Result/Reason/NoLocalPart.php',
    'Egulias\\EmailValidator\\Result\\Reason\\RFCWarnings' => $vendorDir . '/egulias/email-validator/src/Result/Reason/RFCWarnings.php',
    'Egulias\\EmailValidator\\Result\\Reason\\Reason' => $vendorDir . '/egulias/email-validator/src/Result/Reason/Reason.php',
    'Egulias\\EmailValidator\\Result\\Reason\\SpoofEmail' => $vendorDir . '/egulias/email-validator/src/Result/Reason/SpoofEmail.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnOpenedComment' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnOpenedComment.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnableToGetDNSRecord' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnableToGetDNSRecord.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnclosedComment' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnclosedComment.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnclosedQuotedString' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnclosedQuotedString.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnusualElements' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnusualElements.php',
    'Egulias\\EmailValidator\\Result\\Result' => $vendorDir . '/egulias/email-validator/src/Result/Result.php',
    'Egulias\\EmailValidator\\Result\\SpoofEmail' => $vendorDir . '/egulias/email-validator/src/Result/SpoofEmail.php',
    'Egulias\\EmailValidator\\Result\\ValidEmail' => $vendorDir . '/egulias/email-validator/src/Result/ValidEmail.php',
    'Egulias\\EmailValidator\\Validation\\DNSCheckValidation' => $vendorDir . '/egulias/email-validator/src/Validation/DNSCheckValidation.php',
    'Egulias\\EmailValidator\\Validation\\DNSGetRecordWrapper' => $vendorDir . '/egulias/email-validator/src/Validation/DNSGetRecordWrapper.php',
    'Egulias\\EmailValidator\\Validation\\DNSRecords' => $vendorDir . '/egulias/email-validator/src/Validation/DNSRecords.php',
    'Egulias\\EmailValidator\\Validation\\EmailValidation' => $vendorDir . '/egulias/email-validator/src/Validation/EmailValidation.php',
    'Egulias\\EmailValidator\\Validation\\Exception\\EmptyValidationList' => $vendorDir . '/egulias/email-validator/src/Validation/Exception/EmptyValidationList.php',
    'Egulias\\EmailValidator\\Validation\\Extra\\SpoofCheckValidation' => $vendorDir . '/egulias/email-validator/src/Validation/Extra/SpoofCheckValidation.php',
    'Egulias\\EmailValidator\\Validation\\MessageIDValidation' => $vendorDir . '/egulias/email-validator/src/Validation/MessageIDValidation.php',
    'Egulias\\EmailValidator\\Validation\\MultipleValidationWithAnd' => $vendorDir . '/egulias/email-validator/src/Validation/MultipleValidationWithAnd.php',
    'Egulias\\EmailValidator\\Validation\\NoRFCWarningsValidation' => $vendorDir . '/egulias/email-validator/src/Validation/NoRFCWarningsValidation.php',
    'Egulias\\EmailValidator\\Validation\\RFCValidation' => $vendorDir . '/egulias/email-validator/src/Validation/RFCValidation.php',
    'Egulias\\EmailValidator\\Warning\\AddressLiteral' => $vendorDir . '/egulias/email-validator/src/Warning/AddressLiteral.php',
    'Egulias\\EmailValidator\\Warning\\CFWSNearAt' => $vendorDir . '/egulias/email-validator/src/Warning/CFWSNearAt.php',
    'Egulias\\EmailValidator\\Warning\\CFWSWithFWS' => $vendorDir . '/egulias/email-validator/src/Warning/CFWSWithFWS.php',
    'Egulias\\EmailValidator\\Warning\\Comment' => $vendorDir . '/egulias/email-validator/src/Warning/Comment.php',
    'Egulias\\EmailValidator\\Warning\\DeprecatedComment' => $vendorDir . '/egulias/email-validator/src/Warning/DeprecatedComment.php',
    'Egulias\\EmailValidator\\Warning\\DomainLiteral' => $vendorDir . '/egulias/email-validator/src/Warning/DomainLiteral.php',
    'Egulias\\EmailValidator\\Warning\\EmailTooLong' => $vendorDir . '/egulias/email-validator/src/Warning/EmailTooLong.php',
    'Egulias\\EmailValidator\\Warning\\IPV6BadChar' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6BadChar.php',
    'Egulias\\EmailValidator\\Warning\\IPV6ColonEnd' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6ColonEnd.php',
    'Egulias\\EmailValidator\\Warning\\IPV6ColonStart' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6ColonStart.php',
    'Egulias\\EmailValidator\\Warning\\IPV6Deprecated' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6Deprecated.php',
    'Egulias\\EmailValidator\\Warning\\IPV6DoubleColon' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6DoubleColon.php',
    'Egulias\\EmailValidator\\Warning\\IPV6GroupCount' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6GroupCount.php',
    'Egulias\\EmailValidator\\Warning\\IPV6MaxGroups' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6MaxGroups.php',
    'Egulias\\EmailValidator\\Warning\\LocalTooLong' => $vendorDir . '/egulias/email-validator/src/Warning/LocalTooLong.php',
    'Egulias\\EmailValidator\\Warning\\NoDNSMXRecord' => $vendorDir . '/egulias/email-validator/src/Warning/NoDNSMXRecord.php',
    'Egulias\\EmailValidator\\Warning\\ObsoleteDTEXT' => $vendorDir . '/egulias/email-validator/src/Warning/ObsoleteDTEXT.php',
    'Egulias\\EmailValidator\\Warning\\QuotedPart' => $vendorDir . '/egulias/email-validator/src/Warning/QuotedPart.php',
    'Egulias\\EmailValidator\\Warning\\QuotedString' => $vendorDir . '/egulias/email-validator/src/Warning/QuotedString.php',
    'Egulias\\EmailValidator\\Warning\\TLD' => $vendorDir . '/egulias/email-validator/src/Warning/TLD.php',
    'Egulias\\EmailValidator\\Warning\\Warning' => $vendorDir . '/egulias/email-validator/src/Warning/Warning.php',
    'Faker\\Calculator\\Ean' => $vendorDir . '/fakerphp/faker/src/Faker/Calculator/Ean.php',
    'Faker\\Calculator\\Iban' => $vendorDir . '/fakerphp/faker/src/Faker/Calculator/Iban.php',
    'Faker\\Calculator\\Inn' => $vendorDir . '/fakerphp/faker/src/Faker/Calculator/Inn.php',
    'Faker\\Calculator\\Isbn' => $vendorDir . '/fakerphp/faker/src/Faker/Calculator/Isbn.php',
    'Faker\\Calculator\\Luhn' => $vendorDir . '/fakerphp/faker/src/Faker/Calculator/Luhn.php',
    'Faker\\Calculator\\TCNo' => $vendorDir . '/fakerphp/faker/src/Faker/Calculator/TCNo.php',
    'Faker\\ChanceGenerator' => $vendorDir . '/fakerphp/faker/src/Faker/ChanceGenerator.php',
    'Faker\\Container\\Container' => $vendorDir . '/fakerphp/faker/src/Faker/Container/Container.php',
    'Faker\\Container\\ContainerBuilder' => $vendorDir . '/fakerphp/faker/src/Faker/Container/ContainerBuilder.php',
    'Faker\\Container\\ContainerException' => $vendorDir . '/fakerphp/faker/src/Faker/Container/ContainerException.php',
    'Faker\\Container\\ContainerInterface' => $vendorDir . '/fakerphp/faker/src/Faker/Container/ContainerInterface.php',
    'Faker\\Container\\NotInContainerException' => $vendorDir . '/fakerphp/faker/src/Faker/Container/NotInContainerException.php',
    'Faker\\Core\\Barcode' => $vendorDir . '/fakerphp/faker/src/Faker/Core/Barcode.php',
    'Faker\\Core\\Blood' => $vendorDir . '/fakerphp/faker/src/Faker/Core/Blood.php',
    'Faker\\Core\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Core/Color.php',
    'Faker\\Core\\Coordinates' => $vendorDir . '/fakerphp/faker/src/Faker/Core/Coordinates.php',
    'Faker\\Core\\DateTime' => $vendorDir . '/fakerphp/faker/src/Faker/Core/DateTime.php',
    'Faker\\Core\\File' => $vendorDir . '/fakerphp/faker/src/Faker/Core/File.php',
    'Faker\\Core\\Number' => $vendorDir . '/fakerphp/faker/src/Faker/Core/Number.php',
    'Faker\\Core\\Uuid' => $vendorDir . '/fakerphp/faker/src/Faker/Core/Uuid.php',
    'Faker\\Core\\Version' => $vendorDir . '/fakerphp/faker/src/Faker/Core/Version.php',
    'Faker\\DefaultGenerator' => $vendorDir . '/fakerphp/faker/src/Faker/DefaultGenerator.php',
    'Faker\\Documentor' => $vendorDir . '/fakerphp/faker/src/Faker/Documentor.php',
    'Faker\\Extension\\AddressExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/AddressExtension.php',
    'Faker\\Extension\\BarcodeExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/BarcodeExtension.php',
    'Faker\\Extension\\BloodExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/BloodExtension.php',
    'Faker\\Extension\\ColorExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/ColorExtension.php',
    'Faker\\Extension\\CompanyExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/CompanyExtension.php',
    'Faker\\Extension\\CountryExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/CountryExtension.php',
    'Faker\\Extension\\DateTimeExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/DateTimeExtension.php',
    'Faker\\Extension\\Extension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/Extension.php',
    'Faker\\Extension\\ExtensionNotFound' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/ExtensionNotFound.php',
    'Faker\\Extension\\FileExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/FileExtension.php',
    'Faker\\Extension\\GeneratorAwareExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/GeneratorAwareExtension.php',
    'Faker\\Extension\\GeneratorAwareExtensionTrait' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/GeneratorAwareExtensionTrait.php',
    'Faker\\Extension\\Helper' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/Helper.php',
    'Faker\\Extension\\NumberExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/NumberExtension.php',
    'Faker\\Extension\\PersonExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/PersonExtension.php',
    'Faker\\Extension\\PhoneNumberExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/PhoneNumberExtension.php',
    'Faker\\Extension\\UuidExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/UuidExtension.php',
    'Faker\\Extension\\VersionExtension' => $vendorDir . '/fakerphp/faker/src/Faker/Extension/VersionExtension.php',
    'Faker\\Factory' => $vendorDir . '/fakerphp/faker/src/Faker/Factory.php',
    'Faker\\Generator' => $vendorDir . '/fakerphp/faker/src/Faker/Generator.php',
    'Faker\\Guesser\\Name' => $vendorDir . '/fakerphp/faker/src/Faker/Guesser/Name.php',
    'Faker\\ORM\\CakePHP\\ColumnTypeGuesser' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/CakePHP/ColumnTypeGuesser.php',
    'Faker\\ORM\\CakePHP\\EntityPopulator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/CakePHP/EntityPopulator.php',
    'Faker\\ORM\\CakePHP\\Populator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/CakePHP/Populator.php',
    'Faker\\ORM\\Doctrine\\ColumnTypeGuesser' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Doctrine/ColumnTypeGuesser.php',
    'Faker\\ORM\\Doctrine\\EntityPopulator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Doctrine/EntityPopulator.php',
    'Faker\\ORM\\Doctrine\\Populator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Doctrine/Populator.php',
    'Faker\\ORM\\Mandango\\ColumnTypeGuesser' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Mandango/ColumnTypeGuesser.php',
    'Faker\\ORM\\Mandango\\EntityPopulator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Mandango/EntityPopulator.php',
    'Faker\\ORM\\Mandango\\Populator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Mandango/Populator.php',
    'Faker\\ORM\\Propel2\\ColumnTypeGuesser' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Propel2/ColumnTypeGuesser.php',
    'Faker\\ORM\\Propel2\\EntityPopulator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Propel2/EntityPopulator.php',
    'Faker\\ORM\\Propel2\\Populator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Propel2/Populator.php',
    'Faker\\ORM\\Propel\\ColumnTypeGuesser' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Propel/ColumnTypeGuesser.php',
    'Faker\\ORM\\Propel\\EntityPopulator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Propel/EntityPopulator.php',
    'Faker\\ORM\\Propel\\Populator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Propel/Populator.php',
    'Faker\\ORM\\Spot\\ColumnTypeGuesser' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Spot/ColumnTypeGuesser.php',
    'Faker\\ORM\\Spot\\EntityPopulator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Spot/EntityPopulator.php',
    'Faker\\ORM\\Spot\\Populator' => $vendorDir . '/fakerphp/faker/src/Faker/ORM/Spot/Populator.php',
    'Faker\\Provider\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Address.php',
    'Faker\\Provider\\Barcode' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Barcode.php',
    'Faker\\Provider\\Base' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Base.php',
    'Faker\\Provider\\Biased' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Biased.php',
    'Faker\\Provider\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Color.php',
    'Faker\\Provider\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Company.php',
    'Faker\\Provider\\DateTime' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/DateTime.php',
    'Faker\\Provider\\File' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/File.php',
    'Faker\\Provider\\HtmlLorem' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/HtmlLorem.php',
    'Faker\\Provider\\Image' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Image.php',
    'Faker\\Provider\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Internet.php',
    'Faker\\Provider\\Lorem' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Lorem.php',
    'Faker\\Provider\\Medical' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Medical.php',
    'Faker\\Provider\\Miscellaneous' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Miscellaneous.php',
    'Faker\\Provider\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Payment.php',
    'Faker\\Provider\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Person.php',
    'Faker\\Provider\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/PhoneNumber.php',
    'Faker\\Provider\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Text.php',
    'Faker\\Provider\\UserAgent' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/UserAgent.php',
    'Faker\\Provider\\Uuid' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/Uuid.php',
    'Faker\\Provider\\ar_EG\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_EG/Address.php',
    'Faker\\Provider\\ar_EG\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_EG/Color.php',
    'Faker\\Provider\\ar_EG\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_EG/Company.php',
    'Faker\\Provider\\ar_EG\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_EG/Internet.php',
    'Faker\\Provider\\ar_EG\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_EG/Payment.php',
    'Faker\\Provider\\ar_EG\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_EG/Person.php',
    'Faker\\Provider\\ar_EG\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_EG/Text.php',
    'Faker\\Provider\\ar_JO\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_JO/Address.php',
    'Faker\\Provider\\ar_JO\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_JO/Company.php',
    'Faker\\Provider\\ar_JO\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_JO/Internet.php',
    'Faker\\Provider\\ar_JO\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_JO/Person.php',
    'Faker\\Provider\\ar_JO\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_JO/Text.php',
    'Faker\\Provider\\ar_SA\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_SA/Address.php',
    'Faker\\Provider\\ar_SA\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_SA/Color.php',
    'Faker\\Provider\\ar_SA\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_SA/Company.php',
    'Faker\\Provider\\ar_SA\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_SA/Internet.php',
    'Faker\\Provider\\ar_SA\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_SA/Payment.php',
    'Faker\\Provider\\ar_SA\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_SA/Person.php',
    'Faker\\Provider\\ar_SA\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ar_SA/Text.php',
    'Faker\\Provider\\at_AT\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/at_AT/Payment.php',
    'Faker\\Provider\\bg_BG\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/bg_BG/Internet.php',
    'Faker\\Provider\\bg_BG\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/bg_BG/Payment.php',
    'Faker\\Provider\\bg_BG\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/bg_BG/Person.php',
    'Faker\\Provider\\bg_BG\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/bg_BG/PhoneNumber.php',
    'Faker\\Provider\\bn_BD\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/bn_BD/Address.php',
    'Faker\\Provider\\bn_BD\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/bn_BD/Company.php',
    'Faker\\Provider\\bn_BD\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/bn_BD/Person.php',
    'Faker\\Provider\\bn_BD\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/bn_BD/PhoneNumber.php',
    'Faker\\Provider\\bn_BD\\Utils' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/bn_BD/Utils.php',
    'Faker\\Provider\\cs_CZ\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/cs_CZ/Address.php',
    'Faker\\Provider\\cs_CZ\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/cs_CZ/Company.php',
    'Faker\\Provider\\cs_CZ\\DateTime' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/cs_CZ/DateTime.php',
    'Faker\\Provider\\cs_CZ\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/cs_CZ/Internet.php',
    'Faker\\Provider\\cs_CZ\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/cs_CZ/Payment.php',
    'Faker\\Provider\\cs_CZ\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/cs_CZ/Person.php',
    'Faker\\Provider\\cs_CZ\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/cs_CZ/PhoneNumber.php',
    'Faker\\Provider\\cs_CZ\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/cs_CZ/Text.php',
    'Faker\\Provider\\da_DK\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/da_DK/Address.php',
    'Faker\\Provider\\da_DK\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/da_DK/Company.php',
    'Faker\\Provider\\da_DK\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/da_DK/Internet.php',
    'Faker\\Provider\\da_DK\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/da_DK/Payment.php',
    'Faker\\Provider\\da_DK\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/da_DK/Person.php',
    'Faker\\Provider\\da_DK\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/da_DK/PhoneNumber.php',
    'Faker\\Provider\\de_AT\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_AT/Address.php',
    'Faker\\Provider\\de_AT\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_AT/Company.php',
    'Faker\\Provider\\de_AT\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_AT/Internet.php',
    'Faker\\Provider\\de_AT\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_AT/Payment.php',
    'Faker\\Provider\\de_AT\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_AT/Person.php',
    'Faker\\Provider\\de_AT\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_AT/PhoneNumber.php',
    'Faker\\Provider\\de_AT\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_AT/Text.php',
    'Faker\\Provider\\de_CH\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_CH/Address.php',
    'Faker\\Provider\\de_CH\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_CH/Company.php',
    'Faker\\Provider\\de_CH\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_CH/Internet.php',
    'Faker\\Provider\\de_CH\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_CH/Payment.php',
    'Faker\\Provider\\de_CH\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_CH/Person.php',
    'Faker\\Provider\\de_CH\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_CH/PhoneNumber.php',
    'Faker\\Provider\\de_CH\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_CH/Text.php',
    'Faker\\Provider\\de_DE\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_DE/Address.php',
    'Faker\\Provider\\de_DE\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_DE/Company.php',
    'Faker\\Provider\\de_DE\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_DE/Internet.php',
    'Faker\\Provider\\de_DE\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_DE/Payment.php',
    'Faker\\Provider\\de_DE\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_DE/Person.php',
    'Faker\\Provider\\de_DE\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_DE/PhoneNumber.php',
    'Faker\\Provider\\de_DE\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/de_DE/Text.php',
    'Faker\\Provider\\el_CY\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_CY/Address.php',
    'Faker\\Provider\\el_CY\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_CY/Company.php',
    'Faker\\Provider\\el_CY\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_CY/Internet.php',
    'Faker\\Provider\\el_CY\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_CY/Payment.php',
    'Faker\\Provider\\el_CY\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_CY/Person.php',
    'Faker\\Provider\\el_CY\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_CY/PhoneNumber.php',
    'Faker\\Provider\\el_GR\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_GR/Address.php',
    'Faker\\Provider\\el_GR\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_GR/Company.php',
    'Faker\\Provider\\el_GR\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_GR/Payment.php',
    'Faker\\Provider\\el_GR\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_GR/Person.php',
    'Faker\\Provider\\el_GR\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_GR/PhoneNumber.php',
    'Faker\\Provider\\el_GR\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/el_GR/Text.php',
    'Faker\\Provider\\en_AU\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_AU/Address.php',
    'Faker\\Provider\\en_AU\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_AU/Internet.php',
    'Faker\\Provider\\en_AU\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_AU/PhoneNumber.php',
    'Faker\\Provider\\en_CA\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_CA/Address.php',
    'Faker\\Provider\\en_CA\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_CA/PhoneNumber.php',
    'Faker\\Provider\\en_GB\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_GB/Address.php',
    'Faker\\Provider\\en_GB\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_GB/Company.php',
    'Faker\\Provider\\en_GB\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_GB/Internet.php',
    'Faker\\Provider\\en_GB\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_GB/Payment.php',
    'Faker\\Provider\\en_GB\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_GB/Person.php',
    'Faker\\Provider\\en_GB\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_GB/PhoneNumber.php',
    'Faker\\Provider\\en_HK\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_HK/Address.php',
    'Faker\\Provider\\en_HK\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_HK/Internet.php',
    'Faker\\Provider\\en_HK\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_HK/PhoneNumber.php',
    'Faker\\Provider\\en_IN\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_IN/Address.php',
    'Faker\\Provider\\en_IN\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_IN/Internet.php',
    'Faker\\Provider\\en_IN\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_IN/Person.php',
    'Faker\\Provider\\en_IN\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_IN/PhoneNumber.php',
    'Faker\\Provider\\en_NG\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_NG/Address.php',
    'Faker\\Provider\\en_NG\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_NG/Internet.php',
    'Faker\\Provider\\en_NG\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_NG/Person.php',
    'Faker\\Provider\\en_NG\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_NG/PhoneNumber.php',
    'Faker\\Provider\\en_NZ\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_NZ/Address.php',
    'Faker\\Provider\\en_NZ\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_NZ/Internet.php',
    'Faker\\Provider\\en_NZ\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_NZ/PhoneNumber.php',
    'Faker\\Provider\\en_PH\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_PH/Address.php',
    'Faker\\Provider\\en_PH\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_PH/PhoneNumber.php',
    'Faker\\Provider\\en_SG\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_SG/Address.php',
    'Faker\\Provider\\en_SG\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_SG/Person.php',
    'Faker\\Provider\\en_SG\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_SG/PhoneNumber.php',
    'Faker\\Provider\\en_UG\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_UG/Address.php',
    'Faker\\Provider\\en_UG\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_UG/Internet.php',
    'Faker\\Provider\\en_UG\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_UG/Person.php',
    'Faker\\Provider\\en_UG\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_UG/PhoneNumber.php',
    'Faker\\Provider\\en_US\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_US/Address.php',
    'Faker\\Provider\\en_US\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_US/Company.php',
    'Faker\\Provider\\en_US\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_US/Payment.php',
    'Faker\\Provider\\en_US\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_US/Person.php',
    'Faker\\Provider\\en_US\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_US/PhoneNumber.php',
    'Faker\\Provider\\en_US\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_US/Text.php',
    'Faker\\Provider\\en_ZA\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_ZA/Address.php',
    'Faker\\Provider\\en_ZA\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_ZA/Company.php',
    'Faker\\Provider\\en_ZA\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_ZA/Internet.php',
    'Faker\\Provider\\en_ZA\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_ZA/Person.php',
    'Faker\\Provider\\en_ZA\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/en_ZA/PhoneNumber.php',
    'Faker\\Provider\\es_AR\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_AR/Address.php',
    'Faker\\Provider\\es_AR\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_AR/Company.php',
    'Faker\\Provider\\es_AR\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_AR/Person.php',
    'Faker\\Provider\\es_AR\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_AR/PhoneNumber.php',
    'Faker\\Provider\\es_ES\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_ES/Address.php',
    'Faker\\Provider\\es_ES\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_ES/Color.php',
    'Faker\\Provider\\es_ES\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_ES/Company.php',
    'Faker\\Provider\\es_ES\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_ES/Internet.php',
    'Faker\\Provider\\es_ES\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_ES/Payment.php',
    'Faker\\Provider\\es_ES\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_ES/Person.php',
    'Faker\\Provider\\es_ES\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_ES/PhoneNumber.php',
    'Faker\\Provider\\es_ES\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_ES/Text.php',
    'Faker\\Provider\\es_PE\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_PE/Address.php',
    'Faker\\Provider\\es_PE\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_PE/Company.php',
    'Faker\\Provider\\es_PE\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_PE/Person.php',
    'Faker\\Provider\\es_PE\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_PE/PhoneNumber.php',
    'Faker\\Provider\\es_VE\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_VE/Address.php',
    'Faker\\Provider\\es_VE\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_VE/Company.php',
    'Faker\\Provider\\es_VE\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_VE/Internet.php',
    'Faker\\Provider\\es_VE\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_VE/Person.php',
    'Faker\\Provider\\es_VE\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/es_VE/PhoneNumber.php',
    'Faker\\Provider\\et_EE\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/et_EE/Person.php',
    'Faker\\Provider\\fa_IR\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fa_IR/Address.php',
    'Faker\\Provider\\fa_IR\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fa_IR/Company.php',
    'Faker\\Provider\\fa_IR\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fa_IR/Internet.php',
    'Faker\\Provider\\fa_IR\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fa_IR/Person.php',
    'Faker\\Provider\\fa_IR\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fa_IR/PhoneNumber.php',
    'Faker\\Provider\\fa_IR\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fa_IR/Text.php',
    'Faker\\Provider\\fi_FI\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fi_FI/Address.php',
    'Faker\\Provider\\fi_FI\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fi_FI/Company.php',
    'Faker\\Provider\\fi_FI\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fi_FI/Internet.php',
    'Faker\\Provider\\fi_FI\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fi_FI/Payment.php',
    'Faker\\Provider\\fi_FI\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fi_FI/Person.php',
    'Faker\\Provider\\fi_FI\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fi_FI/PhoneNumber.php',
    'Faker\\Provider\\fr_BE\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_BE/Address.php',
    'Faker\\Provider\\fr_BE\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_BE/Color.php',
    'Faker\\Provider\\fr_BE\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_BE/Company.php',
    'Faker\\Provider\\fr_BE\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_BE/Internet.php',
    'Faker\\Provider\\fr_BE\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_BE/Payment.php',
    'Faker\\Provider\\fr_BE\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_BE/Person.php',
    'Faker\\Provider\\fr_BE\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_BE/PhoneNumber.php',
    'Faker\\Provider\\fr_CA\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CA/Address.php',
    'Faker\\Provider\\fr_CA\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CA/Color.php',
    'Faker\\Provider\\fr_CA\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CA/Company.php',
    'Faker\\Provider\\fr_CA\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CA/Person.php',
    'Faker\\Provider\\fr_CA\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CA/Text.php',
    'Faker\\Provider\\fr_CH\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CH/Address.php',
    'Faker\\Provider\\fr_CH\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CH/Color.php',
    'Faker\\Provider\\fr_CH\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CH/Company.php',
    'Faker\\Provider\\fr_CH\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CH/Internet.php',
    'Faker\\Provider\\fr_CH\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CH/Payment.php',
    'Faker\\Provider\\fr_CH\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CH/Person.php',
    'Faker\\Provider\\fr_CH\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CH/PhoneNumber.php',
    'Faker\\Provider\\fr_CH\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_CH/Text.php',
    'Faker\\Provider\\fr_FR\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_FR/Address.php',
    'Faker\\Provider\\fr_FR\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_FR/Color.php',
    'Faker\\Provider\\fr_FR\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_FR/Company.php',
    'Faker\\Provider\\fr_FR\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_FR/Internet.php',
    'Faker\\Provider\\fr_FR\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_FR/Payment.php',
    'Faker\\Provider\\fr_FR\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_FR/Person.php',
    'Faker\\Provider\\fr_FR\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_FR/PhoneNumber.php',
    'Faker\\Provider\\fr_FR\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/fr_FR/Text.php',
    'Faker\\Provider\\he_IL\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/he_IL/Address.php',
    'Faker\\Provider\\he_IL\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/he_IL/Company.php',
    'Faker\\Provider\\he_IL\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/he_IL/Payment.php',
    'Faker\\Provider\\he_IL\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/he_IL/Person.php',
    'Faker\\Provider\\he_IL\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/he_IL/PhoneNumber.php',
    'Faker\\Provider\\hr_HR\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hr_HR/Address.php',
    'Faker\\Provider\\hr_HR\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hr_HR/Company.php',
    'Faker\\Provider\\hr_HR\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hr_HR/Payment.php',
    'Faker\\Provider\\hr_HR\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hr_HR/Person.php',
    'Faker\\Provider\\hr_HR\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hr_HR/PhoneNumber.php',
    'Faker\\Provider\\hu_HU\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hu_HU/Address.php',
    'Faker\\Provider\\hu_HU\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hu_HU/Company.php',
    'Faker\\Provider\\hu_HU\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hu_HU/Payment.php',
    'Faker\\Provider\\hu_HU\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hu_HU/Person.php',
    'Faker\\Provider\\hu_HU\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hu_HU/PhoneNumber.php',
    'Faker\\Provider\\hu_HU\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hu_HU/Text.php',
    'Faker\\Provider\\hy_AM\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hy_AM/Address.php',
    'Faker\\Provider\\hy_AM\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hy_AM/Color.php',
    'Faker\\Provider\\hy_AM\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hy_AM/Company.php',
    'Faker\\Provider\\hy_AM\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hy_AM/Internet.php',
    'Faker\\Provider\\hy_AM\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hy_AM/Person.php',
    'Faker\\Provider\\hy_AM\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/hy_AM/PhoneNumber.php',
    'Faker\\Provider\\id_ID\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/id_ID/Address.php',
    'Faker\\Provider\\id_ID\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/id_ID/Color.php',
    'Faker\\Provider\\id_ID\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/id_ID/Company.php',
    'Faker\\Provider\\id_ID\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/id_ID/Internet.php',
    'Faker\\Provider\\id_ID\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/id_ID/Person.php',
    'Faker\\Provider\\id_ID\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/id_ID/PhoneNumber.php',
    'Faker\\Provider\\is_IS\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/is_IS/Address.php',
    'Faker\\Provider\\is_IS\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/is_IS/Company.php',
    'Faker\\Provider\\is_IS\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/is_IS/Internet.php',
    'Faker\\Provider\\is_IS\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/is_IS/Payment.php',
    'Faker\\Provider\\is_IS\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/is_IS/Person.php',
    'Faker\\Provider\\is_IS\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/is_IS/PhoneNumber.php',
    'Faker\\Provider\\it_CH\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_CH/Address.php',
    'Faker\\Provider\\it_CH\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_CH/Company.php',
    'Faker\\Provider\\it_CH\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_CH/Internet.php',
    'Faker\\Provider\\it_CH\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_CH/Payment.php',
    'Faker\\Provider\\it_CH\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_CH/Person.php',
    'Faker\\Provider\\it_CH\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_CH/PhoneNumber.php',
    'Faker\\Provider\\it_CH\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_CH/Text.php',
    'Faker\\Provider\\it_IT\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_IT/Address.php',
    'Faker\\Provider\\it_IT\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_IT/Company.php',
    'Faker\\Provider\\it_IT\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_IT/Internet.php',
    'Faker\\Provider\\it_IT\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_IT/Payment.php',
    'Faker\\Provider\\it_IT\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_IT/Person.php',
    'Faker\\Provider\\it_IT\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_IT/PhoneNumber.php',
    'Faker\\Provider\\it_IT\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/it_IT/Text.php',
    'Faker\\Provider\\ja_JP\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ja_JP/Address.php',
    'Faker\\Provider\\ja_JP\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ja_JP/Company.php',
    'Faker\\Provider\\ja_JP\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ja_JP/Internet.php',
    'Faker\\Provider\\ja_JP\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ja_JP/Person.php',
    'Faker\\Provider\\ja_JP\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ja_JP/PhoneNumber.php',
    'Faker\\Provider\\ja_JP\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ja_JP/Text.php',
    'Faker\\Provider\\ka_GE\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ka_GE/Address.php',
    'Faker\\Provider\\ka_GE\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ka_GE/Color.php',
    'Faker\\Provider\\ka_GE\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ka_GE/Company.php',
    'Faker\\Provider\\ka_GE\\DateTime' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ka_GE/DateTime.php',
    'Faker\\Provider\\ka_GE\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ka_GE/Internet.php',
    'Faker\\Provider\\ka_GE\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ka_GE/Payment.php',
    'Faker\\Provider\\ka_GE\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ka_GE/Person.php',
    'Faker\\Provider\\ka_GE\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ka_GE/PhoneNumber.php',
    'Faker\\Provider\\ka_GE\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ka_GE/Text.php',
    'Faker\\Provider\\kk_KZ\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/kk_KZ/Address.php',
    'Faker\\Provider\\kk_KZ\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/kk_KZ/Color.php',
    'Faker\\Provider\\kk_KZ\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/kk_KZ/Company.php',
    'Faker\\Provider\\kk_KZ\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/kk_KZ/Internet.php',
    'Faker\\Provider\\kk_KZ\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/kk_KZ/Payment.php',
    'Faker\\Provider\\kk_KZ\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/kk_KZ/Person.php',
    'Faker\\Provider\\kk_KZ\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/kk_KZ/PhoneNumber.php',
    'Faker\\Provider\\kk_KZ\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/kk_KZ/Text.php',
    'Faker\\Provider\\ko_KR\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ko_KR/Address.php',
    'Faker\\Provider\\ko_KR\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ko_KR/Company.php',
    'Faker\\Provider\\ko_KR\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ko_KR/Internet.php',
    'Faker\\Provider\\ko_KR\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ko_KR/Person.php',
    'Faker\\Provider\\ko_KR\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ko_KR/PhoneNumber.php',
    'Faker\\Provider\\ko_KR\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ko_KR/Text.php',
    'Faker\\Provider\\lt_LT\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lt_LT/Address.php',
    'Faker\\Provider\\lt_LT\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lt_LT/Company.php',
    'Faker\\Provider\\lt_LT\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lt_LT/Internet.php',
    'Faker\\Provider\\lt_LT\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lt_LT/Payment.php',
    'Faker\\Provider\\lt_LT\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lt_LT/Person.php',
    'Faker\\Provider\\lt_LT\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lt_LT/PhoneNumber.php',
    'Faker\\Provider\\lv_LV\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lv_LV/Address.php',
    'Faker\\Provider\\lv_LV\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lv_LV/Color.php',
    'Faker\\Provider\\lv_LV\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lv_LV/Internet.php',
    'Faker\\Provider\\lv_LV\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lv_LV/Payment.php',
    'Faker\\Provider\\lv_LV\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lv_LV/Person.php',
    'Faker\\Provider\\lv_LV\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/lv_LV/PhoneNumber.php',
    'Faker\\Provider\\me_ME\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/me_ME/Address.php',
    'Faker\\Provider\\me_ME\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/me_ME/Company.php',
    'Faker\\Provider\\me_ME\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/me_ME/Payment.php',
    'Faker\\Provider\\me_ME\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/me_ME/Person.php',
    'Faker\\Provider\\me_ME\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/me_ME/PhoneNumber.php',
    'Faker\\Provider\\mn_MN\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/mn_MN/Person.php',
    'Faker\\Provider\\mn_MN\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/mn_MN/PhoneNumber.php',
    'Faker\\Provider\\ms_MY\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ms_MY/Address.php',
    'Faker\\Provider\\ms_MY\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ms_MY/Company.php',
    'Faker\\Provider\\ms_MY\\Miscellaneous' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ms_MY/Miscellaneous.php',
    'Faker\\Provider\\ms_MY\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ms_MY/Payment.php',
    'Faker\\Provider\\ms_MY\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ms_MY/Person.php',
    'Faker\\Provider\\ms_MY\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ms_MY/PhoneNumber.php',
    'Faker\\Provider\\nb_NO\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nb_NO/Address.php',
    'Faker\\Provider\\nb_NO\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nb_NO/Company.php',
    'Faker\\Provider\\nb_NO\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nb_NO/Payment.php',
    'Faker\\Provider\\nb_NO\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nb_NO/Person.php',
    'Faker\\Provider\\nb_NO\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nb_NO/PhoneNumber.php',
    'Faker\\Provider\\ne_NP\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ne_NP/Address.php',
    'Faker\\Provider\\ne_NP\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ne_NP/Internet.php',
    'Faker\\Provider\\ne_NP\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ne_NP/Payment.php',
    'Faker\\Provider\\ne_NP\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ne_NP/Person.php',
    'Faker\\Provider\\ne_NP\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ne_NP/PhoneNumber.php',
    'Faker\\Provider\\nl_BE\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_BE/Address.php',
    'Faker\\Provider\\nl_BE\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_BE/Company.php',
    'Faker\\Provider\\nl_BE\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_BE/Internet.php',
    'Faker\\Provider\\nl_BE\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_BE/Payment.php',
    'Faker\\Provider\\nl_BE\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_BE/Person.php',
    'Faker\\Provider\\nl_BE\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_BE/PhoneNumber.php',
    'Faker\\Provider\\nl_BE\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_BE/Text.php',
    'Faker\\Provider\\nl_NL\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_NL/Address.php',
    'Faker\\Provider\\nl_NL\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_NL/Color.php',
    'Faker\\Provider\\nl_NL\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_NL/Company.php',
    'Faker\\Provider\\nl_NL\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_NL/Internet.php',
    'Faker\\Provider\\nl_NL\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_NL/Payment.php',
    'Faker\\Provider\\nl_NL\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_NL/Person.php',
    'Faker\\Provider\\nl_NL\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_NL/PhoneNumber.php',
    'Faker\\Provider\\nl_NL\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/nl_NL/Text.php',
    'Faker\\Provider\\pl_PL\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pl_PL/Address.php',
    'Faker\\Provider\\pl_PL\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pl_PL/Color.php',
    'Faker\\Provider\\pl_PL\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pl_PL/Company.php',
    'Faker\\Provider\\pl_PL\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pl_PL/Internet.php',
    'Faker\\Provider\\pl_PL\\LicensePlate' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pl_PL/LicensePlate.php',
    'Faker\\Provider\\pl_PL\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pl_PL/Payment.php',
    'Faker\\Provider\\pl_PL\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pl_PL/Person.php',
    'Faker\\Provider\\pl_PL\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pl_PL/PhoneNumber.php',
    'Faker\\Provider\\pl_PL\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pl_PL/Text.php',
    'Faker\\Provider\\pt_BR\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_BR/Address.php',
    'Faker\\Provider\\pt_BR\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_BR/Company.php',
    'Faker\\Provider\\pt_BR\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_BR/Internet.php',
    'Faker\\Provider\\pt_BR\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_BR/Payment.php',
    'Faker\\Provider\\pt_BR\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_BR/Person.php',
    'Faker\\Provider\\pt_BR\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_BR/PhoneNumber.php',
    'Faker\\Provider\\pt_BR\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_BR/Text.php',
    'Faker\\Provider\\pt_PT\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_PT/Address.php',
    'Faker\\Provider\\pt_PT\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_PT/Company.php',
    'Faker\\Provider\\pt_PT\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_PT/Internet.php',
    'Faker\\Provider\\pt_PT\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_PT/Payment.php',
    'Faker\\Provider\\pt_PT\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_PT/Person.php',
    'Faker\\Provider\\pt_PT\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/pt_PT/PhoneNumber.php',
    'Faker\\Provider\\ro_MD\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_MD/Address.php',
    'Faker\\Provider\\ro_MD\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_MD/Payment.php',
    'Faker\\Provider\\ro_MD\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_MD/Person.php',
    'Faker\\Provider\\ro_MD\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_MD/PhoneNumber.php',
    'Faker\\Provider\\ro_MD\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_MD/Text.php',
    'Faker\\Provider\\ro_RO\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_RO/Address.php',
    'Faker\\Provider\\ro_RO\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_RO/Payment.php',
    'Faker\\Provider\\ro_RO\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_RO/Person.php',
    'Faker\\Provider\\ro_RO\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_RO/PhoneNumber.php',
    'Faker\\Provider\\ro_RO\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ro_RO/Text.php',
    'Faker\\Provider\\ru_RU\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ru_RU/Address.php',
    'Faker\\Provider\\ru_RU\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ru_RU/Color.php',
    'Faker\\Provider\\ru_RU\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ru_RU/Company.php',
    'Faker\\Provider\\ru_RU\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ru_RU/Internet.php',
    'Faker\\Provider\\ru_RU\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ru_RU/Payment.php',
    'Faker\\Provider\\ru_RU\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ru_RU/Person.php',
    'Faker\\Provider\\ru_RU\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ru_RU/PhoneNumber.php',
    'Faker\\Provider\\ru_RU\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/ru_RU/Text.php',
    'Faker\\Provider\\sk_SK\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sk_SK/Address.php',
    'Faker\\Provider\\sk_SK\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sk_SK/Company.php',
    'Faker\\Provider\\sk_SK\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sk_SK/Internet.php',
    'Faker\\Provider\\sk_SK\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sk_SK/Payment.php',
    'Faker\\Provider\\sk_SK\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sk_SK/Person.php',
    'Faker\\Provider\\sk_SK\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sk_SK/PhoneNumber.php',
    'Faker\\Provider\\sl_SI\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sl_SI/Address.php',
    'Faker\\Provider\\sl_SI\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sl_SI/Company.php',
    'Faker\\Provider\\sl_SI\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sl_SI/Internet.php',
    'Faker\\Provider\\sl_SI\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sl_SI/Payment.php',
    'Faker\\Provider\\sl_SI\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sl_SI/Person.php',
    'Faker\\Provider\\sl_SI\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sl_SI/PhoneNumber.php',
    'Faker\\Provider\\sr_Cyrl_RS\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sr_Cyrl_RS/Address.php',
    'Faker\\Provider\\sr_Cyrl_RS\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sr_Cyrl_RS/Payment.php',
    'Faker\\Provider\\sr_Cyrl_RS\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sr_Cyrl_RS/Person.php',
    'Faker\\Provider\\sr_Latn_RS\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sr_Latn_RS/Address.php',
    'Faker\\Provider\\sr_Latn_RS\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sr_Latn_RS/Payment.php',
    'Faker\\Provider\\sr_Latn_RS\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sr_Latn_RS/Person.php',
    'Faker\\Provider\\sr_RS\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sr_RS/Address.php',
    'Faker\\Provider\\sr_RS\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sr_RS/Payment.php',
    'Faker\\Provider\\sr_RS\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sr_RS/Person.php',
    'Faker\\Provider\\sv_SE\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sv_SE/Address.php',
    'Faker\\Provider\\sv_SE\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sv_SE/Company.php',
    'Faker\\Provider\\sv_SE\\Municipality' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sv_SE/Municipality.php',
    'Faker\\Provider\\sv_SE\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sv_SE/Payment.php',
    'Faker\\Provider\\sv_SE\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sv_SE/Person.php',
    'Faker\\Provider\\sv_SE\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/sv_SE/PhoneNumber.php',
    'Faker\\Provider\\th_TH\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/th_TH/Address.php',
    'Faker\\Provider\\th_TH\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/th_TH/Color.php',
    'Faker\\Provider\\th_TH\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/th_TH/Company.php',
    'Faker\\Provider\\th_TH\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/th_TH/Internet.php',
    'Faker\\Provider\\th_TH\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/th_TH/Payment.php',
    'Faker\\Provider\\th_TH\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/th_TH/Person.php',
    'Faker\\Provider\\th_TH\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/th_TH/PhoneNumber.php',
    'Faker\\Provider\\tr_TR\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/tr_TR/Address.php',
    'Faker\\Provider\\tr_TR\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/tr_TR/Color.php',
    'Faker\\Provider\\tr_TR\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/tr_TR/Company.php',
    'Faker\\Provider\\tr_TR\\DateTime' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/tr_TR/DateTime.php',
    'Faker\\Provider\\tr_TR\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/tr_TR/Internet.php',
    'Faker\\Provider\\tr_TR\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/tr_TR/Payment.php',
    'Faker\\Provider\\tr_TR\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/tr_TR/Person.php',
    'Faker\\Provider\\tr_TR\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/tr_TR/PhoneNumber.php',
    'Faker\\Provider\\uk_UA\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/uk_UA/Address.php',
    'Faker\\Provider\\uk_UA\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/uk_UA/Color.php',
    'Faker\\Provider\\uk_UA\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/uk_UA/Company.php',
    'Faker\\Provider\\uk_UA\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/uk_UA/Internet.php',
    'Faker\\Provider\\uk_UA\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/uk_UA/Payment.php',
    'Faker\\Provider\\uk_UA\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/uk_UA/Person.php',
    'Faker\\Provider\\uk_UA\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/uk_UA/PhoneNumber.php',
    'Faker\\Provider\\uk_UA\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/uk_UA/Text.php',
    'Faker\\Provider\\vi_VN\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/vi_VN/Address.php',
    'Faker\\Provider\\vi_VN\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/vi_VN/Color.php',
    'Faker\\Provider\\vi_VN\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/vi_VN/Internet.php',
    'Faker\\Provider\\vi_VN\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/vi_VN/Person.php',
    'Faker\\Provider\\vi_VN\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/vi_VN/PhoneNumber.php',
    'Faker\\Provider\\zh_CN\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_CN/Address.php',
    'Faker\\Provider\\zh_CN\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_CN/Color.php',
    'Faker\\Provider\\zh_CN\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_CN/Company.php',
    'Faker\\Provider\\zh_CN\\DateTime' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_CN/DateTime.php',
    'Faker\\Provider\\zh_CN\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_CN/Internet.php',
    'Faker\\Provider\\zh_CN\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_CN/Payment.php',
    'Faker\\Provider\\zh_CN\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_CN/Person.php',
    'Faker\\Provider\\zh_CN\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_CN/PhoneNumber.php',
    'Faker\\Provider\\zh_TW\\Address' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_TW/Address.php',
    'Faker\\Provider\\zh_TW\\Color' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_TW/Color.php',
    'Faker\\Provider\\zh_TW\\Company' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_TW/Company.php',
    'Faker\\Provider\\zh_TW\\DateTime' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_TW/DateTime.php',
    'Faker\\Provider\\zh_TW\\Internet' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_TW/Internet.php',
    'Faker\\Provider\\zh_TW\\Payment' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_TW/Payment.php',
    'Faker\\Provider\\zh_TW\\Person' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_TW/Person.php',
    'Faker\\Provider\\zh_TW\\PhoneNumber' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_TW/PhoneNumber.php',
    'Faker\\Provider\\zh_TW\\Text' => $vendorDir . '/fakerphp/faker/src/Faker/Provider/zh_TW/Text.php',
    'Faker\\UniqueGenerator' => $vendorDir . '/fakerphp/faker/src/Faker/UniqueGenerator.php',
    'Faker\\ValidGenerator' => $vendorDir . '/fakerphp/faker/src/Faker/ValidGenerator.php',
    'FontLib\\AdobeFontMetrics' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/AdobeFontMetrics.php',
    'FontLib\\BinaryStream' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/BinaryStream.php',
    'FontLib\\EOT\\File' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/EOT/File.php',
    'FontLib\\EOT\\Header' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/EOT/Header.php',
    'FontLib\\EncodingMap' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/EncodingMap.php',
    'FontLib\\Exception\\FontNotFoundException' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Exception/FontNotFoundException.php',
    'FontLib\\Font' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Font.php',
    'FontLib\\Glyph\\Outline' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Glyph/Outline.php',
    'FontLib\\Glyph\\OutlineComponent' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Glyph/OutlineComponent.php',
    'FontLib\\Glyph\\OutlineComposite' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Glyph/OutlineComposite.php',
    'FontLib\\Glyph\\OutlineSimple' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Glyph/OutlineSimple.php',
    'FontLib\\Header' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Header.php',
    'FontLib\\OpenType\\File' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/OpenType/File.php',
    'FontLib\\OpenType\\TableDirectoryEntry' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/OpenType/TableDirectoryEntry.php',
    'FontLib\\Table\\DirectoryEntry' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/DirectoryEntry.php',
    'FontLib\\Table\\Table' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Table.php',
    'FontLib\\Table\\Type\\cmap' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/cmap.php',
    'FontLib\\Table\\Type\\cvt' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/cvt.php',
    'FontLib\\Table\\Type\\fpgm' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/fpgm.php',
    'FontLib\\Table\\Type\\glyf' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/glyf.php',
    'FontLib\\Table\\Type\\head' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/head.php',
    'FontLib\\Table\\Type\\hhea' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/hhea.php',
    'FontLib\\Table\\Type\\hmtx' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/hmtx.php',
    'FontLib\\Table\\Type\\kern' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/kern.php',
    'FontLib\\Table\\Type\\loca' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/loca.php',
    'FontLib\\Table\\Type\\maxp' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/maxp.php',
    'FontLib\\Table\\Type\\name' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/name.php',
    'FontLib\\Table\\Type\\nameRecord' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/nameRecord.php',
    'FontLib\\Table\\Type\\os2' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/os2.php',
    'FontLib\\Table\\Type\\post' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/post.php',
    'FontLib\\Table\\Type\\prep' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/Table/Type/prep.php',
    'FontLib\\TrueType\\Collection' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/TrueType/Collection.php',
    'FontLib\\TrueType\\File' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/TrueType/File.php',
    'FontLib\\TrueType\\Header' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/TrueType/Header.php',
    'FontLib\\TrueType\\TableDirectoryEntry' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/TrueType/TableDirectoryEntry.php',
    'FontLib\\WOFF\\File' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/WOFF/File.php',
    'FontLib\\WOFF\\Header' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/WOFF/Header.php',
    'FontLib\\WOFF\\TableDirectoryEntry' => $vendorDir . '/dompdf/php-font-lib/src/FontLib/WOFF/TableDirectoryEntry.php',
    'Fruitcake\\Cors\\CorsService' => $vendorDir . '/fruitcake/php-cors/src/CorsService.php',
    'Fruitcake\\Cors\\Exceptions\\InvalidOptionException' => $vendorDir . '/fruitcake/php-cors/src/Exceptions/InvalidOptionException.php',
    'GrahamCampbell\\ResultType\\Error' => $vendorDir . '/graham-campbell/result-type/src/Error.php',
    'GrahamCampbell\\ResultType\\Result' => $vendorDir . '/graham-campbell/result-type/src/Result.php',
    'GrahamCampbell\\ResultType\\Success' => $vendorDir . '/graham-campbell/result-type/src/Success.php',
    'GuzzleHttp\\BodySummarizer' => $vendorDir . '/guzzlehttp/guzzle/src/BodySummarizer.php',
    'GuzzleHttp\\BodySummarizerInterface' => $vendorDir . '/guzzlehttp/guzzle/src/BodySummarizerInterface.php',
    'GuzzleHttp\\Client' => $vendorDir . '/guzzlehttp/guzzle/src/Client.php',
    'GuzzleHttp\\ClientInterface' => $vendorDir . '/guzzlehttp/guzzle/src/ClientInterface.php',
    'GuzzleHttp\\ClientTrait' => $vendorDir . '/guzzlehttp/guzzle/src/ClientTrait.php',
    'GuzzleHttp\\Cookie\\CookieJar' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/CookieJar.php',
    'GuzzleHttp\\Cookie\\CookieJarInterface' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/CookieJarInterface.php',
    'GuzzleHttp\\Cookie\\FileCookieJar' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/FileCookieJar.php',
    'GuzzleHttp\\Cookie\\SessionCookieJar' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/SessionCookieJar.php',
    'GuzzleHttp\\Cookie\\SetCookie' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/SetCookie.php',
    'GuzzleHttp\\Exception\\BadResponseException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/BadResponseException.php',
    'GuzzleHttp\\Exception\\ClientException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/ClientException.php',
    'GuzzleHttp\\Exception\\ConnectException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/ConnectException.php',
    'GuzzleHttp\\Exception\\GuzzleException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/GuzzleException.php',
    'GuzzleHttp\\Exception\\InvalidArgumentException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/InvalidArgumentException.php',
    'GuzzleHttp\\Exception\\RequestException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/RequestException.php',
    'GuzzleHttp\\Exception\\ServerException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/ServerException.php',
    'GuzzleHttp\\Exception\\TooManyRedirectsException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/TooManyRedirectsException.php',
    'GuzzleHttp\\Exception\\TransferException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/TransferException.php',
    'GuzzleHttp\\HandlerStack' => $vendorDir . '/guzzlehttp/guzzle/src/HandlerStack.php',
    'GuzzleHttp\\Handler\\CurlFactory' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/CurlFactory.php',
    'GuzzleHttp\\Handler\\CurlFactoryInterface' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/CurlFactoryInterface.php',
    'GuzzleHttp\\Handler\\CurlHandler' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/CurlHandler.php',
    'GuzzleHttp\\Handler\\CurlMultiHandler' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/CurlMultiHandler.php',
    'GuzzleHttp\\Handler\\EasyHandle' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/EasyHandle.php',
    'GuzzleHttp\\Handler\\HeaderProcessor' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/HeaderProcessor.php',
    'GuzzleHttp\\Handler\\MockHandler' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/MockHandler.php',
    'GuzzleHttp\\Handler\\Proxy' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/Proxy.php',
    'GuzzleHttp\\Handler\\StreamHandler' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/StreamHandler.php',
    'GuzzleHttp\\MessageFormatter' => $vendorDir . '/guzzlehttp/guzzle/src/MessageFormatter.php',
    'GuzzleHttp\\MessageFormatterInterface' => $vendorDir . '/guzzlehttp/guzzle/src/MessageFormatterInterface.php',
    'GuzzleHttp\\Middleware' => $vendorDir . '/guzzlehttp/guzzle/src/Middleware.php',
    'GuzzleHttp\\Pool' => $vendorDir . '/guzzlehttp/guzzle/src/Pool.php',
    'GuzzleHttp\\PrepareBodyMiddleware' => $vendorDir . '/guzzlehttp/guzzle/src/PrepareBodyMiddleware.php',
    'GuzzleHttp\\Promise\\AggregateException' => $vendorDir . '/guzzlehttp/promises/src/AggregateException.php',
    'GuzzleHttp\\Promise\\CancellationException' => $vendorDir . '/guzzlehttp/promises/src/CancellationException.php',
    'GuzzleHttp\\Promise\\Coroutine' => $vendorDir . '/guzzlehttp/promises/src/Coroutine.php',
    'GuzzleHttp\\Promise\\Create' => $vendorDir . '/guzzlehttp/promises/src/Create.php',
    'GuzzleHttp\\Promise\\Each' => $vendorDir . '/guzzlehttp/promises/src/Each.php',
    'GuzzleHttp\\Promise\\EachPromise' => $vendorDir . '/guzzlehttp/promises/src/EachPromise.php',
    'GuzzleHttp\\Promise\\FulfilledPromise' => $vendorDir . '/guzzlehttp/promises/src/FulfilledPromise.php',
    'GuzzleHttp\\Promise\\Is' => $vendorDir . '/guzzlehttp/promises/src/Is.php',
    'GuzzleHttp\\Promise\\Promise' => $vendorDir . '/guzzlehttp/promises/src/Promise.php',
    'GuzzleHttp\\Promise\\PromiseInterface' => $vendorDir . '/guzzlehttp/promises/src/PromiseInterface.php',
    'GuzzleHttp\\Promise\\PromisorInterface' => $vendorDir . '/guzzlehttp/promises/src/PromisorInterface.php',
    'GuzzleHttp\\Promise\\RejectedPromise' => $vendorDir . '/guzzlehttp/promises/src/RejectedPromise.php',
    'GuzzleHttp\\Promise\\RejectionException' => $vendorDir . '/guzzlehttp/promises/src/RejectionException.php',
    'GuzzleHttp\\Promise\\TaskQueue' => $vendorDir . '/guzzlehttp/promises/src/TaskQueue.php',
    'GuzzleHttp\\Promise\\TaskQueueInterface' => $vendorDir . '/guzzlehttp/promises/src/TaskQueueInterface.php',
    'GuzzleHttp\\Promise\\Utils' => $vendorDir . '/guzzlehttp/promises/src/Utils.php',
    'GuzzleHttp\\Psr7\\AppendStream' => $vendorDir . '/guzzlehttp/psr7/src/AppendStream.php',
    'GuzzleHttp\\Psr7\\BufferStream' => $vendorDir . '/guzzlehttp/psr7/src/BufferStream.php',
    'GuzzleHttp\\Psr7\\CachingStream' => $vendorDir . '/guzzlehttp/psr7/src/CachingStream.php',
    'GuzzleHttp\\Psr7\\DroppingStream' => $vendorDir . '/guzzlehttp/psr7/src/DroppingStream.php',
    'GuzzleHttp\\Psr7\\Exception\\MalformedUriException' => $vendorDir . '/guzzlehttp/psr7/src/Exception/MalformedUriException.php',
    'GuzzleHttp\\Psr7\\FnStream' => $vendorDir . '/guzzlehttp/psr7/src/FnStream.php',
    'GuzzleHttp\\Psr7\\Header' => $vendorDir . '/guzzlehttp/psr7/src/Header.php',
    'GuzzleHttp\\Psr7\\HttpFactory' => $vendorDir . '/guzzlehttp/psr7/src/HttpFactory.php',
    'GuzzleHttp\\Psr7\\InflateStream' => $vendorDir . '/guzzlehttp/psr7/src/InflateStream.php',
    'GuzzleHttp\\Psr7\\LazyOpenStream' => $vendorDir . '/guzzlehttp/psr7/src/LazyOpenStream.php',
    'GuzzleHttp\\Psr7\\LimitStream' => $vendorDir . '/guzzlehttp/psr7/src/LimitStream.php',
    'GuzzleHttp\\Psr7\\Message' => $vendorDir . '/guzzlehttp/psr7/src/Message.php',
    'GuzzleHttp\\Psr7\\MessageTrait' => $vendorDir . '/guzzlehttp/psr7/src/MessageTrait.php',
    'GuzzleHttp\\Psr7\\MimeType' => $vendorDir . '/guzzlehttp/psr7/src/MimeType.php',
    'GuzzleHttp\\Psr7\\MultipartStream' => $vendorDir . '/guzzlehttp/psr7/src/MultipartStream.php',
    'GuzzleHttp\\Psr7\\NoSeekStream' => $vendorDir . '/guzzlehttp/psr7/src/NoSeekStream.php',
    'GuzzleHttp\\Psr7\\PumpStream' => $vendorDir . '/guzzlehttp/psr7/src/PumpStream.php',
    'GuzzleHttp\\Psr7\\Query' => $vendorDir . '/guzzlehttp/psr7/src/Query.php',
    'GuzzleHttp\\Psr7\\Request' => $vendorDir . '/guzzlehttp/psr7/src/Request.php',
    'GuzzleHttp\\Psr7\\Response' => $vendorDir . '/guzzlehttp/psr7/src/Response.php',
    'GuzzleHttp\\Psr7\\Rfc7230' => $vendorDir . '/guzzlehttp/psr7/src/Rfc7230.php',
    'GuzzleHttp\\Psr7\\ServerRequest' => $vendorDir . '/guzzlehttp/psr7/src/ServerRequest.php',
    'GuzzleHttp\\Psr7\\Stream' => $vendorDir . '/guzzlehttp/psr7/src/Stream.php',
    'GuzzleHttp\\Psr7\\StreamDecoratorTrait' => $vendorDir . '/guzzlehttp/psr7/src/StreamDecoratorTrait.php',
    'GuzzleHttp\\Psr7\\StreamWrapper' => $vendorDir . '/guzzlehttp/psr7/src/StreamWrapper.php',
    'GuzzleHttp\\Psr7\\UploadedFile' => $vendorDir . '/guzzlehttp/psr7/src/UploadedFile.php',
    'GuzzleHttp\\Psr7\\Uri' => $vendorDir . '/guzzlehttp/psr7/src/Uri.php',
    'GuzzleHttp\\Psr7\\UriComparator' => $vendorDir . '/guzzlehttp/psr7/src/UriComparator.php',
    'GuzzleHttp\\Psr7\\UriNormalizer' => $vendorDir . '/guzzlehttp/psr7/src/UriNormalizer.php',
    'GuzzleHttp\\Psr7\\UriResolver' => $vendorDir . '/guzzlehttp/psr7/src/UriResolver.php',
    'GuzzleHttp\\Psr7\\Utils' => $vendorDir . '/guzzlehttp/psr7/src/Utils.php',
    'GuzzleHttp\\RedirectMiddleware' => $vendorDir . '/guzzlehttp/guzzle/src/RedirectMiddleware.php',
    'GuzzleHttp\\RequestOptions' => $vendorDir . '/guzzlehttp/guzzle/src/RequestOptions.php',
    'GuzzleHttp\\RetryMiddleware' => $vendorDir . '/guzzlehttp/guzzle/src/RetryMiddleware.php',
    'GuzzleHttp\\TransferStats' => $vendorDir . '/guzzlehttp/guzzle/src/TransferStats.php',
    'GuzzleHttp\\UriTemplate\\UriTemplate' => $vendorDir . '/guzzlehttp/uri-template/src/UriTemplate.php',
    'GuzzleHttp\\Utils' => $vendorDir . '/guzzlehttp/guzzle/src/Utils.php',
    'HTMLPurifier' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier.php',
    'HTMLPurifier_Arborize' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Arborize.php',
    'HTMLPurifier_AttrCollections' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrCollections.php',
    'HTMLPurifier_AttrDef' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef.php',
    'HTMLPurifier_AttrDef_CSS' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS.php',
    'HTMLPurifier_AttrDef_CSS_AlphaValue' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/AlphaValue.php',
    'HTMLPurifier_AttrDef_CSS_Background' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Background.php',
    'HTMLPurifier_AttrDef_CSS_BackgroundPosition' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/BackgroundPosition.php',
    'HTMLPurifier_AttrDef_CSS_Border' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Border.php',
    'HTMLPurifier_AttrDef_CSS_Color' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Color.php',
    'HTMLPurifier_AttrDef_CSS_Composite' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Composite.php',
    'HTMLPurifier_AttrDef_CSS_DenyElementDecorator' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/DenyElementDecorator.php',
    'HTMLPurifier_AttrDef_CSS_Filter' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Filter.php',
    'HTMLPurifier_AttrDef_CSS_Font' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Font.php',
    'HTMLPurifier_AttrDef_CSS_FontFamily' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/FontFamily.php',
    'HTMLPurifier_AttrDef_CSS_Ident' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Ident.php',
    'HTMLPurifier_AttrDef_CSS_ImportantDecorator' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/ImportantDecorator.php',
    'HTMLPurifier_AttrDef_CSS_Length' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Length.php',
    'HTMLPurifier_AttrDef_CSS_ListStyle' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/ListStyle.php',
    'HTMLPurifier_AttrDef_CSS_Multiple' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Multiple.php',
    'HTMLPurifier_AttrDef_CSS_Number' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Number.php',
    'HTMLPurifier_AttrDef_CSS_Percentage' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Percentage.php',
    'HTMLPurifier_AttrDef_CSS_Ratio' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/Ratio.php',
    'HTMLPurifier_AttrDef_CSS_TextDecoration' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/TextDecoration.php',
    'HTMLPurifier_AttrDef_CSS_URI' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/CSS/URI.php',
    'HTMLPurifier_AttrDef_Clone' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Clone.php',
    'HTMLPurifier_AttrDef_Enum' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Enum.php',
    'HTMLPurifier_AttrDef_HTML_Bool' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Bool.php',
    'HTMLPurifier_AttrDef_HTML_Class' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Class.php',
    'HTMLPurifier_AttrDef_HTML_Color' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Color.php',
    'HTMLPurifier_AttrDef_HTML_ContentEditable' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/ContentEditable.php',
    'HTMLPurifier_AttrDef_HTML_FrameTarget' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/FrameTarget.php',
    'HTMLPurifier_AttrDef_HTML_ID' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/ID.php',
    'HTMLPurifier_AttrDef_HTML_Length' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Length.php',
    'HTMLPurifier_AttrDef_HTML_LinkTypes' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/LinkTypes.php',
    'HTMLPurifier_AttrDef_HTML_MultiLength' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/MultiLength.php',
    'HTMLPurifier_AttrDef_HTML_Nmtokens' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Nmtokens.php',
    'HTMLPurifier_AttrDef_HTML_Pixels' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/HTML/Pixels.php',
    'HTMLPurifier_AttrDef_Integer' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Integer.php',
    'HTMLPurifier_AttrDef_Lang' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Lang.php',
    'HTMLPurifier_AttrDef_Switch' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Switch.php',
    'HTMLPurifier_AttrDef_Text' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/Text.php',
    'HTMLPurifier_AttrDef_URI' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI.php',
    'HTMLPurifier_AttrDef_URI_Email' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/Email.php',
    'HTMLPurifier_AttrDef_URI_Email_SimpleCheck' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/Email/SimpleCheck.php',
    'HTMLPurifier_AttrDef_URI_Host' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/Host.php',
    'HTMLPurifier_AttrDef_URI_IPv4' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/IPv4.php',
    'HTMLPurifier_AttrDef_URI_IPv6' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrDef/URI/IPv6.php',
    'HTMLPurifier_AttrTransform' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform.php',
    'HTMLPurifier_AttrTransform_Background' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Background.php',
    'HTMLPurifier_AttrTransform_BdoDir' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/BdoDir.php',
    'HTMLPurifier_AttrTransform_BgColor' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/BgColor.php',
    'HTMLPurifier_AttrTransform_BoolToCSS' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/BoolToCSS.php',
    'HTMLPurifier_AttrTransform_Border' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Border.php',
    'HTMLPurifier_AttrTransform_EnumToCSS' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/EnumToCSS.php',
    'HTMLPurifier_AttrTransform_ImgRequired' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/ImgRequired.php',
    'HTMLPurifier_AttrTransform_ImgSpace' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/ImgSpace.php',
    'HTMLPurifier_AttrTransform_Input' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Input.php',
    'HTMLPurifier_AttrTransform_Lang' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Lang.php',
    'HTMLPurifier_AttrTransform_Length' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Length.php',
    'HTMLPurifier_AttrTransform_Name' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Name.php',
    'HTMLPurifier_AttrTransform_NameSync' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/NameSync.php',
    'HTMLPurifier_AttrTransform_Nofollow' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Nofollow.php',
    'HTMLPurifier_AttrTransform_SafeEmbed' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/SafeEmbed.php',
    'HTMLPurifier_AttrTransform_SafeObject' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/SafeObject.php',
    'HTMLPurifier_AttrTransform_SafeParam' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/SafeParam.php',
    'HTMLPurifier_AttrTransform_ScriptRequired' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/ScriptRequired.php',
    'HTMLPurifier_AttrTransform_TargetBlank' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/TargetBlank.php',
    'HTMLPurifier_AttrTransform_TargetNoopener' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/TargetNoopener.php',
    'HTMLPurifier_AttrTransform_TargetNoreferrer' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/TargetNoreferrer.php',
    'HTMLPurifier_AttrTransform_Textarea' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTransform/Textarea.php',
    'HTMLPurifier_AttrTypes' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrTypes.php',
    'HTMLPurifier_AttrValidator' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/AttrValidator.php',
    'HTMLPurifier_Bootstrap' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Bootstrap.php',
    'HTMLPurifier_CSSDefinition' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/CSSDefinition.php',
    'HTMLPurifier_ChildDef' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef.php',
    'HTMLPurifier_ChildDef_Chameleon' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Chameleon.php',
    'HTMLPurifier_ChildDef_Custom' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Custom.php',
    'HTMLPurifier_ChildDef_Empty' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Empty.php',
    'HTMLPurifier_ChildDef_List' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/List.php',
    'HTMLPurifier_ChildDef_Optional' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Optional.php',
    'HTMLPurifier_ChildDef_Required' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Required.php',
    'HTMLPurifier_ChildDef_StrictBlockquote' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/StrictBlockquote.php',
    'HTMLPurifier_ChildDef_Table' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ChildDef/Table.php',
    'HTMLPurifier_Config' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Config.php',
    'HTMLPurifier_ConfigSchema' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema.php',
    'HTMLPurifier_ConfigSchema_Builder_ConfigSchema' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Builder/ConfigSchema.php',
    'HTMLPurifier_ConfigSchema_Builder_Xml' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Builder/Xml.php',
    'HTMLPurifier_ConfigSchema_Exception' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Exception.php',
    'HTMLPurifier_ConfigSchema_Interchange' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Interchange.php',
    'HTMLPurifier_ConfigSchema_InterchangeBuilder' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/InterchangeBuilder.php',
    'HTMLPurifier_ConfigSchema_Interchange_Directive' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Interchange/Directive.php',
    'HTMLPurifier_ConfigSchema_Interchange_Id' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Interchange/Id.php',
    'HTMLPurifier_ConfigSchema_Validator' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/Validator.php',
    'HTMLPurifier_ConfigSchema_ValidatorAtom' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ConfigSchema/ValidatorAtom.php',
    'HTMLPurifier_ContentSets' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ContentSets.php',
    'HTMLPurifier_Context' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Context.php',
    'HTMLPurifier_Definition' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Definition.php',
    'HTMLPurifier_DefinitionCache' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache.php',
    'HTMLPurifier_DefinitionCacheFactory' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCacheFactory.php',
    'HTMLPurifier_DefinitionCache_Decorator' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Decorator.php',
    'HTMLPurifier_DefinitionCache_Decorator_Cleanup' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Decorator/Cleanup.php',
    'HTMLPurifier_DefinitionCache_Decorator_Memory' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Decorator/Memory.php',
    'HTMLPurifier_DefinitionCache_Null' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Null.php',
    'HTMLPurifier_DefinitionCache_Serializer' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/DefinitionCache/Serializer.php',
    'HTMLPurifier_Doctype' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Doctype.php',
    'HTMLPurifier_DoctypeRegistry' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/DoctypeRegistry.php',
    'HTMLPurifier_ElementDef' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ElementDef.php',
    'HTMLPurifier_Encoder' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Encoder.php',
    'HTMLPurifier_EntityLookup' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/EntityLookup.php',
    'HTMLPurifier_EntityParser' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/EntityParser.php',
    'HTMLPurifier_ErrorCollector' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ErrorCollector.php',
    'HTMLPurifier_ErrorStruct' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/ErrorStruct.php',
    'HTMLPurifier_Exception' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Exception.php',
    'HTMLPurifier_Filter' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Filter.php',
    'HTMLPurifier_Filter_ExtractStyleBlocks' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Filter/ExtractStyleBlocks.php',
    'HTMLPurifier_Filter_YouTube' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Filter/YouTube.php',
    'HTMLPurifier_Generator' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Generator.php',
    'HTMLPurifier_HTMLDefinition' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLDefinition.php',
    'HTMLPurifier_HTMLModule' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule.php',
    'HTMLPurifier_HTMLModuleManager' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModuleManager.php',
    'HTMLPurifier_HTMLModule_Bdo' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Bdo.php',
    'HTMLPurifier_HTMLModule_CommonAttributes' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/CommonAttributes.php',
    'HTMLPurifier_HTMLModule_Edit' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Edit.php',
    'HTMLPurifier_HTMLModule_Forms' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Forms.php',
    'HTMLPurifier_HTMLModule_Hypertext' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Hypertext.php',
    'HTMLPurifier_HTMLModule_Iframe' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Iframe.php',
    'HTMLPurifier_HTMLModule_Image' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Image.php',
    'HTMLPurifier_HTMLModule_Legacy' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Legacy.php',
    'HTMLPurifier_HTMLModule_List' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/List.php',
    'HTMLPurifier_HTMLModule_Name' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Name.php',
    'HTMLPurifier_HTMLModule_Nofollow' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Nofollow.php',
    'HTMLPurifier_HTMLModule_NonXMLCommonAttributes' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/NonXMLCommonAttributes.php',
    'HTMLPurifier_HTMLModule_Object' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Object.php',
    'HTMLPurifier_HTMLModule_Presentation' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Presentation.php',
    'HTMLPurifier_HTMLModule_Proprietary' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Proprietary.php',
    'HTMLPurifier_HTMLModule_Ruby' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Ruby.php',
    'HTMLPurifier_HTMLModule_SafeEmbed' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/SafeEmbed.php',
    'HTMLPurifier_HTMLModule_SafeObject' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/SafeObject.php',
    'HTMLPurifier_HTMLModule_SafeScripting' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/SafeScripting.php',
    'HTMLPurifier_HTMLModule_Scripting' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Scripting.php',
    'HTMLPurifier_HTMLModule_StyleAttribute' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/StyleAttribute.php',
    'HTMLPurifier_HTMLModule_Tables' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tables.php',
    'HTMLPurifier_HTMLModule_Target' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Target.php',
    'HTMLPurifier_HTMLModule_TargetBlank' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/TargetBlank.php',
    'HTMLPurifier_HTMLModule_TargetNoopener' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/TargetNoopener.php',
    'HTMLPurifier_HTMLModule_TargetNoreferrer' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/TargetNoreferrer.php',
    'HTMLPurifier_HTMLModule_Text' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Text.php',
    'HTMLPurifier_HTMLModule_Tidy' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy.php',
    'HTMLPurifier_HTMLModule_Tidy_Name' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/Name.php',
    'HTMLPurifier_HTMLModule_Tidy_Proprietary' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/Proprietary.php',
    'HTMLPurifier_HTMLModule_Tidy_Strict' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/Strict.php',
    'HTMLPurifier_HTMLModule_Tidy_Transitional' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/Transitional.php',
    'HTMLPurifier_HTMLModule_Tidy_XHTML' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/XHTML.php',
    'HTMLPurifier_HTMLModule_Tidy_XHTMLAndHTML4' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/Tidy/XHTMLAndHTML4.php',
    'HTMLPurifier_HTMLModule_XMLCommonAttributes' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/HTMLModule/XMLCommonAttributes.php',
    'HTMLPurifier_IDAccumulator' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/IDAccumulator.php',
    'HTMLPurifier_Injector' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Injector.php',
    'HTMLPurifier_Injector_AutoParagraph' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Injector/AutoParagraph.php',
    'HTMLPurifier_Injector_DisplayLinkURI' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Injector/DisplayLinkURI.php',
    'HTMLPurifier_Injector_Linkify' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Injector/Linkify.php',
    'HTMLPurifier_Injector_PurifierLinkify' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Injector/PurifierLinkify.php',
    'HTMLPurifier_Injector_RemoveEmpty' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Injector/RemoveEmpty.php',
    'HTMLPurifier_Injector_RemoveSpansWithoutAttributes' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Injector/RemoveSpansWithoutAttributes.php',
    'HTMLPurifier_Injector_SafeObject' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Injector/SafeObject.php',
    'HTMLPurifier_Language' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Language.php',
    'HTMLPurifier_LanguageFactory' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/LanguageFactory.php',
    'HTMLPurifier_Length' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Length.php',
    'HTMLPurifier_Lexer' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Lexer.php',
    'HTMLPurifier_Lexer_DOMLex' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Lexer/DOMLex.php',
    'HTMLPurifier_Lexer_DirectLex' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Lexer/DirectLex.php',
    'HTMLPurifier_Lexer_PH5P' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Lexer/PH5P.php',
    'HTMLPurifier_Node' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Node.php',
    'HTMLPurifier_Node_Comment' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Node/Comment.php',
    'HTMLPurifier_Node_Element' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Node/Element.php',
    'HTMLPurifier_Node_Text' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Node/Text.php',
    'HTMLPurifier_PercentEncoder' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/PercentEncoder.php',
    'HTMLPurifier_Printer' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Printer.php',
    'HTMLPurifier_Printer_CSSDefinition' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Printer/CSSDefinition.php',
    'HTMLPurifier_Printer_ConfigForm' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Printer/ConfigForm.php',
    'HTMLPurifier_Printer_HTMLDefinition' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Printer/HTMLDefinition.php',
    'HTMLPurifier_PropertyList' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/PropertyList.php',
    'HTMLPurifier_PropertyListIterator' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/PropertyListIterator.php',
    'HTMLPurifier_Queue' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Queue.php',
    'HTMLPurifier_Strategy' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Strategy.php',
    'HTMLPurifier_Strategy_Composite' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/Composite.php',
    'HTMLPurifier_Strategy_Core' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/Core.php',
    'HTMLPurifier_Strategy_FixNesting' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/FixNesting.php',
    'HTMLPurifier_Strategy_MakeWellFormed' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/MakeWellFormed.php',
    'HTMLPurifier_Strategy_RemoveForeignElements' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/RemoveForeignElements.php',
    'HTMLPurifier_Strategy_ValidateAttributes' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Strategy/ValidateAttributes.php',
    'HTMLPurifier_StringHash' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/StringHash.php',
    'HTMLPurifier_StringHashParser' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/StringHashParser.php',
    'HTMLPurifier_TagTransform' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/TagTransform.php',
    'HTMLPurifier_TagTransform_Font' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/TagTransform/Font.php',
    'HTMLPurifier_TagTransform_Simple' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/TagTransform/Simple.php',
    'HTMLPurifier_Token' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Token.php',
    'HTMLPurifier_TokenFactory' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/TokenFactory.php',
    'HTMLPurifier_Token_Comment' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Token/Comment.php',
    'HTMLPurifier_Token_Empty' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Token/Empty.php',
    'HTMLPurifier_Token_End' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Token/End.php',
    'HTMLPurifier_Token_Start' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Token/Start.php',
    'HTMLPurifier_Token_Tag' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Token/Tag.php',
    'HTMLPurifier_Token_Text' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Token/Text.php',
    'HTMLPurifier_URI' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URI.php',
    'HTMLPurifier_URIDefinition' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIDefinition.php',
    'HTMLPurifier_URIFilter' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter.php',
    'HTMLPurifier_URIFilter_DisableExternal' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/DisableExternal.php',
    'HTMLPurifier_URIFilter_DisableExternalResources' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/DisableExternalResources.php',
    'HTMLPurifier_URIFilter_DisableResources' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/DisableResources.php',
    'HTMLPurifier_URIFilter_HostBlacklist' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/HostBlacklist.php',
    'HTMLPurifier_URIFilter_MakeAbsolute' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/MakeAbsolute.php',
    'HTMLPurifier_URIFilter_Munge' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/Munge.php',
    'HTMLPurifier_URIFilter_SafeIframe' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIFilter/SafeIframe.php',
    'HTMLPurifier_URIParser' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIParser.php',
    'HTMLPurifier_URIScheme' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme.php',
    'HTMLPurifier_URISchemeRegistry' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URISchemeRegistry.php',
    'HTMLPurifier_URIScheme_data' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/data.php',
    'HTMLPurifier_URIScheme_file' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/file.php',
    'HTMLPurifier_URIScheme_ftp' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/ftp.php',
    'HTMLPurifier_URIScheme_http' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/http.php',
    'HTMLPurifier_URIScheme_https' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/https.php',
    'HTMLPurifier_URIScheme_mailto' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/mailto.php',
    'HTMLPurifier_URIScheme_news' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/news.php',
    'HTMLPurifier_URIScheme_nntp' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/nntp.php',
    'HTMLPurifier_URIScheme_tel' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/URIScheme/tel.php',
    'HTMLPurifier_UnitConverter' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/UnitConverter.php',
    'HTMLPurifier_VarParser' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/VarParser.php',
    'HTMLPurifier_VarParserException' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/VarParserException.php',
    'HTMLPurifier_VarParser_Flexible' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/VarParser/Flexible.php',
    'HTMLPurifier_VarParser_Native' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/VarParser/Native.php',
    'HTMLPurifier_Zipper' => $vendorDir . '/ezyang/htmlpurifier/library/HTMLPurifier/Zipper.php',
    'Hamcrest\\Arrays\\IsArray' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Arrays/IsArray.php',
    'Hamcrest\\Arrays\\IsArrayContaining' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Arrays/IsArrayContaining.php',
    'Hamcrest\\Arrays\\IsArrayContainingInAnyOrder' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Arrays/IsArrayContainingInAnyOrder.php',
    'Hamcrest\\Arrays\\IsArrayContainingInOrder' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Arrays/IsArrayContainingInOrder.php',
    'Hamcrest\\Arrays\\IsArrayContainingKey' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Arrays/IsArrayContainingKey.php',
    'Hamcrest\\Arrays\\IsArrayContainingKeyValuePair' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Arrays/IsArrayContainingKeyValuePair.php',
    'Hamcrest\\Arrays\\IsArrayWithSize' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Arrays/IsArrayWithSize.php',
    'Hamcrest\\Arrays\\MatchingOnce' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Arrays/MatchingOnce.php',
    'Hamcrest\\Arrays\\SeriesMatchingOnce' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Arrays/SeriesMatchingOnce.php',
    'Hamcrest\\AssertionError' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/AssertionError.php',
    'Hamcrest\\BaseDescription' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/BaseDescription.php',
    'Hamcrest\\BaseMatcher' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/BaseMatcher.php',
    'Hamcrest\\Collection\\IsEmptyTraversable' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Collection/IsEmptyTraversable.php',
    'Hamcrest\\Collection\\IsTraversableWithSize' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Collection/IsTraversableWithSize.php',
    'Hamcrest\\Core\\AllOf' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/AllOf.php',
    'Hamcrest\\Core\\AnyOf' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/AnyOf.php',
    'Hamcrest\\Core\\CombinableMatcher' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/CombinableMatcher.php',
    'Hamcrest\\Core\\DescribedAs' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/DescribedAs.php',
    'Hamcrest\\Core\\Every' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/Every.php',
    'Hamcrest\\Core\\HasToString' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/HasToString.php',
    'Hamcrest\\Core\\Is' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/Is.php',
    'Hamcrest\\Core\\IsAnything' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/IsAnything.php',
    'Hamcrest\\Core\\IsCollectionContaining' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/IsCollectionContaining.php',
    'Hamcrest\\Core\\IsEqual' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/IsEqual.php',
    'Hamcrest\\Core\\IsIdentical' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/IsIdentical.php',
    'Hamcrest\\Core\\IsInstanceOf' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/IsInstanceOf.php',
    'Hamcrest\\Core\\IsNot' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/IsNot.php',
    'Hamcrest\\Core\\IsNull' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/IsNull.php',
    'Hamcrest\\Core\\IsSame' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/IsSame.php',
    'Hamcrest\\Core\\IsTypeOf' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/IsTypeOf.php',
    'Hamcrest\\Core\\Set' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/Set.php',
    'Hamcrest\\Core\\ShortcutCombination' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Core/ShortcutCombination.php',
    'Hamcrest\\Description' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Description.php',
    'Hamcrest\\DiagnosingMatcher' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/DiagnosingMatcher.php',
    'Hamcrest\\FeatureMatcher' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/FeatureMatcher.php',
    'Hamcrest\\Internal\\SelfDescribingValue' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Internal/SelfDescribingValue.php',
    'Hamcrest\\Matcher' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Matcher.php',
    'Hamcrest\\MatcherAssert' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/MatcherAssert.php',
    'Hamcrest\\Matchers' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Matchers.php',
    'Hamcrest\\NullDescription' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/NullDescription.php',
    'Hamcrest\\Number\\IsCloseTo' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Number/IsCloseTo.php',
    'Hamcrest\\Number\\OrderingComparison' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Number/OrderingComparison.php',
    'Hamcrest\\SelfDescribing' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/SelfDescribing.php',
    'Hamcrest\\StringDescription' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/StringDescription.php',
    'Hamcrest\\Text\\IsEmptyString' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/IsEmptyString.php',
    'Hamcrest\\Text\\IsEqualIgnoringCase' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/IsEqualIgnoringCase.php',
    'Hamcrest\\Text\\IsEqualIgnoringWhiteSpace' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/IsEqualIgnoringWhiteSpace.php',
    'Hamcrest\\Text\\MatchesPattern' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/MatchesPattern.php',
    'Hamcrest\\Text\\StringContains' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/StringContains.php',
    'Hamcrest\\Text\\StringContainsIgnoringCase' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/StringContainsIgnoringCase.php',
    'Hamcrest\\Text\\StringContainsInOrder' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/StringContainsInOrder.php',
    'Hamcrest\\Text\\StringEndsWith' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/StringEndsWith.php',
    'Hamcrest\\Text\\StringStartsWith' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/StringStartsWith.php',
    'Hamcrest\\Text\\SubstringMatcher' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Text/SubstringMatcher.php',
    'Hamcrest\\TypeSafeDiagnosingMatcher' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/TypeSafeDiagnosingMatcher.php',
    'Hamcrest\\TypeSafeMatcher' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/TypeSafeMatcher.php',
    'Hamcrest\\Type\\IsArray' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsArray.php',
    'Hamcrest\\Type\\IsBoolean' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsBoolean.php',
    'Hamcrest\\Type\\IsCallable' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsCallable.php',
    'Hamcrest\\Type\\IsDouble' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsDouble.php',
    'Hamcrest\\Type\\IsInteger' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsInteger.php',
    'Hamcrest\\Type\\IsNumeric' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsNumeric.php',
    'Hamcrest\\Type\\IsObject' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsObject.php',
    'Hamcrest\\Type\\IsResource' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsResource.php',
    'Hamcrest\\Type\\IsScalar' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsScalar.php',
    'Hamcrest\\Type\\IsString' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Type/IsString.php',
    'Hamcrest\\Util' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Util.php',
    'Hamcrest\\Xml\\HasXPath' => $vendorDir . '/hamcrest/hamcrest-php/hamcrest/Hamcrest/Xml/HasXPath.php',
    'Illuminate\\Auth\\Access\\AuthorizationException' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Access/AuthorizationException.php',
    'Illuminate\\Auth\\Access\\Events\\GateEvaluated' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Access/Events/GateEvaluated.php',
    'Illuminate\\Auth\\Access\\Gate' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Access/Gate.php',
    'Illuminate\\Auth\\Access\\HandlesAuthorization' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Access/HandlesAuthorization.php',
    'Illuminate\\Auth\\Access\\Response' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Access/Response.php',
    'Illuminate\\Auth\\AuthManager' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/AuthManager.php',
    'Illuminate\\Auth\\AuthServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/AuthServiceProvider.php',
    'Illuminate\\Auth\\Authenticatable' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Authenticatable.php',
    'Illuminate\\Auth\\AuthenticationException' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/AuthenticationException.php',
    'Illuminate\\Auth\\Console\\ClearResetsCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Console/ClearResetsCommand.php',
    'Illuminate\\Auth\\CreatesUserProviders' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/CreatesUserProviders.php',
    'Illuminate\\Auth\\DatabaseUserProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/DatabaseUserProvider.php',
    'Illuminate\\Auth\\EloquentUserProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/EloquentUserProvider.php',
    'Illuminate\\Auth\\Events\\Attempting' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/Attempting.php',
    'Illuminate\\Auth\\Events\\Authenticated' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/Authenticated.php',
    'Illuminate\\Auth\\Events\\CurrentDeviceLogout' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/CurrentDeviceLogout.php',
    'Illuminate\\Auth\\Events\\Failed' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/Failed.php',
    'Illuminate\\Auth\\Events\\Lockout' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/Lockout.php',
    'Illuminate\\Auth\\Events\\Login' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/Login.php',
    'Illuminate\\Auth\\Events\\Logout' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/Logout.php',
    'Illuminate\\Auth\\Events\\OtherDeviceLogout' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/OtherDeviceLogout.php',
    'Illuminate\\Auth\\Events\\PasswordReset' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/PasswordReset.php',
    'Illuminate\\Auth\\Events\\Registered' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/Registered.php',
    'Illuminate\\Auth\\Events\\Validated' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/Validated.php',
    'Illuminate\\Auth\\Events\\Verified' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Events/Verified.php',
    'Illuminate\\Auth\\GenericUser' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/GenericUser.php',
    'Illuminate\\Auth\\GuardHelpers' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/GuardHelpers.php',
    'Illuminate\\Auth\\Listeners\\SendEmailVerificationNotification' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Listeners/SendEmailVerificationNotification.php',
    'Illuminate\\Auth\\Middleware\\Authenticate' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Middleware/Authenticate.php',
    'Illuminate\\Auth\\Middleware\\AuthenticateWithBasicAuth' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Middleware/AuthenticateWithBasicAuth.php',
    'Illuminate\\Auth\\Middleware\\Authorize' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Middleware/Authorize.php',
    'Illuminate\\Auth\\Middleware\\EnsureEmailIsVerified' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Middleware/EnsureEmailIsVerified.php',
    'Illuminate\\Auth\\Middleware\\RequirePassword' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Middleware/RequirePassword.php',
    'Illuminate\\Auth\\MustVerifyEmail' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/MustVerifyEmail.php',
    'Illuminate\\Auth\\Notifications\\ResetPassword' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Notifications/ResetPassword.php',
    'Illuminate\\Auth\\Notifications\\VerifyEmail' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Notifications/VerifyEmail.php',
    'Illuminate\\Auth\\Passwords\\CanResetPassword' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Passwords/CanResetPassword.php',
    'Illuminate\\Auth\\Passwords\\DatabaseTokenRepository' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Passwords/DatabaseTokenRepository.php',
    'Illuminate\\Auth\\Passwords\\PasswordBroker' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Passwords/PasswordBroker.php',
    'Illuminate\\Auth\\Passwords\\PasswordBrokerManager' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Passwords/PasswordBrokerManager.php',
    'Illuminate\\Auth\\Passwords\\PasswordResetServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Passwords/PasswordResetServiceProvider.php',
    'Illuminate\\Auth\\Passwords\\TokenRepositoryInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Passwords/TokenRepositoryInterface.php',
    'Illuminate\\Auth\\Recaller' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/Recaller.php',
    'Illuminate\\Auth\\RequestGuard' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/RequestGuard.php',
    'Illuminate\\Auth\\SessionGuard' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/SessionGuard.php',
    'Illuminate\\Auth\\TokenGuard' => $vendorDir . '/laravel/framework/src/Illuminate/Auth/TokenGuard.php',
    'Illuminate\\Broadcasting\\BroadcastController' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/BroadcastController.php',
    'Illuminate\\Broadcasting\\BroadcastEvent' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/BroadcastEvent.php',
    'Illuminate\\Broadcasting\\BroadcastException' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/BroadcastException.php',
    'Illuminate\\Broadcasting\\BroadcastManager' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/BroadcastManager.php',
    'Illuminate\\Broadcasting\\BroadcastServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/BroadcastServiceProvider.php',
    'Illuminate\\Broadcasting\\Broadcasters\\AblyBroadcaster' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/Broadcasters/AblyBroadcaster.php',
    'Illuminate\\Broadcasting\\Broadcasters\\Broadcaster' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/Broadcasters/Broadcaster.php',
    'Illuminate\\Broadcasting\\Broadcasters\\LogBroadcaster' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/Broadcasters/LogBroadcaster.php',
    'Illuminate\\Broadcasting\\Broadcasters\\NullBroadcaster' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/Broadcasters/NullBroadcaster.php',
    'Illuminate\\Broadcasting\\Broadcasters\\PusherBroadcaster' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/Broadcasters/PusherBroadcaster.php',
    'Illuminate\\Broadcasting\\Broadcasters\\RedisBroadcaster' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/Broadcasters/RedisBroadcaster.php',
    'Illuminate\\Broadcasting\\Broadcasters\\UsePusherChannelConventions' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/Broadcasters/UsePusherChannelConventions.php',
    'Illuminate\\Broadcasting\\Channel' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/Channel.php',
    'Illuminate\\Broadcasting\\EncryptedPrivateChannel' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/EncryptedPrivateChannel.php',
    'Illuminate\\Broadcasting\\InteractsWithBroadcasting' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/InteractsWithBroadcasting.php',
    'Illuminate\\Broadcasting\\InteractsWithSockets' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/InteractsWithSockets.php',
    'Illuminate\\Broadcasting\\PendingBroadcast' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/PendingBroadcast.php',
    'Illuminate\\Broadcasting\\PresenceChannel' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/PresenceChannel.php',
    'Illuminate\\Broadcasting\\PrivateChannel' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/PrivateChannel.php',
    'Illuminate\\Broadcasting\\UniqueBroadcastEvent' => $vendorDir . '/laravel/framework/src/Illuminate/Broadcasting/UniqueBroadcastEvent.php',
    'Illuminate\\Bus\\Batch' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/Batch.php',
    'Illuminate\\Bus\\BatchFactory' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/BatchFactory.php',
    'Illuminate\\Bus\\BatchRepository' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/BatchRepository.php',
    'Illuminate\\Bus\\Batchable' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/Batchable.php',
    'Illuminate\\Bus\\BusServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/BusServiceProvider.php',
    'Illuminate\\Bus\\ChainedBatch' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/ChainedBatch.php',
    'Illuminate\\Bus\\DatabaseBatchRepository' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/DatabaseBatchRepository.php',
    'Illuminate\\Bus\\Dispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/Dispatcher.php',
    'Illuminate\\Bus\\DynamoBatchRepository' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/DynamoBatchRepository.php',
    'Illuminate\\Bus\\Events\\BatchDispatched' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/Events/BatchDispatched.php',
    'Illuminate\\Bus\\PendingBatch' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/PendingBatch.php',
    'Illuminate\\Bus\\PrunableBatchRepository' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/PrunableBatchRepository.php',
    'Illuminate\\Bus\\Queueable' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/Queueable.php',
    'Illuminate\\Bus\\UniqueLock' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/UniqueLock.php',
    'Illuminate\\Bus\\UpdatedBatchJobCounts' => $vendorDir . '/laravel/framework/src/Illuminate/Bus/UpdatedBatchJobCounts.php',
    'Illuminate\\Cache\\ApcStore' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/ApcStore.php',
    'Illuminate\\Cache\\ApcWrapper' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/ApcWrapper.php',
    'Illuminate\\Cache\\ArrayLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/ArrayLock.php',
    'Illuminate\\Cache\\ArrayStore' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/ArrayStore.php',
    'Illuminate\\Cache\\CacheLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/CacheLock.php',
    'Illuminate\\Cache\\CacheManager' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/CacheManager.php',
    'Illuminate\\Cache\\CacheServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/CacheServiceProvider.php',
    'Illuminate\\Cache\\Console\\CacheTableCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Console/CacheTableCommand.php',
    'Illuminate\\Cache\\Console\\ClearCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Console/ClearCommand.php',
    'Illuminate\\Cache\\Console\\ForgetCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Console/ForgetCommand.php',
    'Illuminate\\Cache\\Console\\PruneStaleTagsCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Console/PruneStaleTagsCommand.php',
    'Illuminate\\Cache\\DatabaseLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/DatabaseLock.php',
    'Illuminate\\Cache\\DatabaseStore' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/DatabaseStore.php',
    'Illuminate\\Cache\\DynamoDbLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/DynamoDbLock.php',
    'Illuminate\\Cache\\DynamoDbStore' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/DynamoDbStore.php',
    'Illuminate\\Cache\\Events\\CacheEvent' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Events/CacheEvent.php',
    'Illuminate\\Cache\\Events\\CacheHit' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Events/CacheHit.php',
    'Illuminate\\Cache\\Events\\CacheMissed' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Events/CacheMissed.php',
    'Illuminate\\Cache\\Events\\KeyForgotten' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Events/KeyForgotten.php',
    'Illuminate\\Cache\\Events\\KeyWritten' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Events/KeyWritten.php',
    'Illuminate\\Cache\\FileLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/FileLock.php',
    'Illuminate\\Cache\\FileStore' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/FileStore.php',
    'Illuminate\\Cache\\HasCacheLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/HasCacheLock.php',
    'Illuminate\\Cache\\Lock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Lock.php',
    'Illuminate\\Cache\\LuaScripts' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/LuaScripts.php',
    'Illuminate\\Cache\\MemcachedConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/MemcachedConnector.php',
    'Illuminate\\Cache\\MemcachedLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/MemcachedLock.php',
    'Illuminate\\Cache\\MemcachedStore' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/MemcachedStore.php',
    'Illuminate\\Cache\\NoLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/NoLock.php',
    'Illuminate\\Cache\\NullStore' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/NullStore.php',
    'Illuminate\\Cache\\PhpRedisLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/PhpRedisLock.php',
    'Illuminate\\Cache\\RateLimiter' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/RateLimiter.php',
    'Illuminate\\Cache\\RateLimiting\\GlobalLimit' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/RateLimiting/GlobalLimit.php',
    'Illuminate\\Cache\\RateLimiting\\Limit' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/RateLimiting/Limit.php',
    'Illuminate\\Cache\\RateLimiting\\Unlimited' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/RateLimiting/Unlimited.php',
    'Illuminate\\Cache\\RedisLock' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/RedisLock.php',
    'Illuminate\\Cache\\RedisStore' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/RedisStore.php',
    'Illuminate\\Cache\\RedisTagSet' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/RedisTagSet.php',
    'Illuminate\\Cache\\RedisTaggedCache' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/RedisTaggedCache.php',
    'Illuminate\\Cache\\Repository' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/Repository.php',
    'Illuminate\\Cache\\RetrievesMultipleKeys' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/RetrievesMultipleKeys.php',
    'Illuminate\\Cache\\TagSet' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/TagSet.php',
    'Illuminate\\Cache\\TaggableStore' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/TaggableStore.php',
    'Illuminate\\Cache\\TaggedCache' => $vendorDir . '/laravel/framework/src/Illuminate/Cache/TaggedCache.php',
    'Illuminate\\Config\\Repository' => $vendorDir . '/laravel/framework/src/Illuminate/Config/Repository.php',
    'Illuminate\\Console\\Application' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Application.php',
    'Illuminate\\Console\\BufferedConsoleOutput' => $vendorDir . '/laravel/framework/src/Illuminate/Console/BufferedConsoleOutput.php',
    'Illuminate\\Console\\CacheCommandMutex' => $vendorDir . '/laravel/framework/src/Illuminate/Console/CacheCommandMutex.php',
    'Illuminate\\Console\\Command' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Command.php',
    'Illuminate\\Console\\CommandMutex' => $vendorDir . '/laravel/framework/src/Illuminate/Console/CommandMutex.php',
    'Illuminate\\Console\\Concerns\\CallsCommands' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Concerns/CallsCommands.php',
    'Illuminate\\Console\\Concerns\\ConfiguresPrompts' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Concerns/ConfiguresPrompts.php',
    'Illuminate\\Console\\Concerns\\CreatesMatchingTest' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Concerns/CreatesMatchingTest.php',
    'Illuminate\\Console\\Concerns\\HasParameters' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Concerns/HasParameters.php',
    'Illuminate\\Console\\Concerns\\InteractsWithIO' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Concerns/InteractsWithIO.php',
    'Illuminate\\Console\\Concerns\\InteractsWithSignals' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Concerns/InteractsWithSignals.php',
    'Illuminate\\Console\\Concerns\\PromptsForMissingInput' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Concerns/PromptsForMissingInput.php',
    'Illuminate\\Console\\ConfirmableTrait' => $vendorDir . '/laravel/framework/src/Illuminate/Console/ConfirmableTrait.php',
    'Illuminate\\Console\\ContainerCommandLoader' => $vendorDir . '/laravel/framework/src/Illuminate/Console/ContainerCommandLoader.php',
    'Illuminate\\Console\\Contracts\\NewLineAware' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Contracts/NewLineAware.php',
    'Illuminate\\Console\\Events\\ArtisanStarting' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Events/ArtisanStarting.php',
    'Illuminate\\Console\\Events\\CommandFinished' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Events/CommandFinished.php',
    'Illuminate\\Console\\Events\\CommandStarting' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Events/CommandStarting.php',
    'Illuminate\\Console\\Events\\ScheduledBackgroundTaskFinished' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Events/ScheduledBackgroundTaskFinished.php',
    'Illuminate\\Console\\Events\\ScheduledTaskFailed' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Events/ScheduledTaskFailed.php',
    'Illuminate\\Console\\Events\\ScheduledTaskFinished' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Events/ScheduledTaskFinished.php',
    'Illuminate\\Console\\Events\\ScheduledTaskSkipped' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Events/ScheduledTaskSkipped.php',
    'Illuminate\\Console\\Events\\ScheduledTaskStarting' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Events/ScheduledTaskStarting.php',
    'Illuminate\\Console\\GeneratorCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Console/GeneratorCommand.php',
    'Illuminate\\Console\\MigrationGeneratorCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Console/MigrationGeneratorCommand.php',
    'Illuminate\\Console\\OutputStyle' => $vendorDir . '/laravel/framework/src/Illuminate/Console/OutputStyle.php',
    'Illuminate\\Console\\Parser' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Parser.php',
    'Illuminate\\Console\\PromptValidationException' => $vendorDir . '/laravel/framework/src/Illuminate/Console/PromptValidationException.php',
    'Illuminate\\Console\\QuestionHelper' => $vendorDir . '/laravel/framework/src/Illuminate/Console/QuestionHelper.php',
    'Illuminate\\Console\\Scheduling\\CacheAware' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/CacheAware.php',
    'Illuminate\\Console\\Scheduling\\CacheEventMutex' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/CacheEventMutex.php',
    'Illuminate\\Console\\Scheduling\\CacheSchedulingMutex' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/CacheSchedulingMutex.php',
    'Illuminate\\Console\\Scheduling\\CallbackEvent' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/CallbackEvent.php',
    'Illuminate\\Console\\Scheduling\\CommandBuilder' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/CommandBuilder.php',
    'Illuminate\\Console\\Scheduling\\Event' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/Event.php',
    'Illuminate\\Console\\Scheduling\\EventMutex' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/EventMutex.php',
    'Illuminate\\Console\\Scheduling\\ManagesFrequencies' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/ManagesFrequencies.php',
    'Illuminate\\Console\\Scheduling\\Schedule' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/Schedule.php',
    'Illuminate\\Console\\Scheduling\\ScheduleClearCacheCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/ScheduleClearCacheCommand.php',
    'Illuminate\\Console\\Scheduling\\ScheduleFinishCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/ScheduleFinishCommand.php',
    'Illuminate\\Console\\Scheduling\\ScheduleInterruptCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/ScheduleInterruptCommand.php',
    'Illuminate\\Console\\Scheduling\\ScheduleListCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/ScheduleListCommand.php',
    'Illuminate\\Console\\Scheduling\\ScheduleRunCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/ScheduleRunCommand.php',
    'Illuminate\\Console\\Scheduling\\ScheduleTestCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/ScheduleTestCommand.php',
    'Illuminate\\Console\\Scheduling\\ScheduleWorkCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/ScheduleWorkCommand.php',
    'Illuminate\\Console\\Scheduling\\SchedulingMutex' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Scheduling/SchedulingMutex.php',
    'Illuminate\\Console\\Signals' => $vendorDir . '/laravel/framework/src/Illuminate/Console/Signals.php',
    'Illuminate\\Console\\View\\Components\\Alert' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Alert.php',
    'Illuminate\\Console\\View\\Components\\Ask' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Ask.php',
    'Illuminate\\Console\\View\\Components\\AskWithCompletion' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/AskWithCompletion.php',
    'Illuminate\\Console\\View\\Components\\BulletList' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/BulletList.php',
    'Illuminate\\Console\\View\\Components\\Choice' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Choice.php',
    'Illuminate\\Console\\View\\Components\\Component' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Component.php',
    'Illuminate\\Console\\View\\Components\\Confirm' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Confirm.php',
    'Illuminate\\Console\\View\\Components\\Error' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Error.php',
    'Illuminate\\Console\\View\\Components\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Factory.php',
    'Illuminate\\Console\\View\\Components\\Info' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Info.php',
    'Illuminate\\Console\\View\\Components\\Line' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Line.php',
    'Illuminate\\Console\\View\\Components\\Mutators\\EnsureDynamicContentIsHighlighted' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Mutators/EnsureDynamicContentIsHighlighted.php',
    'Illuminate\\Console\\View\\Components\\Mutators\\EnsureNoPunctuation' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Mutators/EnsureNoPunctuation.php',
    'Illuminate\\Console\\View\\Components\\Mutators\\EnsurePunctuation' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Mutators/EnsurePunctuation.php',
    'Illuminate\\Console\\View\\Components\\Mutators\\EnsureRelativePaths' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Mutators/EnsureRelativePaths.php',
    'Illuminate\\Console\\View\\Components\\Secret' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Secret.php',
    'Illuminate\\Console\\View\\Components\\Task' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Task.php',
    'Illuminate\\Console\\View\\Components\\TwoColumnDetail' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/TwoColumnDetail.php',
    'Illuminate\\Console\\View\\Components\\Warn' => $vendorDir . '/laravel/framework/src/Illuminate/Console/View/Components/Warn.php',
    'Illuminate\\Container\\BoundMethod' => $vendorDir . '/laravel/framework/src/Illuminate/Container/BoundMethod.php',
    'Illuminate\\Container\\Container' => $vendorDir . '/laravel/framework/src/Illuminate/Container/Container.php',
    'Illuminate\\Container\\ContextualBindingBuilder' => $vendorDir . '/laravel/framework/src/Illuminate/Container/ContextualBindingBuilder.php',
    'Illuminate\\Container\\EntryNotFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Container/EntryNotFoundException.php',
    'Illuminate\\Container\\RewindableGenerator' => $vendorDir . '/laravel/framework/src/Illuminate/Container/RewindableGenerator.php',
    'Illuminate\\Container\\Util' => $vendorDir . '/laravel/framework/src/Illuminate/Container/Util.php',
    'Illuminate\\Contracts\\Auth\\Access\\Authorizable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/Access/Authorizable.php',
    'Illuminate\\Contracts\\Auth\\Access\\Gate' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/Access/Gate.php',
    'Illuminate\\Contracts\\Auth\\Authenticatable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/Authenticatable.php',
    'Illuminate\\Contracts\\Auth\\CanResetPassword' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/CanResetPassword.php',
    'Illuminate\\Contracts\\Auth\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/Factory.php',
    'Illuminate\\Contracts\\Auth\\Guard' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/Guard.php',
    'Illuminate\\Contracts\\Auth\\Middleware\\AuthenticatesRequests' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/Middleware/AuthenticatesRequests.php',
    'Illuminate\\Contracts\\Auth\\MustVerifyEmail' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/MustVerifyEmail.php',
    'Illuminate\\Contracts\\Auth\\PasswordBroker' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/PasswordBroker.php',
    'Illuminate\\Contracts\\Auth\\PasswordBrokerFactory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/PasswordBrokerFactory.php',
    'Illuminate\\Contracts\\Auth\\StatefulGuard' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/StatefulGuard.php',
    'Illuminate\\Contracts\\Auth\\SupportsBasicAuth' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/SupportsBasicAuth.php',
    'Illuminate\\Contracts\\Auth\\UserProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Auth/UserProvider.php',
    'Illuminate\\Contracts\\Broadcasting\\Broadcaster' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Broadcasting/Broadcaster.php',
    'Illuminate\\Contracts\\Broadcasting\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Broadcasting/Factory.php',
    'Illuminate\\Contracts\\Broadcasting\\HasBroadcastChannel' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Broadcasting/HasBroadcastChannel.php',
    'Illuminate\\Contracts\\Broadcasting\\ShouldBeUnique' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Broadcasting/ShouldBeUnique.php',
    'Illuminate\\Contracts\\Broadcasting\\ShouldBroadcast' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Broadcasting/ShouldBroadcast.php',
    'Illuminate\\Contracts\\Broadcasting\\ShouldBroadcastNow' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Broadcasting/ShouldBroadcastNow.php',
    'Illuminate\\Contracts\\Bus\\Dispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Bus/Dispatcher.php',
    'Illuminate\\Contracts\\Bus\\QueueingDispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Bus/QueueingDispatcher.php',
    'Illuminate\\Contracts\\Cache\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Cache/Factory.php',
    'Illuminate\\Contracts\\Cache\\Lock' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Cache/Lock.php',
    'Illuminate\\Contracts\\Cache\\LockProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Cache/LockProvider.php',
    'Illuminate\\Contracts\\Cache\\LockTimeoutException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Cache/LockTimeoutException.php',
    'Illuminate\\Contracts\\Cache\\Repository' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Cache/Repository.php',
    'Illuminate\\Contracts\\Cache\\Store' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Cache/Store.php',
    'Illuminate\\Contracts\\Config\\Repository' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Config/Repository.php',
    'Illuminate\\Contracts\\Console\\Application' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Console/Application.php',
    'Illuminate\\Contracts\\Console\\Isolatable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Console/Isolatable.php',
    'Illuminate\\Contracts\\Console\\Kernel' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Console/Kernel.php',
    'Illuminate\\Contracts\\Console\\PromptsForMissingInput' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Console/PromptsForMissingInput.php',
    'Illuminate\\Contracts\\Container\\BindingResolutionException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Container/BindingResolutionException.php',
    'Illuminate\\Contracts\\Container\\CircularDependencyException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Container/CircularDependencyException.php',
    'Illuminate\\Contracts\\Container\\Container' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Container/Container.php',
    'Illuminate\\Contracts\\Container\\ContextualBindingBuilder' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Container/ContextualBindingBuilder.php',
    'Illuminate\\Contracts\\Cookie\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Cookie/Factory.php',
    'Illuminate\\Contracts\\Cookie\\QueueingFactory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Cookie/QueueingFactory.php',
    'Illuminate\\Contracts\\Database\\Eloquent\\Builder' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Eloquent/Builder.php',
    'Illuminate\\Contracts\\Database\\Eloquent\\Castable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Eloquent/Castable.php',
    'Illuminate\\Contracts\\Database\\Eloquent\\CastsAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Eloquent/CastsAttributes.php',
    'Illuminate\\Contracts\\Database\\Eloquent\\CastsInboundAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Eloquent/CastsInboundAttributes.php',
    'Illuminate\\Contracts\\Database\\Eloquent\\DeviatesCastableAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Eloquent/DeviatesCastableAttributes.php',
    'Illuminate\\Contracts\\Database\\Eloquent\\SerializesCastableAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Eloquent/SerializesCastableAttributes.php',
    'Illuminate\\Contracts\\Database\\Eloquent\\SupportsPartialRelations' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Eloquent/SupportsPartialRelations.php',
    'Illuminate\\Contracts\\Database\\Events\\MigrationEvent' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Events/MigrationEvent.php',
    'Illuminate\\Contracts\\Database\\ModelIdentifier' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/ModelIdentifier.php',
    'Illuminate\\Contracts\\Database\\Query\\Builder' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Query/Builder.php',
    'Illuminate\\Contracts\\Database\\Query\\ConditionExpression' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Query/ConditionExpression.php',
    'Illuminate\\Contracts\\Database\\Query\\Expression' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Database/Query/Expression.php',
    'Illuminate\\Contracts\\Debug\\ExceptionHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Debug/ExceptionHandler.php',
    'Illuminate\\Contracts\\Encryption\\DecryptException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Encryption/DecryptException.php',
    'Illuminate\\Contracts\\Encryption\\EncryptException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Encryption/EncryptException.php',
    'Illuminate\\Contracts\\Encryption\\Encrypter' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Encryption/Encrypter.php',
    'Illuminate\\Contracts\\Encryption\\StringEncrypter' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Encryption/StringEncrypter.php',
    'Illuminate\\Contracts\\Events\\Dispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Events/Dispatcher.php',
    'Illuminate\\Contracts\\Events\\ShouldDispatchAfterCommit' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Events/ShouldDispatchAfterCommit.php',
    'Illuminate\\Contracts\\Events\\ShouldHandleEventsAfterCommit' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Events/ShouldHandleEventsAfterCommit.php',
    'Illuminate\\Contracts\\Filesystem\\Cloud' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Filesystem/Cloud.php',
    'Illuminate\\Contracts\\Filesystem\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Filesystem/Factory.php',
    'Illuminate\\Contracts\\Filesystem\\FileNotFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Filesystem/FileNotFoundException.php',
    'Illuminate\\Contracts\\Filesystem\\Filesystem' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Filesystem/Filesystem.php',
    'Illuminate\\Contracts\\Filesystem\\LockTimeoutException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Filesystem/LockTimeoutException.php',
    'Illuminate\\Contracts\\Foundation\\Application' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Foundation/Application.php',
    'Illuminate\\Contracts\\Foundation\\CachesConfiguration' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Foundation/CachesConfiguration.php',
    'Illuminate\\Contracts\\Foundation\\CachesRoutes' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Foundation/CachesRoutes.php',
    'Illuminate\\Contracts\\Foundation\\ExceptionRenderer' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Foundation/ExceptionRenderer.php',
    'Illuminate\\Contracts\\Foundation\\MaintenanceMode' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Foundation/MaintenanceMode.php',
    'Illuminate\\Contracts\\Hashing\\Hasher' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Hashing/Hasher.php',
    'Illuminate\\Contracts\\Http\\Kernel' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Http/Kernel.php',
    'Illuminate\\Contracts\\Mail\\Attachable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Mail/Attachable.php',
    'Illuminate\\Contracts\\Mail\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Mail/Factory.php',
    'Illuminate\\Contracts\\Mail\\MailQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Mail/MailQueue.php',
    'Illuminate\\Contracts\\Mail\\Mailable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Mail/Mailable.php',
    'Illuminate\\Contracts\\Mail\\Mailer' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Mail/Mailer.php',
    'Illuminate\\Contracts\\Notifications\\Dispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Notifications/Dispatcher.php',
    'Illuminate\\Contracts\\Notifications\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Notifications/Factory.php',
    'Illuminate\\Contracts\\Pagination\\CursorPaginator' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Pagination/CursorPaginator.php',
    'Illuminate\\Contracts\\Pagination\\LengthAwarePaginator' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Pagination/LengthAwarePaginator.php',
    'Illuminate\\Contracts\\Pagination\\Paginator' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Pagination/Paginator.php',
    'Illuminate\\Contracts\\Pipeline\\Hub' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Pipeline/Hub.php',
    'Illuminate\\Contracts\\Pipeline\\Pipeline' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Pipeline/Pipeline.php',
    'Illuminate\\Contracts\\Process\\InvokedProcess' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Process/InvokedProcess.php',
    'Illuminate\\Contracts\\Process\\ProcessResult' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Process/ProcessResult.php',
    'Illuminate\\Contracts\\Queue\\ClearableQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/ClearableQueue.php',
    'Illuminate\\Contracts\\Queue\\EntityNotFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/EntityNotFoundException.php',
    'Illuminate\\Contracts\\Queue\\EntityResolver' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/EntityResolver.php',
    'Illuminate\\Contracts\\Queue\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/Factory.php',
    'Illuminate\\Contracts\\Queue\\Job' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/Job.php',
    'Illuminate\\Contracts\\Queue\\Monitor' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/Monitor.php',
    'Illuminate\\Contracts\\Queue\\Queue' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/Queue.php',
    'Illuminate\\Contracts\\Queue\\QueueableCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/QueueableCollection.php',
    'Illuminate\\Contracts\\Queue\\QueueableEntity' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/QueueableEntity.php',
    'Illuminate\\Contracts\\Queue\\ShouldBeEncrypted' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/ShouldBeEncrypted.php',
    'Illuminate\\Contracts\\Queue\\ShouldBeUnique' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/ShouldBeUnique.php',
    'Illuminate\\Contracts\\Queue\\ShouldBeUniqueUntilProcessing' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/ShouldBeUniqueUntilProcessing.php',
    'Illuminate\\Contracts\\Queue\\ShouldQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/ShouldQueue.php',
    'Illuminate\\Contracts\\Queue\\ShouldQueueAfterCommit' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Queue/ShouldQueueAfterCommit.php',
    'Illuminate\\Contracts\\Redis\\Connection' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Redis/Connection.php',
    'Illuminate\\Contracts\\Redis\\Connector' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Redis/Connector.php',
    'Illuminate\\Contracts\\Redis\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Redis/Factory.php',
    'Illuminate\\Contracts\\Redis\\LimiterTimeoutException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Redis/LimiterTimeoutException.php',
    'Illuminate\\Contracts\\Routing\\BindingRegistrar' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Routing/BindingRegistrar.php',
    'Illuminate\\Contracts\\Routing\\Registrar' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Routing/Registrar.php',
    'Illuminate\\Contracts\\Routing\\ResponseFactory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Routing/ResponseFactory.php',
    'Illuminate\\Contracts\\Routing\\UrlGenerator' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Routing/UrlGenerator.php',
    'Illuminate\\Contracts\\Routing\\UrlRoutable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Routing/UrlRoutable.php',
    'Illuminate\\Contracts\\Session\\Middleware\\AuthenticatesSessions' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Session/Middleware/AuthenticatesSessions.php',
    'Illuminate\\Contracts\\Session\\Session' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Session/Session.php',
    'Illuminate\\Contracts\\Support\\Arrayable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/Arrayable.php',
    'Illuminate\\Contracts\\Support\\CanBeEscapedWhenCastToString' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/CanBeEscapedWhenCastToString.php',
    'Illuminate\\Contracts\\Support\\DeferrableProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/DeferrableProvider.php',
    'Illuminate\\Contracts\\Support\\DeferringDisplayableValue' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/DeferringDisplayableValue.php',
    'Illuminate\\Contracts\\Support\\Htmlable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/Htmlable.php',
    'Illuminate\\Contracts\\Support\\Jsonable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/Jsonable.php',
    'Illuminate\\Contracts\\Support\\MessageBag' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/MessageBag.php',
    'Illuminate\\Contracts\\Support\\MessageProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/MessageProvider.php',
    'Illuminate\\Contracts\\Support\\Renderable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/Renderable.php',
    'Illuminate\\Contracts\\Support\\Responsable' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/Responsable.php',
    'Illuminate\\Contracts\\Support\\ValidatedData' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Support/ValidatedData.php',
    'Illuminate\\Contracts\\Translation\\HasLocalePreference' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Translation/HasLocalePreference.php',
    'Illuminate\\Contracts\\Translation\\Loader' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Translation/Loader.php',
    'Illuminate\\Contracts\\Translation\\Translator' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Translation/Translator.php',
    'Illuminate\\Contracts\\Validation\\DataAwareRule' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/DataAwareRule.php',
    'Illuminate\\Contracts\\Validation\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/Factory.php',
    'Illuminate\\Contracts\\Validation\\ImplicitRule' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/ImplicitRule.php',
    'Illuminate\\Contracts\\Validation\\InvokableRule' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/InvokableRule.php',
    'Illuminate\\Contracts\\Validation\\Rule' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/Rule.php',
    'Illuminate\\Contracts\\Validation\\UncompromisedVerifier' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/UncompromisedVerifier.php',
    'Illuminate\\Contracts\\Validation\\ValidatesWhenResolved' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/ValidatesWhenResolved.php',
    'Illuminate\\Contracts\\Validation\\ValidationRule' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/ValidationRule.php',
    'Illuminate\\Contracts\\Validation\\Validator' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/Validator.php',
    'Illuminate\\Contracts\\Validation\\ValidatorAwareRule' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/Validation/ValidatorAwareRule.php',
    'Illuminate\\Contracts\\View\\Engine' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/View/Engine.php',
    'Illuminate\\Contracts\\View\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/View/Factory.php',
    'Illuminate\\Contracts\\View\\View' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/View/View.php',
    'Illuminate\\Contracts\\View\\ViewCompilationException' => $vendorDir . '/laravel/framework/src/Illuminate/Contracts/View/ViewCompilationException.php',
    'Illuminate\\Cookie\\CookieJar' => $vendorDir . '/laravel/framework/src/Illuminate/Cookie/CookieJar.php',
    'Illuminate\\Cookie\\CookieServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Cookie/CookieServiceProvider.php',
    'Illuminate\\Cookie\\CookieValuePrefix' => $vendorDir . '/laravel/framework/src/Illuminate/Cookie/CookieValuePrefix.php',
    'Illuminate\\Cookie\\Middleware\\AddQueuedCookiesToResponse' => $vendorDir . '/laravel/framework/src/Illuminate/Cookie/Middleware/AddQueuedCookiesToResponse.php',
    'Illuminate\\Cookie\\Middleware\\EncryptCookies' => $vendorDir . '/laravel/framework/src/Illuminate/Cookie/Middleware/EncryptCookies.php',
    'Illuminate\\Database\\Capsule\\Manager' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Capsule/Manager.php',
    'Illuminate\\Database\\ClassMorphViolationException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/ClassMorphViolationException.php',
    'Illuminate\\Database\\Concerns\\BuildsQueries' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Concerns/BuildsQueries.php',
    'Illuminate\\Database\\Concerns\\CompilesJsonPaths' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Concerns/CompilesJsonPaths.php',
    'Illuminate\\Database\\Concerns\\ExplainsQueries' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Concerns/ExplainsQueries.php',
    'Illuminate\\Database\\Concerns\\ManagesTransactions' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Concerns/ManagesTransactions.php',
    'Illuminate\\Database\\Concerns\\ParsesSearchPath' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Concerns/ParsesSearchPath.php',
    'Illuminate\\Database\\ConfigurationUrlParser' => $vendorDir . '/laravel/framework/src/Illuminate/Database/ConfigurationUrlParser.php',
    'Illuminate\\Database\\Connection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Connection.php',
    'Illuminate\\Database\\ConnectionInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Database/ConnectionInterface.php',
    'Illuminate\\Database\\ConnectionResolver' => $vendorDir . '/laravel/framework/src/Illuminate/Database/ConnectionResolver.php',
    'Illuminate\\Database\\ConnectionResolverInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Database/ConnectionResolverInterface.php',
    'Illuminate\\Database\\Connectors\\ConnectionFactory' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Connectors/ConnectionFactory.php',
    'Illuminate\\Database\\Connectors\\Connector' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Connectors/Connector.php',
    'Illuminate\\Database\\Connectors\\ConnectorInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Connectors/ConnectorInterface.php',
    'Illuminate\\Database\\Connectors\\MySqlConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Connectors/MySqlConnector.php',
    'Illuminate\\Database\\Connectors\\PostgresConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Connectors/PostgresConnector.php',
    'Illuminate\\Database\\Connectors\\SQLiteConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Connectors/SQLiteConnector.php',
    'Illuminate\\Database\\Connectors\\SqlServerConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Connectors/SqlServerConnector.php',
    'Illuminate\\Database\\Console\\DatabaseInspectionCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/DatabaseInspectionCommand.php',
    'Illuminate\\Database\\Console\\DbCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/DbCommand.php',
    'Illuminate\\Database\\Console\\DumpCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/DumpCommand.php',
    'Illuminate\\Database\\Console\\Factories\\FactoryMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Factories/FactoryMakeCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\BaseCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/BaseCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\FreshCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/FreshCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\InstallCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/InstallCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\MigrateCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/MigrateCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\MigrateMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/MigrateMakeCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\RefreshCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/RefreshCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\ResetCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/ResetCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\RollbackCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/RollbackCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\StatusCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/StatusCommand.php',
    'Illuminate\\Database\\Console\\Migrations\\TableGuesser' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Migrations/TableGuesser.php',
    'Illuminate\\Database\\Console\\MonitorCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/MonitorCommand.php',
    'Illuminate\\Database\\Console\\PruneCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/PruneCommand.php',
    'Illuminate\\Database\\Console\\Seeds\\SeedCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Seeds/SeedCommand.php',
    'Illuminate\\Database\\Console\\Seeds\\SeederMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Seeds/SeederMakeCommand.php',
    'Illuminate\\Database\\Console\\Seeds\\WithoutModelEvents' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/Seeds/WithoutModelEvents.php',
    'Illuminate\\Database\\Console\\ShowCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/ShowCommand.php',
    'Illuminate\\Database\\Console\\ShowModelCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/ShowModelCommand.php',
    'Illuminate\\Database\\Console\\TableCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/TableCommand.php',
    'Illuminate\\Database\\Console\\WipeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Console/WipeCommand.php',
    'Illuminate\\Database\\DBAL\\TimestampType' => $vendorDir . '/laravel/framework/src/Illuminate/Database/DBAL/TimestampType.php',
    'Illuminate\\Database\\DatabaseManager' => $vendorDir . '/laravel/framework/src/Illuminate/Database/DatabaseManager.php',
    'Illuminate\\Database\\DatabaseServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Database/DatabaseServiceProvider.php',
    'Illuminate\\Database\\DatabaseTransactionRecord' => $vendorDir . '/laravel/framework/src/Illuminate/Database/DatabaseTransactionRecord.php',
    'Illuminate\\Database\\DatabaseTransactionsManager' => $vendorDir . '/laravel/framework/src/Illuminate/Database/DatabaseTransactionsManager.php',
    'Illuminate\\Database\\DeadlockException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/DeadlockException.php',
    'Illuminate\\Database\\DetectsConcurrencyErrors' => $vendorDir . '/laravel/framework/src/Illuminate/Database/DetectsConcurrencyErrors.php',
    'Illuminate\\Database\\DetectsLostConnections' => $vendorDir . '/laravel/framework/src/Illuminate/Database/DetectsLostConnections.php',
    'Illuminate\\Database\\Eloquent\\Attributes\\ObservedBy' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Attributes/ObservedBy.php',
    'Illuminate\\Database\\Eloquent\\Attributes\\ScopedBy' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Attributes/ScopedBy.php',
    'Illuminate\\Database\\Eloquent\\BroadcastableModelEventOccurred' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/BroadcastableModelEventOccurred.php',
    'Illuminate\\Database\\Eloquent\\BroadcastsEvents' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/BroadcastsEvents.php',
    'Illuminate\\Database\\Eloquent\\BroadcastsEventsAfterCommit' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/BroadcastsEventsAfterCommit.php',
    'Illuminate\\Database\\Eloquent\\Builder' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Builder.php',
    'Illuminate\\Database\\Eloquent\\Casts\\ArrayObject' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/ArrayObject.php',
    'Illuminate\\Database\\Eloquent\\Casts\\AsArrayObject' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/AsArrayObject.php',
    'Illuminate\\Database\\Eloquent\\Casts\\AsCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/AsCollection.php',
    'Illuminate\\Database\\Eloquent\\Casts\\AsEncryptedArrayObject' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/AsEncryptedArrayObject.php',
    'Illuminate\\Database\\Eloquent\\Casts\\AsEncryptedCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/AsEncryptedCollection.php',
    'Illuminate\\Database\\Eloquent\\Casts\\AsEnumArrayObject' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/AsEnumArrayObject.php',
    'Illuminate\\Database\\Eloquent\\Casts\\AsEnumCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/AsEnumCollection.php',
    'Illuminate\\Database\\Eloquent\\Casts\\AsStringable' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/AsStringable.php',
    'Illuminate\\Database\\Eloquent\\Casts\\Attribute' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/Attribute.php',
    'Illuminate\\Database\\Eloquent\\Casts\\Json' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Casts/Json.php',
    'Illuminate\\Database\\Eloquent\\Collection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Collection.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\GuardsAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/GuardsAttributes.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\HasAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/HasAttributes.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\HasEvents' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/HasEvents.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\HasGlobalScopes' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/HasGlobalScopes.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\HasRelationships' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/HasRelationships.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\HasTimestamps' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/HasTimestamps.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\HasUlids' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/HasUlids.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\HasUniqueIds' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/HasUniqueIds.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\HasUuids' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/HasUuids.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\HidesAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/HidesAttributes.php',
    'Illuminate\\Database\\Eloquent\\Concerns\\QueriesRelationships' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Concerns/QueriesRelationships.php',
    'Illuminate\\Database\\Eloquent\\Factories\\BelongsToManyRelationship' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Factories/BelongsToManyRelationship.php',
    'Illuminate\\Database\\Eloquent\\Factories\\BelongsToRelationship' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Factories/BelongsToRelationship.php',
    'Illuminate\\Database\\Eloquent\\Factories\\CrossJoinSequence' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Factories/CrossJoinSequence.php',
    'Illuminate\\Database\\Eloquent\\Factories\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Factories/Factory.php',
    'Illuminate\\Database\\Eloquent\\Factories\\HasFactory' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Factories/HasFactory.php',
    'Illuminate\\Database\\Eloquent\\Factories\\Relationship' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Factories/Relationship.php',
    'Illuminate\\Database\\Eloquent\\Factories\\Sequence' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Factories/Sequence.php',
    'Illuminate\\Database\\Eloquent\\HigherOrderBuilderProxy' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/HigherOrderBuilderProxy.php',
    'Illuminate\\Database\\Eloquent\\InvalidCastException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/InvalidCastException.php',
    'Illuminate\\Database\\Eloquent\\JsonEncodingException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/JsonEncodingException.php',
    'Illuminate\\Database\\Eloquent\\MassAssignmentException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/MassAssignmentException.php',
    'Illuminate\\Database\\Eloquent\\MassPrunable' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/MassPrunable.php',
    'Illuminate\\Database\\Eloquent\\MissingAttributeException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/MissingAttributeException.php',
    'Illuminate\\Database\\Eloquent\\Model' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Model.php',
    'Illuminate\\Database\\Eloquent\\ModelNotFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/ModelNotFoundException.php',
    'Illuminate\\Database\\Eloquent\\PendingHasThroughRelationship' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/PendingHasThroughRelationship.php',
    'Illuminate\\Database\\Eloquent\\Prunable' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Prunable.php',
    'Illuminate\\Database\\Eloquent\\QueueEntityResolver' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/QueueEntityResolver.php',
    'Illuminate\\Database\\Eloquent\\RelationNotFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/RelationNotFoundException.php',
    'Illuminate\\Database\\Eloquent\\Relations\\BelongsTo' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/BelongsTo.php',
    'Illuminate\\Database\\Eloquent\\Relations\\BelongsToMany' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/BelongsToMany.php',
    'Illuminate\\Database\\Eloquent\\Relations\\Concerns\\AsPivot' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/Concerns/AsPivot.php',
    'Illuminate\\Database\\Eloquent\\Relations\\Concerns\\CanBeOneOfMany' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/Concerns/CanBeOneOfMany.php',
    'Illuminate\\Database\\Eloquent\\Relations\\Concerns\\ComparesRelatedModels' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/Concerns/ComparesRelatedModels.php',
    'Illuminate\\Database\\Eloquent\\Relations\\Concerns\\InteractsWithDictionary' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/Concerns/InteractsWithDictionary.php',
    'Illuminate\\Database\\Eloquent\\Relations\\Concerns\\InteractsWithPivotTable' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/Concerns/InteractsWithPivotTable.php',
    'Illuminate\\Database\\Eloquent\\Relations\\Concerns\\SupportsDefaultModels' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/Concerns/SupportsDefaultModels.php',
    'Illuminate\\Database\\Eloquent\\Relations\\HasMany' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/HasMany.php',
    'Illuminate\\Database\\Eloquent\\Relations\\HasManyThrough' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/HasManyThrough.php',
    'Illuminate\\Database\\Eloquent\\Relations\\HasOne' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/HasOne.php',
    'Illuminate\\Database\\Eloquent\\Relations\\HasOneOrMany' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/HasOneOrMany.php',
    'Illuminate\\Database\\Eloquent\\Relations\\HasOneThrough' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/HasOneThrough.php',
    'Illuminate\\Database\\Eloquent\\Relations\\MorphMany' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/MorphMany.php',
    'Illuminate\\Database\\Eloquent\\Relations\\MorphOne' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/MorphOne.php',
    'Illuminate\\Database\\Eloquent\\Relations\\MorphOneOrMany' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/MorphOneOrMany.php',
    'Illuminate\\Database\\Eloquent\\Relations\\MorphPivot' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/MorphPivot.php',
    'Illuminate\\Database\\Eloquent\\Relations\\MorphTo' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/MorphTo.php',
    'Illuminate\\Database\\Eloquent\\Relations\\MorphToMany' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/MorphToMany.php',
    'Illuminate\\Database\\Eloquent\\Relations\\Pivot' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/Pivot.php',
    'Illuminate\\Database\\Eloquent\\Relations\\Relation' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Relations/Relation.php',
    'Illuminate\\Database\\Eloquent\\Scope' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/Scope.php',
    'Illuminate\\Database\\Eloquent\\SoftDeletes' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/SoftDeletes.php',
    'Illuminate\\Database\\Eloquent\\SoftDeletingScope' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Eloquent/SoftDeletingScope.php',
    'Illuminate\\Database\\Events\\ConnectionEstablished' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/ConnectionEstablished.php',
    'Illuminate\\Database\\Events\\ConnectionEvent' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/ConnectionEvent.php',
    'Illuminate\\Database\\Events\\DatabaseBusy' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/DatabaseBusy.php',
    'Illuminate\\Database\\Events\\DatabaseRefreshed' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/DatabaseRefreshed.php',
    'Illuminate\\Database\\Events\\MigrationEnded' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/MigrationEnded.php',
    'Illuminate\\Database\\Events\\MigrationEvent' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/MigrationEvent.php',
    'Illuminate\\Database\\Events\\MigrationStarted' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/MigrationStarted.php',
    'Illuminate\\Database\\Events\\MigrationsEnded' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/MigrationsEnded.php',
    'Illuminate\\Database\\Events\\MigrationsEvent' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/MigrationsEvent.php',
    'Illuminate\\Database\\Events\\MigrationsStarted' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/MigrationsStarted.php',
    'Illuminate\\Database\\Events\\ModelPruningFinished' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/ModelPruningFinished.php',
    'Illuminate\\Database\\Events\\ModelPruningStarting' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/ModelPruningStarting.php',
    'Illuminate\\Database\\Events\\ModelsPruned' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/ModelsPruned.php',
    'Illuminate\\Database\\Events\\NoPendingMigrations' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/NoPendingMigrations.php',
    'Illuminate\\Database\\Events\\QueryExecuted' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/QueryExecuted.php',
    'Illuminate\\Database\\Events\\SchemaDumped' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/SchemaDumped.php',
    'Illuminate\\Database\\Events\\SchemaLoaded' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/SchemaLoaded.php',
    'Illuminate\\Database\\Events\\StatementPrepared' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/StatementPrepared.php',
    'Illuminate\\Database\\Events\\TransactionBeginning' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/TransactionBeginning.php',
    'Illuminate\\Database\\Events\\TransactionCommitted' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/TransactionCommitted.php',
    'Illuminate\\Database\\Events\\TransactionCommitting' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/TransactionCommitting.php',
    'Illuminate\\Database\\Events\\TransactionRolledBack' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Events/TransactionRolledBack.php',
    'Illuminate\\Database\\Grammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Grammar.php',
    'Illuminate\\Database\\LazyLoadingViolationException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/LazyLoadingViolationException.php',
    'Illuminate\\Database\\LostConnectionException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/LostConnectionException.php',
    'Illuminate\\Database\\MigrationServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Database/MigrationServiceProvider.php',
    'Illuminate\\Database\\Migrations\\DatabaseMigrationRepository' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Migrations/DatabaseMigrationRepository.php',
    'Illuminate\\Database\\Migrations\\Migration' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Migrations/Migration.php',
    'Illuminate\\Database\\Migrations\\MigrationCreator' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Migrations/MigrationCreator.php',
    'Illuminate\\Database\\Migrations\\MigrationRepositoryInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Migrations/MigrationRepositoryInterface.php',
    'Illuminate\\Database\\Migrations\\Migrator' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Migrations/Migrator.php',
    'Illuminate\\Database\\MultipleColumnsSelectedException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/MultipleColumnsSelectedException.php',
    'Illuminate\\Database\\MultipleRecordsFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/MultipleRecordsFoundException.php',
    'Illuminate\\Database\\MySqlConnection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/MySqlConnection.php',
    'Illuminate\\Database\\PDO\\Concerns\\ConnectsToDatabase' => $vendorDir . '/laravel/framework/src/Illuminate/Database/PDO/Concerns/ConnectsToDatabase.php',
    'Illuminate\\Database\\PDO\\Connection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/PDO/Connection.php',
    'Illuminate\\Database\\PDO\\MySqlDriver' => $vendorDir . '/laravel/framework/src/Illuminate/Database/PDO/MySqlDriver.php',
    'Illuminate\\Database\\PDO\\PostgresDriver' => $vendorDir . '/laravel/framework/src/Illuminate/Database/PDO/PostgresDriver.php',
    'Illuminate\\Database\\PDO\\SQLiteDriver' => $vendorDir . '/laravel/framework/src/Illuminate/Database/PDO/SQLiteDriver.php',
    'Illuminate\\Database\\PDO\\SqlServerConnection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/PDO/SqlServerConnection.php',
    'Illuminate\\Database\\PDO\\SqlServerDriver' => $vendorDir . '/laravel/framework/src/Illuminate/Database/PDO/SqlServerDriver.php',
    'Illuminate\\Database\\PostgresConnection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/PostgresConnection.php',
    'Illuminate\\Database\\QueryException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/QueryException.php',
    'Illuminate\\Database\\Query\\Builder' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Builder.php',
    'Illuminate\\Database\\Query\\Expression' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Expression.php',
    'Illuminate\\Database\\Query\\Grammars\\Grammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Grammars/Grammar.php',
    'Illuminate\\Database\\Query\\Grammars\\MySqlGrammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Grammars/MySqlGrammar.php',
    'Illuminate\\Database\\Query\\Grammars\\PostgresGrammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Grammars/PostgresGrammar.php',
    'Illuminate\\Database\\Query\\Grammars\\SQLiteGrammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Grammars/SQLiteGrammar.php',
    'Illuminate\\Database\\Query\\Grammars\\SqlServerGrammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Grammars/SqlServerGrammar.php',
    'Illuminate\\Database\\Query\\IndexHint' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/IndexHint.php',
    'Illuminate\\Database\\Query\\JoinClause' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/JoinClause.php',
    'Illuminate\\Database\\Query\\JoinLateralClause' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/JoinLateralClause.php',
    'Illuminate\\Database\\Query\\Processors\\MySqlProcessor' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Processors/MySqlProcessor.php',
    'Illuminate\\Database\\Query\\Processors\\PostgresProcessor' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Processors/PostgresProcessor.php',
    'Illuminate\\Database\\Query\\Processors\\Processor' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Processors/Processor.php',
    'Illuminate\\Database\\Query\\Processors\\SQLiteProcessor' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Processors/SQLiteProcessor.php',
    'Illuminate\\Database\\Query\\Processors\\SqlServerProcessor' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Query/Processors/SqlServerProcessor.php',
    'Illuminate\\Database\\RecordsNotFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/RecordsNotFoundException.php',
    'Illuminate\\Database\\SQLiteConnection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/SQLiteConnection.php',
    'Illuminate\\Database\\SQLiteDatabaseDoesNotExistException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/SQLiteDatabaseDoesNotExistException.php',
    'Illuminate\\Database\\Schema\\Blueprint' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/Blueprint.php',
    'Illuminate\\Database\\Schema\\Builder' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/Builder.php',
    'Illuminate\\Database\\Schema\\ColumnDefinition' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/ColumnDefinition.php',
    'Illuminate\\Database\\Schema\\ForeignIdColumnDefinition' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/ForeignIdColumnDefinition.php',
    'Illuminate\\Database\\Schema\\ForeignKeyDefinition' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/ForeignKeyDefinition.php',
    'Illuminate\\Database\\Schema\\Grammars\\ChangeColumn' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/Grammars/ChangeColumn.php',
    'Illuminate\\Database\\Schema\\Grammars\\Grammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/Grammars/Grammar.php',
    'Illuminate\\Database\\Schema\\Grammars\\MySqlGrammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/Grammars/MySqlGrammar.php',
    'Illuminate\\Database\\Schema\\Grammars\\PostgresGrammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/Grammars/PostgresGrammar.php',
    'Illuminate\\Database\\Schema\\Grammars\\RenameColumn' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/Grammars/RenameColumn.php',
    'Illuminate\\Database\\Schema\\Grammars\\SQLiteGrammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/Grammars/SQLiteGrammar.php',
    'Illuminate\\Database\\Schema\\Grammars\\SqlServerGrammar' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/Grammars/SqlServerGrammar.php',
    'Illuminate\\Database\\Schema\\IndexDefinition' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/IndexDefinition.php',
    'Illuminate\\Database\\Schema\\MySqlBuilder' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/MySqlBuilder.php',
    'Illuminate\\Database\\Schema\\MySqlSchemaState' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/MySqlSchemaState.php',
    'Illuminate\\Database\\Schema\\PostgresBuilder' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/PostgresBuilder.php',
    'Illuminate\\Database\\Schema\\PostgresSchemaState' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/PostgresSchemaState.php',
    'Illuminate\\Database\\Schema\\SQLiteBuilder' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/SQLiteBuilder.php',
    'Illuminate\\Database\\Schema\\SchemaState' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/SchemaState.php',
    'Illuminate\\Database\\Schema\\SqlServerBuilder' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/SqlServerBuilder.php',
    'Illuminate\\Database\\Schema\\SqliteSchemaState' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Schema/SqliteSchemaState.php',
    'Illuminate\\Database\\Seeder' => $vendorDir . '/laravel/framework/src/Illuminate/Database/Seeder.php',
    'Illuminate\\Database\\SqlServerConnection' => $vendorDir . '/laravel/framework/src/Illuminate/Database/SqlServerConnection.php',
    'Illuminate\\Database\\UniqueConstraintViolationException' => $vendorDir . '/laravel/framework/src/Illuminate/Database/UniqueConstraintViolationException.php',
    'Illuminate\\Encryption\\Encrypter' => $vendorDir . '/laravel/framework/src/Illuminate/Encryption/Encrypter.php',
    'Illuminate\\Encryption\\EncryptionServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Encryption/EncryptionServiceProvider.php',
    'Illuminate\\Encryption\\MissingAppKeyException' => $vendorDir . '/laravel/framework/src/Illuminate/Encryption/MissingAppKeyException.php',
    'Illuminate\\Events\\CallQueuedListener' => $vendorDir . '/laravel/framework/src/Illuminate/Events/CallQueuedListener.php',
    'Illuminate\\Events\\Dispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Events/Dispatcher.php',
    'Illuminate\\Events\\EventServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Events/EventServiceProvider.php',
    'Illuminate\\Events\\InvokeQueuedClosure' => $vendorDir . '/laravel/framework/src/Illuminate/Events/InvokeQueuedClosure.php',
    'Illuminate\\Events\\NullDispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Events/NullDispatcher.php',
    'Illuminate\\Events\\QueuedClosure' => $vendorDir . '/laravel/framework/src/Illuminate/Events/QueuedClosure.php',
    'Illuminate\\Filesystem\\AwsS3V3Adapter' => $vendorDir . '/laravel/framework/src/Illuminate/Filesystem/AwsS3V3Adapter.php',
    'Illuminate\\Filesystem\\Filesystem' => $vendorDir . '/laravel/framework/src/Illuminate/Filesystem/Filesystem.php',
    'Illuminate\\Filesystem\\FilesystemAdapter' => $vendorDir . '/laravel/framework/src/Illuminate/Filesystem/FilesystemAdapter.php',
    'Illuminate\\Filesystem\\FilesystemManager' => $vendorDir . '/laravel/framework/src/Illuminate/Filesystem/FilesystemManager.php',
    'Illuminate\\Filesystem\\FilesystemServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Filesystem/FilesystemServiceProvider.php',
    'Illuminate\\Filesystem\\LockableFile' => $vendorDir . '/laravel/framework/src/Illuminate/Filesystem/LockableFile.php',
    'Illuminate\\Foundation\\AliasLoader' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/AliasLoader.php',
    'Illuminate\\Foundation\\Application' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Application.php',
    'Illuminate\\Foundation\\Auth\\Access\\Authorizable' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Auth/Access/Authorizable.php',
    'Illuminate\\Foundation\\Auth\\Access\\AuthorizesRequests' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Auth/Access/AuthorizesRequests.php',
    'Illuminate\\Foundation\\Auth\\AuthenticatesUsers' => $vendorDir . '/laravel/ui/auth-backend/AuthenticatesUsers.php',
    'Illuminate\\Foundation\\Auth\\ConfirmsPasswords' => $vendorDir . '/laravel/ui/auth-backend/ConfirmsPasswords.php',
    'Illuminate\\Foundation\\Auth\\EmailVerificationRequest' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Auth/EmailVerificationRequest.php',
    'Illuminate\\Foundation\\Auth\\RedirectsUsers' => $vendorDir . '/laravel/ui/auth-backend/RedirectsUsers.php',
    'Illuminate\\Foundation\\Auth\\RegistersUsers' => $vendorDir . '/laravel/ui/auth-backend/RegistersUsers.php',
    'Illuminate\\Foundation\\Auth\\ResetsPasswords' => $vendorDir . '/laravel/ui/auth-backend/ResetsPasswords.php',
    'Illuminate\\Foundation\\Auth\\SendsPasswordResetEmails' => $vendorDir . '/laravel/ui/auth-backend/SendsPasswordResetEmails.php',
    'Illuminate\\Foundation\\Auth\\ThrottlesLogins' => $vendorDir . '/laravel/ui/auth-backend/ThrottlesLogins.php',
    'Illuminate\\Foundation\\Auth\\User' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Auth/User.php',
    'Illuminate\\Foundation\\Auth\\VerifiesEmails' => $vendorDir . '/laravel/ui/auth-backend/VerifiesEmails.php',
    'Illuminate\\Foundation\\Bootstrap\\BootProviders' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bootstrap/BootProviders.php',
    'Illuminate\\Foundation\\Bootstrap\\HandleExceptions' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bootstrap/HandleExceptions.php',
    'Illuminate\\Foundation\\Bootstrap\\LoadConfiguration' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bootstrap/LoadConfiguration.php',
    'Illuminate\\Foundation\\Bootstrap\\LoadEnvironmentVariables' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bootstrap/LoadEnvironmentVariables.php',
    'Illuminate\\Foundation\\Bootstrap\\RegisterFacades' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bootstrap/RegisterFacades.php',
    'Illuminate\\Foundation\\Bootstrap\\RegisterProviders' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bootstrap/RegisterProviders.php',
    'Illuminate\\Foundation\\Bootstrap\\SetRequestForConsole' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bootstrap/SetRequestForConsole.php',
    'Illuminate\\Foundation\\Bus\\Dispatchable' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bus/Dispatchable.php',
    'Illuminate\\Foundation\\Bus\\DispatchesJobs' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bus/DispatchesJobs.php',
    'Illuminate\\Foundation\\Bus\\PendingChain' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bus/PendingChain.php',
    'Illuminate\\Foundation\\Bus\\PendingClosureDispatch' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bus/PendingClosureDispatch.php',
    'Illuminate\\Foundation\\Bus\\PendingDispatch' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Bus/PendingDispatch.php',
    'Illuminate\\Foundation\\CacheBasedMaintenanceMode' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/CacheBasedMaintenanceMode.php',
    'Illuminate\\Foundation\\ComposerScripts' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/ComposerScripts.php',
    'Illuminate\\Foundation\\Concerns\\ResolvesDumpSource' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Concerns/ResolvesDumpSource.php',
    'Illuminate\\Foundation\\Console\\AboutCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/AboutCommand.php',
    'Illuminate\\Foundation\\Console\\CastMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/CastMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ChannelListCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ChannelListCommand.php',
    'Illuminate\\Foundation\\Console\\ChannelMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ChannelMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ClearCompiledCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ClearCompiledCommand.php',
    'Illuminate\\Foundation\\Console\\CliDumper' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/CliDumper.php',
    'Illuminate\\Foundation\\Console\\ClosureCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ClosureCommand.php',
    'Illuminate\\Foundation\\Console\\ComponentMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ComponentMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ConfigCacheCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ConfigCacheCommand.php',
    'Illuminate\\Foundation\\Console\\ConfigClearCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ConfigClearCommand.php',
    'Illuminate\\Foundation\\Console\\ConfigShowCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ConfigShowCommand.php',
    'Illuminate\\Foundation\\Console\\ConsoleMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ConsoleMakeCommand.php',
    'Illuminate\\Foundation\\Console\\DocsCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/DocsCommand.php',
    'Illuminate\\Foundation\\Console\\DownCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/DownCommand.php',
    'Illuminate\\Foundation\\Console\\EnvironmentCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/EnvironmentCommand.php',
    'Illuminate\\Foundation\\Console\\EnvironmentDecryptCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/EnvironmentDecryptCommand.php',
    'Illuminate\\Foundation\\Console\\EnvironmentEncryptCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/EnvironmentEncryptCommand.php',
    'Illuminate\\Foundation\\Console\\EventCacheCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/EventCacheCommand.php',
    'Illuminate\\Foundation\\Console\\EventClearCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/EventClearCommand.php',
    'Illuminate\\Foundation\\Console\\EventGenerateCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/EventGenerateCommand.php',
    'Illuminate\\Foundation\\Console\\EventListCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/EventListCommand.php',
    'Illuminate\\Foundation\\Console\\EventMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/EventMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ExceptionMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ExceptionMakeCommand.php',
    'Illuminate\\Foundation\\Console\\JobMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/JobMakeCommand.php',
    'Illuminate\\Foundation\\Console\\Kernel' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/Kernel.php',
    'Illuminate\\Foundation\\Console\\KeyGenerateCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/KeyGenerateCommand.php',
    'Illuminate\\Foundation\\Console\\LangPublishCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/LangPublishCommand.php',
    'Illuminate\\Foundation\\Console\\ListenerMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ListenerMakeCommand.php',
    'Illuminate\\Foundation\\Console\\MailMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/MailMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ModelMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ModelMakeCommand.php',
    'Illuminate\\Foundation\\Console\\NotificationMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/NotificationMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ObserverMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ObserverMakeCommand.php',
    'Illuminate\\Foundation\\Console\\OptimizeClearCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/OptimizeClearCommand.php',
    'Illuminate\\Foundation\\Console\\OptimizeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/OptimizeCommand.php',
    'Illuminate\\Foundation\\Console\\PackageDiscoverCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/PackageDiscoverCommand.php',
    'Illuminate\\Foundation\\Console\\PolicyMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/PolicyMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ProviderMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ProviderMakeCommand.php',
    'Illuminate\\Foundation\\Console\\QueuedCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/QueuedCommand.php',
    'Illuminate\\Foundation\\Console\\RequestMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/RequestMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ResourceMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ResourceMakeCommand.php',
    'Illuminate\\Foundation\\Console\\RouteCacheCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/RouteCacheCommand.php',
    'Illuminate\\Foundation\\Console\\RouteClearCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/RouteClearCommand.php',
    'Illuminate\\Foundation\\Console\\RouteListCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/RouteListCommand.php',
    'Illuminate\\Foundation\\Console\\RuleMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/RuleMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ScopeMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ScopeMakeCommand.php',
    'Illuminate\\Foundation\\Console\\ServeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ServeCommand.php',
    'Illuminate\\Foundation\\Console\\StorageLinkCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/StorageLinkCommand.php',
    'Illuminate\\Foundation\\Console\\StorageUnlinkCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/StorageUnlinkCommand.php',
    'Illuminate\\Foundation\\Console\\StubPublishCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/StubPublishCommand.php',
    'Illuminate\\Foundation\\Console\\TestMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/TestMakeCommand.php',
    'Illuminate\\Foundation\\Console\\UpCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/UpCommand.php',
    'Illuminate\\Foundation\\Console\\VendorPublishCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/VendorPublishCommand.php',
    'Illuminate\\Foundation\\Console\\ViewCacheCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ViewCacheCommand.php',
    'Illuminate\\Foundation\\Console\\ViewClearCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ViewClearCommand.php',
    'Illuminate\\Foundation\\Console\\ViewMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Console/ViewMakeCommand.php',
    'Illuminate\\Foundation\\EnvironmentDetector' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/EnvironmentDetector.php',
    'Illuminate\\Foundation\\Events\\DiscoverEvents' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Events/DiscoverEvents.php',
    'Illuminate\\Foundation\\Events\\Dispatchable' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Events/Dispatchable.php',
    'Illuminate\\Foundation\\Events\\LocaleUpdated' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Events/LocaleUpdated.php',
    'Illuminate\\Foundation\\Events\\MaintenanceModeDisabled' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Events/MaintenanceModeDisabled.php',
    'Illuminate\\Foundation\\Events\\MaintenanceModeEnabled' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Events/MaintenanceModeEnabled.php',
    'Illuminate\\Foundation\\Events\\PublishingStubs' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Events/PublishingStubs.php',
    'Illuminate\\Foundation\\Events\\VendorTagPublished' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Events/VendorTagPublished.php',
    'Illuminate\\Foundation\\Exceptions\\Handler' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Exceptions/Handler.php',
    'Illuminate\\Foundation\\Exceptions\\RegisterErrorViewPaths' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Exceptions/RegisterErrorViewPaths.php',
    'Illuminate\\Foundation\\Exceptions\\ReportableHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Exceptions/ReportableHandler.php',
    'Illuminate\\Foundation\\Exceptions\\Whoops\\WhoopsExceptionRenderer' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Exceptions/Whoops/WhoopsExceptionRenderer.php',
    'Illuminate\\Foundation\\Exceptions\\Whoops\\WhoopsHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Exceptions/Whoops/WhoopsHandler.php',
    'Illuminate\\Foundation\\FileBasedMaintenanceMode' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/FileBasedMaintenanceMode.php',
    'Illuminate\\Foundation\\Http\\Events\\RequestHandled' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Events/RequestHandled.php',
    'Illuminate\\Foundation\\Http\\FormRequest' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/FormRequest.php',
    'Illuminate\\Foundation\\Http\\HtmlDumper' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/HtmlDumper.php',
    'Illuminate\\Foundation\\Http\\Kernel' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Kernel.php',
    'Illuminate\\Foundation\\Http\\MaintenanceModeBypassCookie' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/MaintenanceModeBypassCookie.php',
    'Illuminate\\Foundation\\Http\\Middleware\\CheckForMaintenanceMode' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Middleware/CheckForMaintenanceMode.php',
    'Illuminate\\Foundation\\Http\\Middleware\\ConvertEmptyStringsToNull' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Middleware/ConvertEmptyStringsToNull.php',
    'Illuminate\\Foundation\\Http\\Middleware\\HandlePrecognitiveRequests' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Middleware/HandlePrecognitiveRequests.php',
    'Illuminate\\Foundation\\Http\\Middleware\\PreventRequestsDuringMaintenance' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Middleware/PreventRequestsDuringMaintenance.php',
    'Illuminate\\Foundation\\Http\\Middleware\\TransformsRequest' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Middleware/TransformsRequest.php',
    'Illuminate\\Foundation\\Http\\Middleware\\TrimStrings' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Middleware/TrimStrings.php',
    'Illuminate\\Foundation\\Http\\Middleware\\ValidatePostSize' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Middleware/ValidatePostSize.php',
    'Illuminate\\Foundation\\Http\\Middleware\\VerifyCsrfToken' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Http/Middleware/VerifyCsrfToken.php',
    'Illuminate\\Foundation\\Inspiring' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Inspiring.php',
    'Illuminate\\Foundation\\MaintenanceModeManager' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/MaintenanceModeManager.php',
    'Illuminate\\Foundation\\Mix' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Mix.php',
    'Illuminate\\Foundation\\PackageManifest' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/PackageManifest.php',
    'Illuminate\\Foundation\\Precognition' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Precognition.php',
    'Illuminate\\Foundation\\ProviderRepository' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/ProviderRepository.php',
    'Illuminate\\Foundation\\Providers\\ArtisanServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Providers/ArtisanServiceProvider.php',
    'Illuminate\\Foundation\\Providers\\ComposerServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Providers/ComposerServiceProvider.php',
    'Illuminate\\Foundation\\Providers\\ConsoleSupportServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Providers/ConsoleSupportServiceProvider.php',
    'Illuminate\\Foundation\\Providers\\FormRequestServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Providers/FormRequestServiceProvider.php',
    'Illuminate\\Foundation\\Providers\\FoundationServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Providers/FoundationServiceProvider.php',
    'Illuminate\\Foundation\\Routing\\PrecognitionCallableDispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Routing/PrecognitionCallableDispatcher.php',
    'Illuminate\\Foundation\\Routing\\PrecognitionControllerDispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Routing/PrecognitionControllerDispatcher.php',
    'Illuminate\\Foundation\\Support\\Providers\\AuthServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Support/Providers/AuthServiceProvider.php',
    'Illuminate\\Foundation\\Support\\Providers\\EventServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Support/Providers/EventServiceProvider.php',
    'Illuminate\\Foundation\\Support\\Providers\\RouteServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Support/Providers/RouteServiceProvider.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithAuthentication' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithAuthentication.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithConsole' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithConsole.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithContainer' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithContainer.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithDatabase' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithDatabase.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithDeprecationHandling' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithDeprecationHandling.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithExceptionHandling' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithExceptionHandling.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithRedis' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithRedis.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithSession' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithSession.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithTestCaseLifecycle' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithTestCaseLifecycle.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithTime' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithTime.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\InteractsWithViews' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/InteractsWithViews.php',
    'Illuminate\\Foundation\\Testing\\Concerns\\MakesHttpRequests' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Concerns/MakesHttpRequests.php',
    'Illuminate\\Foundation\\Testing\\DatabaseMigrations' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/DatabaseMigrations.php',
    'Illuminate\\Foundation\\Testing\\DatabaseTransactions' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/DatabaseTransactions.php',
    'Illuminate\\Foundation\\Testing\\DatabaseTransactionsManager' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/DatabaseTransactionsManager.php',
    'Illuminate\\Foundation\\Testing\\DatabaseTruncation' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/DatabaseTruncation.php',
    'Illuminate\\Foundation\\Testing\\LazilyRefreshDatabase' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/LazilyRefreshDatabase.php',
    'Illuminate\\Foundation\\Testing\\RefreshDatabase' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/RefreshDatabase.php',
    'Illuminate\\Foundation\\Testing\\RefreshDatabaseState' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/RefreshDatabaseState.php',
    'Illuminate\\Foundation\\Testing\\TestCase' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/TestCase.php',
    'Illuminate\\Foundation\\Testing\\Traits\\CanConfigureMigrationCommands' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Traits/CanConfigureMigrationCommands.php',
    'Illuminate\\Foundation\\Testing\\WithConsoleEvents' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/WithConsoleEvents.php',
    'Illuminate\\Foundation\\Testing\\WithFaker' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/WithFaker.php',
    'Illuminate\\Foundation\\Testing\\WithoutEvents' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/WithoutEvents.php',
    'Illuminate\\Foundation\\Testing\\WithoutMiddleware' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/WithoutMiddleware.php',
    'Illuminate\\Foundation\\Testing\\Wormhole' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Testing/Wormhole.php',
    'Illuminate\\Foundation\\Validation\\ValidatesRequests' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Validation/ValidatesRequests.php',
    'Illuminate\\Foundation\\Vite' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/Vite.php',
    'Illuminate\\Foundation\\ViteManifestNotFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Foundation/ViteManifestNotFoundException.php',
    'Illuminate\\Hashing\\AbstractHasher' => $vendorDir . '/laravel/framework/src/Illuminate/Hashing/AbstractHasher.php',
    'Illuminate\\Hashing\\Argon2IdHasher' => $vendorDir . '/laravel/framework/src/Illuminate/Hashing/Argon2IdHasher.php',
    'Illuminate\\Hashing\\ArgonHasher' => $vendorDir . '/laravel/framework/src/Illuminate/Hashing/ArgonHasher.php',
    'Illuminate\\Hashing\\BcryptHasher' => $vendorDir . '/laravel/framework/src/Illuminate/Hashing/BcryptHasher.php',
    'Illuminate\\Hashing\\HashManager' => $vendorDir . '/laravel/framework/src/Illuminate/Hashing/HashManager.php',
    'Illuminate\\Hashing\\HashServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Hashing/HashServiceProvider.php',
    'Illuminate\\Http\\Client\\Concerns\\DeterminesStatusCode' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/Concerns/DeterminesStatusCode.php',
    'Illuminate\\Http\\Client\\ConnectionException' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/ConnectionException.php',
    'Illuminate\\Http\\Client\\Events\\ConnectionFailed' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/Events/ConnectionFailed.php',
    'Illuminate\\Http\\Client\\Events\\RequestSending' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/Events/RequestSending.php',
    'Illuminate\\Http\\Client\\Events\\ResponseReceived' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/Events/ResponseReceived.php',
    'Illuminate\\Http\\Client\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/Factory.php',
    'Illuminate\\Http\\Client\\HttpClientException' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/HttpClientException.php',
    'Illuminate\\Http\\Client\\PendingRequest' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/PendingRequest.php',
    'Illuminate\\Http\\Client\\Pool' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/Pool.php',
    'Illuminate\\Http\\Client\\Request' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/Request.php',
    'Illuminate\\Http\\Client\\RequestException' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/RequestException.php',
    'Illuminate\\Http\\Client\\Response' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/Response.php',
    'Illuminate\\Http\\Client\\ResponseSequence' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Client/ResponseSequence.php',
    'Illuminate\\Http\\Concerns\\CanBePrecognitive' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Concerns/CanBePrecognitive.php',
    'Illuminate\\Http\\Concerns\\InteractsWithContentTypes' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Concerns/InteractsWithContentTypes.php',
    'Illuminate\\Http\\Concerns\\InteractsWithFlashData' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Concerns/InteractsWithFlashData.php',
    'Illuminate\\Http\\Concerns\\InteractsWithInput' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Concerns/InteractsWithInput.php',
    'Illuminate\\Http\\Exceptions\\HttpResponseException' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Exceptions/HttpResponseException.php',
    'Illuminate\\Http\\Exceptions\\PostTooLargeException' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Exceptions/PostTooLargeException.php',
    'Illuminate\\Http\\Exceptions\\ThrottleRequestsException' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Exceptions/ThrottleRequestsException.php',
    'Illuminate\\Http\\File' => $vendorDir . '/laravel/framework/src/Illuminate/Http/File.php',
    'Illuminate\\Http\\FileHelpers' => $vendorDir . '/laravel/framework/src/Illuminate/Http/FileHelpers.php',
    'Illuminate\\Http\\JsonResponse' => $vendorDir . '/laravel/framework/src/Illuminate/Http/JsonResponse.php',
    'Illuminate\\Http\\Middleware\\AddLinkHeadersForPreloadedAssets' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Middleware/AddLinkHeadersForPreloadedAssets.php',
    'Illuminate\\Http\\Middleware\\CheckResponseForModifications' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Middleware/CheckResponseForModifications.php',
    'Illuminate\\Http\\Middleware\\FrameGuard' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Middleware/FrameGuard.php',
    'Illuminate\\Http\\Middleware\\HandleCors' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Middleware/HandleCors.php',
    'Illuminate\\Http\\Middleware\\SetCacheHeaders' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Middleware/SetCacheHeaders.php',
    'Illuminate\\Http\\Middleware\\TrustHosts' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Middleware/TrustHosts.php',
    'Illuminate\\Http\\Middleware\\TrustProxies' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Middleware/TrustProxies.php',
    'Illuminate\\Http\\RedirectResponse' => $vendorDir . '/laravel/framework/src/Illuminate/Http/RedirectResponse.php',
    'Illuminate\\Http\\Request' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Request.php',
    'Illuminate\\Http\\Resources\\CollectsResources' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/CollectsResources.php',
    'Illuminate\\Http\\Resources\\ConditionallyLoadsAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/ConditionallyLoadsAttributes.php',
    'Illuminate\\Http\\Resources\\DelegatesToResource' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/DelegatesToResource.php',
    'Illuminate\\Http\\Resources\\Json\\AnonymousResourceCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/Json/AnonymousResourceCollection.php',
    'Illuminate\\Http\\Resources\\Json\\JsonResource' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/Json/JsonResource.php',
    'Illuminate\\Http\\Resources\\Json\\PaginatedResourceResponse' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/Json/PaginatedResourceResponse.php',
    'Illuminate\\Http\\Resources\\Json\\ResourceCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/Json/ResourceCollection.php',
    'Illuminate\\Http\\Resources\\Json\\ResourceResponse' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/Json/ResourceResponse.php',
    'Illuminate\\Http\\Resources\\MergeValue' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/MergeValue.php',
    'Illuminate\\Http\\Resources\\MissingValue' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/MissingValue.php',
    'Illuminate\\Http\\Resources\\PotentiallyMissing' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Resources/PotentiallyMissing.php',
    'Illuminate\\Http\\Response' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Response.php',
    'Illuminate\\Http\\ResponseTrait' => $vendorDir . '/laravel/framework/src/Illuminate/Http/ResponseTrait.php',
    'Illuminate\\Http\\Testing\\File' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Testing/File.php',
    'Illuminate\\Http\\Testing\\FileFactory' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Testing/FileFactory.php',
    'Illuminate\\Http\\Testing\\MimeType' => $vendorDir . '/laravel/framework/src/Illuminate/Http/Testing/MimeType.php',
    'Illuminate\\Http\\UploadedFile' => $vendorDir . '/laravel/framework/src/Illuminate/Http/UploadedFile.php',
    'Illuminate\\Log\\Events\\MessageLogged' => $vendorDir . '/laravel/framework/src/Illuminate/Log/Events/MessageLogged.php',
    'Illuminate\\Log\\LogManager' => $vendorDir . '/laravel/framework/src/Illuminate/Log/LogManager.php',
    'Illuminate\\Log\\LogServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Log/LogServiceProvider.php',
    'Illuminate\\Log\\Logger' => $vendorDir . '/laravel/framework/src/Illuminate/Log/Logger.php',
    'Illuminate\\Log\\ParsesLogConfiguration' => $vendorDir . '/laravel/framework/src/Illuminate/Log/ParsesLogConfiguration.php',
    'Illuminate\\Mail\\Attachment' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Attachment.php',
    'Illuminate\\Mail\\Events\\MessageSending' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Events/MessageSending.php',
    'Illuminate\\Mail\\Events\\MessageSent' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Events/MessageSent.php',
    'Illuminate\\Mail\\MailManager' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/MailManager.php',
    'Illuminate\\Mail\\MailServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/MailServiceProvider.php',
    'Illuminate\\Mail\\Mailable' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Mailable.php',
    'Illuminate\\Mail\\Mailables\\Address' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Mailables/Address.php',
    'Illuminate\\Mail\\Mailables\\Attachment' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Mailables/Attachment.php',
    'Illuminate\\Mail\\Mailables\\Content' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Mailables/Content.php',
    'Illuminate\\Mail\\Mailables\\Envelope' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Mailables/Envelope.php',
    'Illuminate\\Mail\\Mailables\\Headers' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Mailables/Headers.php',
    'Illuminate\\Mail\\Mailer' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Mailer.php',
    'Illuminate\\Mail\\Markdown' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Markdown.php',
    'Illuminate\\Mail\\Message' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Message.php',
    'Illuminate\\Mail\\PendingMail' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/PendingMail.php',
    'Illuminate\\Mail\\SendQueuedMailable' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/SendQueuedMailable.php',
    'Illuminate\\Mail\\SentMessage' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/SentMessage.php',
    'Illuminate\\Mail\\TextMessage' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/TextMessage.php',
    'Illuminate\\Mail\\Transport\\ArrayTransport' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Transport/ArrayTransport.php',
    'Illuminate\\Mail\\Transport\\LogTransport' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Transport/LogTransport.php',
    'Illuminate\\Mail\\Transport\\SesTransport' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Transport/SesTransport.php',
    'Illuminate\\Mail\\Transport\\SesV2Transport' => $vendorDir . '/laravel/framework/src/Illuminate/Mail/Transport/SesV2Transport.php',
    'Illuminate\\Notifications\\Action' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Action.php',
    'Illuminate\\Notifications\\AnonymousNotifiable' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/AnonymousNotifiable.php',
    'Illuminate\\Notifications\\ChannelManager' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/ChannelManager.php',
    'Illuminate\\Notifications\\Channels\\BroadcastChannel' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Channels/BroadcastChannel.php',
    'Illuminate\\Notifications\\Channels\\DatabaseChannel' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Channels/DatabaseChannel.php',
    'Illuminate\\Notifications\\Channels\\MailChannel' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Channels/MailChannel.php',
    'Illuminate\\Notifications\\Console\\NotificationTableCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Console/NotificationTableCommand.php',
    'Illuminate\\Notifications\\DatabaseNotification' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/DatabaseNotification.php',
    'Illuminate\\Notifications\\DatabaseNotificationCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/DatabaseNotificationCollection.php',
    'Illuminate\\Notifications\\Events\\BroadcastNotificationCreated' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Events/BroadcastNotificationCreated.php',
    'Illuminate\\Notifications\\Events\\NotificationFailed' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Events/NotificationFailed.php',
    'Illuminate\\Notifications\\Events\\NotificationSending' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Events/NotificationSending.php',
    'Illuminate\\Notifications\\Events\\NotificationSent' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Events/NotificationSent.php',
    'Illuminate\\Notifications\\HasDatabaseNotifications' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/HasDatabaseNotifications.php',
    'Illuminate\\Notifications\\Messages\\BroadcastMessage' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Messages/BroadcastMessage.php',
    'Illuminate\\Notifications\\Messages\\DatabaseMessage' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Messages/DatabaseMessage.php',
    'Illuminate\\Notifications\\Messages\\MailMessage' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Messages/MailMessage.php',
    'Illuminate\\Notifications\\Messages\\SimpleMessage' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Messages/SimpleMessage.php',
    'Illuminate\\Notifications\\Notifiable' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Notifiable.php',
    'Illuminate\\Notifications\\Notification' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/Notification.php',
    'Illuminate\\Notifications\\NotificationSender' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/NotificationSender.php',
    'Illuminate\\Notifications\\NotificationServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/NotificationServiceProvider.php',
    'Illuminate\\Notifications\\RoutesNotifications' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/RoutesNotifications.php',
    'Illuminate\\Notifications\\SendQueuedNotifications' => $vendorDir . '/laravel/framework/src/Illuminate/Notifications/SendQueuedNotifications.php',
    'Illuminate\\Pagination\\AbstractCursorPaginator' => $vendorDir . '/laravel/framework/src/Illuminate/Pagination/AbstractCursorPaginator.php',
    'Illuminate\\Pagination\\AbstractPaginator' => $vendorDir . '/laravel/framework/src/Illuminate/Pagination/AbstractPaginator.php',
    'Illuminate\\Pagination\\Cursor' => $vendorDir . '/laravel/framework/src/Illuminate/Pagination/Cursor.php',
    'Illuminate\\Pagination\\CursorPaginator' => $vendorDir . '/laravel/framework/src/Illuminate/Pagination/CursorPaginator.php',
    'Illuminate\\Pagination\\LengthAwarePaginator' => $vendorDir . '/laravel/framework/src/Illuminate/Pagination/LengthAwarePaginator.php',
    'Illuminate\\Pagination\\PaginationServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Pagination/PaginationServiceProvider.php',
    'Illuminate\\Pagination\\PaginationState' => $vendorDir . '/laravel/framework/src/Illuminate/Pagination/PaginationState.php',
    'Illuminate\\Pagination\\Paginator' => $vendorDir . '/laravel/framework/src/Illuminate/Pagination/Paginator.php',
    'Illuminate\\Pagination\\UrlWindow' => $vendorDir . '/laravel/framework/src/Illuminate/Pagination/UrlWindow.php',
    'Illuminate\\Pipeline\\Hub' => $vendorDir . '/laravel/framework/src/Illuminate/Pipeline/Hub.php',
    'Illuminate\\Pipeline\\Pipeline' => $vendorDir . '/laravel/framework/src/Illuminate/Pipeline/Pipeline.php',
    'Illuminate\\Pipeline\\PipelineServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Pipeline/PipelineServiceProvider.php',
    'Illuminate\\Process\\Exceptions\\ProcessFailedException' => $vendorDir . '/laravel/framework/src/Illuminate/Process/Exceptions/ProcessFailedException.php',
    'Illuminate\\Process\\Exceptions\\ProcessTimedOutException' => $vendorDir . '/laravel/framework/src/Illuminate/Process/Exceptions/ProcessTimedOutException.php',
    'Illuminate\\Process\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Process/Factory.php',
    'Illuminate\\Process\\FakeInvokedProcess' => $vendorDir . '/laravel/framework/src/Illuminate/Process/FakeInvokedProcess.php',
    'Illuminate\\Process\\FakeProcessDescription' => $vendorDir . '/laravel/framework/src/Illuminate/Process/FakeProcessDescription.php',
    'Illuminate\\Process\\FakeProcessResult' => $vendorDir . '/laravel/framework/src/Illuminate/Process/FakeProcessResult.php',
    'Illuminate\\Process\\FakeProcessSequence' => $vendorDir . '/laravel/framework/src/Illuminate/Process/FakeProcessSequence.php',
    'Illuminate\\Process\\InvokedProcess' => $vendorDir . '/laravel/framework/src/Illuminate/Process/InvokedProcess.php',
    'Illuminate\\Process\\InvokedProcessPool' => $vendorDir . '/laravel/framework/src/Illuminate/Process/InvokedProcessPool.php',
    'Illuminate\\Process\\PendingProcess' => $vendorDir . '/laravel/framework/src/Illuminate/Process/PendingProcess.php',
    'Illuminate\\Process\\Pipe' => $vendorDir . '/laravel/framework/src/Illuminate/Process/Pipe.php',
    'Illuminate\\Process\\Pool' => $vendorDir . '/laravel/framework/src/Illuminate/Process/Pool.php',
    'Illuminate\\Process\\ProcessPoolResults' => $vendorDir . '/laravel/framework/src/Illuminate/Process/ProcessPoolResults.php',
    'Illuminate\\Process\\ProcessResult' => $vendorDir . '/laravel/framework/src/Illuminate/Process/ProcessResult.php',
    'Illuminate\\Queue\\Attributes\\WithoutRelations' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Attributes/WithoutRelations.php',
    'Illuminate\\Queue\\BeanstalkdQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/BeanstalkdQueue.php',
    'Illuminate\\Queue\\CallQueuedClosure' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/CallQueuedClosure.php',
    'Illuminate\\Queue\\CallQueuedHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/CallQueuedHandler.php',
    'Illuminate\\Queue\\Capsule\\Manager' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Capsule/Manager.php',
    'Illuminate\\Queue\\Connectors\\BeanstalkdConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Connectors/BeanstalkdConnector.php',
    'Illuminate\\Queue\\Connectors\\ConnectorInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Connectors/ConnectorInterface.php',
    'Illuminate\\Queue\\Connectors\\DatabaseConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Connectors/DatabaseConnector.php',
    'Illuminate\\Queue\\Connectors\\NullConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Connectors/NullConnector.php',
    'Illuminate\\Queue\\Connectors\\RedisConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Connectors/RedisConnector.php',
    'Illuminate\\Queue\\Connectors\\SqsConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Connectors/SqsConnector.php',
    'Illuminate\\Queue\\Connectors\\SyncConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Connectors/SyncConnector.php',
    'Illuminate\\Queue\\Console\\BatchesTableCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/BatchesTableCommand.php',
    'Illuminate\\Queue\\Console\\ClearCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/ClearCommand.php',
    'Illuminate\\Queue\\Console\\FailedTableCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/FailedTableCommand.php',
    'Illuminate\\Queue\\Console\\FlushFailedCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/FlushFailedCommand.php',
    'Illuminate\\Queue\\Console\\ForgetFailedCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/ForgetFailedCommand.php',
    'Illuminate\\Queue\\Console\\ListFailedCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/ListFailedCommand.php',
    'Illuminate\\Queue\\Console\\ListenCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/ListenCommand.php',
    'Illuminate\\Queue\\Console\\MonitorCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/MonitorCommand.php',
    'Illuminate\\Queue\\Console\\PruneBatchesCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/PruneBatchesCommand.php',
    'Illuminate\\Queue\\Console\\PruneFailedJobsCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/PruneFailedJobsCommand.php',
    'Illuminate\\Queue\\Console\\RestartCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/RestartCommand.php',
    'Illuminate\\Queue\\Console\\RetryBatchCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/RetryBatchCommand.php',
    'Illuminate\\Queue\\Console\\RetryCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/RetryCommand.php',
    'Illuminate\\Queue\\Console\\TableCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/TableCommand.php',
    'Illuminate\\Queue\\Console\\WorkCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Console/WorkCommand.php',
    'Illuminate\\Queue\\DatabaseQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/DatabaseQueue.php',
    'Illuminate\\Queue\\Events\\JobExceptionOccurred' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobExceptionOccurred.php',
    'Illuminate\\Queue\\Events\\JobFailed' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobFailed.php',
    'Illuminate\\Queue\\Events\\JobPopped' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobPopped.php',
    'Illuminate\\Queue\\Events\\JobPopping' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobPopping.php',
    'Illuminate\\Queue\\Events\\JobProcessed' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobProcessed.php',
    'Illuminate\\Queue\\Events\\JobProcessing' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobProcessing.php',
    'Illuminate\\Queue\\Events\\JobQueued' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobQueued.php',
    'Illuminate\\Queue\\Events\\JobQueueing' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobQueueing.php',
    'Illuminate\\Queue\\Events\\JobReleasedAfterException' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobReleasedAfterException.php',
    'Illuminate\\Queue\\Events\\JobRetryRequested' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobRetryRequested.php',
    'Illuminate\\Queue\\Events\\JobTimedOut' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/JobTimedOut.php',
    'Illuminate\\Queue\\Events\\Looping' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/Looping.php',
    'Illuminate\\Queue\\Events\\QueueBusy' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/QueueBusy.php',
    'Illuminate\\Queue\\Events\\WorkerStopping' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Events/WorkerStopping.php',
    'Illuminate\\Queue\\Failed\\CountableFailedJobProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Failed/CountableFailedJobProvider.php',
    'Illuminate\\Queue\\Failed\\DatabaseFailedJobProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Failed/DatabaseFailedJobProvider.php',
    'Illuminate\\Queue\\Failed\\DatabaseUuidFailedJobProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Failed/DatabaseUuidFailedJobProvider.php',
    'Illuminate\\Queue\\Failed\\DynamoDbFailedJobProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Failed/DynamoDbFailedJobProvider.php',
    'Illuminate\\Queue\\Failed\\FailedJobProviderInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Failed/FailedJobProviderInterface.php',
    'Illuminate\\Queue\\Failed\\FileFailedJobProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Failed/FileFailedJobProvider.php',
    'Illuminate\\Queue\\Failed\\NullFailedJobProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Failed/NullFailedJobProvider.php',
    'Illuminate\\Queue\\Failed\\PrunableFailedJobProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Failed/PrunableFailedJobProvider.php',
    'Illuminate\\Queue\\InteractsWithQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/InteractsWithQueue.php',
    'Illuminate\\Queue\\InvalidPayloadException' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/InvalidPayloadException.php',
    'Illuminate\\Queue\\Jobs\\BeanstalkdJob' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Jobs/BeanstalkdJob.php',
    'Illuminate\\Queue\\Jobs\\DatabaseJob' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Jobs/DatabaseJob.php',
    'Illuminate\\Queue\\Jobs\\DatabaseJobRecord' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Jobs/DatabaseJobRecord.php',
    'Illuminate\\Queue\\Jobs\\Job' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Jobs/Job.php',
    'Illuminate\\Queue\\Jobs\\JobName' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Jobs/JobName.php',
    'Illuminate\\Queue\\Jobs\\RedisJob' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Jobs/RedisJob.php',
    'Illuminate\\Queue\\Jobs\\SqsJob' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Jobs/SqsJob.php',
    'Illuminate\\Queue\\Jobs\\SyncJob' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Jobs/SyncJob.php',
    'Illuminate\\Queue\\Listener' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Listener.php',
    'Illuminate\\Queue\\ListenerOptions' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/ListenerOptions.php',
    'Illuminate\\Queue\\LuaScripts' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/LuaScripts.php',
    'Illuminate\\Queue\\ManuallyFailedException' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/ManuallyFailedException.php',
    'Illuminate\\Queue\\MaxAttemptsExceededException' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/MaxAttemptsExceededException.php',
    'Illuminate\\Queue\\Middleware\\RateLimited' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Middleware/RateLimited.php',
    'Illuminate\\Queue\\Middleware\\RateLimitedWithRedis' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Middleware/RateLimitedWithRedis.php',
    'Illuminate\\Queue\\Middleware\\SkipIfBatchCancelled' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Middleware/SkipIfBatchCancelled.php',
    'Illuminate\\Queue\\Middleware\\ThrottlesExceptions' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Middleware/ThrottlesExceptions.php',
    'Illuminate\\Queue\\Middleware\\ThrottlesExceptionsWithRedis' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Middleware/ThrottlesExceptionsWithRedis.php',
    'Illuminate\\Queue\\Middleware\\WithoutOverlapping' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Middleware/WithoutOverlapping.php',
    'Illuminate\\Queue\\NullQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/NullQueue.php',
    'Illuminate\\Queue\\Queue' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Queue.php',
    'Illuminate\\Queue\\QueueManager' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/QueueManager.php',
    'Illuminate\\Queue\\QueueServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/QueueServiceProvider.php',
    'Illuminate\\Queue\\RedisQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/RedisQueue.php',
    'Illuminate\\Queue\\SerializesAndRestoresModelIdentifiers' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/SerializesAndRestoresModelIdentifiers.php',
    'Illuminate\\Queue\\SerializesModels' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/SerializesModels.php',
    'Illuminate\\Queue\\SqsQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/SqsQueue.php',
    'Illuminate\\Queue\\SyncQueue' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/SyncQueue.php',
    'Illuminate\\Queue\\TimeoutExceededException' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/TimeoutExceededException.php',
    'Illuminate\\Queue\\Worker' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/Worker.php',
    'Illuminate\\Queue\\WorkerOptions' => $vendorDir . '/laravel/framework/src/Illuminate/Queue/WorkerOptions.php',
    'Illuminate\\Redis\\Connections\\Connection' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Connections/Connection.php',
    'Illuminate\\Redis\\Connections\\PacksPhpRedisValues' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Connections/PacksPhpRedisValues.php',
    'Illuminate\\Redis\\Connections\\PhpRedisClusterConnection' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Connections/PhpRedisClusterConnection.php',
    'Illuminate\\Redis\\Connections\\PhpRedisConnection' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Connections/PhpRedisConnection.php',
    'Illuminate\\Redis\\Connections\\PredisClusterConnection' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Connections/PredisClusterConnection.php',
    'Illuminate\\Redis\\Connections\\PredisConnection' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Connections/PredisConnection.php',
    'Illuminate\\Redis\\Connectors\\PhpRedisConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Connectors/PhpRedisConnector.php',
    'Illuminate\\Redis\\Connectors\\PredisConnector' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Connectors/PredisConnector.php',
    'Illuminate\\Redis\\Events\\CommandExecuted' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Events/CommandExecuted.php',
    'Illuminate\\Redis\\Limiters\\ConcurrencyLimiter' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Limiters/ConcurrencyLimiter.php',
    'Illuminate\\Redis\\Limiters\\ConcurrencyLimiterBuilder' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Limiters/ConcurrencyLimiterBuilder.php',
    'Illuminate\\Redis\\Limiters\\DurationLimiter' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Limiters/DurationLimiter.php',
    'Illuminate\\Redis\\Limiters\\DurationLimiterBuilder' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/Limiters/DurationLimiterBuilder.php',
    'Illuminate\\Redis\\RedisManager' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/RedisManager.php',
    'Illuminate\\Redis\\RedisServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Redis/RedisServiceProvider.php',
    'Illuminate\\Routing\\AbstractRouteCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/AbstractRouteCollection.php',
    'Illuminate\\Routing\\CallableDispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/CallableDispatcher.php',
    'Illuminate\\Routing\\CompiledRouteCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/CompiledRouteCollection.php',
    'Illuminate\\Routing\\Console\\ControllerMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Console/ControllerMakeCommand.php',
    'Illuminate\\Routing\\Console\\MiddlewareMakeCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Console/MiddlewareMakeCommand.php',
    'Illuminate\\Routing\\Contracts\\CallableDispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Contracts/CallableDispatcher.php',
    'Illuminate\\Routing\\Contracts\\ControllerDispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Contracts/ControllerDispatcher.php',
    'Illuminate\\Routing\\Controller' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Controller.php',
    'Illuminate\\Routing\\ControllerDispatcher' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/ControllerDispatcher.php',
    'Illuminate\\Routing\\ControllerMiddlewareOptions' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/ControllerMiddlewareOptions.php',
    'Illuminate\\Routing\\Controllers\\HasMiddleware' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Controllers/HasMiddleware.php',
    'Illuminate\\Routing\\Controllers\\Middleware' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Controllers/Middleware.php',
    'Illuminate\\Routing\\CreatesRegularExpressionRouteConstraints' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/CreatesRegularExpressionRouteConstraints.php',
    'Illuminate\\Routing\\Events\\PreparingResponse' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Events/PreparingResponse.php',
    'Illuminate\\Routing\\Events\\ResponsePrepared' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Events/ResponsePrepared.php',
    'Illuminate\\Routing\\Events\\RouteMatched' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Events/RouteMatched.php',
    'Illuminate\\Routing\\Events\\Routing' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Events/Routing.php',
    'Illuminate\\Routing\\Exceptions\\BackedEnumCaseNotFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Exceptions/BackedEnumCaseNotFoundException.php',
    'Illuminate\\Routing\\Exceptions\\InvalidSignatureException' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Exceptions/InvalidSignatureException.php',
    'Illuminate\\Routing\\Exceptions\\StreamedResponseException' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Exceptions/StreamedResponseException.php',
    'Illuminate\\Routing\\Exceptions\\UrlGenerationException' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Exceptions/UrlGenerationException.php',
    'Illuminate\\Routing\\FiltersControllerMiddleware' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/FiltersControllerMiddleware.php',
    'Illuminate\\Routing\\ImplicitRouteBinding' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/ImplicitRouteBinding.php',
    'Illuminate\\Routing\\Matching\\HostValidator' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Matching/HostValidator.php',
    'Illuminate\\Routing\\Matching\\MethodValidator' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Matching/MethodValidator.php',
    'Illuminate\\Routing\\Matching\\SchemeValidator' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Matching/SchemeValidator.php',
    'Illuminate\\Routing\\Matching\\UriValidator' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Matching/UriValidator.php',
    'Illuminate\\Routing\\Matching\\ValidatorInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Matching/ValidatorInterface.php',
    'Illuminate\\Routing\\MiddlewareNameResolver' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/MiddlewareNameResolver.php',
    'Illuminate\\Routing\\Middleware\\SubstituteBindings' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Middleware/SubstituteBindings.php',
    'Illuminate\\Routing\\Middleware\\ThrottleRequests' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Middleware/ThrottleRequests.php',
    'Illuminate\\Routing\\Middleware\\ThrottleRequestsWithRedis' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Middleware/ThrottleRequestsWithRedis.php',
    'Illuminate\\Routing\\Middleware\\ValidateSignature' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Middleware/ValidateSignature.php',
    'Illuminate\\Routing\\PendingResourceRegistration' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/PendingResourceRegistration.php',
    'Illuminate\\Routing\\PendingSingletonResourceRegistration' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/PendingSingletonResourceRegistration.php',
    'Illuminate\\Routing\\Pipeline' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Pipeline.php',
    'Illuminate\\Routing\\RedirectController' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RedirectController.php',
    'Illuminate\\Routing\\Redirector' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Redirector.php',
    'Illuminate\\Routing\\ResolvesRouteDependencies' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/ResolvesRouteDependencies.php',
    'Illuminate\\Routing\\ResourceRegistrar' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/ResourceRegistrar.php',
    'Illuminate\\Routing\\ResponseFactory' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/ResponseFactory.php',
    'Illuminate\\Routing\\Route' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Route.php',
    'Illuminate\\Routing\\RouteAction' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteAction.php',
    'Illuminate\\Routing\\RouteBinding' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteBinding.php',
    'Illuminate\\Routing\\RouteCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteCollection.php',
    'Illuminate\\Routing\\RouteCollectionInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteCollectionInterface.php',
    'Illuminate\\Routing\\RouteDependencyResolverTrait' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteDependencyResolverTrait.php',
    'Illuminate\\Routing\\RouteFileRegistrar' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteFileRegistrar.php',
    'Illuminate\\Routing\\RouteGroup' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteGroup.php',
    'Illuminate\\Routing\\RouteParameterBinder' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteParameterBinder.php',
    'Illuminate\\Routing\\RouteRegistrar' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteRegistrar.php',
    'Illuminate\\Routing\\RouteSignatureParameters' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteSignatureParameters.php',
    'Illuminate\\Routing\\RouteUri' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteUri.php',
    'Illuminate\\Routing\\RouteUrlGenerator' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RouteUrlGenerator.php',
    'Illuminate\\Routing\\Router' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/Router.php',
    'Illuminate\\Routing\\RoutingServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/RoutingServiceProvider.php',
    'Illuminate\\Routing\\SortedMiddleware' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/SortedMiddleware.php',
    'Illuminate\\Routing\\UrlGenerator' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/UrlGenerator.php',
    'Illuminate\\Routing\\ViewController' => $vendorDir . '/laravel/framework/src/Illuminate/Routing/ViewController.php',
    'Illuminate\\Session\\ArraySessionHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Session/ArraySessionHandler.php',
    'Illuminate\\Session\\CacheBasedSessionHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Session/CacheBasedSessionHandler.php',
    'Illuminate\\Session\\Console\\SessionTableCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Session/Console/SessionTableCommand.php',
    'Illuminate\\Session\\CookieSessionHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Session/CookieSessionHandler.php',
    'Illuminate\\Session\\DatabaseSessionHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Session/DatabaseSessionHandler.php',
    'Illuminate\\Session\\EncryptedStore' => $vendorDir . '/laravel/framework/src/Illuminate/Session/EncryptedStore.php',
    'Illuminate\\Session\\ExistenceAwareInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Session/ExistenceAwareInterface.php',
    'Illuminate\\Session\\FileSessionHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Session/FileSessionHandler.php',
    'Illuminate\\Session\\Middleware\\AuthenticateSession' => $vendorDir . '/laravel/framework/src/Illuminate/Session/Middleware/AuthenticateSession.php',
    'Illuminate\\Session\\Middleware\\StartSession' => $vendorDir . '/laravel/framework/src/Illuminate/Session/Middleware/StartSession.php',
    'Illuminate\\Session\\NullSessionHandler' => $vendorDir . '/laravel/framework/src/Illuminate/Session/NullSessionHandler.php',
    'Illuminate\\Session\\SessionManager' => $vendorDir . '/laravel/framework/src/Illuminate/Session/SessionManager.php',
    'Illuminate\\Session\\SessionServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Session/SessionServiceProvider.php',
    'Illuminate\\Session\\Store' => $vendorDir . '/laravel/framework/src/Illuminate/Session/Store.php',
    'Illuminate\\Session\\SymfonySessionDecorator' => $vendorDir . '/laravel/framework/src/Illuminate/Session/SymfonySessionDecorator.php',
    'Illuminate\\Session\\TokenMismatchException' => $vendorDir . '/laravel/framework/src/Illuminate/Session/TokenMismatchException.php',
    'Illuminate\\Support\\AggregateServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Support/AggregateServiceProvider.php',
    'Illuminate\\Support\\Arr' => $vendorDir . '/laravel/framework/src/Illuminate/Collections/Arr.php',
    'Illuminate\\Support\\Benchmark' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Benchmark.php',
    'Illuminate\\Support\\Carbon' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Carbon.php',
    'Illuminate\\Support\\Collection' => $vendorDir . '/laravel/framework/src/Illuminate/Collections/Collection.php',
    'Illuminate\\Support\\Composer' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Composer.php',
    'Illuminate\\Support\\ConfigurationUrlParser' => $vendorDir . '/laravel/framework/src/Illuminate/Support/ConfigurationUrlParser.php',
    'Illuminate\\Support\\DateFactory' => $vendorDir . '/laravel/framework/src/Illuminate/Support/DateFactory.php',
    'Illuminate\\Support\\DefaultProviders' => $vendorDir . '/laravel/framework/src/Illuminate/Support/DefaultProviders.php',
    'Illuminate\\Support\\Enumerable' => $vendorDir . '/laravel/framework/src/Illuminate/Collections/Enumerable.php',
    'Illuminate\\Support\\Env' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Env.php',
    'Illuminate\\Support\\Exceptions\\MathException' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Exceptions/MathException.php',
    'Illuminate\\Support\\Facades\\App' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/App.php',
    'Illuminate\\Support\\Facades\\Artisan' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Artisan.php',
    'Illuminate\\Support\\Facades\\Auth' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Auth.php',
    'Illuminate\\Support\\Facades\\Blade' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Blade.php',
    'Illuminate\\Support\\Facades\\Broadcast' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Broadcast.php',
    'Illuminate\\Support\\Facades\\Bus' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Bus.php',
    'Illuminate\\Support\\Facades\\Cache' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Cache.php',
    'Illuminate\\Support\\Facades\\Config' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Config.php',
    'Illuminate\\Support\\Facades\\Cookie' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Cookie.php',
    'Illuminate\\Support\\Facades\\Crypt' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Crypt.php',
    'Illuminate\\Support\\Facades\\DB' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/DB.php',
    'Illuminate\\Support\\Facades\\Date' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Date.php',
    'Illuminate\\Support\\Facades\\Event' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Event.php',
    'Illuminate\\Support\\Facades\\Facade' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Facade.php',
    'Illuminate\\Support\\Facades\\File' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/File.php',
    'Illuminate\\Support\\Facades\\Gate' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Gate.php',
    'Illuminate\\Support\\Facades\\Hash' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Hash.php',
    'Illuminate\\Support\\Facades\\Http' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Http.php',
    'Illuminate\\Support\\Facades\\Lang' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Lang.php',
    'Illuminate\\Support\\Facades\\Log' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Log.php',
    'Illuminate\\Support\\Facades\\Mail' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Mail.php',
    'Illuminate\\Support\\Facades\\Notification' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Notification.php',
    'Illuminate\\Support\\Facades\\ParallelTesting' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/ParallelTesting.php',
    'Illuminate\\Support\\Facades\\Password' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Password.php',
    'Illuminate\\Support\\Facades\\Pipeline' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Pipeline.php',
    'Illuminate\\Support\\Facades\\Process' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Process.php',
    'Illuminate\\Support\\Facades\\Queue' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Queue.php',
    'Illuminate\\Support\\Facades\\RateLimiter' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/RateLimiter.php',
    'Illuminate\\Support\\Facades\\Redirect' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Redirect.php',
    'Illuminate\\Support\\Facades\\Redis' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Redis.php',
    'Illuminate\\Support\\Facades\\Request' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Request.php',
    'Illuminate\\Support\\Facades\\Response' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Response.php',
    'Illuminate\\Support\\Facades\\Route' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Route.php',
    'Illuminate\\Support\\Facades\\Schema' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Schema.php',
    'Illuminate\\Support\\Facades\\Session' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Session.php',
    'Illuminate\\Support\\Facades\\Storage' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Storage.php',
    'Illuminate\\Support\\Facades\\URL' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/URL.php',
    'Illuminate\\Support\\Facades\\Validator' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Validator.php',
    'Illuminate\\Support\\Facades\\View' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/View.php',
    'Illuminate\\Support\\Facades\\Vite' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Facades/Vite.php',
    'Illuminate\\Support\\Fluent' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Fluent.php',
    'Illuminate\\Support\\HigherOrderCollectionProxy' => $vendorDir . '/laravel/framework/src/Illuminate/Collections/HigherOrderCollectionProxy.php',
    'Illuminate\\Support\\HigherOrderTapProxy' => $vendorDir . '/laravel/framework/src/Illuminate/Support/HigherOrderTapProxy.php',
    'Illuminate\\Support\\HigherOrderWhenProxy' => $vendorDir . '/laravel/framework/src/Illuminate/Conditionable/HigherOrderWhenProxy.php',
    'Illuminate\\Support\\HtmlString' => $vendorDir . '/laravel/framework/src/Illuminate/Support/HtmlString.php',
    'Illuminate\\Support\\InteractsWithTime' => $vendorDir . '/laravel/framework/src/Illuminate/Support/InteractsWithTime.php',
    'Illuminate\\Support\\ItemNotFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Collections/ItemNotFoundException.php',
    'Illuminate\\Support\\Js' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Js.php',
    'Illuminate\\Support\\LazyCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Collections/LazyCollection.php',
    'Illuminate\\Support\\Lottery' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Lottery.php',
    'Illuminate\\Support\\Manager' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Manager.php',
    'Illuminate\\Support\\MessageBag' => $vendorDir . '/laravel/framework/src/Illuminate/Support/MessageBag.php',
    'Illuminate\\Support\\MultipleInstanceManager' => $vendorDir . '/laravel/framework/src/Illuminate/Support/MultipleInstanceManager.php',
    'Illuminate\\Support\\MultipleItemsFoundException' => $vendorDir . '/laravel/framework/src/Illuminate/Collections/MultipleItemsFoundException.php',
    'Illuminate\\Support\\NamespacedItemResolver' => $vendorDir . '/laravel/framework/src/Illuminate/Support/NamespacedItemResolver.php',
    'Illuminate\\Support\\Number' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Number.php',
    'Illuminate\\Support\\Optional' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Optional.php',
    'Illuminate\\Support\\Pluralizer' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Pluralizer.php',
    'Illuminate\\Support\\ProcessUtils' => $vendorDir . '/laravel/framework/src/Illuminate/Support/ProcessUtils.php',
    'Illuminate\\Support\\Reflector' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Reflector.php',
    'Illuminate\\Support\\ServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Support/ServiceProvider.php',
    'Illuminate\\Support\\Sleep' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Sleep.php',
    'Illuminate\\Support\\Str' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Str.php',
    'Illuminate\\Support\\Stringable' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Stringable.php',
    'Illuminate\\Support\\Testing\\Fakes\\BatchFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/BatchFake.php',
    'Illuminate\\Support\\Testing\\Fakes\\BatchRepositoryFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/BatchRepositoryFake.php',
    'Illuminate\\Support\\Testing\\Fakes\\BusFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/BusFake.php',
    'Illuminate\\Support\\Testing\\Fakes\\ChainedBatchTruthTest' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/ChainedBatchTruthTest.php',
    'Illuminate\\Support\\Testing\\Fakes\\EventFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/EventFake.php',
    'Illuminate\\Support\\Testing\\Fakes\\Fake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/Fake.php',
    'Illuminate\\Support\\Testing\\Fakes\\MailFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/MailFake.php',
    'Illuminate\\Support\\Testing\\Fakes\\NotificationFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/NotificationFake.php',
    'Illuminate\\Support\\Testing\\Fakes\\PendingBatchFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/PendingBatchFake.php',
    'Illuminate\\Support\\Testing\\Fakes\\PendingChainFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/PendingChainFake.php',
    'Illuminate\\Support\\Testing\\Fakes\\PendingMailFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/PendingMailFake.php',
    'Illuminate\\Support\\Testing\\Fakes\\QueueFake' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Testing/Fakes/QueueFake.php',
    'Illuminate\\Support\\Timebox' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Timebox.php',
    'Illuminate\\Support\\Traits\\CapsuleManagerTrait' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Traits/CapsuleManagerTrait.php',
    'Illuminate\\Support\\Traits\\Conditionable' => $vendorDir . '/laravel/framework/src/Illuminate/Conditionable/Traits/Conditionable.php',
    'Illuminate\\Support\\Traits\\EnumeratesValues' => $vendorDir . '/laravel/framework/src/Illuminate/Collections/Traits/EnumeratesValues.php',
    'Illuminate\\Support\\Traits\\ForwardsCalls' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Traits/ForwardsCalls.php',
    'Illuminate\\Support\\Traits\\Localizable' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Traits/Localizable.php',
    'Illuminate\\Support\\Traits\\Macroable' => $vendorDir . '/laravel/framework/src/Illuminate/Macroable/Traits/Macroable.php',
    'Illuminate\\Support\\Traits\\ReflectsClosures' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Traits/ReflectsClosures.php',
    'Illuminate\\Support\\Traits\\Tappable' => $vendorDir . '/laravel/framework/src/Illuminate/Support/Traits/Tappable.php',
    'Illuminate\\Support\\ValidatedInput' => $vendorDir . '/laravel/framework/src/Illuminate/Support/ValidatedInput.php',
    'Illuminate\\Support\\ViewErrorBag' => $vendorDir . '/laravel/framework/src/Illuminate/Support/ViewErrorBag.php',
    'Illuminate\\Testing\\Assert' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Assert.php',
    'Illuminate\\Testing\\AssertableJsonString' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/AssertableJsonString.php',
    'Illuminate\\Testing\\Concerns\\AssertsStatusCodes' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Concerns/AssertsStatusCodes.php',
    'Illuminate\\Testing\\Concerns\\RunsInParallel' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Concerns/RunsInParallel.php',
    'Illuminate\\Testing\\Concerns\\TestDatabases' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Concerns/TestDatabases.php',
    'Illuminate\\Testing\\Constraints\\ArraySubset' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Constraints/ArraySubset.php',
    'Illuminate\\Testing\\Constraints\\CountInDatabase' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Constraints/CountInDatabase.php',
    'Illuminate\\Testing\\Constraints\\HasInDatabase' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Constraints/HasInDatabase.php',
    'Illuminate\\Testing\\Constraints\\NotSoftDeletedInDatabase' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Constraints/NotSoftDeletedInDatabase.php',
    'Illuminate\\Testing\\Constraints\\SeeInOrder' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Constraints/SeeInOrder.php',
    'Illuminate\\Testing\\Constraints\\SoftDeletedInDatabase' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Constraints/SoftDeletedInDatabase.php',
    'Illuminate\\Testing\\Exceptions\\InvalidArgumentException' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Exceptions/InvalidArgumentException.php',
    'Illuminate\\Testing\\Fluent\\AssertableJson' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Fluent/AssertableJson.php',
    'Illuminate\\Testing\\Fluent\\Concerns\\Debugging' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Fluent/Concerns/Debugging.php',
    'Illuminate\\Testing\\Fluent\\Concerns\\Has' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Fluent/Concerns/Has.php',
    'Illuminate\\Testing\\Fluent\\Concerns\\Interaction' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Fluent/Concerns/Interaction.php',
    'Illuminate\\Testing\\Fluent\\Concerns\\Matching' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/Fluent/Concerns/Matching.php',
    'Illuminate\\Testing\\LoggedExceptionCollection' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/LoggedExceptionCollection.php',
    'Illuminate\\Testing\\ParallelConsoleOutput' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/ParallelConsoleOutput.php',
    'Illuminate\\Testing\\ParallelRunner' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/ParallelRunner.php',
    'Illuminate\\Testing\\ParallelTesting' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/ParallelTesting.php',
    'Illuminate\\Testing\\ParallelTestingServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/ParallelTestingServiceProvider.php',
    'Illuminate\\Testing\\PendingCommand' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/PendingCommand.php',
    'Illuminate\\Testing\\TestComponent' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/TestComponent.php',
    'Illuminate\\Testing\\TestResponse' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/TestResponse.php',
    'Illuminate\\Testing\\TestView' => $vendorDir . '/laravel/framework/src/Illuminate/Testing/TestView.php',
    'Illuminate\\Translation\\ArrayLoader' => $vendorDir . '/laravel/framework/src/Illuminate/Translation/ArrayLoader.php',
    'Illuminate\\Translation\\CreatesPotentiallyTranslatedStrings' => $vendorDir . '/laravel/framework/src/Illuminate/Translation/CreatesPotentiallyTranslatedStrings.php',
    'Illuminate\\Translation\\FileLoader' => $vendorDir . '/laravel/framework/src/Illuminate/Translation/FileLoader.php',
    'Illuminate\\Translation\\MessageSelector' => $vendorDir . '/laravel/framework/src/Illuminate/Translation/MessageSelector.php',
    'Illuminate\\Translation\\PotentiallyTranslatedString' => $vendorDir . '/laravel/framework/src/Illuminate/Translation/PotentiallyTranslatedString.php',
    'Illuminate\\Translation\\TranslationServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Translation/TranslationServiceProvider.php',
    'Illuminate\\Translation\\Translator' => $vendorDir . '/laravel/framework/src/Illuminate/Translation/Translator.php',
    'Illuminate\\Validation\\ClosureValidationRule' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/ClosureValidationRule.php',
    'Illuminate\\Validation\\Concerns\\FilterEmailValidation' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Concerns/FilterEmailValidation.php',
    'Illuminate\\Validation\\Concerns\\FormatsMessages' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Concerns/FormatsMessages.php',
    'Illuminate\\Validation\\Concerns\\ReplacesAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Concerns/ReplacesAttributes.php',
    'Illuminate\\Validation\\Concerns\\ValidatesAttributes' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Concerns/ValidatesAttributes.php',
    'Illuminate\\Validation\\ConditionalRules' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/ConditionalRules.php',
    'Illuminate\\Validation\\DatabasePresenceVerifier' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/DatabasePresenceVerifier.php',
    'Illuminate\\Validation\\DatabasePresenceVerifierInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/DatabasePresenceVerifierInterface.php',
    'Illuminate\\Validation\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Factory.php',
    'Illuminate\\Validation\\InvokableValidationRule' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/InvokableValidationRule.php',
    'Illuminate\\Validation\\NestedRules' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/NestedRules.php',
    'Illuminate\\Validation\\NotPwnedVerifier' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/NotPwnedVerifier.php',
    'Illuminate\\Validation\\PresenceVerifierInterface' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/PresenceVerifierInterface.php',
    'Illuminate\\Validation\\Rule' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rule.php',
    'Illuminate\\Validation\\Rules\\Can' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/Can.php',
    'Illuminate\\Validation\\Rules\\DatabaseRule' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/DatabaseRule.php',
    'Illuminate\\Validation\\Rules\\Dimensions' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/Dimensions.php',
    'Illuminate\\Validation\\Rules\\Enum' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/Enum.php',
    'Illuminate\\Validation\\Rules\\ExcludeIf' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/ExcludeIf.php',
    'Illuminate\\Validation\\Rules\\Exists' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/Exists.php',
    'Illuminate\\Validation\\Rules\\File' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/File.php',
    'Illuminate\\Validation\\Rules\\ImageFile' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/ImageFile.php',
    'Illuminate\\Validation\\Rules\\In' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/In.php',
    'Illuminate\\Validation\\Rules\\NotIn' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/NotIn.php',
    'Illuminate\\Validation\\Rules\\Password' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/Password.php',
    'Illuminate\\Validation\\Rules\\ProhibitedIf' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/ProhibitedIf.php',
    'Illuminate\\Validation\\Rules\\RequiredIf' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/RequiredIf.php',
    'Illuminate\\Validation\\Rules\\Unique' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Rules/Unique.php',
    'Illuminate\\Validation\\UnauthorizedException' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/UnauthorizedException.php',
    'Illuminate\\Validation\\ValidatesWhenResolvedTrait' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/ValidatesWhenResolvedTrait.php',
    'Illuminate\\Validation\\ValidationData' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/ValidationData.php',
    'Illuminate\\Validation\\ValidationException' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/ValidationException.php',
    'Illuminate\\Validation\\ValidationRuleParser' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/ValidationRuleParser.php',
    'Illuminate\\Validation\\ValidationServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/ValidationServiceProvider.php',
    'Illuminate\\Validation\\Validator' => $vendorDir . '/laravel/framework/src/Illuminate/Validation/Validator.php',
    'Illuminate\\View\\AnonymousComponent' => $vendorDir . '/laravel/framework/src/Illuminate/View/AnonymousComponent.php',
    'Illuminate\\View\\AppendableAttributeValue' => $vendorDir . '/laravel/framework/src/Illuminate/View/AppendableAttributeValue.php',
    'Illuminate\\View\\Compilers\\BladeCompiler' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/BladeCompiler.php',
    'Illuminate\\View\\Compilers\\Compiler' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Compiler.php',
    'Illuminate\\View\\Compilers\\CompilerInterface' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/CompilerInterface.php',
    'Illuminate\\View\\Compilers\\ComponentTagCompiler' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/ComponentTagCompiler.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesAuthorizations' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesAuthorizations.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesClasses' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesClasses.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesComments' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesComments.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesComponents' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesComponents.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesConditionals' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesConditionals.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesEchos' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesEchos.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesErrors' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesErrors.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesFragments' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesFragments.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesHelpers' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesHelpers.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesIncludes' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesIncludes.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesInjections' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesInjections.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesJs' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesJs.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesJson' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesJson.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesLayouts' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesLayouts.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesLoops' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesLoops.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesRawPhp' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesRawPhp.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesSessions' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesSessions.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesStacks' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesStacks.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesStyles' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesStyles.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesTranslations' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesTranslations.php',
    'Illuminate\\View\\Compilers\\Concerns\\CompilesUseStatements' => $vendorDir . '/laravel/framework/src/Illuminate/View/Compilers/Concerns/CompilesUseStatements.php',
    'Illuminate\\View\\Component' => $vendorDir . '/laravel/framework/src/Illuminate/View/Component.php',
    'Illuminate\\View\\ComponentAttributeBag' => $vendorDir . '/laravel/framework/src/Illuminate/View/ComponentAttributeBag.php',
    'Illuminate\\View\\ComponentSlot' => $vendorDir . '/laravel/framework/src/Illuminate/View/ComponentSlot.php',
    'Illuminate\\View\\Concerns\\ManagesComponents' => $vendorDir . '/laravel/framework/src/Illuminate/View/Concerns/ManagesComponents.php',
    'Illuminate\\View\\Concerns\\ManagesEvents' => $vendorDir . '/laravel/framework/src/Illuminate/View/Concerns/ManagesEvents.php',
    'Illuminate\\View\\Concerns\\ManagesFragments' => $vendorDir . '/laravel/framework/src/Illuminate/View/Concerns/ManagesFragments.php',
    'Illuminate\\View\\Concerns\\ManagesLayouts' => $vendorDir . '/laravel/framework/src/Illuminate/View/Concerns/ManagesLayouts.php',
    'Illuminate\\View\\Concerns\\ManagesLoops' => $vendorDir . '/laravel/framework/src/Illuminate/View/Concerns/ManagesLoops.php',
    'Illuminate\\View\\Concerns\\ManagesStacks' => $vendorDir . '/laravel/framework/src/Illuminate/View/Concerns/ManagesStacks.php',
    'Illuminate\\View\\Concerns\\ManagesTranslations' => $vendorDir . '/laravel/framework/src/Illuminate/View/Concerns/ManagesTranslations.php',
    'Illuminate\\View\\DynamicComponent' => $vendorDir . '/laravel/framework/src/Illuminate/View/DynamicComponent.php',
    'Illuminate\\View\\Engines\\CompilerEngine' => $vendorDir . '/laravel/framework/src/Illuminate/View/Engines/CompilerEngine.php',
    'Illuminate\\View\\Engines\\Engine' => $vendorDir . '/laravel/framework/src/Illuminate/View/Engines/Engine.php',
    'Illuminate\\View\\Engines\\EngineResolver' => $vendorDir . '/laravel/framework/src/Illuminate/View/Engines/EngineResolver.php',
    'Illuminate\\View\\Engines\\FileEngine' => $vendorDir . '/laravel/framework/src/Illuminate/View/Engines/FileEngine.php',
    'Illuminate\\View\\Engines\\PhpEngine' => $vendorDir . '/laravel/framework/src/Illuminate/View/Engines/PhpEngine.php',
    'Illuminate\\View\\Factory' => $vendorDir . '/laravel/framework/src/Illuminate/View/Factory.php',
    'Illuminate\\View\\FileViewFinder' => $vendorDir . '/laravel/framework/src/Illuminate/View/FileViewFinder.php',
    'Illuminate\\View\\InvokableComponentVariable' => $vendorDir . '/laravel/framework/src/Illuminate/View/InvokableComponentVariable.php',
    'Illuminate\\View\\Middleware\\ShareErrorsFromSession' => $vendorDir . '/laravel/framework/src/Illuminate/View/Middleware/ShareErrorsFromSession.php',
    'Illuminate\\View\\View' => $vendorDir . '/laravel/framework/src/Illuminate/View/View.php',
    'Illuminate\\View\\ViewException' => $vendorDir . '/laravel/framework/src/Illuminate/View/ViewException.php',
    'Illuminate\\View\\ViewFinderInterface' => $vendorDir . '/laravel/framework/src/Illuminate/View/ViewFinderInterface.php',
    'Illuminate\\View\\ViewName' => $vendorDir . '/laravel/framework/src/Illuminate/View/ViewName.php',
    'Illuminate\\View\\ViewServiceProvider' => $vendorDir . '/laravel/framework/src/Illuminate/View/ViewServiceProvider.php',
    'Intervention\\Image\\AbstractColor' => $vendorDir . '/intervention/image/src/Intervention/Image/AbstractColor.php',
    'Intervention\\Image\\AbstractDecoder' => $vendorDir . '/intervention/image/src/Intervention/Image/AbstractDecoder.php',
    'Intervention\\Image\\AbstractDriver' => $vendorDir . '/intervention/image/src/Intervention/Image/AbstractDriver.php',
    'Intervention\\Image\\AbstractEncoder' => $vendorDir . '/intervention/image/src/Intervention/Image/AbstractEncoder.php',
    'Intervention\\Image\\AbstractFont' => $vendorDir . '/intervention/image/src/Intervention/Image/AbstractFont.php',
    'Intervention\\Image\\AbstractShape' => $vendorDir . '/intervention/image/src/Intervention/Image/AbstractShape.php',
    'Intervention\\Image\\Commands\\AbstractCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/AbstractCommand.php',
    'Intervention\\Image\\Commands\\Argument' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/Argument.php',
    'Intervention\\Image\\Commands\\ChecksumCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/ChecksumCommand.php',
    'Intervention\\Image\\Commands\\CircleCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/CircleCommand.php',
    'Intervention\\Image\\Commands\\EllipseCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/EllipseCommand.php',
    'Intervention\\Image\\Commands\\ExifCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/ExifCommand.php',
    'Intervention\\Image\\Commands\\IptcCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/IptcCommand.php',
    'Intervention\\Image\\Commands\\LineCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/LineCommand.php',
    'Intervention\\Image\\Commands\\OrientateCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/OrientateCommand.php',
    'Intervention\\Image\\Commands\\PolygonCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/PolygonCommand.php',
    'Intervention\\Image\\Commands\\PsrResponseCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/PsrResponseCommand.php',
    'Intervention\\Image\\Commands\\RectangleCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/RectangleCommand.php',
    'Intervention\\Image\\Commands\\ResponseCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/ResponseCommand.php',
    'Intervention\\Image\\Commands\\StreamCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/StreamCommand.php',
    'Intervention\\Image\\Commands\\TextCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Commands/TextCommand.php',
    'Intervention\\Image\\Constraint' => $vendorDir . '/intervention/image/src/Intervention/Image/Constraint.php',
    'Intervention\\Image\\Exception\\ImageException' => $vendorDir . '/intervention/image/src/Intervention/Image/Exception/ImageException.php',
    'Intervention\\Image\\Exception\\InvalidArgumentException' => $vendorDir . '/intervention/image/src/Intervention/Image/Exception/InvalidArgumentException.php',
    'Intervention\\Image\\Exception\\MissingDependencyException' => $vendorDir . '/intervention/image/src/Intervention/Image/Exception/MissingDependencyException.php',
    'Intervention\\Image\\Exception\\NotFoundException' => $vendorDir . '/intervention/image/src/Intervention/Image/Exception/NotFoundException.php',
    'Intervention\\Image\\Exception\\NotReadableException' => $vendorDir . '/intervention/image/src/Intervention/Image/Exception/NotReadableException.php',
    'Intervention\\Image\\Exception\\NotSupportedException' => $vendorDir . '/intervention/image/src/Intervention/Image/Exception/NotSupportedException.php',
    'Intervention\\Image\\Exception\\NotWritableException' => $vendorDir . '/intervention/image/src/Intervention/Image/Exception/NotWritableException.php',
    'Intervention\\Image\\Exception\\RuntimeException' => $vendorDir . '/intervention/image/src/Intervention/Image/Exception/RuntimeException.php',
    'Intervention\\Image\\Facades\\Image' => $vendorDir . '/intervention/image/src/Intervention/Image/Facades/Image.php',
    'Intervention\\Image\\File' => $vendorDir . '/intervention/image/src/Intervention/Image/File.php',
    'Intervention\\Image\\Filters\\DemoFilter' => $vendorDir . '/intervention/image/src/Intervention/Image/Filters/DemoFilter.php',
    'Intervention\\Image\\Filters\\FilterInterface' => $vendorDir . '/intervention/image/src/Intervention/Image/Filters/FilterInterface.php',
    'Intervention\\Image\\Gd\\Color' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Color.php',
    'Intervention\\Image\\Gd\\Commands\\BackupCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/BackupCommand.php',
    'Intervention\\Image\\Gd\\Commands\\BlurCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/BlurCommand.php',
    'Intervention\\Image\\Gd\\Commands\\BrightnessCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/BrightnessCommand.php',
    'Intervention\\Image\\Gd\\Commands\\ColorizeCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/ColorizeCommand.php',
    'Intervention\\Image\\Gd\\Commands\\ContrastCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/ContrastCommand.php',
    'Intervention\\Image\\Gd\\Commands\\CropCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/CropCommand.php',
    'Intervention\\Image\\Gd\\Commands\\DestroyCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/DestroyCommand.php',
    'Intervention\\Image\\Gd\\Commands\\FillCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/FillCommand.php',
    'Intervention\\Image\\Gd\\Commands\\FitCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/FitCommand.php',
    'Intervention\\Image\\Gd\\Commands\\FlipCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/FlipCommand.php',
    'Intervention\\Image\\Gd\\Commands\\GammaCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/GammaCommand.php',
    'Intervention\\Image\\Gd\\Commands\\GetSizeCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/GetSizeCommand.php',
    'Intervention\\Image\\Gd\\Commands\\GreyscaleCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/GreyscaleCommand.php',
    'Intervention\\Image\\Gd\\Commands\\HeightenCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/HeightenCommand.php',
    'Intervention\\Image\\Gd\\Commands\\InsertCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/InsertCommand.php',
    'Intervention\\Image\\Gd\\Commands\\InterlaceCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/InterlaceCommand.php',
    'Intervention\\Image\\Gd\\Commands\\InvertCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/InvertCommand.php',
    'Intervention\\Image\\Gd\\Commands\\LimitColorsCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/LimitColorsCommand.php',
    'Intervention\\Image\\Gd\\Commands\\MaskCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/MaskCommand.php',
    'Intervention\\Image\\Gd\\Commands\\OpacityCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/OpacityCommand.php',
    'Intervention\\Image\\Gd\\Commands\\PickColorCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/PickColorCommand.php',
    'Intervention\\Image\\Gd\\Commands\\PixelCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/PixelCommand.php',
    'Intervention\\Image\\Gd\\Commands\\PixelateCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/PixelateCommand.php',
    'Intervention\\Image\\Gd\\Commands\\ResetCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/ResetCommand.php',
    'Intervention\\Image\\Gd\\Commands\\ResizeCanvasCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/ResizeCanvasCommand.php',
    'Intervention\\Image\\Gd\\Commands\\ResizeCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/ResizeCommand.php',
    'Intervention\\Image\\Gd\\Commands\\RotateCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/RotateCommand.php',
    'Intervention\\Image\\Gd\\Commands\\SharpenCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/SharpenCommand.php',
    'Intervention\\Image\\Gd\\Commands\\TrimCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/TrimCommand.php',
    'Intervention\\Image\\Gd\\Commands\\WidenCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Commands/WidenCommand.php',
    'Intervention\\Image\\Gd\\Decoder' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Decoder.php',
    'Intervention\\Image\\Gd\\Driver' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Driver.php',
    'Intervention\\Image\\Gd\\Encoder' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Encoder.php',
    'Intervention\\Image\\Gd\\Font' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Font.php',
    'Intervention\\Image\\Gd\\Shapes\\CircleShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Shapes/CircleShape.php',
    'Intervention\\Image\\Gd\\Shapes\\EllipseShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Shapes/EllipseShape.php',
    'Intervention\\Image\\Gd\\Shapes\\LineShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Shapes/LineShape.php',
    'Intervention\\Image\\Gd\\Shapes\\PolygonShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Shapes/PolygonShape.php',
    'Intervention\\Image\\Gd\\Shapes\\RectangleShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Gd/Shapes/RectangleShape.php',
    'Intervention\\Image\\Image' => $vendorDir . '/intervention/image/src/Intervention/Image/Image.php',
    'Intervention\\Image\\ImageManager' => $vendorDir . '/intervention/image/src/Intervention/Image/ImageManager.php',
    'Intervention\\Image\\ImageManagerStatic' => $vendorDir . '/intervention/image/src/Intervention/Image/ImageManagerStatic.php',
    'Intervention\\Image\\ImageServiceProvider' => $vendorDir . '/intervention/image/src/Intervention/Image/ImageServiceProvider.php',
    'Intervention\\Image\\ImageServiceProviderLaravel4' => $vendorDir . '/intervention/image/src/Intervention/Image/ImageServiceProviderLaravel4.php',
    'Intervention\\Image\\ImageServiceProviderLaravelRecent' => $vendorDir . '/intervention/image/src/Intervention/Image/ImageServiceProviderLaravelRecent.php',
    'Intervention\\Image\\ImageServiceProviderLeague' => $vendorDir . '/intervention/image/src/Intervention/Image/ImageServiceProviderLeague.php',
    'Intervention\\Image\\ImageServiceProviderLumen' => $vendorDir . '/intervention/image/src/Intervention/Image/ImageServiceProviderLumen.php',
    'Intervention\\Image\\Imagick\\Color' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Color.php',
    'Intervention\\Image\\Imagick\\Commands\\BackupCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/BackupCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\BlurCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/BlurCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\BrightnessCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/BrightnessCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\ColorizeCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/ColorizeCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\ContrastCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/ContrastCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\CropCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/CropCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\DestroyCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/DestroyCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\ExifCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/ExifCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\FillCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/FillCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\FitCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/FitCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\FlipCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/FlipCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\GammaCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/GammaCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\GetSizeCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/GetSizeCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\GreyscaleCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/GreyscaleCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\HeightenCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/HeightenCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\InsertCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/InsertCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\InterlaceCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/InterlaceCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\InvertCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/InvertCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\LimitColorsCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/LimitColorsCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\MaskCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/MaskCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\OpacityCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/OpacityCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\PickColorCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/PickColorCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\PixelCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/PixelCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\PixelateCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/PixelateCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\ResetCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/ResetCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\ResizeCanvasCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/ResizeCanvasCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\ResizeCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/ResizeCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\RotateCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/RotateCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\SharpenCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/SharpenCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\TrimCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/TrimCommand.php',
    'Intervention\\Image\\Imagick\\Commands\\WidenCommand' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Commands/WidenCommand.php',
    'Intervention\\Image\\Imagick\\Decoder' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Decoder.php',
    'Intervention\\Image\\Imagick\\Driver' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Driver.php',
    'Intervention\\Image\\Imagick\\Encoder' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Encoder.php',
    'Intervention\\Image\\Imagick\\Font' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Font.php',
    'Intervention\\Image\\Imagick\\Shapes\\CircleShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Shapes/CircleShape.php',
    'Intervention\\Image\\Imagick\\Shapes\\EllipseShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Shapes/EllipseShape.php',
    'Intervention\\Image\\Imagick\\Shapes\\LineShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Shapes/LineShape.php',
    'Intervention\\Image\\Imagick\\Shapes\\PolygonShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Shapes/PolygonShape.php',
    'Intervention\\Image\\Imagick\\Shapes\\RectangleShape' => $vendorDir . '/intervention/image/src/Intervention/Image/Imagick/Shapes/RectangleShape.php',
    'Intervention\\Image\\Point' => $vendorDir . '/intervention/image/src/Intervention/Image/Point.php',
    'Intervention\\Image\\Response' => $vendorDir . '/intervention/image/src/Intervention/Image/Response.php',
    'Intervention\\Image\\Size' => $vendorDir . '/intervention/image/src/Intervention/Image/Size.php',
    'KitLoong\\MigrationsGenerator\\Database\\DatabaseColumnType' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/DatabaseColumnType.php',
    'KitLoong\\MigrationsGenerator\\Database\\DatabaseSchema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/DatabaseSchema.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\DatabaseColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/DatabaseColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\DatabaseForeignKey' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/DatabaseForeignKey.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\DatabaseIndex' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/DatabaseIndex.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\DatabaseProcedure' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/DatabaseProcedure.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\DatabaseTable' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/DatabaseTable.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\DatabaseUDTColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/DatabaseUDTColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\DatabaseView' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/DatabaseView.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\MySQL\\MySQLColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/MySQL/MySQLColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\MySQL\\MySQLColumnType' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/MySQL/MySQLColumnType.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\MySQL\\MySQLForeignKey' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/MySQL/MySQLForeignKey.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\MySQL\\MySQLIndex' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/MySQL/MySQLIndex.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\MySQL\\MySQLProcedure' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/MySQL/MySQLProcedure.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\MySQL\\MySQLTable' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/MySQL/MySQLTable.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\MySQL\\MySQLUDTColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/MySQL/MySQLUDTColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\MySQL\\MySQLView' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/MySQL/MySQLView.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\PgSQL\\PgSQLColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/PgSQL/PgSQLColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\PgSQL\\PgSQLColumnType' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/PgSQL/PgSQLColumnType.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\PgSQL\\PgSQLForeignKey' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/PgSQL/PgSQLForeignKey.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\PgSQL\\PgSQLIndex' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/PgSQL/PgSQLIndex.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\PgSQL\\PgSQLParser' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/PgSQL/PgSQLParser.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\PgSQL\\PgSQLProcedure' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/PgSQL/PgSQLProcedure.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\PgSQL\\PgSQLTable' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/PgSQL/PgSQLTable.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\PgSQL\\PgSQLUDTColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/PgSQL/PgSQLUDTColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\PgSQL\\PgSQLView' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/PgSQL/PgSQLView.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLSrv\\SQLSrvColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLSrv/SQLSrvColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLSrv\\SQLSrvColumnType' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLSrv/SQLSrvColumnType.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLSrv\\SQLSrvForeignKey' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLSrv/SQLSrvForeignKey.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLSrv\\SQLSrvIndex' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLSrv/SQLSrvIndex.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLSrv\\SQLSrvParser' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLSrv/SQLSrvParser.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLSrv\\SQLSrvProcedure' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLSrv/SQLSrvProcedure.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLSrv\\SQLSrvTable' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLSrv/SQLSrvTable.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLSrv\\SQLSrvUDTColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLSrv/SQLSrvUDTColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLSrv\\SQLSrvView' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLSrv/SQLSrvView.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLite\\SQLiteColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLite/SQLiteColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLite\\SQLiteColumnType' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLite/SQLiteColumnType.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLite\\SQLiteForeignKey' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLite/SQLiteForeignKey.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLite\\SQLiteIndex' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLite/SQLiteIndex.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLite\\SQLiteTable' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLite/SQLiteTable.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLite\\SQLiteUDTColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLite/SQLiteUDTColumn.php',
    'KitLoong\\MigrationsGenerator\\Database\\Models\\SQLite\\SQLiteView' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/Models/SQLite/SQLiteView.php',
    'KitLoong\\MigrationsGenerator\\Database\\MySQLSchema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/MySQLSchema.php',
    'KitLoong\\MigrationsGenerator\\Database\\PgSQLSchema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/PgSQLSchema.php',
    'KitLoong\\MigrationsGenerator\\Database\\SQLSrvSchema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/SQLSrvSchema.php',
    'KitLoong\\MigrationsGenerator\\Database\\SQLiteSchema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Database/SQLiteSchema.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Driver' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Driver.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\ColumnName' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/ColumnName.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Method\\ColumnModifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Method/ColumnModifier.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Method\\ColumnType' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Method/ColumnType.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Method\\DBBuilder' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Method/DBBuilder.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Method\\Foreign' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Method/Foreign.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Method\\IndexType' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Method/IndexType.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Method\\MethodName' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Method/MethodName.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Method\\SchemaBuilder' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Method/SchemaBuilder.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Method\\TableMethod' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Method/TableMethod.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Property\\PropertyName' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Property/PropertyName.php',
    'KitLoong\\MigrationsGenerator\\Enum\\Migrations\\Property\\TableProperty' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Enum/Migrations/Property/TableProperty.php',
    'KitLoong\\MigrationsGenerator\\MigrateGenerateCommand' => $vendorDir . '/kitloong/laravel-migrations-generator/src/MigrateGenerateCommand.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\DBStatementBlueprint' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/DBStatementBlueprint.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\DBUnpreparedBlueprint' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/DBUnpreparedBlueprint.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\Method' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/Method.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\Property' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/Property.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\SchemaBlueprint' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/SchemaBlueprint.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\Support\\MergeTimestamps' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/Support/MergeTimestamps.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\Support\\MethodStringHelper' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/Support/MethodStringHelper.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\Support\\Stringable' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/Support/Stringable.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\TableBlueprint' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/TableBlueprint.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Blueprint\\WritableBlueprint' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Blueprint/WritableBlueprint.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Enum\\MigrationFileType' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Enum/MigrationFileType.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Enum\\Space' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Enum/Space.php',
    'KitLoong\\MigrationsGenerator\\Migration\\ForeignKeyMigration' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/ForeignKeyMigration.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\ColumnGenerator' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/ColumnGenerator.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\BooleanColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/BooleanColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\ColumnTypeGenerator' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/ColumnTypeGenerator.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\DatetimeColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/DatetimeColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\DecimalColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/DecimalColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\DoubleColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/DoubleColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\FloatColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/FloatColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\IntegerColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/IntegerColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\MiscColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/MiscColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\OmitNameColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/OmitNameColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\PresetValuesColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/PresetValuesColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\SoftDeleteColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/SoftDeleteColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\SpatialColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/SpatialColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Columns\\StringColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Columns/StringColumn.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\ForeignKeyGenerator' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/ForeignKeyGenerator.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\IndexGenerator' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/IndexGenerator.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Modifiers\\CharsetModifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Modifiers/CharsetModifier.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Modifiers\\CollationModifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Modifiers/CollationModifier.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Modifiers\\CommentModifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Modifiers/CommentModifier.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Modifiers\\DefaultModifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Modifiers/DefaultModifier.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Modifiers\\IndexModifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Modifiers/IndexModifier.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Modifiers\\Modifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Modifiers/Modifier.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Modifiers\\NullableModifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Modifiers/NullableModifier.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Modifiers\\StoredAsModifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Modifiers/StoredAsModifier.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Generator\\Modifiers\\VirtualAsModifier' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Generator/Modifiers/VirtualAsModifier.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Migrator\\Migrator' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Migrator/Migrator.php',
    'KitLoong\\MigrationsGenerator\\Migration\\ProcedureMigration' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/ProcedureMigration.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Squash' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Squash.php',
    'KitLoong\\MigrationsGenerator\\Migration\\TableMigration' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/TableMigration.php',
    'KitLoong\\MigrationsGenerator\\Migration\\ViewMigration' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/ViewMigration.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Writer\\MigrationStub' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Writer/MigrationStub.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Writer\\MigrationWriter' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Writer/MigrationWriter.php',
    'KitLoong\\MigrationsGenerator\\Migration\\Writer\\SquashWriter' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Migration/Writer/SquashWriter.php',
    'KitLoong\\MigrationsGenerator\\MigrationsGeneratorServiceProvider' => $vendorDir . '/kitloong/laravel-migrations-generator/src/MigrationsGeneratorServiceProvider.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\Entities\\MariaDB\\CheckConstraint' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/Entities/MariaDB/CheckConstraint.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\Entities\\MySQL\\ShowColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/Entities/MySQL/ShowColumn.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\Entities\\PgSQL\\IndexDefinition' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/Entities/PgSQL/IndexDefinition.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\Entities\\ProcedureDefinition' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/Entities/ProcedureDefinition.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\Entities\\SQLSrv\\ColumnDefinition' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/Entities/SQLSrv/ColumnDefinition.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\MariaDBRepository' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/MariaDBRepository.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\MySQLRepository' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/MySQLRepository.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\PgSQLRepository' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/PgSQLRepository.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\Repository' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/Repository.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\SQLSrvRepository' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/SQLSrvRepository.php',
    'KitLoong\\MigrationsGenerator\\Repositories\\SQLiteRepository' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Repositories/SQLiteRepository.php',
    'KitLoong\\MigrationsGenerator\\Schema\\Models\\Column' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/Models/Column.php',
    'KitLoong\\MigrationsGenerator\\Schema\\Models\\ForeignKey' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/Models/ForeignKey.php',
    'KitLoong\\MigrationsGenerator\\Schema\\Models\\Index' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/Models/Index.php',
    'KitLoong\\MigrationsGenerator\\Schema\\Models\\Model' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/Models/Model.php',
    'KitLoong\\MigrationsGenerator\\Schema\\Models\\Procedure' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/Models/Procedure.php',
    'KitLoong\\MigrationsGenerator\\Schema\\Models\\Table' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/Models/Table.php',
    'KitLoong\\MigrationsGenerator\\Schema\\Models\\UDTColumn' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/Models/UDTColumn.php',
    'KitLoong\\MigrationsGenerator\\Schema\\Models\\View' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/Models/View.php',
    'KitLoong\\MigrationsGenerator\\Schema\\MySQLSchema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/MySQLSchema.php',
    'KitLoong\\MigrationsGenerator\\Schema\\PgSQLSchema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/PgSQLSchema.php',
    'KitLoong\\MigrationsGenerator\\Schema\\SQLSrvSchema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/SQLSrvSchema.php',
    'KitLoong\\MigrationsGenerator\\Schema\\SQLiteSchema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/SQLiteSchema.php',
    'KitLoong\\MigrationsGenerator\\Schema\\Schema' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Schema/Schema.php',
    'KitLoong\\MigrationsGenerator\\Setting' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Setting.php',
    'KitLoong\\MigrationsGenerator\\Support\\AssetNameQuote' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Support/AssetNameQuote.php',
    'KitLoong\\MigrationsGenerator\\Support\\CheckLaravelVersion' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Support/CheckLaravelVersion.php',
    'KitLoong\\MigrationsGenerator\\Support\\CheckMigrationMethod' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Support/CheckMigrationMethod.php',
    'KitLoong\\MigrationsGenerator\\Support\\IndexNameHelper' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Support/IndexNameHelper.php',
    'KitLoong\\MigrationsGenerator\\Support\\MigrationNameHelper' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Support/MigrationNameHelper.php',
    'KitLoong\\MigrationsGenerator\\Support\\Regex' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Support/Regex.php',
    'KitLoong\\MigrationsGenerator\\Support\\TableName' => $vendorDir . '/kitloong/laravel-migrations-generator/src/Support/TableName.php',
    'Laminas\\Diactoros\\AbstractSerializer' => $vendorDir . '/laminas/laminas-diactoros/src/AbstractSerializer.php',
    'Laminas\\Diactoros\\CallbackStream' => $vendorDir . '/laminas/laminas-diactoros/src/CallbackStream.php',
    'Laminas\\Diactoros\\ConfigProvider' => $vendorDir . '/laminas/laminas-diactoros/src/ConfigProvider.php',
    'Laminas\\Diactoros\\Exception\\DeserializationException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/DeserializationException.php',
    'Laminas\\Diactoros\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/ExceptionInterface.php',
    'Laminas\\Diactoros\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/InvalidArgumentException.php',
    'Laminas\\Diactoros\\Exception\\InvalidForwardedHeaderNameException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/InvalidForwardedHeaderNameException.php',
    'Laminas\\Diactoros\\Exception\\InvalidProxyAddressException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/InvalidProxyAddressException.php',
    'Laminas\\Diactoros\\Exception\\InvalidStreamPointerPositionException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/InvalidStreamPointerPositionException.php',
    'Laminas\\Diactoros\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/RuntimeException.php',
    'Laminas\\Diactoros\\Exception\\SerializationException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/SerializationException.php',
    'Laminas\\Diactoros\\Exception\\UnreadableStreamException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/UnreadableStreamException.php',
    'Laminas\\Diactoros\\Exception\\UnrecognizedProtocolVersionException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/UnrecognizedProtocolVersionException.php',
    'Laminas\\Diactoros\\Exception\\UnrewindableStreamException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/UnrewindableStreamException.php',
    'Laminas\\Diactoros\\Exception\\UnseekableStreamException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/UnseekableStreamException.php',
    'Laminas\\Diactoros\\Exception\\UntellableStreamException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/UntellableStreamException.php',
    'Laminas\\Diactoros\\Exception\\UnwritableStreamException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/UnwritableStreamException.php',
    'Laminas\\Diactoros\\Exception\\UploadedFileAlreadyMovedException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/UploadedFileAlreadyMovedException.php',
    'Laminas\\Diactoros\\Exception\\UploadedFileErrorException' => $vendorDir . '/laminas/laminas-diactoros/src/Exception/UploadedFileErrorException.php',
    'Laminas\\Diactoros\\HeaderSecurity' => $vendorDir . '/laminas/laminas-diactoros/src/HeaderSecurity.php',
    'Laminas\\Diactoros\\MessageTrait' => $vendorDir . '/laminas/laminas-diactoros/src/MessageTrait.php',
    'Laminas\\Diactoros\\Module' => $vendorDir . '/laminas/laminas-diactoros/src/Module.php',
    'Laminas\\Diactoros\\PhpInputStream' => $vendorDir . '/laminas/laminas-diactoros/src/PhpInputStream.php',
    'Laminas\\Diactoros\\RelativeStream' => $vendorDir . '/laminas/laminas-diactoros/src/RelativeStream.php',
    'Laminas\\Diactoros\\Request' => $vendorDir . '/laminas/laminas-diactoros/src/Request.php',
    'Laminas\\Diactoros\\RequestFactory' => $vendorDir . '/laminas/laminas-diactoros/src/RequestFactory.php',
    'Laminas\\Diactoros\\RequestTrait' => $vendorDir . '/laminas/laminas-diactoros/src/RequestTrait.php',
    'Laminas\\Diactoros\\Request\\ArraySerializer' => $vendorDir . '/laminas/laminas-diactoros/src/Request/ArraySerializer.php',
    'Laminas\\Diactoros\\Request\\Serializer' => $vendorDir . '/laminas/laminas-diactoros/src/Request/Serializer.php',
    'Laminas\\Diactoros\\Response' => $vendorDir . '/laminas/laminas-diactoros/src/Response.php',
    'Laminas\\Diactoros\\ResponseFactory' => $vendorDir . '/laminas/laminas-diactoros/src/ResponseFactory.php',
    'Laminas\\Diactoros\\Response\\ArraySerializer' => $vendorDir . '/laminas/laminas-diactoros/src/Response/ArraySerializer.php',
    'Laminas\\Diactoros\\Response\\EmptyResponse' => $vendorDir . '/laminas/laminas-diactoros/src/Response/EmptyResponse.php',
    'Laminas\\Diactoros\\Response\\HtmlResponse' => $vendorDir . '/laminas/laminas-diactoros/src/Response/HtmlResponse.php',
    'Laminas\\Diactoros\\Response\\InjectContentTypeTrait' => $vendorDir . '/laminas/laminas-diactoros/src/Response/InjectContentTypeTrait.php',
    'Laminas\\Diactoros\\Response\\JsonResponse' => $vendorDir . '/laminas/laminas-diactoros/src/Response/JsonResponse.php',
    'Laminas\\Diactoros\\Response\\RedirectResponse' => $vendorDir . '/laminas/laminas-diactoros/src/Response/RedirectResponse.php',
    'Laminas\\Diactoros\\Response\\Serializer' => $vendorDir . '/laminas/laminas-diactoros/src/Response/Serializer.php',
    'Laminas\\Diactoros\\Response\\TextResponse' => $vendorDir . '/laminas/laminas-diactoros/src/Response/TextResponse.php',
    'Laminas\\Diactoros\\Response\\XmlResponse' => $vendorDir . '/laminas/laminas-diactoros/src/Response/XmlResponse.php',
    'Laminas\\Diactoros\\ServerRequest' => $vendorDir . '/laminas/laminas-diactoros/src/ServerRequest.php',
    'Laminas\\Diactoros\\ServerRequestFactory' => $vendorDir . '/laminas/laminas-diactoros/src/ServerRequestFactory.php',
    'Laminas\\Diactoros\\ServerRequestFilter\\DoNotFilter' => $vendorDir . '/laminas/laminas-diactoros/src/ServerRequestFilter/DoNotFilter.php',
    'Laminas\\Diactoros\\ServerRequestFilter\\FilterServerRequestInterface' => $vendorDir . '/laminas/laminas-diactoros/src/ServerRequestFilter/FilterServerRequestInterface.php',
    'Laminas\\Diactoros\\ServerRequestFilter\\FilterUsingXForwardedHeaders' => $vendorDir . '/laminas/laminas-diactoros/src/ServerRequestFilter/FilterUsingXForwardedHeaders.php',
    'Laminas\\Diactoros\\ServerRequestFilter\\IPRange' => $vendorDir . '/laminas/laminas-diactoros/src/ServerRequestFilter/IPRange.php',
    'Laminas\\Diactoros\\Stream' => $vendorDir . '/laminas/laminas-diactoros/src/Stream.php',
    'Laminas\\Diactoros\\StreamFactory' => $vendorDir . '/laminas/laminas-diactoros/src/StreamFactory.php',
    'Laminas\\Diactoros\\UploadedFile' => $vendorDir . '/laminas/laminas-diactoros/src/UploadedFile.php',
    'Laminas\\Diactoros\\UploadedFileFactory' => $vendorDir . '/laminas/laminas-diactoros/src/UploadedFileFactory.php',
    'Laminas\\Diactoros\\Uri' => $vendorDir . '/laminas/laminas-diactoros/src/Uri.php',
    'Laminas\\Diactoros\\UriFactory' => $vendorDir . '/laminas/laminas-diactoros/src/UriFactory.php',
    'Laramin\\Utility\\Controller\\UtilityController' => $vendorDir . '/laramin/utility/src/Controller/UtilityController.php',
    'Laramin\\Utility\\GoToCore' => $vendorDir . '/laramin/utility/src/GoToCore.php',
    'Laramin\\Utility\\Helpmate' => $vendorDir . '/laramin/utility/src/Helpmate.php',
    'Laramin\\Utility\\Onumoti' => $vendorDir . '/laramin/utility/src/Onumoti.php',
    'Laramin\\Utility\\Utility' => $vendorDir . '/laramin/utility/src/Utility.php',
    'Laramin\\Utility\\UtilityServiceProvider' => $vendorDir . '/laramin/utility/src/UtilityServiceProvider.php',
    'Laramin\\Utility\\VugiChugi' => $vendorDir . '/laramin/utility/src/VugiChugi.php',
    'Laravel\\Prompts\\Concerns\\Colors' => $vendorDir . '/laravel/prompts/src/Concerns/Colors.php',
    'Laravel\\Prompts\\Concerns\\Cursor' => $vendorDir . '/laravel/prompts/src/Concerns/Cursor.php',
    'Laravel\\Prompts\\Concerns\\Erase' => $vendorDir . '/laravel/prompts/src/Concerns/Erase.php',
    'Laravel\\Prompts\\Concerns\\Events' => $vendorDir . '/laravel/prompts/src/Concerns/Events.php',
    'Laravel\\Prompts\\Concerns\\FakesInputOutput' => $vendorDir . '/laravel/prompts/src/Concerns/FakesInputOutput.php',
    'Laravel\\Prompts\\Concerns\\Fallback' => $vendorDir . '/laravel/prompts/src/Concerns/Fallback.php',
    'Laravel\\Prompts\\Concerns\\Interactivity' => $vendorDir . '/laravel/prompts/src/Concerns/Interactivity.php',
    'Laravel\\Prompts\\Concerns\\Scrolling' => $vendorDir . '/laravel/prompts/src/Concerns/Scrolling.php',
    'Laravel\\Prompts\\Concerns\\Termwind' => $vendorDir . '/laravel/prompts/src/Concerns/Termwind.php',
    'Laravel\\Prompts\\Concerns\\Themes' => $vendorDir . '/laravel/prompts/src/Concerns/Themes.php',
    'Laravel\\Prompts\\Concerns\\Truncation' => $vendorDir . '/laravel/prompts/src/Concerns/Truncation.php',
    'Laravel\\Prompts\\Concerns\\TypedValue' => $vendorDir . '/laravel/prompts/src/Concerns/TypedValue.php',
    'Laravel\\Prompts\\ConfirmPrompt' => $vendorDir . '/laravel/prompts/src/ConfirmPrompt.php',
    'Laravel\\Prompts\\Exceptions\\FormRevertedException' => $vendorDir . '/laravel/prompts/src/Exceptions/FormRevertedException.php',
    'Laravel\\Prompts\\Exceptions\\NonInteractiveValidationException' => $vendorDir . '/laravel/prompts/src/Exceptions/NonInteractiveValidationException.php',
    'Laravel\\Prompts\\FormBuilder' => $vendorDir . '/laravel/prompts/src/FormBuilder.php',
    'Laravel\\Prompts\\FormStep' => $vendorDir . '/laravel/prompts/src/FormStep.php',
    'Laravel\\Prompts\\Key' => $vendorDir . '/laravel/prompts/src/Key.php',
    'Laravel\\Prompts\\MultiSearchPrompt' => $vendorDir . '/laravel/prompts/src/MultiSearchPrompt.php',
    'Laravel\\Prompts\\MultiSelectPrompt' => $vendorDir . '/laravel/prompts/src/MultiSelectPrompt.php',
    'Laravel\\Prompts\\Note' => $vendorDir . '/laravel/prompts/src/Note.php',
    'Laravel\\Prompts\\Output\\BufferedConsoleOutput' => $vendorDir . '/laravel/prompts/src/Output/BufferedConsoleOutput.php',
    'Laravel\\Prompts\\Output\\ConsoleOutput' => $vendorDir . '/laravel/prompts/src/Output/ConsoleOutput.php',
    'Laravel\\Prompts\\PasswordPrompt' => $vendorDir . '/laravel/prompts/src/PasswordPrompt.php',
    'Laravel\\Prompts\\PausePrompt' => $vendorDir . '/laravel/prompts/src/PausePrompt.php',
    'Laravel\\Prompts\\Progress' => $vendorDir . '/laravel/prompts/src/Progress.php',
    'Laravel\\Prompts\\Prompt' => $vendorDir . '/laravel/prompts/src/Prompt.php',
    'Laravel\\Prompts\\SearchPrompt' => $vendorDir . '/laravel/prompts/src/SearchPrompt.php',
    'Laravel\\Prompts\\SelectPrompt' => $vendorDir . '/laravel/prompts/src/SelectPrompt.php',
    'Laravel\\Prompts\\Spinner' => $vendorDir . '/laravel/prompts/src/Spinner.php',
    'Laravel\\Prompts\\SuggestPrompt' => $vendorDir . '/laravel/prompts/src/SuggestPrompt.php',
    'Laravel\\Prompts\\Table' => $vendorDir . '/laravel/prompts/src/Table.php',
    'Laravel\\Prompts\\Terminal' => $vendorDir . '/laravel/prompts/src/Terminal.php',
    'Laravel\\Prompts\\TextPrompt' => $vendorDir . '/laravel/prompts/src/TextPrompt.php',
    'Laravel\\Prompts\\TextareaPrompt' => $vendorDir . '/laravel/prompts/src/TextareaPrompt.php',
    'Laravel\\Prompts\\Themes\\Contracts\\Scrolling' => $vendorDir . '/laravel/prompts/src/Themes/Contracts/Scrolling.php',
    'Laravel\\Prompts\\Themes\\Default\\Concerns\\DrawsBoxes' => $vendorDir . '/laravel/prompts/src/Themes/Default/Concerns/DrawsBoxes.php',
    'Laravel\\Prompts\\Themes\\Default\\Concerns\\DrawsScrollbars' => $vendorDir . '/laravel/prompts/src/Themes/Default/Concerns/DrawsScrollbars.php',
    'Laravel\\Prompts\\Themes\\Default\\Concerns\\InteractsWithStrings' => $vendorDir . '/laravel/prompts/src/Themes/Default/Concerns/InteractsWithStrings.php',
    'Laravel\\Prompts\\Themes\\Default\\ConfirmPromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/ConfirmPromptRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\MultiSearchPromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/MultiSearchPromptRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\MultiSelectPromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/MultiSelectPromptRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\NoteRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/NoteRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\PasswordPromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/PasswordPromptRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\PausePromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/PausePromptRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\ProgressRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/ProgressRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\Renderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/Renderer.php',
    'Laravel\\Prompts\\Themes\\Default\\SearchPromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/SearchPromptRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\SelectPromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/SelectPromptRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\SpinnerRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/SpinnerRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\SuggestPromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/SuggestPromptRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\TableRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/TableRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\TextPromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/TextPromptRenderer.php',
    'Laravel\\Prompts\\Themes\\Default\\TextareaPromptRenderer' => $vendorDir . '/laravel/prompts/src/Themes/Default/TextareaPromptRenderer.php',
    'Laravel\\Sail\\Console\\AddCommand' => $vendorDir . '/laravel/sail/src/Console/AddCommand.php',
    'Laravel\\Sail\\Console\\Concerns\\InteractsWithDockerComposeServices' => $vendorDir . '/laravel/sail/src/Console/Concerns/InteractsWithDockerComposeServices.php',
    'Laravel\\Sail\\Console\\InstallCommand' => $vendorDir . '/laravel/sail/src/Console/InstallCommand.php',
    'Laravel\\Sail\\Console\\PublishCommand' => $vendorDir . '/laravel/sail/src/Console/PublishCommand.php',
    'Laravel\\Sail\\SailServiceProvider' => $vendorDir . '/laravel/sail/src/SailServiceProvider.php',
    'Laravel\\Sanctum\\Console\\Commands\\PruneExpired' => $vendorDir . '/laravel/sanctum/src/Console/Commands/PruneExpired.php',
    'Laravel\\Sanctum\\Contracts\\HasAbilities' => $vendorDir . '/laravel/sanctum/src/Contracts/HasAbilities.php',
    'Laravel\\Sanctum\\Contracts\\HasApiTokens' => $vendorDir . '/laravel/sanctum/src/Contracts/HasApiTokens.php',
    'Laravel\\Sanctum\\Events\\TokenAuthenticated' => $vendorDir . '/laravel/sanctum/src/Events/TokenAuthenticated.php',
    'Laravel\\Sanctum\\Exceptions\\MissingAbilityException' => $vendorDir . '/laravel/sanctum/src/Exceptions/MissingAbilityException.php',
    'Laravel\\Sanctum\\Exceptions\\MissingScopeException' => $vendorDir . '/laravel/sanctum/src/Exceptions/MissingScopeException.php',
    'Laravel\\Sanctum\\Guard' => $vendorDir . '/laravel/sanctum/src/Guard.php',
    'Laravel\\Sanctum\\HasApiTokens' => $vendorDir . '/laravel/sanctum/src/HasApiTokens.php',
    'Laravel\\Sanctum\\Http\\Controllers\\CsrfCookieController' => $vendorDir . '/laravel/sanctum/src/Http/Controllers/CsrfCookieController.php',
    'Laravel\\Sanctum\\Http\\Middleware\\AuthenticateSession' => $vendorDir . '/laravel/sanctum/src/Http/Middleware/AuthenticateSession.php',
    'Laravel\\Sanctum\\Http\\Middleware\\CheckAbilities' => $vendorDir . '/laravel/sanctum/src/Http/Middleware/CheckAbilities.php',
    'Laravel\\Sanctum\\Http\\Middleware\\CheckForAnyAbility' => $vendorDir . '/laravel/sanctum/src/Http/Middleware/CheckForAnyAbility.php',
    'Laravel\\Sanctum\\Http\\Middleware\\CheckForAnyScope' => $vendorDir . '/laravel/sanctum/src/Http/Middleware/CheckForAnyScope.php',
    'Laravel\\Sanctum\\Http\\Middleware\\CheckScopes' => $vendorDir . '/laravel/sanctum/src/Http/Middleware/CheckScopes.php',
    'Laravel\\Sanctum\\Http\\Middleware\\EnsureFrontendRequestsAreStateful' => $vendorDir . '/laravel/sanctum/src/Http/Middleware/EnsureFrontendRequestsAreStateful.php',
    'Laravel\\Sanctum\\NewAccessToken' => $vendorDir . '/laravel/sanctum/src/NewAccessToken.php',
    'Laravel\\Sanctum\\PersonalAccessToken' => $vendorDir . '/laravel/sanctum/src/PersonalAccessToken.php',
    'Laravel\\Sanctum\\Sanctum' => $vendorDir . '/laravel/sanctum/src/Sanctum.php',
    'Laravel\\Sanctum\\SanctumServiceProvider' => $vendorDir . '/laravel/sanctum/src/SanctumServiceProvider.php',
    'Laravel\\Sanctum\\TransientToken' => $vendorDir . '/laravel/sanctum/src/TransientToken.php',
    'Laravel\\SerializableClosure\\Contracts\\Serializable' => $vendorDir . '/laravel/serializable-closure/src/Contracts/Serializable.php',
    'Laravel\\SerializableClosure\\Contracts\\Signer' => $vendorDir . '/laravel/serializable-closure/src/Contracts/Signer.php',
    'Laravel\\SerializableClosure\\Exceptions\\InvalidSignatureException' => $vendorDir . '/laravel/serializable-closure/src/Exceptions/InvalidSignatureException.php',
    'Laravel\\SerializableClosure\\Exceptions\\MissingSecretKeyException' => $vendorDir . '/laravel/serializable-closure/src/Exceptions/MissingSecretKeyException.php',
    'Laravel\\SerializableClosure\\Exceptions\\PhpVersionNotSupportedException' => $vendorDir . '/laravel/serializable-closure/src/Exceptions/PhpVersionNotSupportedException.php',
    'Laravel\\SerializableClosure\\SerializableClosure' => $vendorDir . '/laravel/serializable-closure/src/SerializableClosure.php',
    'Laravel\\SerializableClosure\\Serializers\\Native' => $vendorDir . '/laravel/serializable-closure/src/Serializers/Native.php',
    'Laravel\\SerializableClosure\\Serializers\\Signed' => $vendorDir . '/laravel/serializable-closure/src/Serializers/Signed.php',
    'Laravel\\SerializableClosure\\Signers\\Hmac' => $vendorDir . '/laravel/serializable-closure/src/Signers/Hmac.php',
    'Laravel\\SerializableClosure\\Support\\ClosureScope' => $vendorDir . '/laravel/serializable-closure/src/Support/ClosureScope.php',
    'Laravel\\SerializableClosure\\Support\\ClosureStream' => $vendorDir . '/laravel/serializable-closure/src/Support/ClosureStream.php',
    'Laravel\\SerializableClosure\\Support\\ReflectionClosure' => $vendorDir . '/laravel/serializable-closure/src/Support/ReflectionClosure.php',
    'Laravel\\SerializableClosure\\Support\\SelfReference' => $vendorDir . '/laravel/serializable-closure/src/Support/SelfReference.php',
    'Laravel\\SerializableClosure\\UnsignedSerializableClosure' => $vendorDir . '/laravel/serializable-closure/src/UnsignedSerializableClosure.php',
    'Laravel\\Tinker\\ClassAliasAutoloader' => $vendorDir . '/laravel/tinker/src/ClassAliasAutoloader.php',
    'Laravel\\Tinker\\Console\\TinkerCommand' => $vendorDir . '/laravel/tinker/src/Console/TinkerCommand.php',
    'Laravel\\Tinker\\TinkerCaster' => $vendorDir . '/laravel/tinker/src/TinkerCaster.php',
    'Laravel\\Tinker\\TinkerServiceProvider' => $vendorDir . '/laravel/tinker/src/TinkerServiceProvider.php',
    'Laravel\\Ui\\AuthCommand' => $vendorDir . '/laravel/ui/src/AuthCommand.php',
    'Laravel\\Ui\\AuthRouteMethods' => $vendorDir . '/laravel/ui/src/AuthRouteMethods.php',
    'Laravel\\Ui\\ControllersCommand' => $vendorDir . '/laravel/ui/src/ControllersCommand.php',
    'Laravel\\Ui\\Presets\\Bootstrap' => $vendorDir . '/laravel/ui/src/Presets/Bootstrap.php',
    'Laravel\\Ui\\Presets\\Preset' => $vendorDir . '/laravel/ui/src/Presets/Preset.php',
    'Laravel\\Ui\\Presets\\React' => $vendorDir . '/laravel/ui/src/Presets/React.php',
    'Laravel\\Ui\\Presets\\Vue' => $vendorDir . '/laravel/ui/src/Presets/Vue.php',
    'Laravel\\Ui\\UiCommand' => $vendorDir . '/laravel/ui/src/UiCommand.php',
    'Laravel\\Ui\\UiServiceProvider' => $vendorDir . '/laravel/ui/src/UiServiceProvider.php',
    'Lcobucci\\Clock\\Clock' => $vendorDir . '/lcobucci/clock/src/Clock.php',
    'Lcobucci\\Clock\\FrozenClock' => $vendorDir . '/lcobucci/clock/src/FrozenClock.php',
    'Lcobucci\\Clock\\SystemClock' => $vendorDir . '/lcobucci/clock/src/SystemClock.php',
    'Lcobucci\\JWT\\Builder' => $vendorDir . '/lcobucci/jwt/src/Builder.php',
    'Lcobucci\\JWT\\ClaimsFormatter' => $vendorDir . '/lcobucci/jwt/src/ClaimsFormatter.php',
    'Lcobucci\\JWT\\Configuration' => $vendorDir . '/lcobucci/jwt/src/Configuration.php',
    'Lcobucci\\JWT\\Decoder' => $vendorDir . '/lcobucci/jwt/src/Decoder.php',
    'Lcobucci\\JWT\\Encoder' => $vendorDir . '/lcobucci/jwt/src/Encoder.php',
    'Lcobucci\\JWT\\Encoding\\CannotDecodeContent' => $vendorDir . '/lcobucci/jwt/src/Encoding/CannotDecodeContent.php',
    'Lcobucci\\JWT\\Encoding\\CannotEncodeContent' => $vendorDir . '/lcobucci/jwt/src/Encoding/CannotEncodeContent.php',
    'Lcobucci\\JWT\\Encoding\\ChainedFormatter' => $vendorDir . '/lcobucci/jwt/src/Encoding/ChainedFormatter.php',
    'Lcobucci\\JWT\\Encoding\\JoseEncoder' => $vendorDir . '/lcobucci/jwt/src/Encoding/JoseEncoder.php',
    'Lcobucci\\JWT\\Encoding\\MicrosecondBasedDateConversion' => $vendorDir . '/lcobucci/jwt/src/Encoding/MicrosecondBasedDateConversion.php',
    'Lcobucci\\JWT\\Encoding\\UnifyAudience' => $vendorDir . '/lcobucci/jwt/src/Encoding/UnifyAudience.php',
    'Lcobucci\\JWT\\Exception' => $vendorDir . '/lcobucci/jwt/src/Exception.php',
    'Lcobucci\\JWT\\Parser' => $vendorDir . '/lcobucci/jwt/src/Parser.php',
    'Lcobucci\\JWT\\Signer' => $vendorDir . '/lcobucci/jwt/src/Signer.php',
    'Lcobucci\\JWT\\Signer\\CannotSignPayload' => $vendorDir . '/lcobucci/jwt/src/Signer/CannotSignPayload.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\ConversionFailed' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/ConversionFailed.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\MultibyteStringConverter' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/MultibyteStringConverter.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\Sha256' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/Sha256.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\Sha384' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/Sha384.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\Sha512' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/Sha512.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\SignatureConverter' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/SignatureConverter.php',
    'Lcobucci\\JWT\\Signer\\Hmac' => $vendorDir . '/lcobucci/jwt/src/Signer/Hmac.php',
    'Lcobucci\\JWT\\Signer\\Hmac\\Sha256' => $vendorDir . '/lcobucci/jwt/src/Signer/Hmac/Sha256.php',
    'Lcobucci\\JWT\\Signer\\Hmac\\Sha384' => $vendorDir . '/lcobucci/jwt/src/Signer/Hmac/Sha384.php',
    'Lcobucci\\JWT\\Signer\\Hmac\\Sha512' => $vendorDir . '/lcobucci/jwt/src/Signer/Hmac/Sha512.php',
    'Lcobucci\\JWT\\Signer\\InvalidKeyProvided' => $vendorDir . '/lcobucci/jwt/src/Signer/InvalidKeyProvided.php',
    'Lcobucci\\JWT\\Signer\\Key' => $vendorDir . '/lcobucci/jwt/src/Signer/Key.php',
    'Lcobucci\\JWT\\Signer\\Key\\FileCouldNotBeRead' => $vendorDir . '/lcobucci/jwt/src/Signer/Key/FileCouldNotBeRead.php',
    'Lcobucci\\JWT\\Signer\\Key\\InMemory' => $vendorDir . '/lcobucci/jwt/src/Signer/Key/InMemory.php',
    'Lcobucci\\JWT\\Signer\\Key\\LocalFileReference' => $vendorDir . '/lcobucci/jwt/src/Signer/Key/LocalFileReference.php',
    'Lcobucci\\JWT\\Signer\\None' => $vendorDir . '/lcobucci/jwt/src/Signer/None.php',
    'Lcobucci\\JWT\\Signer\\OpenSSL' => $vendorDir . '/lcobucci/jwt/src/Signer/OpenSSL.php',
    'Lcobucci\\JWT\\Signer\\Rsa' => $vendorDir . '/lcobucci/jwt/src/Signer/Rsa.php',
    'Lcobucci\\JWT\\Signer\\Rsa\\Sha256' => $vendorDir . '/lcobucci/jwt/src/Signer/Rsa/Sha256.php',
    'Lcobucci\\JWT\\Signer\\Rsa\\Sha384' => $vendorDir . '/lcobucci/jwt/src/Signer/Rsa/Sha384.php',
    'Lcobucci\\JWT\\Signer\\Rsa\\Sha512' => $vendorDir . '/lcobucci/jwt/src/Signer/Rsa/Sha512.php',
    'Lcobucci\\JWT\\Token' => $vendorDir . '/lcobucci/jwt/src/Token.php',
    'Lcobucci\\JWT\\Token\\Builder' => $vendorDir . '/lcobucci/jwt/src/Token/Builder.php',
    'Lcobucci\\JWT\\Token\\DataSet' => $vendorDir . '/lcobucci/jwt/src/Token/DataSet.php',
    'Lcobucci\\JWT\\Token\\InvalidTokenStructure' => $vendorDir . '/lcobucci/jwt/src/Token/InvalidTokenStructure.php',
    'Lcobucci\\JWT\\Token\\Parser' => $vendorDir . '/lcobucci/jwt/src/Token/Parser.php',
    'Lcobucci\\JWT\\Token\\Plain' => $vendorDir . '/lcobucci/jwt/src/Token/Plain.php',
    'Lcobucci\\JWT\\Token\\RegisteredClaimGiven' => $vendorDir . '/lcobucci/jwt/src/Token/RegisteredClaimGiven.php',
    'Lcobucci\\JWT\\Token\\RegisteredClaims' => $vendorDir . '/lcobucci/jwt/src/Token/RegisteredClaims.php',
    'Lcobucci\\JWT\\Token\\Signature' => $vendorDir . '/lcobucci/jwt/src/Token/Signature.php',
    'Lcobucci\\JWT\\Token\\UnsupportedHeaderFound' => $vendorDir . '/lcobucci/jwt/src/Token/UnsupportedHeaderFound.php',
    'Lcobucci\\JWT\\Validation\\Constraint' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint.php',
    'Lcobucci\\JWT\\Validation\\ConstraintViolation' => $vendorDir . '/lcobucci/jwt/src/Validation/ConstraintViolation.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\IdentifiedBy' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/IdentifiedBy.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\IssuedBy' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/IssuedBy.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\LeewayCannotBeNegative' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/LeewayCannotBeNegative.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\PermittedFor' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/PermittedFor.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\RelatedTo' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/RelatedTo.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\SignedWith' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/SignedWith.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\ValidAt' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/ValidAt.php',
    'Lcobucci\\JWT\\Validation\\NoConstraintsGiven' => $vendorDir . '/lcobucci/jwt/src/Validation/NoConstraintsGiven.php',
    'Lcobucci\\JWT\\Validation\\RequiredConstraintsViolated' => $vendorDir . '/lcobucci/jwt/src/Validation/RequiredConstraintsViolated.php',
    'Lcobucci\\JWT\\Validation\\Validator' => $vendorDir . '/lcobucci/jwt/src/Validation/Validator.php',
    'Lcobucci\\JWT\\Validator' => $vendorDir . '/lcobucci/jwt/src/Validator.php',
    'League\\CommonMark\\CommonMarkConverter' => $vendorDir . '/league/commonmark/src/CommonMarkConverter.php',
    'League\\CommonMark\\ConverterInterface' => $vendorDir . '/league/commonmark/src/ConverterInterface.php',
    'League\\CommonMark\\Delimiter\\Bracket' => $vendorDir . '/league/commonmark/src/Delimiter/Bracket.php',
    'League\\CommonMark\\Delimiter\\Delimiter' => $vendorDir . '/league/commonmark/src/Delimiter/Delimiter.php',
    'League\\CommonMark\\Delimiter\\DelimiterInterface' => $vendorDir . '/league/commonmark/src/Delimiter/DelimiterInterface.php',
    'League\\CommonMark\\Delimiter\\DelimiterParser' => $vendorDir . '/league/commonmark/src/Delimiter/DelimiterParser.php',
    'League\\CommonMark\\Delimiter\\DelimiterStack' => $vendorDir . '/league/commonmark/src/Delimiter/DelimiterStack.php',
    'League\\CommonMark\\Delimiter\\Processor\\CacheableDelimiterProcessorInterface' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/CacheableDelimiterProcessorInterface.php',
    'League\\CommonMark\\Delimiter\\Processor\\DelimiterProcessorCollection' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/DelimiterProcessorCollection.php',
    'League\\CommonMark\\Delimiter\\Processor\\DelimiterProcessorCollectionInterface' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/DelimiterProcessorCollectionInterface.php',
    'League\\CommonMark\\Delimiter\\Processor\\DelimiterProcessorInterface' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/DelimiterProcessorInterface.php',
    'League\\CommonMark\\Delimiter\\Processor\\StaggeredDelimiterProcessor' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/StaggeredDelimiterProcessor.php',
    'League\\CommonMark\\Environment\\Environment' => $vendorDir . '/league/commonmark/src/Environment/Environment.php',
    'League\\CommonMark\\Environment\\EnvironmentAwareInterface' => $vendorDir . '/league/commonmark/src/Environment/EnvironmentAwareInterface.php',
    'League\\CommonMark\\Environment\\EnvironmentBuilderInterface' => $vendorDir . '/league/commonmark/src/Environment/EnvironmentBuilderInterface.php',
    'League\\CommonMark\\Environment\\EnvironmentInterface' => $vendorDir . '/league/commonmark/src/Environment/EnvironmentInterface.php',
    'League\\CommonMark\\Event\\AbstractEvent' => $vendorDir . '/league/commonmark/src/Event/AbstractEvent.php',
    'League\\CommonMark\\Event\\DocumentParsedEvent' => $vendorDir . '/league/commonmark/src/Event/DocumentParsedEvent.php',
    'League\\CommonMark\\Event\\DocumentPreParsedEvent' => $vendorDir . '/league/commonmark/src/Event/DocumentPreParsedEvent.php',
    'League\\CommonMark\\Event\\DocumentPreRenderEvent' => $vendorDir . '/league/commonmark/src/Event/DocumentPreRenderEvent.php',
    'League\\CommonMark\\Event\\DocumentRenderedEvent' => $vendorDir . '/league/commonmark/src/Event/DocumentRenderedEvent.php',
    'League\\CommonMark\\Event\\ListenerData' => $vendorDir . '/league/commonmark/src/Event/ListenerData.php',
    'League\\CommonMark\\Exception\\AlreadyInitializedException' => $vendorDir . '/league/commonmark/src/Exception/AlreadyInitializedException.php',
    'League\\CommonMark\\Exception\\CommonMarkException' => $vendorDir . '/league/commonmark/src/Exception/CommonMarkException.php',
    'League\\CommonMark\\Exception\\IOException' => $vendorDir . '/league/commonmark/src/Exception/IOException.php',
    'League\\CommonMark\\Exception\\InvalidArgumentException' => $vendorDir . '/league/commonmark/src/Exception/InvalidArgumentException.php',
    'League\\CommonMark\\Exception\\LogicException' => $vendorDir . '/league/commonmark/src/Exception/LogicException.php',
    'League\\CommonMark\\Exception\\MissingDependencyException' => $vendorDir . '/league/commonmark/src/Exception/MissingDependencyException.php',
    'League\\CommonMark\\Exception\\UnexpectedEncodingException' => $vendorDir . '/league/commonmark/src/Exception/UnexpectedEncodingException.php',
    'League\\CommonMark\\Extension\\Attributes\\AttributesExtension' => $vendorDir . '/league/commonmark/src/Extension/Attributes/AttributesExtension.php',
    'League\\CommonMark\\Extension\\Attributes\\Event\\AttributesListener' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Event/AttributesListener.php',
    'League\\CommonMark\\Extension\\Attributes\\Node\\Attributes' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Node/Attributes.php',
    'League\\CommonMark\\Extension\\Attributes\\Node\\AttributesInline' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Node/AttributesInline.php',
    'League\\CommonMark\\Extension\\Attributes\\Parser\\AttributesBlockContinueParser' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Parser/AttributesBlockContinueParser.php',
    'League\\CommonMark\\Extension\\Attributes\\Parser\\AttributesBlockStartParser' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Parser/AttributesBlockStartParser.php',
    'League\\CommonMark\\Extension\\Attributes\\Parser\\AttributesInlineParser' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Parser/AttributesInlineParser.php',
    'League\\CommonMark\\Extension\\Attributes\\Util\\AttributesHelper' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Util/AttributesHelper.php',
    'League\\CommonMark\\Extension\\Autolink\\AutolinkExtension' => $vendorDir . '/league/commonmark/src/Extension/Autolink/AutolinkExtension.php',
    'League\\CommonMark\\Extension\\Autolink\\EmailAutolinkParser' => $vendorDir . '/league/commonmark/src/Extension/Autolink/EmailAutolinkParser.php',
    'League\\CommonMark\\Extension\\Autolink\\UrlAutolinkParser' => $vendorDir . '/league/commonmark/src/Extension/Autolink/UrlAutolinkParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\CommonMarkCoreExtension' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/CommonMarkCoreExtension.php',
    'League\\CommonMark\\Extension\\CommonMark\\Delimiter\\Processor\\EmphasisDelimiterProcessor' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Delimiter/Processor/EmphasisDelimiterProcessor.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\BlockQuote' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/BlockQuote.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\FencedCode' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/FencedCode.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\Heading' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/Heading.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\HtmlBlock' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/HtmlBlock.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\IndentedCode' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/IndentedCode.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\ListBlock' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/ListBlock.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\ListData' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/ListData.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\ListItem' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/ListItem.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\ThematicBreak' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/ThematicBreak.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\AbstractWebResource' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/AbstractWebResource.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Code' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Code.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Emphasis' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Emphasis.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\HtmlInline' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/HtmlInline.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Image' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Image.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Link' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Link.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Strong' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Strong.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\BlockQuoteParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/BlockQuoteParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\BlockQuoteStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/BlockQuoteStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\FencedCodeParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/FencedCodeParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\FencedCodeStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/FencedCodeStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\HeadingParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/HeadingParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\HeadingStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/HeadingStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\HtmlBlockParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/HtmlBlockParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\HtmlBlockStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/HtmlBlockStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\IndentedCodeParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/IndentedCodeParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\IndentedCodeStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/IndentedCodeStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ListBlockParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ListBlockParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ListBlockStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ListBlockStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ListItemParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ListItemParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ThematicBreakParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ThematicBreakParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ThematicBreakStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ThematicBreakStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\AutolinkParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/AutolinkParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\BacktickParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/BacktickParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\BangParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/BangParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\CloseBracketParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/CloseBracketParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\EntityParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/EntityParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\EscapableParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/EscapableParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\HtmlInlineParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/HtmlInlineParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\OpenBracketParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/OpenBracketParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\BlockQuoteRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/BlockQuoteRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\FencedCodeRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/FencedCodeRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\HeadingRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/HeadingRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\HtmlBlockRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/HtmlBlockRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\IndentedCodeRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/IndentedCodeRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\ListBlockRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/ListBlockRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\ListItemRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/ListItemRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\ThematicBreakRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/ThematicBreakRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\CodeRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/CodeRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\EmphasisRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/EmphasisRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\HtmlInlineRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/HtmlInlineRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\ImageRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/ImageRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\LinkRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/LinkRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\StrongRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/StrongRenderer.php',
    'League\\CommonMark\\Extension\\ConfigurableExtensionInterface' => $vendorDir . '/league/commonmark/src/Extension/ConfigurableExtensionInterface.php',
    'League\\CommonMark\\Extension\\DefaultAttributes\\ApplyDefaultAttributesProcessor' => $vendorDir . '/league/commonmark/src/Extension/DefaultAttributes/ApplyDefaultAttributesProcessor.php',
    'League\\CommonMark\\Extension\\DefaultAttributes\\DefaultAttributesExtension' => $vendorDir . '/league/commonmark/src/Extension/DefaultAttributes/DefaultAttributesExtension.php',
    'League\\CommonMark\\Extension\\DescriptionList\\DescriptionListExtension' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/DescriptionListExtension.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Event\\ConsecutiveDescriptionListMerger' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Event/ConsecutiveDescriptionListMerger.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Event\\LooseDescriptionHandler' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Event/LooseDescriptionHandler.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Node\\Description' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Node/Description.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Node\\DescriptionList' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Node/DescriptionList.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Node\\DescriptionTerm' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Node/DescriptionTerm.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Parser\\DescriptionContinueParser' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Parser/DescriptionContinueParser.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Parser\\DescriptionListContinueParser' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Parser/DescriptionListContinueParser.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Parser\\DescriptionStartParser' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Parser/DescriptionStartParser.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Parser\\DescriptionTermContinueParser' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Parser/DescriptionTermContinueParser.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Renderer\\DescriptionListRenderer' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Renderer/DescriptionListRenderer.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Renderer\\DescriptionRenderer' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Renderer/DescriptionRenderer.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Renderer\\DescriptionTermRenderer' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Renderer/DescriptionTermRenderer.php',
    'League\\CommonMark\\Extension\\DisallowedRawHtml\\DisallowedRawHtmlExtension' => $vendorDir . '/league/commonmark/src/Extension/DisallowedRawHtml/DisallowedRawHtmlExtension.php',
    'League\\CommonMark\\Extension\\DisallowedRawHtml\\DisallowedRawHtmlRenderer' => $vendorDir . '/league/commonmark/src/Extension/DisallowedRawHtml/DisallowedRawHtmlRenderer.php',
    'League\\CommonMark\\Extension\\Embed\\Bridge\\OscaroteroEmbedAdapter' => $vendorDir . '/league/commonmark/src/Extension/Embed/Bridge/OscaroteroEmbedAdapter.php',
    'League\\CommonMark\\Extension\\Embed\\DomainFilteringAdapter' => $vendorDir . '/league/commonmark/src/Extension/Embed/DomainFilteringAdapter.php',
    'League\\CommonMark\\Extension\\Embed\\Embed' => $vendorDir . '/league/commonmark/src/Extension/Embed/Embed.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedAdapterInterface' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedAdapterInterface.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedExtension' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedExtension.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedParser' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedParser.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedProcessor' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedProcessor.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedRenderer' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedRenderer.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedStartParser' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedStartParser.php',
    'League\\CommonMark\\Extension\\ExtensionInterface' => $vendorDir . '/league/commonmark/src/Extension/ExtensionInterface.php',
    'League\\CommonMark\\Extension\\ExternalLink\\ExternalLinkExtension' => $vendorDir . '/league/commonmark/src/Extension/ExternalLink/ExternalLinkExtension.php',
    'League\\CommonMark\\Extension\\ExternalLink\\ExternalLinkProcessor' => $vendorDir . '/league/commonmark/src/Extension/ExternalLink/ExternalLinkProcessor.php',
    'League\\CommonMark\\Extension\\Footnote\\Event\\AnonymousFootnotesListener' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Event/AnonymousFootnotesListener.php',
    'League\\CommonMark\\Extension\\Footnote\\Event\\FixOrphanedFootnotesAndRefsListener' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Event/FixOrphanedFootnotesAndRefsListener.php',
    'League\\CommonMark\\Extension\\Footnote\\Event\\GatherFootnotesListener' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Event/GatherFootnotesListener.php',
    'League\\CommonMark\\Extension\\Footnote\\Event\\NumberFootnotesListener' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Event/NumberFootnotesListener.php',
    'League\\CommonMark\\Extension\\Footnote\\FootnoteExtension' => $vendorDir . '/league/commonmark/src/Extension/Footnote/FootnoteExtension.php',
    'League\\CommonMark\\Extension\\Footnote\\Node\\Footnote' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Node/Footnote.php',
    'League\\CommonMark\\Extension\\Footnote\\Node\\FootnoteBackref' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Node/FootnoteBackref.php',
    'League\\CommonMark\\Extension\\Footnote\\Node\\FootnoteContainer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Node/FootnoteContainer.php',
    'League\\CommonMark\\Extension\\Footnote\\Node\\FootnoteRef' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Node/FootnoteRef.php',
    'League\\CommonMark\\Extension\\Footnote\\Parser\\AnonymousFootnoteRefParser' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Parser/AnonymousFootnoteRefParser.php',
    'League\\CommonMark\\Extension\\Footnote\\Parser\\FootnoteParser' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Parser/FootnoteParser.php',
    'League\\CommonMark\\Extension\\Footnote\\Parser\\FootnoteRefParser' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Parser/FootnoteRefParser.php',
    'League\\CommonMark\\Extension\\Footnote\\Parser\\FootnoteStartParser' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Parser/FootnoteStartParser.php',
    'League\\CommonMark\\Extension\\Footnote\\Renderer\\FootnoteBackrefRenderer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Renderer/FootnoteBackrefRenderer.php',
    'League\\CommonMark\\Extension\\Footnote\\Renderer\\FootnoteContainerRenderer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Renderer/FootnoteContainerRenderer.php',
    'League\\CommonMark\\Extension\\Footnote\\Renderer\\FootnoteRefRenderer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Renderer/FootnoteRefRenderer.php',
    'League\\CommonMark\\Extension\\Footnote\\Renderer\\FootnoteRenderer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Renderer/FootnoteRenderer.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Data\\FrontMatterDataParserInterface' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Data/FrontMatterDataParserInterface.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Data\\LibYamlFrontMatterParser' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Data/LibYamlFrontMatterParser.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Data\\SymfonyYamlFrontMatterParser' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Data/SymfonyYamlFrontMatterParser.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Exception\\InvalidFrontMatterException' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Exception/InvalidFrontMatterException.php',
    'League\\CommonMark\\Extension\\FrontMatter\\FrontMatterExtension' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/FrontMatterExtension.php',
    'League\\CommonMark\\Extension\\FrontMatter\\FrontMatterParser' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/FrontMatterParser.php',
    'League\\CommonMark\\Extension\\FrontMatter\\FrontMatterParserInterface' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/FrontMatterParserInterface.php',
    'League\\CommonMark\\Extension\\FrontMatter\\FrontMatterProviderInterface' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/FrontMatterProviderInterface.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Input\\MarkdownInputWithFrontMatter' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Input/MarkdownInputWithFrontMatter.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Listener\\FrontMatterPostRenderListener' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Listener/FrontMatterPostRenderListener.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Listener\\FrontMatterPreParser' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Listener/FrontMatterPreParser.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Output\\RenderedContentWithFrontMatter' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Output/RenderedContentWithFrontMatter.php',
    'League\\CommonMark\\Extension\\GithubFlavoredMarkdownExtension' => $vendorDir . '/league/commonmark/src/Extension/GithubFlavoredMarkdownExtension.php',
    'League\\CommonMark\\Extension\\HeadingPermalink\\HeadingPermalink' => $vendorDir . '/league/commonmark/src/Extension/HeadingPermalink/HeadingPermalink.php',
    'League\\CommonMark\\Extension\\HeadingPermalink\\HeadingPermalinkExtension' => $vendorDir . '/league/commonmark/src/Extension/HeadingPermalink/HeadingPermalinkExtension.php',
    'League\\CommonMark\\Extension\\HeadingPermalink\\HeadingPermalinkProcessor' => $vendorDir . '/league/commonmark/src/Extension/HeadingPermalink/HeadingPermalinkProcessor.php',
    'League\\CommonMark\\Extension\\HeadingPermalink\\HeadingPermalinkRenderer' => $vendorDir . '/league/commonmark/src/Extension/HeadingPermalink/HeadingPermalinkRenderer.php',
    'League\\CommonMark\\Extension\\InlinesOnly\\ChildRenderer' => $vendorDir . '/league/commonmark/src/Extension/InlinesOnly/ChildRenderer.php',
    'League\\CommonMark\\Extension\\InlinesOnly\\InlinesOnlyExtension' => $vendorDir . '/league/commonmark/src/Extension/InlinesOnly/InlinesOnlyExtension.php',
    'League\\CommonMark\\Extension\\Mention\\Generator\\CallbackGenerator' => $vendorDir . '/league/commonmark/src/Extension/Mention/Generator/CallbackGenerator.php',
    'League\\CommonMark\\Extension\\Mention\\Generator\\MentionGeneratorInterface' => $vendorDir . '/league/commonmark/src/Extension/Mention/Generator/MentionGeneratorInterface.php',
    'League\\CommonMark\\Extension\\Mention\\Generator\\StringTemplateLinkGenerator' => $vendorDir . '/league/commonmark/src/Extension/Mention/Generator/StringTemplateLinkGenerator.php',
    'League\\CommonMark\\Extension\\Mention\\Mention' => $vendorDir . '/league/commonmark/src/Extension/Mention/Mention.php',
    'League\\CommonMark\\Extension\\Mention\\MentionExtension' => $vendorDir . '/league/commonmark/src/Extension/Mention/MentionExtension.php',
    'League\\CommonMark\\Extension\\Mention\\MentionParser' => $vendorDir . '/league/commonmark/src/Extension/Mention/MentionParser.php',
    'League\\CommonMark\\Extension\\SmartPunct\\DashParser' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/DashParser.php',
    'League\\CommonMark\\Extension\\SmartPunct\\EllipsesParser' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/EllipsesParser.php',
    'League\\CommonMark\\Extension\\SmartPunct\\Quote' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/Quote.php',
    'League\\CommonMark\\Extension\\SmartPunct\\QuoteParser' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/QuoteParser.php',
    'League\\CommonMark\\Extension\\SmartPunct\\QuoteProcessor' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/QuoteProcessor.php',
    'League\\CommonMark\\Extension\\SmartPunct\\ReplaceUnpairedQuotesListener' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/ReplaceUnpairedQuotesListener.php',
    'League\\CommonMark\\Extension\\SmartPunct\\SmartPunctExtension' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/SmartPunctExtension.php',
    'League\\CommonMark\\Extension\\Strikethrough\\Strikethrough' => $vendorDir . '/league/commonmark/src/Extension/Strikethrough/Strikethrough.php',
    'League\\CommonMark\\Extension\\Strikethrough\\StrikethroughDelimiterProcessor' => $vendorDir . '/league/commonmark/src/Extension/Strikethrough/StrikethroughDelimiterProcessor.php',
    'League\\CommonMark\\Extension\\Strikethrough\\StrikethroughExtension' => $vendorDir . '/league/commonmark/src/Extension/Strikethrough/StrikethroughExtension.php',
    'League\\CommonMark\\Extension\\Strikethrough\\StrikethroughRenderer' => $vendorDir . '/league/commonmark/src/Extension/Strikethrough/StrikethroughRenderer.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Node\\TableOfContents' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Node/TableOfContents.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Node\\TableOfContentsPlaceholder' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Node/TableOfContentsPlaceholder.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Normalizer\\AsIsNormalizerStrategy' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Normalizer/AsIsNormalizerStrategy.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Normalizer\\FlatNormalizerStrategy' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Normalizer/FlatNormalizerStrategy.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Normalizer\\NormalizerStrategyInterface' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Normalizer/NormalizerStrategyInterface.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Normalizer\\RelativeNormalizerStrategy' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Normalizer/RelativeNormalizerStrategy.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsBuilder' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsBuilder.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsExtension' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsExtension.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsGenerator' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsGenerator.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsGeneratorInterface' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsGeneratorInterface.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsPlaceholderParser' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsPlaceholderParser.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsPlaceholderRenderer' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsPlaceholderRenderer.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsRenderer' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsRenderer.php',
    'League\\CommonMark\\Extension\\Table\\Table' => $vendorDir . '/league/commonmark/src/Extension/Table/Table.php',
    'League\\CommonMark\\Extension\\Table\\TableCell' => $vendorDir . '/league/commonmark/src/Extension/Table/TableCell.php',
    'League\\CommonMark\\Extension\\Table\\TableCellRenderer' => $vendorDir . '/league/commonmark/src/Extension/Table/TableCellRenderer.php',
    'League\\CommonMark\\Extension\\Table\\TableExtension' => $vendorDir . '/league/commonmark/src/Extension/Table/TableExtension.php',
    'League\\CommonMark\\Extension\\Table\\TableParser' => $vendorDir . '/league/commonmark/src/Extension/Table/TableParser.php',
    'League\\CommonMark\\Extension\\Table\\TableRenderer' => $vendorDir . '/league/commonmark/src/Extension/Table/TableRenderer.php',
    'League\\CommonMark\\Extension\\Table\\TableRow' => $vendorDir . '/league/commonmark/src/Extension/Table/TableRow.php',
    'League\\CommonMark\\Extension\\Table\\TableRowRenderer' => $vendorDir . '/league/commonmark/src/Extension/Table/TableRowRenderer.php',
    'League\\CommonMark\\Extension\\Table\\TableSection' => $vendorDir . '/league/commonmark/src/Extension/Table/TableSection.php',
    'League\\CommonMark\\Extension\\Table\\TableSectionRenderer' => $vendorDir . '/league/commonmark/src/Extension/Table/TableSectionRenderer.php',
    'League\\CommonMark\\Extension\\Table\\TableStartParser' => $vendorDir . '/league/commonmark/src/Extension/Table/TableStartParser.php',
    'League\\CommonMark\\Extension\\TaskList\\TaskListExtension' => $vendorDir . '/league/commonmark/src/Extension/TaskList/TaskListExtension.php',
    'League\\CommonMark\\Extension\\TaskList\\TaskListItemMarker' => $vendorDir . '/league/commonmark/src/Extension/TaskList/TaskListItemMarker.php',
    'League\\CommonMark\\Extension\\TaskList\\TaskListItemMarkerParser' => $vendorDir . '/league/commonmark/src/Extension/TaskList/TaskListItemMarkerParser.php',
    'League\\CommonMark\\Extension\\TaskList\\TaskListItemMarkerRenderer' => $vendorDir . '/league/commonmark/src/Extension/TaskList/TaskListItemMarkerRenderer.php',
    'League\\CommonMark\\GithubFlavoredMarkdownConverter' => $vendorDir . '/league/commonmark/src/GithubFlavoredMarkdownConverter.php',
    'League\\CommonMark\\Input\\MarkdownInput' => $vendorDir . '/league/commonmark/src/Input/MarkdownInput.php',
    'League\\CommonMark\\Input\\MarkdownInputInterface' => $vendorDir . '/league/commonmark/src/Input/MarkdownInputInterface.php',
    'League\\CommonMark\\MarkdownConverter' => $vendorDir . '/league/commonmark/src/MarkdownConverter.php',
    'League\\CommonMark\\MarkdownConverterInterface' => $vendorDir . '/league/commonmark/src/MarkdownConverterInterface.php',
    'League\\CommonMark\\Node\\Block\\AbstractBlock' => $vendorDir . '/league/commonmark/src/Node/Block/AbstractBlock.php',
    'League\\CommonMark\\Node\\Block\\Document' => $vendorDir . '/league/commonmark/src/Node/Block/Document.php',
    'League\\CommonMark\\Node\\Block\\Paragraph' => $vendorDir . '/league/commonmark/src/Node/Block/Paragraph.php',
    'League\\CommonMark\\Node\\Block\\TightBlockInterface' => $vendorDir . '/league/commonmark/src/Node/Block/TightBlockInterface.php',
    'League\\CommonMark\\Node\\Inline\\AbstractInline' => $vendorDir . '/league/commonmark/src/Node/Inline/AbstractInline.php',
    'League\\CommonMark\\Node\\Inline\\AbstractStringContainer' => $vendorDir . '/league/commonmark/src/Node/Inline/AbstractStringContainer.php',
    'League\\CommonMark\\Node\\Inline\\AdjacentTextMerger' => $vendorDir . '/league/commonmark/src/Node/Inline/AdjacentTextMerger.php',
    'League\\CommonMark\\Node\\Inline\\DelimitedInterface' => $vendorDir . '/league/commonmark/src/Node/Inline/DelimitedInterface.php',
    'League\\CommonMark\\Node\\Inline\\Newline' => $vendorDir . '/league/commonmark/src/Node/Inline/Newline.php',
    'League\\CommonMark\\Node\\Inline\\Text' => $vendorDir . '/league/commonmark/src/Node/Inline/Text.php',
    'League\\CommonMark\\Node\\Node' => $vendorDir . '/league/commonmark/src/Node/Node.php',
    'League\\CommonMark\\Node\\NodeIterator' => $vendorDir . '/league/commonmark/src/Node/NodeIterator.php',
    'League\\CommonMark\\Node\\NodeWalker' => $vendorDir . '/league/commonmark/src/Node/NodeWalker.php',
    'League\\CommonMark\\Node\\NodeWalkerEvent' => $vendorDir . '/league/commonmark/src/Node/NodeWalkerEvent.php',
    'League\\CommonMark\\Node\\Query' => $vendorDir . '/league/commonmark/src/Node/Query.php',
    'League\\CommonMark\\Node\\Query\\AndExpr' => $vendorDir . '/league/commonmark/src/Node/Query/AndExpr.php',
    'League\\CommonMark\\Node\\Query\\ExpressionInterface' => $vendorDir . '/league/commonmark/src/Node/Query/ExpressionInterface.php',
    'League\\CommonMark\\Node\\Query\\OrExpr' => $vendorDir . '/league/commonmark/src/Node/Query/OrExpr.php',
    'League\\CommonMark\\Node\\RawMarkupContainerInterface' => $vendorDir . '/league/commonmark/src/Node/RawMarkupContainerInterface.php',
    'League\\CommonMark\\Node\\StringContainerHelper' => $vendorDir . '/league/commonmark/src/Node/StringContainerHelper.php',
    'League\\CommonMark\\Node\\StringContainerInterface' => $vendorDir . '/league/commonmark/src/Node/StringContainerInterface.php',
    'League\\CommonMark\\Normalizer\\SlugNormalizer' => $vendorDir . '/league/commonmark/src/Normalizer/SlugNormalizer.php',
    'League\\CommonMark\\Normalizer\\TextNormalizer' => $vendorDir . '/league/commonmark/src/Normalizer/TextNormalizer.php',
    'League\\CommonMark\\Normalizer\\TextNormalizerInterface' => $vendorDir . '/league/commonmark/src/Normalizer/TextNormalizerInterface.php',
    'League\\CommonMark\\Normalizer\\UniqueSlugNormalizer' => $vendorDir . '/league/commonmark/src/Normalizer/UniqueSlugNormalizer.php',
    'League\\CommonMark\\Normalizer\\UniqueSlugNormalizerInterface' => $vendorDir . '/league/commonmark/src/Normalizer/UniqueSlugNormalizerInterface.php',
    'League\\CommonMark\\Output\\RenderedContent' => $vendorDir . '/league/commonmark/src/Output/RenderedContent.php',
    'League\\CommonMark\\Output\\RenderedContentInterface' => $vendorDir . '/league/commonmark/src/Output/RenderedContentInterface.php',
    'League\\CommonMark\\Parser\\Block\\AbstractBlockContinueParser' => $vendorDir . '/league/commonmark/src/Parser/Block/AbstractBlockContinueParser.php',
    'League\\CommonMark\\Parser\\Block\\BlockContinue' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockContinue.php',
    'League\\CommonMark\\Parser\\Block\\BlockContinueParserInterface' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockContinueParserInterface.php',
    'League\\CommonMark\\Parser\\Block\\BlockContinueParserWithInlinesInterface' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockContinueParserWithInlinesInterface.php',
    'League\\CommonMark\\Parser\\Block\\BlockStart' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockStart.php',
    'League\\CommonMark\\Parser\\Block\\BlockStartParserInterface' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockStartParserInterface.php',
    'League\\CommonMark\\Parser\\Block\\DocumentBlockParser' => $vendorDir . '/league/commonmark/src/Parser/Block/DocumentBlockParser.php',
    'League\\CommonMark\\Parser\\Block\\ParagraphParser' => $vendorDir . '/league/commonmark/src/Parser/Block/ParagraphParser.php',
    'League\\CommonMark\\Parser\\Block\\SkipLinesStartingWithLettersParser' => $vendorDir . '/league/commonmark/src/Parser/Block/SkipLinesStartingWithLettersParser.php',
    'League\\CommonMark\\Parser\\Cursor' => $vendorDir . '/league/commonmark/src/Parser/Cursor.php',
    'League\\CommonMark\\Parser\\CursorState' => $vendorDir . '/league/commonmark/src/Parser/CursorState.php',
    'League\\CommonMark\\Parser\\InlineParserContext' => $vendorDir . '/league/commonmark/src/Parser/InlineParserContext.php',
    'League\\CommonMark\\Parser\\InlineParserEngine' => $vendorDir . '/league/commonmark/src/Parser/InlineParserEngine.php',
    'League\\CommonMark\\Parser\\InlineParserEngineInterface' => $vendorDir . '/league/commonmark/src/Parser/InlineParserEngineInterface.php',
    'League\\CommonMark\\Parser\\Inline\\InlineParserInterface' => $vendorDir . '/league/commonmark/src/Parser/Inline/InlineParserInterface.php',
    'League\\CommonMark\\Parser\\Inline\\InlineParserMatch' => $vendorDir . '/league/commonmark/src/Parser/Inline/InlineParserMatch.php',
    'League\\CommonMark\\Parser\\Inline\\NewlineParser' => $vendorDir . '/league/commonmark/src/Parser/Inline/NewlineParser.php',
    'League\\CommonMark\\Parser\\MarkdownParser' => $vendorDir . '/league/commonmark/src/Parser/MarkdownParser.php',
    'League\\CommonMark\\Parser\\MarkdownParserInterface' => $vendorDir . '/league/commonmark/src/Parser/MarkdownParserInterface.php',
    'League\\CommonMark\\Parser\\MarkdownParserState' => $vendorDir . '/league/commonmark/src/Parser/MarkdownParserState.php',
    'League\\CommonMark\\Parser\\MarkdownParserStateInterface' => $vendorDir . '/league/commonmark/src/Parser/MarkdownParserStateInterface.php',
    'League\\CommonMark\\Parser\\ParserLogicException' => $vendorDir . '/league/commonmark/src/Parser/ParserLogicException.php',
    'League\\CommonMark\\Reference\\MemoryLimitedReferenceMap' => $vendorDir . '/league/commonmark/src/Reference/MemoryLimitedReferenceMap.php',
    'League\\CommonMark\\Reference\\Reference' => $vendorDir . '/league/commonmark/src/Reference/Reference.php',
    'League\\CommonMark\\Reference\\ReferenceInterface' => $vendorDir . '/league/commonmark/src/Reference/ReferenceInterface.php',
    'League\\CommonMark\\Reference\\ReferenceMap' => $vendorDir . '/league/commonmark/src/Reference/ReferenceMap.php',
    'League\\CommonMark\\Reference\\ReferenceMapInterface' => $vendorDir . '/league/commonmark/src/Reference/ReferenceMapInterface.php',
    'League\\CommonMark\\Reference\\ReferenceParser' => $vendorDir . '/league/commonmark/src/Reference/ReferenceParser.php',
    'League\\CommonMark\\Reference\\ReferenceableInterface' => $vendorDir . '/league/commonmark/src/Reference/ReferenceableInterface.php',
    'League\\CommonMark\\Renderer\\Block\\DocumentRenderer' => $vendorDir . '/league/commonmark/src/Renderer/Block/DocumentRenderer.php',
    'League\\CommonMark\\Renderer\\Block\\ParagraphRenderer' => $vendorDir . '/league/commonmark/src/Renderer/Block/ParagraphRenderer.php',
    'League\\CommonMark\\Renderer\\ChildNodeRendererInterface' => $vendorDir . '/league/commonmark/src/Renderer/ChildNodeRendererInterface.php',
    'League\\CommonMark\\Renderer\\DocumentRendererInterface' => $vendorDir . '/league/commonmark/src/Renderer/DocumentRendererInterface.php',
    'League\\CommonMark\\Renderer\\HtmlDecorator' => $vendorDir . '/league/commonmark/src/Renderer/HtmlDecorator.php',
    'League\\CommonMark\\Renderer\\HtmlRenderer' => $vendorDir . '/league/commonmark/src/Renderer/HtmlRenderer.php',
    'League\\CommonMark\\Renderer\\Inline\\NewlineRenderer' => $vendorDir . '/league/commonmark/src/Renderer/Inline/NewlineRenderer.php',
    'League\\CommonMark\\Renderer\\Inline\\TextRenderer' => $vendorDir . '/league/commonmark/src/Renderer/Inline/TextRenderer.php',
    'League\\CommonMark\\Renderer\\MarkdownRendererInterface' => $vendorDir . '/league/commonmark/src/Renderer/MarkdownRendererInterface.php',
    'League\\CommonMark\\Renderer\\NoMatchingRendererException' => $vendorDir . '/league/commonmark/src/Renderer/NoMatchingRendererException.php',
    'League\\CommonMark\\Renderer\\NodeRendererInterface' => $vendorDir . '/league/commonmark/src/Renderer/NodeRendererInterface.php',
    'League\\CommonMark\\Util\\ArrayCollection' => $vendorDir . '/league/commonmark/src/Util/ArrayCollection.php',
    'League\\CommonMark\\Util\\Html5EntityDecoder' => $vendorDir . '/league/commonmark/src/Util/Html5EntityDecoder.php',
    'League\\CommonMark\\Util\\HtmlElement' => $vendorDir . '/league/commonmark/src/Util/HtmlElement.php',
    'League\\CommonMark\\Util\\HtmlFilter' => $vendorDir . '/league/commonmark/src/Util/HtmlFilter.php',
    'League\\CommonMark\\Util\\LinkParserHelper' => $vendorDir . '/league/commonmark/src/Util/LinkParserHelper.php',
    'League\\CommonMark\\Util\\PrioritizedList' => $vendorDir . '/league/commonmark/src/Util/PrioritizedList.php',
    'League\\CommonMark\\Util\\RegexHelper' => $vendorDir . '/league/commonmark/src/Util/RegexHelper.php',
    'League\\CommonMark\\Util\\SpecReader' => $vendorDir . '/league/commonmark/src/Util/SpecReader.php',
    'League\\CommonMark\\Util\\UrlEncoder' => $vendorDir . '/league/commonmark/src/Util/UrlEncoder.php',
    'League\\CommonMark\\Util\\Xml' => $vendorDir . '/league/commonmark/src/Util/Xml.php',
    'League\\CommonMark\\Xml\\FallbackNodeXmlRenderer' => $vendorDir . '/league/commonmark/src/Xml/FallbackNodeXmlRenderer.php',
    'League\\CommonMark\\Xml\\MarkdownToXmlConverter' => $vendorDir . '/league/commonmark/src/Xml/MarkdownToXmlConverter.php',
    'League\\CommonMark\\Xml\\XmlNodeRendererInterface' => $vendorDir . '/league/commonmark/src/Xml/XmlNodeRendererInterface.php',
    'League\\CommonMark\\Xml\\XmlRenderer' => $vendorDir . '/league/commonmark/src/Xml/XmlRenderer.php',
    'League\\Config\\Configuration' => $vendorDir . '/league/config/src/Configuration.php',
    'League\\Config\\ConfigurationAwareInterface' => $vendorDir . '/league/config/src/ConfigurationAwareInterface.php',
    'League\\Config\\ConfigurationBuilderInterface' => $vendorDir . '/league/config/src/ConfigurationBuilderInterface.php',
    'League\\Config\\ConfigurationInterface' => $vendorDir . '/league/config/src/ConfigurationInterface.php',
    'League\\Config\\ConfigurationProviderInterface' => $vendorDir . '/league/config/src/ConfigurationProviderInterface.php',
    'League\\Config\\Exception\\ConfigurationExceptionInterface' => $vendorDir . '/league/config/src/Exception/ConfigurationExceptionInterface.php',
    'League\\Config\\Exception\\InvalidConfigurationException' => $vendorDir . '/league/config/src/Exception/InvalidConfigurationException.php',
    'League\\Config\\Exception\\UnknownOptionException' => $vendorDir . '/league/config/src/Exception/UnknownOptionException.php',
    'League\\Config\\Exception\\ValidationException' => $vendorDir . '/league/config/src/Exception/ValidationException.php',
    'League\\Config\\MutableConfigurationInterface' => $vendorDir . '/league/config/src/MutableConfigurationInterface.php',
    'League\\Config\\ReadOnlyConfiguration' => $vendorDir . '/league/config/src/ReadOnlyConfiguration.php',
    'League\\Config\\SchemaBuilderInterface' => $vendorDir . '/league/config/src/SchemaBuilderInterface.php',
    'League\\Flysystem\\CalculateChecksumFromStream' => $vendorDir . '/league/flysystem/src/CalculateChecksumFromStream.php',
    'League\\Flysystem\\ChecksumAlgoIsNotSupported' => $vendorDir . '/league/flysystem/src/ChecksumAlgoIsNotSupported.php',
    'League\\Flysystem\\ChecksumProvider' => $vendorDir . '/league/flysystem/src/ChecksumProvider.php',
    'League\\Flysystem\\Config' => $vendorDir . '/league/flysystem/src/Config.php',
    'League\\Flysystem\\CorruptedPathDetected' => $vendorDir . '/league/flysystem/src/CorruptedPathDetected.php',
    'League\\Flysystem\\DecoratedAdapter' => $vendorDir . '/league/flysystem/src/DecoratedAdapter.php',
    'League\\Flysystem\\DirectoryAttributes' => $vendorDir . '/league/flysystem/src/DirectoryAttributes.php',
    'League\\Flysystem\\DirectoryListing' => $vendorDir . '/league/flysystem/src/DirectoryListing.php',
    'League\\Flysystem\\FileAttributes' => $vendorDir . '/league/flysystem/src/FileAttributes.php',
    'League\\Flysystem\\Filesystem' => $vendorDir . '/league/flysystem/src/Filesystem.php',
    'League\\Flysystem\\FilesystemAdapter' => $vendorDir . '/league/flysystem/src/FilesystemAdapter.php',
    'League\\Flysystem\\FilesystemException' => $vendorDir . '/league/flysystem/src/FilesystemException.php',
    'League\\Flysystem\\FilesystemOperationFailed' => $vendorDir . '/league/flysystem/src/FilesystemOperationFailed.php',
    'League\\Flysystem\\FilesystemOperator' => $vendorDir . '/league/flysystem/src/FilesystemOperator.php',
    'League\\Flysystem\\FilesystemReader' => $vendorDir . '/league/flysystem/src/FilesystemReader.php',
    'League\\Flysystem\\FilesystemWriter' => $vendorDir . '/league/flysystem/src/FilesystemWriter.php',
    'League\\Flysystem\\InvalidStreamProvided' => $vendorDir . '/league/flysystem/src/InvalidStreamProvided.php',
    'League\\Flysystem\\InvalidVisibilityProvided' => $vendorDir . '/league/flysystem/src/InvalidVisibilityProvided.php',
    'League\\Flysystem\\Local\\FallbackMimeTypeDetector' => $vendorDir . '/league/flysystem-local/FallbackMimeTypeDetector.php',
    'League\\Flysystem\\Local\\LocalFilesystemAdapter' => $vendorDir . '/league/flysystem-local/LocalFilesystemAdapter.php',
    'League\\Flysystem\\MountManager' => $vendorDir . '/league/flysystem/src/MountManager.php',
    'League\\Flysystem\\PathNormalizer' => $vendorDir . '/league/flysystem/src/PathNormalizer.php',
    'League\\Flysystem\\PathPrefixer' => $vendorDir . '/league/flysystem/src/PathPrefixer.php',
    'League\\Flysystem\\PathTraversalDetected' => $vendorDir . '/league/flysystem/src/PathTraversalDetected.php',
    'League\\Flysystem\\PortableVisibilityGuard' => $vendorDir . '/league/flysystem/src/PortableVisibilityGuard.php',
    'League\\Flysystem\\ProxyArrayAccessToProperties' => $vendorDir . '/league/flysystem/src/ProxyArrayAccessToProperties.php',
    'League\\Flysystem\\ResolveIdenticalPathConflict' => $vendorDir . '/league/flysystem/src/ResolveIdenticalPathConflict.php',
    'League\\Flysystem\\StorageAttributes' => $vendorDir . '/league/flysystem/src/StorageAttributes.php',
    'League\\Flysystem\\SymbolicLinkEncountered' => $vendorDir . '/league/flysystem/src/SymbolicLinkEncountered.php',
    'League\\Flysystem\\UnableToCheckDirectoryExistence' => $vendorDir . '/league/flysystem/src/UnableToCheckDirectoryExistence.php',
    'League\\Flysystem\\UnableToCheckExistence' => $vendorDir . '/league/flysystem/src/UnableToCheckExistence.php',
    'League\\Flysystem\\UnableToCheckFileExistence' => $vendorDir . '/league/flysystem/src/UnableToCheckFileExistence.php',
    'League\\Flysystem\\UnableToCopyFile' => $vendorDir . '/league/flysystem/src/UnableToCopyFile.php',
    'League\\Flysystem\\UnableToCreateDirectory' => $vendorDir . '/league/flysystem/src/UnableToCreateDirectory.php',
    'League\\Flysystem\\UnableToDeleteDirectory' => $vendorDir . '/league/flysystem/src/UnableToDeleteDirectory.php',
    'League\\Flysystem\\UnableToDeleteFile' => $vendorDir . '/league/flysystem/src/UnableToDeleteFile.php',
    'League\\Flysystem\\UnableToGeneratePublicUrl' => $vendorDir . '/league/flysystem/src/UnableToGeneratePublicUrl.php',
    'League\\Flysystem\\UnableToGenerateTemporaryUrl' => $vendorDir . '/league/flysystem/src/UnableToGenerateTemporaryUrl.php',
    'League\\Flysystem\\UnableToListContents' => $vendorDir . '/league/flysystem/src/UnableToListContents.php',
    'League\\Flysystem\\UnableToMountFilesystem' => $vendorDir . '/league/flysystem/src/UnableToMountFilesystem.php',
    'League\\Flysystem\\UnableToMoveFile' => $vendorDir . '/league/flysystem/src/UnableToMoveFile.php',
    'League\\Flysystem\\UnableToProvideChecksum' => $vendorDir . '/league/flysystem/src/UnableToProvideChecksum.php',
    'League\\Flysystem\\UnableToReadFile' => $vendorDir . '/league/flysystem/src/UnableToReadFile.php',
    'League\\Flysystem\\UnableToResolveFilesystemMount' => $vendorDir . '/league/flysystem/src/UnableToResolveFilesystemMount.php',
    'League\\Flysystem\\UnableToRetrieveMetadata' => $vendorDir . '/league/flysystem/src/UnableToRetrieveMetadata.php',
    'League\\Flysystem\\UnableToSetVisibility' => $vendorDir . '/league/flysystem/src/UnableToSetVisibility.php',
    'League\\Flysystem\\UnableToWriteFile' => $vendorDir . '/league/flysystem/src/UnableToWriteFile.php',
    'League\\Flysystem\\UnixVisibility\\PortableVisibilityConverter' => $vendorDir . '/league/flysystem/src/UnixVisibility/PortableVisibilityConverter.php',
    'League\\Flysystem\\UnixVisibility\\VisibilityConverter' => $vendorDir . '/league/flysystem/src/UnixVisibility/VisibilityConverter.php',
    'League\\Flysystem\\UnreadableFileEncountered' => $vendorDir . '/league/flysystem/src/UnreadableFileEncountered.php',
    'League\\Flysystem\\UrlGeneration\\ChainedPublicUrlGenerator' => $vendorDir . '/league/flysystem/src/UrlGeneration/ChainedPublicUrlGenerator.php',
    'League\\Flysystem\\UrlGeneration\\PrefixPublicUrlGenerator' => $vendorDir . '/league/flysystem/src/UrlGeneration/PrefixPublicUrlGenerator.php',
    'League\\Flysystem\\UrlGeneration\\PublicUrlGenerator' => $vendorDir . '/league/flysystem/src/UrlGeneration/PublicUrlGenerator.php',
    'League\\Flysystem\\UrlGeneration\\ShardedPrefixPublicUrlGenerator' => $vendorDir . '/league/flysystem/src/UrlGeneration/ShardedPrefixPublicUrlGenerator.php',
    'League\\Flysystem\\UrlGeneration\\TemporaryUrlGenerator' => $vendorDir . '/league/flysystem/src/UrlGeneration/TemporaryUrlGenerator.php',
    'League\\Flysystem\\Visibility' => $vendorDir . '/league/flysystem/src/Visibility.php',
    'League\\Flysystem\\WhitespacePathNormalizer' => $vendorDir . '/league/flysystem/src/WhitespacePathNormalizer.php',
    'League\\MimeTypeDetection\\EmptyExtensionToMimeTypeMap' => $vendorDir . '/league/mime-type-detection/src/EmptyExtensionToMimeTypeMap.php',
    'League\\MimeTypeDetection\\ExtensionLookup' => $vendorDir . '/league/mime-type-detection/src/ExtensionLookup.php',
    'League\\MimeTypeDetection\\ExtensionMimeTypeDetector' => $vendorDir . '/league/mime-type-detection/src/ExtensionMimeTypeDetector.php',
    'League\\MimeTypeDetection\\ExtensionToMimeTypeMap' => $vendorDir . '/league/mime-type-detection/src/ExtensionToMimeTypeMap.php',
    'League\\MimeTypeDetection\\FinfoMimeTypeDetector' => $vendorDir . '/league/mime-type-detection/src/FinfoMimeTypeDetector.php',
    'League\\MimeTypeDetection\\GeneratedExtensionToMimeTypeMap' => $vendorDir . '/league/mime-type-detection/src/GeneratedExtensionToMimeTypeMap.php',
    'League\\MimeTypeDetection\\MimeTypeDetector' => $vendorDir . '/league/mime-type-detection/src/MimeTypeDetector.php',
    'League\\MimeTypeDetection\\OverridingExtensionToMimeTypeMap' => $vendorDir . '/league/mime-type-detection/src/OverridingExtensionToMimeTypeMap.php',
    'Maatwebsite\\Excel\\Cache\\BatchCache' => $vendorDir . '/maatwebsite/excel/src/Cache/BatchCache.php',
    'Maatwebsite\\Excel\\Cache\\BatchCacheDeprecated' => $vendorDir . '/maatwebsite/excel/src/Cache/BatchCacheDeprecated.php',
    'Maatwebsite\\Excel\\Cache\\CacheManager' => $vendorDir . '/maatwebsite/excel/src/Cache/CacheManager.php',
    'Maatwebsite\\Excel\\Cache\\MemoryCache' => $vendorDir . '/maatwebsite/excel/src/Cache/MemoryCache.php',
    'Maatwebsite\\Excel\\Cache\\MemoryCacheDeprecated' => $vendorDir . '/maatwebsite/excel/src/Cache/MemoryCacheDeprecated.php',
    'Maatwebsite\\Excel\\Cell' => $vendorDir . '/maatwebsite/excel/src/Cell.php',
    'Maatwebsite\\Excel\\ChunkReader' => $vendorDir . '/maatwebsite/excel/src/ChunkReader.php',
    'Maatwebsite\\Excel\\Concerns\\Exportable' => $vendorDir . '/maatwebsite/excel/src/Concerns/Exportable.php',
    'Maatwebsite\\Excel\\Concerns\\FromArray' => $vendorDir . '/maatwebsite/excel/src/Concerns/FromArray.php',
    'Maatwebsite\\Excel\\Concerns\\FromCollection' => $vendorDir . '/maatwebsite/excel/src/Concerns/FromCollection.php',
    'Maatwebsite\\Excel\\Concerns\\FromGenerator' => $vendorDir . '/maatwebsite/excel/src/Concerns/FromGenerator.php',
    'Maatwebsite\\Excel\\Concerns\\FromIterator' => $vendorDir . '/maatwebsite/excel/src/Concerns/FromIterator.php',
    'Maatwebsite\\Excel\\Concerns\\FromQuery' => $vendorDir . '/maatwebsite/excel/src/Concerns/FromQuery.php',
    'Maatwebsite\\Excel\\Concerns\\FromView' => $vendorDir . '/maatwebsite/excel/src/Concerns/FromView.php',
    'Maatwebsite\\Excel\\Concerns\\HasReferencesToOtherSheets' => $vendorDir . '/maatwebsite/excel/src/Concerns/HasReferencesToOtherSheets.php',
    'Maatwebsite\\Excel\\Concerns\\Importable' => $vendorDir . '/maatwebsite/excel/src/Concerns/Importable.php',
    'Maatwebsite\\Excel\\Concerns\\MapsCsvSettings' => $vendorDir . '/maatwebsite/excel/src/Concerns/MapsCsvSettings.php',
    'Maatwebsite\\Excel\\Concerns\\OnEachRow' => $vendorDir . '/maatwebsite/excel/src/Concerns/OnEachRow.php',
    'Maatwebsite\\Excel\\Concerns\\PersistRelations' => $vendorDir . '/maatwebsite/excel/src/Concerns/PersistRelations.php',
    'Maatwebsite\\Excel\\Concerns\\RegistersEventListeners' => $vendorDir . '/maatwebsite/excel/src/Concerns/RegistersEventListeners.php',
    'Maatwebsite\\Excel\\Concerns\\RemembersChunkOffset' => $vendorDir . '/maatwebsite/excel/src/Concerns/RemembersChunkOffset.php',
    'Maatwebsite\\Excel\\Concerns\\RemembersRowNumber' => $vendorDir . '/maatwebsite/excel/src/Concerns/RemembersRowNumber.php',
    'Maatwebsite\\Excel\\Concerns\\ShouldAutoSize' => $vendorDir . '/maatwebsite/excel/src/Concerns/ShouldAutoSize.php',
    'Maatwebsite\\Excel\\Concerns\\ShouldQueueWithoutChain' => $vendorDir . '/maatwebsite/excel/src/Concerns/ShouldQueueWithoutChain.php',
    'Maatwebsite\\Excel\\Concerns\\SkipsEmptyRows' => $vendorDir . '/maatwebsite/excel/src/Concerns/SkipsEmptyRows.php',
    'Maatwebsite\\Excel\\Concerns\\SkipsErrors' => $vendorDir . '/maatwebsite/excel/src/Concerns/SkipsErrors.php',
    'Maatwebsite\\Excel\\Concerns\\SkipsFailures' => $vendorDir . '/maatwebsite/excel/src/Concerns/SkipsFailures.php',
    'Maatwebsite\\Excel\\Concerns\\SkipsOnError' => $vendorDir . '/maatwebsite/excel/src/Concerns/SkipsOnError.php',
    'Maatwebsite\\Excel\\Concerns\\SkipsOnFailure' => $vendorDir . '/maatwebsite/excel/src/Concerns/SkipsOnFailure.php',
    'Maatwebsite\\Excel\\Concerns\\SkipsUnknownSheets' => $vendorDir . '/maatwebsite/excel/src/Concerns/SkipsUnknownSheets.php',
    'Maatwebsite\\Excel\\Concerns\\ToArray' => $vendorDir . '/maatwebsite/excel/src/Concerns/ToArray.php',
    'Maatwebsite\\Excel\\Concerns\\ToCollection' => $vendorDir . '/maatwebsite/excel/src/Concerns/ToCollection.php',
    'Maatwebsite\\Excel\\Concerns\\ToModel' => $vendorDir . '/maatwebsite/excel/src/Concerns/ToModel.php',
    'Maatwebsite\\Excel\\Concerns\\WithBackgroundColor' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithBackgroundColor.php',
    'Maatwebsite\\Excel\\Concerns\\WithBatchInserts' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithBatchInserts.php',
    'Maatwebsite\\Excel\\Concerns\\WithCalculatedFormulas' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithCalculatedFormulas.php',
    'Maatwebsite\\Excel\\Concerns\\WithCharts' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithCharts.php',
    'Maatwebsite\\Excel\\Concerns\\WithChunkReading' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithChunkReading.php',
    'Maatwebsite\\Excel\\Concerns\\WithColumnFormatting' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithColumnFormatting.php',
    'Maatwebsite\\Excel\\Concerns\\WithColumnLimit' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithColumnLimit.php',
    'Maatwebsite\\Excel\\Concerns\\WithColumnWidths' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithColumnWidths.php',
    'Maatwebsite\\Excel\\Concerns\\WithConditionalSheets' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithConditionalSheets.php',
    'Maatwebsite\\Excel\\Concerns\\WithCustomChunkSize' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithCustomChunkSize.php',
    'Maatwebsite\\Excel\\Concerns\\WithCustomCsvSettings' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithCustomCsvSettings.php',
    'Maatwebsite\\Excel\\Concerns\\WithCustomQuerySize' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithCustomQuerySize.php',
    'Maatwebsite\\Excel\\Concerns\\WithCustomStartCell' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithCustomStartCell.php',
    'Maatwebsite\\Excel\\Concerns\\WithCustomValueBinder' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithCustomValueBinder.php',
    'Maatwebsite\\Excel\\Concerns\\WithDefaultStyles' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithDefaultStyles.php',
    'Maatwebsite\\Excel\\Concerns\\WithDrawings' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithDrawings.php',
    'Maatwebsite\\Excel\\Concerns\\WithEvents' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithEvents.php',
    'Maatwebsite\\Excel\\Concerns\\WithFormatData' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithFormatData.php',
    'Maatwebsite\\Excel\\Concerns\\WithGroupedHeadingRow' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithGroupedHeadingRow.php',
    'Maatwebsite\\Excel\\Concerns\\WithHeadingRow' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithHeadingRow.php',
    'Maatwebsite\\Excel\\Concerns\\WithHeadings' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithHeadings.php',
    'Maatwebsite\\Excel\\Concerns\\WithLimit' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithLimit.php',
    'Maatwebsite\\Excel\\Concerns\\WithMappedCells' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithMappedCells.php',
    'Maatwebsite\\Excel\\Concerns\\WithMapping' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithMapping.php',
    'Maatwebsite\\Excel\\Concerns\\WithMultipleSheets' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithMultipleSheets.php',
    'Maatwebsite\\Excel\\Concerns\\WithPreCalculateFormulas' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithPreCalculateFormulas.php',
    'Maatwebsite\\Excel\\Concerns\\WithProgressBar' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithProgressBar.php',
    'Maatwebsite\\Excel\\Concerns\\WithProperties' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithProperties.php',
    'Maatwebsite\\Excel\\Concerns\\WithReadFilter' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithReadFilter.php',
    'Maatwebsite\\Excel\\Concerns\\WithSkipDuplicates' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithSkipDuplicates.php',
    'Maatwebsite\\Excel\\Concerns\\WithStartRow' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithStartRow.php',
    'Maatwebsite\\Excel\\Concerns\\WithStrictNullComparison' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithStrictNullComparison.php',
    'Maatwebsite\\Excel\\Concerns\\WithStyles' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithStyles.php',
    'Maatwebsite\\Excel\\Concerns\\WithTitle' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithTitle.php',
    'Maatwebsite\\Excel\\Concerns\\WithUpsertColumns' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithUpsertColumns.php',
    'Maatwebsite\\Excel\\Concerns\\WithUpserts' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithUpserts.php',
    'Maatwebsite\\Excel\\Concerns\\WithValidation' => $vendorDir . '/maatwebsite/excel/src/Concerns/WithValidation.php',
    'Maatwebsite\\Excel\\Console\\ExportMakeCommand' => $vendorDir . '/maatwebsite/excel/src/Console/ExportMakeCommand.php',
    'Maatwebsite\\Excel\\Console\\ImportMakeCommand' => $vendorDir . '/maatwebsite/excel/src/Console/ImportMakeCommand.php',
    'Maatwebsite\\Excel\\Console\\WithModelStub' => $vendorDir . '/maatwebsite/excel/src/Console/WithModelStub.php',
    'Maatwebsite\\Excel\\DefaultValueBinder' => $vendorDir . '/maatwebsite/excel/src/DefaultValueBinder.php',
    'Maatwebsite\\Excel\\DelegatedMacroable' => $vendorDir . '/maatwebsite/excel/src/DelegatedMacroable.php',
    'Maatwebsite\\Excel\\Events\\AfterBatch' => $vendorDir . '/maatwebsite/excel/src/Events/AfterBatch.php',
    'Maatwebsite\\Excel\\Events\\AfterChunk' => $vendorDir . '/maatwebsite/excel/src/Events/AfterChunk.php',
    'Maatwebsite\\Excel\\Events\\AfterImport' => $vendorDir . '/maatwebsite/excel/src/Events/AfterImport.php',
    'Maatwebsite\\Excel\\Events\\AfterSheet' => $vendorDir . '/maatwebsite/excel/src/Events/AfterSheet.php',
    'Maatwebsite\\Excel\\Events\\BeforeExport' => $vendorDir . '/maatwebsite/excel/src/Events/BeforeExport.php',
    'Maatwebsite\\Excel\\Events\\BeforeImport' => $vendorDir . '/maatwebsite/excel/src/Events/BeforeImport.php',
    'Maatwebsite\\Excel\\Events\\BeforeSheet' => $vendorDir . '/maatwebsite/excel/src/Events/BeforeSheet.php',
    'Maatwebsite\\Excel\\Events\\BeforeWriting' => $vendorDir . '/maatwebsite/excel/src/Events/BeforeWriting.php',
    'Maatwebsite\\Excel\\Events\\Event' => $vendorDir . '/maatwebsite/excel/src/Events/Event.php',
    'Maatwebsite\\Excel\\Events\\ImportFailed' => $vendorDir . '/maatwebsite/excel/src/Events/ImportFailed.php',
    'Maatwebsite\\Excel\\Excel' => $vendorDir . '/maatwebsite/excel/src/Excel.php',
    'Maatwebsite\\Excel\\ExcelServiceProvider' => $vendorDir . '/maatwebsite/excel/src/ExcelServiceProvider.php',
    'Maatwebsite\\Excel\\Exceptions\\ConcernConflictException' => $vendorDir . '/maatwebsite/excel/src/Exceptions/ConcernConflictException.php',
    'Maatwebsite\\Excel\\Exceptions\\LaravelExcelException' => $vendorDir . '/maatwebsite/excel/src/Exceptions/LaravelExcelException.php',
    'Maatwebsite\\Excel\\Exceptions\\NoFilePathGivenException' => $vendorDir . '/maatwebsite/excel/src/Exceptions/NoFilePathGivenException.php',
    'Maatwebsite\\Excel\\Exceptions\\NoFilenameGivenException' => $vendorDir . '/maatwebsite/excel/src/Exceptions/NoFilenameGivenException.php',
    'Maatwebsite\\Excel\\Exceptions\\NoSheetsFoundException' => $vendorDir . '/maatwebsite/excel/src/Exceptions/NoSheetsFoundException.php',
    'Maatwebsite\\Excel\\Exceptions\\NoTypeDetectedException' => $vendorDir . '/maatwebsite/excel/src/Exceptions/NoTypeDetectedException.php',
    'Maatwebsite\\Excel\\Exceptions\\RowSkippedException' => $vendorDir . '/maatwebsite/excel/src/Exceptions/RowSkippedException.php',
    'Maatwebsite\\Excel\\Exceptions\\SheetNotFoundException' => $vendorDir . '/maatwebsite/excel/src/Exceptions/SheetNotFoundException.php',
    'Maatwebsite\\Excel\\Exceptions\\UnreadableFileException' => $vendorDir . '/maatwebsite/excel/src/Exceptions/UnreadableFileException.php',
    'Maatwebsite\\Excel\\Exporter' => $vendorDir . '/maatwebsite/excel/src/Exporter.php',
    'Maatwebsite\\Excel\\Facades\\Excel' => $vendorDir . '/maatwebsite/excel/src/Facades/Excel.php',
    'Maatwebsite\\Excel\\Factories\\ReaderFactory' => $vendorDir . '/maatwebsite/excel/src/Factories/ReaderFactory.php',
    'Maatwebsite\\Excel\\Factories\\WriterFactory' => $vendorDir . '/maatwebsite/excel/src/Factories/WriterFactory.php',
    'Maatwebsite\\Excel\\Fakes\\ExcelFake' => $vendorDir . '/maatwebsite/excel/src/Fakes/ExcelFake.php',
    'Maatwebsite\\Excel\\Files\\Disk' => $vendorDir . '/maatwebsite/excel/src/Files/Disk.php',
    'Maatwebsite\\Excel\\Files\\Filesystem' => $vendorDir . '/maatwebsite/excel/src/Files/Filesystem.php',
    'Maatwebsite\\Excel\\Files\\LocalTemporaryFile' => $vendorDir . '/maatwebsite/excel/src/Files/LocalTemporaryFile.php',
    'Maatwebsite\\Excel\\Files\\RemoteTemporaryFile' => $vendorDir . '/maatwebsite/excel/src/Files/RemoteTemporaryFile.php',
    'Maatwebsite\\Excel\\Files\\TemporaryFile' => $vendorDir . '/maatwebsite/excel/src/Files/TemporaryFile.php',
    'Maatwebsite\\Excel\\Files\\TemporaryFileFactory' => $vendorDir . '/maatwebsite/excel/src/Files/TemporaryFileFactory.php',
    'Maatwebsite\\Excel\\Filters\\ChunkReadFilter' => $vendorDir . '/maatwebsite/excel/src/Filters/ChunkReadFilter.php',
    'Maatwebsite\\Excel\\Filters\\LimitFilter' => $vendorDir . '/maatwebsite/excel/src/Filters/LimitFilter.php',
    'Maatwebsite\\Excel\\HasEventBus' => $vendorDir . '/maatwebsite/excel/src/HasEventBus.php',
    'Maatwebsite\\Excel\\HeadingRowImport' => $vendorDir . '/maatwebsite/excel/src/HeadingRowImport.php',
    'Maatwebsite\\Excel\\Helpers\\ArrayHelper' => $vendorDir . '/maatwebsite/excel/src/Helpers/ArrayHelper.php',
    'Maatwebsite\\Excel\\Helpers\\CellHelper' => $vendorDir . '/maatwebsite/excel/src/Helpers/CellHelper.php',
    'Maatwebsite\\Excel\\Helpers\\FileTypeDetector' => $vendorDir . '/maatwebsite/excel/src/Helpers/FileTypeDetector.php',
    'Maatwebsite\\Excel\\Importer' => $vendorDir . '/maatwebsite/excel/src/Importer.php',
    'Maatwebsite\\Excel\\Imports\\EndRowFinder' => $vendorDir . '/maatwebsite/excel/src/Imports/EndRowFinder.php',
    'Maatwebsite\\Excel\\Imports\\HeadingRowExtractor' => $vendorDir . '/maatwebsite/excel/src/Imports/HeadingRowExtractor.php',
    'Maatwebsite\\Excel\\Imports\\HeadingRowFormatter' => $vendorDir . '/maatwebsite/excel/src/Imports/HeadingRowFormatter.php',
    'Maatwebsite\\Excel\\Imports\\ModelImporter' => $vendorDir . '/maatwebsite/excel/src/Imports/ModelImporter.php',
    'Maatwebsite\\Excel\\Imports\\ModelManager' => $vendorDir . '/maatwebsite/excel/src/Imports/ModelManager.php',
    'Maatwebsite\\Excel\\Imports\\Persistence\\CascadePersistManager' => $vendorDir . '/maatwebsite/excel/src/Imports/Persistence/CascadePersistManager.php',
    'Maatwebsite\\Excel\\Jobs\\AfterImportJob' => $vendorDir . '/maatwebsite/excel/src/Jobs/AfterImportJob.php',
    'Maatwebsite\\Excel\\Jobs\\AppendDataToSheet' => $vendorDir . '/maatwebsite/excel/src/Jobs/AppendDataToSheet.php',
    'Maatwebsite\\Excel\\Jobs\\AppendPaginatedToSheet' => $vendorDir . '/maatwebsite/excel/src/Jobs/AppendPaginatedToSheet.php',
    'Maatwebsite\\Excel\\Jobs\\AppendQueryToSheet' => $vendorDir . '/maatwebsite/excel/src/Jobs/AppendQueryToSheet.php',
    'Maatwebsite\\Excel\\Jobs\\AppendViewToSheet' => $vendorDir . '/maatwebsite/excel/src/Jobs/AppendViewToSheet.php',
    'Maatwebsite\\Excel\\Jobs\\CloseSheet' => $vendorDir . '/maatwebsite/excel/src/Jobs/CloseSheet.php',
    'Maatwebsite\\Excel\\Jobs\\ExtendedQueueable' => $vendorDir . '/maatwebsite/excel/src/Jobs/ExtendedQueueable.php',
    'Maatwebsite\\Excel\\Jobs\\Middleware\\LocalizeJob' => $vendorDir . '/maatwebsite/excel/src/Jobs/Middleware/LocalizeJob.php',
    'Maatwebsite\\Excel\\Jobs\\ProxyFailures' => $vendorDir . '/maatwebsite/excel/src/Jobs/ProxyFailures.php',
    'Maatwebsite\\Excel\\Jobs\\QueueExport' => $vendorDir . '/maatwebsite/excel/src/Jobs/QueueExport.php',
    'Maatwebsite\\Excel\\Jobs\\QueueImport' => $vendorDir . '/maatwebsite/excel/src/Jobs/QueueImport.php',
    'Maatwebsite\\Excel\\Jobs\\ReadChunk' => $vendorDir . '/maatwebsite/excel/src/Jobs/ReadChunk.php',
    'Maatwebsite\\Excel\\Jobs\\StoreQueuedExport' => $vendorDir . '/maatwebsite/excel/src/Jobs/StoreQueuedExport.php',
    'Maatwebsite\\Excel\\MappedReader' => $vendorDir . '/maatwebsite/excel/src/MappedReader.php',
    'Maatwebsite\\Excel\\Middleware\\CellMiddleware' => $vendorDir . '/maatwebsite/excel/src/Middleware/CellMiddleware.php',
    'Maatwebsite\\Excel\\Middleware\\ConvertEmptyCellValuesToNull' => $vendorDir . '/maatwebsite/excel/src/Middleware/ConvertEmptyCellValuesToNull.php',
    'Maatwebsite\\Excel\\Middleware\\TrimCellValue' => $vendorDir . '/maatwebsite/excel/src/Middleware/TrimCellValue.php',
    'Maatwebsite\\Excel\\Mixins\\DownloadCollectionMixin' => $vendorDir . '/maatwebsite/excel/src/Mixins/DownloadCollectionMixin.php',
    'Maatwebsite\\Excel\\Mixins\\DownloadQueryMacro' => $vendorDir . '/maatwebsite/excel/src/Mixins/DownloadQueryMacro.php',
    'Maatwebsite\\Excel\\Mixins\\ImportAsMacro' => $vendorDir . '/maatwebsite/excel/src/Mixins/ImportAsMacro.php',
    'Maatwebsite\\Excel\\Mixins\\ImportMacro' => $vendorDir . '/maatwebsite/excel/src/Mixins/ImportMacro.php',
    'Maatwebsite\\Excel\\Mixins\\StoreCollectionMixin' => $vendorDir . '/maatwebsite/excel/src/Mixins/StoreCollectionMixin.php',
    'Maatwebsite\\Excel\\Mixins\\StoreQueryMacro' => $vendorDir . '/maatwebsite/excel/src/Mixins/StoreQueryMacro.php',
    'Maatwebsite\\Excel\\QueuedWriter' => $vendorDir . '/maatwebsite/excel/src/QueuedWriter.php',
    'Maatwebsite\\Excel\\Reader' => $vendorDir . '/maatwebsite/excel/src/Reader.php',
    'Maatwebsite\\Excel\\RegistersCustomConcerns' => $vendorDir . '/maatwebsite/excel/src/RegistersCustomConcerns.php',
    'Maatwebsite\\Excel\\Row' => $vendorDir . '/maatwebsite/excel/src/Row.php',
    'Maatwebsite\\Excel\\SettingsProvider' => $vendorDir . '/maatwebsite/excel/src/SettingsProvider.php',
    'Maatwebsite\\Excel\\Sheet' => $vendorDir . '/maatwebsite/excel/src/Sheet.php',
    'Maatwebsite\\Excel\\Transactions\\DbTransactionHandler' => $vendorDir . '/maatwebsite/excel/src/Transactions/DbTransactionHandler.php',
    'Maatwebsite\\Excel\\Transactions\\NullTransactionHandler' => $vendorDir . '/maatwebsite/excel/src/Transactions/NullTransactionHandler.php',
    'Maatwebsite\\Excel\\Transactions\\TransactionHandler' => $vendorDir . '/maatwebsite/excel/src/Transactions/TransactionHandler.php',
    'Maatwebsite\\Excel\\Transactions\\TransactionManager' => $vendorDir . '/maatwebsite/excel/src/Transactions/TransactionManager.php',
    'Maatwebsite\\Excel\\Validators\\Failure' => $vendorDir . '/maatwebsite/excel/src/Validators/Failure.php',
    'Maatwebsite\\Excel\\Validators\\RowValidator' => $vendorDir . '/maatwebsite/excel/src/Validators/RowValidator.php',
    'Maatwebsite\\Excel\\Validators\\ValidationException' => $vendorDir . '/maatwebsite/excel/src/Validators/ValidationException.php',
    'Maatwebsite\\Excel\\Writer' => $vendorDir . '/maatwebsite/excel/src/Writer.php',
    'Mailjet\\Client' => $vendorDir . '/mailjet/mailjet-apiv3-php/src/Mailjet/Client.php',
    'Mailjet\\Config' => $vendorDir . '/mailjet/mailjet-apiv3-php/src/Mailjet/Config.php',
    'Mailjet\\Normalizer\\ContactNormalizer' => $vendorDir . '/mailjet/mailjet-apiv3-php/src/Mailjet/Normalizer/ContactNormalizer.php',
    'Mailjet\\Normalizer\\NormalizerInterface' => $vendorDir . '/mailjet/mailjet-apiv3-php/src/Mailjet/Normalizer/NormalizerInterface.php',
    'Mailjet\\Request' => $vendorDir . '/mailjet/mailjet-apiv3-php/src/Mailjet/Request.php',
    'Mailjet\\Resources' => $vendorDir . '/mailjet/mailjet-apiv3-php/src/Mailjet/Resources.php',
    'Mailjet\\Response' => $vendorDir . '/mailjet/mailjet-apiv3-php/src/Mailjet/Response.php',
    'Masterminds\\HTML5' => $vendorDir . '/masterminds/html5/src/HTML5.php',
    'Masterminds\\HTML5\\Elements' => $vendorDir . '/masterminds/html5/src/HTML5/Elements.php',
    'Masterminds\\HTML5\\Entities' => $vendorDir . '/masterminds/html5/src/HTML5/Entities.php',
    'Masterminds\\HTML5\\Exception' => $vendorDir . '/masterminds/html5/src/HTML5/Exception.php',
    'Masterminds\\HTML5\\InstructionProcessor' => $vendorDir . '/masterminds/html5/src/HTML5/InstructionProcessor.php',
    'Masterminds\\HTML5\\Parser\\CharacterReference' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/CharacterReference.php',
    'Masterminds\\HTML5\\Parser\\DOMTreeBuilder' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/DOMTreeBuilder.php',
    'Masterminds\\HTML5\\Parser\\EventHandler' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/EventHandler.php',
    'Masterminds\\HTML5\\Parser\\FileInputStream' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/FileInputStream.php',
    'Masterminds\\HTML5\\Parser\\InputStream' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/InputStream.php',
    'Masterminds\\HTML5\\Parser\\ParseError' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/ParseError.php',
    'Masterminds\\HTML5\\Parser\\Scanner' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/Scanner.php',
    'Masterminds\\HTML5\\Parser\\StringInputStream' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/StringInputStream.php',
    'Masterminds\\HTML5\\Parser\\Tokenizer' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/Tokenizer.php',
    'Masterminds\\HTML5\\Parser\\TreeBuildingRules' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/TreeBuildingRules.php',
    'Masterminds\\HTML5\\Parser\\UTF8Utils' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/UTF8Utils.php',
    'Masterminds\\HTML5\\Serializer\\HTML5Entities' => $vendorDir . '/masterminds/html5/src/HTML5/Serializer/HTML5Entities.php',
    'Masterminds\\HTML5\\Serializer\\OutputRules' => $vendorDir . '/masterminds/html5/src/HTML5/Serializer/OutputRules.php',
    'Masterminds\\HTML5\\Serializer\\RulesInterface' => $vendorDir . '/masterminds/html5/src/HTML5/Serializer/RulesInterface.php',
    'Masterminds\\HTML5\\Serializer\\Traverser' => $vendorDir . '/masterminds/html5/src/HTML5/Serializer/Traverser.php',
    'Matrix\\Builder' => $vendorDir . '/markbaker/matrix/classes/src/Builder.php',
    'Matrix\\Decomposition\\Decomposition' => $vendorDir . '/markbaker/matrix/classes/src/Decomposition/Decomposition.php',
    'Matrix\\Decomposition\\LU' => $vendorDir . '/markbaker/matrix/classes/src/Decomposition/LU.php',
    'Matrix\\Decomposition\\QR' => $vendorDir . '/markbaker/matrix/classes/src/Decomposition/QR.php',
    'Matrix\\Div0Exception' => $vendorDir . '/markbaker/matrix/classes/src/Div0Exception.php',
    'Matrix\\Exception' => $vendorDir . '/markbaker/matrix/classes/src/Exception.php',
    'Matrix\\Functions' => $vendorDir . '/markbaker/matrix/classes/src/Functions.php',
    'Matrix\\Matrix' => $vendorDir . '/markbaker/matrix/classes/src/Matrix.php',
    'Matrix\\Operations' => $vendorDir . '/markbaker/matrix/classes/src/Operations.php',
    'Matrix\\Operators\\Addition' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Addition.php',
    'Matrix\\Operators\\DirectSum' => $vendorDir . '/markbaker/matrix/classes/src/Operators/DirectSum.php',
    'Matrix\\Operators\\Division' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Division.php',
    'Matrix\\Operators\\Multiplication' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Multiplication.php',
    'Matrix\\Operators\\Operator' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Operator.php',
    'Matrix\\Operators\\Subtraction' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Subtraction.php',
    'MessageBird\\Client' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Client.php',
    'MessageBird\\Common\\Authentication' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Common/Authentication.php',
    'MessageBird\\Common\\HttpClient' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Common/HttpClient.php',
    'MessageBird\\Common\\ResponseError' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Common/ResponseError.php',
    'MessageBird\\Exceptions\\AuthenticateException' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Exceptions/AuthenticateException.php',
    'MessageBird\\Exceptions\\BalanceException' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Exceptions/BalanceException.php',
    'MessageBird\\Exceptions\\HttpException' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Exceptions/HttpException.php',
    'MessageBird\\Exceptions\\MessageBirdException' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Exceptions/MessageBirdException.php',
    'MessageBird\\Exceptions\\RequestException' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Exceptions/RequestException.php',
    'MessageBird\\Exceptions\\ServerException' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Exceptions/ServerException.php',
    'MessageBird\\Exceptions\\ValidationException' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Exceptions/ValidationException.php',
    'MessageBird\\Objects\\Balance' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Balance.php',
    'MessageBird\\Objects\\Base' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Base.php',
    'MessageBird\\Objects\\BaseList' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/BaseList.php',
    'MessageBird\\Objects\\Chat\\Channel' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Chat/Channel.php',
    'MessageBird\\Objects\\Chat\\Contact' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Chat/Contact.php',
    'MessageBird\\Objects\\Chat\\Message' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Chat/Message.php',
    'MessageBird\\Objects\\Chat\\Platform' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Chat/Platform.php',
    'MessageBird\\Objects\\Contact' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Contact.php',
    'MessageBird\\Objects\\Conversation\\Channel' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/Channel.php',
    'MessageBird\\Objects\\Conversation\\Contact' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/Contact.php',
    'MessageBird\\Objects\\Conversation\\Content' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/Content.php',
    'MessageBird\\Objects\\Conversation\\Conversation' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/Conversation.php',
    'MessageBird\\Objects\\Conversation\\Fallback' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/Fallback.php',
    'MessageBird\\Objects\\Conversation\\HSM\\Currency' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/HSM/Currency.php',
    'MessageBird\\Objects\\Conversation\\HSM\\Language' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/HSM/Language.php',
    'MessageBird\\Objects\\Conversation\\HSM\\Message' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/HSM/Message.php',
    'MessageBird\\Objects\\Conversation\\HSM\\Params' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/HSM/Params.php',
    'MessageBird\\Objects\\Conversation\\Message' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/Message.php',
    'MessageBird\\Objects\\Conversation\\MessageReference' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/MessageReference.php',
    'MessageBird\\Objects\\Conversation\\SendMessage' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/SendMessage.php',
    'MessageBird\\Objects\\Conversation\\SendMessageResult' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/SendMessageResult.php',
    'MessageBird\\Objects\\Conversation\\Webhook' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Conversation/Webhook.php',
    'MessageBird\\Objects\\Group' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Group.php',
    'MessageBird\\Objects\\Hlr' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Hlr.php',
    'MessageBird\\Objects\\Lookup' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Lookup.php',
    'MessageBird\\Objects\\Message' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Message.php',
    'MessageBird\\Objects\\MmsMessage' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/MmsMessage.php',
    'MessageBird\\Objects\\Number' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Number.php',
    'MessageBird\\Objects\\NumberPurchaseRequest' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/NumberPurchaseRequest.php',
    'MessageBird\\Objects\\PartnerAccount\\AccessKey' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/PartnerAccount/AccessKey.php',
    'MessageBird\\Objects\\PartnerAccount\\Account' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/PartnerAccount/Account.php',
    'MessageBird\\Objects\\Recipient' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Recipient.php',
    'MessageBird\\Objects\\SignedRequest' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/SignedRequest.php',
    'MessageBird\\Objects\\Verify' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Verify.php',
    'MessageBird\\Objects\\VoiceMessage' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/VoiceMessage.php',
    'MessageBird\\Objects\\Voice\\BaseList' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Voice/BaseList.php',
    'MessageBird\\Objects\\Voice\\Call' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Voice/Call.php',
    'MessageBird\\Objects\\Voice\\CallFlow' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Voice/CallFlow.php',
    'MessageBird\\Objects\\Voice\\Leg' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Voice/Leg.php',
    'MessageBird\\Objects\\Voice\\Recording' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Voice/Recording.php',
    'MessageBird\\Objects\\Voice\\Step' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Voice/Step.php',
    'MessageBird\\Objects\\Voice\\Transcription' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Voice/Transcription.php',
    'MessageBird\\Objects\\Voice\\Webhook' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Objects/Voice/Webhook.php',
    'MessageBird\\RequestValidator' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/RequestValidator.php',
    'MessageBird\\Resources\\AvailablePhoneNumbers' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/AvailablePhoneNumbers.php',
    'MessageBird\\Resources\\Balance' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Balance.php',
    'MessageBird\\Resources\\Base' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Base.php',
    'MessageBird\\Resources\\Chat\\Channel' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Chat/Channel.php',
    'MessageBird\\Resources\\Chat\\Contact' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Chat/Contact.php',
    'MessageBird\\Resources\\Chat\\Message' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Chat/Message.php',
    'MessageBird\\Resources\\Chat\\Platform' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Chat/Platform.php',
    'MessageBird\\Resources\\Contacts' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Contacts.php',
    'MessageBird\\Resources\\Conversation\\Contacts' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Conversation/Contacts.php',
    'MessageBird\\Resources\\Conversation\\Content' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Conversation/Content.php',
    'MessageBird\\Resources\\Conversation\\Conversations' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Conversation/Conversations.php',
    'MessageBird\\Resources\\Conversation\\Messages' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Conversation/Messages.php',
    'MessageBird\\Resources\\Conversation\\Send' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Conversation/Send.php',
    'MessageBird\\Resources\\Conversation\\Webhooks' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Conversation/Webhooks.php',
    'MessageBird\\Resources\\Groups' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Groups.php',
    'MessageBird\\Resources\\Hlr' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Hlr.php',
    'MessageBird\\Resources\\Lookup' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Lookup.php',
    'MessageBird\\Resources\\LookupHlr' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/LookupHlr.php',
    'MessageBird\\Resources\\Messages' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Messages.php',
    'MessageBird\\Resources\\MmsMessages' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/MmsMessages.php',
    'MessageBird\\Resources\\PartnerAccount\\Accounts' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/PartnerAccount/Accounts.php',
    'MessageBird\\Resources\\PhoneNumbers' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/PhoneNumbers.php',
    'MessageBird\\Resources\\Verify' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Verify.php',
    'MessageBird\\Resources\\VoiceMessage' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/VoiceMessage.php',
    'MessageBird\\Resources\\Voice\\Base' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Voice/Base.php',
    'MessageBird\\Resources\\Voice\\CallFlows' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Voice/CallFlows.php',
    'MessageBird\\Resources\\Voice\\Calls' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Voice/Calls.php',
    'MessageBird\\Resources\\Voice\\Legs' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Voice/Legs.php',
    'MessageBird\\Resources\\Voice\\Recordings' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Voice/Recordings.php',
    'MessageBird\\Resources\\Voice\\Transcriptions' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Voice/Transcriptions.php',
    'MessageBird\\Resources\\Voice\\Webhooks' => $vendorDir . '/messagebird/php-rest-api/src/MessageBird/Resources/Voice/Webhooks.php',
    'Mockery\\Adapter\\Phpunit\\MockeryPHPUnitIntegration' => $vendorDir . '/mockery/mockery/library/Mockery/Adapter/Phpunit/MockeryPHPUnitIntegration.php',
    'Mockery\\Adapter\\Phpunit\\MockeryPHPUnitIntegrationAssertPostConditions' => $vendorDir . '/mockery/mockery/library/Mockery/Adapter/Phpunit/MockeryPHPUnitIntegrationAssertPostConditions.php',
    'Mockery\\Adapter\\Phpunit\\MockeryTestCase' => $vendorDir . '/mockery/mockery/library/Mockery/Adapter/Phpunit/MockeryTestCase.php',
    'Mockery\\Adapter\\Phpunit\\MockeryTestCaseSetUp' => $vendorDir . '/mockery/mockery/library/Mockery/Adapter/Phpunit/MockeryTestCaseSetUp.php',
    'Mockery\\Adapter\\Phpunit\\TestListener' => $vendorDir . '/mockery/mockery/library/Mockery/Adapter/Phpunit/TestListener.php',
    'Mockery\\Adapter\\Phpunit\\TestListenerTrait' => $vendorDir . '/mockery/mockery/library/Mockery/Adapter/Phpunit/TestListenerTrait.php',
    'Mockery\\ClosureWrapper' => $vendorDir . '/mockery/mockery/library/Mockery/ClosureWrapper.php',
    'Mockery\\CompositeExpectation' => $vendorDir . '/mockery/mockery/library/Mockery/CompositeExpectation.php',
    'Mockery\\Configuration' => $vendorDir . '/mockery/mockery/library/Mockery/Configuration.php',
    'Mockery\\Container' => $vendorDir . '/mockery/mockery/library/Mockery/Container.php',
    'Mockery\\CountValidator\\AtLeast' => $vendorDir . '/mockery/mockery/library/Mockery/CountValidator/AtLeast.php',
    'Mockery\\CountValidator\\AtMost' => $vendorDir . '/mockery/mockery/library/Mockery/CountValidator/AtMost.php',
    'Mockery\\CountValidator\\CountValidatorAbstract' => $vendorDir . '/mockery/mockery/library/Mockery/CountValidator/CountValidatorAbstract.php',
    'Mockery\\CountValidator\\CountValidatorInterface' => $vendorDir . '/mockery/mockery/library/Mockery/CountValidator/CountValidatorInterface.php',
    'Mockery\\CountValidator\\Exact' => $vendorDir . '/mockery/mockery/library/Mockery/CountValidator/Exact.php',
    'Mockery\\CountValidator\\Exception' => $vendorDir . '/mockery/mockery/library/Mockery/CountValidator/Exception.php',
    'Mockery\\Exception' => $vendorDir . '/mockery/mockery/library/Mockery/Exception.php',
    'Mockery\\Exception\\BadMethodCallException' => $vendorDir . '/mockery/mockery/library/Mockery/Exception/BadMethodCallException.php',
    'Mockery\\Exception\\InvalidArgumentException' => $vendorDir . '/mockery/mockery/library/Mockery/Exception/InvalidArgumentException.php',
    'Mockery\\Exception\\InvalidCountException' => $vendorDir . '/mockery/mockery/library/Mockery/Exception/InvalidCountException.php',
    'Mockery\\Exception\\InvalidOrderException' => $vendorDir . '/mockery/mockery/library/Mockery/Exception/InvalidOrderException.php',
    'Mockery\\Exception\\MockeryExceptionInterface' => $vendorDir . '/mockery/mockery/library/Mockery/Exception/MockeryExceptionInterface.php',
    'Mockery\\Exception\\NoMatchingExpectationException' => $vendorDir . '/mockery/mockery/library/Mockery/Exception/NoMatchingExpectationException.php',
    'Mockery\\Exception\\RuntimeException' => $vendorDir . '/mockery/mockery/library/Mockery/Exception/RuntimeException.php',
    'Mockery\\Expectation' => $vendorDir . '/mockery/mockery/library/Mockery/Expectation.php',
    'Mockery\\ExpectationDirector' => $vendorDir . '/mockery/mockery/library/Mockery/ExpectationDirector.php',
    'Mockery\\ExpectationInterface' => $vendorDir . '/mockery/mockery/library/Mockery/ExpectationInterface.php',
    'Mockery\\ExpectsHigherOrderMessage' => $vendorDir . '/mockery/mockery/library/Mockery/ExpectsHigherOrderMessage.php',
    'Mockery\\Generator\\CachingGenerator' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/CachingGenerator.php',
    'Mockery\\Generator\\DefinedTargetClass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/DefinedTargetClass.php',
    'Mockery\\Generator\\Generator' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/Generator.php',
    'Mockery\\Generator\\Method' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/Method.php',
    'Mockery\\Generator\\MockConfiguration' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/MockConfiguration.php',
    'Mockery\\Generator\\MockConfigurationBuilder' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/MockConfigurationBuilder.php',
    'Mockery\\Generator\\MockDefinition' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/MockDefinition.php',
    'Mockery\\Generator\\MockNameBuilder' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/MockNameBuilder.php',
    'Mockery\\Generator\\Parameter' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/Parameter.php',
    'Mockery\\Generator\\StringManipulationGenerator' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulationGenerator.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\AvoidMethodClashPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/AvoidMethodClashPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\CallTypeHintPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/CallTypeHintPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\ClassAttributesPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/ClassAttributesPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\ClassNamePass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/ClassNamePass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\ClassPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/ClassPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\ConstantsPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/ConstantsPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\InstanceMockPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/InstanceMockPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\InterfacePass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/InterfacePass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\MagicMethodTypeHintsPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/MagicMethodTypeHintsPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\MethodDefinitionPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/MethodDefinitionPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\Pass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/Pass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\RemoveBuiltinMethodsThatAreFinalPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/RemoveBuiltinMethodsThatAreFinalPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\RemoveDestructorPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/RemoveDestructorPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\RemoveUnserializeForInternalSerializableClassesPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/RemoveUnserializeForInternalSerializableClassesPass.php',
    'Mockery\\Generator\\StringManipulation\\Pass\\TraitPass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/StringManipulation/Pass/TraitPass.php',
    'Mockery\\Generator\\TargetClassInterface' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/TargetClassInterface.php',
    'Mockery\\Generator\\UndefinedTargetClass' => $vendorDir . '/mockery/mockery/library/Mockery/Generator/UndefinedTargetClass.php',
    'Mockery\\HigherOrderMessage' => $vendorDir . '/mockery/mockery/library/Mockery/HigherOrderMessage.php',
    'Mockery\\Instantiator' => $vendorDir . '/mockery/mockery/library/Mockery/Instantiator.php',
    'Mockery\\LegacyMockInterface' => $vendorDir . '/mockery/mockery/library/Mockery/LegacyMockInterface.php',
    'Mockery\\Loader\\EvalLoader' => $vendorDir . '/mockery/mockery/library/Mockery/Loader/EvalLoader.php',
    'Mockery\\Loader\\Loader' => $vendorDir . '/mockery/mockery/library/Mockery/Loader/Loader.php',
    'Mockery\\Loader\\RequireLoader' => $vendorDir . '/mockery/mockery/library/Mockery/Loader/RequireLoader.php',
    'Mockery\\Matcher\\AndAnyOtherArgs' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/AndAnyOtherArgs.php',
    'Mockery\\Matcher\\Any' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/Any.php',
    'Mockery\\Matcher\\AnyArgs' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/AnyArgs.php',
    'Mockery\\Matcher\\AnyOf' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/AnyOf.php',
    'Mockery\\Matcher\\ArgumentListMatcher' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/ArgumentListMatcher.php',
    'Mockery\\Matcher\\Closure' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/Closure.php',
    'Mockery\\Matcher\\Contains' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/Contains.php',
    'Mockery\\Matcher\\Ducktype' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/Ducktype.php',
    'Mockery\\Matcher\\HasKey' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/HasKey.php',
    'Mockery\\Matcher\\HasValue' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/HasValue.php',
    'Mockery\\Matcher\\IsEqual' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/IsEqual.php',
    'Mockery\\Matcher\\IsSame' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/IsSame.php',
    'Mockery\\Matcher\\MatcherAbstract' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/MatcherAbstract.php',
    'Mockery\\Matcher\\MatcherInterface' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/MatcherInterface.php',
    'Mockery\\Matcher\\MultiArgumentClosure' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/MultiArgumentClosure.php',
    'Mockery\\Matcher\\MustBe' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/MustBe.php',
    'Mockery\\Matcher\\NoArgs' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/NoArgs.php',
    'Mockery\\Matcher\\Not' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/Not.php',
    'Mockery\\Matcher\\NotAnyOf' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/NotAnyOf.php',
    'Mockery\\Matcher\\Pattern' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/Pattern.php',
    'Mockery\\Matcher\\Subset' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/Subset.php',
    'Mockery\\Matcher\\Type' => $vendorDir . '/mockery/mockery/library/Mockery/Matcher/Type.php',
    'Mockery\\MethodCall' => $vendorDir . '/mockery/mockery/library/Mockery/MethodCall.php',
    'Mockery\\Mock' => $vendorDir . '/mockery/mockery/library/Mockery/Mock.php',
    'Mockery\\MockInterface' => $vendorDir . '/mockery/mockery/library/Mockery/MockInterface.php',
    'Mockery\\QuickDefinitionsConfiguration' => $vendorDir . '/mockery/mockery/library/Mockery/QuickDefinitionsConfiguration.php',
    'Mockery\\ReceivedMethodCalls' => $vendorDir . '/mockery/mockery/library/Mockery/ReceivedMethodCalls.php',
    'Mockery\\Reflector' => $vendorDir . '/mockery/mockery/library/Mockery/Reflector.php',
    'Mockery\\Undefined' => $vendorDir . '/mockery/mockery/library/Mockery/Undefined.php',
    'Mockery\\VerificationDirector' => $vendorDir . '/mockery/mockery/library/Mockery/VerificationDirector.php',
    'Mockery\\VerificationExpectation' => $vendorDir . '/mockery/mockery/library/Mockery/VerificationExpectation.php',
    'Mollie\\Api\\CompatibilityChecker' => $vendorDir . '/mollie/mollie-api-php/src/CompatibilityChecker.php',
    'Mollie\\Api\\Endpoints\\BalanceEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/BalanceEndpoint.php',
    'Mollie\\Api\\Endpoints\\BalanceReportEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/BalanceReportEndpoint.php',
    'Mollie\\Api\\Endpoints\\BalanceTransactionEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/BalanceTransactionEndpoint.php',
    'Mollie\\Api\\Endpoints\\ChargebackEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/ChargebackEndpoint.php',
    'Mollie\\Api\\Endpoints\\ClientEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/ClientEndpoint.php',
    'Mollie\\Api\\Endpoints\\ClientLinkEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/ClientLinkEndpoint.php',
    'Mollie\\Api\\Endpoints\\CollectionEndpointAbstract' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/CollectionEndpointAbstract.php',
    'Mollie\\Api\\Endpoints\\CustomerEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/CustomerEndpoint.php',
    'Mollie\\Api\\Endpoints\\CustomerPaymentsEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/CustomerPaymentsEndpoint.php',
    'Mollie\\Api\\Endpoints\\EndpointAbstract' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/EndpointAbstract.php',
    'Mollie\\Api\\Endpoints\\InvoiceEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/InvoiceEndpoint.php',
    'Mollie\\Api\\Endpoints\\MandateEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/MandateEndpoint.php',
    'Mollie\\Api\\Endpoints\\MethodEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/MethodEndpoint.php',
    'Mollie\\Api\\Endpoints\\MethodIssuerEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/MethodIssuerEndpoint.php',
    'Mollie\\Api\\Endpoints\\OnboardingEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/OnboardingEndpoint.php',
    'Mollie\\Api\\Endpoints\\OrderEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/OrderEndpoint.php',
    'Mollie\\Api\\Endpoints\\OrderLineEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/OrderLineEndpoint.php',
    'Mollie\\Api\\Endpoints\\OrderPaymentEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/OrderPaymentEndpoint.php',
    'Mollie\\Api\\Endpoints\\OrderRefundEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/OrderRefundEndpoint.php',
    'Mollie\\Api\\Endpoints\\OrganizationEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/OrganizationEndpoint.php',
    'Mollie\\Api\\Endpoints\\OrganizationPartnerEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/OrganizationPartnerEndpoint.php',
    'Mollie\\Api\\Endpoints\\PaymentCaptureEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/PaymentCaptureEndpoint.php',
    'Mollie\\Api\\Endpoints\\PaymentChargebackEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/PaymentChargebackEndpoint.php',
    'Mollie\\Api\\Endpoints\\PaymentEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/PaymentEndpoint.php',
    'Mollie\\Api\\Endpoints\\PaymentLinkEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/PaymentLinkEndpoint.php',
    'Mollie\\Api\\Endpoints\\PaymentLinkPaymentEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/PaymentLinkPaymentEndpoint.php',
    'Mollie\\Api\\Endpoints\\PaymentRefundEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/PaymentRefundEndpoint.php',
    'Mollie\\Api\\Endpoints\\PaymentRouteEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/PaymentRouteEndpoint.php',
    'Mollie\\Api\\Endpoints\\PermissionEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/PermissionEndpoint.php',
    'Mollie\\Api\\Endpoints\\ProfileEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/ProfileEndpoint.php',
    'Mollie\\Api\\Endpoints\\ProfileMethodEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/ProfileMethodEndpoint.php',
    'Mollie\\Api\\Endpoints\\RefundEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/RefundEndpoint.php',
    'Mollie\\Api\\Endpoints\\SalesInvoiceEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/SalesInvoiceEndpoint.php',
    'Mollie\\Api\\Endpoints\\SessionEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/SessionEndpoint.php',
    'Mollie\\Api\\Endpoints\\SettlementCaptureEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/SettlementCaptureEndpoint.php',
    'Mollie\\Api\\Endpoints\\SettlementChargebackEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/SettlementChargebackEndpoint.php',
    'Mollie\\Api\\Endpoints\\SettlementPaymentEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/SettlementPaymentEndpoint.php',
    'Mollie\\Api\\Endpoints\\SettlementRefundEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/SettlementRefundEndpoint.php',
    'Mollie\\Api\\Endpoints\\SettlementsEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/SettlementsEndpoint.php',
    'Mollie\\Api\\Endpoints\\ShipmentEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/ShipmentEndpoint.php',
    'Mollie\\Api\\Endpoints\\SubscriptionEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/SubscriptionEndpoint.php',
    'Mollie\\Api\\Endpoints\\SubscriptionPaymentEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/SubscriptionPaymentEndpoint.php',
    'Mollie\\Api\\Endpoints\\TerminalEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/TerminalEndpoint.php',
    'Mollie\\Api\\Endpoints\\WalletEndpoint' => $vendorDir . '/mollie/mollie-api-php/src/Endpoints/WalletEndpoint.php',
    'Mollie\\Api\\Exceptions\\ApiException' => $vendorDir . '/mollie/mollie-api-php/src/Exceptions/ApiException.php',
    'Mollie\\Api\\Exceptions\\CurlConnectTimeoutException' => $vendorDir . '/mollie/mollie-api-php/src/Exceptions/CurlConnectTimeoutException.php',
    'Mollie\\Api\\Exceptions\\HttpAdapterDoesNotSupportDebuggingException' => $vendorDir . '/mollie/mollie-api-php/src/Exceptions/HttpAdapterDoesNotSupportDebuggingException.php',
    'Mollie\\Api\\Exceptions\\IncompatiblePlatform' => $vendorDir . '/mollie/mollie-api-php/src/Exceptions/IncompatiblePlatform.php',
    'Mollie\\Api\\Exceptions\\UnrecognizedClientException' => $vendorDir . '/mollie/mollie-api-php/src/Exceptions/UnrecognizedClientException.php',
    'Mollie\\Api\\HttpAdapter\\CurlMollieHttpAdapter' => $vendorDir . '/mollie/mollie-api-php/src/HttpAdapter/CurlMollieHttpAdapter.php',
    'Mollie\\Api\\HttpAdapter\\Guzzle6And7MollieHttpAdapter' => $vendorDir . '/mollie/mollie-api-php/src/HttpAdapter/Guzzle6And7MollieHttpAdapter.php',
    'Mollie\\Api\\HttpAdapter\\Guzzle6And7RetryMiddlewareFactory' => $vendorDir . '/mollie/mollie-api-php/src/HttpAdapter/Guzzle6And7RetryMiddlewareFactory.php',
    'Mollie\\Api\\HttpAdapter\\MollieHttpAdapterInterface' => $vendorDir . '/mollie/mollie-api-php/src/HttpAdapter/MollieHttpAdapterInterface.php',
    'Mollie\\Api\\HttpAdapter\\MollieHttpAdapterPicker' => $vendorDir . '/mollie/mollie-api-php/src/HttpAdapter/MollieHttpAdapterPicker.php',
    'Mollie\\Api\\HttpAdapter\\MollieHttpAdapterPickerInterface' => $vendorDir . '/mollie/mollie-api-php/src/HttpAdapter/MollieHttpAdapterPickerInterface.php',
    'Mollie\\Api\\Idempotency\\DefaultIdempotencyKeyGenerator' => $vendorDir . '/mollie/mollie-api-php/src/Idempotency/DefaultIdempotencyKeyGenerator.php',
    'Mollie\\Api\\Idempotency\\FakeIdempotencyKeyGenerator' => $vendorDir . '/mollie/mollie-api-php/src/Idempotency/FakeIdempotencyKeyGenerator.php',
    'Mollie\\Api\\Idempotency\\IdempotencyKeyGeneratorContract' => $vendorDir . '/mollie/mollie-api-php/src/Idempotency/IdempotencyKeyGeneratorContract.php',
    'Mollie\\Api\\MollieApiClient' => $vendorDir . '/mollie/mollie-api-php/src/MollieApiClient.php',
    'Mollie\\Api\\Resources\\Balance' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Balance.php',
    'Mollie\\Api\\Resources\\BalanceCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/BalanceCollection.php',
    'Mollie\\Api\\Resources\\BalanceReport' => $vendorDir . '/mollie/mollie-api-php/src/Resources/BalanceReport.php',
    'Mollie\\Api\\Resources\\BalanceTransaction' => $vendorDir . '/mollie/mollie-api-php/src/Resources/BalanceTransaction.php',
    'Mollie\\Api\\Resources\\BalanceTransactionCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/BalanceTransactionCollection.php',
    'Mollie\\Api\\Resources\\BaseCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/BaseCollection.php',
    'Mollie\\Api\\Resources\\BaseResource' => $vendorDir . '/mollie/mollie-api-php/src/Resources/BaseResource.php',
    'Mollie\\Api\\Resources\\Capture' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Capture.php',
    'Mollie\\Api\\Resources\\CaptureCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/CaptureCollection.php',
    'Mollie\\Api\\Resources\\Chargeback' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Chargeback.php',
    'Mollie\\Api\\Resources\\ChargebackCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/ChargebackCollection.php',
    'Mollie\\Api\\Resources\\Client' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Client.php',
    'Mollie\\Api\\Resources\\ClientCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/ClientCollection.php',
    'Mollie\\Api\\Resources\\ClientLink' => $vendorDir . '/mollie/mollie-api-php/src/Resources/ClientLink.php',
    'Mollie\\Api\\Resources\\CurrentProfile' => $vendorDir . '/mollie/mollie-api-php/src/Resources/CurrentProfile.php',
    'Mollie\\Api\\Resources\\CursorCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/CursorCollection.php',
    'Mollie\\Api\\Resources\\Customer' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Customer.php',
    'Mollie\\Api\\Resources\\CustomerCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/CustomerCollection.php',
    'Mollie\\Api\\Resources\\HasPresetOptions' => $vendorDir . '/mollie/mollie-api-php/src/Resources/HasPresetOptions.php',
    'Mollie\\Api\\Resources\\Invoice' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Invoice.php',
    'Mollie\\Api\\Resources\\InvoiceCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/InvoiceCollection.php',
    'Mollie\\Api\\Resources\\Issuer' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Issuer.php',
    'Mollie\\Api\\Resources\\IssuerCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/IssuerCollection.php',
    'Mollie\\Api\\Resources\\LazyCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/LazyCollection.php',
    'Mollie\\Api\\Resources\\Mandate' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Mandate.php',
    'Mollie\\Api\\Resources\\MandateCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/MandateCollection.php',
    'Mollie\\Api\\Resources\\Method' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Method.php',
    'Mollie\\Api\\Resources\\MethodCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/MethodCollection.php',
    'Mollie\\Api\\Resources\\MethodPrice' => $vendorDir . '/mollie/mollie-api-php/src/Resources/MethodPrice.php',
    'Mollie\\Api\\Resources\\MethodPriceCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/MethodPriceCollection.php',
    'Mollie\\Api\\Resources\\Onboarding' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Onboarding.php',
    'Mollie\\Api\\Resources\\Order' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Order.php',
    'Mollie\\Api\\Resources\\OrderCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/OrderCollection.php',
    'Mollie\\Api\\Resources\\OrderLine' => $vendorDir . '/mollie/mollie-api-php/src/Resources/OrderLine.php',
    'Mollie\\Api\\Resources\\OrderLineCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/OrderLineCollection.php',
    'Mollie\\Api\\Resources\\Organization' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Organization.php',
    'Mollie\\Api\\Resources\\OrganizationCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/OrganizationCollection.php',
    'Mollie\\Api\\Resources\\Partner' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Partner.php',
    'Mollie\\Api\\Resources\\Payment' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Payment.php',
    'Mollie\\Api\\Resources\\PaymentCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/PaymentCollection.php',
    'Mollie\\Api\\Resources\\PaymentLink' => $vendorDir . '/mollie/mollie-api-php/src/Resources/PaymentLink.php',
    'Mollie\\Api\\Resources\\PaymentLinkCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/PaymentLinkCollection.php',
    'Mollie\\Api\\Resources\\Permission' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Permission.php',
    'Mollie\\Api\\Resources\\PermissionCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/PermissionCollection.php',
    'Mollie\\Api\\Resources\\Profile' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Profile.php',
    'Mollie\\Api\\Resources\\ProfileCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/ProfileCollection.php',
    'Mollie\\Api\\Resources\\Refund' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Refund.php',
    'Mollie\\Api\\Resources\\RefundCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/RefundCollection.php',
    'Mollie\\Api\\Resources\\ResourceFactory' => $vendorDir . '/mollie/mollie-api-php/src/Resources/ResourceFactory.php',
    'Mollie\\Api\\Resources\\Route' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Route.php',
    'Mollie\\Api\\Resources\\RouteCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/RouteCollection.php',
    'Mollie\\Api\\Resources\\SalesInvoice' => $vendorDir . '/mollie/mollie-api-php/src/Resources/SalesInvoice.php',
    'Mollie\\Api\\Resources\\SalesInvoiceCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/SalesInvoiceCollection.php',
    'Mollie\\Api\\Resources\\Session' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Session.php',
    'Mollie\\Api\\Resources\\SessionCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/SessionCollection.php',
    'Mollie\\Api\\Resources\\Settlement' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Settlement.php',
    'Mollie\\Api\\Resources\\SettlementCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/SettlementCollection.php',
    'Mollie\\Api\\Resources\\Shipment' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Shipment.php',
    'Mollie\\Api\\Resources\\ShipmentCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/ShipmentCollection.php',
    'Mollie\\Api\\Resources\\Subscription' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Subscription.php',
    'Mollie\\Api\\Resources\\SubscriptionCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/SubscriptionCollection.php',
    'Mollie\\Api\\Resources\\Terminal' => $vendorDir . '/mollie/mollie-api-php/src/Resources/Terminal.php',
    'Mollie\\Api\\Resources\\TerminalCollection' => $vendorDir . '/mollie/mollie-api-php/src/Resources/TerminalCollection.php',
    'Mollie\\Api\\Types\\ApprovalPrompt' => $vendorDir . '/mollie/mollie-api-php/src/Types/ApprovalPrompt.php',
    'Mollie\\Api\\Types\\BalanceTransferFrequency' => $vendorDir . '/mollie/mollie-api-php/src/Types/BalanceTransferFrequency.php',
    'Mollie\\Api\\Types\\InvoiceStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/InvoiceStatus.php',
    'Mollie\\Api\\Types\\MandateMethod' => $vendorDir . '/mollie/mollie-api-php/src/Types/MandateMethod.php',
    'Mollie\\Api\\Types\\MandateStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/MandateStatus.php',
    'Mollie\\Api\\Types\\OnboardingStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/OnboardingStatus.php',
    'Mollie\\Api\\Types\\OrderLineStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/OrderLineStatus.php',
    'Mollie\\Api\\Types\\OrderLineType' => $vendorDir . '/mollie/mollie-api-php/src/Types/OrderLineType.php',
    'Mollie\\Api\\Types\\OrderLineUpdateOperationType' => $vendorDir . '/mollie/mollie-api-php/src/Types/OrderLineUpdateOperationType.php',
    'Mollie\\Api\\Types\\OrderStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/OrderStatus.php',
    'Mollie\\Api\\Types\\PaymentMethod' => $vendorDir . '/mollie/mollie-api-php/src/Types/PaymentMethod.php',
    'Mollie\\Api\\Types\\PaymentMethodStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/PaymentMethodStatus.php',
    'Mollie\\Api\\Types\\PaymentStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/PaymentStatus.php',
    'Mollie\\Api\\Types\\PaymentTerm' => $vendorDir . '/mollie/mollie-api-php/src/Types/PaymentTerm.php',
    'Mollie\\Api\\Types\\ProfileStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/ProfileStatus.php',
    'Mollie\\Api\\Types\\RefundStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/RefundStatus.php',
    'Mollie\\Api\\Types\\SalesInvoiceStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/SalesInvoiceStatus.php',
    'Mollie\\Api\\Types\\SequenceType' => $vendorDir . '/mollie/mollie-api-php/src/Types/SequenceType.php',
    'Mollie\\Api\\Types\\SessionStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/SessionStatus.php',
    'Mollie\\Api\\Types\\SettlementStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/SettlementStatus.php',
    'Mollie\\Api\\Types\\SubscriptionStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/SubscriptionStatus.php',
    'Mollie\\Api\\Types\\TerminalStatus' => $vendorDir . '/mollie/mollie-api-php/src/Types/TerminalStatus.php',
    'Mollie\\Laravel\\Facades\\Mollie' => $vendorDir . '/mollie/laravel-mollie/src/Facades/Mollie.php',
    'Mollie\\Laravel\\MollieConnectProvider' => $vendorDir . '/mollie/laravel-mollie/src/MollieConnectProvider.php',
    'Mollie\\Laravel\\MollieLaravelHttpClientAdapter' => $vendorDir . '/mollie/laravel-mollie/src/MollieLaravelHttpClientAdapter.php',
    'Mollie\\Laravel\\MollieManager' => $vendorDir . '/mollie/laravel-mollie/src/MollieManager.php',
    'Mollie\\Laravel\\MollieServiceProvider' => $vendorDir . '/mollie/laravel-mollie/src/MollieServiceProvider.php',
    'Mollie\\Laravel\\Wrappers\\MollieApiWrapper' => $vendorDir . '/mollie/laravel-mollie/src/Wrappers/MollieApiWrapper.php',
    'Monolog\\Attribute\\AsMonologProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Attribute/AsMonologProcessor.php',
    'Monolog\\Attribute\\WithMonologChannel' => $vendorDir . '/monolog/monolog/src/Monolog/Attribute/WithMonologChannel.php',
    'Monolog\\DateTimeImmutable' => $vendorDir . '/monolog/monolog/src/Monolog/DateTimeImmutable.php',
    'Monolog\\ErrorHandler' => $vendorDir . '/monolog/monolog/src/Monolog/ErrorHandler.php',
    'Monolog\\Formatter\\ChromePHPFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/ChromePHPFormatter.php',
    'Monolog\\Formatter\\ElasticaFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/ElasticaFormatter.php',
    'Monolog\\Formatter\\ElasticsearchFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/ElasticsearchFormatter.php',
    'Monolog\\Formatter\\FlowdockFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/FlowdockFormatter.php',
    'Monolog\\Formatter\\FluentdFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/FluentdFormatter.php',
    'Monolog\\Formatter\\FormatterInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/FormatterInterface.php',
    'Monolog\\Formatter\\GelfMessageFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/GelfMessageFormatter.php',
    'Monolog\\Formatter\\GoogleCloudLoggingFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/GoogleCloudLoggingFormatter.php',
    'Monolog\\Formatter\\HtmlFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/HtmlFormatter.php',
    'Monolog\\Formatter\\JsonFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/JsonFormatter.php',
    'Monolog\\Formatter\\LineFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/LineFormatter.php',
    'Monolog\\Formatter\\LogglyFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/LogglyFormatter.php',
    'Monolog\\Formatter\\LogmaticFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/LogmaticFormatter.php',
    'Monolog\\Formatter\\LogstashFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/LogstashFormatter.php',
    'Monolog\\Formatter\\MongoDBFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/MongoDBFormatter.php',
    'Monolog\\Formatter\\NormalizerFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/NormalizerFormatter.php',
    'Monolog\\Formatter\\ScalarFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/ScalarFormatter.php',
    'Monolog\\Formatter\\SyslogFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/SyslogFormatter.php',
    'Monolog\\Formatter\\WildfireFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/WildfireFormatter.php',
    'Monolog\\Handler\\AbstractHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/AbstractHandler.php',
    'Monolog\\Handler\\AbstractProcessingHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/AbstractProcessingHandler.php',
    'Monolog\\Handler\\AbstractSyslogHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/AbstractSyslogHandler.php',
    'Monolog\\Handler\\AmqpHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/AmqpHandler.php',
    'Monolog\\Handler\\BrowserConsoleHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/BrowserConsoleHandler.php',
    'Monolog\\Handler\\BufferHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/BufferHandler.php',
    'Monolog\\Handler\\ChromePHPHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ChromePHPHandler.php',
    'Monolog\\Handler\\CouchDBHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/CouchDBHandler.php',
    'Monolog\\Handler\\CubeHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/CubeHandler.php',
    'Monolog\\Handler\\Curl\\Util' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/Curl/Util.php',
    'Monolog\\Handler\\DeduplicationHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/DeduplicationHandler.php',
    'Monolog\\Handler\\DoctrineCouchDBHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/DoctrineCouchDBHandler.php',
    'Monolog\\Handler\\DynamoDbHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/DynamoDbHandler.php',
    'Monolog\\Handler\\ElasticaHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ElasticaHandler.php',
    'Monolog\\Handler\\ElasticsearchHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ElasticsearchHandler.php',
    'Monolog\\Handler\\ErrorLogHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ErrorLogHandler.php',
    'Monolog\\Handler\\FallbackGroupHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FallbackGroupHandler.php',
    'Monolog\\Handler\\FilterHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FilterHandler.php',
    'Monolog\\Handler\\FingersCrossedHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FingersCrossedHandler.php',
    'Monolog\\Handler\\FingersCrossed\\ActivationStrategyInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FingersCrossed/ActivationStrategyInterface.php',
    'Monolog\\Handler\\FingersCrossed\\ChannelLevelActivationStrategy' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FingersCrossed/ChannelLevelActivationStrategy.php',
    'Monolog\\Handler\\FingersCrossed\\ErrorLevelActivationStrategy' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FingersCrossed/ErrorLevelActivationStrategy.php',
    'Monolog\\Handler\\FirePHPHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FirePHPHandler.php',
    'Monolog\\Handler\\FleepHookHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FleepHookHandler.php',
    'Monolog\\Handler\\FlowdockHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FlowdockHandler.php',
    'Monolog\\Handler\\FormattableHandlerInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FormattableHandlerInterface.php',
    'Monolog\\Handler\\FormattableHandlerTrait' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FormattableHandlerTrait.php',
    'Monolog\\Handler\\GelfHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/GelfHandler.php',
    'Monolog\\Handler\\GroupHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/GroupHandler.php',
    'Monolog\\Handler\\Handler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/Handler.php',
    'Monolog\\Handler\\HandlerInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/HandlerInterface.php',
    'Monolog\\Handler\\HandlerWrapper' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/HandlerWrapper.php',
    'Monolog\\Handler\\IFTTTHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/IFTTTHandler.php',
    'Monolog\\Handler\\InsightOpsHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/InsightOpsHandler.php',
    'Monolog\\Handler\\LogEntriesHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/LogEntriesHandler.php',
    'Monolog\\Handler\\LogglyHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/LogglyHandler.php',
    'Monolog\\Handler\\LogmaticHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/LogmaticHandler.php',
    'Monolog\\Handler\\MailHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/MailHandler.php',
    'Monolog\\Handler\\MandrillHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/MandrillHandler.php',
    'Monolog\\Handler\\MissingExtensionException' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/MissingExtensionException.php',
    'Monolog\\Handler\\MongoDBHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/MongoDBHandler.php',
    'Monolog\\Handler\\NativeMailerHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/NativeMailerHandler.php',
    'Monolog\\Handler\\NewRelicHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/NewRelicHandler.php',
    'Monolog\\Handler\\NoopHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/NoopHandler.php',
    'Monolog\\Handler\\NullHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/NullHandler.php',
    'Monolog\\Handler\\OverflowHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/OverflowHandler.php',
    'Monolog\\Handler\\PHPConsoleHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/PHPConsoleHandler.php',
    'Monolog\\Handler\\ProcessHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ProcessHandler.php',
    'Monolog\\Handler\\ProcessableHandlerInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ProcessableHandlerInterface.php',
    'Monolog\\Handler\\ProcessableHandlerTrait' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ProcessableHandlerTrait.php',
    'Monolog\\Handler\\PsrHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/PsrHandler.php',
    'Monolog\\Handler\\PushoverHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/PushoverHandler.php',
    'Monolog\\Handler\\RedisHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/RedisHandler.php',
    'Monolog\\Handler\\RedisPubSubHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/RedisPubSubHandler.php',
    'Monolog\\Handler\\RollbarHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/RollbarHandler.php',
    'Monolog\\Handler\\RotatingFileHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/RotatingFileHandler.php',
    'Monolog\\Handler\\SamplingHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SamplingHandler.php',
    'Monolog\\Handler\\SendGridHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SendGridHandler.php',
    'Monolog\\Handler\\SlackHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SlackHandler.php',
    'Monolog\\Handler\\SlackWebhookHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SlackWebhookHandler.php',
    'Monolog\\Handler\\Slack\\SlackRecord' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/Slack/SlackRecord.php',
    'Monolog\\Handler\\SocketHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SocketHandler.php',
    'Monolog\\Handler\\SqsHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SqsHandler.php',
    'Monolog\\Handler\\StreamHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/StreamHandler.php',
    'Monolog\\Handler\\SymfonyMailerHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SymfonyMailerHandler.php',
    'Monolog\\Handler\\SyslogHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SyslogHandler.php',
    'Monolog\\Handler\\SyslogUdpHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SyslogUdpHandler.php',
    'Monolog\\Handler\\SyslogUdp\\UdpSocket' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SyslogUdp/UdpSocket.php',
    'Monolog\\Handler\\TelegramBotHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/TelegramBotHandler.php',
    'Monolog\\Handler\\TestHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/TestHandler.php',
    'Monolog\\Handler\\WebRequestRecognizerTrait' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/WebRequestRecognizerTrait.php',
    'Monolog\\Handler\\WhatFailureGroupHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/WhatFailureGroupHandler.php',
    'Monolog\\Handler\\ZendMonitorHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ZendMonitorHandler.php',
    'Monolog\\JsonSerializableDateTimeImmutable' => $vendorDir . '/monolog/monolog/src/Monolog/JsonSerializableDateTimeImmutable.php',
    'Monolog\\Level' => $vendorDir . '/monolog/monolog/src/Monolog/Level.php',
    'Monolog\\LogRecord' => $vendorDir . '/monolog/monolog/src/Monolog/LogRecord.php',
    'Monolog\\Logger' => $vendorDir . '/monolog/monolog/src/Monolog/Logger.php',
    'Monolog\\Processor\\ClosureContextProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/ClosureContextProcessor.php',
    'Monolog\\Processor\\GitProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/GitProcessor.php',
    'Monolog\\Processor\\HostnameProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/HostnameProcessor.php',
    'Monolog\\Processor\\IntrospectionProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/IntrospectionProcessor.php',
    'Monolog\\Processor\\LoadAverageProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/LoadAverageProcessor.php',
    'Monolog\\Processor\\MemoryPeakUsageProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/MemoryPeakUsageProcessor.php',
    'Monolog\\Processor\\MemoryProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/MemoryProcessor.php',
    'Monolog\\Processor\\MemoryUsageProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/MemoryUsageProcessor.php',
    'Monolog\\Processor\\MercurialProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/MercurialProcessor.php',
    'Monolog\\Processor\\ProcessIdProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/ProcessIdProcessor.php',
    'Monolog\\Processor\\ProcessorInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/ProcessorInterface.php',
    'Monolog\\Processor\\PsrLogMessageProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/PsrLogMessageProcessor.php',
    'Monolog\\Processor\\TagProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/TagProcessor.php',
    'Monolog\\Processor\\UidProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/UidProcessor.php',
    'Monolog\\Processor\\WebProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/WebProcessor.php',
    'Monolog\\Registry' => $vendorDir . '/monolog/monolog/src/Monolog/Registry.php',
    'Monolog\\ResettableInterface' => $vendorDir . '/monolog/monolog/src/Monolog/ResettableInterface.php',
    'Monolog\\SignalHandler' => $vendorDir . '/monolog/monolog/src/Monolog/SignalHandler.php',
    'Monolog\\Test\\TestCase' => $vendorDir . '/monolog/monolog/src/Monolog/Test/TestCase.php',
    'Monolog\\Utils' => $vendorDir . '/monolog/monolog/src/Monolog/Utils.php',
    'Nette\\ArgumentOutOfRangeException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\DeprecatedException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\DirectoryNotFoundException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\FileNotFoundException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\HtmlStringable' => $vendorDir . '/nette/utils/src/HtmlStringable.php',
    'Nette\\IOException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\InvalidArgumentException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\InvalidStateException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\Iterators\\CachingIterator' => $vendorDir . '/nette/utils/src/Iterators/CachingIterator.php',
    'Nette\\Iterators\\Mapper' => $vendorDir . '/nette/utils/src/Iterators/Mapper.php',
    'Nette\\Localization\\ITranslator' => $vendorDir . '/nette/utils/src/compatibility.php',
    'Nette\\Localization\\Translator' => $vendorDir . '/nette/utils/src/Translator.php',
    'Nette\\MemberAccessException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\NotImplementedException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\NotSupportedException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\OutOfRangeException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\Schema\\Context' => $vendorDir . '/nette/schema/src/Schema/Context.php',
    'Nette\\Schema\\DynamicParameter' => $vendorDir . '/nette/schema/src/Schema/DynamicParameter.php',
    'Nette\\Schema\\Elements\\AnyOf' => $vendorDir . '/nette/schema/src/Schema/Elements/AnyOf.php',
    'Nette\\Schema\\Elements\\Base' => $vendorDir . '/nette/schema/src/Schema/Elements/Base.php',
    'Nette\\Schema\\Elements\\Structure' => $vendorDir . '/nette/schema/src/Schema/Elements/Structure.php',
    'Nette\\Schema\\Elements\\Type' => $vendorDir . '/nette/schema/src/Schema/Elements/Type.php',
    'Nette\\Schema\\Expect' => $vendorDir . '/nette/schema/src/Schema/Expect.php',
    'Nette\\Schema\\Helpers' => $vendorDir . '/nette/schema/src/Schema/Helpers.php',
    'Nette\\Schema\\Message' => $vendorDir . '/nette/schema/src/Schema/Message.php',
    'Nette\\Schema\\Processor' => $vendorDir . '/nette/schema/src/Schema/Processor.php',
    'Nette\\Schema\\Schema' => $vendorDir . '/nette/schema/src/Schema/Schema.php',
    'Nette\\Schema\\ValidationException' => $vendorDir . '/nette/schema/src/Schema/ValidationException.php',
    'Nette\\SmartObject' => $vendorDir . '/nette/utils/src/SmartObject.php',
    'Nette\\StaticClass' => $vendorDir . '/nette/utils/src/StaticClass.php',
    'Nette\\UnexpectedValueException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\Utils\\ArrayHash' => $vendorDir . '/nette/utils/src/Utils/ArrayHash.php',
    'Nette\\Utils\\ArrayList' => $vendorDir . '/nette/utils/src/Utils/ArrayList.php',
    'Nette\\Utils\\Arrays' => $vendorDir . '/nette/utils/src/Utils/Arrays.php',
    'Nette\\Utils\\AssertionException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\Callback' => $vendorDir . '/nette/utils/src/Utils/Callback.php',
    'Nette\\Utils\\DateTime' => $vendorDir . '/nette/utils/src/Utils/DateTime.php',
    'Nette\\Utils\\FileInfo' => $vendorDir . '/nette/utils/src/Utils/FileInfo.php',
    'Nette\\Utils\\FileSystem' => $vendorDir . '/nette/utils/src/Utils/FileSystem.php',
    'Nette\\Utils\\Finder' => $vendorDir . '/nette/utils/src/Utils/Finder.php',
    'Nette\\Utils\\Floats' => $vendorDir . '/nette/utils/src/Utils/Floats.php',
    'Nette\\Utils\\Helpers' => $vendorDir . '/nette/utils/src/Utils/Helpers.php',
    'Nette\\Utils\\Html' => $vendorDir . '/nette/utils/src/Utils/Html.php',
    'Nette\\Utils\\IHtmlString' => $vendorDir . '/nette/utils/src/compatibility.php',
    'Nette\\Utils\\Image' => $vendorDir . '/nette/utils/src/Utils/Image.php',
    'Nette\\Utils\\ImageColor' => $vendorDir . '/nette/utils/src/Utils/ImageColor.php',
    'Nette\\Utils\\ImageException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\ImageType' => $vendorDir . '/nette/utils/src/Utils/ImageType.php',
    'Nette\\Utils\\Iterables' => $vendorDir . '/nette/utils/src/Utils/Iterables.php',
    'Nette\\Utils\\Json' => $vendorDir . '/nette/utils/src/Utils/Json.php',
    'Nette\\Utils\\JsonException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\ObjectHelpers' => $vendorDir . '/nette/utils/src/Utils/ObjectHelpers.php',
    'Nette\\Utils\\Paginator' => $vendorDir . '/nette/utils/src/Utils/Paginator.php',
    'Nette\\Utils\\Random' => $vendorDir . '/nette/utils/src/Utils/Random.php',
    'Nette\\Utils\\Reflection' => $vendorDir . '/nette/utils/src/Utils/Reflection.php',
    'Nette\\Utils\\ReflectionMethod' => $vendorDir . '/nette/utils/src/Utils/ReflectionMethod.php',
    'Nette\\Utils\\RegexpException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\Strings' => $vendorDir . '/nette/utils/src/Utils/Strings.php',
    'Nette\\Utils\\Type' => $vendorDir . '/nette/utils/src/Utils/Type.php',
    'Nette\\Utils\\UnknownImageFileException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\Validators' => $vendorDir . '/nette/utils/src/Utils/Validators.php',
    'Normalizer' => $vendorDir . '/symfony/polyfill-intl-normalizer/Resources/stubs/Normalizer.php',
    'NunoMaduro\\Collision\\Adapters\\Laravel\\CollisionServiceProvider' => $vendorDir . '/nunomaduro/collision/src/Adapters/Laravel/CollisionServiceProvider.php',
    'NunoMaduro\\Collision\\Adapters\\Laravel\\Commands\\TestCommand' => $vendorDir . '/nunomaduro/collision/src/Adapters/Laravel/Commands/TestCommand.php',
    'NunoMaduro\\Collision\\Adapters\\Laravel\\ExceptionHandler' => $vendorDir . '/nunomaduro/collision/src/Adapters/Laravel/ExceptionHandler.php',
    'NunoMaduro\\Collision\\Adapters\\Laravel\\Exceptions\\NotSupportedYetException' => $vendorDir . '/nunomaduro/collision/src/Adapters/Laravel/Exceptions/NotSupportedYetException.php',
    'NunoMaduro\\Collision\\Adapters\\Laravel\\Exceptions\\RequirementsException' => $vendorDir . '/nunomaduro/collision/src/Adapters/Laravel/Exceptions/RequirementsException.php',
    'NunoMaduro\\Collision\\Adapters\\Laravel\\IgnitionSolutionsRepository' => $vendorDir . '/nunomaduro/collision/src/Adapters/Laravel/IgnitionSolutionsRepository.php',
    'NunoMaduro\\Collision\\Adapters\\Laravel\\Inspector' => $vendorDir . '/nunomaduro/collision/src/Adapters/Laravel/Inspector.php',
    'NunoMaduro\\Collision\\Adapters\\Phpunit\\ConfigureIO' => $vendorDir . '/nunomaduro/collision/src/Adapters/Phpunit/ConfigureIO.php',
    'NunoMaduro\\Collision\\Adapters\\Phpunit\\Printers\\DefaultPrinter' => $vendorDir . '/nunomaduro/collision/src/Adapters/Phpunit/Printers/DefaultPrinter.php',
    'NunoMaduro\\Collision\\Adapters\\Phpunit\\Printers\\ReportablePrinter' => $vendorDir . '/nunomaduro/collision/src/Adapters/Phpunit/Printers/ReportablePrinter.php',
    'NunoMaduro\\Collision\\Adapters\\Phpunit\\State' => $vendorDir . '/nunomaduro/collision/src/Adapters/Phpunit/State.php',
    'NunoMaduro\\Collision\\Adapters\\Phpunit\\Style' => $vendorDir . '/nunomaduro/collision/src/Adapters/Phpunit/Style.php',
    'NunoMaduro\\Collision\\Adapters\\Phpunit\\Subscribers\\EnsurePrinterIsRegisteredSubscriber' => $vendorDir . '/nunomaduro/collision/src/Adapters/Phpunit/Subscribers/EnsurePrinterIsRegisteredSubscriber.php',
    'NunoMaduro\\Collision\\Adapters\\Phpunit\\Subscribers\\Subscriber' => $vendorDir . '/nunomaduro/collision/src/Adapters/Phpunit/Subscribers/Subscriber.php',
    'NunoMaduro\\Collision\\Adapters\\Phpunit\\Support\\ResultReflection' => $vendorDir . '/nunomaduro/collision/src/Adapters/Phpunit/Support/ResultReflection.php',
    'NunoMaduro\\Collision\\Adapters\\Phpunit\\TestResult' => $vendorDir . '/nunomaduro/collision/src/Adapters/Phpunit/TestResult.php',
    'NunoMaduro\\Collision\\ArgumentFormatter' => $vendorDir . '/nunomaduro/collision/src/ArgumentFormatter.php',
    'NunoMaduro\\Collision\\ConsoleColor' => $vendorDir . '/nunomaduro/collision/src/ConsoleColor.php',
    'NunoMaduro\\Collision\\Contracts\\Adapters\\Phpunit\\HasPrintableTestCaseName' => $vendorDir . '/nunomaduro/collision/src/Contracts/Adapters/Phpunit/HasPrintableTestCaseName.php',
    'NunoMaduro\\Collision\\Contracts\\RenderableOnCollisionEditor' => $vendorDir . '/nunomaduro/collision/src/Contracts/RenderableOnCollisionEditor.php',
    'NunoMaduro\\Collision\\Contracts\\RenderlessEditor' => $vendorDir . '/nunomaduro/collision/src/Contracts/RenderlessEditor.php',
    'NunoMaduro\\Collision\\Contracts\\RenderlessTrace' => $vendorDir . '/nunomaduro/collision/src/Contracts/RenderlessTrace.php',
    'NunoMaduro\\Collision\\Contracts\\SolutionsRepository' => $vendorDir . '/nunomaduro/collision/src/Contracts/SolutionsRepository.php',
    'NunoMaduro\\Collision\\Coverage' => $vendorDir . '/nunomaduro/collision/src/Coverage.php',
    'NunoMaduro\\Collision\\Exceptions\\InvalidStyleException' => $vendorDir . '/nunomaduro/collision/src/Exceptions/InvalidStyleException.php',
    'NunoMaduro\\Collision\\Exceptions\\ShouldNotHappen' => $vendorDir . '/nunomaduro/collision/src/Exceptions/ShouldNotHappen.php',
    'NunoMaduro\\Collision\\Exceptions\\TestException' => $vendorDir . '/nunomaduro/collision/src/Exceptions/TestException.php',
    'NunoMaduro\\Collision\\Exceptions\\TestOutcome' => $vendorDir . '/nunomaduro/collision/src/Exceptions/TestOutcome.php',
    'NunoMaduro\\Collision\\Handler' => $vendorDir . '/nunomaduro/collision/src/Handler.php',
    'NunoMaduro\\Collision\\Highlighter' => $vendorDir . '/nunomaduro/collision/src/Highlighter.php',
    'NunoMaduro\\Collision\\Provider' => $vendorDir . '/nunomaduro/collision/src/Provider.php',
    'NunoMaduro\\Collision\\SolutionsRepositories\\NullSolutionsRepository' => $vendorDir . '/nunomaduro/collision/src/SolutionsRepositories/NullSolutionsRepository.php',
    'NunoMaduro\\Collision\\Writer' => $vendorDir . '/nunomaduro/collision/src/Writer.php',
    'Override' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/Override.php',
    'PHPMailer\\PHPMailer\\DSNConfigurator' => $vendorDir . '/phpmailer/phpmailer/src/DSNConfigurator.php',
    'PHPMailer\\PHPMailer\\Exception' => $vendorDir . '/phpmailer/phpmailer/src/Exception.php',
    'PHPMailer\\PHPMailer\\OAuth' => $vendorDir . '/phpmailer/phpmailer/src/OAuth.php',
    'PHPMailer\\PHPMailer\\OAuthTokenProvider' => $vendorDir . '/phpmailer/phpmailer/src/OAuthTokenProvider.php',
    'PHPMailer\\PHPMailer\\PHPMailer' => $vendorDir . '/phpmailer/phpmailer/src/PHPMailer.php',
    'PHPMailer\\PHPMailer\\POP3' => $vendorDir . '/phpmailer/phpmailer/src/POP3.php',
    'PHPMailer\\PHPMailer\\SMTP' => $vendorDir . '/phpmailer/phpmailer/src/SMTP.php',
    'PHPUnit\\Event\\Application\\Finished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Application/Finished.php',
    'PHPUnit\\Event\\Application\\FinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Application/FinishedSubscriber.php',
    'PHPUnit\\Event\\Application\\Started' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Application/Started.php',
    'PHPUnit\\Event\\Application\\StartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Application/StartedSubscriber.php',
    'PHPUnit\\Event\\Code\\ClassMethod' => $vendorDir . '/phpunit/phpunit/src/Event/Value/ClassMethod.php',
    'PHPUnit\\Event\\Code\\ComparisonFailure' => $vendorDir . '/phpunit/phpunit/src/Event/Value/ComparisonFailure.php',
    'PHPUnit\\Event\\Code\\ComparisonFailureBuilder' => $vendorDir . '/phpunit/phpunit/src/Event/Value/ComparisonFailureBuilder.php',
    'PHPUnit\\Event\\Code\\NoTestCaseObjectOnCallStackException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/NoTestCaseObjectOnCallStackException.php',
    'PHPUnit\\Event\\Code\\Phpt' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/Phpt.php',
    'PHPUnit\\Event\\Code\\Test' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/Test.php',
    'PHPUnit\\Event\\Code\\TestCollection' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestCollection.php',
    'PHPUnit\\Event\\Code\\TestCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestCollectionIterator.php',
    'PHPUnit\\Event\\Code\\TestDox' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestDox.php',
    'PHPUnit\\Event\\Code\\TestDoxBuilder' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestDoxBuilder.php',
    'PHPUnit\\Event\\Code\\TestMethod' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestMethod.php',
    'PHPUnit\\Event\\Code\\TestMethodBuilder' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestMethodBuilder.php',
    'PHPUnit\\Event\\Code\\Throwable' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Throwable.php',
    'PHPUnit\\Event\\Code\\ThrowableBuilder' => $vendorDir . '/phpunit/phpunit/src/Event/Value/ThrowableBuilder.php',
    'PHPUnit\\Event\\CollectingDispatcher' => $vendorDir . '/phpunit/phpunit/src/Event/Dispatcher/CollectingDispatcher.php',
    'PHPUnit\\Event\\DeferringDispatcher' => $vendorDir . '/phpunit/phpunit/src/Event/Dispatcher/DeferringDispatcher.php',
    'PHPUnit\\Event\\DirectDispatcher' => $vendorDir . '/phpunit/phpunit/src/Event/Dispatcher/DirectDispatcher.php',
    'PHPUnit\\Event\\Dispatcher' => $vendorDir . '/phpunit/phpunit/src/Event/Dispatcher/Dispatcher.php',
    'PHPUnit\\Event\\DispatchingEmitter' => $vendorDir . '/phpunit/phpunit/src/Event/Emitter/DispatchingEmitter.php',
    'PHPUnit\\Event\\Emitter' => $vendorDir . '/phpunit/phpunit/src/Event/Emitter/Emitter.php',
    'PHPUnit\\Event\\Event' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Event.php',
    'PHPUnit\\Event\\EventAlreadyAssignedException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/EventAlreadyAssignedException.php',
    'PHPUnit\\Event\\EventCollection' => $vendorDir . '/phpunit/phpunit/src/Event/Events/EventCollection.php',
    'PHPUnit\\Event\\EventCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/Event/Events/EventCollectionIterator.php',
    'PHPUnit\\Event\\EventFacadeIsSealedException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/EventFacadeIsSealedException.php',
    'PHPUnit\\Event\\Exception' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/Exception.php',
    'PHPUnit\\Event\\Facade' => $vendorDir . '/phpunit/phpunit/src/Event/Facade.php',
    'PHPUnit\\Event\\InvalidArgumentException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/InvalidArgumentException.php',
    'PHPUnit\\Event\\InvalidEventException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/InvalidEventException.php',
    'PHPUnit\\Event\\InvalidSubscriberException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/InvalidSubscriberException.php',
    'PHPUnit\\Event\\MapError' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/MapError.php',
    'PHPUnit\\Event\\NoPreviousThrowableException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/NoPreviousThrowableException.php',
    'PHPUnit\\Event\\RuntimeException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/RuntimeException.php',
    'PHPUnit\\Event\\Runtime\\OperatingSystem' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Runtime/OperatingSystem.php',
    'PHPUnit\\Event\\Runtime\\PHP' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Runtime/PHP.php',
    'PHPUnit\\Event\\Runtime\\PHPUnit' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Runtime/PHPUnit.php',
    'PHPUnit\\Event\\Runtime\\Runtime' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Runtime/Runtime.php',
    'PHPUnit\\Event\\SubscribableDispatcher' => $vendorDir . '/phpunit/phpunit/src/Event/Dispatcher/SubscribableDispatcher.php',
    'PHPUnit\\Event\\Subscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Subscriber.php',
    'PHPUnit\\Event\\SubscriberTypeAlreadyRegisteredException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/SubscriberTypeAlreadyRegisteredException.php',
    'PHPUnit\\Event\\Telemetry\\Duration' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/Duration.php',
    'PHPUnit\\Event\\Telemetry\\GarbageCollectorStatus' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/GarbageCollectorStatus.php',
    'PHPUnit\\Event\\Telemetry\\GarbageCollectorStatusProvider' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/GarbageCollectorStatusProvider.php',
    'PHPUnit\\Event\\Telemetry\\HRTime' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/HRTime.php',
    'PHPUnit\\Event\\Telemetry\\Info' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/Info.php',
    'PHPUnit\\Event\\Telemetry\\MemoryMeter' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/MemoryMeter.php',
    'PHPUnit\\Event\\Telemetry\\MemoryUsage' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/MemoryUsage.php',
    'PHPUnit\\Event\\Telemetry\\Php81GarbageCollectorStatusProvider' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/Php81GarbageCollectorStatusProvider.php',
    'PHPUnit\\Event\\Telemetry\\Php83GarbageCollectorStatusProvider' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/Php83GarbageCollectorStatusProvider.php',
    'PHPUnit\\Event\\Telemetry\\Snapshot' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/Snapshot.php',
    'PHPUnit\\Event\\Telemetry\\StopWatch' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/StopWatch.php',
    'PHPUnit\\Event\\Telemetry\\System' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/System.php',
    'PHPUnit\\Event\\Telemetry\\SystemMemoryMeter' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/SystemMemoryMeter.php',
    'PHPUnit\\Event\\Telemetry\\SystemStopWatch' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/SystemStopWatch.php',
    'PHPUnit\\Event\\Telemetry\\SystemStopWatchWithOffset' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Telemetry/SystemStopWatchWithOffset.php',
    'PHPUnit\\Event\\TestData\\DataFromDataProvider' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestData/DataFromDataProvider.php',
    'PHPUnit\\Event\\TestData\\DataFromTestDependency' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestData/DataFromTestDependency.php',
    'PHPUnit\\Event\\TestData\\MoreThanOneDataSetFromDataProviderException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/MoreThanOneDataSetFromDataProviderException.php',
    'PHPUnit\\Event\\TestData\\NoDataSetFromDataProviderException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/NoDataSetFromDataProviderException.php',
    'PHPUnit\\Event\\TestData\\TestData' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestData/TestData.php',
    'PHPUnit\\Event\\TestData\\TestDataCollection' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestData/TestDataCollection.php',
    'PHPUnit\\Event\\TestData\\TestDataCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/Event/Value/Test/TestData/TestDataCollectionIterator.php',
    'PHPUnit\\Event\\TestRunner\\BootstrapFinished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/BootstrapFinished.php',
    'PHPUnit\\Event\\TestRunner\\BootstrapFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/BootstrapFinishedSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\Configured' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/Configured.php',
    'PHPUnit\\Event\\TestRunner\\ConfiguredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ConfiguredSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\DeprecationTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/DeprecationTriggered.php',
    'PHPUnit\\Event\\TestRunner\\DeprecationTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/DeprecationTriggeredSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\EventFacadeSealed' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/EventFacadeSealed.php',
    'PHPUnit\\Event\\TestRunner\\EventFacadeSealedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/EventFacadeSealedSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\ExecutionAborted' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExecutionAborted.php',
    'PHPUnit\\Event\\TestRunner\\ExecutionAbortedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExecutionAbortedSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\ExecutionFinished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExecutionFinished.php',
    'PHPUnit\\Event\\TestRunner\\ExecutionFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExecutionFinishedSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\ExecutionStarted' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExecutionStarted.php',
    'PHPUnit\\Event\\TestRunner\\ExecutionStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExecutionStartedSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\ExtensionBootstrapped' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExtensionBootstrapped.php',
    'PHPUnit\\Event\\TestRunner\\ExtensionBootstrappedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExtensionBootstrappedSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\ExtensionLoadedFromPhar' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExtensionLoadedFromPhar.php',
    'PHPUnit\\Event\\TestRunner\\ExtensionLoadedFromPharSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/ExtensionLoadedFromPharSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\Finished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/Finished.php',
    'PHPUnit\\Event\\TestRunner\\FinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/FinishedSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\GarbageCollectionDisabled' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/GarbageCollectionDisabled.php',
    'PHPUnit\\Event\\TestRunner\\GarbageCollectionDisabledSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/GarbageCollectionDisabledSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\GarbageCollectionEnabled' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/GarbageCollectionEnabled.php',
    'PHPUnit\\Event\\TestRunner\\GarbageCollectionEnabledSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/GarbageCollectionEnabledSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\GarbageCollectionTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/GarbageCollectionTriggered.php',
    'PHPUnit\\Event\\TestRunner\\GarbageCollectionTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/GarbageCollectionTriggeredSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\Started' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/Started.php',
    'PHPUnit\\Event\\TestRunner\\StartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/StartedSubscriber.php',
    'PHPUnit\\Event\\TestRunner\\WarningTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/WarningTriggered.php',
    'PHPUnit\\Event\\TestRunner\\WarningTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestRunner/WarningTriggeredSubscriber.php',
    'PHPUnit\\Event\\TestSuite\\Filtered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/Filtered.php',
    'PHPUnit\\Event\\TestSuite\\FilteredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/FilteredSubscriber.php',
    'PHPUnit\\Event\\TestSuite\\Finished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/Finished.php',
    'PHPUnit\\Event\\TestSuite\\FinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/FinishedSubscriber.php',
    'PHPUnit\\Event\\TestSuite\\Loaded' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/Loaded.php',
    'PHPUnit\\Event\\TestSuite\\LoadedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/LoadedSubscriber.php',
    'PHPUnit\\Event\\TestSuite\\Skipped' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/Skipped.php',
    'PHPUnit\\Event\\TestSuite\\SkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/SkippedSubscriber.php',
    'PHPUnit\\Event\\TestSuite\\Sorted' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/Sorted.php',
    'PHPUnit\\Event\\TestSuite\\SortedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/SortedSubscriber.php',
    'PHPUnit\\Event\\TestSuite\\Started' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/Started.php',
    'PHPUnit\\Event\\TestSuite\\StartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/TestSuite/StartedSubscriber.php',
    'PHPUnit\\Event\\TestSuite\\TestSuite' => $vendorDir . '/phpunit/phpunit/src/Event/Value/TestSuite/TestSuite.php',
    'PHPUnit\\Event\\TestSuite\\TestSuiteBuilder' => $vendorDir . '/phpunit/phpunit/src/Event/Value/TestSuite/TestSuiteBuilder.php',
    'PHPUnit\\Event\\TestSuite\\TestSuiteForTestClass' => $vendorDir . '/phpunit/phpunit/src/Event/Value/TestSuite/TestSuiteForTestClass.php',
    'PHPUnit\\Event\\TestSuite\\TestSuiteForTestMethodWithDataProvider' => $vendorDir . '/phpunit/phpunit/src/Event/Value/TestSuite/TestSuiteForTestMethodWithDataProvider.php',
    'PHPUnit\\Event\\TestSuite\\TestSuiteWithName' => $vendorDir . '/phpunit/phpunit/src/Event/Value/TestSuite/TestSuiteWithName.php',
    'PHPUnit\\Event\\Test\\AfterLastTestMethodCalled' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/AfterLastTestMethodCalled.php',
    'PHPUnit\\Event\\Test\\AfterLastTestMethodCalledSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/AfterLastTestMethodCalledSubscriber.php',
    'PHPUnit\\Event\\Test\\AfterLastTestMethodFinished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/AfterLastTestMethodFinished.php',
    'PHPUnit\\Event\\Test\\AfterLastTestMethodFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/AfterLastTestMethodFinishedSubscriber.php',
    'PHPUnit\\Event\\Test\\AfterTestMethodCalled' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/AfterTestMethodCalled.php',
    'PHPUnit\\Event\\Test\\AfterTestMethodCalledSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/AfterTestMethodCalledSubscriber.php',
    'PHPUnit\\Event\\Test\\AfterTestMethodFinished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/AfterTestMethodFinished.php',
    'PHPUnit\\Event\\Test\\AfterTestMethodFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/AfterTestMethodFinishedSubscriber.php',
    'PHPUnit\\Event\\Test\\AssertionFailed' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Assertion/AssertionFailed.php',
    'PHPUnit\\Event\\Test\\AssertionFailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Assertion/AssertionFailedSubscriber.php',
    'PHPUnit\\Event\\Test\\AssertionSucceeded' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Assertion/AssertionSucceeded.php',
    'PHPUnit\\Event\\Test\\AssertionSucceededSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Assertion/AssertionSucceededSubscriber.php',
    'PHPUnit\\Event\\Test\\BeforeFirstTestMethodCalled' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeFirstTestMethodCalled.php',
    'PHPUnit\\Event\\Test\\BeforeFirstTestMethodCalledSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeFirstTestMethodCalledSubscriber.php',
    'PHPUnit\\Event\\Test\\BeforeFirstTestMethodErrored' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeFirstTestMethodErrored.php',
    'PHPUnit\\Event\\Test\\BeforeFirstTestMethodErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeFirstTestMethodErroredSubscriber.php',
    'PHPUnit\\Event\\Test\\BeforeFirstTestMethodFinished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeFirstTestMethodFinished.php',
    'PHPUnit\\Event\\Test\\BeforeFirstTestMethodFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeFirstTestMethodFinishedSubscriber.php',
    'PHPUnit\\Event\\Test\\BeforeTestMethodCalled' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeTestMethodCalled.php',
    'PHPUnit\\Event\\Test\\BeforeTestMethodCalledSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeTestMethodCalledSubscriber.php',
    'PHPUnit\\Event\\Test\\BeforeTestMethodFinished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeTestMethodFinished.php',
    'PHPUnit\\Event\\Test\\BeforeTestMethodFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/BeforeTestMethodFinishedSubscriber.php',
    'PHPUnit\\Event\\Test\\ComparatorRegistered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/ComparatorRegistered.php',
    'PHPUnit\\Event\\Test\\ComparatorRegisteredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/ComparatorRegisteredSubscriber.php',
    'PHPUnit\\Event\\Test\\ConsideredRisky' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/ConsideredRisky.php',
    'PHPUnit\\Event\\Test\\ConsideredRiskySubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/ConsideredRiskySubscriber.php',
    'PHPUnit\\Event\\Test\\DataProviderMethodCalled' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/DataProviderMethodCalled.php',
    'PHPUnit\\Event\\Test\\DataProviderMethodCalledSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/DataProviderMethodCalledSubscriber.php',
    'PHPUnit\\Event\\Test\\DataProviderMethodFinished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/DataProviderMethodFinished.php',
    'PHPUnit\\Event\\Test\\DataProviderMethodFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/DataProviderMethodFinishedSubscriber.php',
    'PHPUnit\\Event\\Test\\DeprecationTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/DeprecationTriggered.php',
    'PHPUnit\\Event\\Test\\DeprecationTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/DeprecationTriggeredSubscriber.php',
    'PHPUnit\\Event\\Test\\ErrorTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/ErrorTriggered.php',
    'PHPUnit\\Event\\Test\\ErrorTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/ErrorTriggeredSubscriber.php',
    'PHPUnit\\Event\\Test\\Errored' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/Errored.php',
    'PHPUnit\\Event\\Test\\ErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/ErroredSubscriber.php',
    'PHPUnit\\Event\\Test\\Failed' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/Failed.php',
    'PHPUnit\\Event\\Test\\FailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/FailedSubscriber.php',
    'PHPUnit\\Event\\Test\\Finished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/Finished.php',
    'PHPUnit\\Event\\Test\\FinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/FinishedSubscriber.php',
    'PHPUnit\\Event\\Test\\MarkedIncomplete' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/MarkedIncomplete.php',
    'PHPUnit\\Event\\Test\\MarkedIncompleteSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/MarkedIncompleteSubscriber.php',
    'PHPUnit\\Event\\Test\\MockObjectCreated' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectCreated.php',
    'PHPUnit\\Event\\Test\\MockObjectCreatedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectCreatedSubscriber.php',
    'PHPUnit\\Event\\Test\\MockObjectForAbstractClassCreated' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectForAbstractClassCreated.php',
    'PHPUnit\\Event\\Test\\MockObjectForAbstractClassCreatedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectForAbstractClassCreatedSubscriber.php',
    'PHPUnit\\Event\\Test\\MockObjectForIntersectionOfInterfacesCreated' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectForIntersectionOfInterfacesCreated.php',
    'PHPUnit\\Event\\Test\\MockObjectForIntersectionOfInterfacesCreatedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectForIntersectionOfInterfacesCreatedSubscriber.php',
    'PHPUnit\\Event\\Test\\MockObjectForTraitCreated' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectForTraitCreated.php',
    'PHPUnit\\Event\\Test\\MockObjectForTraitCreatedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectForTraitCreatedSubscriber.php',
    'PHPUnit\\Event\\Test\\MockObjectFromWsdlCreated' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectFromWsdlCreated.php',
    'PHPUnit\\Event\\Test\\MockObjectFromWsdlCreatedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/MockObjectFromWsdlCreatedSubscriber.php',
    'PHPUnit\\Event\\Test\\NoComparisonFailureException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/NoComparisonFailureException.php',
    'PHPUnit\\Event\\Test\\NoticeTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/NoticeTriggered.php',
    'PHPUnit\\Event\\Test\\NoticeTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/NoticeTriggeredSubscriber.php',
    'PHPUnit\\Event\\Test\\PartialMockObjectCreated' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/PartialMockObjectCreated.php',
    'PHPUnit\\Event\\Test\\PartialMockObjectCreatedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/PartialMockObjectCreatedSubscriber.php',
    'PHPUnit\\Event\\Test\\Passed' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/Passed.php',
    'PHPUnit\\Event\\Test\\PassedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/PassedSubscriber.php',
    'PHPUnit\\Event\\Test\\PhpDeprecationTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpDeprecationTriggered.php',
    'PHPUnit\\Event\\Test\\PhpDeprecationTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpDeprecationTriggeredSubscriber.php',
    'PHPUnit\\Event\\Test\\PhpNoticeTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpNoticeTriggered.php',
    'PHPUnit\\Event\\Test\\PhpNoticeTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpNoticeTriggeredSubscriber.php',
    'PHPUnit\\Event\\Test\\PhpWarningTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpWarningTriggered.php',
    'PHPUnit\\Event\\Test\\PhpWarningTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpWarningTriggeredSubscriber.php',
    'PHPUnit\\Event\\Test\\PhpunitDeprecationTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpunitDeprecationTriggered.php',
    'PHPUnit\\Event\\Test\\PhpunitDeprecationTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpunitDeprecationTriggeredSubscriber.php',
    'PHPUnit\\Event\\Test\\PhpunitErrorTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpunitErrorTriggered.php',
    'PHPUnit\\Event\\Test\\PhpunitErrorTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpunitErrorTriggeredSubscriber.php',
    'PHPUnit\\Event\\Test\\PhpunitWarningTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpunitWarningTriggered.php',
    'PHPUnit\\Event\\Test\\PhpunitWarningTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/PhpunitWarningTriggeredSubscriber.php',
    'PHPUnit\\Event\\Test\\PostConditionCalled' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/PostConditionCalled.php',
    'PHPUnit\\Event\\Test\\PostConditionCalledSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/PostConditionCalledSubscriber.php',
    'PHPUnit\\Event\\Test\\PostConditionFinished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/PostConditionFinished.php',
    'PHPUnit\\Event\\Test\\PostConditionFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/PostConditionFinishedSubscriber.php',
    'PHPUnit\\Event\\Test\\PreConditionCalled' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/PreConditionCalled.php',
    'PHPUnit\\Event\\Test\\PreConditionCalledSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/PreConditionCalledSubscriber.php',
    'PHPUnit\\Event\\Test\\PreConditionFinished' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/PreConditionFinished.php',
    'PHPUnit\\Event\\Test\\PreConditionFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/HookMethod/PreConditionFinishedSubscriber.php',
    'PHPUnit\\Event\\Test\\PreparationFailed' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/PreparationFailed.php',
    'PHPUnit\\Event\\Test\\PreparationFailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/PreparationFailedSubscriber.php',
    'PHPUnit\\Event\\Test\\PreparationStarted' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/PreparationStarted.php',
    'PHPUnit\\Event\\Test\\PreparationStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/PreparationStartedSubscriber.php',
    'PHPUnit\\Event\\Test\\Prepared' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/Prepared.php',
    'PHPUnit\\Event\\Test\\PreparedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Lifecycle/PreparedSubscriber.php',
    'PHPUnit\\Event\\Test\\PrintedUnexpectedOutput' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/PrintedUnexpectedOutput.php',
    'PHPUnit\\Event\\Test\\PrintedUnexpectedOutputSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/PrintedUnexpectedOutputSubscriber.php',
    'PHPUnit\\Event\\Test\\Skipped' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/Skipped.php',
    'PHPUnit\\Event\\Test\\SkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Outcome/SkippedSubscriber.php',
    'PHPUnit\\Event\\Test\\TestProxyCreated' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/TestProxyCreated.php',
    'PHPUnit\\Event\\Test\\TestProxyCreatedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/TestProxyCreatedSubscriber.php',
    'PHPUnit\\Event\\Test\\TestStubCreated' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/TestStubCreated.php',
    'PHPUnit\\Event\\Test\\TestStubCreatedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/TestStubCreatedSubscriber.php',
    'PHPUnit\\Event\\Test\\TestStubForIntersectionOfInterfacesCreated' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/TestStubForIntersectionOfInterfacesCreated.php',
    'PHPUnit\\Event\\Test\\TestStubForIntersectionOfInterfacesCreatedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/TestDouble/TestStubForIntersectionOfInterfacesCreatedSubscriber.php',
    'PHPUnit\\Event\\Test\\WarningTriggered' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/WarningTriggered.php',
    'PHPUnit\\Event\\Test\\WarningTriggeredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Event/Events/Test/Issue/WarningTriggeredSubscriber.php',
    'PHPUnit\\Event\\Tracer\\Tracer' => $vendorDir . '/phpunit/phpunit/src/Event/Tracer.php',
    'PHPUnit\\Event\\TypeMap' => $vendorDir . '/phpunit/phpunit/src/Event/TypeMap.php',
    'PHPUnit\\Event\\UnknownEventException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/UnknownEventException.php',
    'PHPUnit\\Event\\UnknownEventTypeException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/UnknownEventTypeException.php',
    'PHPUnit\\Event\\UnknownSubscriberException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/UnknownSubscriberException.php',
    'PHPUnit\\Event\\UnknownSubscriberTypeException' => $vendorDir . '/phpunit/phpunit/src/Event/Exception/UnknownSubscriberTypeException.php',
    'PHPUnit\\Exception' => $vendorDir . '/phpunit/phpunit/src/Exception.php',
    'PHPUnit\\Framework\\ActualValueIsNotAnObjectException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/ObjectEquals/ActualValueIsNotAnObjectException.php',
    'PHPUnit\\Framework\\Assert' => $vendorDir . '/phpunit/phpunit/src/Framework/Assert.php',
    'PHPUnit\\Framework\\AssertionFailedError' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/AssertionFailedError.php',
    'PHPUnit\\Framework\\Attributes\\After' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/After.php',
    'PHPUnit\\Framework\\Attributes\\AfterClass' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/AfterClass.php',
    'PHPUnit\\Framework\\Attributes\\BackupGlobals' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/BackupGlobals.php',
    'PHPUnit\\Framework\\Attributes\\BackupStaticProperties' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/BackupStaticProperties.php',
    'PHPUnit\\Framework\\Attributes\\Before' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/Before.php',
    'PHPUnit\\Framework\\Attributes\\BeforeClass' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/BeforeClass.php',
    'PHPUnit\\Framework\\Attributes\\CodeCoverageIgnore' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/CodeCoverageIgnore.php',
    'PHPUnit\\Framework\\Attributes\\CoversClass' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/CoversClass.php',
    'PHPUnit\\Framework\\Attributes\\CoversFunction' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/CoversFunction.php',
    'PHPUnit\\Framework\\Attributes\\CoversNothing' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/CoversNothing.php',
    'PHPUnit\\Framework\\Attributes\\DataProvider' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DataProvider.php',
    'PHPUnit\\Framework\\Attributes\\DataProviderExternal' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DataProviderExternal.php',
    'PHPUnit\\Framework\\Attributes\\Depends' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/Depends.php',
    'PHPUnit\\Framework\\Attributes\\DependsExternal' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DependsExternal.php',
    'PHPUnit\\Framework\\Attributes\\DependsExternalUsingDeepClone' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DependsExternalUsingDeepClone.php',
    'PHPUnit\\Framework\\Attributes\\DependsExternalUsingShallowClone' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DependsExternalUsingShallowClone.php',
    'PHPUnit\\Framework\\Attributes\\DependsOnClass' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DependsOnClass.php',
    'PHPUnit\\Framework\\Attributes\\DependsOnClassUsingDeepClone' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DependsOnClassUsingDeepClone.php',
    'PHPUnit\\Framework\\Attributes\\DependsOnClassUsingShallowClone' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DependsOnClassUsingShallowClone.php',
    'PHPUnit\\Framework\\Attributes\\DependsUsingDeepClone' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DependsUsingDeepClone.php',
    'PHPUnit\\Framework\\Attributes\\DependsUsingShallowClone' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DependsUsingShallowClone.php',
    'PHPUnit\\Framework\\Attributes\\DoesNotPerformAssertions' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/DoesNotPerformAssertions.php',
    'PHPUnit\\Framework\\Attributes\\ExcludeGlobalVariableFromBackup' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/ExcludeGlobalVariableFromBackup.php',
    'PHPUnit\\Framework\\Attributes\\ExcludeStaticPropertyFromBackup' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/ExcludeStaticPropertyFromBackup.php',
    'PHPUnit\\Framework\\Attributes\\Group' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/Group.php',
    'PHPUnit\\Framework\\Attributes\\IgnoreClassForCodeCoverage' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/IgnoreClassForCodeCoverage.php',
    'PHPUnit\\Framework\\Attributes\\IgnoreDeprecations' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/IgnoreDeprecations.php',
    'PHPUnit\\Framework\\Attributes\\IgnoreFunctionForCodeCoverage' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/IgnoreFunctionForCodeCoverage.php',
    'PHPUnit\\Framework\\Attributes\\IgnoreMethodForCodeCoverage' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/IgnoreMethodForCodeCoverage.php',
    'PHPUnit\\Framework\\Attributes\\Large' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/Large.php',
    'PHPUnit\\Framework\\Attributes\\Medium' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/Medium.php',
    'PHPUnit\\Framework\\Attributes\\PostCondition' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/PostCondition.php',
    'PHPUnit\\Framework\\Attributes\\PreCondition' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/PreCondition.php',
    'PHPUnit\\Framework\\Attributes\\PreserveGlobalState' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/PreserveGlobalState.php',
    'PHPUnit\\Framework\\Attributes\\RequiresFunction' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RequiresFunction.php',
    'PHPUnit\\Framework\\Attributes\\RequiresMethod' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RequiresMethod.php',
    'PHPUnit\\Framework\\Attributes\\RequiresOperatingSystem' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RequiresOperatingSystem.php',
    'PHPUnit\\Framework\\Attributes\\RequiresOperatingSystemFamily' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RequiresOperatingSystemFamily.php',
    'PHPUnit\\Framework\\Attributes\\RequiresPhp' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RequiresPhp.php',
    'PHPUnit\\Framework\\Attributes\\RequiresPhpExtension' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RequiresPhpExtension.php',
    'PHPUnit\\Framework\\Attributes\\RequiresPhpunit' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RequiresPhpunit.php',
    'PHPUnit\\Framework\\Attributes\\RequiresSetting' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RequiresSetting.php',
    'PHPUnit\\Framework\\Attributes\\RunClassInSeparateProcess' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RunClassInSeparateProcess.php',
    'PHPUnit\\Framework\\Attributes\\RunInSeparateProcess' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RunInSeparateProcess.php',
    'PHPUnit\\Framework\\Attributes\\RunTestsInSeparateProcesses' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/RunTestsInSeparateProcesses.php',
    'PHPUnit\\Framework\\Attributes\\Small' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/Small.php',
    'PHPUnit\\Framework\\Attributes\\Test' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/Test.php',
    'PHPUnit\\Framework\\Attributes\\TestDox' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/TestDox.php',
    'PHPUnit\\Framework\\Attributes\\TestWith' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/TestWith.php',
    'PHPUnit\\Framework\\Attributes\\TestWithJson' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/TestWithJson.php',
    'PHPUnit\\Framework\\Attributes\\Ticket' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/Ticket.php',
    'PHPUnit\\Framework\\Attributes\\UsesClass' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/UsesClass.php',
    'PHPUnit\\Framework\\Attributes\\UsesFunction' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/UsesFunction.php',
    'PHPUnit\\Framework\\Attributes\\WithoutErrorHandler' => $vendorDir . '/phpunit/phpunit/src/Framework/Attributes/WithoutErrorHandler.php',
    'PHPUnit\\Framework\\CodeCoverageException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/CodeCoverageException.php',
    'PHPUnit\\Framework\\ComparisonMethodDoesNotAcceptParameterTypeException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/ObjectEquals/ComparisonMethodDoesNotAcceptParameterTypeException.php',
    'PHPUnit\\Framework\\ComparisonMethodDoesNotDeclareBoolReturnTypeException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/ObjectEquals/ComparisonMethodDoesNotDeclareBoolReturnTypeException.php',
    'PHPUnit\\Framework\\ComparisonMethodDoesNotDeclareExactlyOneParameterException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/ObjectEquals/ComparisonMethodDoesNotDeclareExactlyOneParameterException.php',
    'PHPUnit\\Framework\\ComparisonMethodDoesNotDeclareParameterTypeException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/ObjectEquals/ComparisonMethodDoesNotDeclareParameterTypeException.php',
    'PHPUnit\\Framework\\ComparisonMethodDoesNotExistException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/ObjectEquals/ComparisonMethodDoesNotExistException.php',
    'PHPUnit\\Framework\\Constraint\\ArrayHasKey' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Traversable/ArrayHasKey.php',
    'PHPUnit\\Framework\\Constraint\\BinaryOperator' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Operator/BinaryOperator.php',
    'PHPUnit\\Framework\\Constraint\\Callback' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Callback.php',
    'PHPUnit\\Framework\\Constraint\\Constraint' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Constraint.php',
    'PHPUnit\\Framework\\Constraint\\Count' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Cardinality/Count.php',
    'PHPUnit\\Framework\\Constraint\\DirectoryExists' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Filesystem/DirectoryExists.php',
    'PHPUnit\\Framework\\Constraint\\Exception' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Exception/Exception.php',
    'PHPUnit\\Framework\\Constraint\\ExceptionCode' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Exception/ExceptionCode.php',
    'PHPUnit\\Framework\\Constraint\\ExceptionMessageIsOrContains' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Exception/ExceptionMessageIsOrContains.php',
    'PHPUnit\\Framework\\Constraint\\ExceptionMessageMatchesRegularExpression' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Exception/ExceptionMessageMatchesRegularExpression.php',
    'PHPUnit\\Framework\\Constraint\\FileExists' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Filesystem/FileExists.php',
    'PHPUnit\\Framework\\Constraint\\GreaterThan' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Cardinality/GreaterThan.php',
    'PHPUnit\\Framework\\Constraint\\IsAnything' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/IsAnything.php',
    'PHPUnit\\Framework\\Constraint\\IsEmpty' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Cardinality/IsEmpty.php',
    'PHPUnit\\Framework\\Constraint\\IsEqual' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Equality/IsEqual.php',
    'PHPUnit\\Framework\\Constraint\\IsEqualCanonicalizing' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Equality/IsEqualCanonicalizing.php',
    'PHPUnit\\Framework\\Constraint\\IsEqualIgnoringCase' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Equality/IsEqualIgnoringCase.php',
    'PHPUnit\\Framework\\Constraint\\IsEqualWithDelta' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Equality/IsEqualWithDelta.php',
    'PHPUnit\\Framework\\Constraint\\IsFalse' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Boolean/IsFalse.php',
    'PHPUnit\\Framework\\Constraint\\IsFinite' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Math/IsFinite.php',
    'PHPUnit\\Framework\\Constraint\\IsIdentical' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/IsIdentical.php',
    'PHPUnit\\Framework\\Constraint\\IsInfinite' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Math/IsInfinite.php',
    'PHPUnit\\Framework\\Constraint\\IsInstanceOf' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Type/IsInstanceOf.php',
    'PHPUnit\\Framework\\Constraint\\IsJson' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/String/IsJson.php',
    'PHPUnit\\Framework\\Constraint\\IsList' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Traversable/IsList.php',
    'PHPUnit\\Framework\\Constraint\\IsNan' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Math/IsNan.php',
    'PHPUnit\\Framework\\Constraint\\IsNull' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Type/IsNull.php',
    'PHPUnit\\Framework\\Constraint\\IsReadable' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Filesystem/IsReadable.php',
    'PHPUnit\\Framework\\Constraint\\IsTrue' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Boolean/IsTrue.php',
    'PHPUnit\\Framework\\Constraint\\IsType' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Type/IsType.php',
    'PHPUnit\\Framework\\Constraint\\IsWritable' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Filesystem/IsWritable.php',
    'PHPUnit\\Framework\\Constraint\\JsonMatches' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/JsonMatches.php',
    'PHPUnit\\Framework\\Constraint\\LessThan' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Cardinality/LessThan.php',
    'PHPUnit\\Framework\\Constraint\\LogicalAnd' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Operator/LogicalAnd.php',
    'PHPUnit\\Framework\\Constraint\\LogicalNot' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Operator/LogicalNot.php',
    'PHPUnit\\Framework\\Constraint\\LogicalOr' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Operator/LogicalOr.php',
    'PHPUnit\\Framework\\Constraint\\LogicalXor' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Operator/LogicalXor.php',
    'PHPUnit\\Framework\\Constraint\\ObjectEquals' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Object/ObjectEquals.php',
    'PHPUnit\\Framework\\Constraint\\ObjectHasProperty' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Object/ObjectHasProperty.php',
    'PHPUnit\\Framework\\Constraint\\Operator' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Operator/Operator.php',
    'PHPUnit\\Framework\\Constraint\\RegularExpression' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/String/RegularExpression.php',
    'PHPUnit\\Framework\\Constraint\\SameSize' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Cardinality/SameSize.php',
    'PHPUnit\\Framework\\Constraint\\StringContains' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/String/StringContains.php',
    'PHPUnit\\Framework\\Constraint\\StringEndsWith' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/String/StringEndsWith.php',
    'PHPUnit\\Framework\\Constraint\\StringEqualsStringIgnoringLineEndings' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/String/StringEqualsStringIgnoringLineEndings.php',
    'PHPUnit\\Framework\\Constraint\\StringMatchesFormatDescription' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/String/StringMatchesFormatDescription.php',
    'PHPUnit\\Framework\\Constraint\\StringStartsWith' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/String/StringStartsWith.php',
    'PHPUnit\\Framework\\Constraint\\TraversableContains' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Traversable/TraversableContains.php',
    'PHPUnit\\Framework\\Constraint\\TraversableContainsEqual' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Traversable/TraversableContainsEqual.php',
    'PHPUnit\\Framework\\Constraint\\TraversableContainsIdentical' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Traversable/TraversableContainsIdentical.php',
    'PHPUnit\\Framework\\Constraint\\TraversableContainsOnly' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Traversable/TraversableContainsOnly.php',
    'PHPUnit\\Framework\\Constraint\\UnaryOperator' => $vendorDir . '/phpunit/phpunit/src/Framework/Constraint/Operator/UnaryOperator.php',
    'PHPUnit\\Framework\\DataProviderTestSuite' => $vendorDir . '/phpunit/phpunit/src/Framework/DataProviderTestSuite.php',
    'PHPUnit\\Framework\\EmptyStringException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/EmptyStringException.php',
    'PHPUnit\\Framework\\Exception' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/Exception.php',
    'PHPUnit\\Framework\\ExecutionOrderDependency' => $vendorDir . '/phpunit/phpunit/src/Framework/ExecutionOrderDependency.php',
    'PHPUnit\\Framework\\ExpectationFailedException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/ExpectationFailedException.php',
    'PHPUnit\\Framework\\GeneratorNotSupportedException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/GeneratorNotSupportedException.php',
    'PHPUnit\\Framework\\IncompleteTest' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/Incomplete/IncompleteTest.php',
    'PHPUnit\\Framework\\IncompleteTestError' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/Incomplete/IncompleteTestError.php',
    'PHPUnit\\Framework\\InvalidArgumentException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/InvalidArgumentException.php',
    'PHPUnit\\Framework\\InvalidCoversTargetException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/InvalidCoversTargetException.php',
    'PHPUnit\\Framework\\InvalidDataProviderException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/InvalidDataProviderException.php',
    'PHPUnit\\Framework\\InvalidDependencyException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/InvalidDependencyException.php',
    'PHPUnit\\Framework\\MockObject\\BadMethodCallException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/BadMethodCallException.php',
    'PHPUnit\\Framework\\MockObject\\Builder\\Identity' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Builder/Identity.php',
    'PHPUnit\\Framework\\MockObject\\Builder\\InvocationMocker' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Builder/InvocationMocker.php',
    'PHPUnit\\Framework\\MockObject\\Builder\\InvocationStubber' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Builder/InvocationStubber.php',
    'PHPUnit\\Framework\\MockObject\\Builder\\MethodNameMatch' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Builder/MethodNameMatch.php',
    'PHPUnit\\Framework\\MockObject\\Builder\\ParametersMatch' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Builder/ParametersMatch.php',
    'PHPUnit\\Framework\\MockObject\\Builder\\Stub' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Builder/Stub.php',
    'PHPUnit\\Framework\\MockObject\\CannotUseOnlyMethodsException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/CannotUseOnlyMethodsException.php',
    'PHPUnit\\Framework\\MockObject\\ConfigurableMethod' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/ConfigurableMethod.php',
    'PHPUnit\\Framework\\MockObject\\DoubledCloneMethod' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Api/DoubledCloneMethod.php',
    'PHPUnit\\Framework\\MockObject\\Exception' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/Exception.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\CannotUseAddMethodsException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/CannotUseAddMethodsException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\ClassIsEnumerationException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/ClassIsEnumerationException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\ClassIsFinalException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/ClassIsFinalException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\ClassIsReadonlyException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/ClassIsReadonlyException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\DuplicateMethodException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/DuplicateMethodException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\Exception' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/Exception.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\Generator' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Generator.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\InvalidMethodNameException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/InvalidMethodNameException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\MockClass' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/MockClass.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\MockMethod' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/MockMethod.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\MockMethodSet' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/MockMethodSet.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\MockTrait' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/MockTrait.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\MockType' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/MockType.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\NameAlreadyInUseException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/NameAlreadyInUseException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\OriginalConstructorInvocationRequiredException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/OriginalConstructorInvocationRequiredException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\ReflectionException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/ReflectionException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\RuntimeException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/RuntimeException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\SoapExtensionNotAvailableException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/SoapExtensionNotAvailableException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\TemplateLoader' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/TemplateLoader.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\UnknownClassException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/UnknownClassException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\UnknownTraitException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/UnknownTraitException.php',
    'PHPUnit\\Framework\\MockObject\\Generator\\UnknownTypeException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Generator/Exception/UnknownTypeException.php',
    'PHPUnit\\Framework\\MockObject\\IncompatibleReturnValueException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/IncompatibleReturnValueException.php',
    'PHPUnit\\Framework\\MockObject\\Invocation' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Invocation.php',
    'PHPUnit\\Framework\\MockObject\\InvocationHandler' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/InvocationHandler.php',
    'PHPUnit\\Framework\\MockObject\\MatchBuilderNotFoundException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/MatchBuilderNotFoundException.php',
    'PHPUnit\\Framework\\MockObject\\Matcher' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Matcher.php',
    'PHPUnit\\Framework\\MockObject\\MatcherAlreadyRegisteredException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/MatcherAlreadyRegisteredException.php',
    'PHPUnit\\Framework\\MockObject\\Method' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Api/Method.php',
    'PHPUnit\\Framework\\MockObject\\MethodCannotBeConfiguredException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/MethodCannotBeConfiguredException.php',
    'PHPUnit\\Framework\\MockObject\\MethodNameAlreadyConfiguredException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/MethodNameAlreadyConfiguredException.php',
    'PHPUnit\\Framework\\MockObject\\MethodNameConstraint' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/MethodNameConstraint.php',
    'PHPUnit\\Framework\\MockObject\\MethodNameNotConfiguredException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/MethodNameNotConfiguredException.php',
    'PHPUnit\\Framework\\MockObject\\MethodParametersAlreadyConfiguredException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/MethodParametersAlreadyConfiguredException.php',
    'PHPUnit\\Framework\\MockObject\\MockBuilder' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/MockBuilder.php',
    'PHPUnit\\Framework\\MockObject\\MockObject' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Interface/MockObject.php',
    'PHPUnit\\Framework\\MockObject\\MockObjectApi' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Api/MockObjectApi.php',
    'PHPUnit\\Framework\\MockObject\\MockObjectInternal' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Interface/MockObjectInternal.php',
    'PHPUnit\\Framework\\MockObject\\NeverReturningMethodException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/NeverReturningMethodException.php',
    'PHPUnit\\Framework\\MockObject\\NoMoreReturnValuesConfiguredException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/NoMoreReturnValuesConfiguredException.php',
    'PHPUnit\\Framework\\MockObject\\ProxiedCloneMethod' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Api/ProxiedCloneMethod.php',
    'PHPUnit\\Framework\\MockObject\\ReturnValueGenerator' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/ReturnValueGenerator.php',
    'PHPUnit\\Framework\\MockObject\\ReturnValueNotConfiguredException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/ReturnValueNotConfiguredException.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\AnyInvokedCount' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/AnyInvokedCount.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\AnyParameters' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/AnyParameters.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\InvocationOrder' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/InvocationOrder.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\InvokedAtLeastCount' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/InvokedAtLeastCount.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\InvokedAtLeastOnce' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/InvokedAtLeastOnce.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\InvokedAtMostCount' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/InvokedAtMostCount.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\InvokedCount' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/InvokedCount.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\MethodName' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/MethodName.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\Parameters' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/Parameters.php',
    'PHPUnit\\Framework\\MockObject\\Rule\\ParametersRule' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Rule/ParametersRule.php',
    'PHPUnit\\Framework\\MockObject\\RuntimeException' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Exception/RuntimeException.php',
    'PHPUnit\\Framework\\MockObject\\Stub' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Interface/Stub.php',
    'PHPUnit\\Framework\\MockObject\\StubApi' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Api/StubApi.php',
    'PHPUnit\\Framework\\MockObject\\StubInternal' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Interface/StubInternal.php',
    'PHPUnit\\Framework\\MockObject\\Stub\\ConsecutiveCalls' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Stub/ConsecutiveCalls.php',
    'PHPUnit\\Framework\\MockObject\\Stub\\Exception' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Stub/Exception.php',
    'PHPUnit\\Framework\\MockObject\\Stub\\ReturnArgument' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Stub/ReturnArgument.php',
    'PHPUnit\\Framework\\MockObject\\Stub\\ReturnCallback' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Stub/ReturnCallback.php',
    'PHPUnit\\Framework\\MockObject\\Stub\\ReturnReference' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Stub/ReturnReference.php',
    'PHPUnit\\Framework\\MockObject\\Stub\\ReturnSelf' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Stub/ReturnSelf.php',
    'PHPUnit\\Framework\\MockObject\\Stub\\ReturnStub' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Stub/ReturnStub.php',
    'PHPUnit\\Framework\\MockObject\\Stub\\ReturnValueMap' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Stub/ReturnValueMap.php',
    'PHPUnit\\Framework\\MockObject\\Stub\\Stub' => $vendorDir . '/phpunit/phpunit/src/Framework/MockObject/Runtime/Stub/Stub.php',
    'PHPUnit\\Framework\\NoChildTestSuiteException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/NoChildTestSuiteException.php',
    'PHPUnit\\Framework\\PhptAssertionFailedError' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/PhptAssertionFailedError.php',
    'PHPUnit\\Framework\\ProcessIsolationException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/ProcessIsolationException.php',
    'PHPUnit\\Framework\\Reorderable' => $vendorDir . '/phpunit/phpunit/src/Framework/Reorderable.php',
    'PHPUnit\\Framework\\SelfDescribing' => $vendorDir . '/phpunit/phpunit/src/Framework/SelfDescribing.php',
    'PHPUnit\\Framework\\SkippedTest' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/Skipped/SkippedTest.php',
    'PHPUnit\\Framework\\SkippedTestSuiteError' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/Skipped/SkippedTestSuiteError.php',
    'PHPUnit\\Framework\\SkippedWithMessageException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/Skipped/SkippedWithMessageException.php',
    'PHPUnit\\Framework\\Test' => $vendorDir . '/phpunit/phpunit/src/Framework/Test.php',
    'PHPUnit\\Framework\\TestBuilder' => $vendorDir . '/phpunit/phpunit/src/Framework/TestBuilder.php',
    'PHPUnit\\Framework\\TestCase' => $vendorDir . '/phpunit/phpunit/src/Framework/TestCase.php',
    'PHPUnit\\Framework\\TestRunner' => $vendorDir . '/phpunit/phpunit/src/Framework/TestRunner.php',
    'PHPUnit\\Framework\\TestSize\\Known' => $vendorDir . '/phpunit/phpunit/src/Framework/TestSize/Known.php',
    'PHPUnit\\Framework\\TestSize\\Large' => $vendorDir . '/phpunit/phpunit/src/Framework/TestSize/Large.php',
    'PHPUnit\\Framework\\TestSize\\Medium' => $vendorDir . '/phpunit/phpunit/src/Framework/TestSize/Medium.php',
    'PHPUnit\\Framework\\TestSize\\Small' => $vendorDir . '/phpunit/phpunit/src/Framework/TestSize/Small.php',
    'PHPUnit\\Framework\\TestSize\\TestSize' => $vendorDir . '/phpunit/phpunit/src/Framework/TestSize/TestSize.php',
    'PHPUnit\\Framework\\TestSize\\Unknown' => $vendorDir . '/phpunit/phpunit/src/Framework/TestSize/Unknown.php',
    'PHPUnit\\Framework\\TestStatus\\Deprecation' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Deprecation.php',
    'PHPUnit\\Framework\\TestStatus\\Error' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Error.php',
    'PHPUnit\\Framework\\TestStatus\\Failure' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Failure.php',
    'PHPUnit\\Framework\\TestStatus\\Incomplete' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Incomplete.php',
    'PHPUnit\\Framework\\TestStatus\\Known' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Known.php',
    'PHPUnit\\Framework\\TestStatus\\Notice' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Notice.php',
    'PHPUnit\\Framework\\TestStatus\\Risky' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Risky.php',
    'PHPUnit\\Framework\\TestStatus\\Skipped' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Skipped.php',
    'PHPUnit\\Framework\\TestStatus\\Success' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Success.php',
    'PHPUnit\\Framework\\TestStatus\\TestStatus' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/TestStatus.php',
    'PHPUnit\\Framework\\TestStatus\\Unknown' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Unknown.php',
    'PHPUnit\\Framework\\TestStatus\\Warning' => $vendorDir . '/phpunit/phpunit/src/Framework/TestStatus/Warning.php',
    'PHPUnit\\Framework\\TestSuite' => $vendorDir . '/phpunit/phpunit/src/Framework/TestSuite.php',
    'PHPUnit\\Framework\\TestSuiteIterator' => $vendorDir . '/phpunit/phpunit/src/Framework/TestSuiteIterator.php',
    'PHPUnit\\Framework\\UnknownClassOrInterfaceException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/UnknownClassOrInterfaceException.php',
    'PHPUnit\\Framework\\UnknownTypeException' => $vendorDir . '/phpunit/phpunit/src/Framework/Exception/UnknownTypeException.php',
    'PHPUnit\\Logging\\EventLogger' => $vendorDir . '/phpunit/phpunit/src/Logging/EventLogger.php',
    'PHPUnit\\Logging\\JUnit\\JunitXmlLogger' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/JunitXmlLogger.php',
    'PHPUnit\\Logging\\JUnit\\Subscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/Subscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestErroredSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestFailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestFailedSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestFinishedSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestMarkedIncompleteSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestMarkedIncompleteSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestPreparationFailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestPreparationFailedSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestPreparationStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestPreparationStartedSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestPreparedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestPreparedSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestRunnerExecutionFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestRunnerExecutionFinishedSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestSkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestSkippedSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestSuiteFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestSuiteFinishedSubscriber.php',
    'PHPUnit\\Logging\\JUnit\\TestSuiteStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/JUnit/Subscriber/TestSuiteStartedSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\Subscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/Subscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TeamCityLogger' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/TeamCityLogger.php',
    'PHPUnit\\Logging\\TeamCity\\TestConsideredRiskySubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestConsideredRiskySubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestErroredSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestFailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestFailedSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestFinishedSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestMarkedIncompleteSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestMarkedIncompleteSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestPreparedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestPreparedSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestRunnerExecutionFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestRunnerExecutionFinishedSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestSkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestSkippedSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestSuiteBeforeFirstTestMethodErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestSuiteBeforeFirstTestMethodErroredSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestSuiteFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestSuiteFinishedSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestSuiteSkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestSuiteSkippedSubscriber.php',
    'PHPUnit\\Logging\\TeamCity\\TestSuiteStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TeamCity/Subscriber/TestSuiteStartedSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\HtmlRenderer' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/HtmlRenderer.php',
    'PHPUnit\\Logging\\TestDox\\NamePrettifier' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/NamePrettifier.php',
    'PHPUnit\\Logging\\TestDox\\PlainTextRenderer' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/PlainTextRenderer.php',
    'PHPUnit\\Logging\\TestDox\\Subscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/Subscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestConsideredRiskySubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestConsideredRiskySubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestErroredSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestFailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestFailedSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestFinishedSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestMarkedIncompleteSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestMarkedIncompleteSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestPassedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestPassedSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestPreparedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestPreparedSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestResult' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/TestResult.php',
    'PHPUnit\\Logging\\TestDox\\TestResultCollection' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/TestResultCollection.php',
    'PHPUnit\\Logging\\TestDox\\TestResultCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/TestResultCollectionIterator.php',
    'PHPUnit\\Logging\\TestDox\\TestResultCollector' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/TestResultCollector.php',
    'PHPUnit\\Logging\\TestDox\\TestSkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestSkippedSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestTriggeredDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestTriggeredDeprecationSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestTriggeredNoticeSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestTriggeredNoticeSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestTriggeredPhpDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestTriggeredPhpDeprecationSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestTriggeredPhpNoticeSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestTriggeredPhpNoticeSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestTriggeredPhpWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestTriggeredPhpWarningSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestTriggeredPhpunitDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestTriggeredPhpunitDeprecationSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestTriggeredPhpunitErrorSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestTriggeredPhpunitErrorSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestTriggeredPhpunitWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestTriggeredPhpunitWarningSubscriber.php',
    'PHPUnit\\Logging\\TestDox\\TestTriggeredWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/Logging/TestDox/TestResult/Subscriber/TestTriggeredWarningSubscriber.php',
    'PHPUnit\\Metadata\\After' => $vendorDir . '/phpunit/phpunit/src/Metadata/After.php',
    'PHPUnit\\Metadata\\AfterClass' => $vendorDir . '/phpunit/phpunit/src/Metadata/AfterClass.php',
    'PHPUnit\\Metadata\\Annotation\\Parser\\DocBlock' => $vendorDir . '/phpunit/phpunit/src/Metadata/Parser/Annotation/DocBlock.php',
    'PHPUnit\\Metadata\\Annotation\\Parser\\Registry' => $vendorDir . '/phpunit/phpunit/src/Metadata/Parser/Annotation/Registry.php',
    'PHPUnit\\Metadata\\AnnotationsAreNotSupportedForInternalClassesException' => $vendorDir . '/phpunit/phpunit/src/Metadata/Exception/AnnotationsAreNotSupportedForInternalClassesException.php',
    'PHPUnit\\Metadata\\Api\\CodeCoverage' => $vendorDir . '/phpunit/phpunit/src/Metadata/Api/CodeCoverage.php',
    'PHPUnit\\Metadata\\Api\\DataProvider' => $vendorDir . '/phpunit/phpunit/src/Metadata/Api/DataProvider.php',
    'PHPUnit\\Metadata\\Api\\Dependencies' => $vendorDir . '/phpunit/phpunit/src/Metadata/Api/Dependencies.php',
    'PHPUnit\\Metadata\\Api\\Groups' => $vendorDir . '/phpunit/phpunit/src/Metadata/Api/Groups.php',
    'PHPUnit\\Metadata\\Api\\HookMethods' => $vendorDir . '/phpunit/phpunit/src/Metadata/Api/HookMethods.php',
    'PHPUnit\\Metadata\\Api\\Requirements' => $vendorDir . '/phpunit/phpunit/src/Metadata/Api/Requirements.php',
    'PHPUnit\\Metadata\\BackupGlobals' => $vendorDir . '/phpunit/phpunit/src/Metadata/BackupGlobals.php',
    'PHPUnit\\Metadata\\BackupStaticProperties' => $vendorDir . '/phpunit/phpunit/src/Metadata/BackupStaticProperties.php',
    'PHPUnit\\Metadata\\Before' => $vendorDir . '/phpunit/phpunit/src/Metadata/Before.php',
    'PHPUnit\\Metadata\\BeforeClass' => $vendorDir . '/phpunit/phpunit/src/Metadata/BeforeClass.php',
    'PHPUnit\\Metadata\\Covers' => $vendorDir . '/phpunit/phpunit/src/Metadata/Covers.php',
    'PHPUnit\\Metadata\\CoversClass' => $vendorDir . '/phpunit/phpunit/src/Metadata/CoversClass.php',
    'PHPUnit\\Metadata\\CoversDefaultClass' => $vendorDir . '/phpunit/phpunit/src/Metadata/CoversDefaultClass.php',
    'PHPUnit\\Metadata\\CoversFunction' => $vendorDir . '/phpunit/phpunit/src/Metadata/CoversFunction.php',
    'PHPUnit\\Metadata\\CoversNothing' => $vendorDir . '/phpunit/phpunit/src/Metadata/CoversNothing.php',
    'PHPUnit\\Metadata\\DataProvider' => $vendorDir . '/phpunit/phpunit/src/Metadata/DataProvider.php',
    'PHPUnit\\Metadata\\DependsOnClass' => $vendorDir . '/phpunit/phpunit/src/Metadata/DependsOnClass.php',
    'PHPUnit\\Metadata\\DependsOnMethod' => $vendorDir . '/phpunit/phpunit/src/Metadata/DependsOnMethod.php',
    'PHPUnit\\Metadata\\DoesNotPerformAssertions' => $vendorDir . '/phpunit/phpunit/src/Metadata/DoesNotPerformAssertions.php',
    'PHPUnit\\Metadata\\Exception' => $vendorDir . '/phpunit/phpunit/src/Metadata/Exception/Exception.php',
    'PHPUnit\\Metadata\\ExcludeGlobalVariableFromBackup' => $vendorDir . '/phpunit/phpunit/src/Metadata/ExcludeGlobalVariableFromBackup.php',
    'PHPUnit\\Metadata\\ExcludeStaticPropertyFromBackup' => $vendorDir . '/phpunit/phpunit/src/Metadata/ExcludeStaticPropertyFromBackup.php',
    'PHPUnit\\Metadata\\Group' => $vendorDir . '/phpunit/phpunit/src/Metadata/Group.php',
    'PHPUnit\\Metadata\\IgnoreClassForCodeCoverage' => $vendorDir . '/phpunit/phpunit/src/Metadata/IgnoreClassForCodeCoverage.php',
    'PHPUnit\\Metadata\\IgnoreDeprecations' => $vendorDir . '/phpunit/phpunit/src/Metadata/IgnoreDeprecations.php',
    'PHPUnit\\Metadata\\IgnoreFunctionForCodeCoverage' => $vendorDir . '/phpunit/phpunit/src/Metadata/IgnoreFunctionForCodeCoverage.php',
    'PHPUnit\\Metadata\\IgnoreMethodForCodeCoverage' => $vendorDir . '/phpunit/phpunit/src/Metadata/IgnoreMethodForCodeCoverage.php',
    'PHPUnit\\Metadata\\InvalidVersionRequirementException' => $vendorDir . '/phpunit/phpunit/src/Metadata/Exception/InvalidVersionRequirementException.php',
    'PHPUnit\\Metadata\\Metadata' => $vendorDir . '/phpunit/phpunit/src/Metadata/Metadata.php',
    'PHPUnit\\Metadata\\MetadataCollection' => $vendorDir . '/phpunit/phpunit/src/Metadata/MetadataCollection.php',
    'PHPUnit\\Metadata\\MetadataCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/Metadata/MetadataCollectionIterator.php',
    'PHPUnit\\Metadata\\NoVersionRequirementException' => $vendorDir . '/phpunit/phpunit/src/Metadata/Exception/NoVersionRequirementException.php',
    'PHPUnit\\Metadata\\Parser\\AnnotationParser' => $vendorDir . '/phpunit/phpunit/src/Metadata/Parser/AnnotationParser.php',
    'PHPUnit\\Metadata\\Parser\\AttributeParser' => $vendorDir . '/phpunit/phpunit/src/Metadata/Parser/AttributeParser.php',
    'PHPUnit\\Metadata\\Parser\\CachingParser' => $vendorDir . '/phpunit/phpunit/src/Metadata/Parser/CachingParser.php',
    'PHPUnit\\Metadata\\Parser\\Parser' => $vendorDir . '/phpunit/phpunit/src/Metadata/Parser/Parser.php',
    'PHPUnit\\Metadata\\Parser\\ParserChain' => $vendorDir . '/phpunit/phpunit/src/Metadata/Parser/ParserChain.php',
    'PHPUnit\\Metadata\\Parser\\Registry' => $vendorDir . '/phpunit/phpunit/src/Metadata/Parser/Registry.php',
    'PHPUnit\\Metadata\\PostCondition' => $vendorDir . '/phpunit/phpunit/src/Metadata/PostCondition.php',
    'PHPUnit\\Metadata\\PreCondition' => $vendorDir . '/phpunit/phpunit/src/Metadata/PreCondition.php',
    'PHPUnit\\Metadata\\PreserveGlobalState' => $vendorDir . '/phpunit/phpunit/src/Metadata/PreserveGlobalState.php',
    'PHPUnit\\Metadata\\ReflectionException' => $vendorDir . '/phpunit/phpunit/src/Metadata/Exception/ReflectionException.php',
    'PHPUnit\\Metadata\\RequiresFunction' => $vendorDir . '/phpunit/phpunit/src/Metadata/RequiresFunction.php',
    'PHPUnit\\Metadata\\RequiresMethod' => $vendorDir . '/phpunit/phpunit/src/Metadata/RequiresMethod.php',
    'PHPUnit\\Metadata\\RequiresOperatingSystem' => $vendorDir . '/phpunit/phpunit/src/Metadata/RequiresOperatingSystem.php',
    'PHPUnit\\Metadata\\RequiresOperatingSystemFamily' => $vendorDir . '/phpunit/phpunit/src/Metadata/RequiresOperatingSystemFamily.php',
    'PHPUnit\\Metadata\\RequiresPhp' => $vendorDir . '/phpunit/phpunit/src/Metadata/RequiresPhp.php',
    'PHPUnit\\Metadata\\RequiresPhpExtension' => $vendorDir . '/phpunit/phpunit/src/Metadata/RequiresPhpExtension.php',
    'PHPUnit\\Metadata\\RequiresPhpunit' => $vendorDir . '/phpunit/phpunit/src/Metadata/RequiresPhpunit.php',
    'PHPUnit\\Metadata\\RequiresSetting' => $vendorDir . '/phpunit/phpunit/src/Metadata/RequiresSetting.php',
    'PHPUnit\\Metadata\\RunClassInSeparateProcess' => $vendorDir . '/phpunit/phpunit/src/Metadata/RunClassInSeparateProcess.php',
    'PHPUnit\\Metadata\\RunInSeparateProcess' => $vendorDir . '/phpunit/phpunit/src/Metadata/RunInSeparateProcess.php',
    'PHPUnit\\Metadata\\RunTestsInSeparateProcesses' => $vendorDir . '/phpunit/phpunit/src/Metadata/RunTestsInSeparateProcesses.php',
    'PHPUnit\\Metadata\\Test' => $vendorDir . '/phpunit/phpunit/src/Metadata/Test.php',
    'PHPUnit\\Metadata\\TestDox' => $vendorDir . '/phpunit/phpunit/src/Metadata/TestDox.php',
    'PHPUnit\\Metadata\\TestWith' => $vendorDir . '/phpunit/phpunit/src/Metadata/TestWith.php',
    'PHPUnit\\Metadata\\Uses' => $vendorDir . '/phpunit/phpunit/src/Metadata/Uses.php',
    'PHPUnit\\Metadata\\UsesClass' => $vendorDir . '/phpunit/phpunit/src/Metadata/UsesClass.php',
    'PHPUnit\\Metadata\\UsesDefaultClass' => $vendorDir . '/phpunit/phpunit/src/Metadata/UsesDefaultClass.php',
    'PHPUnit\\Metadata\\UsesFunction' => $vendorDir . '/phpunit/phpunit/src/Metadata/UsesFunction.php',
    'PHPUnit\\Metadata\\Version\\ComparisonRequirement' => $vendorDir . '/phpunit/phpunit/src/Metadata/Version/ComparisonRequirement.php',
    'PHPUnit\\Metadata\\Version\\ConstraintRequirement' => $vendorDir . '/phpunit/phpunit/src/Metadata/Version/ConstraintRequirement.php',
    'PHPUnit\\Metadata\\Version\\Requirement' => $vendorDir . '/phpunit/phpunit/src/Metadata/Version/Requirement.php',
    'PHPUnit\\Metadata\\WithoutErrorHandler' => $vendorDir . '/phpunit/phpunit/src/Metadata/WithoutErrorHandler.php',
    'PHPUnit\\Runner\\Baseline\\Baseline' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Baseline.php',
    'PHPUnit\\Runner\\Baseline\\CannotLoadBaselineException' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Exception/CannotLoadBaselineException.php',
    'PHPUnit\\Runner\\Baseline\\FileDoesNotHaveLineException' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Exception/FileDoesNotHaveLineException.php',
    'PHPUnit\\Runner\\Baseline\\Generator' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Generator.php',
    'PHPUnit\\Runner\\Baseline\\Issue' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Issue.php',
    'PHPUnit\\Runner\\Baseline\\Reader' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Reader.php',
    'PHPUnit\\Runner\\Baseline\\RelativePathCalculator' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/RelativePathCalculator.php',
    'PHPUnit\\Runner\\Baseline\\Subscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Subscriber/Subscriber.php',
    'PHPUnit\\Runner\\Baseline\\TestTriggeredDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Subscriber/TestTriggeredDeprecationSubscriber.php',
    'PHPUnit\\Runner\\Baseline\\TestTriggeredNoticeSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Subscriber/TestTriggeredNoticeSubscriber.php',
    'PHPUnit\\Runner\\Baseline\\TestTriggeredPhpDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Subscriber/TestTriggeredPhpDeprecationSubscriber.php',
    'PHPUnit\\Runner\\Baseline\\TestTriggeredPhpNoticeSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Subscriber/TestTriggeredPhpNoticeSubscriber.php',
    'PHPUnit\\Runner\\Baseline\\TestTriggeredPhpWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Subscriber/TestTriggeredPhpWarningSubscriber.php',
    'PHPUnit\\Runner\\Baseline\\TestTriggeredWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Subscriber/TestTriggeredWarningSubscriber.php',
    'PHPUnit\\Runner\\Baseline\\Writer' => $vendorDir . '/phpunit/phpunit/src/Runner/Baseline/Writer.php',
    'PHPUnit\\Runner\\ClassCannotBeFoundException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/ClassCannotBeFoundException.php',
    'PHPUnit\\Runner\\ClassDoesNotExtendTestCaseException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/ClassDoesNotExtendTestCaseException.php',
    'PHPUnit\\Runner\\ClassIsAbstractException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/ClassIsAbstractException.php',
    'PHPUnit\\Runner\\CodeCoverage' => $vendorDir . '/phpunit/phpunit/src/Runner/CodeCoverage.php',
    'PHPUnit\\Runner\\DirectoryDoesNotExistException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/DirectoryDoesNotExistException.php',
    'PHPUnit\\Runner\\ErrorException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/ErrorException.php',
    'PHPUnit\\Runner\\ErrorHandler' => $vendorDir . '/phpunit/phpunit/src/Runner/ErrorHandler.php',
    'PHPUnit\\Runner\\Exception' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/Exception.php',
    'PHPUnit\\Runner\\Extension\\Extension' => $vendorDir . '/phpunit/phpunit/src/Runner/Extension/Extension.php',
    'PHPUnit\\Runner\\Extension\\ExtensionBootstrapper' => $vendorDir . '/phpunit/phpunit/src/Runner/Extension/ExtensionBootstrapper.php',
    'PHPUnit\\Runner\\Extension\\Facade' => $vendorDir . '/phpunit/phpunit/src/Runner/Extension/Facade.php',
    'PHPUnit\\Runner\\Extension\\ParameterCollection' => $vendorDir . '/phpunit/phpunit/src/Runner/Extension/ParameterCollection.php',
    'PHPUnit\\Runner\\Extension\\PharLoader' => $vendorDir . '/phpunit/phpunit/src/Runner/Extension/PharLoader.php',
    'PHPUnit\\Runner\\FileDoesNotExistException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/FileDoesNotExistException.php',
    'PHPUnit\\Runner\\Filter\\ExcludeGroupFilterIterator' => $vendorDir . '/phpunit/phpunit/src/Runner/Filter/ExcludeGroupFilterIterator.php',
    'PHPUnit\\Runner\\Filter\\Factory' => $vendorDir . '/phpunit/phpunit/src/Runner/Filter/Factory.php',
    'PHPUnit\\Runner\\Filter\\GroupFilterIterator' => $vendorDir . '/phpunit/phpunit/src/Runner/Filter/GroupFilterIterator.php',
    'PHPUnit\\Runner\\Filter\\IncludeGroupFilterIterator' => $vendorDir . '/phpunit/phpunit/src/Runner/Filter/IncludeGroupFilterIterator.php',
    'PHPUnit\\Runner\\Filter\\NameFilterIterator' => $vendorDir . '/phpunit/phpunit/src/Runner/Filter/NameFilterIterator.php',
    'PHPUnit\\Runner\\Filter\\TestIdFilterIterator' => $vendorDir . '/phpunit/phpunit/src/Runner/Filter/TestIdFilterIterator.php',
    'PHPUnit\\Runner\\GarbageCollection\\ExecutionFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/GarbageCollection/Subscriber/ExecutionFinishedSubscriber.php',
    'PHPUnit\\Runner\\GarbageCollection\\ExecutionStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/GarbageCollection/Subscriber/ExecutionStartedSubscriber.php',
    'PHPUnit\\Runner\\GarbageCollection\\GarbageCollectionHandler' => $vendorDir . '/phpunit/phpunit/src/Runner/GarbageCollection/GarbageCollectionHandler.php',
    'PHPUnit\\Runner\\GarbageCollection\\Subscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/GarbageCollection/Subscriber/Subscriber.php',
    'PHPUnit\\Runner\\GarbageCollection\\TestFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/GarbageCollection/Subscriber/TestFinishedSubscriber.php',
    'PHPUnit\\Runner\\InvalidOrderException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/InvalidOrderException.php',
    'PHPUnit\\Runner\\InvalidPhptFileException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/InvalidPhptFileException.php',
    'PHPUnit\\Runner\\ParameterDoesNotExistException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/ParameterDoesNotExistException.php',
    'PHPUnit\\Runner\\PhptExternalFileCannotBeLoadedException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/PhptExternalFileCannotBeLoadedException.php',
    'PHPUnit\\Runner\\PhptTestCase' => $vendorDir . '/phpunit/phpunit/src/Runner/PhptTestCase.php',
    'PHPUnit\\Runner\\ResultCache\\DefaultResultCache' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/DefaultResultCache.php',
    'PHPUnit\\Runner\\ResultCache\\NullResultCache' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/NullResultCache.php',
    'PHPUnit\\Runner\\ResultCache\\ResultCache' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/ResultCache.php',
    'PHPUnit\\Runner\\ResultCache\\ResultCacheHandler' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/ResultCacheHandler.php',
    'PHPUnit\\Runner\\ResultCache\\Subscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/Subscriber.php',
    'PHPUnit\\Runner\\ResultCache\\TestConsideredRiskySubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/TestConsideredRiskySubscriber.php',
    'PHPUnit\\Runner\\ResultCache\\TestErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/TestErroredSubscriber.php',
    'PHPUnit\\Runner\\ResultCache\\TestFailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/TestFailedSubscriber.php',
    'PHPUnit\\Runner\\ResultCache\\TestFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/TestFinishedSubscriber.php',
    'PHPUnit\\Runner\\ResultCache\\TestMarkedIncompleteSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/TestMarkedIncompleteSubscriber.php',
    'PHPUnit\\Runner\\ResultCache\\TestPreparedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/TestPreparedSubscriber.php',
    'PHPUnit\\Runner\\ResultCache\\TestSkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/TestSkippedSubscriber.php',
    'PHPUnit\\Runner\\ResultCache\\TestSuiteFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/TestSuiteFinishedSubscriber.php',
    'PHPUnit\\Runner\\ResultCache\\TestSuiteStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/ResultCache/Subscriber/TestSuiteStartedSubscriber.php',
    'PHPUnit\\Runner\\TestSuiteLoader' => $vendorDir . '/phpunit/phpunit/src/Runner/TestSuiteLoader.php',
    'PHPUnit\\Runner\\TestSuiteSorter' => $vendorDir . '/phpunit/phpunit/src/Runner/TestSuiteSorter.php',
    'PHPUnit\\Runner\\UnsupportedPhptSectionException' => $vendorDir . '/phpunit/phpunit/src/Runner/Exception/UnsupportedPhptSectionException.php',
    'PHPUnit\\Runner\\Version' => $vendorDir . '/phpunit/phpunit/src/Runner/Version.php',
    'PHPUnit\\TestRunner\\TestResult\\BeforeTestClassMethodErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/BeforeTestClassMethodErroredSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\Collector' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Collector.php',
    'PHPUnit\\TestRunner\\TestResult\\ExecutionStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/ExecutionStartedSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\Facade' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Facade.php',
    'PHPUnit\\TestRunner\\TestResult\\Issues\\Issue' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Issue.php',
    'PHPUnit\\TestRunner\\TestResult\\PassedTests' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/PassedTests.php',
    'PHPUnit\\TestRunner\\TestResult\\Subscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/Subscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestConsideredRiskySubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestConsideredRiskySubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestErroredSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestFailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestFailedSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestFinishedSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestMarkedIncompleteSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestMarkedIncompleteSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestPreparedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestPreparedSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestResult' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/TestResult.php',
    'PHPUnit\\TestRunner\\TestResult\\TestRunnerTriggeredDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestRunnerTriggeredDeprecationSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestRunnerTriggeredWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestRunnerTriggeredWarningSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestSkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestSkippedSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestSuiteFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestSuiteFinishedSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestSuiteSkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestSuiteSkippedSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestSuiteStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestSuiteStartedSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredDeprecationSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredErrorSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredErrorSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredNoticeSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredNoticeSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredPhpDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredPhpDeprecationSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredPhpNoticeSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredPhpNoticeSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredPhpWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredPhpWarningSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredPhpunitDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredPhpunitDeprecationSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredPhpunitErrorSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredPhpunitErrorSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredPhpunitWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredPhpunitWarningSubscriber.php',
    'PHPUnit\\TestRunner\\TestResult\\TestTriggeredWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/Runner/TestResult/Subscriber/TestTriggeredWarningSubscriber.php',
    'PHPUnit\\TextUI\\Application' => $vendorDir . '/phpunit/phpunit/src/TextUI/Application.php',
    'PHPUnit\\TextUI\\CannotOpenSocketException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Exception/CannotOpenSocketException.php',
    'PHPUnit\\TextUI\\CliArguments\\Builder' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Cli/Builder.php',
    'PHPUnit\\TextUI\\CliArguments\\Configuration' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Cli/Configuration.php',
    'PHPUnit\\TextUI\\CliArguments\\Exception' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Cli/Exception.php',
    'PHPUnit\\TextUI\\CliArguments\\XmlConfigurationFileFinder' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Cli/XmlConfigurationFileFinder.php',
    'PHPUnit\\TextUI\\Command\\AtLeastVersionCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/AtLeastVersionCommand.php',
    'PHPUnit\\TextUI\\Command\\Command' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Command.php',
    'PHPUnit\\TextUI\\Command\\GenerateConfigurationCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/GenerateConfigurationCommand.php',
    'PHPUnit\\TextUI\\Command\\ListGroupsCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/ListGroupsCommand.php',
    'PHPUnit\\TextUI\\Command\\ListTestSuitesCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/ListTestSuitesCommand.php',
    'PHPUnit\\TextUI\\Command\\ListTestsAsTextCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/ListTestsAsTextCommand.php',
    'PHPUnit\\TextUI\\Command\\ListTestsAsXmlCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/ListTestsAsXmlCommand.php',
    'PHPUnit\\TextUI\\Command\\MigrateConfigurationCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/MigrateConfigurationCommand.php',
    'PHPUnit\\TextUI\\Command\\Result' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Result.php',
    'PHPUnit\\TextUI\\Command\\ShowHelpCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/ShowHelpCommand.php',
    'PHPUnit\\TextUI\\Command\\ShowVersionCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/ShowVersionCommand.php',
    'PHPUnit\\TextUI\\Command\\VersionCheckCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/VersionCheckCommand.php',
    'PHPUnit\\TextUI\\Command\\WarmCodeCoverageCacheCommand' => $vendorDir . '/phpunit/phpunit/src/TextUI/Command/Commands/WarmCodeCoverageCacheCommand.php',
    'PHPUnit\\TextUI\\Configuration\\Builder' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Builder.php',
    'PHPUnit\\TextUI\\Configuration\\CodeCoverageFilterRegistry' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/CodeCoverageFilterRegistry.php',
    'PHPUnit\\TextUI\\Configuration\\CodeCoverageReportNotConfiguredException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/CodeCoverageReportNotConfiguredException.php',
    'PHPUnit\\TextUI\\Configuration\\Configuration' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Configuration.php',
    'PHPUnit\\TextUI\\Configuration\\ConfigurationCannotBeBuiltException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/ConfigurationCannotBeBuiltException.php',
    'PHPUnit\\TextUI\\Configuration\\Constant' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/Constant.php',
    'PHPUnit\\TextUI\\Configuration\\ConstantCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/ConstantCollection.php',
    'PHPUnit\\TextUI\\Configuration\\ConstantCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/ConstantCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\Directory' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/Directory.php',
    'PHPUnit\\TextUI\\Configuration\\DirectoryCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/DirectoryCollection.php',
    'PHPUnit\\TextUI\\Configuration\\DirectoryCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/DirectoryCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\Exception' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/Exception.php',
    'PHPUnit\\TextUI\\Configuration\\ExtensionBootstrap' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/ExtensionBootstrap.php',
    'PHPUnit\\TextUI\\Configuration\\ExtensionBootstrapCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/ExtensionBootstrapCollection.php',
    'PHPUnit\\TextUI\\Configuration\\ExtensionBootstrapCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/ExtensionBootstrapCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\File' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/File.php',
    'PHPUnit\\TextUI\\Configuration\\FileCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/FileCollection.php',
    'PHPUnit\\TextUI\\Configuration\\FileCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/FileCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\FilterDirectory' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/FilterDirectory.php',
    'PHPUnit\\TextUI\\Configuration\\FilterDirectoryCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/FilterDirectoryCollection.php',
    'PHPUnit\\TextUI\\Configuration\\FilterDirectoryCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/FilterDirectoryCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\FilterNotConfiguredException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/FilterNotConfiguredException.php',
    'PHPUnit\\TextUI\\Configuration\\Group' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/Group.php',
    'PHPUnit\\TextUI\\Configuration\\GroupCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/GroupCollection.php',
    'PHPUnit\\TextUI\\Configuration\\GroupCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/GroupCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\IniSetting' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/IniSetting.php',
    'PHPUnit\\TextUI\\Configuration\\IniSettingCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/IniSettingCollection.php',
    'PHPUnit\\TextUI\\Configuration\\IniSettingCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/IniSettingCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\LoggingNotConfiguredException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/LoggingNotConfiguredException.php',
    'PHPUnit\\TextUI\\Configuration\\Merger' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Merger.php',
    'PHPUnit\\TextUI\\Configuration\\NoBaselineException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/NoBaselineException.php',
    'PHPUnit\\TextUI\\Configuration\\NoBootstrapException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/NoBootstrapException.php',
    'PHPUnit\\TextUI\\Configuration\\NoCacheDirectoryException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/NoCacheDirectoryException.php',
    'PHPUnit\\TextUI\\Configuration\\NoCliArgumentException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/NoCliArgumentException.php',
    'PHPUnit\\TextUI\\Configuration\\NoConfigurationFileException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/NoConfigurationFileException.php',
    'PHPUnit\\TextUI\\Configuration\\NoCoverageCacheDirectoryException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/NoCoverageCacheDirectoryException.php',
    'PHPUnit\\TextUI\\Configuration\\NoCustomCssFileException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/NoCustomCssFileException.php',
    'PHPUnit\\TextUI\\Configuration\\NoDefaultTestSuiteException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/NoDefaultTestSuiteException.php',
    'PHPUnit\\TextUI\\Configuration\\NoPharExtensionDirectoryException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/NoPharExtensionDirectoryException.php',
    'PHPUnit\\TextUI\\Configuration\\Php' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/Php.php',
    'PHPUnit\\TextUI\\Configuration\\PhpHandler' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/PhpHandler.php',
    'PHPUnit\\TextUI\\Configuration\\Registry' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Registry.php',
    'PHPUnit\\TextUI\\Configuration\\Source' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/Source.php',
    'PHPUnit\\TextUI\\Configuration\\SourceFilter' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/SourceFilter.php',
    'PHPUnit\\TextUI\\Configuration\\SourceMapper' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/SourceMapper.php',
    'PHPUnit\\TextUI\\Configuration\\TestDirectory' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/TestDirectory.php',
    'PHPUnit\\TextUI\\Configuration\\TestDirectoryCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/TestDirectoryCollection.php',
    'PHPUnit\\TextUI\\Configuration\\TestDirectoryCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/TestDirectoryCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\TestFile' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/TestFile.php',
    'PHPUnit\\TextUI\\Configuration\\TestFileCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/TestFileCollection.php',
    'PHPUnit\\TextUI\\Configuration\\TestFileCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/TestFileCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\TestSuite' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/TestSuite.php',
    'PHPUnit\\TextUI\\Configuration\\TestSuiteBuilder' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/TestSuiteBuilder.php',
    'PHPUnit\\TextUI\\Configuration\\TestSuiteCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/TestSuiteCollection.php',
    'PHPUnit\\TextUI\\Configuration\\TestSuiteCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/TestSuiteCollectionIterator.php',
    'PHPUnit\\TextUI\\Configuration\\Variable' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/Variable.php',
    'PHPUnit\\TextUI\\Configuration\\VariableCollection' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/VariableCollection.php',
    'PHPUnit\\TextUI\\Configuration\\VariableCollectionIterator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Value/VariableCollectionIterator.php',
    'PHPUnit\\TextUI\\Exception' => $vendorDir . '/phpunit/phpunit/src/TextUI/Exception/Exception.php',
    'PHPUnit\\TextUI\\Help' => $vendorDir . '/phpunit/phpunit/src/TextUI/Help.php',
    'PHPUnit\\TextUI\\InvalidSocketException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Exception/InvalidSocketException.php',
    'PHPUnit\\TextUI\\Output\\DefaultPrinter' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Printer/DefaultPrinter.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\BeforeTestClassMethodErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/BeforeTestClassMethodErroredSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\ProgressPrinter' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/ProgressPrinter.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\Subscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/Subscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestConsideredRiskySubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestConsideredRiskySubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestErroredSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestErroredSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestFailedSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestFailedSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestFinishedSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestFinishedSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestMarkedIncompleteSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestMarkedIncompleteSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestPreparedSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestPreparedSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestRunnerExecutionStartedSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestRunnerExecutionStartedSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestSkippedSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestSkippedSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestTriggeredDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestTriggeredDeprecationSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestTriggeredErrorSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestTriggeredErrorSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestTriggeredNoticeSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestTriggeredNoticeSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestTriggeredPhpDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestTriggeredPhpDeprecationSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestTriggeredPhpNoticeSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestTriggeredPhpNoticeSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestTriggeredPhpWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestTriggeredPhpWarningSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestTriggeredPhpunitDeprecationSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestTriggeredPhpunitDeprecationSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestTriggeredPhpunitWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestTriggeredPhpunitWarningSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ProgressPrinter\\TestTriggeredWarningSubscriber' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ProgressPrinter/Subscriber/TestTriggeredWarningSubscriber.php',
    'PHPUnit\\TextUI\\Output\\Default\\ResultPrinter' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/ResultPrinter.php',
    'PHPUnit\\TextUI\\Output\\Default\\UnexpectedOutputPrinter' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Default/UnexpectedOutputPrinter.php',
    'PHPUnit\\TextUI\\Output\\Facade' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Facade.php',
    'PHPUnit\\TextUI\\Output\\NullPrinter' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Printer/NullPrinter.php',
    'PHPUnit\\TextUI\\Output\\Printer' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/Printer/Printer.php',
    'PHPUnit\\TextUI\\Output\\SummaryPrinter' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/SummaryPrinter.php',
    'PHPUnit\\TextUI\\Output\\TestDox\\ResultPrinter' => $vendorDir . '/phpunit/phpunit/src/TextUI/Output/TestDox/ResultPrinter.php',
    'PHPUnit\\TextUI\\RuntimeException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Exception/RuntimeException.php',
    'PHPUnit\\TextUI\\ShellExitCodeCalculator' => $vendorDir . '/phpunit/phpunit/src/TextUI/ShellExitCodeCalculator.php',
    'PHPUnit\\TextUI\\TestDirectoryNotFoundException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Exception/TestDirectoryNotFoundException.php',
    'PHPUnit\\TextUI\\TestFileNotFoundException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Exception/TestFileNotFoundException.php',
    'PHPUnit\\TextUI\\TestRunner' => $vendorDir . '/phpunit/phpunit/src/TextUI/TestRunner.php',
    'PHPUnit\\TextUI\\TestSuiteFilterProcessor' => $vendorDir . '/phpunit/phpunit/src/TextUI/TestSuiteFilterProcessor.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CannotFindSchemaException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Exception/CannotFindSchemaException.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CodeCoverage\\CodeCoverage' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/CodeCoverage/CodeCoverage.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CodeCoverage\\Report\\Clover' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/CodeCoverage/Report/Clover.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CodeCoverage\\Report\\Cobertura' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/CodeCoverage/Report/Cobertura.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CodeCoverage\\Report\\Crap4j' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/CodeCoverage/Report/Crap4j.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CodeCoverage\\Report\\Html' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/CodeCoverage/Report/Html.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CodeCoverage\\Report\\Php' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/CodeCoverage/Report/Php.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CodeCoverage\\Report\\Text' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/CodeCoverage/Report/Text.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CodeCoverage\\Report\\Xml' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/CodeCoverage/Report/Xml.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Configuration' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Configuration.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\ConvertLogTypes' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/ConvertLogTypes.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CoverageCloverToReport' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/CoverageCloverToReport.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CoverageCrap4jToReport' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/CoverageCrap4jToReport.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CoverageHtmlToReport' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/CoverageHtmlToReport.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CoveragePhpToReport' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/CoveragePhpToReport.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CoverageTextToReport' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/CoverageTextToReport.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\CoverageXmlToReport' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/CoverageXmlToReport.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\DefaultConfiguration' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/DefaultConfiguration.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Exception' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Exception.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\FailedSchemaDetectionResult' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/SchemaDetector/FailedSchemaDetectionResult.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Generator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Generator.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Groups' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Groups.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\IntroduceCacheDirectoryAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/IntroduceCacheDirectoryAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\IntroduceCoverageElement' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/IntroduceCoverageElement.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\LoadedFromFileConfiguration' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/LoadedFromFileConfiguration.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Loader' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Loader.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\LogToReportMigration' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/LogToReportMigration.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Logging\\Junit' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Logging/Junit.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Logging\\Logging' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Logging/Logging.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Logging\\TeamCity' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Logging/TeamCity.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Logging\\TestDox\\Html' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Logging/TestDox/Html.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Logging\\TestDox\\Text' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Logging/TestDox/Text.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Migration' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/Migration.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\MigrationBuilder' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/MigrationBuilder.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\MigrationException' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/MigrationException.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Migrator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrator.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\MoveAttributesFromFilterWhitelistToCoverage' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/MoveAttributesFromFilterWhitelistToCoverage.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\MoveAttributesFromRootToCoverage' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/MoveAttributesFromRootToCoverage.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\MoveCoverageDirectoriesToSource' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/MoveCoverageDirectoriesToSource.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\MoveWhitelistExcludesToCoverage' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/MoveWhitelistExcludesToCoverage.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\MoveWhitelistIncludesToCoverage' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/MoveWhitelistIncludesToCoverage.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\PHPUnit' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/PHPUnit.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveBeStrictAboutResourceUsageDuringSmallTestsAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveBeStrictAboutResourceUsageDuringSmallTestsAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveBeStrictAboutTodoAnnotatedTestsAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveBeStrictAboutTodoAnnotatedTestsAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveCacheResultFileAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveCacheResultFileAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveCacheTokensAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveCacheTokensAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveConversionToExceptionsAttributes' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveConversionToExceptionsAttributes.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveCoverageElementCacheDirectoryAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveCoverageElementCacheDirectoryAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveCoverageElementProcessUncoveredFilesAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveCoverageElementProcessUncoveredFilesAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveEmptyFilter' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveEmptyFilter.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveListeners' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveListeners.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveLogTypes' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveLogTypes.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveLoggingElements' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveLoggingElements.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveNoInteractionAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveNoInteractionAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemovePrinterAttributes' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemovePrinterAttributes.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveTestDoxGroupsElement' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveTestDoxGroupsElement.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveTestSuiteLoaderAttributes' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveTestSuiteLoaderAttributes.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RemoveVerboseAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RemoveVerboseAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RenameBackupStaticAttributesAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RenameBackupStaticAttributesAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RenameBeStrictAboutCoversAnnotationAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RenameBeStrictAboutCoversAnnotationAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\RenameForceCoversAnnotationAttribute' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/RenameForceCoversAnnotationAttribute.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\SchemaDetectionResult' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/SchemaDetector/SchemaDetectionResult.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\SchemaDetector' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/SchemaDetector/SchemaDetector.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\SchemaFinder' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/SchemaFinder.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\SnapshotNodeList' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/SnapshotNodeList.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\SuccessfulSchemaDetectionResult' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/SchemaDetector/SuccessfulSchemaDetectionResult.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\TestSuiteMapper' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/TestSuiteMapper.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\UpdateSchemaLocation' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Migration/Migrations/UpdateSchemaLocation.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\ValidationResult' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Validator/ValidationResult.php',
    'PHPUnit\\TextUI\\XmlConfiguration\\Validator' => $vendorDir . '/phpunit/phpunit/src/TextUI/Configuration/Xml/Validator/Validator.php',
    'PHPUnit\\Util\\Cloner' => $vendorDir . '/phpunit/phpunit/src/Util/Cloner.php',
    'PHPUnit\\Util\\Color' => $vendorDir . '/phpunit/phpunit/src/Util/Color.php',
    'PHPUnit\\Util\\Exception' => $vendorDir . '/phpunit/phpunit/src/Util/Exception/Exception.php',
    'PHPUnit\\Util\\ExcludeList' => $vendorDir . '/phpunit/phpunit/src/Util/ExcludeList.php',
    'PHPUnit\\Util\\Exporter' => $vendorDir . '/phpunit/phpunit/src/Util/Exporter.php',
    'PHPUnit\\Util\\Filesystem' => $vendorDir . '/phpunit/phpunit/src/Util/Filesystem.php',
    'PHPUnit\\Util\\Filter' => $vendorDir . '/phpunit/phpunit/src/Util/Filter.php',
    'PHPUnit\\Util\\GlobalState' => $vendorDir . '/phpunit/phpunit/src/Util/GlobalState.php',
    'PHPUnit\\Util\\Http\\Downloader' => $vendorDir . '/phpunit/phpunit/src/Util/Http/Downloader.php',
    'PHPUnit\\Util\\Http\\PhpDownloader' => $vendorDir . '/phpunit/phpunit/src/Util/Http/PhpDownloader.php',
    'PHPUnit\\Util\\InvalidDirectoryException' => $vendorDir . '/phpunit/phpunit/src/Util/Exception/InvalidDirectoryException.php',
    'PHPUnit\\Util\\InvalidJsonException' => $vendorDir . '/phpunit/phpunit/src/Util/Exception/InvalidJsonException.php',
    'PHPUnit\\Util\\InvalidVersionOperatorException' => $vendorDir . '/phpunit/phpunit/src/Util/Exception/InvalidVersionOperatorException.php',
    'PHPUnit\\Util\\Json' => $vendorDir . '/phpunit/phpunit/src/Util/Json.php',
    'PHPUnit\\Util\\PHP\\AbstractPhpProcess' => $vendorDir . '/phpunit/phpunit/src/Util/PHP/AbstractPhpProcess.php',
    'PHPUnit\\Util\\PHP\\DefaultPhpProcess' => $vendorDir . '/phpunit/phpunit/src/Util/PHP/DefaultPhpProcess.php',
    'PHPUnit\\Util\\PHP\\PhpProcessException' => $vendorDir . '/phpunit/phpunit/src/Util/Exception/PhpProcessException.php',
    'PHPUnit\\Util\\Reflection' => $vendorDir . '/phpunit/phpunit/src/Util/Reflection.php',
    'PHPUnit\\Util\\Test' => $vendorDir . '/phpunit/phpunit/src/Util/Test.php',
    'PHPUnit\\Util\\ThrowableToStringMapper' => $vendorDir . '/phpunit/phpunit/src/Util/ThrowableToStringMapper.php',
    'PHPUnit\\Util\\VersionComparisonOperator' => $vendorDir . '/phpunit/phpunit/src/Util/VersionComparisonOperator.php',
    'PHPUnit\\Util\\Xml' => $vendorDir . '/phpunit/phpunit/src/Util/Xml/Xml.php',
    'PHPUnit\\Util\\Xml\\Loader' => $vendorDir . '/phpunit/phpunit/src/Util/Xml/Loader.php',
    'PHPUnit\\Util\\Xml\\XmlException' => $vendorDir . '/phpunit/phpunit/src/Util/Exception/XmlException.php',
    'PackageVersions\\FallbackVersions' => $vendorDir . '/composer/package-versions-deprecated/src/PackageVersions/FallbackVersions.php',
    'PackageVersions\\Installer' => $vendorDir . '/composer/package-versions-deprecated/src/PackageVersions/Installer.php',
    'PackageVersions\\Versions' => $vendorDir . '/composer/package-versions-deprecated/src/PackageVersions/Versions.php',
    'PharIo\\Manifest\\Application' => $vendorDir . '/phar-io/manifest/src/values/Application.php',
    'PharIo\\Manifest\\ApplicationName' => $vendorDir . '/phar-io/manifest/src/values/ApplicationName.php',
    'PharIo\\Manifest\\Author' => $vendorDir . '/phar-io/manifest/src/values/Author.php',
    'PharIo\\Manifest\\AuthorCollection' => $vendorDir . '/phar-io/manifest/src/values/AuthorCollection.php',
    'PharIo\\Manifest\\AuthorCollectionIterator' => $vendorDir . '/phar-io/manifest/src/values/AuthorCollectionIterator.php',
    'PharIo\\Manifest\\AuthorElement' => $vendorDir . '/phar-io/manifest/src/xml/AuthorElement.php',
    'PharIo\\Manifest\\AuthorElementCollection' => $vendorDir . '/phar-io/manifest/src/xml/AuthorElementCollection.php',
    'PharIo\\Manifest\\BundledComponent' => $vendorDir . '/phar-io/manifest/src/values/BundledComponent.php',
    'PharIo\\Manifest\\BundledComponentCollection' => $vendorDir . '/phar-io/manifest/src/values/BundledComponentCollection.php',
    'PharIo\\Manifest\\BundledComponentCollectionIterator' => $vendorDir . '/phar-io/manifest/src/values/BundledComponentCollectionIterator.php',
    'PharIo\\Manifest\\BundlesElement' => $vendorDir . '/phar-io/manifest/src/xml/BundlesElement.php',
    'PharIo\\Manifest\\ComponentElement' => $vendorDir . '/phar-io/manifest/src/xml/ComponentElement.php',
    'PharIo\\Manifest\\ComponentElementCollection' => $vendorDir . '/phar-io/manifest/src/xml/ComponentElementCollection.php',
    'PharIo\\Manifest\\ContainsElement' => $vendorDir . '/phar-io/manifest/src/xml/ContainsElement.php',
    'PharIo\\Manifest\\CopyrightElement' => $vendorDir . '/phar-io/manifest/src/xml/CopyrightElement.php',
    'PharIo\\Manifest\\CopyrightInformation' => $vendorDir . '/phar-io/manifest/src/values/CopyrightInformation.php',
    'PharIo\\Manifest\\ElementCollection' => $vendorDir . '/phar-io/manifest/src/xml/ElementCollection.php',
    'PharIo\\Manifest\\ElementCollectionException' => $vendorDir . '/phar-io/manifest/src/exceptions/ElementCollectionException.php',
    'PharIo\\Manifest\\Email' => $vendorDir . '/phar-io/manifest/src/values/Email.php',
    'PharIo\\Manifest\\Exception' => $vendorDir . '/phar-io/manifest/src/exceptions/Exception.php',
    'PharIo\\Manifest\\ExtElement' => $vendorDir . '/phar-io/manifest/src/xml/ExtElement.php',
    'PharIo\\Manifest\\ExtElementCollection' => $vendorDir . '/phar-io/manifest/src/xml/ExtElementCollection.php',
    'PharIo\\Manifest\\Extension' => $vendorDir . '/phar-io/manifest/src/values/Extension.php',
    'PharIo\\Manifest\\ExtensionElement' => $vendorDir . '/phar-io/manifest/src/xml/ExtensionElement.php',
    'PharIo\\Manifest\\InvalidApplicationNameException' => $vendorDir . '/phar-io/manifest/src/exceptions/InvalidApplicationNameException.php',
    'PharIo\\Manifest\\InvalidEmailException' => $vendorDir . '/phar-io/manifest/src/exceptions/InvalidEmailException.php',
    'PharIo\\Manifest\\InvalidUrlException' => $vendorDir . '/phar-io/manifest/src/exceptions/InvalidUrlException.php',
    'PharIo\\Manifest\\Library' => $vendorDir . '/phar-io/manifest/src/values/Library.php',
    'PharIo\\Manifest\\License' => $vendorDir . '/phar-io/manifest/src/values/License.php',
    'PharIo\\Manifest\\LicenseElement' => $vendorDir . '/phar-io/manifest/src/xml/LicenseElement.php',
    'PharIo\\Manifest\\Manifest' => $vendorDir . '/phar-io/manifest/src/values/Manifest.php',
    'PharIo\\Manifest\\ManifestDocument' => $vendorDir . '/phar-io/manifest/src/xml/ManifestDocument.php',
    'PharIo\\Manifest\\ManifestDocumentException' => $vendorDir . '/phar-io/manifest/src/exceptions/ManifestDocumentException.php',
    'PharIo\\Manifest\\ManifestDocumentLoadingException' => $vendorDir . '/phar-io/manifest/src/exceptions/ManifestDocumentLoadingException.php',
    'PharIo\\Manifest\\ManifestDocumentMapper' => $vendorDir . '/phar-io/manifest/src/ManifestDocumentMapper.php',
    'PharIo\\Manifest\\ManifestDocumentMapperException' => $vendorDir . '/phar-io/manifest/src/exceptions/ManifestDocumentMapperException.php',
    'PharIo\\Manifest\\ManifestElement' => $vendorDir . '/phar-io/manifest/src/xml/ManifestElement.php',
    'PharIo\\Manifest\\ManifestElementException' => $vendorDir . '/phar-io/manifest/src/exceptions/ManifestElementException.php',
    'PharIo\\Manifest\\ManifestLoader' => $vendorDir . '/phar-io/manifest/src/ManifestLoader.php',
    'PharIo\\Manifest\\ManifestLoaderException' => $vendorDir . '/phar-io/manifest/src/exceptions/ManifestLoaderException.php',
    'PharIo\\Manifest\\ManifestSerializer' => $vendorDir . '/phar-io/manifest/src/ManifestSerializer.php',
    'PharIo\\Manifest\\NoEmailAddressException' => $vendorDir . '/phar-io/manifest/src/exceptions/NoEmailAddressException.php',
    'PharIo\\Manifest\\PhpElement' => $vendorDir . '/phar-io/manifest/src/xml/PhpElement.php',
    'PharIo\\Manifest\\PhpExtensionRequirement' => $vendorDir . '/phar-io/manifest/src/values/PhpExtensionRequirement.php',
    'PharIo\\Manifest\\PhpVersionRequirement' => $vendorDir . '/phar-io/manifest/src/values/PhpVersionRequirement.php',
    'PharIo\\Manifest\\Requirement' => $vendorDir . '/phar-io/manifest/src/values/Requirement.php',
    'PharIo\\Manifest\\RequirementCollection' => $vendorDir . '/phar-io/manifest/src/values/RequirementCollection.php',
    'PharIo\\Manifest\\RequirementCollectionIterator' => $vendorDir . '/phar-io/manifest/src/values/RequirementCollectionIterator.php',
    'PharIo\\Manifest\\RequiresElement' => $vendorDir . '/phar-io/manifest/src/xml/RequiresElement.php',
    'PharIo\\Manifest\\Type' => $vendorDir . '/phar-io/manifest/src/values/Type.php',
    'PharIo\\Manifest\\Url' => $vendorDir . '/phar-io/manifest/src/values/Url.php',
    'PharIo\\Version\\AbstractVersionConstraint' => $vendorDir . '/phar-io/version/src/constraints/AbstractVersionConstraint.php',
    'PharIo\\Version\\AndVersionConstraintGroup' => $vendorDir . '/phar-io/version/src/constraints/AndVersionConstraintGroup.php',
    'PharIo\\Version\\AnyVersionConstraint' => $vendorDir . '/phar-io/version/src/constraints/AnyVersionConstraint.php',
    'PharIo\\Version\\BuildMetaData' => $vendorDir . '/phar-io/version/src/BuildMetaData.php',
    'PharIo\\Version\\ExactVersionConstraint' => $vendorDir . '/phar-io/version/src/constraints/ExactVersionConstraint.php',
    'PharIo\\Version\\Exception' => $vendorDir . '/phar-io/version/src/exceptions/Exception.php',
    'PharIo\\Version\\GreaterThanOrEqualToVersionConstraint' => $vendorDir . '/phar-io/version/src/constraints/GreaterThanOrEqualToVersionConstraint.php',
    'PharIo\\Version\\InvalidPreReleaseSuffixException' => $vendorDir . '/phar-io/version/src/exceptions/InvalidPreReleaseSuffixException.php',
    'PharIo\\Version\\InvalidVersionException' => $vendorDir . '/phar-io/version/src/exceptions/InvalidVersionException.php',
    'PharIo\\Version\\NoBuildMetaDataException' => $vendorDir . '/phar-io/version/src/exceptions/NoBuildMetaDataException.php',
    'PharIo\\Version\\NoPreReleaseSuffixException' => $vendorDir . '/phar-io/version/src/exceptions/NoPreReleaseSuffixException.php',
    'PharIo\\Version\\OrVersionConstraintGroup' => $vendorDir . '/phar-io/version/src/constraints/OrVersionConstraintGroup.php',
    'PharIo\\Version\\PreReleaseSuffix' => $vendorDir . '/phar-io/version/src/PreReleaseSuffix.php',
    'PharIo\\Version\\SpecificMajorAndMinorVersionConstraint' => $vendorDir . '/phar-io/version/src/constraints/SpecificMajorAndMinorVersionConstraint.php',
    'PharIo\\Version\\SpecificMajorVersionConstraint' => $vendorDir . '/phar-io/version/src/constraints/SpecificMajorVersionConstraint.php',
    'PharIo\\Version\\UnsupportedVersionConstraintException' => $vendorDir . '/phar-io/version/src/exceptions/UnsupportedVersionConstraintException.php',
    'PharIo\\Version\\Version' => $vendorDir . '/phar-io/version/src/Version.php',
    'PharIo\\Version\\VersionConstraint' => $vendorDir . '/phar-io/version/src/constraints/VersionConstraint.php',
    'PharIo\\Version\\VersionConstraintParser' => $vendorDir . '/phar-io/version/src/VersionConstraintParser.php',
    'PharIo\\Version\\VersionConstraintValue' => $vendorDir . '/phar-io/version/src/VersionConstraintValue.php',
    'PharIo\\Version\\VersionNumber' => $vendorDir . '/phar-io/version/src/VersionNumber.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\ArrayEnabled' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/ArrayEnabled.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\BinaryComparison' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/BinaryComparison.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Calculation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Calculation.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Category' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Category.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DAverage' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DAverage.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DCount' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DCount.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DCountA' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DCountA.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DGet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DGet.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DMax' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DMax.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DMin' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DMin.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DProduct' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DProduct.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DStDev' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DStDev.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DStDevP' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DStDevP.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DSum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DSum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DVar' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DVar.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DVarP' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DVarP.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DatabaseAbstract' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DatabaseAbstract.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTime' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTime.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Constants' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Constants.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Current' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Current.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Date' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Date.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\DateParts' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/DateParts.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\DateValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/DateValue.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Days' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Days.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Days360' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Days360.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Difference' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Difference.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Month' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Month.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\NetworkDays' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/NetworkDays.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Time' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Time.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\TimeParts' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/TimeParts.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\TimeValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/TimeValue.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Week' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Week.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\WorkDay' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/WorkDay.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\YearFrac' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/YearFrac.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\ArrayArgumentHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/ArrayArgumentHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\ArrayArgumentProcessor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/ArrayArgumentProcessor.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\BranchPruner' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/BranchPruner.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\CyclicReferenceStack' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/CyclicReferenceStack.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\FormattedNumber' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/FormattedNumber.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\Logger' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/Logger.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\Operands\\Operand' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/Operands/Operand.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\Operands\\StructuredReference' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/Operands/StructuredReference.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BesselI' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BesselI.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BesselJ' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BesselJ.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BesselK' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BesselK.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BesselY' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BesselY.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BitWise' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BitWise.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\Compare' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/Compare.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\Complex' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/Complex.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ComplexFunctions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ComplexFunctions.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ComplexOperations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ComplexOperations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\Constants' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/Constants.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertBinary' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertBinary.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertDecimal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertDecimal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertHex' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertHex.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertOctal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertOctal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertUOM' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertUOM.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\EngineeringValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/EngineeringValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\Erf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/Erf.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ErfC' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ErfC.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\ExceptionHandler' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/ExceptionHandler.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Amortization' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Amortization.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\CashFlowValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/CashFlowValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic\\Cumulative' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic/Cumulative.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic\\Interest' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic/Interest.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic\\InterestAndPrincipal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic/InterestAndPrincipal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic\\Payments' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic/Payments.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Single' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Single.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Variable\\NonPeriodic' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Variable/NonPeriodic.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Variable\\Periodic' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Variable/Periodic.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Constants' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Constants.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Coupons' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Coupons.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Depreciation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Depreciation.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Dollar' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Dollar.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\FinancialValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/FinancialValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\InterestRate' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/InterestRate.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\AccruedInterest' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/AccruedInterest.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\Price' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/Price.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\Rates' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/Rates.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\SecurityValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/SecurityValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\Yields' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/Yields.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\TreasuryBill' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/TreasuryBill.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\FormulaParser' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/FormulaParser.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\FormulaToken' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/FormulaToken.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Functions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Functions.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Information\\ErrorValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Information/ErrorValue.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Information\\ExcelError' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Information/ExcelError.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Information\\Value' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Information/Value.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Internal\\MakeMatrix' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Internal/MakeMatrix.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Internal\\WildcardMatch' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Internal/WildcardMatch.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Logical' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Logical.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Logical\\Boolean' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Logical/Boolean.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Logical\\Conditional' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Logical/Conditional.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Logical\\Operations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Logical/Operations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Address' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Address.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\ExcelMatch' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/ExcelMatch.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Filter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Filter.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Formula' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Formula.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\HLookup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/HLookup.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Hyperlink' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Hyperlink.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Indirect' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Indirect.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Lookup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Lookup.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\LookupBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/LookupBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\LookupRefValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/LookupRefValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Matrix' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Matrix.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Offset' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Offset.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\RowColumnInformation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/RowColumnInformation.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Selection' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Selection.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Sort' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Sort.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Unique' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Unique.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\VLookup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/VLookup.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Absolute' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Absolute.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Angle' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Angle.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Arabic' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Arabic.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Base' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Base.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Ceiling' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Ceiling.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Combinations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Combinations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Exp' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Exp.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Factorial' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Factorial.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Floor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Floor.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Gcd' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Gcd.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\IntClass' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/IntClass.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Lcm' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Lcm.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Logarithms' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Logarithms.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\MatrixFunctions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/MatrixFunctions.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Operations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Operations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Random' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Random.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Roman' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Roman.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Round' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Round.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\SeriesSum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/SeriesSum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Sign' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Sign.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Sqrt' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Sqrt.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Subtotal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Subtotal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Sum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Sum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\SumSquares' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/SumSquares.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Cosecant' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Cosecant.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Cosine' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Cosine.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Cotangent' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Cotangent.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Secant' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Secant.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Sine' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Sine.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Tangent' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Tangent.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trunc' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trunc.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\AggregateBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/AggregateBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Averages' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Averages.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Averages\\Mean' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Averages/Mean.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Conditional' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Conditional.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Confidence' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Confidence.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Counts' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Counts.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Deviations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Deviations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Beta' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Beta.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Binomial' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Binomial.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\ChiSquared' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/ChiSquared.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\DistributionValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/DistributionValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Exponential' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Exponential.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\F' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/F.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Fisher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Fisher.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Gamma' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Gamma.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\GammaBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/GammaBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\HyperGeometric' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/HyperGeometric.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\LogNormal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/LogNormal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\NewtonRaphson' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/NewtonRaphson.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Normal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Normal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Poisson' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Poisson.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\StandardNormal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/StandardNormal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\StudentT' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/StudentT.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Weibull' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Weibull.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\MaxMinBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/MaxMinBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Maximum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Maximum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Minimum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Minimum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Percentiles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Percentiles.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Permutations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Permutations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Size' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Size.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\StandardDeviations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/StandardDeviations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Standardize' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Standardize.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\StatisticalValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/StatisticalValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Trends' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Trends.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\VarianceBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/VarianceBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Variances' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Variances.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\CaseConvert' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/CaseConvert.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\CharacterConvert' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/CharacterConvert.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Concatenate' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Concatenate.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Extract' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Extract.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Format' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Format.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Replace' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Replace.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Search' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Search.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Text' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Text.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Trim' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Trim.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Token\\Stack' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Token/Stack.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Web' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Web.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Web\\Service' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Web/Service.php',
    'PhpOffice\\PhpSpreadsheet\\CellReferenceHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/CellReferenceHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\AddressHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/AddressHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\AddressRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/AddressRange.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\AdvancedValueBinder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/AdvancedValueBinder.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\Cell' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/Cell.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\CellAddress' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/CellAddress.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\CellRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/CellRange.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\ColumnRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/ColumnRange.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\Coordinate' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/Coordinate.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\DataType' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/DataType.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\DataValidation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/DataValidation.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\DataValidator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/DataValidator.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\DefaultValueBinder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/DefaultValueBinder.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\Hyperlink' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/Hyperlink.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\IValueBinder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/IValueBinder.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\IgnoredErrors' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/IgnoredErrors.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\RowRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/RowRange.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\StringValueBinder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/StringValueBinder.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Axis' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Axis.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\AxisText' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/AxisText.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Chart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Chart.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\ChartColor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/ChartColor.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\DataSeries' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/DataSeries.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\DataSeriesValues' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/DataSeriesValues.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\GridLines' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/GridLines.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Layout' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Layout.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Legend' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Legend.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\PlotArea' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/PlotArea.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Renderer\\IRenderer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Renderer/IRenderer.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Renderer\\JpGraph' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Renderer/JpGraph.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Renderer\\JpGraphRendererBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Renderer/JpGraphRendererBase.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Renderer\\MtJpGraphRenderer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Renderer/MtJpGraphRenderer.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Title' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Title.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\TrendLine' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/TrendLine.php',
    'PhpOffice\\PhpSpreadsheet\\Collection\\Cells' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Collection/Cells.php',
    'PhpOffice\\PhpSpreadsheet\\Collection\\CellsFactory' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Collection/CellsFactory.php',
    'PhpOffice\\PhpSpreadsheet\\Collection\\Memory\\SimpleCache1' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Collection/Memory/SimpleCache1.php',
    'PhpOffice\\PhpSpreadsheet\\Collection\\Memory\\SimpleCache3' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Collection/Memory/SimpleCache3.php',
    'PhpOffice\\PhpSpreadsheet\\Comment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Comment.php',
    'PhpOffice\\PhpSpreadsheet\\DefinedName' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/DefinedName.php',
    'PhpOffice\\PhpSpreadsheet\\Document\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Document/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Document\\Security' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Document/Security.php',
    'PhpOffice\\PhpSpreadsheet\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\HashTable' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/HashTable.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Dimension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Dimension.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Downloader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Downloader.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Handler' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Handler.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Html' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Html.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Sample' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Sample.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Size' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Size.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\TextGrid' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/TextGrid.php',
    'PhpOffice\\PhpSpreadsheet\\IComparable' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/IComparable.php',
    'PhpOffice\\PhpSpreadsheet\\IOFactory' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/IOFactory.php',
    'PhpOffice\\PhpSpreadsheet\\NamedFormula' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/NamedFormula.php',
    'PhpOffice\\PhpSpreadsheet\\NamedRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/NamedRange.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\BaseReader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/BaseReader.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Csv' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Csv.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Csv\\Delimiter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Csv/Delimiter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\DefaultReadFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/DefaultReadFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Gnumeric' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Gnumeric.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Gnumeric\\PageSetup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Gnumeric/PageSetup.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Gnumeric\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Gnumeric/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Gnumeric\\Styles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Gnumeric/Styles.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Html' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Html.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\IReadFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/IReadFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\IReader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/IReader.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\AutoFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/AutoFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\BaseLoader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/BaseLoader.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\DefinedNames' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/DefinedNames.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\FormulaTranslator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/FormulaTranslator.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\PageSettings' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/PageSettings.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Security\\XmlScanner' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Security/XmlScanner.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Slk' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Slk.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Color' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Color.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Color\\BIFF5' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Color/BIFF5.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Color\\BIFF8' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Color/BIFF8.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Color\\BuiltIn' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Color/BuiltIn.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\ConditionalFormatting' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/ConditionalFormatting.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\DataValidationHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/DataValidationHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\ErrorCode' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/ErrorCode.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Escher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Escher.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\MD5' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/MD5.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\RC4' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/RC4.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Style\\Border' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Style/Border.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Style\\CellAlignment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Style/CellAlignment.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Style\\CellFont' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Style/CellFont.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Style\\FillPattern' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Style/FillPattern.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\AutoFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/AutoFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\BaseParserClass' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/BaseParserClass.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Chart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Chart.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\ColumnAndRowAttributes' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/ColumnAndRowAttributes.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\ConditionalStyles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/ConditionalStyles.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\DataValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/DataValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Hyperlinks' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Hyperlinks.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Namespaces' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Namespaces.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\PageSetup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/PageSetup.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\SharedFormula' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/SharedFormula.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\SheetViewOptions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/SheetViewOptions.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\SheetViews' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/SheetViews.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Styles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Styles.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\TableReader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/TableReader.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Theme' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Theme.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\WorkbookView' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/WorkbookView.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\DataValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/DataValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\PageSettings' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/PageSettings.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\Alignment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/Alignment.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\Border' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/Border.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\Fill' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/Fill.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\Font' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/Font.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\NumberFormat' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/NumberFormat.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\StyleBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/StyleBase.php',
    'PhpOffice\\PhpSpreadsheet\\ReferenceHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/ReferenceHelper.php',
    'PhpOffice\\PhpSpreadsheet\\RichText\\ITextElement' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/RichText/ITextElement.php',
    'PhpOffice\\PhpSpreadsheet\\RichText\\RichText' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/RichText/RichText.php',
    'PhpOffice\\PhpSpreadsheet\\RichText\\Run' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/RichText/Run.php',
    'PhpOffice\\PhpSpreadsheet\\RichText\\TextElement' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/RichText/TextElement.php',
    'PhpOffice\\PhpSpreadsheet\\Settings' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Settings.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\CodePage' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/CodePage.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Date' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Date.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Drawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Drawing.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DgContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DgContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DgContainer\\SpgrContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DgContainer/SpgrContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DgContainer\\SpgrContainer\\SpContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DgContainer/SpgrContainer/SpContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DggContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DggContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DggContainer\\BstoreContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DggContainer/BstoreContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DggContainer\\BstoreContainer\\BSE' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DggContainer/BstoreContainer/BSE.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DggContainer\\BstoreContainer\\BSE\\Blip' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DggContainer/BstoreContainer/BSE/Blip.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\File' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/File.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Font' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Font.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\IntOrFloat' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/IntOrFloat.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLERead' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLERead.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE\\ChainedBlockStream' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE/ChainedBlockStream.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE\\PPS' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE/PPS.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE\\PPS\\File' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE/PPS/File.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE\\PPS\\Root' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE/PPS/Root.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\PasswordHasher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/PasswordHasher.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\StringHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/StringHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\TimeZone' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/TimeZone.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\BestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/BestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\ExponentialBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/ExponentialBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\LinearBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/LinearBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\LogarithmicBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/LogarithmicBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\PolynomialBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/PolynomialBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\PowerBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/PowerBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\Trend' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/Trend.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\XMLWriter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/XMLWriter.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Xls' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Xls.php',
    'PhpOffice\\PhpSpreadsheet\\Spreadsheet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Spreadsheet.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Alignment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Alignment.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Border' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Border.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Borders' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Borders.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Color' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Color.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Conditional' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Conditional.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\CellMatcher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/CellMatcher.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\CellStyleAssessor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/CellStyleAssessor.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalDataBar' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalDataBar.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalDataBarExtension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalDataBarExtension.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalFormatValueObject' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalFormatValueObject.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalFormattingRuleExtension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalFormattingRuleExtension.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\StyleMerger' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/StyleMerger.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\Blanks' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/Blanks.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\CellValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/CellValue.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\DateValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/DateValue.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\Duplicates' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/Duplicates.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\Errors' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/Errors.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\Expression' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/Expression.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\TextValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/TextValue.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\WizardAbstract' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/WizardAbstract.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\WizardInterface' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/WizardInterface.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Fill' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Fill.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Font' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Font.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\BaseFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/BaseFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\DateFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/DateFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Formatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Formatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\FractionFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/FractionFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\NumberFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/NumberFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\PercentageFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/PercentageFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Accounting' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Accounting.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Currency' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Currency.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Date' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Date.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\DateTime' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/DateTime.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\DateTimeWizard' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/DateTimeWizard.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Duration' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Duration.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Locale' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Locale.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Number' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Number.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\NumberBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/NumberBase.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Percentage' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Percentage.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Scientific' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Scientific.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Time' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Time.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Wizard' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Wizard.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Protection' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Protection.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\RgbTint' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/RgbTint.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Style' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Style.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Supervisor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Supervisor.php',
    'PhpOffice\\PhpSpreadsheet\\Theme' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Theme.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\AutoFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/AutoFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\AutoFilter\\Column' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/AutoFilter/Column.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\AutoFilter\\Column\\Rule' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/AutoFilter/Column/Rule.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\AutoFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/AutoFit.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\BaseDrawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/BaseDrawing.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\CellIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/CellIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Column' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Column.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\ColumnCellIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/ColumnCellIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\ColumnDimension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/ColumnDimension.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\ColumnIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/ColumnIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Dimension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Dimension.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Drawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Drawing.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Drawing\\Shadow' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Drawing/Shadow.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\HeaderFooter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/HeaderFooter.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\HeaderFooterDrawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/HeaderFooterDrawing.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Iterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Iterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\MemoryDrawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/MemoryDrawing.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\PageBreak' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/PageBreak.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\PageMargins' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/PageMargins.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\PageSetup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/PageSetup.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Protection' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Protection.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Row' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Row.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\RowCellIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/RowCellIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\RowDimension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/RowDimension.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\RowIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/RowIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\SheetView' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/SheetView.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Table' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Table.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Table\\Column' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Table/Column.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Table\\TableStyle' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Table/TableStyle.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Validations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Validations.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Worksheet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Worksheet.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\BaseWriter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/BaseWriter.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Csv' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Csv.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Html' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Html.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\IWriter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/IWriter.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\AutoFilters' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/AutoFilters.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Cell\\Comment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Cell/Comment.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Cell\\Style' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Cell/Style.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Content' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Content.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Formula' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Formula.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Meta' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Meta.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\MetaInf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/MetaInf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Mimetype' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Mimetype.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\NamedExpressions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/NamedExpressions.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Settings' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Settings.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Styles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Styles.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Thumbnails' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Thumbnails.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\WriterPart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/WriterPart.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Pdf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Pdf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Pdf\\Dompdf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Pdf/Dompdf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Pdf\\Mpdf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Pdf/Mpdf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Pdf\\Tcpdf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Pdf/Tcpdf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\BIFFwriter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/BIFFwriter.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\CellDataValidation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/CellDataValidation.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\ConditionalHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/ConditionalHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\ErrorCode' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/ErrorCode.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Escher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Escher.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Font' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Font.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Parser' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Parser.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Style\\CellAlignment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Style/CellAlignment.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Style\\CellBorder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Style/CellBorder.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Style\\CellFill' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Style/CellFill.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Style\\ColorMap' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Style/ColorMap.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Workbook' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Workbook.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Worksheet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Worksheet.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Xf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Xf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\AutoFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/AutoFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Chart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Chart.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Comments' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Comments.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\ContentTypes' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/ContentTypes.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\DefinedNames' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/DefinedNames.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\DocProps' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/DocProps.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Drawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Drawing.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\FunctionPrefix' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/FunctionPrefix.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Rels' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Rels.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\RelsRibbon' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/RelsRibbon.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\RelsVBA' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/RelsVBA.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\StringTable' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/StringTable.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Style' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Style.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Table' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Table.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Theme' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Theme.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Workbook' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Workbook.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Worksheet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Worksheet.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\WriterPart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/WriterPart.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\ZipStream0' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/ZipStream0.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\ZipStream2' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/ZipStream2.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\ZipStream3' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/ZipStream3.php',
    'PhpOption\\LazyOption' => $vendorDir . '/phpoption/phpoption/src/PhpOption/LazyOption.php',
    'PhpOption\\None' => $vendorDir . '/phpoption/phpoption/src/PhpOption/None.php',
    'PhpOption\\Option' => $vendorDir . '/phpoption/phpoption/src/PhpOption/Option.php',
    'PhpOption\\Some' => $vendorDir . '/phpoption/phpoption/src/PhpOption/Some.php',
    'PhpParser\\Builder' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder.php',
    'PhpParser\\BuilderFactory' => $vendorDir . '/nikic/php-parser/lib/PhpParser/BuilderFactory.php',
    'PhpParser\\BuilderHelpers' => $vendorDir . '/nikic/php-parser/lib/PhpParser/BuilderHelpers.php',
    'PhpParser\\Builder\\ClassConst' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/ClassConst.php',
    'PhpParser\\Builder\\Class_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Class_.php',
    'PhpParser\\Builder\\Declaration' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Declaration.php',
    'PhpParser\\Builder\\EnumCase' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/EnumCase.php',
    'PhpParser\\Builder\\Enum_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Enum_.php',
    'PhpParser\\Builder\\FunctionLike' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/FunctionLike.php',
    'PhpParser\\Builder\\Function_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Function_.php',
    'PhpParser\\Builder\\Interface_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Interface_.php',
    'PhpParser\\Builder\\Method' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Method.php',
    'PhpParser\\Builder\\Namespace_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Namespace_.php',
    'PhpParser\\Builder\\Param' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Param.php',
    'PhpParser\\Builder\\Property' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Property.php',
    'PhpParser\\Builder\\TraitUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/TraitUse.php',
    'PhpParser\\Builder\\TraitUseAdaptation' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/TraitUseAdaptation.php',
    'PhpParser\\Builder\\Trait_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Trait_.php',
    'PhpParser\\Builder\\Use_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Builder/Use_.php',
    'PhpParser\\Comment' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Comment.php',
    'PhpParser\\Comment\\Doc' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Comment/Doc.php',
    'PhpParser\\ConstExprEvaluationException' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ConstExprEvaluationException.php',
    'PhpParser\\ConstExprEvaluator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ConstExprEvaluator.php',
    'PhpParser\\Error' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Error.php',
    'PhpParser\\ErrorHandler' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ErrorHandler.php',
    'PhpParser\\ErrorHandler\\Collecting' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ErrorHandler/Collecting.php',
    'PhpParser\\ErrorHandler\\Throwing' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ErrorHandler/Throwing.php',
    'PhpParser\\Internal\\DiffElem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/DiffElem.php',
    'PhpParser\\Internal\\Differ' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/Differ.php',
    'PhpParser\\Internal\\PrintableNewAnonClassNode' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/PrintableNewAnonClassNode.php',
    'PhpParser\\Internal\\TokenPolyfill' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/TokenPolyfill.php',
    'PhpParser\\Internal\\TokenStream' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Internal/TokenStream.php',
    'PhpParser\\JsonDecoder' => $vendorDir . '/nikic/php-parser/lib/PhpParser/JsonDecoder.php',
    'PhpParser\\Lexer' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer.php',
    'PhpParser\\Lexer\\Emulative' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/Emulative.php',
    'PhpParser\\Lexer\\TokenEmulator\\AsymmetricVisibilityTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/AsymmetricVisibilityTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\AttributeEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/AttributeEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\EnumTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/EnumTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\ExplicitOctalEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/ExplicitOctalEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\KeywordEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/KeywordEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\MatchTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/MatchTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\NullsafeTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/NullsafeTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\PropertyTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/PropertyTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\ReadonlyFunctionTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/ReadonlyFunctionTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\ReadonlyTokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/ReadonlyTokenEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\ReverseEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/ReverseEmulator.php',
    'PhpParser\\Lexer\\TokenEmulator\\TokenEmulator' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Lexer/TokenEmulator/TokenEmulator.php',
    'PhpParser\\Modifiers' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Modifiers.php',
    'PhpParser\\NameContext' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NameContext.php',
    'PhpParser\\Node' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node.php',
    'PhpParser\\NodeAbstract' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeAbstract.php',
    'PhpParser\\NodeDumper' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeDumper.php',
    'PhpParser\\NodeFinder' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeFinder.php',
    'PhpParser\\NodeTraverser' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeTraverser.php',
    'PhpParser\\NodeTraverserInterface' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeTraverserInterface.php',
    'PhpParser\\NodeVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor.php',
    'PhpParser\\NodeVisitorAbstract' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitorAbstract.php',
    'PhpParser\\NodeVisitor\\CloningVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/CloningVisitor.php',
    'PhpParser\\NodeVisitor\\CommentAnnotatingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/CommentAnnotatingVisitor.php',
    'PhpParser\\NodeVisitor\\FindingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/FindingVisitor.php',
    'PhpParser\\NodeVisitor\\FirstFindingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/FirstFindingVisitor.php',
    'PhpParser\\NodeVisitor\\NameResolver' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/NameResolver.php',
    'PhpParser\\NodeVisitor\\NodeConnectingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/NodeConnectingVisitor.php',
    'PhpParser\\NodeVisitor\\ParentConnectingVisitor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/NodeVisitor/ParentConnectingVisitor.php',
    'PhpParser\\Node\\Arg' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Arg.php',
    'PhpParser\\Node\\ArrayItem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/ArrayItem.php',
    'PhpParser\\Node\\Attribute' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Attribute.php',
    'PhpParser\\Node\\AttributeGroup' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/AttributeGroup.php',
    'PhpParser\\Node\\ClosureUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/ClosureUse.php',
    'PhpParser\\Node\\ComplexType' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/ComplexType.php',
    'PhpParser\\Node\\Const_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Const_.php',
    'PhpParser\\Node\\DeclareItem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/DeclareItem.php',
    'PhpParser\\Node\\Expr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr.php',
    'PhpParser\\Node\\Expr\\ArrayDimFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ArrayDimFetch.php',
    'PhpParser\\Node\\Expr\\Array_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Array_.php',
    'PhpParser\\Node\\Expr\\ArrowFunction' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ArrowFunction.php',
    'PhpParser\\Node\\Expr\\Assign' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Assign.php',
    'PhpParser\\Node\\Expr\\AssignOp' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp.php',
    'PhpParser\\Node\\Expr\\AssignOp\\BitwiseAnd' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/BitwiseAnd.php',
    'PhpParser\\Node\\Expr\\AssignOp\\BitwiseOr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/BitwiseOr.php',
    'PhpParser\\Node\\Expr\\AssignOp\\BitwiseXor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/BitwiseXor.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Coalesce' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Coalesce.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Concat' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Concat.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Div' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Div.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Minus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Minus.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Mod' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Mod.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Mul' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Mul.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Plus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Plus.php',
    'PhpParser\\Node\\Expr\\AssignOp\\Pow' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/Pow.php',
    'PhpParser\\Node\\Expr\\AssignOp\\ShiftLeft' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/ShiftLeft.php',
    'PhpParser\\Node\\Expr\\AssignOp\\ShiftRight' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignOp/ShiftRight.php',
    'PhpParser\\Node\\Expr\\AssignRef' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/AssignRef.php',
    'PhpParser\\Node\\Expr\\BinaryOp' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BitwiseAnd' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BitwiseAnd.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BitwiseOr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BitwiseOr.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BitwiseXor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BitwiseXor.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BooleanAnd' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BooleanAnd.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\BooleanOr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/BooleanOr.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Coalesce' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Coalesce.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Concat' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Concat.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Div' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Div.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Equal' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Equal.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Greater' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Greater.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\GreaterOrEqual' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/GreaterOrEqual.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Identical' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Identical.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\LogicalAnd' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/LogicalAnd.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\LogicalOr' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/LogicalOr.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\LogicalXor' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/LogicalXor.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Minus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Minus.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Mod' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Mod.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Mul' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Mul.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\NotEqual' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/NotEqual.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\NotIdentical' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/NotIdentical.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Plus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Plus.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Pow' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Pow.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\ShiftLeft' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/ShiftLeft.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\ShiftRight' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/ShiftRight.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Smaller' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Smaller.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\SmallerOrEqual' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/SmallerOrEqual.php',
    'PhpParser\\Node\\Expr\\BinaryOp\\Spaceship' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BinaryOp/Spaceship.php',
    'PhpParser\\Node\\Expr\\BitwiseNot' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BitwiseNot.php',
    'PhpParser\\Node\\Expr\\BooleanNot' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/BooleanNot.php',
    'PhpParser\\Node\\Expr\\CallLike' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/CallLike.php',
    'PhpParser\\Node\\Expr\\Cast' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast.php',
    'PhpParser\\Node\\Expr\\Cast\\Array_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Array_.php',
    'PhpParser\\Node\\Expr\\Cast\\Bool_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Bool_.php',
    'PhpParser\\Node\\Expr\\Cast\\Double' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Double.php',
    'PhpParser\\Node\\Expr\\Cast\\Int_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Int_.php',
    'PhpParser\\Node\\Expr\\Cast\\Object_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Object_.php',
    'PhpParser\\Node\\Expr\\Cast\\String_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/String_.php',
    'PhpParser\\Node\\Expr\\Cast\\Unset_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Cast/Unset_.php',
    'PhpParser\\Node\\Expr\\ClassConstFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ClassConstFetch.php',
    'PhpParser\\Node\\Expr\\Clone_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Clone_.php',
    'PhpParser\\Node\\Expr\\Closure' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Closure.php',
    'PhpParser\\Node\\Expr\\ConstFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ConstFetch.php',
    'PhpParser\\Node\\Expr\\Empty_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Empty_.php',
    'PhpParser\\Node\\Expr\\Error' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Error.php',
    'PhpParser\\Node\\Expr\\ErrorSuppress' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ErrorSuppress.php',
    'PhpParser\\Node\\Expr\\Eval_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Eval_.php',
    'PhpParser\\Node\\Expr\\Exit_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Exit_.php',
    'PhpParser\\Node\\Expr\\FuncCall' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/FuncCall.php',
    'PhpParser\\Node\\Expr\\Include_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Include_.php',
    'PhpParser\\Node\\Expr\\Instanceof_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Instanceof_.php',
    'PhpParser\\Node\\Expr\\Isset_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Isset_.php',
    'PhpParser\\Node\\Expr\\List_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/List_.php',
    'PhpParser\\Node\\Expr\\Match_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Match_.php',
    'PhpParser\\Node\\Expr\\MethodCall' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/MethodCall.php',
    'PhpParser\\Node\\Expr\\New_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/New_.php',
    'PhpParser\\Node\\Expr\\NullsafeMethodCall' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/NullsafeMethodCall.php',
    'PhpParser\\Node\\Expr\\NullsafePropertyFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/NullsafePropertyFetch.php',
    'PhpParser\\Node\\Expr\\PostDec' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PostDec.php',
    'PhpParser\\Node\\Expr\\PostInc' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PostInc.php',
    'PhpParser\\Node\\Expr\\PreDec' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PreDec.php',
    'PhpParser\\Node\\Expr\\PreInc' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PreInc.php',
    'PhpParser\\Node\\Expr\\Print_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Print_.php',
    'PhpParser\\Node\\Expr\\PropertyFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/PropertyFetch.php',
    'PhpParser\\Node\\Expr\\ShellExec' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/ShellExec.php',
    'PhpParser\\Node\\Expr\\StaticCall' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/StaticCall.php',
    'PhpParser\\Node\\Expr\\StaticPropertyFetch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/StaticPropertyFetch.php',
    'PhpParser\\Node\\Expr\\Ternary' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Ternary.php',
    'PhpParser\\Node\\Expr\\Throw_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Throw_.php',
    'PhpParser\\Node\\Expr\\UnaryMinus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/UnaryMinus.php',
    'PhpParser\\Node\\Expr\\UnaryPlus' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/UnaryPlus.php',
    'PhpParser\\Node\\Expr\\Variable' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Variable.php',
    'PhpParser\\Node\\Expr\\YieldFrom' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/YieldFrom.php',
    'PhpParser\\Node\\Expr\\Yield_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Expr/Yield_.php',
    'PhpParser\\Node\\FunctionLike' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/FunctionLike.php',
    'PhpParser\\Node\\Identifier' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Identifier.php',
    'PhpParser\\Node\\InterpolatedStringPart' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/InterpolatedStringPart.php',
    'PhpParser\\Node\\IntersectionType' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/IntersectionType.php',
    'PhpParser\\Node\\MatchArm' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/MatchArm.php',
    'PhpParser\\Node\\Name' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Name.php',
    'PhpParser\\Node\\Name\\FullyQualified' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Name/FullyQualified.php',
    'PhpParser\\Node\\Name\\Relative' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Name/Relative.php',
    'PhpParser\\Node\\NullableType' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/NullableType.php',
    'PhpParser\\Node\\Param' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Param.php',
    'PhpParser\\Node\\PropertyHook' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/PropertyHook.php',
    'PhpParser\\Node\\PropertyItem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/PropertyItem.php',
    'PhpParser\\Node\\Scalar' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar.php',
    'PhpParser\\Node\\Scalar\\Float_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/Float_.php',
    'PhpParser\\Node\\Scalar\\Int_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/Int_.php',
    'PhpParser\\Node\\Scalar\\InterpolatedString' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/InterpolatedString.php',
    'PhpParser\\Node\\Scalar\\MagicConst' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Class_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Class_.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Dir' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Dir.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\File' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/File.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Function_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Function_.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Line' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Line.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Method' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Method.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Namespace_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Namespace_.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Property' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Property.php',
    'PhpParser\\Node\\Scalar\\MagicConst\\Trait_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/MagicConst/Trait_.php',
    'PhpParser\\Node\\Scalar\\String_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Scalar/String_.php',
    'PhpParser\\Node\\StaticVar' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/StaticVar.php',
    'PhpParser\\Node\\Stmt' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt.php',
    'PhpParser\\Node\\Stmt\\Block' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Block.php',
    'PhpParser\\Node\\Stmt\\Break_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Break_.php',
    'PhpParser\\Node\\Stmt\\Case_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Case_.php',
    'PhpParser\\Node\\Stmt\\Catch_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Catch_.php',
    'PhpParser\\Node\\Stmt\\ClassConst' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/ClassConst.php',
    'PhpParser\\Node\\Stmt\\ClassLike' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/ClassLike.php',
    'PhpParser\\Node\\Stmt\\ClassMethod' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/ClassMethod.php',
    'PhpParser\\Node\\Stmt\\Class_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Class_.php',
    'PhpParser\\Node\\Stmt\\Const_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Const_.php',
    'PhpParser\\Node\\Stmt\\Continue_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Continue_.php',
    'PhpParser\\Node\\Stmt\\Declare_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Declare_.php',
    'PhpParser\\Node\\Stmt\\Do_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Do_.php',
    'PhpParser\\Node\\Stmt\\Echo_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Echo_.php',
    'PhpParser\\Node\\Stmt\\ElseIf_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/ElseIf_.php',
    'PhpParser\\Node\\Stmt\\Else_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Else_.php',
    'PhpParser\\Node\\Stmt\\EnumCase' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/EnumCase.php',
    'PhpParser\\Node\\Stmt\\Enum_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Enum_.php',
    'PhpParser\\Node\\Stmt\\Expression' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Expression.php',
    'PhpParser\\Node\\Stmt\\Finally_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Finally_.php',
    'PhpParser\\Node\\Stmt\\For_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/For_.php',
    'PhpParser\\Node\\Stmt\\Foreach_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Foreach_.php',
    'PhpParser\\Node\\Stmt\\Function_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Function_.php',
    'PhpParser\\Node\\Stmt\\Global_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Global_.php',
    'PhpParser\\Node\\Stmt\\Goto_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Goto_.php',
    'PhpParser\\Node\\Stmt\\GroupUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/GroupUse.php',
    'PhpParser\\Node\\Stmt\\HaltCompiler' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/HaltCompiler.php',
    'PhpParser\\Node\\Stmt\\If_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/If_.php',
    'PhpParser\\Node\\Stmt\\InlineHTML' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/InlineHTML.php',
    'PhpParser\\Node\\Stmt\\Interface_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Interface_.php',
    'PhpParser\\Node\\Stmt\\Label' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Label.php',
    'PhpParser\\Node\\Stmt\\Namespace_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Namespace_.php',
    'PhpParser\\Node\\Stmt\\Nop' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Nop.php',
    'PhpParser\\Node\\Stmt\\Property' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Property.php',
    'PhpParser\\Node\\Stmt\\Return_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Return_.php',
    'PhpParser\\Node\\Stmt\\Static_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Static_.php',
    'PhpParser\\Node\\Stmt\\Switch_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Switch_.php',
    'PhpParser\\Node\\Stmt\\TraitUse' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TraitUse.php',
    'PhpParser\\Node\\Stmt\\TraitUseAdaptation' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TraitUseAdaptation.php',
    'PhpParser\\Node\\Stmt\\TraitUseAdaptation\\Alias' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TraitUseAdaptation/Alias.php',
    'PhpParser\\Node\\Stmt\\TraitUseAdaptation\\Precedence' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TraitUseAdaptation/Precedence.php',
    'PhpParser\\Node\\Stmt\\Trait_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Trait_.php',
    'PhpParser\\Node\\Stmt\\TryCatch' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/TryCatch.php',
    'PhpParser\\Node\\Stmt\\Unset_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Unset_.php',
    'PhpParser\\Node\\Stmt\\Use_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/Use_.php',
    'PhpParser\\Node\\Stmt\\While_' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/Stmt/While_.php',
    'PhpParser\\Node\\UnionType' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/UnionType.php',
    'PhpParser\\Node\\UseItem' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/UseItem.php',
    'PhpParser\\Node\\VarLikeIdentifier' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/VarLikeIdentifier.php',
    'PhpParser\\Node\\VariadicPlaceholder' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Node/VariadicPlaceholder.php',
    'PhpParser\\Parser' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Parser.php',
    'PhpParser\\ParserAbstract' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ParserAbstract.php',
    'PhpParser\\ParserFactory' => $vendorDir . '/nikic/php-parser/lib/PhpParser/ParserFactory.php',
    'PhpParser\\Parser\\Php7' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Parser/Php7.php',
    'PhpParser\\Parser\\Php8' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Parser/Php8.php',
    'PhpParser\\PhpVersion' => $vendorDir . '/nikic/php-parser/lib/PhpParser/PhpVersion.php',
    'PhpParser\\PrettyPrinter' => $vendorDir . '/nikic/php-parser/lib/PhpParser/PrettyPrinter.php',
    'PhpParser\\PrettyPrinterAbstract' => $vendorDir . '/nikic/php-parser/lib/PhpParser/PrettyPrinterAbstract.php',
    'PhpParser\\PrettyPrinter\\Standard' => $vendorDir . '/nikic/php-parser/lib/PhpParser/PrettyPrinter/Standard.php',
    'PhpParser\\Token' => $vendorDir . '/nikic/php-parser/lib/PhpParser/Token.php',
    'PhpToken' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/PhpToken.php',
    'Psr\\Cache\\CacheException' => $vendorDir . '/psr/cache/src/CacheException.php',
    'Psr\\Cache\\CacheItemInterface' => $vendorDir . '/psr/cache/src/CacheItemInterface.php',
    'Psr\\Cache\\CacheItemPoolInterface' => $vendorDir . '/psr/cache/src/CacheItemPoolInterface.php',
    'Psr\\Cache\\InvalidArgumentException' => $vendorDir . '/psr/cache/src/InvalidArgumentException.php',
    'Psr\\Clock\\ClockInterface' => $vendorDir . '/psr/clock/src/ClockInterface.php',
    'Psr\\Container\\ContainerExceptionInterface' => $vendorDir . '/psr/container/src/ContainerExceptionInterface.php',
    'Psr\\Container\\ContainerInterface' => $vendorDir . '/psr/container/src/ContainerInterface.php',
    'Psr\\Container\\NotFoundExceptionInterface' => $vendorDir . '/psr/container/src/NotFoundExceptionInterface.php',
    'Psr\\EventDispatcher\\EventDispatcherInterface' => $vendorDir . '/psr/event-dispatcher/src/EventDispatcherInterface.php',
    'Psr\\EventDispatcher\\ListenerProviderInterface' => $vendorDir . '/psr/event-dispatcher/src/ListenerProviderInterface.php',
    'Psr\\EventDispatcher\\StoppableEventInterface' => $vendorDir . '/psr/event-dispatcher/src/StoppableEventInterface.php',
    'Psr\\Http\\Client\\ClientExceptionInterface' => $vendorDir . '/psr/http-client/src/ClientExceptionInterface.php',
    'Psr\\Http\\Client\\ClientInterface' => $vendorDir . '/psr/http-client/src/ClientInterface.php',
    'Psr\\Http\\Client\\NetworkExceptionInterface' => $vendorDir . '/psr/http-client/src/NetworkExceptionInterface.php',
    'Psr\\Http\\Client\\RequestExceptionInterface' => $vendorDir . '/psr/http-client/src/RequestExceptionInterface.php',
    'Psr\\Http\\Message\\MessageInterface' => $vendorDir . '/psr/http-message/src/MessageInterface.php',
    'Psr\\Http\\Message\\RequestFactoryInterface' => $vendorDir . '/psr/http-factory/src/RequestFactoryInterface.php',
    'Psr\\Http\\Message\\RequestInterface' => $vendorDir . '/psr/http-message/src/RequestInterface.php',
    'Psr\\Http\\Message\\ResponseFactoryInterface' => $vendorDir . '/psr/http-factory/src/ResponseFactoryInterface.php',
    'Psr\\Http\\Message\\ResponseInterface' => $vendorDir . '/psr/http-message/src/ResponseInterface.php',
    'Psr\\Http\\Message\\ServerRequestFactoryInterface' => $vendorDir . '/psr/http-factory/src/ServerRequestFactoryInterface.php',
    'Psr\\Http\\Message\\ServerRequestInterface' => $vendorDir . '/psr/http-message/src/ServerRequestInterface.php',
    'Psr\\Http\\Message\\StreamFactoryInterface' => $vendorDir . '/psr/http-factory/src/StreamFactoryInterface.php',
    'Psr\\Http\\Message\\StreamInterface' => $vendorDir . '/psr/http-message/src/StreamInterface.php',
    'Psr\\Http\\Message\\UploadedFileFactoryInterface' => $vendorDir . '/psr/http-factory/src/UploadedFileFactoryInterface.php',
    'Psr\\Http\\Message\\UploadedFileInterface' => $vendorDir . '/psr/http-message/src/UploadedFileInterface.php',
    'Psr\\Http\\Message\\UriFactoryInterface' => $vendorDir . '/psr/http-factory/src/UriFactoryInterface.php',
    'Psr\\Http\\Message\\UriInterface' => $vendorDir . '/psr/http-message/src/UriInterface.php',
    'Psr\\Log\\AbstractLogger' => $vendorDir . '/psr/log/src/AbstractLogger.php',
    'Psr\\Log\\InvalidArgumentException' => $vendorDir . '/psr/log/src/InvalidArgumentException.php',
    'Psr\\Log\\LogLevel' => $vendorDir . '/psr/log/src/LogLevel.php',
    'Psr\\Log\\LoggerAwareInterface' => $vendorDir . '/psr/log/src/LoggerAwareInterface.php',
    'Psr\\Log\\LoggerAwareTrait' => $vendorDir . '/psr/log/src/LoggerAwareTrait.php',
    'Psr\\Log\\LoggerInterface' => $vendorDir . '/psr/log/src/LoggerInterface.php',
    'Psr\\Log\\LoggerTrait' => $vendorDir . '/psr/log/src/LoggerTrait.php',
    'Psr\\Log\\NullLogger' => $vendorDir . '/psr/log/src/NullLogger.php',
    'Psr\\SimpleCache\\CacheException' => $vendorDir . '/psr/simple-cache/src/CacheException.php',
    'Psr\\SimpleCache\\CacheInterface' => $vendorDir . '/psr/simple-cache/src/CacheInterface.php',
    'Psr\\SimpleCache\\InvalidArgumentException' => $vendorDir . '/psr/simple-cache/src/InvalidArgumentException.php',
    'Psy\\CodeCleaner' => $vendorDir . '/psy/psysh/src/CodeCleaner.php',
    'Psy\\CodeCleaner\\AbstractClassPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/AbstractClassPass.php',
    'Psy\\CodeCleaner\\AssignThisVariablePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/AssignThisVariablePass.php',
    'Psy\\CodeCleaner\\CallTimePassByReferencePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/CallTimePassByReferencePass.php',
    'Psy\\CodeCleaner\\CalledClassPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/CalledClassPass.php',
    'Psy\\CodeCleaner\\CodeCleanerPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/CodeCleanerPass.php',
    'Psy\\CodeCleaner\\EmptyArrayDimFetchPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/EmptyArrayDimFetchPass.php',
    'Psy\\CodeCleaner\\ExitPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/ExitPass.php',
    'Psy\\CodeCleaner\\FinalClassPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/FinalClassPass.php',
    'Psy\\CodeCleaner\\FunctionContextPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/FunctionContextPass.php',
    'Psy\\CodeCleaner\\FunctionReturnInWriteContextPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/FunctionReturnInWriteContextPass.php',
    'Psy\\CodeCleaner\\ImplicitReturnPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/ImplicitReturnPass.php',
    'Psy\\CodeCleaner\\IssetPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/IssetPass.php',
    'Psy\\CodeCleaner\\LabelContextPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/LabelContextPass.php',
    'Psy\\CodeCleaner\\LeavePsyshAlonePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/LeavePsyshAlonePass.php',
    'Psy\\CodeCleaner\\ListPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/ListPass.php',
    'Psy\\CodeCleaner\\LoopContextPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/LoopContextPass.php',
    'Psy\\CodeCleaner\\MagicConstantsPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/MagicConstantsPass.php',
    'Psy\\CodeCleaner\\NamespaceAwarePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/NamespaceAwarePass.php',
    'Psy\\CodeCleaner\\NamespacePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/NamespacePass.php',
    'Psy\\CodeCleaner\\NoReturnValue' => $vendorDir . '/psy/psysh/src/CodeCleaner/NoReturnValue.php',
    'Psy\\CodeCleaner\\PassableByReferencePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/PassableByReferencePass.php',
    'Psy\\CodeCleaner\\RequirePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/RequirePass.php',
    'Psy\\CodeCleaner\\ReturnTypePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/ReturnTypePass.php',
    'Psy\\CodeCleaner\\StrictTypesPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/StrictTypesPass.php',
    'Psy\\CodeCleaner\\UseStatementPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/UseStatementPass.php',
    'Psy\\CodeCleaner\\ValidClassNamePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/ValidClassNamePass.php',
    'Psy\\CodeCleaner\\ValidConstructorPass' => $vendorDir . '/psy/psysh/src/CodeCleaner/ValidConstructorPass.php',
    'Psy\\CodeCleaner\\ValidFunctionNamePass' => $vendorDir . '/psy/psysh/src/CodeCleaner/ValidFunctionNamePass.php',
    'Psy\\Command\\BufferCommand' => $vendorDir . '/psy/psysh/src/Command/BufferCommand.php',
    'Psy\\Command\\ClearCommand' => $vendorDir . '/psy/psysh/src/Command/ClearCommand.php',
    'Psy\\Command\\CodeArgumentParser' => $vendorDir . '/psy/psysh/src/Command/CodeArgumentParser.php',
    'Psy\\Command\\Command' => $vendorDir . '/psy/psysh/src/Command/Command.php',
    'Psy\\Command\\DocCommand' => $vendorDir . '/psy/psysh/src/Command/DocCommand.php',
    'Psy\\Command\\DumpCommand' => $vendorDir . '/psy/psysh/src/Command/DumpCommand.php',
    'Psy\\Command\\EditCommand' => $vendorDir . '/psy/psysh/src/Command/EditCommand.php',
    'Psy\\Command\\ExitCommand' => $vendorDir . '/psy/psysh/src/Command/ExitCommand.php',
    'Psy\\Command\\HelpCommand' => $vendorDir . '/psy/psysh/src/Command/HelpCommand.php',
    'Psy\\Command\\HistoryCommand' => $vendorDir . '/psy/psysh/src/Command/HistoryCommand.php',
    'Psy\\Command\\ListCommand' => $vendorDir . '/psy/psysh/src/Command/ListCommand.php',
    'Psy\\Command\\ListCommand\\ClassConstantEnumerator' => $vendorDir . '/psy/psysh/src/Command/ListCommand/ClassConstantEnumerator.php',
    'Psy\\Command\\ListCommand\\ClassEnumerator' => $vendorDir . '/psy/psysh/src/Command/ListCommand/ClassEnumerator.php',
    'Psy\\Command\\ListCommand\\ConstantEnumerator' => $vendorDir . '/psy/psysh/src/Command/ListCommand/ConstantEnumerator.php',
    'Psy\\Command\\ListCommand\\Enumerator' => $vendorDir . '/psy/psysh/src/Command/ListCommand/Enumerator.php',
    'Psy\\Command\\ListCommand\\FunctionEnumerator' => $vendorDir . '/psy/psysh/src/Command/ListCommand/FunctionEnumerator.php',
    'Psy\\Command\\ListCommand\\GlobalVariableEnumerator' => $vendorDir . '/psy/psysh/src/Command/ListCommand/GlobalVariableEnumerator.php',
    'Psy\\Command\\ListCommand\\MethodEnumerator' => $vendorDir . '/psy/psysh/src/Command/ListCommand/MethodEnumerator.php',
    'Psy\\Command\\ListCommand\\PropertyEnumerator' => $vendorDir . '/psy/psysh/src/Command/ListCommand/PropertyEnumerator.php',
    'Psy\\Command\\ListCommand\\VariableEnumerator' => $vendorDir . '/psy/psysh/src/Command/ListCommand/VariableEnumerator.php',
    'Psy\\Command\\ParseCommand' => $vendorDir . '/psy/psysh/src/Command/ParseCommand.php',
    'Psy\\Command\\PsyVersionCommand' => $vendorDir . '/psy/psysh/src/Command/PsyVersionCommand.php',
    'Psy\\Command\\ReflectingCommand' => $vendorDir . '/psy/psysh/src/Command/ReflectingCommand.php',
    'Psy\\Command\\ShowCommand' => $vendorDir . '/psy/psysh/src/Command/ShowCommand.php',
    'Psy\\Command\\SudoCommand' => $vendorDir . '/psy/psysh/src/Command/SudoCommand.php',
    'Psy\\Command\\ThrowUpCommand' => $vendorDir . '/psy/psysh/src/Command/ThrowUpCommand.php',
    'Psy\\Command\\TimeitCommand' => $vendorDir . '/psy/psysh/src/Command/TimeitCommand.php',
    'Psy\\Command\\TimeitCommand\\TimeitVisitor' => $vendorDir . '/psy/psysh/src/Command/TimeitCommand/TimeitVisitor.php',
    'Psy\\Command\\TraceCommand' => $vendorDir . '/psy/psysh/src/Command/TraceCommand.php',
    'Psy\\Command\\WhereamiCommand' => $vendorDir . '/psy/psysh/src/Command/WhereamiCommand.php',
    'Psy\\Command\\WtfCommand' => $vendorDir . '/psy/psysh/src/Command/WtfCommand.php',
    'Psy\\ConfigPaths' => $vendorDir . '/psy/psysh/src/ConfigPaths.php',
    'Psy\\Configuration' => $vendorDir . '/psy/psysh/src/Configuration.php',
    'Psy\\Context' => $vendorDir . '/psy/psysh/src/Context.php',
    'Psy\\ContextAware' => $vendorDir . '/psy/psysh/src/ContextAware.php',
    'Psy\\EnvInterface' => $vendorDir . '/psy/psysh/src/EnvInterface.php',
    'Psy\\Exception\\BreakException' => $vendorDir . '/psy/psysh/src/Exception/BreakException.php',
    'Psy\\Exception\\DeprecatedException' => $vendorDir . '/psy/psysh/src/Exception/DeprecatedException.php',
    'Psy\\Exception\\ErrorException' => $vendorDir . '/psy/psysh/src/Exception/ErrorException.php',
    'Psy\\Exception\\Exception' => $vendorDir . '/psy/psysh/src/Exception/Exception.php',
    'Psy\\Exception\\FatalErrorException' => $vendorDir . '/psy/psysh/src/Exception/FatalErrorException.php',
    'Psy\\Exception\\ParseErrorException' => $vendorDir . '/psy/psysh/src/Exception/ParseErrorException.php',
    'Psy\\Exception\\RuntimeException' => $vendorDir . '/psy/psysh/src/Exception/RuntimeException.php',
    'Psy\\Exception\\ThrowUpException' => $vendorDir . '/psy/psysh/src/Exception/ThrowUpException.php',
    'Psy\\Exception\\UnexpectedTargetException' => $vendorDir . '/psy/psysh/src/Exception/UnexpectedTargetException.php',
    'Psy\\ExecutionClosure' => $vendorDir . '/psy/psysh/src/ExecutionClosure.php',
    'Psy\\ExecutionLoopClosure' => $vendorDir . '/psy/psysh/src/ExecutionLoopClosure.php',
    'Psy\\ExecutionLoop\\AbstractListener' => $vendorDir . '/psy/psysh/src/ExecutionLoop/AbstractListener.php',
    'Psy\\ExecutionLoop\\Listener' => $vendorDir . '/psy/psysh/src/ExecutionLoop/Listener.php',
    'Psy\\ExecutionLoop\\ProcessForker' => $vendorDir . '/psy/psysh/src/ExecutionLoop/ProcessForker.php',
    'Psy\\ExecutionLoop\\RunkitReloader' => $vendorDir . '/psy/psysh/src/ExecutionLoop/RunkitReloader.php',
    'Psy\\Formatter\\CodeFormatter' => $vendorDir . '/psy/psysh/src/Formatter/CodeFormatter.php',
    'Psy\\Formatter\\DocblockFormatter' => $vendorDir . '/psy/psysh/src/Formatter/DocblockFormatter.php',
    'Psy\\Formatter\\ReflectorFormatter' => $vendorDir . '/psy/psysh/src/Formatter/ReflectorFormatter.php',
    'Psy\\Formatter\\SignatureFormatter' => $vendorDir . '/psy/psysh/src/Formatter/SignatureFormatter.php',
    'Psy\\Formatter\\TraceFormatter' => $vendorDir . '/psy/psysh/src/Formatter/TraceFormatter.php',
    'Psy\\Input\\CodeArgument' => $vendorDir . '/psy/psysh/src/Input/CodeArgument.php',
    'Psy\\Input\\FilterOptions' => $vendorDir . '/psy/psysh/src/Input/FilterOptions.php',
    'Psy\\Input\\ShellInput' => $vendorDir . '/psy/psysh/src/Input/ShellInput.php',
    'Psy\\Input\\SilentInput' => $vendorDir . '/psy/psysh/src/Input/SilentInput.php',
    'Psy\\Output\\OutputPager' => $vendorDir . '/psy/psysh/src/Output/OutputPager.php',
    'Psy\\Output\\PassthruPager' => $vendorDir . '/psy/psysh/src/Output/PassthruPager.php',
    'Psy\\Output\\ProcOutputPager' => $vendorDir . '/psy/psysh/src/Output/ProcOutputPager.php',
    'Psy\\Output\\ShellOutput' => $vendorDir . '/psy/psysh/src/Output/ShellOutput.php',
    'Psy\\Output\\Theme' => $vendorDir . '/psy/psysh/src/Output/Theme.php',
    'Psy\\ParserFactory' => $vendorDir . '/psy/psysh/src/ParserFactory.php',
    'Psy\\Readline\\GNUReadline' => $vendorDir . '/psy/psysh/src/Readline/GNUReadline.php',
    'Psy\\Readline\\Hoa\\Autocompleter' => $vendorDir . '/psy/psysh/src/Readline/Hoa/Autocompleter.php',
    'Psy\\Readline\\Hoa\\AutocompleterAggregate' => $vendorDir . '/psy/psysh/src/Readline/Hoa/AutocompleterAggregate.php',
    'Psy\\Readline\\Hoa\\AutocompleterPath' => $vendorDir . '/psy/psysh/src/Readline/Hoa/AutocompleterPath.php',
    'Psy\\Readline\\Hoa\\AutocompleterWord' => $vendorDir . '/psy/psysh/src/Readline/Hoa/AutocompleterWord.php',
    'Psy\\Readline\\Hoa\\Console' => $vendorDir . '/psy/psysh/src/Readline/Hoa/Console.php',
    'Psy\\Readline\\Hoa\\ConsoleCursor' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ConsoleCursor.php',
    'Psy\\Readline\\Hoa\\ConsoleException' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ConsoleException.php',
    'Psy\\Readline\\Hoa\\ConsoleInput' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ConsoleInput.php',
    'Psy\\Readline\\Hoa\\ConsoleOutput' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ConsoleOutput.php',
    'Psy\\Readline\\Hoa\\ConsoleProcessus' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ConsoleProcessus.php',
    'Psy\\Readline\\Hoa\\ConsoleTput' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ConsoleTput.php',
    'Psy\\Readline\\Hoa\\ConsoleWindow' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ConsoleWindow.php',
    'Psy\\Readline\\Hoa\\Event' => $vendorDir . '/psy/psysh/src/Readline/Hoa/Event.php',
    'Psy\\Readline\\Hoa\\EventBucket' => $vendorDir . '/psy/psysh/src/Readline/Hoa/EventBucket.php',
    'Psy\\Readline\\Hoa\\EventException' => $vendorDir . '/psy/psysh/src/Readline/Hoa/EventException.php',
    'Psy\\Readline\\Hoa\\EventListenable' => $vendorDir . '/psy/psysh/src/Readline/Hoa/EventListenable.php',
    'Psy\\Readline\\Hoa\\EventListener' => $vendorDir . '/psy/psysh/src/Readline/Hoa/EventListener.php',
    'Psy\\Readline\\Hoa\\EventListens' => $vendorDir . '/psy/psysh/src/Readline/Hoa/EventListens.php',
    'Psy\\Readline\\Hoa\\EventSource' => $vendorDir . '/psy/psysh/src/Readline/Hoa/EventSource.php',
    'Psy\\Readline\\Hoa\\Exception' => $vendorDir . '/psy/psysh/src/Readline/Hoa/Exception.php',
    'Psy\\Readline\\Hoa\\ExceptionIdle' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ExceptionIdle.php',
    'Psy\\Readline\\Hoa\\File' => $vendorDir . '/psy/psysh/src/Readline/Hoa/File.php',
    'Psy\\Readline\\Hoa\\FileDirectory' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileDirectory.php',
    'Psy\\Readline\\Hoa\\FileDoesNotExistException' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileDoesNotExistException.php',
    'Psy\\Readline\\Hoa\\FileException' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileException.php',
    'Psy\\Readline\\Hoa\\FileFinder' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileFinder.php',
    'Psy\\Readline\\Hoa\\FileGeneric' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileGeneric.php',
    'Psy\\Readline\\Hoa\\FileLink' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileLink.php',
    'Psy\\Readline\\Hoa\\FileLinkRead' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileLinkRead.php',
    'Psy\\Readline\\Hoa\\FileLinkReadWrite' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileLinkReadWrite.php',
    'Psy\\Readline\\Hoa\\FileRead' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileRead.php',
    'Psy\\Readline\\Hoa\\FileReadWrite' => $vendorDir . '/psy/psysh/src/Readline/Hoa/FileReadWrite.php',
    'Psy\\Readline\\Hoa\\IStream' => $vendorDir . '/psy/psysh/src/Readline/Hoa/IStream.php',
    'Psy\\Readline\\Hoa\\IteratorFileSystem' => $vendorDir . '/psy/psysh/src/Readline/Hoa/IteratorFileSystem.php',
    'Psy\\Readline\\Hoa\\IteratorRecursiveDirectory' => $vendorDir . '/psy/psysh/src/Readline/Hoa/IteratorRecursiveDirectory.php',
    'Psy\\Readline\\Hoa\\IteratorSplFileInfo' => $vendorDir . '/psy/psysh/src/Readline/Hoa/IteratorSplFileInfo.php',
    'Psy\\Readline\\Hoa\\Protocol' => $vendorDir . '/psy/psysh/src/Readline/Hoa/Protocol.php',
    'Psy\\Readline\\Hoa\\ProtocolException' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ProtocolException.php',
    'Psy\\Readline\\Hoa\\ProtocolNode' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ProtocolNode.php',
    'Psy\\Readline\\Hoa\\ProtocolNodeLibrary' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ProtocolNodeLibrary.php',
    'Psy\\Readline\\Hoa\\ProtocolWrapper' => $vendorDir . '/psy/psysh/src/Readline/Hoa/ProtocolWrapper.php',
    'Psy\\Readline\\Hoa\\Readline' => $vendorDir . '/psy/psysh/src/Readline/Hoa/Readline.php',
    'Psy\\Readline\\Hoa\\Stream' => $vendorDir . '/psy/psysh/src/Readline/Hoa/Stream.php',
    'Psy\\Readline\\Hoa\\StreamBufferable' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamBufferable.php',
    'Psy\\Readline\\Hoa\\StreamContext' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamContext.php',
    'Psy\\Readline\\Hoa\\StreamException' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamException.php',
    'Psy\\Readline\\Hoa\\StreamIn' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamIn.php',
    'Psy\\Readline\\Hoa\\StreamLockable' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamLockable.php',
    'Psy\\Readline\\Hoa\\StreamOut' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamOut.php',
    'Psy\\Readline\\Hoa\\StreamPathable' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamPathable.php',
    'Psy\\Readline\\Hoa\\StreamPointable' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamPointable.php',
    'Psy\\Readline\\Hoa\\StreamStatable' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamStatable.php',
    'Psy\\Readline\\Hoa\\StreamTouchable' => $vendorDir . '/psy/psysh/src/Readline/Hoa/StreamTouchable.php',
    'Psy\\Readline\\Hoa\\Ustring' => $vendorDir . '/psy/psysh/src/Readline/Hoa/Ustring.php',
    'Psy\\Readline\\Hoa\\Xcallable' => $vendorDir . '/psy/psysh/src/Readline/Hoa/Xcallable.php',
    'Psy\\Readline\\Libedit' => $vendorDir . '/psy/psysh/src/Readline/Libedit.php',
    'Psy\\Readline\\Readline' => $vendorDir . '/psy/psysh/src/Readline/Readline.php',
    'Psy\\Readline\\Transient' => $vendorDir . '/psy/psysh/src/Readline/Transient.php',
    'Psy\\Readline\\Userland' => $vendorDir . '/psy/psysh/src/Readline/Userland.php',
    'Psy\\Reflection\\ReflectionConstant' => $vendorDir . '/psy/psysh/src/Reflection/ReflectionConstant.php',
    'Psy\\Reflection\\ReflectionLanguageConstruct' => $vendorDir . '/psy/psysh/src/Reflection/ReflectionLanguageConstruct.php',
    'Psy\\Reflection\\ReflectionLanguageConstructParameter' => $vendorDir . '/psy/psysh/src/Reflection/ReflectionLanguageConstructParameter.php',
    'Psy\\Reflection\\ReflectionNamespace' => $vendorDir . '/psy/psysh/src/Reflection/ReflectionNamespace.php',
    'Psy\\Shell' => $vendorDir . '/psy/psysh/src/Shell.php',
    'Psy\\Sudo' => $vendorDir . '/psy/psysh/src/Sudo.php',
    'Psy\\Sudo\\SudoVisitor' => $vendorDir . '/psy/psysh/src/Sudo/SudoVisitor.php',
    'Psy\\SuperglobalsEnv' => $vendorDir . '/psy/psysh/src/SuperglobalsEnv.php',
    'Psy\\SystemEnv' => $vendorDir . '/psy/psysh/src/SystemEnv.php',
    'Psy\\TabCompletion\\AutoCompleter' => $vendorDir . '/psy/psysh/src/TabCompletion/AutoCompleter.php',
    'Psy\\TabCompletion\\Matcher\\AbstractContextAwareMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/AbstractContextAwareMatcher.php',
    'Psy\\TabCompletion\\Matcher\\AbstractDefaultParametersMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/AbstractDefaultParametersMatcher.php',
    'Psy\\TabCompletion\\Matcher\\AbstractMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/AbstractMatcher.php',
    'Psy\\TabCompletion\\Matcher\\ClassAttributesMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/ClassAttributesMatcher.php',
    'Psy\\TabCompletion\\Matcher\\ClassMethodDefaultParametersMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/ClassMethodDefaultParametersMatcher.php',
    'Psy\\TabCompletion\\Matcher\\ClassMethodsMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/ClassMethodsMatcher.php',
    'Psy\\TabCompletion\\Matcher\\ClassNamesMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/ClassNamesMatcher.php',
    'Psy\\TabCompletion\\Matcher\\CommandsMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/CommandsMatcher.php',
    'Psy\\TabCompletion\\Matcher\\ConstantsMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/ConstantsMatcher.php',
    'Psy\\TabCompletion\\Matcher\\FunctionDefaultParametersMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/FunctionDefaultParametersMatcher.php',
    'Psy\\TabCompletion\\Matcher\\FunctionsMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/FunctionsMatcher.php',
    'Psy\\TabCompletion\\Matcher\\KeywordsMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/KeywordsMatcher.php',
    'Psy\\TabCompletion\\Matcher\\MongoClientMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/MongoClientMatcher.php',
    'Psy\\TabCompletion\\Matcher\\MongoDatabaseMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/MongoDatabaseMatcher.php',
    'Psy\\TabCompletion\\Matcher\\ObjectAttributesMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/ObjectAttributesMatcher.php',
    'Psy\\TabCompletion\\Matcher\\ObjectMethodDefaultParametersMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/ObjectMethodDefaultParametersMatcher.php',
    'Psy\\TabCompletion\\Matcher\\ObjectMethodsMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/ObjectMethodsMatcher.php',
    'Psy\\TabCompletion\\Matcher\\VariablesMatcher' => $vendorDir . '/psy/psysh/src/TabCompletion/Matcher/VariablesMatcher.php',
    'Psy\\Util\\Docblock' => $vendorDir . '/psy/psysh/src/Util/Docblock.php',
    'Psy\\Util\\Json' => $vendorDir . '/psy/psysh/src/Util/Json.php',
    'Psy\\Util\\Mirror' => $vendorDir . '/psy/psysh/src/Util/Mirror.php',
    'Psy\\Util\\Str' => $vendorDir . '/psy/psysh/src/Util/Str.php',
    'Psy\\VarDumper\\Cloner' => $vendorDir . '/psy/psysh/src/VarDumper/Cloner.php',
    'Psy\\VarDumper\\Dumper' => $vendorDir . '/psy/psysh/src/VarDumper/Dumper.php',
    'Psy\\VarDumper\\Presenter' => $vendorDir . '/psy/psysh/src/VarDumper/Presenter.php',
    'Psy\\VarDumper\\PresenterAware' => $vendorDir . '/psy/psysh/src/VarDumper/PresenterAware.php',
    'Psy\\VersionUpdater\\Checker' => $vendorDir . '/psy/psysh/src/VersionUpdater/Checker.php',
    'Psy\\VersionUpdater\\Downloader' => $vendorDir . '/psy/psysh/src/VersionUpdater/Downloader.php',
    'Psy\\VersionUpdater\\Downloader\\CurlDownloader' => $vendorDir . '/psy/psysh/src/VersionUpdater/Downloader/CurlDownloader.php',
    'Psy\\VersionUpdater\\Downloader\\Factory' => $vendorDir . '/psy/psysh/src/VersionUpdater/Downloader/Factory.php',
    'Psy\\VersionUpdater\\Downloader\\FileDownloader' => $vendorDir . '/psy/psysh/src/VersionUpdater/Downloader/FileDownloader.php',
    'Psy\\VersionUpdater\\GitHubChecker' => $vendorDir . '/psy/psysh/src/VersionUpdater/GitHubChecker.php',
    'Psy\\VersionUpdater\\Installer' => $vendorDir . '/psy/psysh/src/VersionUpdater/Installer.php',
    'Psy\\VersionUpdater\\IntervalChecker' => $vendorDir . '/psy/psysh/src/VersionUpdater/IntervalChecker.php',
    'Psy\\VersionUpdater\\NoopChecker' => $vendorDir . '/psy/psysh/src/VersionUpdater/NoopChecker.php',
    'Psy\\VersionUpdater\\SelfUpdate' => $vendorDir . '/psy/psysh/src/VersionUpdater/SelfUpdate.php',
    'Ramsey\\Collection\\AbstractArray' => $vendorDir . '/ramsey/collection/src/AbstractArray.php',
    'Ramsey\\Collection\\AbstractCollection' => $vendorDir . '/ramsey/collection/src/AbstractCollection.php',
    'Ramsey\\Collection\\AbstractSet' => $vendorDir . '/ramsey/collection/src/AbstractSet.php',
    'Ramsey\\Collection\\ArrayInterface' => $vendorDir . '/ramsey/collection/src/ArrayInterface.php',
    'Ramsey\\Collection\\Collection' => $vendorDir . '/ramsey/collection/src/Collection.php',
    'Ramsey\\Collection\\CollectionInterface' => $vendorDir . '/ramsey/collection/src/CollectionInterface.php',
    'Ramsey\\Collection\\DoubleEndedQueue' => $vendorDir . '/ramsey/collection/src/DoubleEndedQueue.php',
    'Ramsey\\Collection\\DoubleEndedQueueInterface' => $vendorDir . '/ramsey/collection/src/DoubleEndedQueueInterface.php',
    'Ramsey\\Collection\\Exception\\CollectionException' => $vendorDir . '/ramsey/collection/src/Exception/CollectionException.php',
    'Ramsey\\Collection\\Exception\\CollectionMismatchException' => $vendorDir . '/ramsey/collection/src/Exception/CollectionMismatchException.php',
    'Ramsey\\Collection\\Exception\\InvalidArgumentException' => $vendorDir . '/ramsey/collection/src/Exception/InvalidArgumentException.php',
    'Ramsey\\Collection\\Exception\\InvalidPropertyOrMethod' => $vendorDir . '/ramsey/collection/src/Exception/InvalidPropertyOrMethod.php',
    'Ramsey\\Collection\\Exception\\NoSuchElementException' => $vendorDir . '/ramsey/collection/src/Exception/NoSuchElementException.php',
    'Ramsey\\Collection\\Exception\\OutOfBoundsException' => $vendorDir . '/ramsey/collection/src/Exception/OutOfBoundsException.php',
    'Ramsey\\Collection\\Exception\\UnsupportedOperationException' => $vendorDir . '/ramsey/collection/src/Exception/UnsupportedOperationException.php',
    'Ramsey\\Collection\\GenericArray' => $vendorDir . '/ramsey/collection/src/GenericArray.php',
    'Ramsey\\Collection\\Map\\AbstractMap' => $vendorDir . '/ramsey/collection/src/Map/AbstractMap.php',
    'Ramsey\\Collection\\Map\\AbstractTypedMap' => $vendorDir . '/ramsey/collection/src/Map/AbstractTypedMap.php',
    'Ramsey\\Collection\\Map\\AssociativeArrayMap' => $vendorDir . '/ramsey/collection/src/Map/AssociativeArrayMap.php',
    'Ramsey\\Collection\\Map\\MapInterface' => $vendorDir . '/ramsey/collection/src/Map/MapInterface.php',
    'Ramsey\\Collection\\Map\\NamedParameterMap' => $vendorDir . '/ramsey/collection/src/Map/NamedParameterMap.php',
    'Ramsey\\Collection\\Map\\TypedMap' => $vendorDir . '/ramsey/collection/src/Map/TypedMap.php',
    'Ramsey\\Collection\\Map\\TypedMapInterface' => $vendorDir . '/ramsey/collection/src/Map/TypedMapInterface.php',
    'Ramsey\\Collection\\Queue' => $vendorDir . '/ramsey/collection/src/Queue.php',
    'Ramsey\\Collection\\QueueInterface' => $vendorDir . '/ramsey/collection/src/QueueInterface.php',
    'Ramsey\\Collection\\Set' => $vendorDir . '/ramsey/collection/src/Set.php',
    'Ramsey\\Collection\\Sort' => $vendorDir . '/ramsey/collection/src/Sort.php',
    'Ramsey\\Collection\\Tool\\TypeTrait' => $vendorDir . '/ramsey/collection/src/Tool/TypeTrait.php',
    'Ramsey\\Collection\\Tool\\ValueExtractorTrait' => $vendorDir . '/ramsey/collection/src/Tool/ValueExtractorTrait.php',
    'Ramsey\\Collection\\Tool\\ValueToStringTrait' => $vendorDir . '/ramsey/collection/src/Tool/ValueToStringTrait.php',
    'Ramsey\\Uuid\\BinaryUtils' => $vendorDir . '/ramsey/uuid/src/BinaryUtils.php',
    'Ramsey\\Uuid\\Builder\\BuilderCollection' => $vendorDir . '/ramsey/uuid/src/Builder/BuilderCollection.php',
    'Ramsey\\Uuid\\Builder\\DefaultUuidBuilder' => $vendorDir . '/ramsey/uuid/src/Builder/DefaultUuidBuilder.php',
    'Ramsey\\Uuid\\Builder\\DegradedUuidBuilder' => $vendorDir . '/ramsey/uuid/src/Builder/DegradedUuidBuilder.php',
    'Ramsey\\Uuid\\Builder\\FallbackBuilder' => $vendorDir . '/ramsey/uuid/src/Builder/FallbackBuilder.php',
    'Ramsey\\Uuid\\Builder\\UuidBuilderInterface' => $vendorDir . '/ramsey/uuid/src/Builder/UuidBuilderInterface.php',
    'Ramsey\\Uuid\\Codec\\CodecInterface' => $vendorDir . '/ramsey/uuid/src/Codec/CodecInterface.php',
    'Ramsey\\Uuid\\Codec\\GuidStringCodec' => $vendorDir . '/ramsey/uuid/src/Codec/GuidStringCodec.php',
    'Ramsey\\Uuid\\Codec\\OrderedTimeCodec' => $vendorDir . '/ramsey/uuid/src/Codec/OrderedTimeCodec.php',
    'Ramsey\\Uuid\\Codec\\StringCodec' => $vendorDir . '/ramsey/uuid/src/Codec/StringCodec.php',
    'Ramsey\\Uuid\\Codec\\TimestampFirstCombCodec' => $vendorDir . '/ramsey/uuid/src/Codec/TimestampFirstCombCodec.php',
    'Ramsey\\Uuid\\Codec\\TimestampLastCombCodec' => $vendorDir . '/ramsey/uuid/src/Codec/TimestampLastCombCodec.php',
    'Ramsey\\Uuid\\Converter\\NumberConverterInterface' => $vendorDir . '/ramsey/uuid/src/Converter/NumberConverterInterface.php',
    'Ramsey\\Uuid\\Converter\\Number\\BigNumberConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Number/BigNumberConverter.php',
    'Ramsey\\Uuid\\Converter\\Number\\DegradedNumberConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Number/DegradedNumberConverter.php',
    'Ramsey\\Uuid\\Converter\\Number\\GenericNumberConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Number/GenericNumberConverter.php',
    'Ramsey\\Uuid\\Converter\\TimeConverterInterface' => $vendorDir . '/ramsey/uuid/src/Converter/TimeConverterInterface.php',
    'Ramsey\\Uuid\\Converter\\Time\\BigNumberTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/BigNumberTimeConverter.php',
    'Ramsey\\Uuid\\Converter\\Time\\DegradedTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/DegradedTimeConverter.php',
    'Ramsey\\Uuid\\Converter\\Time\\GenericTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/GenericTimeConverter.php',
    'Ramsey\\Uuid\\Converter\\Time\\PhpTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/PhpTimeConverter.php',
    'Ramsey\\Uuid\\Converter\\Time\\UnixTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/UnixTimeConverter.php',
    'Ramsey\\Uuid\\DegradedUuid' => $vendorDir . '/ramsey/uuid/src/DegradedUuid.php',
    'Ramsey\\Uuid\\DeprecatedUuidInterface' => $vendorDir . '/ramsey/uuid/src/DeprecatedUuidInterface.php',
    'Ramsey\\Uuid\\DeprecatedUuidMethodsTrait' => $vendorDir . '/ramsey/uuid/src/DeprecatedUuidMethodsTrait.php',
    'Ramsey\\Uuid\\Exception\\BuilderNotFoundException' => $vendorDir . '/ramsey/uuid/src/Exception/BuilderNotFoundException.php',
    'Ramsey\\Uuid\\Exception\\DateTimeException' => $vendorDir . '/ramsey/uuid/src/Exception/DateTimeException.php',
    'Ramsey\\Uuid\\Exception\\DceSecurityException' => $vendorDir . '/ramsey/uuid/src/Exception/DceSecurityException.php',
    'Ramsey\\Uuid\\Exception\\InvalidArgumentException' => $vendorDir . '/ramsey/uuid/src/Exception/InvalidArgumentException.php',
    'Ramsey\\Uuid\\Exception\\InvalidBytesException' => $vendorDir . '/ramsey/uuid/src/Exception/InvalidBytesException.php',
    'Ramsey\\Uuid\\Exception\\InvalidUuidStringException' => $vendorDir . '/ramsey/uuid/src/Exception/InvalidUuidStringException.php',
    'Ramsey\\Uuid\\Exception\\NameException' => $vendorDir . '/ramsey/uuid/src/Exception/NameException.php',
    'Ramsey\\Uuid\\Exception\\NodeException' => $vendorDir . '/ramsey/uuid/src/Exception/NodeException.php',
    'Ramsey\\Uuid\\Exception\\RandomSourceException' => $vendorDir . '/ramsey/uuid/src/Exception/RandomSourceException.php',
    'Ramsey\\Uuid\\Exception\\TimeSourceException' => $vendorDir . '/ramsey/uuid/src/Exception/TimeSourceException.php',
    'Ramsey\\Uuid\\Exception\\UnableToBuildUuidException' => $vendorDir . '/ramsey/uuid/src/Exception/UnableToBuildUuidException.php',
    'Ramsey\\Uuid\\Exception\\UnsupportedOperationException' => $vendorDir . '/ramsey/uuid/src/Exception/UnsupportedOperationException.php',
    'Ramsey\\Uuid\\Exception\\UuidExceptionInterface' => $vendorDir . '/ramsey/uuid/src/Exception/UuidExceptionInterface.php',
    'Ramsey\\Uuid\\FeatureSet' => $vendorDir . '/ramsey/uuid/src/FeatureSet.php',
    'Ramsey\\Uuid\\Fields\\FieldsInterface' => $vendorDir . '/ramsey/uuid/src/Fields/FieldsInterface.php',
    'Ramsey\\Uuid\\Fields\\SerializableFieldsTrait' => $vendorDir . '/ramsey/uuid/src/Fields/SerializableFieldsTrait.php',
    'Ramsey\\Uuid\\Generator\\CombGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/CombGenerator.php',
    'Ramsey\\Uuid\\Generator\\DceSecurityGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/DceSecurityGenerator.php',
    'Ramsey\\Uuid\\Generator\\DceSecurityGeneratorInterface' => $vendorDir . '/ramsey/uuid/src/Generator/DceSecurityGeneratorInterface.php',
    'Ramsey\\Uuid\\Generator\\DefaultNameGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/DefaultNameGenerator.php',
    'Ramsey\\Uuid\\Generator\\DefaultTimeGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/DefaultTimeGenerator.php',
    'Ramsey\\Uuid\\Generator\\NameGeneratorFactory' => $vendorDir . '/ramsey/uuid/src/Generator/NameGeneratorFactory.php',
    'Ramsey\\Uuid\\Generator\\NameGeneratorInterface' => $vendorDir . '/ramsey/uuid/src/Generator/NameGeneratorInterface.php',
    'Ramsey\\Uuid\\Generator\\PeclUuidNameGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/PeclUuidNameGenerator.php',
    'Ramsey\\Uuid\\Generator\\PeclUuidRandomGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/PeclUuidRandomGenerator.php',
    'Ramsey\\Uuid\\Generator\\PeclUuidTimeGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/PeclUuidTimeGenerator.php',
    'Ramsey\\Uuid\\Generator\\RandomBytesGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/RandomBytesGenerator.php',
    'Ramsey\\Uuid\\Generator\\RandomGeneratorFactory' => $vendorDir . '/ramsey/uuid/src/Generator/RandomGeneratorFactory.php',
    'Ramsey\\Uuid\\Generator\\RandomGeneratorInterface' => $vendorDir . '/ramsey/uuid/src/Generator/RandomGeneratorInterface.php',
    'Ramsey\\Uuid\\Generator\\RandomLibAdapter' => $vendorDir . '/ramsey/uuid/src/Generator/RandomLibAdapter.php',
    'Ramsey\\Uuid\\Generator\\TimeGeneratorFactory' => $vendorDir . '/ramsey/uuid/src/Generator/TimeGeneratorFactory.php',
    'Ramsey\\Uuid\\Generator\\TimeGeneratorInterface' => $vendorDir . '/ramsey/uuid/src/Generator/TimeGeneratorInterface.php',
    'Ramsey\\Uuid\\Generator\\UnixTimeGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/UnixTimeGenerator.php',
    'Ramsey\\Uuid\\Guid\\Fields' => $vendorDir . '/ramsey/uuid/src/Guid/Fields.php',
    'Ramsey\\Uuid\\Guid\\Guid' => $vendorDir . '/ramsey/uuid/src/Guid/Guid.php',
    'Ramsey\\Uuid\\Guid\\GuidBuilder' => $vendorDir . '/ramsey/uuid/src/Guid/GuidBuilder.php',
    'Ramsey\\Uuid\\Lazy\\LazyUuidFromString' => $vendorDir . '/ramsey/uuid/src/Lazy/LazyUuidFromString.php',
    'Ramsey\\Uuid\\Math\\BrickMathCalculator' => $vendorDir . '/ramsey/uuid/src/Math/BrickMathCalculator.php',
    'Ramsey\\Uuid\\Math\\CalculatorInterface' => $vendorDir . '/ramsey/uuid/src/Math/CalculatorInterface.php',
    'Ramsey\\Uuid\\Math\\RoundingMode' => $vendorDir . '/ramsey/uuid/src/Math/RoundingMode.php',
    'Ramsey\\Uuid\\Nonstandard\\Fields' => $vendorDir . '/ramsey/uuid/src/Nonstandard/Fields.php',
    'Ramsey\\Uuid\\Nonstandard\\Uuid' => $vendorDir . '/ramsey/uuid/src/Nonstandard/Uuid.php',
    'Ramsey\\Uuid\\Nonstandard\\UuidBuilder' => $vendorDir . '/ramsey/uuid/src/Nonstandard/UuidBuilder.php',
    'Ramsey\\Uuid\\Nonstandard\\UuidV6' => $vendorDir . '/ramsey/uuid/src/Nonstandard/UuidV6.php',
    'Ramsey\\Uuid\\Provider\\DceSecurityProviderInterface' => $vendorDir . '/ramsey/uuid/src/Provider/DceSecurityProviderInterface.php',
    'Ramsey\\Uuid\\Provider\\Dce\\SystemDceSecurityProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Dce/SystemDceSecurityProvider.php',
    'Ramsey\\Uuid\\Provider\\NodeProviderInterface' => $vendorDir . '/ramsey/uuid/src/Provider/NodeProviderInterface.php',
    'Ramsey\\Uuid\\Provider\\Node\\FallbackNodeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Node/FallbackNodeProvider.php',
    'Ramsey\\Uuid\\Provider\\Node\\NodeProviderCollection' => $vendorDir . '/ramsey/uuid/src/Provider/Node/NodeProviderCollection.php',
    'Ramsey\\Uuid\\Provider\\Node\\RandomNodeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Node/RandomNodeProvider.php',
    'Ramsey\\Uuid\\Provider\\Node\\StaticNodeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Node/StaticNodeProvider.php',
    'Ramsey\\Uuid\\Provider\\Node\\SystemNodeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Node/SystemNodeProvider.php',
    'Ramsey\\Uuid\\Provider\\TimeProviderInterface' => $vendorDir . '/ramsey/uuid/src/Provider/TimeProviderInterface.php',
    'Ramsey\\Uuid\\Provider\\Time\\FixedTimeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Time/FixedTimeProvider.php',
    'Ramsey\\Uuid\\Provider\\Time\\SystemTimeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Time/SystemTimeProvider.php',
    'Ramsey\\Uuid\\Rfc4122\\Fields' => $vendorDir . '/ramsey/uuid/src/Rfc4122/Fields.php',
    'Ramsey\\Uuid\\Rfc4122\\FieldsInterface' => $vendorDir . '/ramsey/uuid/src/Rfc4122/FieldsInterface.php',
    'Ramsey\\Uuid\\Rfc4122\\MaxTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/MaxTrait.php',
    'Ramsey\\Uuid\\Rfc4122\\MaxUuid' => $vendorDir . '/ramsey/uuid/src/Rfc4122/MaxUuid.php',
    'Ramsey\\Uuid\\Rfc4122\\NilTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/NilTrait.php',
    'Ramsey\\Uuid\\Rfc4122\\NilUuid' => $vendorDir . '/ramsey/uuid/src/Rfc4122/NilUuid.php',
    'Ramsey\\Uuid\\Rfc4122\\TimeTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/TimeTrait.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidBuilder' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidBuilder.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidInterface' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidInterface.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV1' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV1.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV2' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV2.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV3' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV3.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV4' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV4.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV5' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV5.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV6' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV6.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV7' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV7.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV8' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV8.php',
    'Ramsey\\Uuid\\Rfc4122\\Validator' => $vendorDir . '/ramsey/uuid/src/Rfc4122/Validator.php',
    'Ramsey\\Uuid\\Rfc4122\\VariantTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/VariantTrait.php',
    'Ramsey\\Uuid\\Rfc4122\\VersionTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/VersionTrait.php',
    'Ramsey\\Uuid\\Type\\Decimal' => $vendorDir . '/ramsey/uuid/src/Type/Decimal.php',
    'Ramsey\\Uuid\\Type\\Hexadecimal' => $vendorDir . '/ramsey/uuid/src/Type/Hexadecimal.php',
    'Ramsey\\Uuid\\Type\\Integer' => $vendorDir . '/ramsey/uuid/src/Type/Integer.php',
    'Ramsey\\Uuid\\Type\\NumberInterface' => $vendorDir . '/ramsey/uuid/src/Type/NumberInterface.php',
    'Ramsey\\Uuid\\Type\\Time' => $vendorDir . '/ramsey/uuid/src/Type/Time.php',
    'Ramsey\\Uuid\\Type\\TypeInterface' => $vendorDir . '/ramsey/uuid/src/Type/TypeInterface.php',
    'Ramsey\\Uuid\\Uuid' => $vendorDir . '/ramsey/uuid/src/Uuid.php',
    'Ramsey\\Uuid\\UuidFactory' => $vendorDir . '/ramsey/uuid/src/UuidFactory.php',
    'Ramsey\\Uuid\\UuidFactoryInterface' => $vendorDir . '/ramsey/uuid/src/UuidFactoryInterface.php',
    'Ramsey\\Uuid\\UuidInterface' => $vendorDir . '/ramsey/uuid/src/UuidInterface.php',
    'Ramsey\\Uuid\\Validator\\GenericValidator' => $vendorDir . '/ramsey/uuid/src/Validator/GenericValidator.php',
    'Ramsey\\Uuid\\Validator\\ValidatorInterface' => $vendorDir . '/ramsey/uuid/src/Validator/ValidatorInterface.php',
    'Razorpay\\Api\\Account' => $vendorDir . '/razorpay/razorpay/src/Account.php',
    'Razorpay\\Api\\Addon' => $vendorDir . '/razorpay/razorpay/src/Addon.php',
    'Razorpay\\Api\\Api' => $vendorDir . '/razorpay/razorpay/src/Api.php',
    'Razorpay\\Api\\ArrayableInterface' => $vendorDir . '/razorpay/razorpay/src/ArrayableInterface.php',
    'Razorpay\\Api\\Card' => $vendorDir . '/razorpay/razorpay/src/Card.php',
    'Razorpay\\Api\\Collection' => $vendorDir . '/razorpay/razorpay/src/Collection.php',
    'Razorpay\\Api\\Customer' => $vendorDir . '/razorpay/razorpay/src/Customer.php',
    'Razorpay\\Api\\Dispute' => $vendorDir . '/razorpay/razorpay/src/Dispute.php',
    'Razorpay\\Api\\Document' => $vendorDir . '/razorpay/razorpay/src/Document.php',
    'Razorpay\\Api\\Entity' => $vendorDir . '/razorpay/razorpay/src/Entity.php',
    'Razorpay\\Api\\Errors\\BadRequestError' => $vendorDir . '/razorpay/razorpay/src/Errors/BadRequestError.php',
    'Razorpay\\Api\\Errors\\Error' => $vendorDir . '/razorpay/razorpay/src/Errors/Error.php',
    'Razorpay\\Api\\Errors\\ErrorCode' => $vendorDir . '/razorpay/razorpay/src/Errors/ErrorCode.php',
    'Razorpay\\Api\\Errors\\GatewayError' => $vendorDir . '/razorpay/razorpay/src/Errors/GatewayError.php',
    'Razorpay\\Api\\Errors\\ServerError' => $vendorDir . '/razorpay/razorpay/src/Errors/ServerError.php',
    'Razorpay\\Api\\Errors\\SignatureVerificationError' => $vendorDir . '/razorpay/razorpay/src/Errors/SignatureVerificationError.php',
    'Razorpay\\Api\\FundAccount' => $vendorDir . '/razorpay/razorpay/src/FundAccount.php',
    'Razorpay\\Api\\Iin' => $vendorDir . '/razorpay/razorpay/src/Iin.php',
    'Razorpay\\Api\\Invoice' => $vendorDir . '/razorpay/razorpay/src/Invoice.php',
    'Razorpay\\Api\\Item' => $vendorDir . '/razorpay/razorpay/src/Item.php',
    'Razorpay\\Api\\Order' => $vendorDir . '/razorpay/razorpay/src/Order.php',
    'Razorpay\\Api\\Payment' => $vendorDir . '/razorpay/razorpay/src/Payment.php',
    'Razorpay\\Api\\PaymentLink' => $vendorDir . '/razorpay/razorpay/src/PaymentLink.php',
    'Razorpay\\Api\\PaymentPage' => $vendorDir . '/razorpay/razorpay/src/PaymentPage.php',
    'Razorpay\\Api\\Plan' => $vendorDir . '/razorpay/razorpay/src/Plan.php',
    'Razorpay\\Api\\Product' => $vendorDir . '/razorpay/razorpay/src/Product.php',
    'Razorpay\\Api\\QrCode' => $vendorDir . '/razorpay/razorpay/src/QrCode.php',
    'Razorpay\\Api\\Refund' => $vendorDir . '/razorpay/razorpay/src/Refund.php',
    'Razorpay\\Api\\Request' => $vendorDir . '/razorpay/razorpay/src/Request.php',
    'Razorpay\\Api\\Resource' => $vendorDir . '/razorpay/razorpay/src/Resource.php',
    'Razorpay\\Api\\Settlement' => $vendorDir . '/razorpay/razorpay/src/Settlement.php',
    'Razorpay\\Api\\Stakeholder' => $vendorDir . '/razorpay/razorpay/src/Stakeholder.php',
    'Razorpay\\Api\\Subscription' => $vendorDir . '/razorpay/razorpay/src/Subscription.php',
    'Razorpay\\Api\\Token' => $vendorDir . '/razorpay/razorpay/src/Token.php',
    'Razorpay\\Api\\Transfer' => $vendorDir . '/razorpay/razorpay/src/Transfer.php',
    'Razorpay\\Api\\Utility' => $vendorDir . '/razorpay/razorpay/src/Utility.php',
    'Razorpay\\Api\\VirtualAccount' => $vendorDir . '/razorpay/razorpay/src/VirtualAccount.php',
    'Razorpay\\Api\\Webhook' => $vendorDir . '/razorpay/razorpay/src/Webhook.php',
    'Razorpay\\Tests\\AddonTest' => $vendorDir . '/razorpay/razorpay/tests/AddonTest.php',
    'Razorpay\\Tests\\ApiTest' => $vendorDir . '/razorpay/razorpay/tests/ApiTest.php',
    'Razorpay\\Tests\\CardTest' => $vendorDir . '/razorpay/razorpay/tests/CardTest.php',
    'Razorpay\\Tests\\CoverageTest' => $vendorDir . '/razorpay/razorpay/tests/CoverageTest.php',
    'Razorpay\\Tests\\CustomerTest' => $vendorDir . '/razorpay/razorpay/tests/CustomerTest.php',
    'Razorpay\\Tests\\EmandateTest' => $vendorDir . '/razorpay/razorpay/tests/EmandateTest.php',
    'Razorpay\\Tests\\ExceptionTest' => $vendorDir . '/razorpay/razorpay/tests/ExceptionTest.php',
    'Razorpay\\Tests\\FundTest' => $vendorDir . '/razorpay/razorpay/tests/FundTest.php',
    'Razorpay\\Tests\\IinTest' => $vendorDir . '/razorpay/razorpay/tests/IinTest.php',
    'Razorpay\\Tests\\InvoiceTest' => $vendorDir . '/razorpay/razorpay/tests/InvoiceTest.php',
    'Razorpay\\Tests\\ItemTest' => $vendorDir . '/razorpay/razorpay/tests/ItemTest.php',
    'Razorpay\\Tests\\OrdersTest' => $vendorDir . '/razorpay/razorpay/tests/OrdersTest.php',
    'Razorpay\\Tests\\PaperNachTest' => $vendorDir . '/razorpay/razorpay/tests/PaperNachTest.php',
    'Razorpay\\Tests\\PartnerTest' => $vendorDir . '/razorpay/razorpay/tests/PartnerTest.php',
    'Razorpay\\Tests\\PaymentLinkTest' => $vendorDir . '/razorpay/razorpay/tests/PaymentLinkTest.php',
    'Razorpay\\Tests\\PaymentTest' => $vendorDir . '/razorpay/razorpay/tests/PaymentTest.php',
    'Razorpay\\Tests\\PlanTest' => $vendorDir . '/razorpay/razorpay/tests/PlanTest.php',
    'Razorpay\\Tests\\QrCodeTest' => $vendorDir . '/razorpay/razorpay/tests/QrCodeTest.php',
    'Razorpay\\Tests\\RefundTest' => $vendorDir . '/razorpay/razorpay/tests/RefundTest.php',
    'Razorpay\\Tests\\RegisterEmandateTest' => $vendorDir . '/razorpay/razorpay/tests/RegisterEmandateTest.php',
    'Razorpay\\Tests\\RegisterNachTest' => $vendorDir . '/razorpay/razorpay/tests/RegisterNachTest.php',
    'Razorpay\\Tests\\RequestTest' => $vendorDir . '/razorpay/razorpay/tests/RequestTest.php',
    'Razorpay\\Tests\\SettlementTest' => $vendorDir . '/razorpay/razorpay/tests/SettlementTest.php',
    'Razorpay\\Tests\\SignatureVerificationTest' => $vendorDir . '/razorpay/razorpay/tests/SignatureVerificationTest.php',
    'Razorpay\\Tests\\SubscriptionTest' => $vendorDir . '/razorpay/razorpay/tests/SubscriptionTest.php',
    'Razorpay\\Tests\\TestCase' => $vendorDir . '/razorpay/razorpay/tests/TestCase.php',
    'Razorpay\\Tests\\TokenTest' => $vendorDir . '/razorpay/razorpay/tests/TokenTest.php',
    'Razorpay\\Tests\\TransferTest' => $vendorDir . '/razorpay/razorpay/tests/TransferTest.php',
    'Razorpay\\Tests\\UpiTest' => $vendorDir . '/razorpay/razorpay/tests/UpiTest.php',
    'Razorpay\\Tests\\VirtualAccountTest' => $vendorDir . '/razorpay/razorpay/tests/VirtualAccountTest.php',
    'Requests' => $vendorDir . '/rmccue/requests/library/Requests.php',
    'SQLite3Exception' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/SQLite3Exception.php',
    'Sabberworm\\CSS\\CSSList\\AtRuleBlockList' => $vendorDir . '/sabberworm/php-css-parser/src/CSSList/AtRuleBlockList.php',
    'Sabberworm\\CSS\\CSSList\\CSSBlockList' => $vendorDir . '/sabberworm/php-css-parser/src/CSSList/CSSBlockList.php',
    'Sabberworm\\CSS\\CSSList\\CSSList' => $vendorDir . '/sabberworm/php-css-parser/src/CSSList/CSSList.php',
    'Sabberworm\\CSS\\CSSList\\Document' => $vendorDir . '/sabberworm/php-css-parser/src/CSSList/Document.php',
    'Sabberworm\\CSS\\CSSList\\KeyFrame' => $vendorDir . '/sabberworm/php-css-parser/src/CSSList/KeyFrame.php',
    'Sabberworm\\CSS\\Comment\\Comment' => $vendorDir . '/sabberworm/php-css-parser/src/Comment/Comment.php',
    'Sabberworm\\CSS\\Comment\\Commentable' => $vendorDir . '/sabberworm/php-css-parser/src/Comment/Commentable.php',
    'Sabberworm\\CSS\\OutputFormat' => $vendorDir . '/sabberworm/php-css-parser/src/OutputFormat.php',
    'Sabberworm\\CSS\\OutputFormatter' => $vendorDir . '/sabberworm/php-css-parser/src/OutputFormatter.php',
    'Sabberworm\\CSS\\Parser' => $vendorDir . '/sabberworm/php-css-parser/src/Parser.php',
    'Sabberworm\\CSS\\Parsing\\Anchor' => $vendorDir . '/sabberworm/php-css-parser/src/Parsing/Anchor.php',
    'Sabberworm\\CSS\\Parsing\\OutputException' => $vendorDir . '/sabberworm/php-css-parser/src/Parsing/OutputException.php',
    'Sabberworm\\CSS\\Parsing\\ParserState' => $vendorDir . '/sabberworm/php-css-parser/src/Parsing/ParserState.php',
    'Sabberworm\\CSS\\Parsing\\SourceException' => $vendorDir . '/sabberworm/php-css-parser/src/Parsing/SourceException.php',
    'Sabberworm\\CSS\\Parsing\\UnexpectedEOFException' => $vendorDir . '/sabberworm/php-css-parser/src/Parsing/UnexpectedEOFException.php',
    'Sabberworm\\CSS\\Parsing\\UnexpectedTokenException' => $vendorDir . '/sabberworm/php-css-parser/src/Parsing/UnexpectedTokenException.php',
    'Sabberworm\\CSS\\Property\\AtRule' => $vendorDir . '/sabberworm/php-css-parser/src/Property/AtRule.php',
    'Sabberworm\\CSS\\Property\\CSSNamespace' => $vendorDir . '/sabberworm/php-css-parser/src/Property/CSSNamespace.php',
    'Sabberworm\\CSS\\Property\\Charset' => $vendorDir . '/sabberworm/php-css-parser/src/Property/Charset.php',
    'Sabberworm\\CSS\\Property\\Import' => $vendorDir . '/sabberworm/php-css-parser/src/Property/Import.php',
    'Sabberworm\\CSS\\Property\\KeyframeSelector' => $vendorDir . '/sabberworm/php-css-parser/src/Property/KeyframeSelector.php',
    'Sabberworm\\CSS\\Property\\Selector' => $vendorDir . '/sabberworm/php-css-parser/src/Property/Selector.php',
    'Sabberworm\\CSS\\Renderable' => $vendorDir . '/sabberworm/php-css-parser/src/Renderable.php',
    'Sabberworm\\CSS\\RuleSet\\AtRuleSet' => $vendorDir . '/sabberworm/php-css-parser/src/RuleSet/AtRuleSet.php',
    'Sabberworm\\CSS\\RuleSet\\DeclarationBlock' => $vendorDir . '/sabberworm/php-css-parser/src/RuleSet/DeclarationBlock.php',
    'Sabberworm\\CSS\\RuleSet\\RuleSet' => $vendorDir . '/sabberworm/php-css-parser/src/RuleSet/RuleSet.php',
    'Sabberworm\\CSS\\Rule\\Rule' => $vendorDir . '/sabberworm/php-css-parser/src/Rule/Rule.php',
    'Sabberworm\\CSS\\Settings' => $vendorDir . '/sabberworm/php-css-parser/src/Settings.php',
    'Sabberworm\\CSS\\Value\\CSSFunction' => $vendorDir . '/sabberworm/php-css-parser/src/Value/CSSFunction.php',
    'Sabberworm\\CSS\\Value\\CSSString' => $vendorDir . '/sabberworm/php-css-parser/src/Value/CSSString.php',
    'Sabberworm\\CSS\\Value\\CalcFunction' => $vendorDir . '/sabberworm/php-css-parser/src/Value/CalcFunction.php',
    'Sabberworm\\CSS\\Value\\CalcRuleValueList' => $vendorDir . '/sabberworm/php-css-parser/src/Value/CalcRuleValueList.php',
    'Sabberworm\\CSS\\Value\\Color' => $vendorDir . '/sabberworm/php-css-parser/src/Value/Color.php',
    'Sabberworm\\CSS\\Value\\LineName' => $vendorDir . '/sabberworm/php-css-parser/src/Value/LineName.php',
    'Sabberworm\\CSS\\Value\\PrimitiveValue' => $vendorDir . '/sabberworm/php-css-parser/src/Value/PrimitiveValue.php',
    'Sabberworm\\CSS\\Value\\RuleValueList' => $vendorDir . '/sabberworm/php-css-parser/src/Value/RuleValueList.php',
    'Sabberworm\\CSS\\Value\\Size' => $vendorDir . '/sabberworm/php-css-parser/src/Value/Size.php',
    'Sabberworm\\CSS\\Value\\URL' => $vendorDir . '/sabberworm/php-css-parser/src/Value/URL.php',
    'Sabberworm\\CSS\\Value\\Value' => $vendorDir . '/sabberworm/php-css-parser/src/Value/Value.php',
    'Sabberworm\\CSS\\Value\\ValueList' => $vendorDir . '/sabberworm/php-css-parser/src/Value/ValueList.php',
    'SebastianBergmann\\CliParser\\AmbiguousOptionException' => $vendorDir . '/sebastian/cli-parser/src/exceptions/AmbiguousOptionException.php',
    'SebastianBergmann\\CliParser\\Exception' => $vendorDir . '/sebastian/cli-parser/src/exceptions/Exception.php',
    'SebastianBergmann\\CliParser\\OptionDoesNotAllowArgumentException' => $vendorDir . '/sebastian/cli-parser/src/exceptions/OptionDoesNotAllowArgumentException.php',
    'SebastianBergmann\\CliParser\\Parser' => $vendorDir . '/sebastian/cli-parser/src/Parser.php',
    'SebastianBergmann\\CliParser\\RequiredOptionArgumentMissingException' => $vendorDir . '/sebastian/cli-parser/src/exceptions/RequiredOptionArgumentMissingException.php',
    'SebastianBergmann\\CliParser\\UnknownOptionException' => $vendorDir . '/sebastian/cli-parser/src/exceptions/UnknownOptionException.php',
    'SebastianBergmann\\CodeCoverage\\BranchAndPathCoverageNotSupportedException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/BranchAndPathCoverageNotSupportedException.php',
    'SebastianBergmann\\CodeCoverage\\CodeCoverage' => $vendorDir . '/phpunit/php-code-coverage/src/CodeCoverage.php',
    'SebastianBergmann\\CodeCoverage\\Data\\ProcessedCodeCoverageData' => $vendorDir . '/phpunit/php-code-coverage/src/Data/ProcessedCodeCoverageData.php',
    'SebastianBergmann\\CodeCoverage\\Data\\RawCodeCoverageData' => $vendorDir . '/phpunit/php-code-coverage/src/Data/RawCodeCoverageData.php',
    'SebastianBergmann\\CodeCoverage\\DeadCodeDetectionNotSupportedException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/DeadCodeDetectionNotSupportedException.php',
    'SebastianBergmann\\CodeCoverage\\Driver\\Driver' => $vendorDir . '/phpunit/php-code-coverage/src/Driver/Driver.php',
    'SebastianBergmann\\CodeCoverage\\Driver\\PathExistsButIsNotDirectoryException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/PathExistsButIsNotDirectoryException.php',
    'SebastianBergmann\\CodeCoverage\\Driver\\PcovDriver' => $vendorDir . '/phpunit/php-code-coverage/src/Driver/PcovDriver.php',
    'SebastianBergmann\\CodeCoverage\\Driver\\PcovNotAvailableException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/PcovNotAvailableException.php',
    'SebastianBergmann\\CodeCoverage\\Driver\\Selector' => $vendorDir . '/phpunit/php-code-coverage/src/Driver/Selector.php',
    'SebastianBergmann\\CodeCoverage\\Driver\\WriteOperationFailedException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/WriteOperationFailedException.php',
    'SebastianBergmann\\CodeCoverage\\Driver\\XdebugDriver' => $vendorDir . '/phpunit/php-code-coverage/src/Driver/XdebugDriver.php',
    'SebastianBergmann\\CodeCoverage\\Driver\\XdebugNotAvailableException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/XdebugNotAvailableException.php',
    'SebastianBergmann\\CodeCoverage\\Driver\\XdebugNotEnabledException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/XdebugNotEnabledException.php',
    'SebastianBergmann\\CodeCoverage\\Exception' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/Exception.php',
    'SebastianBergmann\\CodeCoverage\\FileCouldNotBeWrittenException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/FileCouldNotBeWrittenException.php',
    'SebastianBergmann\\CodeCoverage\\Filter' => $vendorDir . '/phpunit/php-code-coverage/src/Filter.php',
    'SebastianBergmann\\CodeCoverage\\InvalidArgumentException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/InvalidArgumentException.php',
    'SebastianBergmann\\CodeCoverage\\NoCodeCoverageDriverAvailableException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/NoCodeCoverageDriverAvailableException.php',
    'SebastianBergmann\\CodeCoverage\\NoCodeCoverageDriverWithPathCoverageSupportAvailableException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/NoCodeCoverageDriverWithPathCoverageSupportAvailableException.php',
    'SebastianBergmann\\CodeCoverage\\Node\\AbstractNode' => $vendorDir . '/phpunit/php-code-coverage/src/Node/AbstractNode.php',
    'SebastianBergmann\\CodeCoverage\\Node\\Builder' => $vendorDir . '/phpunit/php-code-coverage/src/Node/Builder.php',
    'SebastianBergmann\\CodeCoverage\\Node\\CrapIndex' => $vendorDir . '/phpunit/php-code-coverage/src/Node/CrapIndex.php',
    'SebastianBergmann\\CodeCoverage\\Node\\Directory' => $vendorDir . '/phpunit/php-code-coverage/src/Node/Directory.php',
    'SebastianBergmann\\CodeCoverage\\Node\\File' => $vendorDir . '/phpunit/php-code-coverage/src/Node/File.php',
    'SebastianBergmann\\CodeCoverage\\Node\\Iterator' => $vendorDir . '/phpunit/php-code-coverage/src/Node/Iterator.php',
    'SebastianBergmann\\CodeCoverage\\ParserException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/ParserException.php',
    'SebastianBergmann\\CodeCoverage\\ReflectionException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/ReflectionException.php',
    'SebastianBergmann\\CodeCoverage\\ReportAlreadyFinalizedException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/ReportAlreadyFinalizedException.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Clover' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Clover.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Cobertura' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Cobertura.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Crap4j' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Crap4j.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Html\\Colors' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Html/Colors.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Html\\CustomCssFile' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Html/CustomCssFile.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Html\\Dashboard' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Html/Renderer/Dashboard.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Html\\Directory' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Html/Renderer/Directory.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Html\\Facade' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Html/Facade.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Html\\File' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Html/Renderer/File.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Html\\Renderer' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Html/Renderer.php',
    'SebastianBergmann\\CodeCoverage\\Report\\PHP' => $vendorDir . '/phpunit/php-code-coverage/src/Report/PHP.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Text' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Text.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Thresholds' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Thresholds.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\BuildInformation' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/BuildInformation.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Coverage' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Coverage.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Directory' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Directory.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Facade' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Facade.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\File' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/File.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Method' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Method.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Node' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Node.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Project' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Project.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Report' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Report.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Source' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Source.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Tests' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Tests.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Totals' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Totals.php',
    'SebastianBergmann\\CodeCoverage\\Report\\Xml\\Unit' => $vendorDir . '/phpunit/php-code-coverage/src/Report/Xml/Unit.php',
    'SebastianBergmann\\CodeCoverage\\StaticAnalysisCacheNotConfiguredException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/StaticAnalysisCacheNotConfiguredException.php',
    'SebastianBergmann\\CodeCoverage\\StaticAnalysis\\CacheWarmer' => $vendorDir . '/phpunit/php-code-coverage/src/StaticAnalysis/CacheWarmer.php',
    'SebastianBergmann\\CodeCoverage\\StaticAnalysis\\CachingFileAnalyser' => $vendorDir . '/phpunit/php-code-coverage/src/StaticAnalysis/CachingFileAnalyser.php',
    'SebastianBergmann\\CodeCoverage\\StaticAnalysis\\CodeUnitFindingVisitor' => $vendorDir . '/phpunit/php-code-coverage/src/StaticAnalysis/CodeUnitFindingVisitor.php',
    'SebastianBergmann\\CodeCoverage\\StaticAnalysis\\ExecutableLinesFindingVisitor' => $vendorDir . '/phpunit/php-code-coverage/src/StaticAnalysis/ExecutableLinesFindingVisitor.php',
    'SebastianBergmann\\CodeCoverage\\StaticAnalysis\\FileAnalyser' => $vendorDir . '/phpunit/php-code-coverage/src/StaticAnalysis/FileAnalyser.php',
    'SebastianBergmann\\CodeCoverage\\StaticAnalysis\\IgnoredLinesFindingVisitor' => $vendorDir . '/phpunit/php-code-coverage/src/StaticAnalysis/IgnoredLinesFindingVisitor.php',
    'SebastianBergmann\\CodeCoverage\\StaticAnalysis\\ParsingFileAnalyser' => $vendorDir . '/phpunit/php-code-coverage/src/StaticAnalysis/ParsingFileAnalyser.php',
    'SebastianBergmann\\CodeCoverage\\TestIdMissingException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/TestIdMissingException.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestSize\\Known' => $vendorDir . '/phpunit/php-code-coverage/src/TestSize/Known.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestSize\\Large' => $vendorDir . '/phpunit/php-code-coverage/src/TestSize/Large.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestSize\\Medium' => $vendorDir . '/phpunit/php-code-coverage/src/TestSize/Medium.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestSize\\Small' => $vendorDir . '/phpunit/php-code-coverage/src/TestSize/Small.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestSize\\TestSize' => $vendorDir . '/phpunit/php-code-coverage/src/TestSize/TestSize.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestSize\\Unknown' => $vendorDir . '/phpunit/php-code-coverage/src/TestSize/Unknown.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestStatus\\Failure' => $vendorDir . '/phpunit/php-code-coverage/src/TestStatus/Failure.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestStatus\\Known' => $vendorDir . '/phpunit/php-code-coverage/src/TestStatus/Known.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestStatus\\Success' => $vendorDir . '/phpunit/php-code-coverage/src/TestStatus/Success.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestStatus\\TestStatus' => $vendorDir . '/phpunit/php-code-coverage/src/TestStatus/TestStatus.php',
    'SebastianBergmann\\CodeCoverage\\Test\\TestStatus\\Unknown' => $vendorDir . '/phpunit/php-code-coverage/src/TestStatus/Unknown.php',
    'SebastianBergmann\\CodeCoverage\\UnintentionallyCoveredCodeException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/UnintentionallyCoveredCodeException.php',
    'SebastianBergmann\\CodeCoverage\\Util\\DirectoryCouldNotBeCreatedException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/DirectoryCouldNotBeCreatedException.php',
    'SebastianBergmann\\CodeCoverage\\Util\\Filesystem' => $vendorDir . '/phpunit/php-code-coverage/src/Util/Filesystem.php',
    'SebastianBergmann\\CodeCoverage\\Util\\Percentage' => $vendorDir . '/phpunit/php-code-coverage/src/Util/Percentage.php',
    'SebastianBergmann\\CodeCoverage\\Version' => $vendorDir . '/phpunit/php-code-coverage/src/Version.php',
    'SebastianBergmann\\CodeCoverage\\XmlException' => $vendorDir . '/phpunit/php-code-coverage/src/Exception/XmlException.php',
    'SebastianBergmann\\CodeUnitReverseLookup\\Wizard' => $vendorDir . '/sebastian/code-unit-reverse-lookup/src/Wizard.php',
    'SebastianBergmann\\CodeUnit\\ClassMethodUnit' => $vendorDir . '/sebastian/code-unit/src/ClassMethodUnit.php',
    'SebastianBergmann\\CodeUnit\\ClassUnit' => $vendorDir . '/sebastian/code-unit/src/ClassUnit.php',
    'SebastianBergmann\\CodeUnit\\CodeUnit' => $vendorDir . '/sebastian/code-unit/src/CodeUnit.php',
    'SebastianBergmann\\CodeUnit\\CodeUnitCollection' => $vendorDir . '/sebastian/code-unit/src/CodeUnitCollection.php',
    'SebastianBergmann\\CodeUnit\\CodeUnitCollectionIterator' => $vendorDir . '/sebastian/code-unit/src/CodeUnitCollectionIterator.php',
    'SebastianBergmann\\CodeUnit\\Exception' => $vendorDir . '/sebastian/code-unit/src/exceptions/Exception.php',
    'SebastianBergmann\\CodeUnit\\FileUnit' => $vendorDir . '/sebastian/code-unit/src/FileUnit.php',
    'SebastianBergmann\\CodeUnit\\FunctionUnit' => $vendorDir . '/sebastian/code-unit/src/FunctionUnit.php',
    'SebastianBergmann\\CodeUnit\\InterfaceMethodUnit' => $vendorDir . '/sebastian/code-unit/src/InterfaceMethodUnit.php',
    'SebastianBergmann\\CodeUnit\\InterfaceUnit' => $vendorDir . '/sebastian/code-unit/src/InterfaceUnit.php',
    'SebastianBergmann\\CodeUnit\\InvalidCodeUnitException' => $vendorDir . '/sebastian/code-unit/src/exceptions/InvalidCodeUnitException.php',
    'SebastianBergmann\\CodeUnit\\Mapper' => $vendorDir . '/sebastian/code-unit/src/Mapper.php',
    'SebastianBergmann\\CodeUnit\\NoTraitException' => $vendorDir . '/sebastian/code-unit/src/exceptions/NoTraitException.php',
    'SebastianBergmann\\CodeUnit\\ReflectionException' => $vendorDir . '/sebastian/code-unit/src/exceptions/ReflectionException.php',
    'SebastianBergmann\\CodeUnit\\TraitMethodUnit' => $vendorDir . '/sebastian/code-unit/src/TraitMethodUnit.php',
    'SebastianBergmann\\CodeUnit\\TraitUnit' => $vendorDir . '/sebastian/code-unit/src/TraitUnit.php',
    'SebastianBergmann\\Comparator\\ArrayComparator' => $vendorDir . '/sebastian/comparator/src/ArrayComparator.php',
    'SebastianBergmann\\Comparator\\Comparator' => $vendorDir . '/sebastian/comparator/src/Comparator.php',
    'SebastianBergmann\\Comparator\\ComparisonFailure' => $vendorDir . '/sebastian/comparator/src/ComparisonFailure.php',
    'SebastianBergmann\\Comparator\\DOMNodeComparator' => $vendorDir . '/sebastian/comparator/src/DOMNodeComparator.php',
    'SebastianBergmann\\Comparator\\DateTimeComparator' => $vendorDir . '/sebastian/comparator/src/DateTimeComparator.php',
    'SebastianBergmann\\Comparator\\Exception' => $vendorDir . '/sebastian/comparator/src/exceptions/Exception.php',
    'SebastianBergmann\\Comparator\\ExceptionComparator' => $vendorDir . '/sebastian/comparator/src/ExceptionComparator.php',
    'SebastianBergmann\\Comparator\\Factory' => $vendorDir . '/sebastian/comparator/src/Factory.php',
    'SebastianBergmann\\Comparator\\MockObjectComparator' => $vendorDir . '/sebastian/comparator/src/MockObjectComparator.php',
    'SebastianBergmann\\Comparator\\NumericComparator' => $vendorDir . '/sebastian/comparator/src/NumericComparator.php',
    'SebastianBergmann\\Comparator\\ObjectComparator' => $vendorDir . '/sebastian/comparator/src/ObjectComparator.php',
    'SebastianBergmann\\Comparator\\ResourceComparator' => $vendorDir . '/sebastian/comparator/src/ResourceComparator.php',
    'SebastianBergmann\\Comparator\\RuntimeException' => $vendorDir . '/sebastian/comparator/src/exceptions/RuntimeException.php',
    'SebastianBergmann\\Comparator\\ScalarComparator' => $vendorDir . '/sebastian/comparator/src/ScalarComparator.php',
    'SebastianBergmann\\Comparator\\SplObjectStorageComparator' => $vendorDir . '/sebastian/comparator/src/SplObjectStorageComparator.php',
    'SebastianBergmann\\Comparator\\TypeComparator' => $vendorDir . '/sebastian/comparator/src/TypeComparator.php',
    'SebastianBergmann\\Complexity\\Calculator' => $vendorDir . '/sebastian/complexity/src/Calculator.php',
    'SebastianBergmann\\Complexity\\Complexity' => $vendorDir . '/sebastian/complexity/src/Complexity/Complexity.php',
    'SebastianBergmann\\Complexity\\ComplexityCalculatingVisitor' => $vendorDir . '/sebastian/complexity/src/Visitor/ComplexityCalculatingVisitor.php',
    'SebastianBergmann\\Complexity\\ComplexityCollection' => $vendorDir . '/sebastian/complexity/src/Complexity/ComplexityCollection.php',
    'SebastianBergmann\\Complexity\\ComplexityCollectionIterator' => $vendorDir . '/sebastian/complexity/src/Complexity/ComplexityCollectionIterator.php',
    'SebastianBergmann\\Complexity\\CyclomaticComplexityCalculatingVisitor' => $vendorDir . '/sebastian/complexity/src/Visitor/CyclomaticComplexityCalculatingVisitor.php',
    'SebastianBergmann\\Complexity\\Exception' => $vendorDir . '/sebastian/complexity/src/Exception/Exception.php',
    'SebastianBergmann\\Complexity\\RuntimeException' => $vendorDir . '/sebastian/complexity/src/Exception/RuntimeException.php',
    'SebastianBergmann\\Diff\\Chunk' => $vendorDir . '/sebastian/diff/src/Chunk.php',
    'SebastianBergmann\\Diff\\ConfigurationException' => $vendorDir . '/sebastian/diff/src/Exception/ConfigurationException.php',
    'SebastianBergmann\\Diff\\Diff' => $vendorDir . '/sebastian/diff/src/Diff.php',
    'SebastianBergmann\\Diff\\Differ' => $vendorDir . '/sebastian/diff/src/Differ.php',
    'SebastianBergmann\\Diff\\Exception' => $vendorDir . '/sebastian/diff/src/Exception/Exception.php',
    'SebastianBergmann\\Diff\\InvalidArgumentException' => $vendorDir . '/sebastian/diff/src/Exception/InvalidArgumentException.php',
    'SebastianBergmann\\Diff\\Line' => $vendorDir . '/sebastian/diff/src/Line.php',
    'SebastianBergmann\\Diff\\LongestCommonSubsequenceCalculator' => $vendorDir . '/sebastian/diff/src/LongestCommonSubsequenceCalculator.php',
    'SebastianBergmann\\Diff\\MemoryEfficientLongestCommonSubsequenceCalculator' => $vendorDir . '/sebastian/diff/src/MemoryEfficientLongestCommonSubsequenceCalculator.php',
    'SebastianBergmann\\Diff\\Output\\AbstractChunkOutputBuilder' => $vendorDir . '/sebastian/diff/src/Output/AbstractChunkOutputBuilder.php',
    'SebastianBergmann\\Diff\\Output\\DiffOnlyOutputBuilder' => $vendorDir . '/sebastian/diff/src/Output/DiffOnlyOutputBuilder.php',
    'SebastianBergmann\\Diff\\Output\\DiffOutputBuilderInterface' => $vendorDir . '/sebastian/diff/src/Output/DiffOutputBuilderInterface.php',
    'SebastianBergmann\\Diff\\Output\\StrictUnifiedDiffOutputBuilder' => $vendorDir . '/sebastian/diff/src/Output/StrictUnifiedDiffOutputBuilder.php',
    'SebastianBergmann\\Diff\\Output\\UnifiedDiffOutputBuilder' => $vendorDir . '/sebastian/diff/src/Output/UnifiedDiffOutputBuilder.php',
    'SebastianBergmann\\Diff\\Parser' => $vendorDir . '/sebastian/diff/src/Parser.php',
    'SebastianBergmann\\Diff\\TimeEfficientLongestCommonSubsequenceCalculator' => $vendorDir . '/sebastian/diff/src/TimeEfficientLongestCommonSubsequenceCalculator.php',
    'SebastianBergmann\\Environment\\Console' => $vendorDir . '/sebastian/environment/src/Console.php',
    'SebastianBergmann\\Environment\\Runtime' => $vendorDir . '/sebastian/environment/src/Runtime.php',
    'SebastianBergmann\\Exporter\\Exporter' => $vendorDir . '/sebastian/exporter/src/Exporter.php',
    'SebastianBergmann\\FileIterator\\ExcludeIterator' => $vendorDir . '/phpunit/php-file-iterator/src/ExcludeIterator.php',
    'SebastianBergmann\\FileIterator\\Facade' => $vendorDir . '/phpunit/php-file-iterator/src/Facade.php',
    'SebastianBergmann\\FileIterator\\Factory' => $vendorDir . '/phpunit/php-file-iterator/src/Factory.php',
    'SebastianBergmann\\FileIterator\\Iterator' => $vendorDir . '/phpunit/php-file-iterator/src/Iterator.php',
    'SebastianBergmann\\GlobalState\\CodeExporter' => $vendorDir . '/sebastian/global-state/src/CodeExporter.php',
    'SebastianBergmann\\GlobalState\\Exception' => $vendorDir . '/sebastian/global-state/src/exceptions/Exception.php',
    'SebastianBergmann\\GlobalState\\ExcludeList' => $vendorDir . '/sebastian/global-state/src/ExcludeList.php',
    'SebastianBergmann\\GlobalState\\Restorer' => $vendorDir . '/sebastian/global-state/src/Restorer.php',
    'SebastianBergmann\\GlobalState\\RuntimeException' => $vendorDir . '/sebastian/global-state/src/exceptions/RuntimeException.php',
    'SebastianBergmann\\GlobalState\\Snapshot' => $vendorDir . '/sebastian/global-state/src/Snapshot.php',
    'SebastianBergmann\\Invoker\\Exception' => $vendorDir . '/phpunit/php-invoker/src/exceptions/Exception.php',
    'SebastianBergmann\\Invoker\\Invoker' => $vendorDir . '/phpunit/php-invoker/src/Invoker.php',
    'SebastianBergmann\\Invoker\\ProcessControlExtensionNotLoadedException' => $vendorDir . '/phpunit/php-invoker/src/exceptions/ProcessControlExtensionNotLoadedException.php',
    'SebastianBergmann\\Invoker\\TimeoutException' => $vendorDir . '/phpunit/php-invoker/src/exceptions/TimeoutException.php',
    'SebastianBergmann\\LinesOfCode\\Counter' => $vendorDir . '/sebastian/lines-of-code/src/Counter.php',
    'SebastianBergmann\\LinesOfCode\\Exception' => $vendorDir . '/sebastian/lines-of-code/src/Exception/Exception.php',
    'SebastianBergmann\\LinesOfCode\\IllogicalValuesException' => $vendorDir . '/sebastian/lines-of-code/src/Exception/IllogicalValuesException.php',
    'SebastianBergmann\\LinesOfCode\\LineCountingVisitor' => $vendorDir . '/sebastian/lines-of-code/src/LineCountingVisitor.php',
    'SebastianBergmann\\LinesOfCode\\LinesOfCode' => $vendorDir . '/sebastian/lines-of-code/src/LinesOfCode.php',
    'SebastianBergmann\\LinesOfCode\\NegativeValueException' => $vendorDir . '/sebastian/lines-of-code/src/Exception/NegativeValueException.php',
    'SebastianBergmann\\LinesOfCode\\RuntimeException' => $vendorDir . '/sebastian/lines-of-code/src/Exception/RuntimeException.php',
    'SebastianBergmann\\ObjectEnumerator\\Enumerator' => $vendorDir . '/sebastian/object-enumerator/src/Enumerator.php',
    'SebastianBergmann\\ObjectReflector\\ObjectReflector' => $vendorDir . '/sebastian/object-reflector/src/ObjectReflector.php',
    'SebastianBergmann\\RecursionContext\\Context' => $vendorDir . '/sebastian/recursion-context/src/Context.php',
    'SebastianBergmann\\Template\\Exception' => $vendorDir . '/phpunit/php-text-template/src/exceptions/Exception.php',
    'SebastianBergmann\\Template\\InvalidArgumentException' => $vendorDir . '/phpunit/php-text-template/src/exceptions/InvalidArgumentException.php',
    'SebastianBergmann\\Template\\RuntimeException' => $vendorDir . '/phpunit/php-text-template/src/exceptions/RuntimeException.php',
    'SebastianBergmann\\Template\\Template' => $vendorDir . '/phpunit/php-text-template/src/Template.php',
    'SebastianBergmann\\Timer\\Duration' => $vendorDir . '/phpunit/php-timer/src/Duration.php',
    'SebastianBergmann\\Timer\\Exception' => $vendorDir . '/phpunit/php-timer/src/exceptions/Exception.php',
    'SebastianBergmann\\Timer\\NoActiveTimerException' => $vendorDir . '/phpunit/php-timer/src/exceptions/NoActiveTimerException.php',
    'SebastianBergmann\\Timer\\ResourceUsageFormatter' => $vendorDir . '/phpunit/php-timer/src/ResourceUsageFormatter.php',
    'SebastianBergmann\\Timer\\TimeSinceStartOfRequestNotAvailableException' => $vendorDir . '/phpunit/php-timer/src/exceptions/TimeSinceStartOfRequestNotAvailableException.php',
    'SebastianBergmann\\Timer\\Timer' => $vendorDir . '/phpunit/php-timer/src/Timer.php',
    'SebastianBergmann\\Type\\CallableType' => $vendorDir . '/sebastian/type/src/type/CallableType.php',
    'SebastianBergmann\\Type\\Exception' => $vendorDir . '/sebastian/type/src/exception/Exception.php',
    'SebastianBergmann\\Type\\FalseType' => $vendorDir . '/sebastian/type/src/type/FalseType.php',
    'SebastianBergmann\\Type\\GenericObjectType' => $vendorDir . '/sebastian/type/src/type/GenericObjectType.php',
    'SebastianBergmann\\Type\\IntersectionType' => $vendorDir . '/sebastian/type/src/type/IntersectionType.php',
    'SebastianBergmann\\Type\\IterableType' => $vendorDir . '/sebastian/type/src/type/IterableType.php',
    'SebastianBergmann\\Type\\MixedType' => $vendorDir . '/sebastian/type/src/type/MixedType.php',
    'SebastianBergmann\\Type\\NeverType' => $vendorDir . '/sebastian/type/src/type/NeverType.php',
    'SebastianBergmann\\Type\\NullType' => $vendorDir . '/sebastian/type/src/type/NullType.php',
    'SebastianBergmann\\Type\\ObjectType' => $vendorDir . '/sebastian/type/src/type/ObjectType.php',
    'SebastianBergmann\\Type\\Parameter' => $vendorDir . '/sebastian/type/src/Parameter.php',
    'SebastianBergmann\\Type\\ReflectionMapper' => $vendorDir . '/sebastian/type/src/ReflectionMapper.php',
    'SebastianBergmann\\Type\\RuntimeException' => $vendorDir . '/sebastian/type/src/exception/RuntimeException.php',
    'SebastianBergmann\\Type\\SimpleType' => $vendorDir . '/sebastian/type/src/type/SimpleType.php',
    'SebastianBergmann\\Type\\StaticType' => $vendorDir . '/sebastian/type/src/type/StaticType.php',
    'SebastianBergmann\\Type\\TrueType' => $vendorDir . '/sebastian/type/src/type/TrueType.php',
    'SebastianBergmann\\Type\\Type' => $vendorDir . '/sebastian/type/src/type/Type.php',
    'SebastianBergmann\\Type\\TypeName' => $vendorDir . '/sebastian/type/src/TypeName.php',
    'SebastianBergmann\\Type\\UnionType' => $vendorDir . '/sebastian/type/src/type/UnionType.php',
    'SebastianBergmann\\Type\\UnknownType' => $vendorDir . '/sebastian/type/src/type/UnknownType.php',
    'SebastianBergmann\\Type\\VoidType' => $vendorDir . '/sebastian/type/src/type/VoidType.php',
    'SebastianBergmann\\Version' => $vendorDir . '/sebastian/version/src/Version.php',
    'SendGrid' => $vendorDir . '/sendgrid/sendgrid/lib/SendGrid.php',
    'SendGrid\\Client' => $vendorDir . '/sendgrid/php-http-client/lib/Client.php',
    'SendGrid\\Contacts\\Recipient' => $vendorDir . '/sendgrid/sendgrid/lib/contacts/Recipient.php',
    'SendGrid\\Contacts\\RecipientForm' => $vendorDir . '/sendgrid/sendgrid/lib/contacts/RecipientForm.php',
    'SendGrid\\EventWebhook\\EventWebhook' => $vendorDir . '/sendgrid/sendgrid/lib/eventwebhook/EventWebhook.php',
    'SendGrid\\EventWebhook\\EventWebhookHeader' => $vendorDir . '/sendgrid/sendgrid/lib/eventwebhook/EventWebhookHeader.php',
    'SendGrid\\Exception\\InvalidRequest' => $vendorDir . '/sendgrid/php-http-client/lib/Exception/InvalidRequest.php',
    'SendGrid\\Helper\\Assert' => $vendorDir . '/sendgrid/sendgrid/lib/helper/Assert.php',
    'SendGrid\\Mail\\Asm' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Asm.php',
    'SendGrid\\Mail\\Attachment' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Attachment.php',
    'SendGrid\\Mail\\BatchId' => $vendorDir . '/sendgrid/sendgrid/lib/mail/BatchId.php',
    'SendGrid\\Mail\\Bcc' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Bcc.php',
    'SendGrid\\Mail\\BccSettings' => $vendorDir . '/sendgrid/sendgrid/lib/mail/BccSettings.php',
    'SendGrid\\Mail\\BypassBounceManagement' => $vendorDir . '/sendgrid/sendgrid/lib/mail/BypassBounceManagement.php',
    'SendGrid\\Mail\\BypassListManagement' => $vendorDir . '/sendgrid/sendgrid/lib/mail/BypassListManagement.php',
    'SendGrid\\Mail\\BypassSpamManagement' => $vendorDir . '/sendgrid/sendgrid/lib/mail/BypassSpamManagement.php',
    'SendGrid\\Mail\\BypassUnsubscribeManagement' => $vendorDir . '/sendgrid/sendgrid/lib/mail/BypassUnsubscribeManagement.php',
    'SendGrid\\Mail\\Category' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Category.php',
    'SendGrid\\Mail\\Cc' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Cc.php',
    'SendGrid\\Mail\\ClickTracking' => $vendorDir . '/sendgrid/sendgrid/lib/mail/ClickTracking.php',
    'SendGrid\\Mail\\Content' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Content.php',
    'SendGrid\\Mail\\CustomArg' => $vendorDir . '/sendgrid/sendgrid/lib/mail/CustomArg.php',
    'SendGrid\\Mail\\EmailAddress' => $vendorDir . '/sendgrid/sendgrid/lib/mail/EmailAddress.php',
    'SendGrid\\Mail\\Footer' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Footer.php',
    'SendGrid\\Mail\\From' => $vendorDir . '/sendgrid/sendgrid/lib/mail/From.php',
    'SendGrid\\Mail\\Ganalytics' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Ganalytics.php',
    'SendGrid\\Mail\\GroupId' => $vendorDir . '/sendgrid/sendgrid/lib/mail/GroupId.php',
    'SendGrid\\Mail\\GroupsToDisplay' => $vendorDir . '/sendgrid/sendgrid/lib/mail/GroupsToDisplay.php',
    'SendGrid\\Mail\\Header' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Header.php',
    'SendGrid\\Mail\\HtmlContent' => $vendorDir . '/sendgrid/sendgrid/lib/mail/HtmlContent.php',
    'SendGrid\\Mail\\IpPoolName' => $vendorDir . '/sendgrid/sendgrid/lib/mail/IpPoolName.php',
    'SendGrid\\Mail\\Mail' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Mail.php',
    'SendGrid\\Mail\\MailSettings' => $vendorDir . '/sendgrid/sendgrid/lib/mail/MailSettings.php',
    'SendGrid\\Mail\\MimeType' => $vendorDir . '/sendgrid/sendgrid/lib/mail/MimeType.php',
    'SendGrid\\Mail\\OpenTracking' => $vendorDir . '/sendgrid/sendgrid/lib/mail/OpenTracking.php',
    'SendGrid\\Mail\\Personalization' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Personalization.php',
    'SendGrid\\Mail\\PlainTextContent' => $vendorDir . '/sendgrid/sendgrid/lib/mail/PlainTextContent.php',
    'SendGrid\\Mail\\ReplyTo' => $vendorDir . '/sendgrid/sendgrid/lib/mail/ReplyTo.php',
    'SendGrid\\Mail\\SandBoxMode' => $vendorDir . '/sendgrid/sendgrid/lib/mail/SandBoxMode.php',
    'SendGrid\\Mail\\Section' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Section.php',
    'SendGrid\\Mail\\SendAt' => $vendorDir . '/sendgrid/sendgrid/lib/mail/SendAt.php',
    'SendGrid\\Mail\\SpamCheck' => $vendorDir . '/sendgrid/sendgrid/lib/mail/SpamCheck.php',
    'SendGrid\\Mail\\Subject' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Subject.php',
    'SendGrid\\Mail\\SubscriptionTracking' => $vendorDir . '/sendgrid/sendgrid/lib/mail/SubscriptionTracking.php',
    'SendGrid\\Mail\\Substitution' => $vendorDir . '/sendgrid/sendgrid/lib/mail/Substitution.php',
    'SendGrid\\Mail\\TemplateId' => $vendorDir . '/sendgrid/sendgrid/lib/mail/TemplateId.php',
    'SendGrid\\Mail\\To' => $vendorDir . '/sendgrid/sendgrid/lib/mail/To.php',
    'SendGrid\\Mail\\TrackingSettings' => $vendorDir . '/sendgrid/sendgrid/lib/mail/TrackingSettings.php',
    'SendGrid\\Mail\\TypeException' => $vendorDir . '/sendgrid/sendgrid/lib/mail/TypeException.php',
    'SendGrid\\Response' => $vendorDir . '/sendgrid/php-http-client/lib/Response.php',
    'SendGrid\\Stats\\Stats' => $vendorDir . '/sendgrid/sendgrid/lib/stats/Stats.php',
    'Spatie\\Backtrace\\Arguments\\ArgumentReducers' => $vendorDir . '/spatie/backtrace/src/Arguments/ArgumentReducers.php',
    'Spatie\\Backtrace\\Arguments\\ProvidedArgument' => $vendorDir . '/spatie/backtrace/src/Arguments/ProvidedArgument.php',
    'Spatie\\Backtrace\\Arguments\\ReduceArgumentPayloadAction' => $vendorDir . '/spatie/backtrace/src/Arguments/ReduceArgumentPayloadAction.php',
    'Spatie\\Backtrace\\Arguments\\ReduceArgumentsAction' => $vendorDir . '/spatie/backtrace/src/Arguments/ReduceArgumentsAction.php',
    'Spatie\\Backtrace\\Arguments\\ReducedArgument\\ReducedArgument' => $vendorDir . '/spatie/backtrace/src/Arguments/ReducedArgument/ReducedArgument.php',
    'Spatie\\Backtrace\\Arguments\\ReducedArgument\\ReducedArgumentContract' => $vendorDir . '/spatie/backtrace/src/Arguments/ReducedArgument/ReducedArgumentContract.php',
    'Spatie\\Backtrace\\Arguments\\ReducedArgument\\TruncatedReducedArgument' => $vendorDir . '/spatie/backtrace/src/Arguments/ReducedArgument/TruncatedReducedArgument.php',
    'Spatie\\Backtrace\\Arguments\\ReducedArgument\\UnReducedArgument' => $vendorDir . '/spatie/backtrace/src/Arguments/ReducedArgument/UnReducedArgument.php',
    'Spatie\\Backtrace\\Arguments\\ReducedArgument\\VariadicReducedArgument' => $vendorDir . '/spatie/backtrace/src/Arguments/ReducedArgument/VariadicReducedArgument.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\ArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/ArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\ArrayArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/ArrayArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\BaseTypeArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/BaseTypeArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\ClosureArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/ClosureArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\DateTimeArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/DateTimeArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\DateTimeZoneArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/DateTimeZoneArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\EnumArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/EnumArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\MinimalArrayArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/MinimalArrayArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\SensitiveParameterArrayReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/SensitiveParameterArrayReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\StdClassArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/StdClassArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\StringableArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/StringableArgumentReducer.php',
    'Spatie\\Backtrace\\Arguments\\Reducers\\SymphonyRequestArgumentReducer' => $vendorDir . '/spatie/backtrace/src/Arguments/Reducers/SymphonyRequestArgumentReducer.php',
    'Spatie\\Backtrace\\Backtrace' => $vendorDir . '/spatie/backtrace/src/Backtrace.php',
    'Spatie\\Backtrace\\CodeSnippets\\CodeSnippet' => $vendorDir . '/spatie/backtrace/src/CodeSnippets/CodeSnippet.php',
    'Spatie\\Backtrace\\CodeSnippets\\FileSnippetProvider' => $vendorDir . '/spatie/backtrace/src/CodeSnippets/FileSnippetProvider.php',
    'Spatie\\Backtrace\\CodeSnippets\\LaravelSerializableClosureSnippetProvider' => $vendorDir . '/spatie/backtrace/src/CodeSnippets/LaravelSerializableClosureSnippetProvider.php',
    'Spatie\\Backtrace\\CodeSnippets\\NullSnippetProvider' => $vendorDir . '/spatie/backtrace/src/CodeSnippets/NullSnippetProvider.php',
    'Spatie\\Backtrace\\CodeSnippets\\SnippetProvider' => $vendorDir . '/spatie/backtrace/src/CodeSnippets/SnippetProvider.php',
    'Spatie\\Backtrace\\Frame' => $vendorDir . '/spatie/backtrace/src/Frame.php',
    'Spatie\\ErrorSolutions\\Contracts\\BaseSolution' => $vendorDir . '/spatie/error-solutions/src/Contracts/BaseSolution.php',
    'Spatie\\ErrorSolutions\\Contracts\\HasSolutionsForThrowable' => $vendorDir . '/spatie/error-solutions/src/Contracts/HasSolutionsForThrowable.php',
    'Spatie\\ErrorSolutions\\Contracts\\ProvidesSolution' => $vendorDir . '/spatie/error-solutions/src/Contracts/ProvidesSolution.php',
    'Spatie\\ErrorSolutions\\Contracts\\RunnableSolution' => $vendorDir . '/spatie/error-solutions/src/Contracts/RunnableSolution.php',
    'Spatie\\ErrorSolutions\\Contracts\\Solution' => $vendorDir . '/spatie/error-solutions/src/Contracts/Solution.php',
    'Spatie\\ErrorSolutions\\Contracts\\SolutionProviderRepository' => $vendorDir . '/spatie/error-solutions/src/Contracts/SolutionProviderRepository.php',
    'Spatie\\ErrorSolutions\\DiscoverSolutionProviders' => $vendorDir . '/spatie/error-solutions/src/DiscoverSolutionProviders.php',
    'Spatie\\ErrorSolutions\\SolutionProviderRepository' => $vendorDir . '/spatie/error-solutions/src/SolutionProviderRepository.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\BadMethodCallSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/BadMethodCallSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\DefaultDbNameSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/DefaultDbNameSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\GenericLaravelExceptionSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/GenericLaravelExceptionSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\IncorrectValetDbCredentialsSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/IncorrectValetDbCredentialsSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\InvalidRouteActionSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/InvalidRouteActionSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\LazyLoadingViolationSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/LazyLoadingViolationSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\MissingAppKeySolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/MissingAppKeySolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\MissingColumnSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/MissingColumnSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\MissingImportSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/MissingImportSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\MissingLivewireComponentSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/MissingLivewireComponentSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\MissingMixManifestSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/MissingMixManifestSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\MissingViteManifestSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/MissingViteManifestSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\OpenAiSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/OpenAiSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\RouteNotDefinedSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/RouteNotDefinedSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\RunningLaravelDuskInProductionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/RunningLaravelDuskInProductionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\SailNetworkSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/SailNetworkSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\TableNotFoundSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/TableNotFoundSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\UndefinedLivewireMethodSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/UndefinedLivewireMethodSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\UndefinedLivewirePropertySolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/UndefinedLivewirePropertySolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\UndefinedViewVariableSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/UndefinedViewVariableSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\UnknownMariadbCollationSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/UnknownMariadbCollationSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\UnknownMysql8CollationSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/UnknownMysql8CollationSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\UnknownValidationSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/UnknownValidationSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\Laravel\\ViewNotFoundSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/Laravel/ViewNotFoundSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\MergeConflictSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/MergeConflictSolutionProvider.php',
    'Spatie\\ErrorSolutions\\SolutionProviders\\UndefinedPropertySolutionProvider' => $vendorDir . '/spatie/error-solutions/src/SolutionProviders/UndefinedPropertySolutionProvider.php',
    'Spatie\\ErrorSolutions\\Solutions\\Concerns\\IsProvidedByFlare' => $vendorDir . '/spatie/error-solutions/src/Solutions/Concerns/IsProvidedByFlare.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\GenerateAppKeySolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/GenerateAppKeySolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\LivewireDiscoverSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/LivewireDiscoverSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\MakeViewVariableOptionalSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/MakeViewVariableOptionalSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\RunMigrationsSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/RunMigrationsSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\SuggestLivewireMethodNameSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/SuggestLivewireMethodNameSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\SuggestLivewirePropertyNameSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/SuggestLivewirePropertyNameSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\SuggestUsingCorrectDbNameSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/SuggestUsingCorrectDbNameSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\SuggestUsingMariadbDatabaseSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/SuggestUsingMariadbDatabaseSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\SuggestUsingMysql8DatabaseSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/SuggestUsingMysql8DatabaseSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\Laravel\\UseDefaultValetDbCredentialsSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/Laravel/UseDefaultValetDbCredentialsSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\OpenAi\\DummyCache' => $vendorDir . '/spatie/error-solutions/src/Solutions/OpenAi/DummyCache.php',
    'Spatie\\ErrorSolutions\\Solutions\\OpenAi\\OpenAiPromptViewModel' => $vendorDir . '/spatie/error-solutions/src/Solutions/OpenAi/OpenAiPromptViewModel.php',
    'Spatie\\ErrorSolutions\\Solutions\\OpenAi\\OpenAiSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/OpenAi/OpenAiSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\OpenAi\\OpenAiSolutionProvider' => $vendorDir . '/spatie/error-solutions/src/Solutions/OpenAi/OpenAiSolutionProvider.php',
    'Spatie\\ErrorSolutions\\Solutions\\OpenAi\\OpenAiSolutionResponse' => $vendorDir . '/spatie/error-solutions/src/Solutions/OpenAi/OpenAiSolutionResponse.php',
    'Spatie\\ErrorSolutions\\Solutions\\SolutionTransformer' => $vendorDir . '/spatie/error-solutions/src/Solutions/SolutionTransformer.php',
    'Spatie\\ErrorSolutions\\Solutions\\SuggestCorrectVariableNameSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/SuggestCorrectVariableNameSolution.php',
    'Spatie\\ErrorSolutions\\Solutions\\SuggestImportSolution' => $vendorDir . '/spatie/error-solutions/src/Solutions/SuggestImportSolution.php',
    'Spatie\\ErrorSolutions\\Support\\AiPromptRenderer' => $vendorDir . '/spatie/error-solutions/src/Support/AiPromptRenderer.php',
    'Spatie\\ErrorSolutions\\Support\\Laravel\\Composer\\Composer' => $vendorDir . '/spatie/error-solutions/src/Support/Laravel/Composer/Composer.php',
    'Spatie\\ErrorSolutions\\Support\\Laravel\\Composer\\ComposerClassMap' => $vendorDir . '/spatie/error-solutions/src/Support/Laravel/Composer/ComposerClassMap.php',
    'Spatie\\ErrorSolutions\\Support\\Laravel\\Composer\\FakeComposer' => $vendorDir . '/spatie/error-solutions/src/Support/Laravel/Composer/FakeComposer.php',
    'Spatie\\ErrorSolutions\\Support\\Laravel\\LaravelVersion' => $vendorDir . '/spatie/error-solutions/src/Support/Laravel/LaravelVersion.php',
    'Spatie\\ErrorSolutions\\Support\\Laravel\\LivewireComponentParser' => $vendorDir . '/spatie/error-solutions/src/Support/Laravel/LivewireComponentParser.php',
    'Spatie\\ErrorSolutions\\Support\\Laravel\\StringComparator' => $vendorDir . '/spatie/error-solutions/src/Support/Laravel/StringComparator.php',
    'Spatie\\FlareClient\\Api' => $vendorDir . '/spatie/flare-client-php/src/Api.php',
    'Spatie\\FlareClient\\Concerns\\HasContext' => $vendorDir . '/spatie/flare-client-php/src/Concerns/HasContext.php',
    'Spatie\\FlareClient\\Concerns\\UsesTime' => $vendorDir . '/spatie/flare-client-php/src/Concerns/UsesTime.php',
    'Spatie\\FlareClient\\Context\\BaseContextProviderDetector' => $vendorDir . '/spatie/flare-client-php/src/Context/BaseContextProviderDetector.php',
    'Spatie\\FlareClient\\Context\\ConsoleContextProvider' => $vendorDir . '/spatie/flare-client-php/src/Context/ConsoleContextProvider.php',
    'Spatie\\FlareClient\\Context\\ContextProvider' => $vendorDir . '/spatie/flare-client-php/src/Context/ContextProvider.php',
    'Spatie\\FlareClient\\Context\\ContextProviderDetector' => $vendorDir . '/spatie/flare-client-php/src/Context/ContextProviderDetector.php',
    'Spatie\\FlareClient\\Context\\RequestContextProvider' => $vendorDir . '/spatie/flare-client-php/src/Context/RequestContextProvider.php',
    'Spatie\\FlareClient\\Contracts\\ProvidesFlareContext' => $vendorDir . '/spatie/flare-client-php/src/Contracts/ProvidesFlareContext.php',
    'Spatie\\FlareClient\\Enums\\MessageLevels' => $vendorDir . '/spatie/flare-client-php/src/Enums/MessageLevels.php',
    'Spatie\\FlareClient\\Enums\\OverriddenGrouping' => $vendorDir . '/spatie/flare-client-php/src/Enums/OverriddenGrouping.php',
    'Spatie\\FlareClient\\Flare' => $vendorDir . '/spatie/flare-client-php/src/Flare.php',
    'Spatie\\FlareClient\\FlareMiddleware\\AddDocumentationLinks' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/AddDocumentationLinks.php',
    'Spatie\\FlareClient\\FlareMiddleware\\AddEnvironmentInformation' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/AddEnvironmentInformation.php',
    'Spatie\\FlareClient\\FlareMiddleware\\AddGitInformation' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/AddGitInformation.php',
    'Spatie\\FlareClient\\FlareMiddleware\\AddGlows' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/AddGlows.php',
    'Spatie\\FlareClient\\FlareMiddleware\\AddNotifierName' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/AddNotifierName.php',
    'Spatie\\FlareClient\\FlareMiddleware\\AddSolutions' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/AddSolutions.php',
    'Spatie\\FlareClient\\FlareMiddleware\\CensorRequestBodyFields' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/CensorRequestBodyFields.php',
    'Spatie\\FlareClient\\FlareMiddleware\\CensorRequestHeaders' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/CensorRequestHeaders.php',
    'Spatie\\FlareClient\\FlareMiddleware\\FlareMiddleware' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/FlareMiddleware.php',
    'Spatie\\FlareClient\\FlareMiddleware\\RemoveRequestIp' => $vendorDir . '/spatie/flare-client-php/src/FlareMiddleware/RemoveRequestIp.php',
    'Spatie\\FlareClient\\Frame' => $vendorDir . '/spatie/flare-client-php/src/Frame.php',
    'Spatie\\FlareClient\\Glows\\Glow' => $vendorDir . '/spatie/flare-client-php/src/Glows/Glow.php',
    'Spatie\\FlareClient\\Glows\\GlowRecorder' => $vendorDir . '/spatie/flare-client-php/src/Glows/GlowRecorder.php',
    'Spatie\\FlareClient\\Http\\Client' => $vendorDir . '/spatie/flare-client-php/src/Http/Client.php',
    'Spatie\\FlareClient\\Http\\Exceptions\\BadResponse' => $vendorDir . '/spatie/flare-client-php/src/Http/Exceptions/BadResponse.php',
    'Spatie\\FlareClient\\Http\\Exceptions\\BadResponseCode' => $vendorDir . '/spatie/flare-client-php/src/Http/Exceptions/BadResponseCode.php',
    'Spatie\\FlareClient\\Http\\Exceptions\\InvalidData' => $vendorDir . '/spatie/flare-client-php/src/Http/Exceptions/InvalidData.php',
    'Spatie\\FlareClient\\Http\\Exceptions\\MissingParameter' => $vendorDir . '/spatie/flare-client-php/src/Http/Exceptions/MissingParameter.php',
    'Spatie\\FlareClient\\Http\\Exceptions\\NotFound' => $vendorDir . '/spatie/flare-client-php/src/Http/Exceptions/NotFound.php',
    'Spatie\\FlareClient\\Http\\Response' => $vendorDir . '/spatie/flare-client-php/src/Http/Response.php',
    'Spatie\\FlareClient\\Report' => $vendorDir . '/spatie/flare-client-php/src/Report.php',
    'Spatie\\FlareClient\\Solutions\\ReportSolution' => $vendorDir . '/spatie/flare-client-php/src/Solutions/ReportSolution.php',
    'Spatie\\FlareClient\\Support\\PhpStackFrameArgumentsFixer' => $vendorDir . '/spatie/flare-client-php/src/Support/PhpStackFrameArgumentsFixer.php',
    'Spatie\\FlareClient\\Time\\SystemTime' => $vendorDir . '/spatie/flare-client-php/src/Time/SystemTime.php',
    'Spatie\\FlareClient\\Time\\Time' => $vendorDir . '/spatie/flare-client-php/src/Time/Time.php',
    'Spatie\\FlareClient\\Truncation\\AbstractTruncationStrategy' => $vendorDir . '/spatie/flare-client-php/src/Truncation/AbstractTruncationStrategy.php',
    'Spatie\\FlareClient\\Truncation\\ReportTrimmer' => $vendorDir . '/spatie/flare-client-php/src/Truncation/ReportTrimmer.php',
    'Spatie\\FlareClient\\Truncation\\TrimContextItemsStrategy' => $vendorDir . '/spatie/flare-client-php/src/Truncation/TrimContextItemsStrategy.php',
    'Spatie\\FlareClient\\Truncation\\TrimStackFrameArgumentsStrategy' => $vendorDir . '/spatie/flare-client-php/src/Truncation/TrimStackFrameArgumentsStrategy.php',
    'Spatie\\FlareClient\\Truncation\\TrimStringsStrategy' => $vendorDir . '/spatie/flare-client-php/src/Truncation/TrimStringsStrategy.php',
    'Spatie\\FlareClient\\Truncation\\TruncationStrategy' => $vendorDir . '/spatie/flare-client-php/src/Truncation/TruncationStrategy.php',
    'Spatie\\FlareClient\\View' => $vendorDir . '/spatie/flare-client-php/src/View.php',
    'Spatie\\Ignition\\Config\\FileConfigManager' => $vendorDir . '/spatie/ignition/src/Config/FileConfigManager.php',
    'Spatie\\Ignition\\Config\\IgnitionConfig' => $vendorDir . '/spatie/ignition/src/Config/IgnitionConfig.php',
    'Spatie\\Ignition\\Contracts\\BaseSolution' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Contracts/BaseSolution.php',
    'Spatie\\Ignition\\Contracts\\ConfigManager' => $vendorDir . '/spatie/ignition/src/Contracts/ConfigManager.php',
    'Spatie\\Ignition\\Contracts\\HasSolutionsForThrowable' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Contracts/HasSolutionsForThrowable.php',
    'Spatie\\Ignition\\Contracts\\ProvidesSolution' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Contracts/ProvidesSolution.php',
    'Spatie\\Ignition\\Contracts\\RunnableSolution' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Contracts/RunnableSolution.php',
    'Spatie\\Ignition\\Contracts\\Solution' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Contracts/Solution.php',
    'Spatie\\Ignition\\Contracts\\SolutionProviderRepository' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Contracts/SolutionProviderRepository.php',
    'Spatie\\Ignition\\ErrorPage\\ErrorPageViewModel' => $vendorDir . '/spatie/ignition/src/ErrorPage/ErrorPageViewModel.php',
    'Spatie\\Ignition\\ErrorPage\\Renderer' => $vendorDir . '/spatie/ignition/src/ErrorPage/Renderer.php',
    'Spatie\\Ignition\\Ignition' => $vendorDir . '/spatie/ignition/src/Ignition.php',
    'Spatie\\Ignition\\Solutions\\OpenAi\\DummyCache' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/OpenAi/DummyCache.php',
    'Spatie\\Ignition\\Solutions\\OpenAi\\OpenAiPromptViewModel' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/OpenAi/OpenAiPromptViewModel.php',
    'Spatie\\Ignition\\Solutions\\OpenAi\\OpenAiSolution' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/OpenAi/OpenAiSolution.php',
    'Spatie\\Ignition\\Solutions\\OpenAi\\OpenAiSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/OpenAi/OpenAiSolutionProvider.php',
    'Spatie\\Ignition\\Solutions\\OpenAi\\OpenAiSolutionResponse' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/OpenAi/OpenAiSolutionResponse.php',
    'Spatie\\Ignition\\Solutions\\SolutionProviders\\BadMethodCallSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/SolutionProviders/BadMethodCallSolutionProvider.php',
    'Spatie\\Ignition\\Solutions\\SolutionProviders\\MergeConflictSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/SolutionProviders/MergeConflictSolutionProvider.php',
    'Spatie\\Ignition\\Solutions\\SolutionProviders\\SolutionProviderRepository' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/SolutionProviders/SolutionProviderRepository.php',
    'Spatie\\Ignition\\Solutions\\SolutionProviders\\UndefinedPropertySolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/SolutionProviders/UndefinedPropertySolutionProvider.php',
    'Spatie\\Ignition\\Solutions\\SolutionTransformer' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/SolutionTransformer.php',
    'Spatie\\Ignition\\Solutions\\SuggestCorrectVariableNameSolution' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/SuggestCorrectVariableNameSolution.php',
    'Spatie\\Ignition\\Solutions\\SuggestImportSolution' => $vendorDir . '/spatie/error-solutions/legacy/ignition/Solutions/SuggestImportSolution.php',
    'Spatie\\LaravelIgnition\\ArgumentReducers\\CollectionArgumentReducer' => $vendorDir . '/spatie/laravel-ignition/src/ArgumentReducers/CollectionArgumentReducer.php',
    'Spatie\\LaravelIgnition\\ArgumentReducers\\ModelArgumentReducer' => $vendorDir . '/spatie/laravel-ignition/src/ArgumentReducers/ModelArgumentReducer.php',
    'Spatie\\LaravelIgnition\\Commands\\SolutionMakeCommand' => $vendorDir . '/spatie/laravel-ignition/src/Commands/SolutionMakeCommand.php',
    'Spatie\\LaravelIgnition\\Commands\\SolutionProviderMakeCommand' => $vendorDir . '/spatie/laravel-ignition/src/Commands/SolutionProviderMakeCommand.php',
    'Spatie\\LaravelIgnition\\Commands\\TestCommand' => $vendorDir . '/spatie/laravel-ignition/src/Commands/TestCommand.php',
    'Spatie\\LaravelIgnition\\ContextProviders\\LaravelConsoleContextProvider' => $vendorDir . '/spatie/laravel-ignition/src/ContextProviders/LaravelConsoleContextProvider.php',
    'Spatie\\LaravelIgnition\\ContextProviders\\LaravelContextProviderDetector' => $vendorDir . '/spatie/laravel-ignition/src/ContextProviders/LaravelContextProviderDetector.php',
    'Spatie\\LaravelIgnition\\ContextProviders\\LaravelLivewireRequestContextProvider' => $vendorDir . '/spatie/laravel-ignition/src/ContextProviders/LaravelLivewireRequestContextProvider.php',
    'Spatie\\LaravelIgnition\\ContextProviders\\LaravelRequestContextProvider' => $vendorDir . '/spatie/laravel-ignition/src/ContextProviders/LaravelRequestContextProvider.php',
    'Spatie\\LaravelIgnition\\Exceptions\\CannotExecuteSolutionForNonLocalIp' => $vendorDir . '/spatie/laravel-ignition/src/Exceptions/CannotExecuteSolutionForNonLocalIp.php',
    'Spatie\\LaravelIgnition\\Exceptions\\InvalidConfig' => $vendorDir . '/spatie/laravel-ignition/src/Exceptions/InvalidConfig.php',
    'Spatie\\LaravelIgnition\\Exceptions\\ViewException' => $vendorDir . '/spatie/laravel-ignition/src/Exceptions/ViewException.php',
    'Spatie\\LaravelIgnition\\Exceptions\\ViewExceptionWithSolution' => $vendorDir . '/spatie/laravel-ignition/src/Exceptions/ViewExceptionWithSolution.php',
    'Spatie\\LaravelIgnition\\Facades\\Flare' => $vendorDir . '/spatie/laravel-ignition/src/Facades/Flare.php',
    'Spatie\\LaravelIgnition\\FlareMiddleware\\AddContext' => $vendorDir . '/spatie/laravel-ignition/src/FlareMiddleware/AddContext.php',
    'Spatie\\LaravelIgnition\\FlareMiddleware\\AddDumps' => $vendorDir . '/spatie/laravel-ignition/src/FlareMiddleware/AddDumps.php',
    'Spatie\\LaravelIgnition\\FlareMiddleware\\AddEnvironmentInformation' => $vendorDir . '/spatie/laravel-ignition/src/FlareMiddleware/AddEnvironmentInformation.php',
    'Spatie\\LaravelIgnition\\FlareMiddleware\\AddExceptionHandledStatus' => $vendorDir . '/spatie/laravel-ignition/src/FlareMiddleware/AddExceptionHandledStatus.php',
    'Spatie\\LaravelIgnition\\FlareMiddleware\\AddExceptionInformation' => $vendorDir . '/spatie/laravel-ignition/src/FlareMiddleware/AddExceptionInformation.php',
    'Spatie\\LaravelIgnition\\FlareMiddleware\\AddJobs' => $vendorDir . '/spatie/laravel-ignition/src/FlareMiddleware/AddJobs.php',
    'Spatie\\LaravelIgnition\\FlareMiddleware\\AddLogs' => $vendorDir . '/spatie/laravel-ignition/src/FlareMiddleware/AddLogs.php',
    'Spatie\\LaravelIgnition\\FlareMiddleware\\AddNotifierName' => $vendorDir . '/spatie/laravel-ignition/src/FlareMiddleware/AddNotifierName.php',
    'Spatie\\LaravelIgnition\\FlareMiddleware\\AddQueries' => $vendorDir . '/spatie/laravel-ignition/src/FlareMiddleware/AddQueries.php',
    'Spatie\\LaravelIgnition\\Http\\Controllers\\ExecuteSolutionController' => $vendorDir . '/spatie/laravel-ignition/src/Http/Controllers/ExecuteSolutionController.php',
    'Spatie\\LaravelIgnition\\Http\\Controllers\\HealthCheckController' => $vendorDir . '/spatie/laravel-ignition/src/Http/Controllers/HealthCheckController.php',
    'Spatie\\LaravelIgnition\\Http\\Controllers\\UpdateConfigController' => $vendorDir . '/spatie/laravel-ignition/src/Http/Controllers/UpdateConfigController.php',
    'Spatie\\LaravelIgnition\\Http\\Middleware\\RunnableSolutionsEnabled' => $vendorDir . '/spatie/laravel-ignition/src/Http/Middleware/RunnableSolutionsEnabled.php',
    'Spatie\\LaravelIgnition\\Http\\Requests\\ExecuteSolutionRequest' => $vendorDir . '/spatie/laravel-ignition/src/Http/Requests/ExecuteSolutionRequest.php',
    'Spatie\\LaravelIgnition\\Http\\Requests\\UpdateConfigRequest' => $vendorDir . '/spatie/laravel-ignition/src/Http/Requests/UpdateConfigRequest.php',
    'Spatie\\LaravelIgnition\\IgnitionServiceProvider' => $vendorDir . '/spatie/laravel-ignition/src/IgnitionServiceProvider.php',
    'Spatie\\LaravelIgnition\\Recorders\\DumpRecorder\\Dump' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/DumpRecorder/Dump.php',
    'Spatie\\LaravelIgnition\\Recorders\\DumpRecorder\\DumpHandler' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/DumpRecorder/DumpHandler.php',
    'Spatie\\LaravelIgnition\\Recorders\\DumpRecorder\\DumpRecorder' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/DumpRecorder/DumpRecorder.php',
    'Spatie\\LaravelIgnition\\Recorders\\DumpRecorder\\HtmlDumper' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/DumpRecorder/HtmlDumper.php',
    'Spatie\\LaravelIgnition\\Recorders\\DumpRecorder\\MultiDumpHandler' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/DumpRecorder/MultiDumpHandler.php',
    'Spatie\\LaravelIgnition\\Recorders\\JobRecorder\\JobRecorder' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/JobRecorder/JobRecorder.php',
    'Spatie\\LaravelIgnition\\Recorders\\LogRecorder\\LogMessage' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/LogRecorder/LogMessage.php',
    'Spatie\\LaravelIgnition\\Recorders\\LogRecorder\\LogRecorder' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/LogRecorder/LogRecorder.php',
    'Spatie\\LaravelIgnition\\Recorders\\QueryRecorder\\Query' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/QueryRecorder/Query.php',
    'Spatie\\LaravelIgnition\\Recorders\\QueryRecorder\\QueryRecorder' => $vendorDir . '/spatie/laravel-ignition/src/Recorders/QueryRecorder/QueryRecorder.php',
    'Spatie\\LaravelIgnition\\Renderers\\ErrorPageRenderer' => $vendorDir . '/spatie/laravel-ignition/src/Renderers/ErrorPageRenderer.php',
    'Spatie\\LaravelIgnition\\Renderers\\IgnitionExceptionRenderer' => $vendorDir . '/spatie/laravel-ignition/src/Renderers/IgnitionExceptionRenderer.php',
    'Spatie\\LaravelIgnition\\Solutions\\GenerateAppKeySolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/GenerateAppKeySolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\LivewireDiscoverSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/LivewireDiscoverSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\MakeViewVariableOptionalSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/MakeViewVariableOptionalSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\RunMigrationsSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/RunMigrationsSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\DefaultDbNameSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/DefaultDbNameSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\GenericLaravelExceptionSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/GenericLaravelExceptionSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\IncorrectValetDbCredentialsSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/IncorrectValetDbCredentialsSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\InvalidRouteActionSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/InvalidRouteActionSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\LazyLoadingViolationSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/LazyLoadingViolationSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\MissingAppKeySolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/MissingAppKeySolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\MissingColumnSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/MissingColumnSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\MissingImportSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/MissingImportSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\MissingLivewireComponentSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/MissingLivewireComponentSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\MissingMixManifestSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/MissingMixManifestSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\MissingViteManifestSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/MissingViteManifestSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\OpenAiSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/OpenAiSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\RouteNotDefinedSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/RouteNotDefinedSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\RunningLaravelDuskInProductionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/RunningLaravelDuskInProductionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\SailNetworkSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/SailNetworkSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\SolutionProviderRepository' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/SolutionProviderRepository.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\TableNotFoundSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/TableNotFoundSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\UndefinedLivewireMethodSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/UndefinedLivewireMethodSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\UndefinedLivewirePropertySolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/UndefinedLivewirePropertySolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\UndefinedViewVariableSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/UndefinedViewVariableSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\UnknownMariadbCollationSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/UnknownMariadbCollationSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\UnknownMysql8CollationSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/UnknownMysql8CollationSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\UnknownValidationSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/UnknownValidationSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionProviders\\ViewNotFoundSolutionProvider' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SolutionProviders/ViewNotFoundSolutionProvider.php',
    'Spatie\\LaravelIgnition\\Solutions\\SolutionTransformers\\LaravelSolutionTransformer' => $vendorDir . '/spatie/laravel-ignition/src/Solutions/SolutionTransformers/LaravelSolutionTransformer.php',
    'Spatie\\LaravelIgnition\\Solutions\\SuggestCorrectVariableNameSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SuggestCorrectVariableNameSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\SuggestImportSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SuggestImportSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\SuggestLivewireMethodNameSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SuggestLivewireMethodNameSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\SuggestLivewirePropertyNameSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SuggestLivewirePropertyNameSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\SuggestUsingCorrectDbNameSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SuggestUsingCorrectDbNameSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\SuggestUsingMariadbDatabaseSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SuggestUsingMariadbDatabaseSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\SuggestUsingMysql8DatabaseSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/SuggestUsingMysql8DatabaseSolution.php',
    'Spatie\\LaravelIgnition\\Solutions\\UseDefaultValetDbCredentialsSolution' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Solutions/UseDefaultValetDbCredentialsSolution.php',
    'Spatie\\LaravelIgnition\\Support\\FlareLogHandler' => $vendorDir . '/spatie/laravel-ignition/src/Support/FlareLogHandler.php',
    'Spatie\\LaravelIgnition\\Support\\LaravelDocumentationLinkFinder' => $vendorDir . '/spatie/laravel-ignition/src/Support/LaravelDocumentationLinkFinder.php',
    'Spatie\\LaravelIgnition\\Support\\LaravelVersion' => $vendorDir . '/spatie/laravel-ignition/src/Support/LaravelVersion.php',
    'Spatie\\LaravelIgnition\\Support\\RunnableSolutionsGuard' => $vendorDir . '/spatie/laravel-ignition/src/Support/RunnableSolutionsGuard.php',
    'Spatie\\LaravelIgnition\\Support\\SentReports' => $vendorDir . '/spatie/laravel-ignition/src/Support/SentReports.php',
    'Spatie\\LaravelIgnition\\Support\\StringComparator' => $vendorDir . '/spatie/error-solutions/legacy/laravel-ignition/Support/StringComparator.php',
    'Spatie\\LaravelIgnition\\Views\\BladeSourceMapCompiler' => $vendorDir . '/spatie/laravel-ignition/src/Views/BladeSourceMapCompiler.php',
    'Spatie\\LaravelIgnition\\Views\\ViewExceptionMapper' => $vendorDir . '/spatie/laravel-ignition/src/Views/ViewExceptionMapper.php',
    'Spatie\\Permission\\Commands\\CacheReset' => $vendorDir . '/spatie/laravel-permission/src/Commands/CacheReset.php',
    'Spatie\\Permission\\Commands\\CreatePermission' => $vendorDir . '/spatie/laravel-permission/src/Commands/CreatePermission.php',
    'Spatie\\Permission\\Commands\\CreateRole' => $vendorDir . '/spatie/laravel-permission/src/Commands/CreateRole.php',
    'Spatie\\Permission\\Commands\\Show' => $vendorDir . '/spatie/laravel-permission/src/Commands/Show.php',
    'Spatie\\Permission\\Commands\\UpgradeForTeams' => $vendorDir . '/spatie/laravel-permission/src/Commands/UpgradeForTeams.php',
    'Spatie\\Permission\\Contracts\\Permission' => $vendorDir . '/spatie/laravel-permission/src/Contracts/Permission.php',
    'Spatie\\Permission\\Contracts\\Role' => $vendorDir . '/spatie/laravel-permission/src/Contracts/Role.php',
    'Spatie\\Permission\\Contracts\\Wildcard' => $vendorDir . '/spatie/laravel-permission/src/Contracts/Wildcard.php',
    'Spatie\\Permission\\Exceptions\\GuardDoesNotMatch' => $vendorDir . '/spatie/laravel-permission/src/Exceptions/GuardDoesNotMatch.php',
    'Spatie\\Permission\\Exceptions\\PermissionAlreadyExists' => $vendorDir . '/spatie/laravel-permission/src/Exceptions/PermissionAlreadyExists.php',
    'Spatie\\Permission\\Exceptions\\PermissionDoesNotExist' => $vendorDir . '/spatie/laravel-permission/src/Exceptions/PermissionDoesNotExist.php',
    'Spatie\\Permission\\Exceptions\\RoleAlreadyExists' => $vendorDir . '/spatie/laravel-permission/src/Exceptions/RoleAlreadyExists.php',
    'Spatie\\Permission\\Exceptions\\RoleDoesNotExist' => $vendorDir . '/spatie/laravel-permission/src/Exceptions/RoleDoesNotExist.php',
    'Spatie\\Permission\\Exceptions\\UnauthorizedException' => $vendorDir . '/spatie/laravel-permission/src/Exceptions/UnauthorizedException.php',
    'Spatie\\Permission\\Exceptions\\WildcardPermissionInvalidArgument' => $vendorDir . '/spatie/laravel-permission/src/Exceptions/WildcardPermissionInvalidArgument.php',
    'Spatie\\Permission\\Exceptions\\WildcardPermissionNotImplementsContract' => $vendorDir . '/spatie/laravel-permission/src/Exceptions/WildcardPermissionNotImplementsContract.php',
    'Spatie\\Permission\\Exceptions\\WildcardPermissionNotProperlyFormatted' => $vendorDir . '/spatie/laravel-permission/src/Exceptions/WildcardPermissionNotProperlyFormatted.php',
    'Spatie\\Permission\\Guard' => $vendorDir . '/spatie/laravel-permission/src/Guard.php',
    'Spatie\\Permission\\Middleware\\PermissionMiddleware' => $vendorDir . '/spatie/laravel-permission/src/Middleware/PermissionMiddleware.php',
    'Spatie\\Permission\\Middleware\\RoleMiddleware' => $vendorDir . '/spatie/laravel-permission/src/Middleware/RoleMiddleware.php',
    'Spatie\\Permission\\Middleware\\RoleOrPermissionMiddleware' => $vendorDir . '/spatie/laravel-permission/src/Middleware/RoleOrPermissionMiddleware.php',
    'Spatie\\Permission\\Models\\Permission' => $vendorDir . '/spatie/laravel-permission/src/Models/Permission.php',
    'Spatie\\Permission\\Models\\Role' => $vendorDir . '/spatie/laravel-permission/src/Models/Role.php',
    'Spatie\\Permission\\PermissionRegistrar' => $vendorDir . '/spatie/laravel-permission/src/PermissionRegistrar.php',
    'Spatie\\Permission\\PermissionServiceProvider' => $vendorDir . '/spatie/laravel-permission/src/PermissionServiceProvider.php',
    'Spatie\\Permission\\Traits\\HasPermissions' => $vendorDir . '/spatie/laravel-permission/src/Traits/HasPermissions.php',
    'Spatie\\Permission\\Traits\\HasRoles' => $vendorDir . '/spatie/laravel-permission/src/Traits/HasRoles.php',
    'Spatie\\Permission\\Traits\\RefreshesPermissionCache' => $vendorDir . '/spatie/laravel-permission/src/Traits/RefreshesPermissionCache.php',
    'Spatie\\Permission\\WildcardPermission' => $vendorDir . '/spatie/laravel-permission/src/WildcardPermission.php',
    'StellaMaris\\Clock\\ClockInterface' => $vendorDir . '/stella-maris/clock/src/ClockInterface.php',
    'Stringable' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/Stringable.php',
    'Stripe\\Account' => $vendorDir . '/stripe/stripe-php/lib/Account.php',
    'Stripe\\AccountLink' => $vendorDir . '/stripe/stripe-php/lib/AccountLink.php',
    'Stripe\\AlipayAccount' => $vendorDir . '/stripe/stripe-php/lib/AlipayAccount.php',
    'Stripe\\ApiOperations\\All' => $vendorDir . '/stripe/stripe-php/lib/ApiOperations/All.php',
    'Stripe\\ApiOperations\\Create' => $vendorDir . '/stripe/stripe-php/lib/ApiOperations/Create.php',
    'Stripe\\ApiOperations\\Delete' => $vendorDir . '/stripe/stripe-php/lib/ApiOperations/Delete.php',
    'Stripe\\ApiOperations\\NestedResource' => $vendorDir . '/stripe/stripe-php/lib/ApiOperations/NestedResource.php',
    'Stripe\\ApiOperations\\Request' => $vendorDir . '/stripe/stripe-php/lib/ApiOperations/Request.php',
    'Stripe\\ApiOperations\\Retrieve' => $vendorDir . '/stripe/stripe-php/lib/ApiOperations/Retrieve.php',
    'Stripe\\ApiOperations\\Search' => $vendorDir . '/stripe/stripe-php/lib/ApiOperations/Search.php',
    'Stripe\\ApiOperations\\Update' => $vendorDir . '/stripe/stripe-php/lib/ApiOperations/Update.php',
    'Stripe\\ApiRequestor' => $vendorDir . '/stripe/stripe-php/lib/ApiRequestor.php',
    'Stripe\\ApiResource' => $vendorDir . '/stripe/stripe-php/lib/ApiResource.php',
    'Stripe\\ApiResponse' => $vendorDir . '/stripe/stripe-php/lib/ApiResponse.php',
    'Stripe\\ApplePayDomain' => $vendorDir . '/stripe/stripe-php/lib/ApplePayDomain.php',
    'Stripe\\ApplicationFee' => $vendorDir . '/stripe/stripe-php/lib/ApplicationFee.php',
    'Stripe\\ApplicationFeeRefund' => $vendorDir . '/stripe/stripe-php/lib/ApplicationFeeRefund.php',
    'Stripe\\Balance' => $vendorDir . '/stripe/stripe-php/lib/Balance.php',
    'Stripe\\BalanceTransaction' => $vendorDir . '/stripe/stripe-php/lib/BalanceTransaction.php',
    'Stripe\\BankAccount' => $vendorDir . '/stripe/stripe-php/lib/BankAccount.php',
    'Stripe\\BaseStripeClient' => $vendorDir . '/stripe/stripe-php/lib/BaseStripeClient.php',
    'Stripe\\BaseStripeClientInterface' => $vendorDir . '/stripe/stripe-php/lib/BaseStripeClientInterface.php',
    'Stripe\\BillingPortal\\Configuration' => $vendorDir . '/stripe/stripe-php/lib/BillingPortal/Configuration.php',
    'Stripe\\BillingPortal\\Session' => $vendorDir . '/stripe/stripe-php/lib/BillingPortal/Session.php',
    'Stripe\\BitcoinReceiver' => $vendorDir . '/stripe/stripe-php/lib/BitcoinReceiver.php',
    'Stripe\\BitcoinTransaction' => $vendorDir . '/stripe/stripe-php/lib/BitcoinTransaction.php',
    'Stripe\\Capability' => $vendorDir . '/stripe/stripe-php/lib/Capability.php',
    'Stripe\\Card' => $vendorDir . '/stripe/stripe-php/lib/Card.php',
    'Stripe\\CashBalance' => $vendorDir . '/stripe/stripe-php/lib/CashBalance.php',
    'Stripe\\Charge' => $vendorDir . '/stripe/stripe-php/lib/Charge.php',
    'Stripe\\Checkout\\Session' => $vendorDir . '/stripe/stripe-php/lib/Checkout/Session.php',
    'Stripe\\Collection' => $vendorDir . '/stripe/stripe-php/lib/Collection.php',
    'Stripe\\CountrySpec' => $vendorDir . '/stripe/stripe-php/lib/CountrySpec.php',
    'Stripe\\Coupon' => $vendorDir . '/stripe/stripe-php/lib/Coupon.php',
    'Stripe\\CreditNote' => $vendorDir . '/stripe/stripe-php/lib/CreditNote.php',
    'Stripe\\CreditNoteLineItem' => $vendorDir . '/stripe/stripe-php/lib/CreditNoteLineItem.php',
    'Stripe\\Customer' => $vendorDir . '/stripe/stripe-php/lib/Customer.php',
    'Stripe\\CustomerBalanceTransaction' => $vendorDir . '/stripe/stripe-php/lib/CustomerBalanceTransaction.php',
    'Stripe\\Discount' => $vendorDir . '/stripe/stripe-php/lib/Discount.php',
    'Stripe\\Dispute' => $vendorDir . '/stripe/stripe-php/lib/Dispute.php',
    'Stripe\\EphemeralKey' => $vendorDir . '/stripe/stripe-php/lib/EphemeralKey.php',
    'Stripe\\ErrorObject' => $vendorDir . '/stripe/stripe-php/lib/ErrorObject.php',
    'Stripe\\Event' => $vendorDir . '/stripe/stripe-php/lib/Event.php',
    'Stripe\\Exception\\ApiConnectionException' => $vendorDir . '/stripe/stripe-php/lib/Exception/ApiConnectionException.php',
    'Stripe\\Exception\\ApiErrorException' => $vendorDir . '/stripe/stripe-php/lib/Exception/ApiErrorException.php',
    'Stripe\\Exception\\AuthenticationException' => $vendorDir . '/stripe/stripe-php/lib/Exception/AuthenticationException.php',
    'Stripe\\Exception\\BadMethodCallException' => $vendorDir . '/stripe/stripe-php/lib/Exception/BadMethodCallException.php',
    'Stripe\\Exception\\CardException' => $vendorDir . '/stripe/stripe-php/lib/Exception/CardException.php',
    'Stripe\\Exception\\ExceptionInterface' => $vendorDir . '/stripe/stripe-php/lib/Exception/ExceptionInterface.php',
    'Stripe\\Exception\\IdempotencyException' => $vendorDir . '/stripe/stripe-php/lib/Exception/IdempotencyException.php',
    'Stripe\\Exception\\InvalidArgumentException' => $vendorDir . '/stripe/stripe-php/lib/Exception/InvalidArgumentException.php',
    'Stripe\\Exception\\InvalidRequestException' => $vendorDir . '/stripe/stripe-php/lib/Exception/InvalidRequestException.php',
    'Stripe\\Exception\\OAuth\\ExceptionInterface' => $vendorDir . '/stripe/stripe-php/lib/Exception/OAuth/ExceptionInterface.php',
    'Stripe\\Exception\\OAuth\\InvalidClientException' => $vendorDir . '/stripe/stripe-php/lib/Exception/OAuth/InvalidClientException.php',
    'Stripe\\Exception\\OAuth\\InvalidGrantException' => $vendorDir . '/stripe/stripe-php/lib/Exception/OAuth/InvalidGrantException.php',
    'Stripe\\Exception\\OAuth\\InvalidRequestException' => $vendorDir . '/stripe/stripe-php/lib/Exception/OAuth/InvalidRequestException.php',
    'Stripe\\Exception\\OAuth\\InvalidScopeException' => $vendorDir . '/stripe/stripe-php/lib/Exception/OAuth/InvalidScopeException.php',
    'Stripe\\Exception\\OAuth\\OAuthErrorException' => $vendorDir . '/stripe/stripe-php/lib/Exception/OAuth/OAuthErrorException.php',
    'Stripe\\Exception\\OAuth\\UnknownOAuthErrorException' => $vendorDir . '/stripe/stripe-php/lib/Exception/OAuth/UnknownOAuthErrorException.php',
    'Stripe\\Exception\\OAuth\\UnsupportedGrantTypeException' => $vendorDir . '/stripe/stripe-php/lib/Exception/OAuth/UnsupportedGrantTypeException.php',
    'Stripe\\Exception\\OAuth\\UnsupportedResponseTypeException' => $vendorDir . '/stripe/stripe-php/lib/Exception/OAuth/UnsupportedResponseTypeException.php',
    'Stripe\\Exception\\PermissionException' => $vendorDir . '/stripe/stripe-php/lib/Exception/PermissionException.php',
    'Stripe\\Exception\\RateLimitException' => $vendorDir . '/stripe/stripe-php/lib/Exception/RateLimitException.php',
    'Stripe\\Exception\\SignatureVerificationException' => $vendorDir . '/stripe/stripe-php/lib/Exception/SignatureVerificationException.php',
    'Stripe\\Exception\\UnexpectedValueException' => $vendorDir . '/stripe/stripe-php/lib/Exception/UnexpectedValueException.php',
    'Stripe\\Exception\\UnknownApiErrorException' => $vendorDir . '/stripe/stripe-php/lib/Exception/UnknownApiErrorException.php',
    'Stripe\\ExchangeRate' => $vendorDir . '/stripe/stripe-php/lib/ExchangeRate.php',
    'Stripe\\File' => $vendorDir . '/stripe/stripe-php/lib/File.php',
    'Stripe\\FileLink' => $vendorDir . '/stripe/stripe-php/lib/FileLink.php',
    'Stripe\\FinancialConnections\\Account' => $vendorDir . '/stripe/stripe-php/lib/FinancialConnections/Account.php',
    'Stripe\\FinancialConnections\\AccountOwner' => $vendorDir . '/stripe/stripe-php/lib/FinancialConnections/AccountOwner.php',
    'Stripe\\FinancialConnections\\AccountOwnership' => $vendorDir . '/stripe/stripe-php/lib/FinancialConnections/AccountOwnership.php',
    'Stripe\\FinancialConnections\\Session' => $vendorDir . '/stripe/stripe-php/lib/FinancialConnections/Session.php',
    'Stripe\\FundingInstructions' => $vendorDir . '/stripe/stripe-php/lib/FundingInstructions.php',
    'Stripe\\HttpClient\\ClientInterface' => $vendorDir . '/stripe/stripe-php/lib/HttpClient/ClientInterface.php',
    'Stripe\\HttpClient\\CurlClient' => $vendorDir . '/stripe/stripe-php/lib/HttpClient/CurlClient.php',
    'Stripe\\HttpClient\\StreamingClientInterface' => $vendorDir . '/stripe/stripe-php/lib/HttpClient/StreamingClientInterface.php',
    'Stripe\\Identity\\VerificationReport' => $vendorDir . '/stripe/stripe-php/lib/Identity/VerificationReport.php',
    'Stripe\\Identity\\VerificationSession' => $vendorDir . '/stripe/stripe-php/lib/Identity/VerificationSession.php',
    'Stripe\\Invoice' => $vendorDir . '/stripe/stripe-php/lib/Invoice.php',
    'Stripe\\InvoiceItem' => $vendorDir . '/stripe/stripe-php/lib/InvoiceItem.php',
    'Stripe\\InvoiceLineItem' => $vendorDir . '/stripe/stripe-php/lib/InvoiceLineItem.php',
    'Stripe\\Issuing\\Authorization' => $vendorDir . '/stripe/stripe-php/lib/Issuing/Authorization.php',
    'Stripe\\Issuing\\Card' => $vendorDir . '/stripe/stripe-php/lib/Issuing/Card.php',
    'Stripe\\Issuing\\CardDetails' => $vendorDir . '/stripe/stripe-php/lib/Issuing/CardDetails.php',
    'Stripe\\Issuing\\Cardholder' => $vendorDir . '/stripe/stripe-php/lib/Issuing/Cardholder.php',
    'Stripe\\Issuing\\Dispute' => $vendorDir . '/stripe/stripe-php/lib/Issuing/Dispute.php',
    'Stripe\\Issuing\\Transaction' => $vendorDir . '/stripe/stripe-php/lib/Issuing/Transaction.php',
    'Stripe\\LineItem' => $vendorDir . '/stripe/stripe-php/lib/LineItem.php',
    'Stripe\\LoginLink' => $vendorDir . '/stripe/stripe-php/lib/LoginLink.php',
    'Stripe\\Mandate' => $vendorDir . '/stripe/stripe-php/lib/Mandate.php',
    'Stripe\\OAuth' => $vendorDir . '/stripe/stripe-php/lib/OAuth.php',
    'Stripe\\OAuthErrorObject' => $vendorDir . '/stripe/stripe-php/lib/OAuthErrorObject.php',
    'Stripe\\Order' => $vendorDir . '/stripe/stripe-php/lib/Order.php',
    'Stripe\\OrderItem' => $vendorDir . '/stripe/stripe-php/lib/OrderItem.php',
    'Stripe\\OrderReturn' => $vendorDir . '/stripe/stripe-php/lib/OrderReturn.php',
    'Stripe\\PaymentIntent' => $vendorDir . '/stripe/stripe-php/lib/PaymentIntent.php',
    'Stripe\\PaymentLink' => $vendorDir . '/stripe/stripe-php/lib/PaymentLink.php',
    'Stripe\\PaymentMethod' => $vendorDir . '/stripe/stripe-php/lib/PaymentMethod.php',
    'Stripe\\Payout' => $vendorDir . '/stripe/stripe-php/lib/Payout.php',
    'Stripe\\Person' => $vendorDir . '/stripe/stripe-php/lib/Person.php',
    'Stripe\\Plan' => $vendorDir . '/stripe/stripe-php/lib/Plan.php',
    'Stripe\\Price' => $vendorDir . '/stripe/stripe-php/lib/Price.php',
    'Stripe\\Product' => $vendorDir . '/stripe/stripe-php/lib/Product.php',
    'Stripe\\PromotionCode' => $vendorDir . '/stripe/stripe-php/lib/PromotionCode.php',
    'Stripe\\Quote' => $vendorDir . '/stripe/stripe-php/lib/Quote.php',
    'Stripe\\Radar\\EarlyFraudWarning' => $vendorDir . '/stripe/stripe-php/lib/Radar/EarlyFraudWarning.php',
    'Stripe\\Radar\\ValueList' => $vendorDir . '/stripe/stripe-php/lib/Radar/ValueList.php',
    'Stripe\\Radar\\ValueListItem' => $vendorDir . '/stripe/stripe-php/lib/Radar/ValueListItem.php',
    'Stripe\\Recipient' => $vendorDir . '/stripe/stripe-php/lib/Recipient.php',
    'Stripe\\RecipientTransfer' => $vendorDir . '/stripe/stripe-php/lib/RecipientTransfer.php',
    'Stripe\\Refund' => $vendorDir . '/stripe/stripe-php/lib/Refund.php',
    'Stripe\\Reporting\\ReportRun' => $vendorDir . '/stripe/stripe-php/lib/Reporting/ReportRun.php',
    'Stripe\\Reporting\\ReportType' => $vendorDir . '/stripe/stripe-php/lib/Reporting/ReportType.php',
    'Stripe\\RequestTelemetry' => $vendorDir . '/stripe/stripe-php/lib/RequestTelemetry.php',
    'Stripe\\Review' => $vendorDir . '/stripe/stripe-php/lib/Review.php',
    'Stripe\\SKU' => $vendorDir . '/stripe/stripe-php/lib/SKU.php',
    'Stripe\\SearchResult' => $vendorDir . '/stripe/stripe-php/lib/SearchResult.php',
    'Stripe\\Service\\AbstractService' => $vendorDir . '/stripe/stripe-php/lib/Service/AbstractService.php',
    'Stripe\\Service\\AbstractServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/AbstractServiceFactory.php',
    'Stripe\\Service\\AccountLinkService' => $vendorDir . '/stripe/stripe-php/lib/Service/AccountLinkService.php',
    'Stripe\\Service\\AccountService' => $vendorDir . '/stripe/stripe-php/lib/Service/AccountService.php',
    'Stripe\\Service\\ApplePayDomainService' => $vendorDir . '/stripe/stripe-php/lib/Service/ApplePayDomainService.php',
    'Stripe\\Service\\ApplicationFeeService' => $vendorDir . '/stripe/stripe-php/lib/Service/ApplicationFeeService.php',
    'Stripe\\Service\\BalanceService' => $vendorDir . '/stripe/stripe-php/lib/Service/BalanceService.php',
    'Stripe\\Service\\BalanceTransactionService' => $vendorDir . '/stripe/stripe-php/lib/Service/BalanceTransactionService.php',
    'Stripe\\Service\\BillingPortal\\BillingPortalServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/BillingPortal/BillingPortalServiceFactory.php',
    'Stripe\\Service\\BillingPortal\\ConfigurationService' => $vendorDir . '/stripe/stripe-php/lib/Service/BillingPortal/ConfigurationService.php',
    'Stripe\\Service\\BillingPortal\\SessionService' => $vendorDir . '/stripe/stripe-php/lib/Service/BillingPortal/SessionService.php',
    'Stripe\\Service\\ChargeService' => $vendorDir . '/stripe/stripe-php/lib/Service/ChargeService.php',
    'Stripe\\Service\\Checkout\\CheckoutServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/Checkout/CheckoutServiceFactory.php',
    'Stripe\\Service\\Checkout\\SessionService' => $vendorDir . '/stripe/stripe-php/lib/Service/Checkout/SessionService.php',
    'Stripe\\Service\\CoreServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/CoreServiceFactory.php',
    'Stripe\\Service\\CountrySpecService' => $vendorDir . '/stripe/stripe-php/lib/Service/CountrySpecService.php',
    'Stripe\\Service\\CouponService' => $vendorDir . '/stripe/stripe-php/lib/Service/CouponService.php',
    'Stripe\\Service\\CreditNoteService' => $vendorDir . '/stripe/stripe-php/lib/Service/CreditNoteService.php',
    'Stripe\\Service\\CustomerService' => $vendorDir . '/stripe/stripe-php/lib/Service/CustomerService.php',
    'Stripe\\Service\\DisputeService' => $vendorDir . '/stripe/stripe-php/lib/Service/DisputeService.php',
    'Stripe\\Service\\EphemeralKeyService' => $vendorDir . '/stripe/stripe-php/lib/Service/EphemeralKeyService.php',
    'Stripe\\Service\\EventService' => $vendorDir . '/stripe/stripe-php/lib/Service/EventService.php',
    'Stripe\\Service\\ExchangeRateService' => $vendorDir . '/stripe/stripe-php/lib/Service/ExchangeRateService.php',
    'Stripe\\Service\\FileLinkService' => $vendorDir . '/stripe/stripe-php/lib/Service/FileLinkService.php',
    'Stripe\\Service\\FileService' => $vendorDir . '/stripe/stripe-php/lib/Service/FileService.php',
    'Stripe\\Service\\FinancialConnections\\AccountService' => $vendorDir . '/stripe/stripe-php/lib/Service/FinancialConnections/AccountService.php',
    'Stripe\\Service\\FinancialConnections\\FinancialConnectionsServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/FinancialConnections/FinancialConnectionsServiceFactory.php',
    'Stripe\\Service\\FinancialConnections\\SessionService' => $vendorDir . '/stripe/stripe-php/lib/Service/FinancialConnections/SessionService.php',
    'Stripe\\Service\\Identity\\IdentityServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/Identity/IdentityServiceFactory.php',
    'Stripe\\Service\\Identity\\VerificationReportService' => $vendorDir . '/stripe/stripe-php/lib/Service/Identity/VerificationReportService.php',
    'Stripe\\Service\\Identity\\VerificationSessionService' => $vendorDir . '/stripe/stripe-php/lib/Service/Identity/VerificationSessionService.php',
    'Stripe\\Service\\InvoiceItemService' => $vendorDir . '/stripe/stripe-php/lib/Service/InvoiceItemService.php',
    'Stripe\\Service\\InvoiceService' => $vendorDir . '/stripe/stripe-php/lib/Service/InvoiceService.php',
    'Stripe\\Service\\Issuing\\AuthorizationService' => $vendorDir . '/stripe/stripe-php/lib/Service/Issuing/AuthorizationService.php',
    'Stripe\\Service\\Issuing\\CardService' => $vendorDir . '/stripe/stripe-php/lib/Service/Issuing/CardService.php',
    'Stripe\\Service\\Issuing\\CardholderService' => $vendorDir . '/stripe/stripe-php/lib/Service/Issuing/CardholderService.php',
    'Stripe\\Service\\Issuing\\DisputeService' => $vendorDir . '/stripe/stripe-php/lib/Service/Issuing/DisputeService.php',
    'Stripe\\Service\\Issuing\\IssuingServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/Issuing/IssuingServiceFactory.php',
    'Stripe\\Service\\Issuing\\TransactionService' => $vendorDir . '/stripe/stripe-php/lib/Service/Issuing/TransactionService.php',
    'Stripe\\Service\\MandateService' => $vendorDir . '/stripe/stripe-php/lib/Service/MandateService.php',
    'Stripe\\Service\\OAuthService' => $vendorDir . '/stripe/stripe-php/lib/Service/OAuthService.php',
    'Stripe\\Service\\OrderReturnService' => $vendorDir . '/stripe/stripe-php/lib/Service/OrderReturnService.php',
    'Stripe\\Service\\OrderService' => $vendorDir . '/stripe/stripe-php/lib/Service/OrderService.php',
    'Stripe\\Service\\PaymentIntentService' => $vendorDir . '/stripe/stripe-php/lib/Service/PaymentIntentService.php',
    'Stripe\\Service\\PaymentLinkService' => $vendorDir . '/stripe/stripe-php/lib/Service/PaymentLinkService.php',
    'Stripe\\Service\\PaymentMethodService' => $vendorDir . '/stripe/stripe-php/lib/Service/PaymentMethodService.php',
    'Stripe\\Service\\PayoutService' => $vendorDir . '/stripe/stripe-php/lib/Service/PayoutService.php',
    'Stripe\\Service\\PlanService' => $vendorDir . '/stripe/stripe-php/lib/Service/PlanService.php',
    'Stripe\\Service\\PriceService' => $vendorDir . '/stripe/stripe-php/lib/Service/PriceService.php',
    'Stripe\\Service\\ProductService' => $vendorDir . '/stripe/stripe-php/lib/Service/ProductService.php',
    'Stripe\\Service\\PromotionCodeService' => $vendorDir . '/stripe/stripe-php/lib/Service/PromotionCodeService.php',
    'Stripe\\Service\\QuoteService' => $vendorDir . '/stripe/stripe-php/lib/Service/QuoteService.php',
    'Stripe\\Service\\Radar\\EarlyFraudWarningService' => $vendorDir . '/stripe/stripe-php/lib/Service/Radar/EarlyFraudWarningService.php',
    'Stripe\\Service\\Radar\\RadarServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/Radar/RadarServiceFactory.php',
    'Stripe\\Service\\Radar\\ValueListItemService' => $vendorDir . '/stripe/stripe-php/lib/Service/Radar/ValueListItemService.php',
    'Stripe\\Service\\Radar\\ValueListService' => $vendorDir . '/stripe/stripe-php/lib/Service/Radar/ValueListService.php',
    'Stripe\\Service\\RefundService' => $vendorDir . '/stripe/stripe-php/lib/Service/RefundService.php',
    'Stripe\\Service\\Reporting\\ReportRunService' => $vendorDir . '/stripe/stripe-php/lib/Service/Reporting/ReportRunService.php',
    'Stripe\\Service\\Reporting\\ReportTypeService' => $vendorDir . '/stripe/stripe-php/lib/Service/Reporting/ReportTypeService.php',
    'Stripe\\Service\\Reporting\\ReportingServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/Reporting/ReportingServiceFactory.php',
    'Stripe\\Service\\ReviewService' => $vendorDir . '/stripe/stripe-php/lib/Service/ReviewService.php',
    'Stripe\\Service\\SetupAttemptService' => $vendorDir . '/stripe/stripe-php/lib/Service/SetupAttemptService.php',
    'Stripe\\Service\\SetupIntentService' => $vendorDir . '/stripe/stripe-php/lib/Service/SetupIntentService.php',
    'Stripe\\Service\\ShippingRateService' => $vendorDir . '/stripe/stripe-php/lib/Service/ShippingRateService.php',
    'Stripe\\Service\\Sigma\\ScheduledQueryRunService' => $vendorDir . '/stripe/stripe-php/lib/Service/Sigma/ScheduledQueryRunService.php',
    'Stripe\\Service\\Sigma\\SigmaServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/Sigma/SigmaServiceFactory.php',
    'Stripe\\Service\\SkuService' => $vendorDir . '/stripe/stripe-php/lib/Service/SkuService.php',
    'Stripe\\Service\\SourceService' => $vendorDir . '/stripe/stripe-php/lib/Service/SourceService.php',
    'Stripe\\Service\\SubscriptionItemService' => $vendorDir . '/stripe/stripe-php/lib/Service/SubscriptionItemService.php',
    'Stripe\\Service\\SubscriptionScheduleService' => $vendorDir . '/stripe/stripe-php/lib/Service/SubscriptionScheduleService.php',
    'Stripe\\Service\\SubscriptionService' => $vendorDir . '/stripe/stripe-php/lib/Service/SubscriptionService.php',
    'Stripe\\Service\\TaxCodeService' => $vendorDir . '/stripe/stripe-php/lib/Service/TaxCodeService.php',
    'Stripe\\Service\\TaxRateService' => $vendorDir . '/stripe/stripe-php/lib/Service/TaxRateService.php',
    'Stripe\\Service\\Terminal\\ConfigurationService' => $vendorDir . '/stripe/stripe-php/lib/Service/Terminal/ConfigurationService.php',
    'Stripe\\Service\\Terminal\\ConnectionTokenService' => $vendorDir . '/stripe/stripe-php/lib/Service/Terminal/ConnectionTokenService.php',
    'Stripe\\Service\\Terminal\\LocationService' => $vendorDir . '/stripe/stripe-php/lib/Service/Terminal/LocationService.php',
    'Stripe\\Service\\Terminal\\ReaderService' => $vendorDir . '/stripe/stripe-php/lib/Service/Terminal/ReaderService.php',
    'Stripe\\Service\\Terminal\\TerminalServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/Terminal/TerminalServiceFactory.php',
    'Stripe\\Service\\TestHelpers\\RefundService' => $vendorDir . '/stripe/stripe-php/lib/Service/TestHelpers/RefundService.php',
    'Stripe\\Service\\TestHelpers\\Terminal\\ReaderService' => $vendorDir . '/stripe/stripe-php/lib/Service/TestHelpers/Terminal/ReaderService.php',
    'Stripe\\Service\\TestHelpers\\Terminal\\TerminalServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/TestHelpers/Terminal/TerminalServiceFactory.php',
    'Stripe\\Service\\TestHelpers\\TestClockService' => $vendorDir . '/stripe/stripe-php/lib/Service/TestHelpers/TestClockService.php',
    'Stripe\\Service\\TestHelpers\\TestHelpersServiceFactory' => $vendorDir . '/stripe/stripe-php/lib/Service/TestHelpers/TestHelpersServiceFactory.php',
    'Stripe\\Service\\TokenService' => $vendorDir . '/stripe/stripe-php/lib/Service/TokenService.php',
    'Stripe\\Service\\TopupService' => $vendorDir . '/stripe/stripe-php/lib/Service/TopupService.php',
    'Stripe\\Service\\TransferService' => $vendorDir . '/stripe/stripe-php/lib/Service/TransferService.php',
    'Stripe\\Service\\WebhookEndpointService' => $vendorDir . '/stripe/stripe-php/lib/Service/WebhookEndpointService.php',
    'Stripe\\SetupAttempt' => $vendorDir . '/stripe/stripe-php/lib/SetupAttempt.php',
    'Stripe\\SetupIntent' => $vendorDir . '/stripe/stripe-php/lib/SetupIntent.php',
    'Stripe\\ShippingRate' => $vendorDir . '/stripe/stripe-php/lib/ShippingRate.php',
    'Stripe\\Sigma\\ScheduledQueryRun' => $vendorDir . '/stripe/stripe-php/lib/Sigma/ScheduledQueryRun.php',
    'Stripe\\SingletonApiResource' => $vendorDir . '/stripe/stripe-php/lib/SingletonApiResource.php',
    'Stripe\\Source' => $vendorDir . '/stripe/stripe-php/lib/Source.php',
    'Stripe\\SourceTransaction' => $vendorDir . '/stripe/stripe-php/lib/SourceTransaction.php',
    'Stripe\\Stripe' => $vendorDir . '/stripe/stripe-php/lib/Stripe.php',
    'Stripe\\StripeClient' => $vendorDir . '/stripe/stripe-php/lib/StripeClient.php',
    'Stripe\\StripeClientInterface' => $vendorDir . '/stripe/stripe-php/lib/StripeClientInterface.php',
    'Stripe\\StripeObject' => $vendorDir . '/stripe/stripe-php/lib/StripeObject.php',
    'Stripe\\StripeStreamingClientInterface' => $vendorDir . '/stripe/stripe-php/lib/StripeStreamingClientInterface.php',
    'Stripe\\Subscription' => $vendorDir . '/stripe/stripe-php/lib/Subscription.php',
    'Stripe\\SubscriptionItem' => $vendorDir . '/stripe/stripe-php/lib/SubscriptionItem.php',
    'Stripe\\SubscriptionSchedule' => $vendorDir . '/stripe/stripe-php/lib/SubscriptionSchedule.php',
    'Stripe\\TaxCode' => $vendorDir . '/stripe/stripe-php/lib/TaxCode.php',
    'Stripe\\TaxId' => $vendorDir . '/stripe/stripe-php/lib/TaxId.php',
    'Stripe\\TaxRate' => $vendorDir . '/stripe/stripe-php/lib/TaxRate.php',
    'Stripe\\Terminal\\Configuration' => $vendorDir . '/stripe/stripe-php/lib/Terminal/Configuration.php',
    'Stripe\\Terminal\\ConnectionToken' => $vendorDir . '/stripe/stripe-php/lib/Terminal/ConnectionToken.php',
    'Stripe\\Terminal\\Location' => $vendorDir . '/stripe/stripe-php/lib/Terminal/Location.php',
    'Stripe\\Terminal\\Reader' => $vendorDir . '/stripe/stripe-php/lib/Terminal/Reader.php',
    'Stripe\\TestHelpers\\TestClock' => $vendorDir . '/stripe/stripe-php/lib/TestHelpers/TestClock.php',
    'Stripe\\ThreeDSecure' => $vendorDir . '/stripe/stripe-php/lib/ThreeDSecure.php',
    'Stripe\\Token' => $vendorDir . '/stripe/stripe-php/lib/Token.php',
    'Stripe\\Topup' => $vendorDir . '/stripe/stripe-php/lib/Topup.php',
    'Stripe\\Transfer' => $vendorDir . '/stripe/stripe-php/lib/Transfer.php',
    'Stripe\\TransferReversal' => $vendorDir . '/stripe/stripe-php/lib/TransferReversal.php',
    'Stripe\\UsageRecord' => $vendorDir . '/stripe/stripe-php/lib/UsageRecord.php',
    'Stripe\\UsageRecordSummary' => $vendorDir . '/stripe/stripe-php/lib/UsageRecordSummary.php',
    'Stripe\\Util\\CaseInsensitiveArray' => $vendorDir . '/stripe/stripe-php/lib/Util/CaseInsensitiveArray.php',
    'Stripe\\Util\\DefaultLogger' => $vendorDir . '/stripe/stripe-php/lib/Util/DefaultLogger.php',
    'Stripe\\Util\\LoggerInterface' => $vendorDir . '/stripe/stripe-php/lib/Util/LoggerInterface.php',
    'Stripe\\Util\\ObjectTypes' => $vendorDir . '/stripe/stripe-php/lib/Util/ObjectTypes.php',
    'Stripe\\Util\\RandomGenerator' => $vendorDir . '/stripe/stripe-php/lib/Util/RandomGenerator.php',
    'Stripe\\Util\\RequestOptions' => $vendorDir . '/stripe/stripe-php/lib/Util/RequestOptions.php',
    'Stripe\\Util\\Set' => $vendorDir . '/stripe/stripe-php/lib/Util/Set.php',
    'Stripe\\Util\\Util' => $vendorDir . '/stripe/stripe-php/lib/Util/Util.php',
    'Stripe\\Webhook' => $vendorDir . '/stripe/stripe-php/lib/Webhook.php',
    'Stripe\\WebhookEndpoint' => $vendorDir . '/stripe/stripe-php/lib/WebhookEndpoint.php',
    'Stripe\\WebhookSignature' => $vendorDir . '/stripe/stripe-php/lib/WebhookSignature.php',
    'Svg\\CssLength' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/CssLength.php',
    'Svg\\DefaultStyle' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/DefaultStyle.php',
    'Svg\\Document' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Document.php',
    'Svg\\Gradient\\Stop' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Gradient/Stop.php',
    'Svg\\Style' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Style.php',
    'Svg\\Surface\\CPdf' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Surface/CPdf.php',
    'Svg\\Surface\\SurfaceCpdf' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Surface/SurfaceCpdf.php',
    'Svg\\Surface\\SurfaceInterface' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Surface/SurfaceInterface.php',
    'Svg\\Surface\\SurfacePDFLib' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Surface/SurfacePDFLib.php',
    'Svg\\Tag\\AbstractTag' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/AbstractTag.php',
    'Svg\\Tag\\Anchor' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Anchor.php',
    'Svg\\Tag\\Circle' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Circle.php',
    'Svg\\Tag\\ClipPath' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/ClipPath.php',
    'Svg\\Tag\\Ellipse' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Ellipse.php',
    'Svg\\Tag\\Group' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Group.php',
    'Svg\\Tag\\Image' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Image.php',
    'Svg\\Tag\\Line' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Line.php',
    'Svg\\Tag\\LinearGradient' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/LinearGradient.php',
    'Svg\\Tag\\Path' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Path.php',
    'Svg\\Tag\\Polygon' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Polygon.php',
    'Svg\\Tag\\Polyline' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Polyline.php',
    'Svg\\Tag\\RadialGradient' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/RadialGradient.php',
    'Svg\\Tag\\Rect' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Rect.php',
    'Svg\\Tag\\Shape' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Shape.php',
    'Svg\\Tag\\Stop' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Stop.php',
    'Svg\\Tag\\StyleTag' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/StyleTag.php',
    'Svg\\Tag\\Symbol' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Symbol.php',
    'Svg\\Tag\\Text' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/Text.php',
    'Svg\\Tag\\UseTag' => $vendorDir . '/dompdf/php-svg-lib/src/Svg/Tag/UseTag.php',
    'Symfony\\Component\\Console\\Application' => $vendorDir . '/symfony/console/Application.php',
    'Symfony\\Component\\Console\\Attribute\\AsCommand' => $vendorDir . '/symfony/console/Attribute/AsCommand.php',
    'Symfony\\Component\\Console\\CI\\GithubActionReporter' => $vendorDir . '/symfony/console/CI/GithubActionReporter.php',
    'Symfony\\Component\\Console\\Color' => $vendorDir . '/symfony/console/Color.php',
    'Symfony\\Component\\Console\\CommandLoader\\CommandLoaderInterface' => $vendorDir . '/symfony/console/CommandLoader/CommandLoaderInterface.php',
    'Symfony\\Component\\Console\\CommandLoader\\ContainerCommandLoader' => $vendorDir . '/symfony/console/CommandLoader/ContainerCommandLoader.php',
    'Symfony\\Component\\Console\\CommandLoader\\FactoryCommandLoader' => $vendorDir . '/symfony/console/CommandLoader/FactoryCommandLoader.php',
    'Symfony\\Component\\Console\\Command\\Command' => $vendorDir . '/symfony/console/Command/Command.php',
    'Symfony\\Component\\Console\\Command\\CompleteCommand' => $vendorDir . '/symfony/console/Command/CompleteCommand.php',
    'Symfony\\Component\\Console\\Command\\DumpCompletionCommand' => $vendorDir . '/symfony/console/Command/DumpCompletionCommand.php',
    'Symfony\\Component\\Console\\Command\\HelpCommand' => $vendorDir . '/symfony/console/Command/HelpCommand.php',
    'Symfony\\Component\\Console\\Command\\LazyCommand' => $vendorDir . '/symfony/console/Command/LazyCommand.php',
    'Symfony\\Component\\Console\\Command\\ListCommand' => $vendorDir . '/symfony/console/Command/ListCommand.php',
    'Symfony\\Component\\Console\\Command\\LockableTrait' => $vendorDir . '/symfony/console/Command/LockableTrait.php',
    'Symfony\\Component\\Console\\Command\\SignalableCommandInterface' => $vendorDir . '/symfony/console/Command/SignalableCommandInterface.php',
    'Symfony\\Component\\Console\\Command\\TraceableCommand' => $vendorDir . '/symfony/console/Command/TraceableCommand.php',
    'Symfony\\Component\\Console\\Completion\\CompletionInput' => $vendorDir . '/symfony/console/Completion/CompletionInput.php',
    'Symfony\\Component\\Console\\Completion\\CompletionSuggestions' => $vendorDir . '/symfony/console/Completion/CompletionSuggestions.php',
    'Symfony\\Component\\Console\\Completion\\Output\\BashCompletionOutput' => $vendorDir . '/symfony/console/Completion/Output/BashCompletionOutput.php',
    'Symfony\\Component\\Console\\Completion\\Output\\CompletionOutputInterface' => $vendorDir . '/symfony/console/Completion/Output/CompletionOutputInterface.php',
    'Symfony\\Component\\Console\\Completion\\Output\\FishCompletionOutput' => $vendorDir . '/symfony/console/Completion/Output/FishCompletionOutput.php',
    'Symfony\\Component\\Console\\Completion\\Output\\ZshCompletionOutput' => $vendorDir . '/symfony/console/Completion/Output/ZshCompletionOutput.php',
    'Symfony\\Component\\Console\\Completion\\Suggestion' => $vendorDir . '/symfony/console/Completion/Suggestion.php',
    'Symfony\\Component\\Console\\ConsoleEvents' => $vendorDir . '/symfony/console/ConsoleEvents.php',
    'Symfony\\Component\\Console\\Cursor' => $vendorDir . '/symfony/console/Cursor.php',
    'Symfony\\Component\\Console\\DataCollector\\CommandDataCollector' => $vendorDir . '/symfony/console/DataCollector/CommandDataCollector.php',
    'Symfony\\Component\\Console\\Debug\\CliRequest' => $vendorDir . '/symfony/console/Debug/CliRequest.php',
    'Symfony\\Component\\Console\\DependencyInjection\\AddConsoleCommandPass' => $vendorDir . '/symfony/console/DependencyInjection/AddConsoleCommandPass.php',
    'Symfony\\Component\\Console\\Descriptor\\ApplicationDescription' => $vendorDir . '/symfony/console/Descriptor/ApplicationDescription.php',
    'Symfony\\Component\\Console\\Descriptor\\Descriptor' => $vendorDir . '/symfony/console/Descriptor/Descriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\DescriptorInterface' => $vendorDir . '/symfony/console/Descriptor/DescriptorInterface.php',
    'Symfony\\Component\\Console\\Descriptor\\JsonDescriptor' => $vendorDir . '/symfony/console/Descriptor/JsonDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\MarkdownDescriptor' => $vendorDir . '/symfony/console/Descriptor/MarkdownDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\ReStructuredTextDescriptor' => $vendorDir . '/symfony/console/Descriptor/ReStructuredTextDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\TextDescriptor' => $vendorDir . '/symfony/console/Descriptor/TextDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\XmlDescriptor' => $vendorDir . '/symfony/console/Descriptor/XmlDescriptor.php',
    'Symfony\\Component\\Console\\EventListener\\ErrorListener' => $vendorDir . '/symfony/console/EventListener/ErrorListener.php',
    'Symfony\\Component\\Console\\Event\\ConsoleCommandEvent' => $vendorDir . '/symfony/console/Event/ConsoleCommandEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleErrorEvent' => $vendorDir . '/symfony/console/Event/ConsoleErrorEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleEvent' => $vendorDir . '/symfony/console/Event/ConsoleEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleSignalEvent' => $vendorDir . '/symfony/console/Event/ConsoleSignalEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleTerminateEvent' => $vendorDir . '/symfony/console/Event/ConsoleTerminateEvent.php',
    'Symfony\\Component\\Console\\Exception\\CommandNotFoundException' => $vendorDir . '/symfony/console/Exception/CommandNotFoundException.php',
    'Symfony\\Component\\Console\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/console/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Console\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/console/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Console\\Exception\\InvalidOptionException' => $vendorDir . '/symfony/console/Exception/InvalidOptionException.php',
    'Symfony\\Component\\Console\\Exception\\LogicException' => $vendorDir . '/symfony/console/Exception/LogicException.php',
    'Symfony\\Component\\Console\\Exception\\MissingInputException' => $vendorDir . '/symfony/console/Exception/MissingInputException.php',
    'Symfony\\Component\\Console\\Exception\\NamespaceNotFoundException' => $vendorDir . '/symfony/console/Exception/NamespaceNotFoundException.php',
    'Symfony\\Component\\Console\\Exception\\RunCommandFailedException' => $vendorDir . '/symfony/console/Exception/RunCommandFailedException.php',
    'Symfony\\Component\\Console\\Exception\\RuntimeException' => $vendorDir . '/symfony/console/Exception/RuntimeException.php',
    'Symfony\\Component\\Console\\Formatter\\NullOutputFormatter' => $vendorDir . '/symfony/console/Formatter/NullOutputFormatter.php',
    'Symfony\\Component\\Console\\Formatter\\NullOutputFormatterStyle' => $vendorDir . '/symfony/console/Formatter/NullOutputFormatterStyle.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatter' => $vendorDir . '/symfony/console/Formatter/OutputFormatter.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterInterface' => $vendorDir . '/symfony/console/Formatter/OutputFormatterInterface.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterStyle' => $vendorDir . '/symfony/console/Formatter/OutputFormatterStyle.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterStyleInterface' => $vendorDir . '/symfony/console/Formatter/OutputFormatterStyleInterface.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterStyleStack' => $vendorDir . '/symfony/console/Formatter/OutputFormatterStyleStack.php',
    'Symfony\\Component\\Console\\Formatter\\WrappableOutputFormatterInterface' => $vendorDir . '/symfony/console/Formatter/WrappableOutputFormatterInterface.php',
    'Symfony\\Component\\Console\\Helper\\DebugFormatterHelper' => $vendorDir . '/symfony/console/Helper/DebugFormatterHelper.php',
    'Symfony\\Component\\Console\\Helper\\DescriptorHelper' => $vendorDir . '/symfony/console/Helper/DescriptorHelper.php',
    'Symfony\\Component\\Console\\Helper\\Dumper' => $vendorDir . '/symfony/console/Helper/Dumper.php',
    'Symfony\\Component\\Console\\Helper\\FormatterHelper' => $vendorDir . '/symfony/console/Helper/FormatterHelper.php',
    'Symfony\\Component\\Console\\Helper\\Helper' => $vendorDir . '/symfony/console/Helper/Helper.php',
    'Symfony\\Component\\Console\\Helper\\HelperInterface' => $vendorDir . '/symfony/console/Helper/HelperInterface.php',
    'Symfony\\Component\\Console\\Helper\\HelperSet' => $vendorDir . '/symfony/console/Helper/HelperSet.php',
    'Symfony\\Component\\Console\\Helper\\InputAwareHelper' => $vendorDir . '/symfony/console/Helper/InputAwareHelper.php',
    'Symfony\\Component\\Console\\Helper\\OutputWrapper' => $vendorDir . '/symfony/console/Helper/OutputWrapper.php',
    'Symfony\\Component\\Console\\Helper\\ProcessHelper' => $vendorDir . '/symfony/console/Helper/ProcessHelper.php',
    'Symfony\\Component\\Console\\Helper\\ProgressBar' => $vendorDir . '/symfony/console/Helper/ProgressBar.php',
    'Symfony\\Component\\Console\\Helper\\ProgressIndicator' => $vendorDir . '/symfony/console/Helper/ProgressIndicator.php',
    'Symfony\\Component\\Console\\Helper\\QuestionHelper' => $vendorDir . '/symfony/console/Helper/QuestionHelper.php',
    'Symfony\\Component\\Console\\Helper\\SymfonyQuestionHelper' => $vendorDir . '/symfony/console/Helper/SymfonyQuestionHelper.php',
    'Symfony\\Component\\Console\\Helper\\Table' => $vendorDir . '/symfony/console/Helper/Table.php',
    'Symfony\\Component\\Console\\Helper\\TableCell' => $vendorDir . '/symfony/console/Helper/TableCell.php',
    'Symfony\\Component\\Console\\Helper\\TableCellStyle' => $vendorDir . '/symfony/console/Helper/TableCellStyle.php',
    'Symfony\\Component\\Console\\Helper\\TableRows' => $vendorDir . '/symfony/console/Helper/TableRows.php',
    'Symfony\\Component\\Console\\Helper\\TableSeparator' => $vendorDir . '/symfony/console/Helper/TableSeparator.php',
    'Symfony\\Component\\Console\\Helper\\TableStyle' => $vendorDir . '/symfony/console/Helper/TableStyle.php',
    'Symfony\\Component\\Console\\Input\\ArgvInput' => $vendorDir . '/symfony/console/Input/ArgvInput.php',
    'Symfony\\Component\\Console\\Input\\ArrayInput' => $vendorDir . '/symfony/console/Input/ArrayInput.php',
    'Symfony\\Component\\Console\\Input\\Input' => $vendorDir . '/symfony/console/Input/Input.php',
    'Symfony\\Component\\Console\\Input\\InputArgument' => $vendorDir . '/symfony/console/Input/InputArgument.php',
    'Symfony\\Component\\Console\\Input\\InputAwareInterface' => $vendorDir . '/symfony/console/Input/InputAwareInterface.php',
    'Symfony\\Component\\Console\\Input\\InputDefinition' => $vendorDir . '/symfony/console/Input/InputDefinition.php',
    'Symfony\\Component\\Console\\Input\\InputInterface' => $vendorDir . '/symfony/console/Input/InputInterface.php',
    'Symfony\\Component\\Console\\Input\\InputOption' => $vendorDir . '/symfony/console/Input/InputOption.php',
    'Symfony\\Component\\Console\\Input\\StreamableInputInterface' => $vendorDir . '/symfony/console/Input/StreamableInputInterface.php',
    'Symfony\\Component\\Console\\Input\\StringInput' => $vendorDir . '/symfony/console/Input/StringInput.php',
    'Symfony\\Component\\Console\\Logger\\ConsoleLogger' => $vendorDir . '/symfony/console/Logger/ConsoleLogger.php',
    'Symfony\\Component\\Console\\Messenger\\RunCommandContext' => $vendorDir . '/symfony/console/Messenger/RunCommandContext.php',
    'Symfony\\Component\\Console\\Messenger\\RunCommandMessage' => $vendorDir . '/symfony/console/Messenger/RunCommandMessage.php',
    'Symfony\\Component\\Console\\Messenger\\RunCommandMessageHandler' => $vendorDir . '/symfony/console/Messenger/RunCommandMessageHandler.php',
    'Symfony\\Component\\Console\\Output\\AnsiColorMode' => $vendorDir . '/symfony/console/Output/AnsiColorMode.php',
    'Symfony\\Component\\Console\\Output\\BufferedOutput' => $vendorDir . '/symfony/console/Output/BufferedOutput.php',
    'Symfony\\Component\\Console\\Output\\ConsoleOutput' => $vendorDir . '/symfony/console/Output/ConsoleOutput.php',
    'Symfony\\Component\\Console\\Output\\ConsoleOutputInterface' => $vendorDir . '/symfony/console/Output/ConsoleOutputInterface.php',
    'Symfony\\Component\\Console\\Output\\ConsoleSectionOutput' => $vendorDir . '/symfony/console/Output/ConsoleSectionOutput.php',
    'Symfony\\Component\\Console\\Output\\NullOutput' => $vendorDir . '/symfony/console/Output/NullOutput.php',
    'Symfony\\Component\\Console\\Output\\Output' => $vendorDir . '/symfony/console/Output/Output.php',
    'Symfony\\Component\\Console\\Output\\OutputInterface' => $vendorDir . '/symfony/console/Output/OutputInterface.php',
    'Symfony\\Component\\Console\\Output\\StreamOutput' => $vendorDir . '/symfony/console/Output/StreamOutput.php',
    'Symfony\\Component\\Console\\Output\\TrimmedBufferOutput' => $vendorDir . '/symfony/console/Output/TrimmedBufferOutput.php',
    'Symfony\\Component\\Console\\Question\\ChoiceQuestion' => $vendorDir . '/symfony/console/Question/ChoiceQuestion.php',
    'Symfony\\Component\\Console\\Question\\ConfirmationQuestion' => $vendorDir . '/symfony/console/Question/ConfirmationQuestion.php',
    'Symfony\\Component\\Console\\Question\\Question' => $vendorDir . '/symfony/console/Question/Question.php',
    'Symfony\\Component\\Console\\SignalRegistry\\SignalMap' => $vendorDir . '/symfony/console/SignalRegistry/SignalMap.php',
    'Symfony\\Component\\Console\\SignalRegistry\\SignalRegistry' => $vendorDir . '/symfony/console/SignalRegistry/SignalRegistry.php',
    'Symfony\\Component\\Console\\SingleCommandApplication' => $vendorDir . '/symfony/console/SingleCommandApplication.php',
    'Symfony\\Component\\Console\\Style\\OutputStyle' => $vendorDir . '/symfony/console/Style/OutputStyle.php',
    'Symfony\\Component\\Console\\Style\\StyleInterface' => $vendorDir . '/symfony/console/Style/StyleInterface.php',
    'Symfony\\Component\\Console\\Style\\SymfonyStyle' => $vendorDir . '/symfony/console/Style/SymfonyStyle.php',
    'Symfony\\Component\\Console\\Terminal' => $vendorDir . '/symfony/console/Terminal.php',
    'Symfony\\Component\\Console\\Tester\\ApplicationTester' => $vendorDir . '/symfony/console/Tester/ApplicationTester.php',
    'Symfony\\Component\\Console\\Tester\\CommandCompletionTester' => $vendorDir . '/symfony/console/Tester/CommandCompletionTester.php',
    'Symfony\\Component\\Console\\Tester\\CommandTester' => $vendorDir . '/symfony/console/Tester/CommandTester.php',
    'Symfony\\Component\\Console\\Tester\\Constraint\\CommandIsSuccessful' => $vendorDir . '/symfony/console/Tester/Constraint/CommandIsSuccessful.php',
    'Symfony\\Component\\Console\\Tester\\TesterTrait' => $vendorDir . '/symfony/console/Tester/TesterTrait.php',
    'Symfony\\Component\\CssSelector\\CssSelectorConverter' => $vendorDir . '/symfony/css-selector/CssSelectorConverter.php',
    'Symfony\\Component\\CssSelector\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/css-selector/Exception/ExceptionInterface.php',
    'Symfony\\Component\\CssSelector\\Exception\\ExpressionErrorException' => $vendorDir . '/symfony/css-selector/Exception/ExpressionErrorException.php',
    'Symfony\\Component\\CssSelector\\Exception\\InternalErrorException' => $vendorDir . '/symfony/css-selector/Exception/InternalErrorException.php',
    'Symfony\\Component\\CssSelector\\Exception\\ParseException' => $vendorDir . '/symfony/css-selector/Exception/ParseException.php',
    'Symfony\\Component\\CssSelector\\Exception\\SyntaxErrorException' => $vendorDir . '/symfony/css-selector/Exception/SyntaxErrorException.php',
    'Symfony\\Component\\CssSelector\\Node\\AbstractNode' => $vendorDir . '/symfony/css-selector/Node/AbstractNode.php',
    'Symfony\\Component\\CssSelector\\Node\\AttributeNode' => $vendorDir . '/symfony/css-selector/Node/AttributeNode.php',
    'Symfony\\Component\\CssSelector\\Node\\ClassNode' => $vendorDir . '/symfony/css-selector/Node/ClassNode.php',
    'Symfony\\Component\\CssSelector\\Node\\CombinedSelectorNode' => $vendorDir . '/symfony/css-selector/Node/CombinedSelectorNode.php',
    'Symfony\\Component\\CssSelector\\Node\\ElementNode' => $vendorDir . '/symfony/css-selector/Node/ElementNode.php',
    'Symfony\\Component\\CssSelector\\Node\\FunctionNode' => $vendorDir . '/symfony/css-selector/Node/FunctionNode.php',
    'Symfony\\Component\\CssSelector\\Node\\HashNode' => $vendorDir . '/symfony/css-selector/Node/HashNode.php',
    'Symfony\\Component\\CssSelector\\Node\\NegationNode' => $vendorDir . '/symfony/css-selector/Node/NegationNode.php',
    'Symfony\\Component\\CssSelector\\Node\\NodeInterface' => $vendorDir . '/symfony/css-selector/Node/NodeInterface.php',
    'Symfony\\Component\\CssSelector\\Node\\PseudoNode' => $vendorDir . '/symfony/css-selector/Node/PseudoNode.php',
    'Symfony\\Component\\CssSelector\\Node\\SelectorNode' => $vendorDir . '/symfony/css-selector/Node/SelectorNode.php',
    'Symfony\\Component\\CssSelector\\Node\\Specificity' => $vendorDir . '/symfony/css-selector/Node/Specificity.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\CommentHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/CommentHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\HandlerInterface' => $vendorDir . '/symfony/css-selector/Parser/Handler/HandlerInterface.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\HashHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/HashHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\IdentifierHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/IdentifierHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\NumberHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/NumberHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\StringHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/StringHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\WhitespaceHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/WhitespaceHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Parser' => $vendorDir . '/symfony/css-selector/Parser/Parser.php',
    'Symfony\\Component\\CssSelector\\Parser\\ParserInterface' => $vendorDir . '/symfony/css-selector/Parser/ParserInterface.php',
    'Symfony\\Component\\CssSelector\\Parser\\Reader' => $vendorDir . '/symfony/css-selector/Parser/Reader.php',
    'Symfony\\Component\\CssSelector\\Parser\\Shortcut\\ClassParser' => $vendorDir . '/symfony/css-selector/Parser/Shortcut/ClassParser.php',
    'Symfony\\Component\\CssSelector\\Parser\\Shortcut\\ElementParser' => $vendorDir . '/symfony/css-selector/Parser/Shortcut/ElementParser.php',
    'Symfony\\Component\\CssSelector\\Parser\\Shortcut\\EmptyStringParser' => $vendorDir . '/symfony/css-selector/Parser/Shortcut/EmptyStringParser.php',
    'Symfony\\Component\\CssSelector\\Parser\\Shortcut\\HashParser' => $vendorDir . '/symfony/css-selector/Parser/Shortcut/HashParser.php',
    'Symfony\\Component\\CssSelector\\Parser\\Token' => $vendorDir . '/symfony/css-selector/Parser/Token.php',
    'Symfony\\Component\\CssSelector\\Parser\\TokenStream' => $vendorDir . '/symfony/css-selector/Parser/TokenStream.php',
    'Symfony\\Component\\CssSelector\\Parser\\Tokenizer\\Tokenizer' => $vendorDir . '/symfony/css-selector/Parser/Tokenizer/Tokenizer.php',
    'Symfony\\Component\\CssSelector\\Parser\\Tokenizer\\TokenizerEscaping' => $vendorDir . '/symfony/css-selector/Parser/Tokenizer/TokenizerEscaping.php',
    'Symfony\\Component\\CssSelector\\Parser\\Tokenizer\\TokenizerPatterns' => $vendorDir . '/symfony/css-selector/Parser/Tokenizer/TokenizerPatterns.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\AbstractExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/AbstractExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\AttributeMatchingExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/AttributeMatchingExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\CombinationExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/CombinationExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\ExtensionInterface' => $vendorDir . '/symfony/css-selector/XPath/Extension/ExtensionInterface.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\FunctionExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/FunctionExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\HtmlExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/HtmlExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\NodeExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/NodeExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\PseudoClassExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/PseudoClassExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Translator' => $vendorDir . '/symfony/css-selector/XPath/Translator.php',
    'Symfony\\Component\\CssSelector\\XPath\\TranslatorInterface' => $vendorDir . '/symfony/css-selector/XPath/TranslatorInterface.php',
    'Symfony\\Component\\CssSelector\\XPath\\XPathExpr' => $vendorDir . '/symfony/css-selector/XPath/XPathExpr.php',
    'Symfony\\Component\\ErrorHandler\\BufferingLogger' => $vendorDir . '/symfony/error-handler/BufferingLogger.php',
    'Symfony\\Component\\ErrorHandler\\Debug' => $vendorDir . '/symfony/error-handler/Debug.php',
    'Symfony\\Component\\ErrorHandler\\DebugClassLoader' => $vendorDir . '/symfony/error-handler/DebugClassLoader.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\ClassNotFoundErrorEnhancer' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/ClassNotFoundErrorEnhancer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\ErrorEnhancerInterface' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/ErrorEnhancerInterface.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\UndefinedFunctionErrorEnhancer' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/UndefinedFunctionErrorEnhancer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\UndefinedMethodErrorEnhancer' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/UndefinedMethodErrorEnhancer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorHandler' => $vendorDir . '/symfony/error-handler/ErrorHandler.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\CliErrorRenderer' => $vendorDir . '/symfony/error-handler/ErrorRenderer/CliErrorRenderer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\ErrorRendererInterface' => $vendorDir . '/symfony/error-handler/ErrorRenderer/ErrorRendererInterface.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\FileLinkFormatter' => $vendorDir . '/symfony/error-handler/ErrorRenderer/FileLinkFormatter.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\HtmlErrorRenderer' => $vendorDir . '/symfony/error-handler/ErrorRenderer/HtmlErrorRenderer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\SerializerErrorRenderer' => $vendorDir . '/symfony/error-handler/ErrorRenderer/SerializerErrorRenderer.php',
    'Symfony\\Component\\ErrorHandler\\Error\\ClassNotFoundError' => $vendorDir . '/symfony/error-handler/Error/ClassNotFoundError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\FatalError' => $vendorDir . '/symfony/error-handler/Error/FatalError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\OutOfMemoryError' => $vendorDir . '/symfony/error-handler/Error/OutOfMemoryError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\UndefinedFunctionError' => $vendorDir . '/symfony/error-handler/Error/UndefinedFunctionError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\UndefinedMethodError' => $vendorDir . '/symfony/error-handler/Error/UndefinedMethodError.php',
    'Symfony\\Component\\ErrorHandler\\Exception\\FlattenException' => $vendorDir . '/symfony/error-handler/Exception/FlattenException.php',
    'Symfony\\Component\\ErrorHandler\\Exception\\SilencedErrorContext' => $vendorDir . '/symfony/error-handler/Exception/SilencedErrorContext.php',
    'Symfony\\Component\\ErrorHandler\\Internal\\TentativeTypes' => $vendorDir . '/symfony/error-handler/Internal/TentativeTypes.php',
    'Symfony\\Component\\ErrorHandler\\ThrowableUtils' => $vendorDir . '/symfony/error-handler/ThrowableUtils.php',
    'Symfony\\Component\\EventDispatcher\\Attribute\\AsEventListener' => $vendorDir . '/symfony/event-dispatcher/Attribute/AsEventListener.php',
    'Symfony\\Component\\EventDispatcher\\Debug\\TraceableEventDispatcher' => $vendorDir . '/symfony/event-dispatcher/Debug/TraceableEventDispatcher.php',
    'Symfony\\Component\\EventDispatcher\\Debug\\WrappedListener' => $vendorDir . '/symfony/event-dispatcher/Debug/WrappedListener.php',
    'Symfony\\Component\\EventDispatcher\\DependencyInjection\\AddEventAliasesPass' => $vendorDir . '/symfony/event-dispatcher/DependencyInjection/AddEventAliasesPass.php',
    'Symfony\\Component\\EventDispatcher\\DependencyInjection\\RegisterListenersPass' => $vendorDir . '/symfony/event-dispatcher/DependencyInjection/RegisterListenersPass.php',
    'Symfony\\Component\\EventDispatcher\\EventDispatcher' => $vendorDir . '/symfony/event-dispatcher/EventDispatcher.php',
    'Symfony\\Component\\EventDispatcher\\EventDispatcherInterface' => $vendorDir . '/symfony/event-dispatcher/EventDispatcherInterface.php',
    'Symfony\\Component\\EventDispatcher\\EventSubscriberInterface' => $vendorDir . '/symfony/event-dispatcher/EventSubscriberInterface.php',
    'Symfony\\Component\\EventDispatcher\\GenericEvent' => $vendorDir . '/symfony/event-dispatcher/GenericEvent.php',
    'Symfony\\Component\\EventDispatcher\\ImmutableEventDispatcher' => $vendorDir . '/symfony/event-dispatcher/ImmutableEventDispatcher.php',
    'Symfony\\Component\\Finder\\Comparator\\Comparator' => $vendorDir . '/symfony/finder/Comparator/Comparator.php',
    'Symfony\\Component\\Finder\\Comparator\\DateComparator' => $vendorDir . '/symfony/finder/Comparator/DateComparator.php',
    'Symfony\\Component\\Finder\\Comparator\\NumberComparator' => $vendorDir . '/symfony/finder/Comparator/NumberComparator.php',
    'Symfony\\Component\\Finder\\Exception\\AccessDeniedException' => $vendorDir . '/symfony/finder/Exception/AccessDeniedException.php',
    'Symfony\\Component\\Finder\\Exception\\DirectoryNotFoundException' => $vendorDir . '/symfony/finder/Exception/DirectoryNotFoundException.php',
    'Symfony\\Component\\Finder\\Finder' => $vendorDir . '/symfony/finder/Finder.php',
    'Symfony\\Component\\Finder\\Gitignore' => $vendorDir . '/symfony/finder/Gitignore.php',
    'Symfony\\Component\\Finder\\Glob' => $vendorDir . '/symfony/finder/Glob.php',
    'Symfony\\Component\\Finder\\Iterator\\CustomFilterIterator' => $vendorDir . '/symfony/finder/Iterator/CustomFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\DateRangeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/DateRangeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\DepthRangeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/DepthRangeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\ExcludeDirectoryFilterIterator' => $vendorDir . '/symfony/finder/Iterator/ExcludeDirectoryFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\FileTypeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/FileTypeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\FilecontentFilterIterator' => $vendorDir . '/symfony/finder/Iterator/FilecontentFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\FilenameFilterIterator' => $vendorDir . '/symfony/finder/Iterator/FilenameFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\LazyIterator' => $vendorDir . '/symfony/finder/Iterator/LazyIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\MultiplePcreFilterIterator' => $vendorDir . '/symfony/finder/Iterator/MultiplePcreFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\PathFilterIterator' => $vendorDir . '/symfony/finder/Iterator/PathFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\RecursiveDirectoryIterator' => $vendorDir . '/symfony/finder/Iterator/RecursiveDirectoryIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\SizeRangeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/SizeRangeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\SortableIterator' => $vendorDir . '/symfony/finder/Iterator/SortableIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\VcsIgnoredFilterIterator' => $vendorDir . '/symfony/finder/Iterator/VcsIgnoredFilterIterator.php',
    'Symfony\\Component\\Finder\\SplFileInfo' => $vendorDir . '/symfony/finder/SplFileInfo.php',
    'Symfony\\Component\\HttpFoundation\\AcceptHeader' => $vendorDir . '/symfony/http-foundation/AcceptHeader.php',
    'Symfony\\Component\\HttpFoundation\\AcceptHeaderItem' => $vendorDir . '/symfony/http-foundation/AcceptHeaderItem.php',
    'Symfony\\Component\\HttpFoundation\\BinaryFileResponse' => $vendorDir . '/symfony/http-foundation/BinaryFileResponse.php',
    'Symfony\\Component\\HttpFoundation\\ChainRequestMatcher' => $vendorDir . '/symfony/http-foundation/ChainRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\Cookie' => $vendorDir . '/symfony/http-foundation/Cookie.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\BadRequestException' => $vendorDir . '/symfony/http-foundation/Exception/BadRequestException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\ConflictingHeadersException' => $vendorDir . '/symfony/http-foundation/Exception/ConflictingHeadersException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\JsonException' => $vendorDir . '/symfony/http-foundation/Exception/JsonException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\RequestExceptionInterface' => $vendorDir . '/symfony/http-foundation/Exception/RequestExceptionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\SessionNotFoundException' => $vendorDir . '/symfony/http-foundation/Exception/SessionNotFoundException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\SuspiciousOperationException' => $vendorDir . '/symfony/http-foundation/Exception/SuspiciousOperationException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\UnexpectedValueException' => $vendorDir . '/symfony/http-foundation/Exception/UnexpectedValueException.php',
    'Symfony\\Component\\HttpFoundation\\ExpressionRequestMatcher' => $vendorDir . '/symfony/http-foundation/ExpressionRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\FileBag' => $vendorDir . '/symfony/http-foundation/FileBag.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\AccessDeniedException' => $vendorDir . '/symfony/http-foundation/File/Exception/AccessDeniedException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\CannotWriteFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/CannotWriteFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\ExtensionFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/ExtensionFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\FileException' => $vendorDir . '/symfony/http-foundation/File/Exception/FileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\FileNotFoundException' => $vendorDir . '/symfony/http-foundation/File/Exception/FileNotFoundException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\FormSizeFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/FormSizeFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\IniSizeFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/IniSizeFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\NoFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/NoFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\NoTmpDirFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/NoTmpDirFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\PartialFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/PartialFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\UnexpectedTypeException' => $vendorDir . '/symfony/http-foundation/File/Exception/UnexpectedTypeException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\UploadException' => $vendorDir . '/symfony/http-foundation/File/Exception/UploadException.php',
    'Symfony\\Component\\HttpFoundation\\File\\File' => $vendorDir . '/symfony/http-foundation/File/File.php',
    'Symfony\\Component\\HttpFoundation\\File\\Stream' => $vendorDir . '/symfony/http-foundation/File/Stream.php',
    'Symfony\\Component\\HttpFoundation\\File\\UploadedFile' => $vendorDir . '/symfony/http-foundation/File/UploadedFile.php',
    'Symfony\\Component\\HttpFoundation\\HeaderBag' => $vendorDir . '/symfony/http-foundation/HeaderBag.php',
    'Symfony\\Component\\HttpFoundation\\HeaderUtils' => $vendorDir . '/symfony/http-foundation/HeaderUtils.php',
    'Symfony\\Component\\HttpFoundation\\InputBag' => $vendorDir . '/symfony/http-foundation/InputBag.php',
    'Symfony\\Component\\HttpFoundation\\IpUtils' => $vendorDir . '/symfony/http-foundation/IpUtils.php',
    'Symfony\\Component\\HttpFoundation\\JsonResponse' => $vendorDir . '/symfony/http-foundation/JsonResponse.php',
    'Symfony\\Component\\HttpFoundation\\ParameterBag' => $vendorDir . '/symfony/http-foundation/ParameterBag.php',
    'Symfony\\Component\\HttpFoundation\\RateLimiter\\AbstractRequestRateLimiter' => $vendorDir . '/symfony/http-foundation/RateLimiter/AbstractRequestRateLimiter.php',
    'Symfony\\Component\\HttpFoundation\\RateLimiter\\PeekableRequestRateLimiterInterface' => $vendorDir . '/symfony/http-foundation/RateLimiter/PeekableRequestRateLimiterInterface.php',
    'Symfony\\Component\\HttpFoundation\\RateLimiter\\RequestRateLimiterInterface' => $vendorDir . '/symfony/http-foundation/RateLimiter/RequestRateLimiterInterface.php',
    'Symfony\\Component\\HttpFoundation\\RedirectResponse' => $vendorDir . '/symfony/http-foundation/RedirectResponse.php',
    'Symfony\\Component\\HttpFoundation\\Request' => $vendorDir . '/symfony/http-foundation/Request.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcherInterface' => $vendorDir . '/symfony/http-foundation/RequestMatcherInterface.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\AttributesRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/AttributesRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\ExpressionRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/ExpressionRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\HostRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/HostRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\IpsRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/IpsRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\IsJsonRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/IsJsonRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\MethodRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/MethodRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\PathRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/PathRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\PortRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/PortRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\SchemeRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/SchemeRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestStack' => $vendorDir . '/symfony/http-foundation/RequestStack.php',
    'Symfony\\Component\\HttpFoundation\\Response' => $vendorDir . '/symfony/http-foundation/Response.php',
    'Symfony\\Component\\HttpFoundation\\ResponseHeaderBag' => $vendorDir . '/symfony/http-foundation/ResponseHeaderBag.php',
    'Symfony\\Component\\HttpFoundation\\ServerBag' => $vendorDir . '/symfony/http-foundation/ServerBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Attribute\\AttributeBag' => $vendorDir . '/symfony/http-foundation/Session/Attribute/AttributeBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Attribute\\AttributeBagInterface' => $vendorDir . '/symfony/http-foundation/Session/Attribute/AttributeBagInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\FlashBagAwareSessionInterface' => $vendorDir . '/symfony/http-foundation/Session/FlashBagAwareSessionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Flash\\AutoExpireFlashBag' => $vendorDir . '/symfony/http-foundation/Session/Flash/AutoExpireFlashBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Flash\\FlashBag' => $vendorDir . '/symfony/http-foundation/Session/Flash/FlashBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Flash\\FlashBagInterface' => $vendorDir . '/symfony/http-foundation/Session/Flash/FlashBagInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Session' => $vendorDir . '/symfony/http-foundation/Session/Session.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionBagInterface' => $vendorDir . '/symfony/http-foundation/Session/SessionBagInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionBagProxy' => $vendorDir . '/symfony/http-foundation/Session/SessionBagProxy.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionFactory' => $vendorDir . '/symfony/http-foundation/Session/SessionFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionFactoryInterface' => $vendorDir . '/symfony/http-foundation/Session/SessionFactoryInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionInterface' => $vendorDir . '/symfony/http-foundation/Session/SessionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionUtils' => $vendorDir . '/symfony/http-foundation/Session/SessionUtils.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\AbstractSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/AbstractSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\IdentityMarshaller' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/IdentityMarshaller.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MarshallingSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MarshallingSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MemcachedSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MemcachedSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MigratingSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MigratingSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MongoDbSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MongoDbSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\NativeFileSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/NativeFileSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\NullSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/NullSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\PdoSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/PdoSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\RedisSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/RedisSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\SessionHandlerFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/SessionHandlerFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\StrictSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/StrictSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MetadataBag' => $vendorDir . '/symfony/http-foundation/Session/Storage/MetadataBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MockArraySessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/MockArraySessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MockFileSessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/MockFileSessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MockFileSessionStorageFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/MockFileSessionStorageFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\NativeSessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/NativeSessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\NativeSessionStorageFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/NativeSessionStorageFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\PhpBridgeSessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/PhpBridgeSessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\PhpBridgeSessionStorageFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/PhpBridgeSessionStorageFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Proxy\\AbstractProxy' => $vendorDir . '/symfony/http-foundation/Session/Storage/Proxy/AbstractProxy.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Proxy\\SessionHandlerProxy' => $vendorDir . '/symfony/http-foundation/Session/Storage/Proxy/SessionHandlerProxy.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\SessionStorageFactoryInterface' => $vendorDir . '/symfony/http-foundation/Session/Storage/SessionStorageFactoryInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\SessionStorageInterface' => $vendorDir . '/symfony/http-foundation/Session/Storage/SessionStorageInterface.php',
    'Symfony\\Component\\HttpFoundation\\StreamedJsonResponse' => $vendorDir . '/symfony/http-foundation/StreamedJsonResponse.php',
    'Symfony\\Component\\HttpFoundation\\StreamedResponse' => $vendorDir . '/symfony/http-foundation/StreamedResponse.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\RequestAttributeValueSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/RequestAttributeValueSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseCookieValueSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseCookieValueSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseFormatSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseFormatSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHasCookie' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHasCookie.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHasHeader' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHasHeader.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHeaderLocationSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHeaderLocationSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHeaderSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHeaderSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseIsRedirected' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseIsRedirected.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseIsSuccessful' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseIsSuccessful.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseIsUnprocessable' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseIsUnprocessable.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseStatusCodeSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseStatusCodeSame.php',
    'Symfony\\Component\\HttpFoundation\\UriSigner' => $vendorDir . '/symfony/http-foundation/UriSigner.php',
    'Symfony\\Component\\HttpFoundation\\UrlHelper' => $vendorDir . '/symfony/http-foundation/UrlHelper.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\AsController' => $vendorDir . '/symfony/http-kernel/Attribute/AsController.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\AsTargetedValueResolver' => $vendorDir . '/symfony/http-kernel/Attribute/AsTargetedValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\Cache' => $vendorDir . '/symfony/http-kernel/Attribute/Cache.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapDateTime' => $vendorDir . '/symfony/http-kernel/Attribute/MapDateTime.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapQueryParameter' => $vendorDir . '/symfony/http-kernel/Attribute/MapQueryParameter.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapQueryString' => $vendorDir . '/symfony/http-kernel/Attribute/MapQueryString.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapRequestPayload' => $vendorDir . '/symfony/http-kernel/Attribute/MapRequestPayload.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\ValueResolver' => $vendorDir . '/symfony/http-kernel/Attribute/ValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\WithHttpStatus' => $vendorDir . '/symfony/http-kernel/Attribute/WithHttpStatus.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\WithLogLevel' => $vendorDir . '/symfony/http-kernel/Attribute/WithLogLevel.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\AbstractBundle' => $vendorDir . '/symfony/http-kernel/Bundle/AbstractBundle.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\Bundle' => $vendorDir . '/symfony/http-kernel/Bundle/Bundle.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\BundleExtension' => $vendorDir . '/symfony/http-kernel/Bundle/BundleExtension.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\BundleInterface' => $vendorDir . '/symfony/http-kernel/Bundle/BundleInterface.php',
    'Symfony\\Component\\HttpKernel\\CacheClearer\\CacheClearerInterface' => $vendorDir . '/symfony/http-kernel/CacheClearer/CacheClearerInterface.php',
    'Symfony\\Component\\HttpKernel\\CacheClearer\\ChainCacheClearer' => $vendorDir . '/symfony/http-kernel/CacheClearer/ChainCacheClearer.php',
    'Symfony\\Component\\HttpKernel\\CacheClearer\\Psr6CacheClearer' => $vendorDir . '/symfony/http-kernel/CacheClearer/Psr6CacheClearer.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\CacheWarmer' => $vendorDir . '/symfony/http-kernel/CacheWarmer/CacheWarmer.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\CacheWarmerAggregate' => $vendorDir . '/symfony/http-kernel/CacheWarmer/CacheWarmerAggregate.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\CacheWarmerInterface' => $vendorDir . '/symfony/http-kernel/CacheWarmer/CacheWarmerInterface.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\WarmableInterface' => $vendorDir . '/symfony/http-kernel/CacheWarmer/WarmableInterface.php',
    'Symfony\\Component\\HttpKernel\\Config\\FileLocator' => $vendorDir . '/symfony/http-kernel/Config/FileLocator.php',
    'Symfony\\Component\\HttpKernel\\ControllerMetadata\\ArgumentMetadata' => $vendorDir . '/symfony/http-kernel/ControllerMetadata/ArgumentMetadata.php',
    'Symfony\\Component\\HttpKernel\\ControllerMetadata\\ArgumentMetadataFactory' => $vendorDir . '/symfony/http-kernel/ControllerMetadata/ArgumentMetadataFactory.php',
    'Symfony\\Component\\HttpKernel\\ControllerMetadata\\ArgumentMetadataFactoryInterface' => $vendorDir . '/symfony/http-kernel/ControllerMetadata/ArgumentMetadataFactoryInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\BackedEnumValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/BackedEnumValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\DateTimeValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/DateTimeValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\DefaultValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/DefaultValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\NotTaggedControllerValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/NotTaggedControllerValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\QueryParameterValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/QueryParameterValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\RequestAttributeValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/RequestAttributeValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\RequestPayloadValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/RequestPayloadValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\RequestValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/RequestValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\ServiceValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/ServiceValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\SessionValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/SessionValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\TraceableValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/TraceableValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\UidValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/UidValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\VariadicValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/VariadicValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentValueResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentValueResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ContainerControllerResolver' => $vendorDir . '/symfony/http-kernel/Controller/ContainerControllerResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ControllerReference' => $vendorDir . '/symfony/http-kernel/Controller/ControllerReference.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ControllerResolver' => $vendorDir . '/symfony/http-kernel/Controller/ControllerResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ControllerResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ControllerResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ErrorController' => $vendorDir . '/symfony/http-kernel/Controller/ErrorController.php',
    'Symfony\\Component\\HttpKernel\\Controller\\TraceableArgumentResolver' => $vendorDir . '/symfony/http-kernel/Controller/TraceableArgumentResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\TraceableControllerResolver' => $vendorDir . '/symfony/http-kernel/Controller/TraceableControllerResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ValueResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ValueResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\AjaxDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/AjaxDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\ConfigDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/ConfigDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\DataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/DataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\DataCollectorInterface' => $vendorDir . '/symfony/http-kernel/DataCollector/DataCollectorInterface.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\DumpDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/DumpDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\EventDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/EventDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\ExceptionDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/ExceptionDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\LateDataCollectorInterface' => $vendorDir . '/symfony/http-kernel/DataCollector/LateDataCollectorInterface.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\LoggerDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/LoggerDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\MemoryDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/MemoryDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\RequestDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/RequestDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\RouterDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/RouterDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\TimeDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/TimeDataCollector.php',
    'Symfony\\Component\\HttpKernel\\Debug\\ErrorHandlerConfigurator' => $vendorDir . '/symfony/http-kernel/Debug/ErrorHandlerConfigurator.php',
    'Symfony\\Component\\HttpKernel\\Debug\\FileLinkFormatter' => $vendorDir . '/symfony/http-kernel/Debug/FileLinkFormatter.php',
    'Symfony\\Component\\HttpKernel\\Debug\\TraceableEventDispatcher' => $vendorDir . '/symfony/http-kernel/Debug/TraceableEventDispatcher.php',
    'Symfony\\Component\\HttpKernel\\Debug\\VirtualRequestStack' => $vendorDir . '/symfony/http-kernel/Debug/VirtualRequestStack.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\AddAnnotatedClassesToCachePass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/AddAnnotatedClassesToCachePass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ConfigurableExtension' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ConfigurableExtension.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ControllerArgumentValueResolverPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ControllerArgumentValueResolverPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\Extension' => $vendorDir . '/symfony/http-kernel/DependencyInjection/Extension.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\FragmentRendererPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/FragmentRendererPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\LazyLoadingFragmentHandler' => $vendorDir . '/symfony/http-kernel/DependencyInjection/LazyLoadingFragmentHandler.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\LoggerPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/LoggerPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\MergeExtensionConfigurationPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/MergeExtensionConfigurationPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\RegisterControllerArgumentLocatorsPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/RegisterControllerArgumentLocatorsPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\RegisterLocaleAwareServicesPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/RegisterLocaleAwareServicesPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\RemoveEmptyControllerArgumentLocatorsPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/RemoveEmptyControllerArgumentLocatorsPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ResettableServicePass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ResettableServicePass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ServicesResetter' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ServicesResetter.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\AbstractSessionListener' => $vendorDir . '/symfony/http-kernel/EventListener/AbstractSessionListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\AddRequestFormatsListener' => $vendorDir . '/symfony/http-kernel/EventListener/AddRequestFormatsListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\CacheAttributeListener' => $vendorDir . '/symfony/http-kernel/EventListener/CacheAttributeListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\DebugHandlersListener' => $vendorDir . '/symfony/http-kernel/EventListener/DebugHandlersListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\DisallowRobotsIndexingListener' => $vendorDir . '/symfony/http-kernel/EventListener/DisallowRobotsIndexingListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\DumpListener' => $vendorDir . '/symfony/http-kernel/EventListener/DumpListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ErrorListener' => $vendorDir . '/symfony/http-kernel/EventListener/ErrorListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\FragmentListener' => $vendorDir . '/symfony/http-kernel/EventListener/FragmentListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\LocaleAwareListener' => $vendorDir . '/symfony/http-kernel/EventListener/LocaleAwareListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\LocaleListener' => $vendorDir . '/symfony/http-kernel/EventListener/LocaleListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ProfilerListener' => $vendorDir . '/symfony/http-kernel/EventListener/ProfilerListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ResponseListener' => $vendorDir . '/symfony/http-kernel/EventListener/ResponseListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\RouterListener' => $vendorDir . '/symfony/http-kernel/EventListener/RouterListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\SessionListener' => $vendorDir . '/symfony/http-kernel/EventListener/SessionListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\StreamedResponseListener' => $vendorDir . '/symfony/http-kernel/EventListener/StreamedResponseListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\SurrogateListener' => $vendorDir . '/symfony/http-kernel/EventListener/SurrogateListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ValidateRequestListener' => $vendorDir . '/symfony/http-kernel/EventListener/ValidateRequestListener.php',
    'Symfony\\Component\\HttpKernel\\Event\\ControllerArgumentsEvent' => $vendorDir . '/symfony/http-kernel/Event/ControllerArgumentsEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ControllerEvent' => $vendorDir . '/symfony/http-kernel/Event/ControllerEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ExceptionEvent' => $vendorDir . '/symfony/http-kernel/Event/ExceptionEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\FinishRequestEvent' => $vendorDir . '/symfony/http-kernel/Event/FinishRequestEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\KernelEvent' => $vendorDir . '/symfony/http-kernel/Event/KernelEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\RequestEvent' => $vendorDir . '/symfony/http-kernel/Event/RequestEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ResponseEvent' => $vendorDir . '/symfony/http-kernel/Event/ResponseEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\TerminateEvent' => $vendorDir . '/symfony/http-kernel/Event/TerminateEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ViewEvent' => $vendorDir . '/symfony/http-kernel/Event/ViewEvent.php',
    'Symfony\\Component\\HttpKernel\\Exception\\AccessDeniedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/AccessDeniedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\BadRequestHttpException' => $vendorDir . '/symfony/http-kernel/Exception/BadRequestHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ConflictHttpException' => $vendorDir . '/symfony/http-kernel/Exception/ConflictHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ControllerDoesNotReturnResponseException' => $vendorDir . '/symfony/http-kernel/Exception/ControllerDoesNotReturnResponseException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\GoneHttpException' => $vendorDir . '/symfony/http-kernel/Exception/GoneHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\HttpException' => $vendorDir . '/symfony/http-kernel/Exception/HttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\HttpExceptionInterface' => $vendorDir . '/symfony/http-kernel/Exception/HttpExceptionInterface.php',
    'Symfony\\Component\\HttpKernel\\Exception\\InvalidMetadataException' => $vendorDir . '/symfony/http-kernel/Exception/InvalidMetadataException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\LengthRequiredHttpException' => $vendorDir . '/symfony/http-kernel/Exception/LengthRequiredHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\LockedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/LockedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\MethodNotAllowedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/MethodNotAllowedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\NotAcceptableHttpException' => $vendorDir . '/symfony/http-kernel/Exception/NotAcceptableHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\NotFoundHttpException' => $vendorDir . '/symfony/http-kernel/Exception/NotFoundHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\PreconditionFailedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/PreconditionFailedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\PreconditionRequiredHttpException' => $vendorDir . '/symfony/http-kernel/Exception/PreconditionRequiredHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ResolverNotFoundException' => $vendorDir . '/symfony/http-kernel/Exception/ResolverNotFoundException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ServiceUnavailableHttpException' => $vendorDir . '/symfony/http-kernel/Exception/ServiceUnavailableHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\TooManyRequestsHttpException' => $vendorDir . '/symfony/http-kernel/Exception/TooManyRequestsHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnauthorizedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/UnauthorizedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnexpectedSessionUsageException' => $vendorDir . '/symfony/http-kernel/Exception/UnexpectedSessionUsageException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnprocessableEntityHttpException' => $vendorDir . '/symfony/http-kernel/Exception/UnprocessableEntityHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnsupportedMediaTypeHttpException' => $vendorDir . '/symfony/http-kernel/Exception/UnsupportedMediaTypeHttpException.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\AbstractSurrogateFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/AbstractSurrogateFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\EsiFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/EsiFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentHandler' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentHandler.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentRendererInterface' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentRendererInterface.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentUriGenerator' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentUriGenerator.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentUriGeneratorInterface' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentUriGeneratorInterface.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\HIncludeFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/HIncludeFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\InlineFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/InlineFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\RoutableFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/RoutableFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\SsiFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/SsiFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\AbstractSurrogate' => $vendorDir . '/symfony/http-kernel/HttpCache/AbstractSurrogate.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\Esi' => $vendorDir . '/symfony/http-kernel/HttpCache/Esi.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\HttpCache' => $vendorDir . '/symfony/http-kernel/HttpCache/HttpCache.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\ResponseCacheStrategy' => $vendorDir . '/symfony/http-kernel/HttpCache/ResponseCacheStrategy.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\ResponseCacheStrategyInterface' => $vendorDir . '/symfony/http-kernel/HttpCache/ResponseCacheStrategyInterface.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\Ssi' => $vendorDir . '/symfony/http-kernel/HttpCache/Ssi.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\Store' => $vendorDir . '/symfony/http-kernel/HttpCache/Store.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\StoreInterface' => $vendorDir . '/symfony/http-kernel/HttpCache/StoreInterface.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\SubRequestHandler' => $vendorDir . '/symfony/http-kernel/HttpCache/SubRequestHandler.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\SurrogateInterface' => $vendorDir . '/symfony/http-kernel/HttpCache/SurrogateInterface.php',
    'Symfony\\Component\\HttpKernel\\HttpClientKernel' => $vendorDir . '/symfony/http-kernel/HttpClientKernel.php',
    'Symfony\\Component\\HttpKernel\\HttpKernel' => $vendorDir . '/symfony/http-kernel/HttpKernel.php',
    'Symfony\\Component\\HttpKernel\\HttpKernelBrowser' => $vendorDir . '/symfony/http-kernel/HttpKernelBrowser.php',
    'Symfony\\Component\\HttpKernel\\HttpKernelInterface' => $vendorDir . '/symfony/http-kernel/HttpKernelInterface.php',
    'Symfony\\Component\\HttpKernel\\Kernel' => $vendorDir . '/symfony/http-kernel/Kernel.php',
    'Symfony\\Component\\HttpKernel\\KernelEvents' => $vendorDir . '/symfony/http-kernel/KernelEvents.php',
    'Symfony\\Component\\HttpKernel\\KernelInterface' => $vendorDir . '/symfony/http-kernel/KernelInterface.php',
    'Symfony\\Component\\HttpKernel\\Log\\DebugLoggerConfigurator' => $vendorDir . '/symfony/http-kernel/Log/DebugLoggerConfigurator.php',
    'Symfony\\Component\\HttpKernel\\Log\\DebugLoggerInterface' => $vendorDir . '/symfony/http-kernel/Log/DebugLoggerInterface.php',
    'Symfony\\Component\\HttpKernel\\Log\\Logger' => $vendorDir . '/symfony/http-kernel/Log/Logger.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\FileProfilerStorage' => $vendorDir . '/symfony/http-kernel/Profiler/FileProfilerStorage.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\Profile' => $vendorDir . '/symfony/http-kernel/Profiler/Profile.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\Profiler' => $vendorDir . '/symfony/http-kernel/Profiler/Profiler.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\ProfilerStorageInterface' => $vendorDir . '/symfony/http-kernel/Profiler/ProfilerStorageInterface.php',
    'Symfony\\Component\\HttpKernel\\RebootableInterface' => $vendorDir . '/symfony/http-kernel/RebootableInterface.php',
    'Symfony\\Component\\HttpKernel\\TerminableInterface' => $vendorDir . '/symfony/http-kernel/TerminableInterface.php',
    'Symfony\\Component\\HttpKernel\\UriSigner' => $vendorDir . '/symfony/http-kernel/UriSigner.php',
    'Symfony\\Component\\Mailer\\Command\\MailerTestCommand' => $vendorDir . '/symfony/mailer/Command/MailerTestCommand.php',
    'Symfony\\Component\\Mailer\\DataCollector\\MessageDataCollector' => $vendorDir . '/symfony/mailer/DataCollector/MessageDataCollector.php',
    'Symfony\\Component\\Mailer\\DelayedEnvelope' => $vendorDir . '/symfony/mailer/DelayedEnvelope.php',
    'Symfony\\Component\\Mailer\\Envelope' => $vendorDir . '/symfony/mailer/Envelope.php',
    'Symfony\\Component\\Mailer\\EventListener\\EnvelopeListener' => $vendorDir . '/symfony/mailer/EventListener/EnvelopeListener.php',
    'Symfony\\Component\\Mailer\\EventListener\\MessageListener' => $vendorDir . '/symfony/mailer/EventListener/MessageListener.php',
    'Symfony\\Component\\Mailer\\EventListener\\MessageLoggerListener' => $vendorDir . '/symfony/mailer/EventListener/MessageLoggerListener.php',
    'Symfony\\Component\\Mailer\\EventListener\\MessengerTransportListener' => $vendorDir . '/symfony/mailer/EventListener/MessengerTransportListener.php',
    'Symfony\\Component\\Mailer\\Event\\FailedMessageEvent' => $vendorDir . '/symfony/mailer/Event/FailedMessageEvent.php',
    'Symfony\\Component\\Mailer\\Event\\MessageEvent' => $vendorDir . '/symfony/mailer/Event/MessageEvent.php',
    'Symfony\\Component\\Mailer\\Event\\MessageEvents' => $vendorDir . '/symfony/mailer/Event/MessageEvents.php',
    'Symfony\\Component\\Mailer\\Event\\SentMessageEvent' => $vendorDir . '/symfony/mailer/Event/SentMessageEvent.php',
    'Symfony\\Component\\Mailer\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/mailer/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Mailer\\Exception\\HttpTransportException' => $vendorDir . '/symfony/mailer/Exception/HttpTransportException.php',
    'Symfony\\Component\\Mailer\\Exception\\IncompleteDsnException' => $vendorDir . '/symfony/mailer/Exception/IncompleteDsnException.php',
    'Symfony\\Component\\Mailer\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/mailer/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Mailer\\Exception\\LogicException' => $vendorDir . '/symfony/mailer/Exception/LogicException.php',
    'Symfony\\Component\\Mailer\\Exception\\RuntimeException' => $vendorDir . '/symfony/mailer/Exception/RuntimeException.php',
    'Symfony\\Component\\Mailer\\Exception\\TransportException' => $vendorDir . '/symfony/mailer/Exception/TransportException.php',
    'Symfony\\Component\\Mailer\\Exception\\TransportExceptionInterface' => $vendorDir . '/symfony/mailer/Exception/TransportExceptionInterface.php',
    'Symfony\\Component\\Mailer\\Exception\\UnexpectedResponseException' => $vendorDir . '/symfony/mailer/Exception/UnexpectedResponseException.php',
    'Symfony\\Component\\Mailer\\Exception\\UnsupportedSchemeException' => $vendorDir . '/symfony/mailer/Exception/UnsupportedSchemeException.php',
    'Symfony\\Component\\Mailer\\Header\\MetadataHeader' => $vendorDir . '/symfony/mailer/Header/MetadataHeader.php',
    'Symfony\\Component\\Mailer\\Header\\TagHeader' => $vendorDir . '/symfony/mailer/Header/TagHeader.php',
    'Symfony\\Component\\Mailer\\Mailer' => $vendorDir . '/symfony/mailer/Mailer.php',
    'Symfony\\Component\\Mailer\\MailerInterface' => $vendorDir . '/symfony/mailer/MailerInterface.php',
    'Symfony\\Component\\Mailer\\Messenger\\MessageHandler' => $vendorDir . '/symfony/mailer/Messenger/MessageHandler.php',
    'Symfony\\Component\\Mailer\\Messenger\\SendEmailMessage' => $vendorDir . '/symfony/mailer/Messenger/SendEmailMessage.php',
    'Symfony\\Component\\Mailer\\SentMessage' => $vendorDir . '/symfony/mailer/SentMessage.php',
    'Symfony\\Component\\Mailer\\Test\\Constraint\\EmailCount' => $vendorDir . '/symfony/mailer/Test/Constraint/EmailCount.php',
    'Symfony\\Component\\Mailer\\Test\\Constraint\\EmailIsQueued' => $vendorDir . '/symfony/mailer/Test/Constraint/EmailIsQueued.php',
    'Symfony\\Component\\Mailer\\Test\\TransportFactoryTestCase' => $vendorDir . '/symfony/mailer/Test/TransportFactoryTestCase.php',
    'Symfony\\Component\\Mailer\\Transport' => $vendorDir . '/symfony/mailer/Transport.php',
    'Symfony\\Component\\Mailer\\Transport\\AbstractApiTransport' => $vendorDir . '/symfony/mailer/Transport/AbstractApiTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\AbstractHttpTransport' => $vendorDir . '/symfony/mailer/Transport/AbstractHttpTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\AbstractTransport' => $vendorDir . '/symfony/mailer/Transport/AbstractTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\AbstractTransportFactory' => $vendorDir . '/symfony/mailer/Transport/AbstractTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\Dsn' => $vendorDir . '/symfony/mailer/Transport/Dsn.php',
    'Symfony\\Component\\Mailer\\Transport\\FailoverTransport' => $vendorDir . '/symfony/mailer/Transport/FailoverTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\NativeTransportFactory' => $vendorDir . '/symfony/mailer/Transport/NativeTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\NullTransport' => $vendorDir . '/symfony/mailer/Transport/NullTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\NullTransportFactory' => $vendorDir . '/symfony/mailer/Transport/NullTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\RoundRobinTransport' => $vendorDir . '/symfony/mailer/Transport/RoundRobinTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\SendmailTransport' => $vendorDir . '/symfony/mailer/Transport/SendmailTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\SendmailTransportFactory' => $vendorDir . '/symfony/mailer/Transport/SendmailTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\AuthenticatorInterface' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/AuthenticatorInterface.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\CramMd5Authenticator' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/CramMd5Authenticator.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\LoginAuthenticator' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/LoginAuthenticator.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\PlainAuthenticator' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/PlainAuthenticator.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\XOAuth2Authenticator' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/XOAuth2Authenticator.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\EsmtpTransport' => $vendorDir . '/symfony/mailer/Transport/Smtp/EsmtpTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\EsmtpTransportFactory' => $vendorDir . '/symfony/mailer/Transport/Smtp/EsmtpTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\SmtpTransport' => $vendorDir . '/symfony/mailer/Transport/Smtp/SmtpTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Stream\\AbstractStream' => $vendorDir . '/symfony/mailer/Transport/Smtp/Stream/AbstractStream.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Stream\\ProcessStream' => $vendorDir . '/symfony/mailer/Transport/Smtp/Stream/ProcessStream.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Stream\\SocketStream' => $vendorDir . '/symfony/mailer/Transport/Smtp/Stream/SocketStream.php',
    'Symfony\\Component\\Mailer\\Transport\\TransportFactoryInterface' => $vendorDir . '/symfony/mailer/Transport/TransportFactoryInterface.php',
    'Symfony\\Component\\Mailer\\Transport\\TransportInterface' => $vendorDir . '/symfony/mailer/Transport/TransportInterface.php',
    'Symfony\\Component\\Mailer\\Transport\\Transports' => $vendorDir . '/symfony/mailer/Transport/Transports.php',
    'Symfony\\Component\\Mime\\Address' => $vendorDir . '/symfony/mime/Address.php',
    'Symfony\\Component\\Mime\\BodyRendererInterface' => $vendorDir . '/symfony/mime/BodyRendererInterface.php',
    'Symfony\\Component\\Mime\\CharacterStream' => $vendorDir . '/symfony/mime/CharacterStream.php',
    'Symfony\\Component\\Mime\\Crypto\\DkimOptions' => $vendorDir . '/symfony/mime/Crypto/DkimOptions.php',
    'Symfony\\Component\\Mime\\Crypto\\DkimSigner' => $vendorDir . '/symfony/mime/Crypto/DkimSigner.php',
    'Symfony\\Component\\Mime\\Crypto\\SMime' => $vendorDir . '/symfony/mime/Crypto/SMime.php',
    'Symfony\\Component\\Mime\\Crypto\\SMimeEncrypter' => $vendorDir . '/symfony/mime/Crypto/SMimeEncrypter.php',
    'Symfony\\Component\\Mime\\Crypto\\SMimeSigner' => $vendorDir . '/symfony/mime/Crypto/SMimeSigner.php',
    'Symfony\\Component\\Mime\\DependencyInjection\\AddMimeTypeGuesserPass' => $vendorDir . '/symfony/mime/DependencyInjection/AddMimeTypeGuesserPass.php',
    'Symfony\\Component\\Mime\\DraftEmail' => $vendorDir . '/symfony/mime/DraftEmail.php',
    'Symfony\\Component\\Mime\\Email' => $vendorDir . '/symfony/mime/Email.php',
    'Symfony\\Component\\Mime\\Encoder\\AddressEncoderInterface' => $vendorDir . '/symfony/mime/Encoder/AddressEncoderInterface.php',
    'Symfony\\Component\\Mime\\Encoder\\Base64ContentEncoder' => $vendorDir . '/symfony/mime/Encoder/Base64ContentEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\Base64Encoder' => $vendorDir . '/symfony/mime/Encoder/Base64Encoder.php',
    'Symfony\\Component\\Mime\\Encoder\\Base64MimeHeaderEncoder' => $vendorDir . '/symfony/mime/Encoder/Base64MimeHeaderEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\ContentEncoderInterface' => $vendorDir . '/symfony/mime/Encoder/ContentEncoderInterface.php',
    'Symfony\\Component\\Mime\\Encoder\\EightBitContentEncoder' => $vendorDir . '/symfony/mime/Encoder/EightBitContentEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\EncoderInterface' => $vendorDir . '/symfony/mime/Encoder/EncoderInterface.php',
    'Symfony\\Component\\Mime\\Encoder\\IdnAddressEncoder' => $vendorDir . '/symfony/mime/Encoder/IdnAddressEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\MimeHeaderEncoderInterface' => $vendorDir . '/symfony/mime/Encoder/MimeHeaderEncoderInterface.php',
    'Symfony\\Component\\Mime\\Encoder\\QpContentEncoder' => $vendorDir . '/symfony/mime/Encoder/QpContentEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\QpEncoder' => $vendorDir . '/symfony/mime/Encoder/QpEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\QpMimeHeaderEncoder' => $vendorDir . '/symfony/mime/Encoder/QpMimeHeaderEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\Rfc2231Encoder' => $vendorDir . '/symfony/mime/Encoder/Rfc2231Encoder.php',
    'Symfony\\Component\\Mime\\Exception\\AddressEncoderException' => $vendorDir . '/symfony/mime/Exception/AddressEncoderException.php',
    'Symfony\\Component\\Mime\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/mime/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Mime\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/mime/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Mime\\Exception\\LogicException' => $vendorDir . '/symfony/mime/Exception/LogicException.php',
    'Symfony\\Component\\Mime\\Exception\\RfcComplianceException' => $vendorDir . '/symfony/mime/Exception/RfcComplianceException.php',
    'Symfony\\Component\\Mime\\Exception\\RuntimeException' => $vendorDir . '/symfony/mime/Exception/RuntimeException.php',
    'Symfony\\Component\\Mime\\FileBinaryMimeTypeGuesser' => $vendorDir . '/symfony/mime/FileBinaryMimeTypeGuesser.php',
    'Symfony\\Component\\Mime\\FileinfoMimeTypeGuesser' => $vendorDir . '/symfony/mime/FileinfoMimeTypeGuesser.php',
    'Symfony\\Component\\Mime\\Header\\AbstractHeader' => $vendorDir . '/symfony/mime/Header/AbstractHeader.php',
    'Symfony\\Component\\Mime\\Header\\DateHeader' => $vendorDir . '/symfony/mime/Header/DateHeader.php',
    'Symfony\\Component\\Mime\\Header\\HeaderInterface' => $vendorDir . '/symfony/mime/Header/HeaderInterface.php',
    'Symfony\\Component\\Mime\\Header\\Headers' => $vendorDir . '/symfony/mime/Header/Headers.php',
    'Symfony\\Component\\Mime\\Header\\IdentificationHeader' => $vendorDir . '/symfony/mime/Header/IdentificationHeader.php',
    'Symfony\\Component\\Mime\\Header\\MailboxHeader' => $vendorDir . '/symfony/mime/Header/MailboxHeader.php',
    'Symfony\\Component\\Mime\\Header\\MailboxListHeader' => $vendorDir . '/symfony/mime/Header/MailboxListHeader.php',
    'Symfony\\Component\\Mime\\Header\\ParameterizedHeader' => $vendorDir . '/symfony/mime/Header/ParameterizedHeader.php',
    'Symfony\\Component\\Mime\\Header\\PathHeader' => $vendorDir . '/symfony/mime/Header/PathHeader.php',
    'Symfony\\Component\\Mime\\Header\\UnstructuredHeader' => $vendorDir . '/symfony/mime/Header/UnstructuredHeader.php',
    'Symfony\\Component\\Mime\\HtmlToTextConverter\\DefaultHtmlToTextConverter' => $vendorDir . '/symfony/mime/HtmlToTextConverter/DefaultHtmlToTextConverter.php',
    'Symfony\\Component\\Mime\\HtmlToTextConverter\\HtmlToTextConverterInterface' => $vendorDir . '/symfony/mime/HtmlToTextConverter/HtmlToTextConverterInterface.php',
    'Symfony\\Component\\Mime\\HtmlToTextConverter\\LeagueHtmlToMarkdownConverter' => $vendorDir . '/symfony/mime/HtmlToTextConverter/LeagueHtmlToMarkdownConverter.php',
    'Symfony\\Component\\Mime\\Message' => $vendorDir . '/symfony/mime/Message.php',
    'Symfony\\Component\\Mime\\MessageConverter' => $vendorDir . '/symfony/mime/MessageConverter.php',
    'Symfony\\Component\\Mime\\MimeTypeGuesserInterface' => $vendorDir . '/symfony/mime/MimeTypeGuesserInterface.php',
    'Symfony\\Component\\Mime\\MimeTypes' => $vendorDir . '/symfony/mime/MimeTypes.php',
    'Symfony\\Component\\Mime\\MimeTypesInterface' => $vendorDir . '/symfony/mime/MimeTypesInterface.php',
    'Symfony\\Component\\Mime\\Part\\AbstractMultipartPart' => $vendorDir . '/symfony/mime/Part/AbstractMultipartPart.php',
    'Symfony\\Component\\Mime\\Part\\AbstractPart' => $vendorDir . '/symfony/mime/Part/AbstractPart.php',
    'Symfony\\Component\\Mime\\Part\\DataPart' => $vendorDir . '/symfony/mime/Part/DataPart.php',
    'Symfony\\Component\\Mime\\Part\\File' => $vendorDir . '/symfony/mime/Part/File.php',
    'Symfony\\Component\\Mime\\Part\\MessagePart' => $vendorDir . '/symfony/mime/Part/MessagePart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\AlternativePart' => $vendorDir . '/symfony/mime/Part/Multipart/AlternativePart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\DigestPart' => $vendorDir . '/symfony/mime/Part/Multipart/DigestPart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\FormDataPart' => $vendorDir . '/symfony/mime/Part/Multipart/FormDataPart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\MixedPart' => $vendorDir . '/symfony/mime/Part/Multipart/MixedPart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\RelatedPart' => $vendorDir . '/symfony/mime/Part/Multipart/RelatedPart.php',
    'Symfony\\Component\\Mime\\Part\\SMimePart' => $vendorDir . '/symfony/mime/Part/SMimePart.php',
    'Symfony\\Component\\Mime\\Part\\TextPart' => $vendorDir . '/symfony/mime/Part/TextPart.php',
    'Symfony\\Component\\Mime\\RawMessage' => $vendorDir . '/symfony/mime/RawMessage.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailAddressContains' => $vendorDir . '/symfony/mime/Test/Constraint/EmailAddressContains.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailAttachmentCount' => $vendorDir . '/symfony/mime/Test/Constraint/EmailAttachmentCount.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailHasHeader' => $vendorDir . '/symfony/mime/Test/Constraint/EmailHasHeader.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailHeaderSame' => $vendorDir . '/symfony/mime/Test/Constraint/EmailHeaderSame.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailHtmlBodyContains' => $vendorDir . '/symfony/mime/Test/Constraint/EmailHtmlBodyContains.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailSubjectContains' => $vendorDir . '/symfony/mime/Test/Constraint/EmailSubjectContains.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailTextBodyContains' => $vendorDir . '/symfony/mime/Test/Constraint/EmailTextBodyContains.php',
    'Symfony\\Component\\Process\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/process/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Process\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/process/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Process\\Exception\\LogicException' => $vendorDir . '/symfony/process/Exception/LogicException.php',
    'Symfony\\Component\\Process\\Exception\\ProcessFailedException' => $vendorDir . '/symfony/process/Exception/ProcessFailedException.php',
    'Symfony\\Component\\Process\\Exception\\ProcessSignaledException' => $vendorDir . '/symfony/process/Exception/ProcessSignaledException.php',
    'Symfony\\Component\\Process\\Exception\\ProcessTimedOutException' => $vendorDir . '/symfony/process/Exception/ProcessTimedOutException.php',
    'Symfony\\Component\\Process\\Exception\\RunProcessFailedException' => $vendorDir . '/symfony/process/Exception/RunProcessFailedException.php',
    'Symfony\\Component\\Process\\Exception\\RuntimeException' => $vendorDir . '/symfony/process/Exception/RuntimeException.php',
    'Symfony\\Component\\Process\\ExecutableFinder' => $vendorDir . '/symfony/process/ExecutableFinder.php',
    'Symfony\\Component\\Process\\InputStream' => $vendorDir . '/symfony/process/InputStream.php',
    'Symfony\\Component\\Process\\Messenger\\RunProcessContext' => $vendorDir . '/symfony/process/Messenger/RunProcessContext.php',
    'Symfony\\Component\\Process\\Messenger\\RunProcessMessage' => $vendorDir . '/symfony/process/Messenger/RunProcessMessage.php',
    'Symfony\\Component\\Process\\Messenger\\RunProcessMessageHandler' => $vendorDir . '/symfony/process/Messenger/RunProcessMessageHandler.php',
    'Symfony\\Component\\Process\\PhpExecutableFinder' => $vendorDir . '/symfony/process/PhpExecutableFinder.php',
    'Symfony\\Component\\Process\\PhpProcess' => $vendorDir . '/symfony/process/PhpProcess.php',
    'Symfony\\Component\\Process\\PhpSubprocess' => $vendorDir . '/symfony/process/PhpSubprocess.php',
    'Symfony\\Component\\Process\\Pipes\\AbstractPipes' => $vendorDir . '/symfony/process/Pipes/AbstractPipes.php',
    'Symfony\\Component\\Process\\Pipes\\PipesInterface' => $vendorDir . '/symfony/process/Pipes/PipesInterface.php',
    'Symfony\\Component\\Process\\Pipes\\UnixPipes' => $vendorDir . '/symfony/process/Pipes/UnixPipes.php',
    'Symfony\\Component\\Process\\Pipes\\WindowsPipes' => $vendorDir . '/symfony/process/Pipes/WindowsPipes.php',
    'Symfony\\Component\\Process\\Process' => $vendorDir . '/symfony/process/Process.php',
    'Symfony\\Component\\Process\\ProcessUtils' => $vendorDir . '/symfony/process/ProcessUtils.php',
    'Symfony\\Component\\Routing\\Alias' => $vendorDir . '/symfony/routing/Alias.php',
    'Symfony\\Component\\Routing\\Annotation\\Route' => $vendorDir . '/symfony/routing/Annotation/Route.php',
    'Symfony\\Component\\Routing\\Attribute\\Route' => $vendorDir . '/symfony/routing/Attribute/Route.php',
    'Symfony\\Component\\Routing\\CompiledRoute' => $vendorDir . '/symfony/routing/CompiledRoute.php',
    'Symfony\\Component\\Routing\\DependencyInjection\\AddExpressionLanguageProvidersPass' => $vendorDir . '/symfony/routing/DependencyInjection/AddExpressionLanguageProvidersPass.php',
    'Symfony\\Component\\Routing\\DependencyInjection\\RoutingResolverPass' => $vendorDir . '/symfony/routing/DependencyInjection/RoutingResolverPass.php',
    'Symfony\\Component\\Routing\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/routing/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Routing\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/routing/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Routing\\Exception\\InvalidParameterException' => $vendorDir . '/symfony/routing/Exception/InvalidParameterException.php',
    'Symfony\\Component\\Routing\\Exception\\MethodNotAllowedException' => $vendorDir . '/symfony/routing/Exception/MethodNotAllowedException.php',
    'Symfony\\Component\\Routing\\Exception\\MissingMandatoryParametersException' => $vendorDir . '/symfony/routing/Exception/MissingMandatoryParametersException.php',
    'Symfony\\Component\\Routing\\Exception\\NoConfigurationException' => $vendorDir . '/symfony/routing/Exception/NoConfigurationException.php',
    'Symfony\\Component\\Routing\\Exception\\ResourceNotFoundException' => $vendorDir . '/symfony/routing/Exception/ResourceNotFoundException.php',
    'Symfony\\Component\\Routing\\Exception\\RouteCircularReferenceException' => $vendorDir . '/symfony/routing/Exception/RouteCircularReferenceException.php',
    'Symfony\\Component\\Routing\\Exception\\RouteNotFoundException' => $vendorDir . '/symfony/routing/Exception/RouteNotFoundException.php',
    'Symfony\\Component\\Routing\\Exception\\RuntimeException' => $vendorDir . '/symfony/routing/Exception/RuntimeException.php',
    'Symfony\\Component\\Routing\\Generator\\CompiledUrlGenerator' => $vendorDir . '/symfony/routing/Generator/CompiledUrlGenerator.php',
    'Symfony\\Component\\Routing\\Generator\\ConfigurableRequirementsInterface' => $vendorDir . '/symfony/routing/Generator/ConfigurableRequirementsInterface.php',
    'Symfony\\Component\\Routing\\Generator\\Dumper\\CompiledUrlGeneratorDumper' => $vendorDir . '/symfony/routing/Generator/Dumper/CompiledUrlGeneratorDumper.php',
    'Symfony\\Component\\Routing\\Generator\\Dumper\\GeneratorDumper' => $vendorDir . '/symfony/routing/Generator/Dumper/GeneratorDumper.php',
    'Symfony\\Component\\Routing\\Generator\\Dumper\\GeneratorDumperInterface' => $vendorDir . '/symfony/routing/Generator/Dumper/GeneratorDumperInterface.php',
    'Symfony\\Component\\Routing\\Generator\\UrlGenerator' => $vendorDir . '/symfony/routing/Generator/UrlGenerator.php',
    'Symfony\\Component\\Routing\\Generator\\UrlGeneratorInterface' => $vendorDir . '/symfony/routing/Generator/UrlGeneratorInterface.php',
    'Symfony\\Component\\Routing\\Loader\\AnnotationClassLoader' => $vendorDir . '/symfony/routing/Loader/AnnotationClassLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AnnotationDirectoryLoader' => $vendorDir . '/symfony/routing/Loader/AnnotationDirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AnnotationFileLoader' => $vendorDir . '/symfony/routing/Loader/AnnotationFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeClassLoader' => $vendorDir . '/symfony/routing/Loader/AttributeClassLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeDirectoryLoader' => $vendorDir . '/symfony/routing/Loader/AttributeDirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeFileLoader' => $vendorDir . '/symfony/routing/Loader/AttributeFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\ClosureLoader' => $vendorDir . '/symfony/routing/Loader/ClosureLoader.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\AliasConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/AliasConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\CollectionConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/CollectionConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\ImportConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/ImportConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\RouteConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/RouteConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\RoutingConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/RoutingConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\AddTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/AddTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\HostTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/HostTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\LocalizedRouteTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/LocalizedRouteTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\PrefixTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/PrefixTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\RouteTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/RouteTrait.php',
    'Symfony\\Component\\Routing\\Loader\\ContainerLoader' => $vendorDir . '/symfony/routing/Loader/ContainerLoader.php',
    'Symfony\\Component\\Routing\\Loader\\DirectoryLoader' => $vendorDir . '/symfony/routing/Loader/DirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\GlobFileLoader' => $vendorDir . '/symfony/routing/Loader/GlobFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\ObjectLoader' => $vendorDir . '/symfony/routing/Loader/ObjectLoader.php',
    'Symfony\\Component\\Routing\\Loader\\PhpFileLoader' => $vendorDir . '/symfony/routing/Loader/PhpFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\Psr4DirectoryLoader' => $vendorDir . '/symfony/routing/Loader/Psr4DirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\XmlFileLoader' => $vendorDir . '/symfony/routing/Loader/XmlFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\YamlFileLoader' => $vendorDir . '/symfony/routing/Loader/YamlFileLoader.php',
    'Symfony\\Component\\Routing\\Matcher\\CompiledUrlMatcher' => $vendorDir . '/symfony/routing/Matcher/CompiledUrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\CompiledUrlMatcherDumper' => $vendorDir . '/symfony/routing/Matcher/Dumper/CompiledUrlMatcherDumper.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\CompiledUrlMatcherTrait' => $vendorDir . '/symfony/routing/Matcher/Dumper/CompiledUrlMatcherTrait.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\MatcherDumper' => $vendorDir . '/symfony/routing/Matcher/Dumper/MatcherDumper.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\MatcherDumperInterface' => $vendorDir . '/symfony/routing/Matcher/Dumper/MatcherDumperInterface.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\StaticPrefixCollection' => $vendorDir . '/symfony/routing/Matcher/Dumper/StaticPrefixCollection.php',
    'Symfony\\Component\\Routing\\Matcher\\ExpressionLanguageProvider' => $vendorDir . '/symfony/routing/Matcher/ExpressionLanguageProvider.php',
    'Symfony\\Component\\Routing\\Matcher\\RedirectableUrlMatcher' => $vendorDir . '/symfony/routing/Matcher/RedirectableUrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\RedirectableUrlMatcherInterface' => $vendorDir . '/symfony/routing/Matcher/RedirectableUrlMatcherInterface.php',
    'Symfony\\Component\\Routing\\Matcher\\RequestMatcherInterface' => $vendorDir . '/symfony/routing/Matcher/RequestMatcherInterface.php',
    'Symfony\\Component\\Routing\\Matcher\\TraceableUrlMatcher' => $vendorDir . '/symfony/routing/Matcher/TraceableUrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\UrlMatcher' => $vendorDir . '/symfony/routing/Matcher/UrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\UrlMatcherInterface' => $vendorDir . '/symfony/routing/Matcher/UrlMatcherInterface.php',
    'Symfony\\Component\\Routing\\RequestContext' => $vendorDir . '/symfony/routing/RequestContext.php',
    'Symfony\\Component\\Routing\\RequestContextAwareInterface' => $vendorDir . '/symfony/routing/RequestContextAwareInterface.php',
    'Symfony\\Component\\Routing\\Requirement\\EnumRequirement' => $vendorDir . '/symfony/routing/Requirement/EnumRequirement.php',
    'Symfony\\Component\\Routing\\Requirement\\Requirement' => $vendorDir . '/symfony/routing/Requirement/Requirement.php',
    'Symfony\\Component\\Routing\\Route' => $vendorDir . '/symfony/routing/Route.php',
    'Symfony\\Component\\Routing\\RouteCollection' => $vendorDir . '/symfony/routing/RouteCollection.php',
    'Symfony\\Component\\Routing\\RouteCompiler' => $vendorDir . '/symfony/routing/RouteCompiler.php',
    'Symfony\\Component\\Routing\\RouteCompilerInterface' => $vendorDir . '/symfony/routing/RouteCompilerInterface.php',
    'Symfony\\Component\\Routing\\Router' => $vendorDir . '/symfony/routing/Router.php',
    'Symfony\\Component\\Routing\\RouterInterface' => $vendorDir . '/symfony/routing/RouterInterface.php',
    'Symfony\\Component\\String\\AbstractString' => $vendorDir . '/symfony/string/AbstractString.php',
    'Symfony\\Component\\String\\AbstractUnicodeString' => $vendorDir . '/symfony/string/AbstractUnicodeString.php',
    'Symfony\\Component\\String\\ByteString' => $vendorDir . '/symfony/string/ByteString.php',
    'Symfony\\Component\\String\\CodePointString' => $vendorDir . '/symfony/string/CodePointString.php',
    'Symfony\\Component\\String\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/string/Exception/ExceptionInterface.php',
    'Symfony\\Component\\String\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/string/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\String\\Exception\\RuntimeException' => $vendorDir . '/symfony/string/Exception/RuntimeException.php',
    'Symfony\\Component\\String\\Inflector\\EnglishInflector' => $vendorDir . '/symfony/string/Inflector/EnglishInflector.php',
    'Symfony\\Component\\String\\Inflector\\FrenchInflector' => $vendorDir . '/symfony/string/Inflector/FrenchInflector.php',
    'Symfony\\Component\\String\\Inflector\\InflectorInterface' => $vendorDir . '/symfony/string/Inflector/InflectorInterface.php',
    'Symfony\\Component\\String\\LazyString' => $vendorDir . '/symfony/string/LazyString.php',
    'Symfony\\Component\\String\\Slugger\\AsciiSlugger' => $vendorDir . '/symfony/string/Slugger/AsciiSlugger.php',
    'Symfony\\Component\\String\\Slugger\\SluggerInterface' => $vendorDir . '/symfony/string/Slugger/SluggerInterface.php',
    'Symfony\\Component\\String\\UnicodeString' => $vendorDir . '/symfony/string/UnicodeString.php',
    'Symfony\\Component\\Translation\\CatalogueMetadataAwareInterface' => $vendorDir . '/symfony/translation/CatalogueMetadataAwareInterface.php',
    'Symfony\\Component\\Translation\\Catalogue\\AbstractOperation' => $vendorDir . '/symfony/translation/Catalogue/AbstractOperation.php',
    'Symfony\\Component\\Translation\\Catalogue\\MergeOperation' => $vendorDir . '/symfony/translation/Catalogue/MergeOperation.php',
    'Symfony\\Component\\Translation\\Catalogue\\OperationInterface' => $vendorDir . '/symfony/translation/Catalogue/OperationInterface.php',
    'Symfony\\Component\\Translation\\Catalogue\\TargetOperation' => $vendorDir . '/symfony/translation/Catalogue/TargetOperation.php',
    'Symfony\\Component\\Translation\\Command\\TranslationPullCommand' => $vendorDir . '/symfony/translation/Command/TranslationPullCommand.php',
    'Symfony\\Component\\Translation\\Command\\TranslationPushCommand' => $vendorDir . '/symfony/translation/Command/TranslationPushCommand.php',
    'Symfony\\Component\\Translation\\Command\\TranslationTrait' => $vendorDir . '/symfony/translation/Command/TranslationTrait.php',
    'Symfony\\Component\\Translation\\Command\\XliffLintCommand' => $vendorDir . '/symfony/translation/Command/XliffLintCommand.php',
    'Symfony\\Component\\Translation\\DataCollectorTranslator' => $vendorDir . '/symfony/translation/DataCollectorTranslator.php',
    'Symfony\\Component\\Translation\\DataCollector\\TranslationDataCollector' => $vendorDir . '/symfony/translation/DataCollector/TranslationDataCollector.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\DataCollectorTranslatorPass' => $vendorDir . '/symfony/translation/DependencyInjection/DataCollectorTranslatorPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\LoggingTranslatorPass' => $vendorDir . '/symfony/translation/DependencyInjection/LoggingTranslatorPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\TranslationDumperPass' => $vendorDir . '/symfony/translation/DependencyInjection/TranslationDumperPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\TranslationExtractorPass' => $vendorDir . '/symfony/translation/DependencyInjection/TranslationExtractorPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\TranslatorPass' => $vendorDir . '/symfony/translation/DependencyInjection/TranslatorPass.php',
    'Symfony\\Component\\Translation\\DependencyInjection\\TranslatorPathsPass' => $vendorDir . '/symfony/translation/DependencyInjection/TranslatorPathsPass.php',
    'Symfony\\Component\\Translation\\Dumper\\CsvFileDumper' => $vendorDir . '/symfony/translation/Dumper/CsvFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\DumperInterface' => $vendorDir . '/symfony/translation/Dumper/DumperInterface.php',
    'Symfony\\Component\\Translation\\Dumper\\FileDumper' => $vendorDir . '/symfony/translation/Dumper/FileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\IcuResFileDumper' => $vendorDir . '/symfony/translation/Dumper/IcuResFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\IniFileDumper' => $vendorDir . '/symfony/translation/Dumper/IniFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\JsonFileDumper' => $vendorDir . '/symfony/translation/Dumper/JsonFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\MoFileDumper' => $vendorDir . '/symfony/translation/Dumper/MoFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\PhpFileDumper' => $vendorDir . '/symfony/translation/Dumper/PhpFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\PoFileDumper' => $vendorDir . '/symfony/translation/Dumper/PoFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\QtFileDumper' => $vendorDir . '/symfony/translation/Dumper/QtFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\XliffFileDumper' => $vendorDir . '/symfony/translation/Dumper/XliffFileDumper.php',
    'Symfony\\Component\\Translation\\Dumper\\YamlFileDumper' => $vendorDir . '/symfony/translation/Dumper/YamlFileDumper.php',
    'Symfony\\Component\\Translation\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/translation/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Translation\\Exception\\IncompleteDsnException' => $vendorDir . '/symfony/translation/Exception/IncompleteDsnException.php',
    'Symfony\\Component\\Translation\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/translation/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Translation\\Exception\\InvalidResourceException' => $vendorDir . '/symfony/translation/Exception/InvalidResourceException.php',
    'Symfony\\Component\\Translation\\Exception\\LogicException' => $vendorDir . '/symfony/translation/Exception/LogicException.php',
    'Symfony\\Component\\Translation\\Exception\\MissingRequiredOptionException' => $vendorDir . '/symfony/translation/Exception/MissingRequiredOptionException.php',
    'Symfony\\Component\\Translation\\Exception\\NotFoundResourceException' => $vendorDir . '/symfony/translation/Exception/NotFoundResourceException.php',
    'Symfony\\Component\\Translation\\Exception\\ProviderException' => $vendorDir . '/symfony/translation/Exception/ProviderException.php',
    'Symfony\\Component\\Translation\\Exception\\ProviderExceptionInterface' => $vendorDir . '/symfony/translation/Exception/ProviderExceptionInterface.php',
    'Symfony\\Component\\Translation\\Exception\\RuntimeException' => $vendorDir . '/symfony/translation/Exception/RuntimeException.php',
    'Symfony\\Component\\Translation\\Exception\\UnsupportedSchemeException' => $vendorDir . '/symfony/translation/Exception/UnsupportedSchemeException.php',
    'Symfony\\Component\\Translation\\Extractor\\AbstractFileExtractor' => $vendorDir . '/symfony/translation/Extractor/AbstractFileExtractor.php',
    'Symfony\\Component\\Translation\\Extractor\\ChainExtractor' => $vendorDir . '/symfony/translation/Extractor/ChainExtractor.php',
    'Symfony\\Component\\Translation\\Extractor\\ExtractorInterface' => $vendorDir . '/symfony/translation/Extractor/ExtractorInterface.php',
    'Symfony\\Component\\Translation\\Extractor\\PhpAstExtractor' => $vendorDir . '/symfony/translation/Extractor/PhpAstExtractor.php',
    'Symfony\\Component\\Translation\\Extractor\\PhpExtractor' => $vendorDir . '/symfony/translation/Extractor/PhpExtractor.php',
    'Symfony\\Component\\Translation\\Extractor\\PhpStringTokenParser' => $vendorDir . '/symfony/translation/Extractor/PhpStringTokenParser.php',
    'Symfony\\Component\\Translation\\Extractor\\Visitor\\AbstractVisitor' => $vendorDir . '/symfony/translation/Extractor/Visitor/AbstractVisitor.php',
    'Symfony\\Component\\Translation\\Extractor\\Visitor\\ConstraintVisitor' => $vendorDir . '/symfony/translation/Extractor/Visitor/ConstraintVisitor.php',
    'Symfony\\Component\\Translation\\Extractor\\Visitor\\TransMethodVisitor' => $vendorDir . '/symfony/translation/Extractor/Visitor/TransMethodVisitor.php',
    'Symfony\\Component\\Translation\\Extractor\\Visitor\\TranslatableMessageVisitor' => $vendorDir . '/symfony/translation/Extractor/Visitor/TranslatableMessageVisitor.php',
    'Symfony\\Component\\Translation\\Formatter\\IntlFormatter' => $vendorDir . '/symfony/translation/Formatter/IntlFormatter.php',
    'Symfony\\Component\\Translation\\Formatter\\IntlFormatterInterface' => $vendorDir . '/symfony/translation/Formatter/IntlFormatterInterface.php',
    'Symfony\\Component\\Translation\\Formatter\\MessageFormatter' => $vendorDir . '/symfony/translation/Formatter/MessageFormatter.php',
    'Symfony\\Component\\Translation\\Formatter\\MessageFormatterInterface' => $vendorDir . '/symfony/translation/Formatter/MessageFormatterInterface.php',
    'Symfony\\Component\\Translation\\IdentityTranslator' => $vendorDir . '/symfony/translation/IdentityTranslator.php',
    'Symfony\\Component\\Translation\\Loader\\ArrayLoader' => $vendorDir . '/symfony/translation/Loader/ArrayLoader.php',
    'Symfony\\Component\\Translation\\Loader\\CsvFileLoader' => $vendorDir . '/symfony/translation/Loader/CsvFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\FileLoader' => $vendorDir . '/symfony/translation/Loader/FileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\IcuDatFileLoader' => $vendorDir . '/symfony/translation/Loader/IcuDatFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\IcuResFileLoader' => $vendorDir . '/symfony/translation/Loader/IcuResFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\IniFileLoader' => $vendorDir . '/symfony/translation/Loader/IniFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\JsonFileLoader' => $vendorDir . '/symfony/translation/Loader/JsonFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\LoaderInterface' => $vendorDir . '/symfony/translation/Loader/LoaderInterface.php',
    'Symfony\\Component\\Translation\\Loader\\MoFileLoader' => $vendorDir . '/symfony/translation/Loader/MoFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\PhpFileLoader' => $vendorDir . '/symfony/translation/Loader/PhpFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\PoFileLoader' => $vendorDir . '/symfony/translation/Loader/PoFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\QtFileLoader' => $vendorDir . '/symfony/translation/Loader/QtFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\XliffFileLoader' => $vendorDir . '/symfony/translation/Loader/XliffFileLoader.php',
    'Symfony\\Component\\Translation\\Loader\\YamlFileLoader' => $vendorDir . '/symfony/translation/Loader/YamlFileLoader.php',
    'Symfony\\Component\\Translation\\LocaleSwitcher' => $vendorDir . '/symfony/translation/LocaleSwitcher.php',
    'Symfony\\Component\\Translation\\LoggingTranslator' => $vendorDir . '/symfony/translation/LoggingTranslator.php',
    'Symfony\\Component\\Translation\\MessageCatalogue' => $vendorDir . '/symfony/translation/MessageCatalogue.php',
    'Symfony\\Component\\Translation\\MessageCatalogueInterface' => $vendorDir . '/symfony/translation/MessageCatalogueInterface.php',
    'Symfony\\Component\\Translation\\MetadataAwareInterface' => $vendorDir . '/symfony/translation/MetadataAwareInterface.php',
    'Symfony\\Component\\Translation\\Provider\\AbstractProviderFactory' => $vendorDir . '/symfony/translation/Provider/AbstractProviderFactory.php',
    'Symfony\\Component\\Translation\\Provider\\Dsn' => $vendorDir . '/symfony/translation/Provider/Dsn.php',
    'Symfony\\Component\\Translation\\Provider\\FilteringProvider' => $vendorDir . '/symfony/translation/Provider/FilteringProvider.php',
    'Symfony\\Component\\Translation\\Provider\\NullProvider' => $vendorDir . '/symfony/translation/Provider/NullProvider.php',
    'Symfony\\Component\\Translation\\Provider\\NullProviderFactory' => $vendorDir . '/symfony/translation/Provider/NullProviderFactory.php',
    'Symfony\\Component\\Translation\\Provider\\ProviderFactoryInterface' => $vendorDir . '/symfony/translation/Provider/ProviderFactoryInterface.php',
    'Symfony\\Component\\Translation\\Provider\\ProviderInterface' => $vendorDir . '/symfony/translation/Provider/ProviderInterface.php',
    'Symfony\\Component\\Translation\\Provider\\TranslationProviderCollection' => $vendorDir . '/symfony/translation/Provider/TranslationProviderCollection.php',
    'Symfony\\Component\\Translation\\Provider\\TranslationProviderCollectionFactory' => $vendorDir . '/symfony/translation/Provider/TranslationProviderCollectionFactory.php',
    'Symfony\\Component\\Translation\\PseudoLocalizationTranslator' => $vendorDir . '/symfony/translation/PseudoLocalizationTranslator.php',
    'Symfony\\Component\\Translation\\Reader\\TranslationReader' => $vendorDir . '/symfony/translation/Reader/TranslationReader.php',
    'Symfony\\Component\\Translation\\Reader\\TranslationReaderInterface' => $vendorDir . '/symfony/translation/Reader/TranslationReaderInterface.php',
    'Symfony\\Component\\Translation\\Test\\ProviderFactoryTestCase' => $vendorDir . '/symfony/translation/Test/ProviderFactoryTestCase.php',
    'Symfony\\Component\\Translation\\Test\\ProviderTestCase' => $vendorDir . '/symfony/translation/Test/ProviderTestCase.php',
    'Symfony\\Component\\Translation\\TranslatableMessage' => $vendorDir . '/symfony/translation/TranslatableMessage.php',
    'Symfony\\Component\\Translation\\Translator' => $vendorDir . '/symfony/translation/Translator.php',
    'Symfony\\Component\\Translation\\TranslatorBag' => $vendorDir . '/symfony/translation/TranslatorBag.php',
    'Symfony\\Component\\Translation\\TranslatorBagInterface' => $vendorDir . '/symfony/translation/TranslatorBagInterface.php',
    'Symfony\\Component\\Translation\\Util\\ArrayConverter' => $vendorDir . '/symfony/translation/Util/ArrayConverter.php',
    'Symfony\\Component\\Translation\\Util\\XliffUtils' => $vendorDir . '/symfony/translation/Util/XliffUtils.php',
    'Symfony\\Component\\Translation\\Writer\\TranslationWriter' => $vendorDir . '/symfony/translation/Writer/TranslationWriter.php',
    'Symfony\\Component\\Translation\\Writer\\TranslationWriterInterface' => $vendorDir . '/symfony/translation/Writer/TranslationWriterInterface.php',
    'Symfony\\Component\\Uid\\AbstractUid' => $vendorDir . '/symfony/uid/AbstractUid.php',
    'Symfony\\Component\\Uid\\BinaryUtil' => $vendorDir . '/symfony/uid/BinaryUtil.php',
    'Symfony\\Component\\Uid\\Command\\GenerateUlidCommand' => $vendorDir . '/symfony/uid/Command/GenerateUlidCommand.php',
    'Symfony\\Component\\Uid\\Command\\GenerateUuidCommand' => $vendorDir . '/symfony/uid/Command/GenerateUuidCommand.php',
    'Symfony\\Component\\Uid\\Command\\InspectUlidCommand' => $vendorDir . '/symfony/uid/Command/InspectUlidCommand.php',
    'Symfony\\Component\\Uid\\Command\\InspectUuidCommand' => $vendorDir . '/symfony/uid/Command/InspectUuidCommand.php',
    'Symfony\\Component\\Uid\\Factory\\NameBasedUuidFactory' => $vendorDir . '/symfony/uid/Factory/NameBasedUuidFactory.php',
    'Symfony\\Component\\Uid\\Factory\\RandomBasedUuidFactory' => $vendorDir . '/symfony/uid/Factory/RandomBasedUuidFactory.php',
    'Symfony\\Component\\Uid\\Factory\\TimeBasedUuidFactory' => $vendorDir . '/symfony/uid/Factory/TimeBasedUuidFactory.php',
    'Symfony\\Component\\Uid\\Factory\\UlidFactory' => $vendorDir . '/symfony/uid/Factory/UlidFactory.php',
    'Symfony\\Component\\Uid\\Factory\\UuidFactory' => $vendorDir . '/symfony/uid/Factory/UuidFactory.php',
    'Symfony\\Component\\Uid\\MaxUlid' => $vendorDir . '/symfony/uid/MaxUlid.php',
    'Symfony\\Component\\Uid\\MaxUuid' => $vendorDir . '/symfony/uid/MaxUuid.php',
    'Symfony\\Component\\Uid\\NilUlid' => $vendorDir . '/symfony/uid/NilUlid.php',
    'Symfony\\Component\\Uid\\NilUuid' => $vendorDir . '/symfony/uid/NilUuid.php',
    'Symfony\\Component\\Uid\\TimeBasedUidInterface' => $vendorDir . '/symfony/uid/TimeBasedUidInterface.php',
    'Symfony\\Component\\Uid\\Ulid' => $vendorDir . '/symfony/uid/Ulid.php',
    'Symfony\\Component\\Uid\\Uuid' => $vendorDir . '/symfony/uid/Uuid.php',
    'Symfony\\Component\\Uid\\UuidV1' => $vendorDir . '/symfony/uid/UuidV1.php',
    'Symfony\\Component\\Uid\\UuidV3' => $vendorDir . '/symfony/uid/UuidV3.php',
    'Symfony\\Component\\Uid\\UuidV4' => $vendorDir . '/symfony/uid/UuidV4.php',
    'Symfony\\Component\\Uid\\UuidV5' => $vendorDir . '/symfony/uid/UuidV5.php',
    'Symfony\\Component\\Uid\\UuidV6' => $vendorDir . '/symfony/uid/UuidV6.php',
    'Symfony\\Component\\Uid\\UuidV7' => $vendorDir . '/symfony/uid/UuidV7.php',
    'Symfony\\Component\\Uid\\UuidV8' => $vendorDir . '/symfony/uid/UuidV8.php',
    'Symfony\\Component\\Validator\\Attribute\\HasNamedArguments' => $vendorDir . '/symfony/validator/Attribute/HasNamedArguments.php',
    'Symfony\\Component\\Validator\\Command\\DebugCommand' => $vendorDir . '/symfony/validator/Command/DebugCommand.php',
    'Symfony\\Component\\Validator\\Constraint' => $vendorDir . '/symfony/validator/Constraint.php',
    'Symfony\\Component\\Validator\\ConstraintValidator' => $vendorDir . '/symfony/validator/ConstraintValidator.php',
    'Symfony\\Component\\Validator\\ConstraintValidatorFactory' => $vendorDir . '/symfony/validator/ConstraintValidatorFactory.php',
    'Symfony\\Component\\Validator\\ConstraintValidatorFactoryInterface' => $vendorDir . '/symfony/validator/ConstraintValidatorFactoryInterface.php',
    'Symfony\\Component\\Validator\\ConstraintValidatorInterface' => $vendorDir . '/symfony/validator/ConstraintValidatorInterface.php',
    'Symfony\\Component\\Validator\\ConstraintViolation' => $vendorDir . '/symfony/validator/ConstraintViolation.php',
    'Symfony\\Component\\Validator\\ConstraintViolationInterface' => $vendorDir . '/symfony/validator/ConstraintViolationInterface.php',
    'Symfony\\Component\\Validator\\ConstraintViolationList' => $vendorDir . '/symfony/validator/ConstraintViolationList.php',
    'Symfony\\Component\\Validator\\ConstraintViolationListInterface' => $vendorDir . '/symfony/validator/ConstraintViolationListInterface.php',
    'Symfony\\Component\\Validator\\Constraints\\AbstractComparison' => $vendorDir . '/symfony/validator/Constraints/AbstractComparison.php',
    'Symfony\\Component\\Validator\\Constraints\\AbstractComparisonValidator' => $vendorDir . '/symfony/validator/Constraints/AbstractComparisonValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\All' => $vendorDir . '/symfony/validator/Constraints/All.php',
    'Symfony\\Component\\Validator\\Constraints\\AllValidator' => $vendorDir . '/symfony/validator/Constraints/AllValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\AtLeastOneOf' => $vendorDir . '/symfony/validator/Constraints/AtLeastOneOf.php',
    'Symfony\\Component\\Validator\\Constraints\\AtLeastOneOfValidator' => $vendorDir . '/symfony/validator/Constraints/AtLeastOneOfValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Bic' => $vendorDir . '/symfony/validator/Constraints/Bic.php',
    'Symfony\\Component\\Validator\\Constraints\\BicValidator' => $vendorDir . '/symfony/validator/Constraints/BicValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Blank' => $vendorDir . '/symfony/validator/Constraints/Blank.php',
    'Symfony\\Component\\Validator\\Constraints\\BlankValidator' => $vendorDir . '/symfony/validator/Constraints/BlankValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Callback' => $vendorDir . '/symfony/validator/Constraints/Callback.php',
    'Symfony\\Component\\Validator\\Constraints\\CallbackValidator' => $vendorDir . '/symfony/validator/Constraints/CallbackValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\CardScheme' => $vendorDir . '/symfony/validator/Constraints/CardScheme.php',
    'Symfony\\Component\\Validator\\Constraints\\CardSchemeValidator' => $vendorDir . '/symfony/validator/Constraints/CardSchemeValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Cascade' => $vendorDir . '/symfony/validator/Constraints/Cascade.php',
    'Symfony\\Component\\Validator\\Constraints\\Choice' => $vendorDir . '/symfony/validator/Constraints/Choice.php',
    'Symfony\\Component\\Validator\\Constraints\\ChoiceValidator' => $vendorDir . '/symfony/validator/Constraints/ChoiceValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Cidr' => $vendorDir . '/symfony/validator/Constraints/Cidr.php',
    'Symfony\\Component\\Validator\\Constraints\\CidrValidator' => $vendorDir . '/symfony/validator/Constraints/CidrValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Collection' => $vendorDir . '/symfony/validator/Constraints/Collection.php',
    'Symfony\\Component\\Validator\\Constraints\\CollectionValidator' => $vendorDir . '/symfony/validator/Constraints/CollectionValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Composite' => $vendorDir . '/symfony/validator/Constraints/Composite.php',
    'Symfony\\Component\\Validator\\Constraints\\Compound' => $vendorDir . '/symfony/validator/Constraints/Compound.php',
    'Symfony\\Component\\Validator\\Constraints\\CompoundValidator' => $vendorDir . '/symfony/validator/Constraints/CompoundValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Count' => $vendorDir . '/symfony/validator/Constraints/Count.php',
    'Symfony\\Component\\Validator\\Constraints\\CountValidator' => $vendorDir . '/symfony/validator/Constraints/CountValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Country' => $vendorDir . '/symfony/validator/Constraints/Country.php',
    'Symfony\\Component\\Validator\\Constraints\\CountryValidator' => $vendorDir . '/symfony/validator/Constraints/CountryValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\CssColor' => $vendorDir . '/symfony/validator/Constraints/CssColor.php',
    'Symfony\\Component\\Validator\\Constraints\\CssColorValidator' => $vendorDir . '/symfony/validator/Constraints/CssColorValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Currency' => $vendorDir . '/symfony/validator/Constraints/Currency.php',
    'Symfony\\Component\\Validator\\Constraints\\CurrencyValidator' => $vendorDir . '/symfony/validator/Constraints/CurrencyValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Date' => $vendorDir . '/symfony/validator/Constraints/Date.php',
    'Symfony\\Component\\Validator\\Constraints\\DateTime' => $vendorDir . '/symfony/validator/Constraints/DateTime.php',
    'Symfony\\Component\\Validator\\Constraints\\DateTimeValidator' => $vendorDir . '/symfony/validator/Constraints/DateTimeValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\DateValidator' => $vendorDir . '/symfony/validator/Constraints/DateValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\DisableAutoMapping' => $vendorDir . '/symfony/validator/Constraints/DisableAutoMapping.php',
    'Symfony\\Component\\Validator\\Constraints\\DivisibleBy' => $vendorDir . '/symfony/validator/Constraints/DivisibleBy.php',
    'Symfony\\Component\\Validator\\Constraints\\DivisibleByValidator' => $vendorDir . '/symfony/validator/Constraints/DivisibleByValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Email' => $vendorDir . '/symfony/validator/Constraints/Email.php',
    'Symfony\\Component\\Validator\\Constraints\\EmailValidator' => $vendorDir . '/symfony/validator/Constraints/EmailValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\EnableAutoMapping' => $vendorDir . '/symfony/validator/Constraints/EnableAutoMapping.php',
    'Symfony\\Component\\Validator\\Constraints\\EqualTo' => $vendorDir . '/symfony/validator/Constraints/EqualTo.php',
    'Symfony\\Component\\Validator\\Constraints\\EqualToValidator' => $vendorDir . '/symfony/validator/Constraints/EqualToValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Existence' => $vendorDir . '/symfony/validator/Constraints/Existence.php',
    'Symfony\\Component\\Validator\\Constraints\\Expression' => $vendorDir . '/symfony/validator/Constraints/Expression.php',
    'Symfony\\Component\\Validator\\Constraints\\ExpressionLanguageProvider' => $vendorDir . '/symfony/validator/Constraints/ExpressionLanguageProvider.php',
    'Symfony\\Component\\Validator\\Constraints\\ExpressionLanguageSyntax' => $vendorDir . '/symfony/validator/Constraints/ExpressionLanguageSyntax.php',
    'Symfony\\Component\\Validator\\Constraints\\ExpressionLanguageSyntaxValidator' => $vendorDir . '/symfony/validator/Constraints/ExpressionLanguageSyntaxValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\ExpressionSyntax' => $vendorDir . '/symfony/validator/Constraints/ExpressionSyntax.php',
    'Symfony\\Component\\Validator\\Constraints\\ExpressionSyntaxValidator' => $vendorDir . '/symfony/validator/Constraints/ExpressionSyntaxValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\ExpressionValidator' => $vendorDir . '/symfony/validator/Constraints/ExpressionValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\File' => $vendorDir . '/symfony/validator/Constraints/File.php',
    'Symfony\\Component\\Validator\\Constraints\\FileValidator' => $vendorDir . '/symfony/validator/Constraints/FileValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\GreaterThan' => $vendorDir . '/symfony/validator/Constraints/GreaterThan.php',
    'Symfony\\Component\\Validator\\Constraints\\GreaterThanOrEqual' => $vendorDir . '/symfony/validator/Constraints/GreaterThanOrEqual.php',
    'Symfony\\Component\\Validator\\Constraints\\GreaterThanOrEqualValidator' => $vendorDir . '/symfony/validator/Constraints/GreaterThanOrEqualValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\GreaterThanValidator' => $vendorDir . '/symfony/validator/Constraints/GreaterThanValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\GroupSequence' => $vendorDir . '/symfony/validator/Constraints/GroupSequence.php',
    'Symfony\\Component\\Validator\\Constraints\\GroupSequenceProvider' => $vendorDir . '/symfony/validator/Constraints/GroupSequenceProvider.php',
    'Symfony\\Component\\Validator\\Constraints\\Hostname' => $vendorDir . '/symfony/validator/Constraints/Hostname.php',
    'Symfony\\Component\\Validator\\Constraints\\HostnameValidator' => $vendorDir . '/symfony/validator/Constraints/HostnameValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Iban' => $vendorDir . '/symfony/validator/Constraints/Iban.php',
    'Symfony\\Component\\Validator\\Constraints\\IbanValidator' => $vendorDir . '/symfony/validator/Constraints/IbanValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\IdenticalTo' => $vendorDir . '/symfony/validator/Constraints/IdenticalTo.php',
    'Symfony\\Component\\Validator\\Constraints\\IdenticalToValidator' => $vendorDir . '/symfony/validator/Constraints/IdenticalToValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Image' => $vendorDir . '/symfony/validator/Constraints/Image.php',
    'Symfony\\Component\\Validator\\Constraints\\ImageValidator' => $vendorDir . '/symfony/validator/Constraints/ImageValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Ip' => $vendorDir . '/symfony/validator/Constraints/Ip.php',
    'Symfony\\Component\\Validator\\Constraints\\IpValidator' => $vendorDir . '/symfony/validator/Constraints/IpValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\IsFalse' => $vendorDir . '/symfony/validator/Constraints/IsFalse.php',
    'Symfony\\Component\\Validator\\Constraints\\IsFalseValidator' => $vendorDir . '/symfony/validator/Constraints/IsFalseValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\IsNull' => $vendorDir . '/symfony/validator/Constraints/IsNull.php',
    'Symfony\\Component\\Validator\\Constraints\\IsNullValidator' => $vendorDir . '/symfony/validator/Constraints/IsNullValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\IsTrue' => $vendorDir . '/symfony/validator/Constraints/IsTrue.php',
    'Symfony\\Component\\Validator\\Constraints\\IsTrueValidator' => $vendorDir . '/symfony/validator/Constraints/IsTrueValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Isbn' => $vendorDir . '/symfony/validator/Constraints/Isbn.php',
    'Symfony\\Component\\Validator\\Constraints\\IsbnValidator' => $vendorDir . '/symfony/validator/Constraints/IsbnValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Isin' => $vendorDir . '/symfony/validator/Constraints/Isin.php',
    'Symfony\\Component\\Validator\\Constraints\\IsinValidator' => $vendorDir . '/symfony/validator/Constraints/IsinValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Issn' => $vendorDir . '/symfony/validator/Constraints/Issn.php',
    'Symfony\\Component\\Validator\\Constraints\\IssnValidator' => $vendorDir . '/symfony/validator/Constraints/IssnValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Json' => $vendorDir . '/symfony/validator/Constraints/Json.php',
    'Symfony\\Component\\Validator\\Constraints\\JsonValidator' => $vendorDir . '/symfony/validator/Constraints/JsonValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Language' => $vendorDir . '/symfony/validator/Constraints/Language.php',
    'Symfony\\Component\\Validator\\Constraints\\LanguageValidator' => $vendorDir . '/symfony/validator/Constraints/LanguageValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Length' => $vendorDir . '/symfony/validator/Constraints/Length.php',
    'Symfony\\Component\\Validator\\Constraints\\LengthValidator' => $vendorDir . '/symfony/validator/Constraints/LengthValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\LessThan' => $vendorDir . '/symfony/validator/Constraints/LessThan.php',
    'Symfony\\Component\\Validator\\Constraints\\LessThanOrEqual' => $vendorDir . '/symfony/validator/Constraints/LessThanOrEqual.php',
    'Symfony\\Component\\Validator\\Constraints\\LessThanOrEqualValidator' => $vendorDir . '/symfony/validator/Constraints/LessThanOrEqualValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\LessThanValidator' => $vendorDir . '/symfony/validator/Constraints/LessThanValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Locale' => $vendorDir . '/symfony/validator/Constraints/Locale.php',
    'Symfony\\Component\\Validator\\Constraints\\LocaleValidator' => $vendorDir . '/symfony/validator/Constraints/LocaleValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Luhn' => $vendorDir . '/symfony/validator/Constraints/Luhn.php',
    'Symfony\\Component\\Validator\\Constraints\\LuhnValidator' => $vendorDir . '/symfony/validator/Constraints/LuhnValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Negative' => $vendorDir . '/symfony/validator/Constraints/Negative.php',
    'Symfony\\Component\\Validator\\Constraints\\NegativeOrZero' => $vendorDir . '/symfony/validator/Constraints/NegativeOrZero.php',
    'Symfony\\Component\\Validator\\Constraints\\NoSuspiciousCharacters' => $vendorDir . '/symfony/validator/Constraints/NoSuspiciousCharacters.php',
    'Symfony\\Component\\Validator\\Constraints\\NoSuspiciousCharactersValidator' => $vendorDir . '/symfony/validator/Constraints/NoSuspiciousCharactersValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\NotBlank' => $vendorDir . '/symfony/validator/Constraints/NotBlank.php',
    'Symfony\\Component\\Validator\\Constraints\\NotBlankValidator' => $vendorDir . '/symfony/validator/Constraints/NotBlankValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\NotCompromisedPassword' => $vendorDir . '/symfony/validator/Constraints/NotCompromisedPassword.php',
    'Symfony\\Component\\Validator\\Constraints\\NotCompromisedPasswordValidator' => $vendorDir . '/symfony/validator/Constraints/NotCompromisedPasswordValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\NotEqualTo' => $vendorDir . '/symfony/validator/Constraints/NotEqualTo.php',
    'Symfony\\Component\\Validator\\Constraints\\NotEqualToValidator' => $vendorDir . '/symfony/validator/Constraints/NotEqualToValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\NotIdenticalTo' => $vendorDir . '/symfony/validator/Constraints/NotIdenticalTo.php',
    'Symfony\\Component\\Validator\\Constraints\\NotIdenticalToValidator' => $vendorDir . '/symfony/validator/Constraints/NotIdenticalToValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\NotNull' => $vendorDir . '/symfony/validator/Constraints/NotNull.php',
    'Symfony\\Component\\Validator\\Constraints\\NotNullValidator' => $vendorDir . '/symfony/validator/Constraints/NotNullValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Optional' => $vendorDir . '/symfony/validator/Constraints/Optional.php',
    'Symfony\\Component\\Validator\\Constraints\\PasswordStrength' => $vendorDir . '/symfony/validator/Constraints/PasswordStrength.php',
    'Symfony\\Component\\Validator\\Constraints\\PasswordStrengthValidator' => $vendorDir . '/symfony/validator/Constraints/PasswordStrengthValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Positive' => $vendorDir . '/symfony/validator/Constraints/Positive.php',
    'Symfony\\Component\\Validator\\Constraints\\PositiveOrZero' => $vendorDir . '/symfony/validator/Constraints/PositiveOrZero.php',
    'Symfony\\Component\\Validator\\Constraints\\Range' => $vendorDir . '/symfony/validator/Constraints/Range.php',
    'Symfony\\Component\\Validator\\Constraints\\RangeValidator' => $vendorDir . '/symfony/validator/Constraints/RangeValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Regex' => $vendorDir . '/symfony/validator/Constraints/Regex.php',
    'Symfony\\Component\\Validator\\Constraints\\RegexValidator' => $vendorDir . '/symfony/validator/Constraints/RegexValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Required' => $vendorDir . '/symfony/validator/Constraints/Required.php',
    'Symfony\\Component\\Validator\\Constraints\\Sequentially' => $vendorDir . '/symfony/validator/Constraints/Sequentially.php',
    'Symfony\\Component\\Validator\\Constraints\\SequentiallyValidator' => $vendorDir . '/symfony/validator/Constraints/SequentiallyValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Time' => $vendorDir . '/symfony/validator/Constraints/Time.php',
    'Symfony\\Component\\Validator\\Constraints\\TimeValidator' => $vendorDir . '/symfony/validator/Constraints/TimeValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Timezone' => $vendorDir . '/symfony/validator/Constraints/Timezone.php',
    'Symfony\\Component\\Validator\\Constraints\\TimezoneValidator' => $vendorDir . '/symfony/validator/Constraints/TimezoneValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Traverse' => $vendorDir . '/symfony/validator/Constraints/Traverse.php',
    'Symfony\\Component\\Validator\\Constraints\\Type' => $vendorDir . '/symfony/validator/Constraints/Type.php',
    'Symfony\\Component\\Validator\\Constraints\\TypeValidator' => $vendorDir . '/symfony/validator/Constraints/TypeValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Ulid' => $vendorDir . '/symfony/validator/Constraints/Ulid.php',
    'Symfony\\Component\\Validator\\Constraints\\UlidValidator' => $vendorDir . '/symfony/validator/Constraints/UlidValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Unique' => $vendorDir . '/symfony/validator/Constraints/Unique.php',
    'Symfony\\Component\\Validator\\Constraints\\UniqueValidator' => $vendorDir . '/symfony/validator/Constraints/UniqueValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Url' => $vendorDir . '/symfony/validator/Constraints/Url.php',
    'Symfony\\Component\\Validator\\Constraints\\UrlValidator' => $vendorDir . '/symfony/validator/Constraints/UrlValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Uuid' => $vendorDir . '/symfony/validator/Constraints/Uuid.php',
    'Symfony\\Component\\Validator\\Constraints\\UuidValidator' => $vendorDir . '/symfony/validator/Constraints/UuidValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\Valid' => $vendorDir . '/symfony/validator/Constraints/Valid.php',
    'Symfony\\Component\\Validator\\Constraints\\ValidValidator' => $vendorDir . '/symfony/validator/Constraints/ValidValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\When' => $vendorDir . '/symfony/validator/Constraints/When.php',
    'Symfony\\Component\\Validator\\Constraints\\WhenValidator' => $vendorDir . '/symfony/validator/Constraints/WhenValidator.php',
    'Symfony\\Component\\Validator\\Constraints\\ZeroComparisonConstraintTrait' => $vendorDir . '/symfony/validator/Constraints/ZeroComparisonConstraintTrait.php',
    'Symfony\\Component\\Validator\\ContainerConstraintValidatorFactory' => $vendorDir . '/symfony/validator/ContainerConstraintValidatorFactory.php',
    'Symfony\\Component\\Validator\\Context\\ExecutionContext' => $vendorDir . '/symfony/validator/Context/ExecutionContext.php',
    'Symfony\\Component\\Validator\\Context\\ExecutionContextFactory' => $vendorDir . '/symfony/validator/Context/ExecutionContextFactory.php',
    'Symfony\\Component\\Validator\\Context\\ExecutionContextFactoryInterface' => $vendorDir . '/symfony/validator/Context/ExecutionContextFactoryInterface.php',
    'Symfony\\Component\\Validator\\Context\\ExecutionContextInterface' => $vendorDir . '/symfony/validator/Context/ExecutionContextInterface.php',
    'Symfony\\Component\\Validator\\DataCollector\\ValidatorDataCollector' => $vendorDir . '/symfony/validator/DataCollector/ValidatorDataCollector.php',
    'Symfony\\Component\\Validator\\DependencyInjection\\AddAutoMappingConfigurationPass' => $vendorDir . '/symfony/validator/DependencyInjection/AddAutoMappingConfigurationPass.php',
    'Symfony\\Component\\Validator\\DependencyInjection\\AddConstraintValidatorsPass' => $vendorDir . '/symfony/validator/DependencyInjection/AddConstraintValidatorsPass.php',
    'Symfony\\Component\\Validator\\DependencyInjection\\AddValidatorInitializersPass' => $vendorDir . '/symfony/validator/DependencyInjection/AddValidatorInitializersPass.php',
    'Symfony\\Component\\Validator\\Exception\\BadMethodCallException' => $vendorDir . '/symfony/validator/Exception/BadMethodCallException.php',
    'Symfony\\Component\\Validator\\Exception\\ConstraintDefinitionException' => $vendorDir . '/symfony/validator/Exception/ConstraintDefinitionException.php',
    'Symfony\\Component\\Validator\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/validator/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Validator\\Exception\\GroupDefinitionException' => $vendorDir . '/symfony/validator/Exception/GroupDefinitionException.php',
    'Symfony\\Component\\Validator\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/validator/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Validator\\Exception\\InvalidOptionsException' => $vendorDir . '/symfony/validator/Exception/InvalidOptionsException.php',
    'Symfony\\Component\\Validator\\Exception\\LogicException' => $vendorDir . '/symfony/validator/Exception/LogicException.php',
    'Symfony\\Component\\Validator\\Exception\\MappingException' => $vendorDir . '/symfony/validator/Exception/MappingException.php',
    'Symfony\\Component\\Validator\\Exception\\MissingOptionsException' => $vendorDir . '/symfony/validator/Exception/MissingOptionsException.php',
    'Symfony\\Component\\Validator\\Exception\\NoSuchMetadataException' => $vendorDir . '/symfony/validator/Exception/NoSuchMetadataException.php',
    'Symfony\\Component\\Validator\\Exception\\OutOfBoundsException' => $vendorDir . '/symfony/validator/Exception/OutOfBoundsException.php',
    'Symfony\\Component\\Validator\\Exception\\RuntimeException' => $vendorDir . '/symfony/validator/Exception/RuntimeException.php',
    'Symfony\\Component\\Validator\\Exception\\UnexpectedTypeException' => $vendorDir . '/symfony/validator/Exception/UnexpectedTypeException.php',
    'Symfony\\Component\\Validator\\Exception\\UnexpectedValueException' => $vendorDir . '/symfony/validator/Exception/UnexpectedValueException.php',
    'Symfony\\Component\\Validator\\Exception\\UnsupportedMetadataException' => $vendorDir . '/symfony/validator/Exception/UnsupportedMetadataException.php',
    'Symfony\\Component\\Validator\\Exception\\ValidationFailedException' => $vendorDir . '/symfony/validator/Exception/ValidationFailedException.php',
    'Symfony\\Component\\Validator\\Exception\\ValidatorException' => $vendorDir . '/symfony/validator/Exception/ValidatorException.php',
    'Symfony\\Component\\Validator\\GroupProviderInterface' => $vendorDir . '/symfony/validator/GroupProviderInterface.php',
    'Symfony\\Component\\Validator\\GroupSequenceProviderInterface' => $vendorDir . '/symfony/validator/GroupSequenceProviderInterface.php',
    'Symfony\\Component\\Validator\\Mapping\\AutoMappingStrategy' => $vendorDir . '/symfony/validator/Mapping/AutoMappingStrategy.php',
    'Symfony\\Component\\Validator\\Mapping\\CascadingStrategy' => $vendorDir . '/symfony/validator/Mapping/CascadingStrategy.php',
    'Symfony\\Component\\Validator\\Mapping\\ClassMetadata' => $vendorDir . '/symfony/validator/Mapping/ClassMetadata.php',
    'Symfony\\Component\\Validator\\Mapping\\ClassMetadataInterface' => $vendorDir . '/symfony/validator/Mapping/ClassMetadataInterface.php',
    'Symfony\\Component\\Validator\\Mapping\\Factory\\BlackHoleMetadataFactory' => $vendorDir . '/symfony/validator/Mapping/Factory/BlackHoleMetadataFactory.php',
    'Symfony\\Component\\Validator\\Mapping\\Factory\\LazyLoadingMetadataFactory' => $vendorDir . '/symfony/validator/Mapping/Factory/LazyLoadingMetadataFactory.php',
    'Symfony\\Component\\Validator\\Mapping\\Factory\\MetadataFactoryInterface' => $vendorDir . '/symfony/validator/Mapping/Factory/MetadataFactoryInterface.php',
    'Symfony\\Component\\Validator\\Mapping\\GenericMetadata' => $vendorDir . '/symfony/validator/Mapping/GenericMetadata.php',
    'Symfony\\Component\\Validator\\Mapping\\GetterMetadata' => $vendorDir . '/symfony/validator/Mapping/GetterMetadata.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\AbstractLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/AbstractLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\AnnotationLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/AnnotationLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\AttributeLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/AttributeLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\AutoMappingTrait' => $vendorDir . '/symfony/validator/Mapping/Loader/AutoMappingTrait.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\FileLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/FileLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\FilesLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/FilesLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\LoaderChain' => $vendorDir . '/symfony/validator/Mapping/Loader/LoaderChain.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\LoaderInterface' => $vendorDir . '/symfony/validator/Mapping/Loader/LoaderInterface.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\PropertyInfoLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/PropertyInfoLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\StaticMethodLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/StaticMethodLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\XmlFileLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/XmlFileLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\XmlFilesLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/XmlFilesLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\YamlFileLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/YamlFileLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\Loader\\YamlFilesLoader' => $vendorDir . '/symfony/validator/Mapping/Loader/YamlFilesLoader.php',
    'Symfony\\Component\\Validator\\Mapping\\MemberMetadata' => $vendorDir . '/symfony/validator/Mapping/MemberMetadata.php',
    'Symfony\\Component\\Validator\\Mapping\\MetadataInterface' => $vendorDir . '/symfony/validator/Mapping/MetadataInterface.php',
    'Symfony\\Component\\Validator\\Mapping\\PropertyMetadata' => $vendorDir . '/symfony/validator/Mapping/PropertyMetadata.php',
    'Symfony\\Component\\Validator\\Mapping\\PropertyMetadataInterface' => $vendorDir . '/symfony/validator/Mapping/PropertyMetadataInterface.php',
    'Symfony\\Component\\Validator\\Mapping\\TraversalStrategy' => $vendorDir . '/symfony/validator/Mapping/TraversalStrategy.php',
    'Symfony\\Component\\Validator\\ObjectInitializerInterface' => $vendorDir . '/symfony/validator/ObjectInitializerInterface.php',
    'Symfony\\Component\\Validator\\Test\\ConstraintValidatorTestCase' => $vendorDir . '/symfony/validator/Test/ConstraintValidatorTestCase.php',
    'Symfony\\Component\\Validator\\Util\\PropertyPath' => $vendorDir . '/symfony/validator/Util/PropertyPath.php',
    'Symfony\\Component\\Validator\\Validation' => $vendorDir . '/symfony/validator/Validation.php',
    'Symfony\\Component\\Validator\\ValidatorBuilder' => $vendorDir . '/symfony/validator/ValidatorBuilder.php',
    'Symfony\\Component\\Validator\\Validator\\ContextualValidatorInterface' => $vendorDir . '/symfony/validator/Validator/ContextualValidatorInterface.php',
    'Symfony\\Component\\Validator\\Validator\\LazyProperty' => $vendorDir . '/symfony/validator/Validator/LazyProperty.php',
    'Symfony\\Component\\Validator\\Validator\\RecursiveContextualValidator' => $vendorDir . '/symfony/validator/Validator/RecursiveContextualValidator.php',
    'Symfony\\Component\\Validator\\Validator\\RecursiveValidator' => $vendorDir . '/symfony/validator/Validator/RecursiveValidator.php',
    'Symfony\\Component\\Validator\\Validator\\TraceableValidator' => $vendorDir . '/symfony/validator/Validator/TraceableValidator.php',
    'Symfony\\Component\\Validator\\Validator\\ValidatorInterface' => $vendorDir . '/symfony/validator/Validator/ValidatorInterface.php',
    'Symfony\\Component\\Validator\\Violation\\ConstraintViolationBuilder' => $vendorDir . '/symfony/validator/Violation/ConstraintViolationBuilder.php',
    'Symfony\\Component\\Validator\\Violation\\ConstraintViolationBuilderInterface' => $vendorDir . '/symfony/validator/Violation/ConstraintViolationBuilderInterface.php',
    'Symfony\\Component\\VarDumper\\Caster\\AmqpCaster' => $vendorDir . '/symfony/var-dumper/Caster/AmqpCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ArgsStub' => $vendorDir . '/symfony/var-dumper/Caster/ArgsStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\Caster' => $vendorDir . '/symfony/var-dumper/Caster/Caster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ClassStub' => $vendorDir . '/symfony/var-dumper/Caster/ClassStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\ConstStub' => $vendorDir . '/symfony/var-dumper/Caster/ConstStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\CutArrayStub' => $vendorDir . '/symfony/var-dumper/Caster/CutArrayStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\CutStub' => $vendorDir . '/symfony/var-dumper/Caster/CutStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\DOMCaster' => $vendorDir . '/symfony/var-dumper/Caster/DOMCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DateCaster' => $vendorDir . '/symfony/var-dumper/Caster/DateCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DoctrineCaster' => $vendorDir . '/symfony/var-dumper/Caster/DoctrineCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DsCaster' => $vendorDir . '/symfony/var-dumper/Caster/DsCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DsPairStub' => $vendorDir . '/symfony/var-dumper/Caster/DsPairStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\EnumStub' => $vendorDir . '/symfony/var-dumper/Caster/EnumStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\ExceptionCaster' => $vendorDir . '/symfony/var-dumper/Caster/ExceptionCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\FFICaster' => $vendorDir . '/symfony/var-dumper/Caster/FFICaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\FiberCaster' => $vendorDir . '/symfony/var-dumper/Caster/FiberCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\FrameStub' => $vendorDir . '/symfony/var-dumper/Caster/FrameStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\GmpCaster' => $vendorDir . '/symfony/var-dumper/Caster/GmpCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ImagineCaster' => $vendorDir . '/symfony/var-dumper/Caster/ImagineCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ImgStub' => $vendorDir . '/symfony/var-dumper/Caster/ImgStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\IntlCaster' => $vendorDir . '/symfony/var-dumper/Caster/IntlCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\LinkStub' => $vendorDir . '/symfony/var-dumper/Caster/LinkStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\MemcachedCaster' => $vendorDir . '/symfony/var-dumper/Caster/MemcachedCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\MysqliCaster' => $vendorDir . '/symfony/var-dumper/Caster/MysqliCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\PdoCaster' => $vendorDir . '/symfony/var-dumper/Caster/PdoCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\PgSqlCaster' => $vendorDir . '/symfony/var-dumper/Caster/PgSqlCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ProxyManagerCaster' => $vendorDir . '/symfony/var-dumper/Caster/ProxyManagerCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\RdKafkaCaster' => $vendorDir . '/symfony/var-dumper/Caster/RdKafkaCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\RedisCaster' => $vendorDir . '/symfony/var-dumper/Caster/RedisCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ReflectionCaster' => $vendorDir . '/symfony/var-dumper/Caster/ReflectionCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ResourceCaster' => $vendorDir . '/symfony/var-dumper/Caster/ResourceCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ScalarStub' => $vendorDir . '/symfony/var-dumper/Caster/ScalarStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\SplCaster' => $vendorDir . '/symfony/var-dumper/Caster/SplCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\StubCaster' => $vendorDir . '/symfony/var-dumper/Caster/StubCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\SymfonyCaster' => $vendorDir . '/symfony/var-dumper/Caster/SymfonyCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\TraceStub' => $vendorDir . '/symfony/var-dumper/Caster/TraceStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\UninitializedStub' => $vendorDir . '/symfony/var-dumper/Caster/UninitializedStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\UuidCaster' => $vendorDir . '/symfony/var-dumper/Caster/UuidCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\XmlReaderCaster' => $vendorDir . '/symfony/var-dumper/Caster/XmlReaderCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\XmlResourceCaster' => $vendorDir . '/symfony/var-dumper/Caster/XmlResourceCaster.php',
    'Symfony\\Component\\VarDumper\\Cloner\\AbstractCloner' => $vendorDir . '/symfony/var-dumper/Cloner/AbstractCloner.php',
    'Symfony\\Component\\VarDumper\\Cloner\\ClonerInterface' => $vendorDir . '/symfony/var-dumper/Cloner/ClonerInterface.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Cursor' => $vendorDir . '/symfony/var-dumper/Cloner/Cursor.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Data' => $vendorDir . '/symfony/var-dumper/Cloner/Data.php',
    'Symfony\\Component\\VarDumper\\Cloner\\DumperInterface' => $vendorDir . '/symfony/var-dumper/Cloner/DumperInterface.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Internal\\NoDefault' => $vendorDir . '/symfony/var-dumper/Cloner/Internal/NoDefault.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Stub' => $vendorDir . '/symfony/var-dumper/Cloner/Stub.php',
    'Symfony\\Component\\VarDumper\\Cloner\\VarCloner' => $vendorDir . '/symfony/var-dumper/Cloner/VarCloner.php',
    'Symfony\\Component\\VarDumper\\Command\\Descriptor\\CliDescriptor' => $vendorDir . '/symfony/var-dumper/Command/Descriptor/CliDescriptor.php',
    'Symfony\\Component\\VarDumper\\Command\\Descriptor\\DumpDescriptorInterface' => $vendorDir . '/symfony/var-dumper/Command/Descriptor/DumpDescriptorInterface.php',
    'Symfony\\Component\\VarDumper\\Command\\Descriptor\\HtmlDescriptor' => $vendorDir . '/symfony/var-dumper/Command/Descriptor/HtmlDescriptor.php',
    'Symfony\\Component\\VarDumper\\Command\\ServerDumpCommand' => $vendorDir . '/symfony/var-dumper/Command/ServerDumpCommand.php',
    'Symfony\\Component\\VarDumper\\Dumper\\AbstractDumper' => $vendorDir . '/symfony/var-dumper/Dumper/AbstractDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\CliDumper' => $vendorDir . '/symfony/var-dumper/Dumper/CliDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\CliContextProvider' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/CliContextProvider.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\ContextProviderInterface' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/ContextProviderInterface.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\RequestContextProvider' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/RequestContextProvider.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\SourceContextProvider' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/SourceContextProvider.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextualizedDumper' => $vendorDir . '/symfony/var-dumper/Dumper/ContextualizedDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\DataDumperInterface' => $vendorDir . '/symfony/var-dumper/Dumper/DataDumperInterface.php',
    'Symfony\\Component\\VarDumper\\Dumper\\HtmlDumper' => $vendorDir . '/symfony/var-dumper/Dumper/HtmlDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ServerDumper' => $vendorDir . '/symfony/var-dumper/Dumper/ServerDumper.php',
    'Symfony\\Component\\VarDumper\\Exception\\ThrowingCasterException' => $vendorDir . '/symfony/var-dumper/Exception/ThrowingCasterException.php',
    'Symfony\\Component\\VarDumper\\Server\\Connection' => $vendorDir . '/symfony/var-dumper/Server/Connection.php',
    'Symfony\\Component\\VarDumper\\Server\\DumpServer' => $vendorDir . '/symfony/var-dumper/Server/DumpServer.php',
    'Symfony\\Component\\VarDumper\\Test\\VarDumperTestTrait' => $vendorDir . '/symfony/var-dumper/Test/VarDumperTestTrait.php',
    'Symfony\\Component\\VarDumper\\VarDumper' => $vendorDir . '/symfony/var-dumper/VarDumper.php',
    'Symfony\\Component\\Yaml\\Command\\LintCommand' => $vendorDir . '/symfony/yaml/Command/LintCommand.php',
    'Symfony\\Component\\Yaml\\Dumper' => $vendorDir . '/symfony/yaml/Dumper.php',
    'Symfony\\Component\\Yaml\\Escaper' => $vendorDir . '/symfony/yaml/Escaper.php',
    'Symfony\\Component\\Yaml\\Exception\\DumpException' => $vendorDir . '/symfony/yaml/Exception/DumpException.php',
    'Symfony\\Component\\Yaml\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/yaml/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Yaml\\Exception\\ParseException' => $vendorDir . '/symfony/yaml/Exception/ParseException.php',
    'Symfony\\Component\\Yaml\\Exception\\RuntimeException' => $vendorDir . '/symfony/yaml/Exception/RuntimeException.php',
    'Symfony\\Component\\Yaml\\Inline' => $vendorDir . '/symfony/yaml/Inline.php',
    'Symfony\\Component\\Yaml\\Parser' => $vendorDir . '/symfony/yaml/Parser.php',
    'Symfony\\Component\\Yaml\\Tag\\TaggedValue' => $vendorDir . '/symfony/yaml/Tag/TaggedValue.php',
    'Symfony\\Component\\Yaml\\Unescaper' => $vendorDir . '/symfony/yaml/Unescaper.php',
    'Symfony\\Component\\Yaml\\Yaml' => $vendorDir . '/symfony/yaml/Yaml.php',
    'Symfony\\Contracts\\EventDispatcher\\Event' => $vendorDir . '/symfony/event-dispatcher-contracts/Event.php',
    'Symfony\\Contracts\\EventDispatcher\\EventDispatcherInterface' => $vendorDir . '/symfony/event-dispatcher-contracts/EventDispatcherInterface.php',
    'Symfony\\Contracts\\Service\\Attribute\\Required' => $vendorDir . '/symfony/service-contracts/Attribute/Required.php',
    'Symfony\\Contracts\\Service\\Attribute\\SubscribedService' => $vendorDir . '/symfony/service-contracts/Attribute/SubscribedService.php',
    'Symfony\\Contracts\\Service\\ResetInterface' => $vendorDir . '/symfony/service-contracts/ResetInterface.php',
    'Symfony\\Contracts\\Service\\ServiceCollectionInterface' => $vendorDir . '/symfony/service-contracts/ServiceCollectionInterface.php',
    'Symfony\\Contracts\\Service\\ServiceLocatorTrait' => $vendorDir . '/symfony/service-contracts/ServiceLocatorTrait.php',
    'Symfony\\Contracts\\Service\\ServiceMethodsSubscriberTrait' => $vendorDir . '/symfony/service-contracts/ServiceMethodsSubscriberTrait.php',
    'Symfony\\Contracts\\Service\\ServiceProviderInterface' => $vendorDir . '/symfony/service-contracts/ServiceProviderInterface.php',
    'Symfony\\Contracts\\Service\\ServiceSubscriberInterface' => $vendorDir . '/symfony/service-contracts/ServiceSubscriberInterface.php',
    'Symfony\\Contracts\\Service\\ServiceSubscriberTrait' => $vendorDir . '/symfony/service-contracts/ServiceSubscriberTrait.php',
    'Symfony\\Contracts\\Translation\\LocaleAwareInterface' => $vendorDir . '/symfony/translation-contracts/LocaleAwareInterface.php',
    'Symfony\\Contracts\\Translation\\TranslatableInterface' => $vendorDir . '/symfony/translation-contracts/TranslatableInterface.php',
    'Symfony\\Contracts\\Translation\\TranslatorInterface' => $vendorDir . '/symfony/translation-contracts/TranslatorInterface.php',
    'Symfony\\Contracts\\Translation\\TranslatorTrait' => $vendorDir . '/symfony/translation-contracts/TranslatorTrait.php',
    'Symfony\\Polyfill\\Ctype\\Ctype' => $vendorDir . '/symfony/polyfill-ctype/Ctype.php',
    'Symfony\\Polyfill\\Intl\\Grapheme\\Grapheme' => $vendorDir . '/symfony/polyfill-intl-grapheme/Grapheme.php',
    'Symfony\\Polyfill\\Intl\\Idn\\Idn' => $vendorDir . '/symfony/polyfill-intl-idn/Idn.php',
    'Symfony\\Polyfill\\Intl\\Idn\\Info' => $vendorDir . '/symfony/polyfill-intl-idn/Info.php',
    'Symfony\\Polyfill\\Intl\\Idn\\Resources\\unidata\\DisallowedRanges' => $vendorDir . '/symfony/polyfill-intl-idn/Resources/unidata/DisallowedRanges.php',
    'Symfony\\Polyfill\\Intl\\Idn\\Resources\\unidata\\Regex' => $vendorDir . '/symfony/polyfill-intl-idn/Resources/unidata/Regex.php',
    'Symfony\\Polyfill\\Intl\\Normalizer\\Normalizer' => $vendorDir . '/symfony/polyfill-intl-normalizer/Normalizer.php',
    'Symfony\\Polyfill\\Mbstring\\Mbstring' => $vendorDir . '/symfony/polyfill-mbstring/Mbstring.php',
    'Symfony\\Polyfill\\Php80\\Php80' => $vendorDir . '/symfony/polyfill-php80/Php80.php',
    'Symfony\\Polyfill\\Php80\\PhpToken' => $vendorDir . '/symfony/polyfill-php80/PhpToken.php',
    'Symfony\\Polyfill\\Php83\\Php83' => $vendorDir . '/symfony/polyfill-php83/Php83.php',
    'Symfony\\Polyfill\\Uuid\\Uuid' => $vendorDir . '/symfony/polyfill-uuid/Uuid.php',
    'Termwind\\Actions\\StyleToMethod' => $vendorDir . '/nunomaduro/termwind/src/Actions/StyleToMethod.php',
    'Termwind\\Components\\Anchor' => $vendorDir . '/nunomaduro/termwind/src/Components/Anchor.php',
    'Termwind\\Components\\BreakLine' => $vendorDir . '/nunomaduro/termwind/src/Components/BreakLine.php',
    'Termwind\\Components\\Dd' => $vendorDir . '/nunomaduro/termwind/src/Components/Dd.php',
    'Termwind\\Components\\Div' => $vendorDir . '/nunomaduro/termwind/src/Components/Div.php',
    'Termwind\\Components\\Dl' => $vendorDir . '/nunomaduro/termwind/src/Components/Dl.php',
    'Termwind\\Components\\Dt' => $vendorDir . '/nunomaduro/termwind/src/Components/Dt.php',
    'Termwind\\Components\\Element' => $vendorDir . '/nunomaduro/termwind/src/Components/Element.php',
    'Termwind\\Components\\Hr' => $vendorDir . '/nunomaduro/termwind/src/Components/Hr.php',
    'Termwind\\Components\\Li' => $vendorDir . '/nunomaduro/termwind/src/Components/Li.php',
    'Termwind\\Components\\Ol' => $vendorDir . '/nunomaduro/termwind/src/Components/Ol.php',
    'Termwind\\Components\\Paragraph' => $vendorDir . '/nunomaduro/termwind/src/Components/Paragraph.php',
    'Termwind\\Components\\Raw' => $vendorDir . '/nunomaduro/termwind/src/Components/Raw.php',
    'Termwind\\Components\\Span' => $vendorDir . '/nunomaduro/termwind/src/Components/Span.php',
    'Termwind\\Components\\Ul' => $vendorDir . '/nunomaduro/termwind/src/Components/Ul.php',
    'Termwind\\Enums\\Color' => $vendorDir . '/nunomaduro/termwind/src/Enums/Color.php',
    'Termwind\\Exceptions\\ColorNotFound' => $vendorDir . '/nunomaduro/termwind/src/Exceptions/ColorNotFound.php',
    'Termwind\\Exceptions\\InvalidChild' => $vendorDir . '/nunomaduro/termwind/src/Exceptions/InvalidChild.php',
    'Termwind\\Exceptions\\InvalidColor' => $vendorDir . '/nunomaduro/termwind/src/Exceptions/InvalidColor.php',
    'Termwind\\Exceptions\\InvalidStyle' => $vendorDir . '/nunomaduro/termwind/src/Exceptions/InvalidStyle.php',
    'Termwind\\Exceptions\\StyleNotFound' => $vendorDir . '/nunomaduro/termwind/src/Exceptions/StyleNotFound.php',
    'Termwind\\Helpers\\QuestionHelper' => $vendorDir . '/nunomaduro/termwind/src/Helpers/QuestionHelper.php',
    'Termwind\\HtmlRenderer' => $vendorDir . '/nunomaduro/termwind/src/HtmlRenderer.php',
    'Termwind\\Html\\CodeRenderer' => $vendorDir . '/nunomaduro/termwind/src/Html/CodeRenderer.php',
    'Termwind\\Html\\InheritStyles' => $vendorDir . '/nunomaduro/termwind/src/Html/InheritStyles.php',
    'Termwind\\Html\\PreRenderer' => $vendorDir . '/nunomaduro/termwind/src/Html/PreRenderer.php',
    'Termwind\\Html\\TableRenderer' => $vendorDir . '/nunomaduro/termwind/src/Html/TableRenderer.php',
    'Termwind\\Laravel\\TermwindServiceProvider' => $vendorDir . '/nunomaduro/termwind/src/Laravel/TermwindServiceProvider.php',
    'Termwind\\Question' => $vendorDir . '/nunomaduro/termwind/src/Question.php',
    'Termwind\\Repositories\\Styles' => $vendorDir . '/nunomaduro/termwind/src/Repositories/Styles.php',
    'Termwind\\Terminal' => $vendorDir . '/nunomaduro/termwind/src/Terminal.php',
    'Termwind\\Termwind' => $vendorDir . '/nunomaduro/termwind/src/Termwind.php',
    'Termwind\\ValueObjects\\Node' => $vendorDir . '/nunomaduro/termwind/src/ValueObjects/Node.php',
    'Termwind\\ValueObjects\\Style' => $vendorDir . '/nunomaduro/termwind/src/ValueObjects/Style.php',
    'Termwind\\ValueObjects\\Styles' => $vendorDir . '/nunomaduro/termwind/src/ValueObjects/Styles.php',
    'Tests\\CreatesApplication' => $baseDir . '/tests/CreatesApplication.php',
    'Tests\\TestCase' => $baseDir . '/tests/TestCase.php',
    'Tests\\Unit\\NetworkTest' => $baseDir . '/tests/Unit/NetworkTest.php',
    'TheSeer\\Tokenizer\\Exception' => $vendorDir . '/theseer/tokenizer/src/Exception.php',
    'TheSeer\\Tokenizer\\NamespaceUri' => $vendorDir . '/theseer/tokenizer/src/NamespaceUri.php',
    'TheSeer\\Tokenizer\\NamespaceUriException' => $vendorDir . '/theseer/tokenizer/src/NamespaceUriException.php',
    'TheSeer\\Tokenizer\\Token' => $vendorDir . '/theseer/tokenizer/src/Token.php',
    'TheSeer\\Tokenizer\\TokenCollection' => $vendorDir . '/theseer/tokenizer/src/TokenCollection.php',
    'TheSeer\\Tokenizer\\TokenCollectionException' => $vendorDir . '/theseer/tokenizer/src/TokenCollectionException.php',
    'TheSeer\\Tokenizer\\Tokenizer' => $vendorDir . '/theseer/tokenizer/src/Tokenizer.php',
    'TheSeer\\Tokenizer\\XMLSerializer' => $vendorDir . '/theseer/tokenizer/src/XMLSerializer.php',
    'Tighten\\Ziggy\\BladeRouteGenerator' => $vendorDir . '/tightenco/ziggy/src/BladeRouteGenerator.php',
    'Tighten\\Ziggy\\CommandRouteGenerator' => $vendorDir . '/tightenco/ziggy/src/CommandRouteGenerator.php',
    'Tighten\\Ziggy\\Output\\File' => $vendorDir . '/tightenco/ziggy/src/Output/File.php',
    'Tighten\\Ziggy\\Output\\MergeScript' => $vendorDir . '/tightenco/ziggy/src/Output/MergeScript.php',
    'Tighten\\Ziggy\\Output\\Script' => $vendorDir . '/tightenco/ziggy/src/Output/Script.php',
    'Tighten\\Ziggy\\Output\\Types' => $vendorDir . '/tightenco/ziggy/src/Output/Types.php',
    'Tighten\\Ziggy\\Ziggy' => $vendorDir . '/tightenco/ziggy/src/Ziggy.php',
    'Tighten\\Ziggy\\ZiggyServiceProvider' => $vendorDir . '/tightenco/ziggy/src/ZiggyServiceProvider.php',
    'TijsVerkoyen\\CssToInlineStyles\\CssToInlineStyles' => $vendorDir . '/tijsverkoyen/css-to-inline-styles/src/CssToInlineStyles.php',
    'TijsVerkoyen\\CssToInlineStyles\\Css\\Processor' => $vendorDir . '/tijsverkoyen/css-to-inline-styles/src/Css/Processor.php',
    'TijsVerkoyen\\CssToInlineStyles\\Css\\Property\\Processor' => $vendorDir . '/tijsverkoyen/css-to-inline-styles/src/Css/Property/Processor.php',
    'TijsVerkoyen\\CssToInlineStyles\\Css\\Property\\Property' => $vendorDir . '/tijsverkoyen/css-to-inline-styles/src/Css/Property/Property.php',
    'TijsVerkoyen\\CssToInlineStyles\\Css\\Rule\\Processor' => $vendorDir . '/tijsverkoyen/css-to-inline-styles/src/Css/Rule/Processor.php',
    'TijsVerkoyen\\CssToInlineStyles\\Css\\Rule\\Rule' => $vendorDir . '/tijsverkoyen/css-to-inline-styles/src/Css/Rule/Rule.php',
    'TwilioEmail' => $vendorDir . '/sendgrid/sendgrid/lib/TwilioEmail.php',
    'Twilio\\Deserialize' => $vendorDir . '/twilio/sdk/src/Twilio/Deserialize.php',
    'Twilio\\Domain' => $vendorDir . '/twilio/sdk/src/Twilio/Domain.php',
    'Twilio\\Exceptions\\ConfigurationException' => $vendorDir . '/twilio/sdk/src/Twilio/Exceptions/ConfigurationException.php',
    'Twilio\\Exceptions\\DeserializeException' => $vendorDir . '/twilio/sdk/src/Twilio/Exceptions/DeserializeException.php',
    'Twilio\\Exceptions\\EnvironmentException' => $vendorDir . '/twilio/sdk/src/Twilio/Exceptions/EnvironmentException.php',
    'Twilio\\Exceptions\\HttpException' => $vendorDir . '/twilio/sdk/src/Twilio/Exceptions/HttpException.php',
    'Twilio\\Exceptions\\RestException' => $vendorDir . '/twilio/sdk/src/Twilio/Exceptions/RestException.php',
    'Twilio\\Exceptions\\TwilioException' => $vendorDir . '/twilio/sdk/src/Twilio/Exceptions/TwilioException.php',
    'Twilio\\Exceptions\\TwimlException' => $vendorDir . '/twilio/sdk/src/Twilio/Exceptions/TwimlException.php',
    'Twilio\\Http\\Client' => $vendorDir . '/twilio/sdk/src/Twilio/Http/Client.php',
    'Twilio\\Http\\CurlClient' => $vendorDir . '/twilio/sdk/src/Twilio/Http/CurlClient.php',
    'Twilio\\Http\\File' => $vendorDir . '/twilio/sdk/src/Twilio/Http/File.php',
    'Twilio\\Http\\GuzzleClient' => $vendorDir . '/twilio/sdk/src/Twilio/Http/GuzzleClient.php',
    'Twilio\\Http\\Response' => $vendorDir . '/twilio/sdk/src/Twilio/Http/Response.php',
    'Twilio\\InstanceContext' => $vendorDir . '/twilio/sdk/src/Twilio/InstanceContext.php',
    'Twilio\\InstanceResource' => $vendorDir . '/twilio/sdk/src/Twilio/InstanceResource.php',
    'Twilio\\Jwt\\AccessToken' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/AccessToken.php',
    'Twilio\\Jwt\\ClientToken' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/ClientToken.php',
    'Twilio\\Jwt\\Client\\ScopeURI' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/Client/ScopeURI.php',
    'Twilio\\Jwt\\Grants\\ChatGrant' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/Grants/ChatGrant.php',
    'Twilio\\Jwt\\Grants\\Grant' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/Grants/Grant.php',
    'Twilio\\Jwt\\Grants\\PlaybackGrant' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/Grants/PlaybackGrant.php',
    'Twilio\\Jwt\\Grants\\SyncGrant' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/Grants/SyncGrant.php',
    'Twilio\\Jwt\\Grants\\TaskRouterGrant' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/Grants/TaskRouterGrant.php',
    'Twilio\\Jwt\\Grants\\VideoGrant' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/Grants/VideoGrant.php',
    'Twilio\\Jwt\\Grants\\VoiceGrant' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/Grants/VoiceGrant.php',
    'Twilio\\Jwt\\JWT' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/JWT.php',
    'Twilio\\Jwt\\TaskRouter\\CapabilityToken' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/TaskRouter/CapabilityToken.php',
    'Twilio\\Jwt\\TaskRouter\\Policy' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/TaskRouter/Policy.php',
    'Twilio\\Jwt\\TaskRouter\\TaskQueueCapability' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/TaskRouter/TaskQueueCapability.php',
    'Twilio\\Jwt\\TaskRouter\\WorkerCapability' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/TaskRouter/WorkerCapability.php',
    'Twilio\\Jwt\\TaskRouter\\WorkspaceCapability' => $vendorDir . '/twilio/sdk/src/Twilio/Jwt/TaskRouter/WorkspaceCapability.php',
    'Twilio\\ListResource' => $vendorDir . '/twilio/sdk/src/Twilio/ListResource.php',
    'Twilio\\Options' => $vendorDir . '/twilio/sdk/src/Twilio/Options.php',
    'Twilio\\Page' => $vendorDir . '/twilio/sdk/src/Twilio/Page.php',
    'Twilio\\Rest\\Accounts' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts.php',
    'Twilio\\Rest\\Accounts\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1.php',
    'Twilio\\Rest\\Accounts\\V1\\AuthTokenPromotionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/AuthTokenPromotionContext.php',
    'Twilio\\Rest\\Accounts\\V1\\AuthTokenPromotionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/AuthTokenPromotionInstance.php',
    'Twilio\\Rest\\Accounts\\V1\\AuthTokenPromotionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/AuthTokenPromotionList.php',
    'Twilio\\Rest\\Accounts\\V1\\AuthTokenPromotionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/AuthTokenPromotionPage.php',
    'Twilio\\Rest\\Accounts\\V1\\CredentialInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/CredentialInstance.php',
    'Twilio\\Rest\\Accounts\\V1\\CredentialList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/CredentialList.php',
    'Twilio\\Rest\\Accounts\\V1\\CredentialPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/CredentialPage.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\AwsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/AwsContext.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\AwsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/AwsInstance.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\AwsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/AwsList.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\AwsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/AwsOptions.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\AwsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/AwsPage.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\PublicKeyContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/PublicKeyContext.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\PublicKeyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/PublicKeyInstance.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\PublicKeyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/PublicKeyList.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\PublicKeyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/PublicKeyOptions.php',
    'Twilio\\Rest\\Accounts\\V1\\Credential\\PublicKeyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/Credential/PublicKeyPage.php',
    'Twilio\\Rest\\Accounts\\V1\\SecondaryAuthTokenContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/SecondaryAuthTokenContext.php',
    'Twilio\\Rest\\Accounts\\V1\\SecondaryAuthTokenInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/SecondaryAuthTokenInstance.php',
    'Twilio\\Rest\\Accounts\\V1\\SecondaryAuthTokenList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/SecondaryAuthTokenList.php',
    'Twilio\\Rest\\Accounts\\V1\\SecondaryAuthTokenPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Accounts/V1/SecondaryAuthTokenPage.php',
    'Twilio\\Rest\\Api' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api.php',
    'Twilio\\Rest\\Api\\V2010' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010.php',
    'Twilio\\Rest\\Api\\V2010\\AccountContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/AccountContext.php',
    'Twilio\\Rest\\Api\\V2010\\AccountInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/AccountInstance.php',
    'Twilio\\Rest\\Api\\V2010\\AccountList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/AccountList.php',
    'Twilio\\Rest\\Api\\V2010\\AccountOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/AccountOptions.php',
    'Twilio\\Rest\\Api\\V2010\\AccountPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/AccountPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AddressContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AddressContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AddressInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AddressInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AddressList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AddressList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AddressOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AddressOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AddressPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AddressPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Address\\DependentPhoneNumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Address/DependentPhoneNumberInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Address\\DependentPhoneNumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Address/DependentPhoneNumberList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Address\\DependentPhoneNumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Address/DependentPhoneNumberPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ApplicationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ApplicationContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ApplicationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ApplicationInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ApplicationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ApplicationList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ApplicationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ApplicationOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ApplicationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ApplicationPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AuthorizedConnectAppContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AuthorizedConnectAppContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AuthorizedConnectAppInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AuthorizedConnectAppInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AuthorizedConnectAppList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AuthorizedConnectAppList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AuthorizedConnectAppPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AuthorizedConnectAppPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountryContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountryInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountryList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountryPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\LocalInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/LocalInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\LocalList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/LocalList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\LocalOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/LocalOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\LocalPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/LocalPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\MachineToMachineInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/MachineToMachineInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\MachineToMachineList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/MachineToMachineList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\MachineToMachineOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/MachineToMachineOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\MachineToMachinePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/MachineToMachinePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\MobileInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/MobileInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\MobileList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/MobileList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\MobileOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/MobileOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\MobilePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/MobilePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\NationalInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/NationalInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\NationalList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/NationalList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\NationalOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/NationalOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\NationalPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/NationalPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\SharedCostInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/SharedCostInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\SharedCostList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/SharedCostList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\SharedCostOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/SharedCostOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\SharedCostPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/SharedCostPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\TollFreeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/TollFreeInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\TollFreeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/TollFreeList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\TollFreeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/TollFreeOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\TollFreePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/TollFreePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\VoipInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/VoipInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\VoipList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/VoipList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\VoipOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/VoipOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\AvailablePhoneNumberCountry\\VoipPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/AvailablePhoneNumberCountry/VoipPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\BalanceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/BalanceInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\BalanceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/BalanceList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\BalancePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/BalancePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\CallContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/CallContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\CallInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/CallInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\CallList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/CallList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\CallOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/CallOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\CallPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/CallPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\EventInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/EventInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\EventList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/EventList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\EventPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/EventPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackSummaryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackSummaryContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackSummaryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackSummaryInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackSummaryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackSummaryList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackSummaryOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackSummaryOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\FeedbackSummaryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/FeedbackSummaryPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\NotificationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/NotificationContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\NotificationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/NotificationInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\NotificationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/NotificationList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\NotificationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/NotificationOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\NotificationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/NotificationPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\PaymentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/PaymentContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\PaymentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/PaymentInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\PaymentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/PaymentList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\PaymentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/PaymentOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\PaymentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/PaymentPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\RecordingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/RecordingContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\RecordingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/RecordingInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\RecordingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/RecordingList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\RecordingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/RecordingOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\RecordingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/RecordingPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\SiprecContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/SiprecContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\SiprecInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/SiprecInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\SiprecList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/SiprecList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\SiprecOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/SiprecOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\SiprecPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/SiprecPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\StreamContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/StreamContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\StreamInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/StreamInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\StreamList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/StreamList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\StreamOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/StreamOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\StreamPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/StreamPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\UserDefinedMessageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/UserDefinedMessageInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\UserDefinedMessageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/UserDefinedMessageList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\UserDefinedMessageOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/UserDefinedMessageOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\UserDefinedMessagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/UserDefinedMessagePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\UserDefinedMessageSubscriptionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/UserDefinedMessageSubscriptionContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\UserDefinedMessageSubscriptionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/UserDefinedMessageSubscriptionInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\UserDefinedMessageSubscriptionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/UserDefinedMessageSubscriptionList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\UserDefinedMessageSubscriptionOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/UserDefinedMessageSubscriptionOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Call\\UserDefinedMessageSubscriptionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Call/UserDefinedMessageSubscriptionPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConferenceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConferenceContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConferenceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConferenceInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConferenceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConferenceList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConferenceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConferenceOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConferencePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConferencePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\ParticipantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/ParticipantContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\ParticipantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/ParticipantInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\ParticipantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/ParticipantList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\ParticipantOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/ParticipantOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\ParticipantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/ParticipantPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\RecordingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/RecordingContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\RecordingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/RecordingInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\RecordingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/RecordingList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\RecordingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/RecordingOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Conference\\RecordingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Conference/RecordingPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConnectAppContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConnectAppContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConnectAppInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConnectAppInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConnectAppList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConnectAppList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConnectAppOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConnectAppOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ConnectAppPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ConnectAppPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumberContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumberInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumberList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumberOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumberPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\AssignedAddOnContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/AssignedAddOnContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\AssignedAddOnInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/AssignedAddOnInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\AssignedAddOnList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/AssignedAddOnList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\AssignedAddOnPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/AssignedAddOnPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\AssignedAddOn\\AssignedAddOnExtensionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/AssignedAddOn/AssignedAddOnExtensionContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\AssignedAddOn\\AssignedAddOnExtensionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/AssignedAddOn/AssignedAddOnExtensionInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\AssignedAddOn\\AssignedAddOnExtensionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/AssignedAddOn/AssignedAddOnExtensionList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\AssignedAddOn\\AssignedAddOnExtensionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/AssignedAddOn/AssignedAddOnExtensionPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\LocalInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/LocalInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\LocalList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/LocalList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\LocalOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/LocalOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\LocalPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/LocalPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\MobileInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/MobileInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\MobileList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/MobileList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\MobileOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/MobileOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\MobilePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/MobilePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\TollFreeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/TollFreeInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\TollFreeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/TollFreeList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\TollFreeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/TollFreeOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\IncomingPhoneNumber\\TollFreePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/IncomingPhoneNumber/TollFreePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\KeyContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/KeyContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\KeyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/KeyInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\KeyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/KeyList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\KeyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/KeyOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\KeyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/KeyPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\MessageContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/MessageContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\MessageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/MessageInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\MessageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/MessageList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\MessageOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/MessageOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\MessagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/MessagePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Message\\FeedbackInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Message/FeedbackInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Message\\FeedbackList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Message/FeedbackList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Message\\FeedbackOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Message/FeedbackOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Message\\FeedbackPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Message/FeedbackPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Message\\MediaContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Message/MediaContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Message\\MediaInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Message/MediaInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Message\\MediaList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Message/MediaList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Message\\MediaOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Message/MediaOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Message\\MediaPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Message/MediaPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NewKeyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NewKeyInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NewKeyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NewKeyList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NewKeyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NewKeyOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NewKeyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NewKeyPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NewSigningKeyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NewSigningKeyInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NewSigningKeyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NewSigningKeyList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NewSigningKeyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NewSigningKeyOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NewSigningKeyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NewSigningKeyPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NotificationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NotificationContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NotificationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NotificationInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NotificationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NotificationList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NotificationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NotificationOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\NotificationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/NotificationPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\OutgoingCallerIdContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/OutgoingCallerIdContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\OutgoingCallerIdInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/OutgoingCallerIdInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\OutgoingCallerIdList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/OutgoingCallerIdList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\OutgoingCallerIdOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/OutgoingCallerIdOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\OutgoingCallerIdPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/OutgoingCallerIdPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\QueueContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/QueueContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\QueueInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/QueueInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\QueueList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/QueueList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\QueueOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/QueueOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\QueuePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/QueuePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Queue\\MemberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Queue/MemberContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Queue\\MemberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Queue/MemberInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Queue\\MemberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Queue/MemberList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Queue\\MemberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Queue/MemberOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Queue\\MemberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Queue/MemberPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\RecordingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/RecordingContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\RecordingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/RecordingInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\RecordingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/RecordingList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\RecordingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/RecordingOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\RecordingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/RecordingPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\AddOnResultContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/AddOnResultContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\AddOnResultInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/AddOnResultInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\AddOnResultList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/AddOnResultList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\AddOnResultPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/AddOnResultPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\AddOnResult\\PayloadContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/AddOnResult/PayloadContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\AddOnResult\\PayloadInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/AddOnResult/PayloadInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\AddOnResult\\PayloadList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/AddOnResult/PayloadList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\AddOnResult\\PayloadPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/AddOnResult/PayloadPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\TranscriptionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/TranscriptionContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\TranscriptionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/TranscriptionInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\TranscriptionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/TranscriptionList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Recording\\TranscriptionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Recording/TranscriptionPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ShortCodeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ShortCodeContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ShortCodeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ShortCodeInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ShortCodeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ShortCodeList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ShortCodeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ShortCodeOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ShortCodePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ShortCodePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\SigningKeyContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/SigningKeyContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\SigningKeyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/SigningKeyInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\SigningKeyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/SigningKeyList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\SigningKeyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/SigningKeyOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\SigningKeyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/SigningKeyPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\SipInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/SipInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\SipList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/SipList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\SipPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/SipPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\CredentialListContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/CredentialListContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\CredentialListInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/CredentialListInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\CredentialListList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/CredentialListList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\CredentialListPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/CredentialListPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\CredentialList\\CredentialContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/CredentialList/CredentialContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\CredentialList\\CredentialInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/CredentialList/CredentialInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\CredentialList\\CredentialList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/CredentialList/CredentialList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\CredentialList\\CredentialOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/CredentialList/CredentialOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\CredentialList\\CredentialPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/CredentialList/CredentialPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\DomainContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/DomainContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\DomainInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/DomainInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\DomainList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/DomainList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\DomainOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/DomainOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\DomainPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/DomainPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypesInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypesInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypesList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypesList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypesPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypesPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCallsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCallsInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCallsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCallsList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCallsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCallsPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCalls\\AuthCallsCredentialListMappingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCalls/AuthCallsCredentialListMappingContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCalls\\AuthCallsCredentialListMappingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCalls/AuthCallsCredentialListMappingInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCalls\\AuthCallsCredentialListMappingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCalls/AuthCallsCredentialListMappingList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCalls\\AuthCallsCredentialListMappingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCalls/AuthCallsCredentialListMappingPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCalls\\AuthCallsIpAccessControlListMappingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCalls/AuthCallsIpAccessControlListMappingContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCalls\\AuthCallsIpAccessControlListMappingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCalls/AuthCallsIpAccessControlListMappingInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCalls\\AuthCallsIpAccessControlListMappingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCalls/AuthCallsIpAccessControlListMappingList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeCalls\\AuthCallsIpAccessControlListMappingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeCalls/AuthCallsIpAccessControlListMappingPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeRegistrationsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeRegistrationsInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeRegistrationsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeRegistrationsList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeRegistrationsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeRegistrationsPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeRegistrations\\AuthRegistrationsCredentialListMappingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeRegistrations/AuthRegistrationsCredentialListMappingContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeRegistrations\\AuthRegistrationsCredentialListMappingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeRegistrations/AuthRegistrationsCredentialListMappingInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeRegistrations\\AuthRegistrationsCredentialListMappingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeRegistrations/AuthRegistrationsCredentialListMappingList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\AuthTypes\\AuthTypeRegistrations\\AuthRegistrationsCredentialListMappingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/AuthTypes/AuthTypeRegistrations/AuthRegistrationsCredentialListMappingPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\CredentialListMappingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/CredentialListMappingContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\CredentialListMappingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/CredentialListMappingInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\CredentialListMappingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/CredentialListMappingList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\CredentialListMappingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/CredentialListMappingPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\IpAccessControlListMappingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/IpAccessControlListMappingContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\IpAccessControlListMappingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/IpAccessControlListMappingInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\IpAccessControlListMappingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/IpAccessControlListMappingList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\Domain\\IpAccessControlListMappingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/Domain/IpAccessControlListMappingPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\IpAccessControlListContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/IpAccessControlListContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\IpAccessControlListInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/IpAccessControlListInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\IpAccessControlListList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/IpAccessControlListList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\IpAccessControlListPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/IpAccessControlListPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\IpAccessControlList\\IpAddressContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/IpAccessControlList/IpAddressContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\IpAccessControlList\\IpAddressInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/IpAccessControlList/IpAddressInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\IpAccessControlList\\IpAddressList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/IpAccessControlList/IpAddressList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\IpAccessControlList\\IpAddressOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/IpAccessControlList/IpAddressOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Sip\\IpAccessControlList\\IpAddressPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Sip/IpAccessControlList/IpAddressPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\TokenInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/TokenInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\TokenList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/TokenList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\TokenOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/TokenOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\TokenPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/TokenPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\TranscriptionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/TranscriptionContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\TranscriptionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/TranscriptionInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\TranscriptionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/TranscriptionList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\TranscriptionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/TranscriptionPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\UsageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/UsageInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\UsageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/UsageList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\UsagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/UsagePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\RecordInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/RecordInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\RecordList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/RecordList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\RecordOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/RecordOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\RecordPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/RecordPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\AllTimeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/AllTimeInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\AllTimeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/AllTimeList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\AllTimeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/AllTimeOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\AllTimePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/AllTimePage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\DailyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/DailyInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\DailyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/DailyList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\DailyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/DailyOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\DailyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/DailyPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\LastMonthInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/LastMonthInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\LastMonthList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/LastMonthList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\LastMonthOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/LastMonthOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\LastMonthPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/LastMonthPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\MonthlyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/MonthlyInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\MonthlyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/MonthlyList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\MonthlyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/MonthlyOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\MonthlyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/MonthlyPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\ThisMonthInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/ThisMonthInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\ThisMonthList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/ThisMonthList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\ThisMonthOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/ThisMonthOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\ThisMonthPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/ThisMonthPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\TodayInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/TodayInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\TodayList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/TodayList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\TodayOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/TodayOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\TodayPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/TodayPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\YearlyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/YearlyInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\YearlyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/YearlyList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\YearlyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/YearlyOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\YearlyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/YearlyPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\YesterdayInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/YesterdayInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\YesterdayList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/YesterdayList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\YesterdayOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/YesterdayOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\Record\\YesterdayPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/Record/YesterdayPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\TriggerContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/TriggerContext.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\TriggerInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/TriggerInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\TriggerList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/TriggerList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\TriggerOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/TriggerOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\Usage\\TriggerPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/Usage/TriggerPage.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ValidationRequestInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ValidationRequestInstance.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ValidationRequestList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ValidationRequestList.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ValidationRequestOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ValidationRequestOptions.php',
    'Twilio\\Rest\\Api\\V2010\\Account\\ValidationRequestPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Api/V2010/Account/ValidationRequestPage.php',
    'Twilio\\Rest\\Autopilot' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot.php',
    'Twilio\\Rest\\Autopilot\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1.php',
    'Twilio\\Rest\\Autopilot\\V1\\AssistantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/AssistantContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\AssistantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/AssistantInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\AssistantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/AssistantList.php',
    'Twilio\\Rest\\Autopilot\\V1\\AssistantOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/AssistantOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\AssistantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/AssistantPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\DefaultsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/DefaultsContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\DefaultsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/DefaultsInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\DefaultsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/DefaultsList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\DefaultsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/DefaultsOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\DefaultsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/DefaultsPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\DialogueContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/DialogueContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\DialogueInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/DialogueInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\DialogueList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/DialogueList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\DialoguePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/DialoguePage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldTypeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldTypeContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldTypeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldTypeInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldTypeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldTypeList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldTypeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldTypeOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldTypePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldTypePage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldType\\FieldValueContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldType/FieldValueContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldType\\FieldValueInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldType/FieldValueInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldType\\FieldValueList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldType/FieldValueList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldType\\FieldValueOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldType/FieldValueOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\FieldType\\FieldValuePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/FieldType/FieldValuePage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\ModelBuildContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/ModelBuildContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\ModelBuildInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/ModelBuildInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\ModelBuildList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/ModelBuildList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\ModelBuildOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/ModelBuildOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\ModelBuildPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/ModelBuildPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\QueryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/QueryContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\QueryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/QueryInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\QueryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/QueryList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\QueryOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/QueryOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\QueryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/QueryPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\StyleSheetContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/StyleSheetContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\StyleSheetInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/StyleSheetInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\StyleSheetList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/StyleSheetList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\StyleSheetOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/StyleSheetOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\StyleSheetPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/StyleSheetPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\TaskContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/TaskContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\TaskInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/TaskInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\TaskList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/TaskList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\TaskOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/TaskOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\TaskPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/TaskPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\FieldContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/FieldContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\FieldInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/FieldInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\FieldList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/FieldList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\FieldPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/FieldPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\SampleContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/SampleContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\SampleInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/SampleInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\SampleList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/SampleList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\SampleOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/SampleOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\SamplePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/SamplePage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\TaskActionsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/TaskActionsContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\TaskActionsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/TaskActionsInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\TaskActionsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/TaskActionsList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\TaskActionsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/TaskActionsOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\TaskActionsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/TaskActionsPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\TaskStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/TaskStatisticsContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\TaskStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/TaskStatisticsInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\TaskStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/TaskStatisticsList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\Task\\TaskStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/Task/TaskStatisticsPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\WebhookContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/WebhookContext.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\WebhookInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/WebhookInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\WebhookList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/WebhookList.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\WebhookOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/WebhookOptions.php',
    'Twilio\\Rest\\Autopilot\\V1\\Assistant\\WebhookPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/Assistant/WebhookPage.php',
    'Twilio\\Rest\\Autopilot\\V1\\RestoreAssistantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/RestoreAssistantInstance.php',
    'Twilio\\Rest\\Autopilot\\V1\\RestoreAssistantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/RestoreAssistantList.php',
    'Twilio\\Rest\\Autopilot\\V1\\RestoreAssistantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Autopilot/V1/RestoreAssistantPage.php',
    'Twilio\\Rest\\Bulkexports' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports.php',
    'Twilio\\Rest\\Bulkexports\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1.php',
    'Twilio\\Rest\\Bulkexports\\V1\\ExportConfigurationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/ExportConfigurationContext.php',
    'Twilio\\Rest\\Bulkexports\\V1\\ExportConfigurationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/ExportConfigurationInstance.php',
    'Twilio\\Rest\\Bulkexports\\V1\\ExportConfigurationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/ExportConfigurationList.php',
    'Twilio\\Rest\\Bulkexports\\V1\\ExportConfigurationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/ExportConfigurationOptions.php',
    'Twilio\\Rest\\Bulkexports\\V1\\ExportConfigurationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/ExportConfigurationPage.php',
    'Twilio\\Rest\\Bulkexports\\V1\\ExportContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/ExportContext.php',
    'Twilio\\Rest\\Bulkexports\\V1\\ExportInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/ExportInstance.php',
    'Twilio\\Rest\\Bulkexports\\V1\\ExportList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/ExportList.php',
    'Twilio\\Rest\\Bulkexports\\V1\\ExportPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/ExportPage.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\DayContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/DayContext.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\DayInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/DayInstance.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\DayList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/DayList.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\DayPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/DayPage.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\ExportCustomJobInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/ExportCustomJobInstance.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\ExportCustomJobList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/ExportCustomJobList.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\ExportCustomJobOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/ExportCustomJobOptions.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\ExportCustomJobPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/ExportCustomJobPage.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\JobContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/JobContext.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\JobInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/JobInstance.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\JobList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/JobList.php',
    'Twilio\\Rest\\Bulkexports\\V1\\Export\\JobPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Bulkexports/V1/Export/JobPage.php',
    'Twilio\\Rest\\Chat' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat.php',
    'Twilio\\Rest\\Chat\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1.php',
    'Twilio\\Rest\\Chat\\V1\\CredentialContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/CredentialContext.php',
    'Twilio\\Rest\\Chat\\V1\\CredentialInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/CredentialInstance.php',
    'Twilio\\Rest\\Chat\\V1\\CredentialList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/CredentialList.php',
    'Twilio\\Rest\\Chat\\V1\\CredentialOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/CredentialOptions.php',
    'Twilio\\Rest\\Chat\\V1\\CredentialPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/CredentialPage.php',
    'Twilio\\Rest\\Chat\\V1\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/ServiceContext.php',
    'Twilio\\Rest\\Chat\\V1\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/ServiceInstance.php',
    'Twilio\\Rest\\Chat\\V1\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/ServiceList.php',
    'Twilio\\Rest\\Chat\\V1\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/ServiceOptions.php',
    'Twilio\\Rest\\Chat\\V1\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/ServicePage.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\ChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/ChannelContext.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\ChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/ChannelInstance.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\ChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/ChannelList.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\ChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/ChannelOptions.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\ChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/ChannelPage.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\InviteContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/InviteContext.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\InviteInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/InviteInstance.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\InviteList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/InviteList.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\InviteOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/InviteOptions.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\InvitePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/InvitePage.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MemberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MemberContext.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MemberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MemberInstance.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MemberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MemberList.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MemberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MemberOptions.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MemberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MemberPage.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MessageContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MessageContext.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MessageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MessageInstance.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MessageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MessageList.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MessageOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MessageOptions.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\Channel\\MessagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/Channel/MessagePage.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\RoleContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/RoleContext.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\RoleInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/RoleInstance.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\RoleList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/RoleList.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\RolePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/RolePage.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\UserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/UserContext.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\UserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/UserInstance.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\UserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/UserList.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\UserOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/UserOptions.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\UserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/UserPage.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\User\\UserChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/User/UserChannelInstance.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\User\\UserChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/User/UserChannelList.php',
    'Twilio\\Rest\\Chat\\V1\\Service\\User\\UserChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V1/Service/User/UserChannelPage.php',
    'Twilio\\Rest\\Chat\\V2' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2.php',
    'Twilio\\Rest\\Chat\\V2\\CredentialContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/CredentialContext.php',
    'Twilio\\Rest\\Chat\\V2\\CredentialInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/CredentialInstance.php',
    'Twilio\\Rest\\Chat\\V2\\CredentialList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/CredentialList.php',
    'Twilio\\Rest\\Chat\\V2\\CredentialOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/CredentialOptions.php',
    'Twilio\\Rest\\Chat\\V2\\CredentialPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/CredentialPage.php',
    'Twilio\\Rest\\Chat\\V2\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/ServiceContext.php',
    'Twilio\\Rest\\Chat\\V2\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/ServiceInstance.php',
    'Twilio\\Rest\\Chat\\V2\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/ServiceList.php',
    'Twilio\\Rest\\Chat\\V2\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/ServiceOptions.php',
    'Twilio\\Rest\\Chat\\V2\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/ServicePage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\BindingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/BindingContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\BindingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/BindingInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\BindingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/BindingList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\BindingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/BindingOptions.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\BindingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/BindingPage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\ChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/ChannelContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\ChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/ChannelInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\ChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/ChannelList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\ChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/ChannelOptions.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\ChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/ChannelPage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\InviteContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/InviteContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\InviteInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/InviteInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\InviteList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/InviteList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\InviteOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/InviteOptions.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\InvitePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/InvitePage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MemberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MemberContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MemberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MemberInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MemberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MemberList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MemberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MemberOptions.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MemberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MemberPage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MessageContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MessageContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MessageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MessageInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MessageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MessageList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MessageOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MessageOptions.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\MessagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/MessagePage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\WebhookContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/WebhookContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\WebhookInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/WebhookInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\WebhookList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/WebhookList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\WebhookOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/WebhookOptions.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\Channel\\WebhookPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/Channel/WebhookPage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\RoleContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/RoleContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\RoleInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/RoleInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\RoleList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/RoleList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\RolePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/RolePage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\UserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/UserContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\UserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/UserInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\UserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/UserList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\UserOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/UserOptions.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\UserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/UserPage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserBindingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserBindingContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserBindingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserBindingInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserBindingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserBindingList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserBindingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserBindingOptions.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserBindingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserBindingPage.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserChannelContext.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserChannelInstance.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserChannelList.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserChannelOptions.php',
    'Twilio\\Rest\\Chat\\V2\\Service\\User\\UserChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V2/Service/User/UserChannelPage.php',
    'Twilio\\Rest\\Chat\\V3' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V3.php',
    'Twilio\\Rest\\Chat\\V3\\ChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V3/ChannelContext.php',
    'Twilio\\Rest\\Chat\\V3\\ChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V3/ChannelInstance.php',
    'Twilio\\Rest\\Chat\\V3\\ChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V3/ChannelList.php',
    'Twilio\\Rest\\Chat\\V3\\ChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V3/ChannelOptions.php',
    'Twilio\\Rest\\Chat\\V3\\ChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Chat/V3/ChannelPage.php',
    'Twilio\\Rest\\Client' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Client.php',
    'Twilio\\Rest\\Content' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content.php',
    'Twilio\\Rest\\Content\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1.php',
    'Twilio\\Rest\\Content\\V1\\ContentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/ContentContext.php',
    'Twilio\\Rest\\Content\\V1\\ContentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/ContentInstance.php',
    'Twilio\\Rest\\Content\\V1\\ContentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/ContentList.php',
    'Twilio\\Rest\\Content\\V1\\ContentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/ContentPage.php',
    'Twilio\\Rest\\Content\\V1\\Content\\ApprovalFetchContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/Content/ApprovalFetchContext.php',
    'Twilio\\Rest\\Content\\V1\\Content\\ApprovalFetchInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/Content/ApprovalFetchInstance.php',
    'Twilio\\Rest\\Content\\V1\\Content\\ApprovalFetchList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/Content/ApprovalFetchList.php',
    'Twilio\\Rest\\Content\\V1\\Content\\ApprovalFetchPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/Content/ApprovalFetchPage.php',
    'Twilio\\Rest\\Content\\V1\\LegacyContentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/LegacyContentInstance.php',
    'Twilio\\Rest\\Content\\V1\\LegacyContentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/LegacyContentList.php',
    'Twilio\\Rest\\Content\\V1\\LegacyContentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Content/V1/LegacyContentPage.php',
    'Twilio\\Rest\\Conversations' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations.php',
    'Twilio\\Rest\\Conversations\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1.php',
    'Twilio\\Rest\\Conversations\\V1\\AddressConfigurationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/AddressConfigurationContext.php',
    'Twilio\\Rest\\Conversations\\V1\\AddressConfigurationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/AddressConfigurationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\AddressConfigurationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/AddressConfigurationList.php',
    'Twilio\\Rest\\Conversations\\V1\\AddressConfigurationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/AddressConfigurationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\AddressConfigurationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/AddressConfigurationPage.php',
    'Twilio\\Rest\\Conversations\\V1\\ConfigurationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConfigurationContext.php',
    'Twilio\\Rest\\Conversations\\V1\\ConfigurationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConfigurationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\ConfigurationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConfigurationList.php',
    'Twilio\\Rest\\Conversations\\V1\\ConfigurationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConfigurationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\ConfigurationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConfigurationPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Configuration\\WebhookContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Configuration/WebhookContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Configuration\\WebhookInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Configuration/WebhookInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Configuration\\WebhookList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Configuration/WebhookList.php',
    'Twilio\\Rest\\Conversations\\V1\\Configuration\\WebhookOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Configuration/WebhookOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Configuration\\WebhookPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Configuration/WebhookPage.php',
    'Twilio\\Rest\\Conversations\\V1\\ConversationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConversationContext.php',
    'Twilio\\Rest\\Conversations\\V1\\ConversationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConversationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\ConversationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConversationList.php',
    'Twilio\\Rest\\Conversations\\V1\\ConversationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConversationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\ConversationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ConversationPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\MessageContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/MessageContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\MessageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/MessageInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\MessageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/MessageList.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\MessageOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/MessageOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\MessagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/MessagePage.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\Message\\DeliveryReceiptContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/Message/DeliveryReceiptContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\Message\\DeliveryReceiptInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/Message/DeliveryReceiptInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\Message\\DeliveryReceiptList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/Message/DeliveryReceiptList.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\Message\\DeliveryReceiptPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/Message/DeliveryReceiptPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\ParticipantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/ParticipantContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\ParticipantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/ParticipantInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\ParticipantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/ParticipantList.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\ParticipantOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/ParticipantOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\ParticipantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/ParticipantPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\WebhookContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/WebhookContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\WebhookInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/WebhookInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\WebhookList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/WebhookList.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\WebhookOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/WebhookOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Conversation\\WebhookPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Conversation/WebhookPage.php',
    'Twilio\\Rest\\Conversations\\V1\\CredentialContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/CredentialContext.php',
    'Twilio\\Rest\\Conversations\\V1\\CredentialInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/CredentialInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\CredentialList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/CredentialList.php',
    'Twilio\\Rest\\Conversations\\V1\\CredentialOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/CredentialOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\CredentialPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/CredentialPage.php',
    'Twilio\\Rest\\Conversations\\V1\\ParticipantConversationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ParticipantConversationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\ParticipantConversationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ParticipantConversationList.php',
    'Twilio\\Rest\\Conversations\\V1\\ParticipantConversationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ParticipantConversationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\ParticipantConversationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ParticipantConversationPage.php',
    'Twilio\\Rest\\Conversations\\V1\\RoleContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/RoleContext.php',
    'Twilio\\Rest\\Conversations\\V1\\RoleInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/RoleInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\RoleList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/RoleList.php',
    'Twilio\\Rest\\Conversations\\V1\\RolePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/RolePage.php',
    'Twilio\\Rest\\Conversations\\V1\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ServiceContext.php',
    'Twilio\\Rest\\Conversations\\V1\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ServiceInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ServiceList.php',
    'Twilio\\Rest\\Conversations\\V1\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/ServicePage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\BindingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/BindingContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\BindingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/BindingInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\BindingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/BindingList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\BindingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/BindingOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\BindingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/BindingPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConfigurationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConfigurationContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConfigurationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConfigurationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConfigurationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConfigurationList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConfigurationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConfigurationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConfigurationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConfigurationPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\NotificationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/NotificationContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\NotificationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/NotificationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\NotificationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/NotificationList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\NotificationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/NotificationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\NotificationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/NotificationPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\WebhookContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/WebhookContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\WebhookInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/WebhookInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\WebhookList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/WebhookList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\WebhookOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/WebhookOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Configuration\\WebhookPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Configuration/WebhookPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConversationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConversationContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConversationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConversationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConversationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConversationList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConversationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConversationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ConversationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ConversationPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\MessageContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/MessageContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\MessageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/MessageInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\MessageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/MessageList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\MessageOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/MessageOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\MessagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/MessagePage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\Message\\DeliveryReceiptContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/Message/DeliveryReceiptContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\Message\\DeliveryReceiptInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/Message/DeliveryReceiptInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\Message\\DeliveryReceiptList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/Message/DeliveryReceiptList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\Message\\DeliveryReceiptPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/Message/DeliveryReceiptPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\ParticipantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/ParticipantContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\ParticipantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/ParticipantInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\ParticipantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/ParticipantList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\ParticipantOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/ParticipantOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\ParticipantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/ParticipantPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\WebhookContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/WebhookContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\WebhookInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/WebhookInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\WebhookList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/WebhookList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\WebhookOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/WebhookOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\Conversation\\WebhookPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/Conversation/WebhookPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ParticipantConversationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ParticipantConversationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ParticipantConversationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ParticipantConversationList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ParticipantConversationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ParticipantConversationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\ParticipantConversationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/ParticipantConversationPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\RoleContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/RoleContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\RoleInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/RoleInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\RoleList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/RoleList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\RolePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/RolePage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\UserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/UserContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\UserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/UserInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\UserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/UserList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\UserOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/UserOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\UserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/UserPage.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\User\\UserConversationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/User/UserConversationContext.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\User\\UserConversationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/User/UserConversationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\User\\UserConversationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/User/UserConversationList.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\User\\UserConversationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/User/UserConversationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\Service\\User\\UserConversationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/Service/User/UserConversationPage.php',
    'Twilio\\Rest\\Conversations\\V1\\UserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/UserContext.php',
    'Twilio\\Rest\\Conversations\\V1\\UserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/UserInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\UserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/UserList.php',
    'Twilio\\Rest\\Conversations\\V1\\UserOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/UserOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\UserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/UserPage.php',
    'Twilio\\Rest\\Conversations\\V1\\User\\UserConversationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/User/UserConversationContext.php',
    'Twilio\\Rest\\Conversations\\V1\\User\\UserConversationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/User/UserConversationInstance.php',
    'Twilio\\Rest\\Conversations\\V1\\User\\UserConversationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/User/UserConversationList.php',
    'Twilio\\Rest\\Conversations\\V1\\User\\UserConversationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/User/UserConversationOptions.php',
    'Twilio\\Rest\\Conversations\\V1\\User\\UserConversationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Conversations/V1/User/UserConversationPage.php',
    'Twilio\\Rest\\Events' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events.php',
    'Twilio\\Rest\\Events\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1.php',
    'Twilio\\Rest\\Events\\V1\\EventTypeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/EventTypeContext.php',
    'Twilio\\Rest\\Events\\V1\\EventTypeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/EventTypeInstance.php',
    'Twilio\\Rest\\Events\\V1\\EventTypeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/EventTypeList.php',
    'Twilio\\Rest\\Events\\V1\\EventTypeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/EventTypeOptions.php',
    'Twilio\\Rest\\Events\\V1\\EventTypePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/EventTypePage.php',
    'Twilio\\Rest\\Events\\V1\\SchemaContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SchemaContext.php',
    'Twilio\\Rest\\Events\\V1\\SchemaInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SchemaInstance.php',
    'Twilio\\Rest\\Events\\V1\\SchemaList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SchemaList.php',
    'Twilio\\Rest\\Events\\V1\\SchemaPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SchemaPage.php',
    'Twilio\\Rest\\Events\\V1\\Schema\\SchemaVersionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Schema/SchemaVersionContext.php',
    'Twilio\\Rest\\Events\\V1\\Schema\\SchemaVersionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Schema/SchemaVersionInstance.php',
    'Twilio\\Rest\\Events\\V1\\Schema\\SchemaVersionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Schema/SchemaVersionList.php',
    'Twilio\\Rest\\Events\\V1\\Schema\\SchemaVersionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Schema/SchemaVersionPage.php',
    'Twilio\\Rest\\Events\\V1\\SinkContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SinkContext.php',
    'Twilio\\Rest\\Events\\V1\\SinkInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SinkInstance.php',
    'Twilio\\Rest\\Events\\V1\\SinkList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SinkList.php',
    'Twilio\\Rest\\Events\\V1\\SinkOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SinkOptions.php',
    'Twilio\\Rest\\Events\\V1\\SinkPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SinkPage.php',
    'Twilio\\Rest\\Events\\V1\\Sink\\SinkTestInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Sink/SinkTestInstance.php',
    'Twilio\\Rest\\Events\\V1\\Sink\\SinkTestList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Sink/SinkTestList.php',
    'Twilio\\Rest\\Events\\V1\\Sink\\SinkTestPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Sink/SinkTestPage.php',
    'Twilio\\Rest\\Events\\V1\\Sink\\SinkValidateInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Sink/SinkValidateInstance.php',
    'Twilio\\Rest\\Events\\V1\\Sink\\SinkValidateList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Sink/SinkValidateList.php',
    'Twilio\\Rest\\Events\\V1\\Sink\\SinkValidatePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Sink/SinkValidatePage.php',
    'Twilio\\Rest\\Events\\V1\\SubscriptionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SubscriptionContext.php',
    'Twilio\\Rest\\Events\\V1\\SubscriptionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SubscriptionInstance.php',
    'Twilio\\Rest\\Events\\V1\\SubscriptionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SubscriptionList.php',
    'Twilio\\Rest\\Events\\V1\\SubscriptionOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SubscriptionOptions.php',
    'Twilio\\Rest\\Events\\V1\\SubscriptionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/SubscriptionPage.php',
    'Twilio\\Rest\\Events\\V1\\Subscription\\SubscribedEventContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Subscription/SubscribedEventContext.php',
    'Twilio\\Rest\\Events\\V1\\Subscription\\SubscribedEventInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Subscription/SubscribedEventInstance.php',
    'Twilio\\Rest\\Events\\V1\\Subscription\\SubscribedEventList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Subscription/SubscribedEventList.php',
    'Twilio\\Rest\\Events\\V1\\Subscription\\SubscribedEventOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Subscription/SubscribedEventOptions.php',
    'Twilio\\Rest\\Events\\V1\\Subscription\\SubscribedEventPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Events/V1/Subscription/SubscribedEventPage.php',
    'Twilio\\Rest\\FlexApi' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi.php',
    'Twilio\\Rest\\FlexApi\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1.php',
    'Twilio\\Rest\\FlexApi\\V1\\AssessmentsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/AssessmentsContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\AssessmentsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/AssessmentsInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\AssessmentsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/AssessmentsList.php',
    'Twilio\\Rest\\FlexApi\\V1\\AssessmentsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/AssessmentsOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\AssessmentsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/AssessmentsPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\ChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ChannelContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\ChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ChannelInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\ChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ChannelList.php',
    'Twilio\\Rest\\FlexApi\\V1\\ChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ChannelOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\ChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ChannelPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\ConfigurationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ConfigurationContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\ConfigurationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ConfigurationInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\ConfigurationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ConfigurationList.php',
    'Twilio\\Rest\\FlexApi\\V1\\ConfigurationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ConfigurationOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\ConfigurationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/ConfigurationPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\FlexFlowContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/FlexFlowContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\FlexFlowInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/FlexFlowInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\FlexFlowList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/FlexFlowList.php',
    'Twilio\\Rest\\FlexApi\\V1\\FlexFlowOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/FlexFlowOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\FlexFlowPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/FlexFlowPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsAssessmentsCommentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsAssessmentsCommentInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsAssessmentsCommentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsAssessmentsCommentList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsAssessmentsCommentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsAssessmentsCommentOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsAssessmentsCommentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsAssessmentsCommentPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesCategoryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesCategoryContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesCategoryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesCategoryInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesCategoryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesCategoryList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesCategoryOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesCategoryOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesCategoryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesCategoryPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesQuestionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesQuestionContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesQuestionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesQuestionInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesQuestionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesQuestionList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesQuestionOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesQuestionOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsQuestionnairesQuestionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsQuestionnairesQuestionPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSegmentsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSegmentsContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSegmentsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSegmentsInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSegmentsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSegmentsList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSegmentsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSegmentsOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSegmentsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSegmentsPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSessionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSessionContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSessionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSessionInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSessionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSessionList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSessionOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSessionOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSessionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSessionPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSettingsAnswerSetsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSettingsAnswerSetsInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSettingsAnswerSetsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSettingsAnswerSetsList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSettingsAnswerSetsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSettingsAnswerSetsOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSettingsAnswerSetsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSettingsAnswerSetsPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSettingsCommentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSettingsCommentInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSettingsCommentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSettingsCommentList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSettingsCommentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSettingsCommentOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsSettingsCommentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsSettingsCommentPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsUserRolesContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsUserRolesContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsUserRolesInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsUserRolesInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsUserRolesList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsUserRolesList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsUserRolesOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsUserRolesOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\InsightsUserRolesPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InsightsUserRolesPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\InteractionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InteractionContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\InteractionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InteractionInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\InteractionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InteractionList.php',
    'Twilio\\Rest\\FlexApi\\V1\\InteractionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/InteractionPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannelContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannelInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannelList.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannelOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannelPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannel\\InteractionChannelInviteInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannel/InteractionChannelInviteInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannel\\InteractionChannelInviteList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannel/InteractionChannelInviteList.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannel\\InteractionChannelInvitePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannel/InteractionChannelInvitePage.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannel\\InteractionChannelParticipantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannel/InteractionChannelParticipantContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannel\\InteractionChannelParticipantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannel/InteractionChannelParticipantInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannel\\InteractionChannelParticipantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannel/InteractionChannelParticipantList.php',
    'Twilio\\Rest\\FlexApi\\V1\\Interaction\\InteractionChannel\\InteractionChannelParticipantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/Interaction/InteractionChannel/InteractionChannelParticipantPage.php',
    'Twilio\\Rest\\FlexApi\\V1\\WebChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/WebChannelContext.php',
    'Twilio\\Rest\\FlexApi\\V1\\WebChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/WebChannelInstance.php',
    'Twilio\\Rest\\FlexApi\\V1\\WebChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/WebChannelList.php',
    'Twilio\\Rest\\FlexApi\\V1\\WebChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/WebChannelOptions.php',
    'Twilio\\Rest\\FlexApi\\V1\\WebChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V1/WebChannelPage.php',
    'Twilio\\Rest\\FlexApi\\V2' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V2.php',
    'Twilio\\Rest\\FlexApi\\V2\\WebChannelsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V2/WebChannelsInstance.php',
    'Twilio\\Rest\\FlexApi\\V2\\WebChannelsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V2/WebChannelsList.php',
    'Twilio\\Rest\\FlexApi\\V2\\WebChannelsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V2/WebChannelsOptions.php',
    'Twilio\\Rest\\FlexApi\\V2\\WebChannelsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FlexApi/V2/WebChannelsPage.php',
    'Twilio\\Rest\\FrontlineApi' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FrontlineApi.php',
    'Twilio\\Rest\\FrontlineApi\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FrontlineApi/V1.php',
    'Twilio\\Rest\\FrontlineApi\\V1\\UserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FrontlineApi/V1/UserContext.php',
    'Twilio\\Rest\\FrontlineApi\\V1\\UserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FrontlineApi/V1/UserInstance.php',
    'Twilio\\Rest\\FrontlineApi\\V1\\UserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FrontlineApi/V1/UserList.php',
    'Twilio\\Rest\\FrontlineApi\\V1\\UserOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FrontlineApi/V1/UserOptions.php',
    'Twilio\\Rest\\FrontlineApi\\V1\\UserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/FrontlineApi/V1/UserPage.php',
    'Twilio\\Rest\\Insights' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights.php',
    'Twilio\\Rest\\Insights\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1.php',
    'Twilio\\Rest\\Insights\\V1\\CallContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/CallContext.php',
    'Twilio\\Rest\\Insights\\V1\\CallInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/CallInstance.php',
    'Twilio\\Rest\\Insights\\V1\\CallList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/CallList.php',
    'Twilio\\Rest\\Insights\\V1\\CallPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/CallPage.php',
    'Twilio\\Rest\\Insights\\V1\\CallSummariesInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/CallSummariesInstance.php',
    'Twilio\\Rest\\Insights\\V1\\CallSummariesList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/CallSummariesList.php',
    'Twilio\\Rest\\Insights\\V1\\CallSummariesOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/CallSummariesOptions.php',
    'Twilio\\Rest\\Insights\\V1\\CallSummariesPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/CallSummariesPage.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\AnnotationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/AnnotationContext.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\AnnotationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/AnnotationInstance.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\AnnotationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/AnnotationList.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\AnnotationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/AnnotationOptions.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\AnnotationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/AnnotationPage.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\CallSummaryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/CallSummaryContext.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\CallSummaryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/CallSummaryInstance.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\CallSummaryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/CallSummaryList.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\CallSummaryOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/CallSummaryOptions.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\CallSummaryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/CallSummaryPage.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\EventInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/EventInstance.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\EventList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/EventList.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\EventOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/EventOptions.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\EventPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/EventPage.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\MetricInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/MetricInstance.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\MetricList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/MetricList.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\MetricOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/MetricOptions.php',
    'Twilio\\Rest\\Insights\\V1\\Call\\MetricPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Call/MetricPage.php',
    'Twilio\\Rest\\Insights\\V1\\ConferenceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/ConferenceContext.php',
    'Twilio\\Rest\\Insights\\V1\\ConferenceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/ConferenceInstance.php',
    'Twilio\\Rest\\Insights\\V1\\ConferenceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/ConferenceList.php',
    'Twilio\\Rest\\Insights\\V1\\ConferenceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/ConferenceOptions.php',
    'Twilio\\Rest\\Insights\\V1\\ConferencePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/ConferencePage.php',
    'Twilio\\Rest\\Insights\\V1\\Conference\\ConferenceParticipantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Conference/ConferenceParticipantContext.php',
    'Twilio\\Rest\\Insights\\V1\\Conference\\ConferenceParticipantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Conference/ConferenceParticipantInstance.php',
    'Twilio\\Rest\\Insights\\V1\\Conference\\ConferenceParticipantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Conference/ConferenceParticipantList.php',
    'Twilio\\Rest\\Insights\\V1\\Conference\\ConferenceParticipantOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Conference/ConferenceParticipantOptions.php',
    'Twilio\\Rest\\Insights\\V1\\Conference\\ConferenceParticipantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Conference/ConferenceParticipantPage.php',
    'Twilio\\Rest\\Insights\\V1\\RoomContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/RoomContext.php',
    'Twilio\\Rest\\Insights\\V1\\RoomInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/RoomInstance.php',
    'Twilio\\Rest\\Insights\\V1\\RoomList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/RoomList.php',
    'Twilio\\Rest\\Insights\\V1\\RoomOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/RoomOptions.php',
    'Twilio\\Rest\\Insights\\V1\\RoomPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/RoomPage.php',
    'Twilio\\Rest\\Insights\\V1\\Room\\ParticipantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Room/ParticipantContext.php',
    'Twilio\\Rest\\Insights\\V1\\Room\\ParticipantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Room/ParticipantInstance.php',
    'Twilio\\Rest\\Insights\\V1\\Room\\ParticipantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Room/ParticipantList.php',
    'Twilio\\Rest\\Insights\\V1\\Room\\ParticipantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/Room/ParticipantPage.php',
    'Twilio\\Rest\\Insights\\V1\\SettingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/SettingContext.php',
    'Twilio\\Rest\\Insights\\V1\\SettingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/SettingInstance.php',
    'Twilio\\Rest\\Insights\\V1\\SettingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/SettingList.php',
    'Twilio\\Rest\\Insights\\V1\\SettingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/SettingOptions.php',
    'Twilio\\Rest\\Insights\\V1\\SettingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Insights/V1/SettingPage.php',
    'Twilio\\Rest\\IpMessaging' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging.php',
    'Twilio\\Rest\\IpMessaging\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1.php',
    'Twilio\\Rest\\IpMessaging\\V1\\CredentialContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/CredentialContext.php',
    'Twilio\\Rest\\IpMessaging\\V1\\CredentialInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/CredentialInstance.php',
    'Twilio\\Rest\\IpMessaging\\V1\\CredentialList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/CredentialList.php',
    'Twilio\\Rest\\IpMessaging\\V1\\CredentialOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/CredentialOptions.php',
    'Twilio\\Rest\\IpMessaging\\V1\\CredentialPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/CredentialPage.php',
    'Twilio\\Rest\\IpMessaging\\V1\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/ServiceContext.php',
    'Twilio\\Rest\\IpMessaging\\V1\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/ServiceInstance.php',
    'Twilio\\Rest\\IpMessaging\\V1\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/ServiceList.php',
    'Twilio\\Rest\\IpMessaging\\V1\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/ServiceOptions.php',
    'Twilio\\Rest\\IpMessaging\\V1\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/ServicePage.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\ChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/ChannelContext.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\ChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/ChannelInstance.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\ChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/ChannelList.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\ChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/ChannelOptions.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\ChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/ChannelPage.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\InviteContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/InviteContext.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\InviteInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/InviteInstance.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\InviteList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/InviteList.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\InviteOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/InviteOptions.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\InvitePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/InvitePage.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MemberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MemberContext.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MemberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MemberInstance.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MemberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MemberList.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MemberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MemberOptions.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MemberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MemberPage.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MessageContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MessageContext.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MessageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MessageInstance.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MessageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MessageList.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MessageOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MessageOptions.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\Channel\\MessagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/Channel/MessagePage.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\RoleContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/RoleContext.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\RoleInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/RoleInstance.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\RoleList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/RoleList.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\RolePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/RolePage.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\UserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/UserContext.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\UserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/UserInstance.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\UserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/UserList.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\UserOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/UserOptions.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\UserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/UserPage.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\User\\UserChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/User/UserChannelInstance.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\User\\UserChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/User/UserChannelList.php',
    'Twilio\\Rest\\IpMessaging\\V1\\Service\\User\\UserChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V1/Service/User/UserChannelPage.php',
    'Twilio\\Rest\\IpMessaging\\V2' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2.php',
    'Twilio\\Rest\\IpMessaging\\V2\\CredentialContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/CredentialContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\CredentialInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/CredentialInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\CredentialList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/CredentialList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\CredentialOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/CredentialOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\CredentialPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/CredentialPage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/ServiceContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/ServiceInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/ServiceList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/ServiceOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/ServicePage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\BindingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/BindingContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\BindingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/BindingInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\BindingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/BindingList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\BindingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/BindingOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\BindingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/BindingPage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\ChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/ChannelContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\ChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/ChannelInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\ChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/ChannelList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\ChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/ChannelOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\ChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/ChannelPage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\InviteContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/InviteContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\InviteInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/InviteInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\InviteList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/InviteList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\InviteOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/InviteOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\InvitePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/InvitePage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MemberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MemberContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MemberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MemberInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MemberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MemberList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MemberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MemberOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MemberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MemberPage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MessageContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MessageContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MessageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MessageInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MessageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MessageList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MessageOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MessageOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\MessagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/MessagePage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\WebhookContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/WebhookContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\WebhookInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/WebhookInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\WebhookList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/WebhookList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\WebhookOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/WebhookOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\Channel\\WebhookPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/Channel/WebhookPage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\RoleContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/RoleContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\RoleInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/RoleInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\RoleList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/RoleList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\RolePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/RolePage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\UserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/UserContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\UserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/UserInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\UserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/UserList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\UserOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/UserOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\UserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/UserPage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserBindingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserBindingContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserBindingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserBindingInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserBindingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserBindingList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserBindingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserBindingOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserBindingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserBindingPage.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserChannelContext.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserChannelInstance.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserChannelList.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserChannelOptions.php',
    'Twilio\\Rest\\IpMessaging\\V2\\Service\\User\\UserChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/IpMessaging/V2/Service/User/UserChannelPage.php',
    'Twilio\\Rest\\Lookups' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups.php',
    'Twilio\\Rest\\Lookups\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V1.php',
    'Twilio\\Rest\\Lookups\\V1\\PhoneNumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V1/PhoneNumberContext.php',
    'Twilio\\Rest\\Lookups\\V1\\PhoneNumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V1/PhoneNumberInstance.php',
    'Twilio\\Rest\\Lookups\\V1\\PhoneNumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V1/PhoneNumberList.php',
    'Twilio\\Rest\\Lookups\\V1\\PhoneNumberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V1/PhoneNumberOptions.php',
    'Twilio\\Rest\\Lookups\\V1\\PhoneNumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V1/PhoneNumberPage.php',
    'Twilio\\Rest\\Lookups\\V2' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V2.php',
    'Twilio\\Rest\\Lookups\\V2\\PhoneNumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V2/PhoneNumberContext.php',
    'Twilio\\Rest\\Lookups\\V2\\PhoneNumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V2/PhoneNumberInstance.php',
    'Twilio\\Rest\\Lookups\\V2\\PhoneNumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V2/PhoneNumberList.php',
    'Twilio\\Rest\\Lookups\\V2\\PhoneNumberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V2/PhoneNumberOptions.php',
    'Twilio\\Rest\\Lookups\\V2\\PhoneNumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Lookups/V2/PhoneNumberPage.php',
    'Twilio\\Rest\\Media' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media.php',
    'Twilio\\Rest\\Media\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1.php',
    'Twilio\\Rest\\Media\\V1\\MediaProcessorContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaProcessorContext.php',
    'Twilio\\Rest\\Media\\V1\\MediaProcessorInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaProcessorInstance.php',
    'Twilio\\Rest\\Media\\V1\\MediaProcessorList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaProcessorList.php',
    'Twilio\\Rest\\Media\\V1\\MediaProcessorOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaProcessorOptions.php',
    'Twilio\\Rest\\Media\\V1\\MediaProcessorPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaProcessorPage.php',
    'Twilio\\Rest\\Media\\V1\\MediaRecordingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaRecordingContext.php',
    'Twilio\\Rest\\Media\\V1\\MediaRecordingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaRecordingInstance.php',
    'Twilio\\Rest\\Media\\V1\\MediaRecordingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaRecordingList.php',
    'Twilio\\Rest\\Media\\V1\\MediaRecordingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaRecordingOptions.php',
    'Twilio\\Rest\\Media\\V1\\MediaRecordingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/MediaRecordingPage.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamerContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamerContext.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamerInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamerInstance.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamerList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamerList.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamerOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamerOptions.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamerPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamerPage.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamer\\PlaybackGrantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamer/PlaybackGrantContext.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamer\\PlaybackGrantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamer/PlaybackGrantInstance.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamer\\PlaybackGrantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamer/PlaybackGrantList.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamer\\PlaybackGrantOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamer/PlaybackGrantOptions.php',
    'Twilio\\Rest\\Media\\V1\\PlayerStreamer\\PlaybackGrantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Media/V1/PlayerStreamer/PlaybackGrantPage.php',
    'Twilio\\Rest\\Messaging' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging.php',
    'Twilio\\Rest\\Messaging\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistrationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistrationContext.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistrationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistrationInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistrationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistrationList.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistrationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistrationOptions.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistrationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistrationPage.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistration\\BrandVettingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistration/BrandVettingContext.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistration\\BrandVettingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistration/BrandVettingInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistration\\BrandVettingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistration/BrandVettingList.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistration\\BrandVettingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistration/BrandVettingOptions.php',
    'Twilio\\Rest\\Messaging\\V1\\BrandRegistration\\BrandVettingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/BrandRegistration/BrandVettingPage.php',
    'Twilio\\Rest\\Messaging\\V1\\DeactivationsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DeactivationsContext.php',
    'Twilio\\Rest\\Messaging\\V1\\DeactivationsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DeactivationsInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\DeactivationsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DeactivationsList.php',
    'Twilio\\Rest\\Messaging\\V1\\DeactivationsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DeactivationsOptions.php',
    'Twilio\\Rest\\Messaging\\V1\\DeactivationsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DeactivationsPage.php',
    'Twilio\\Rest\\Messaging\\V1\\DomainCertsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DomainCertsContext.php',
    'Twilio\\Rest\\Messaging\\V1\\DomainCertsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DomainCertsInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\DomainCertsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DomainCertsList.php',
    'Twilio\\Rest\\Messaging\\V1\\DomainCertsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DomainCertsPage.php',
    'Twilio\\Rest\\Messaging\\V1\\DomainConfigContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DomainConfigContext.php',
    'Twilio\\Rest\\Messaging\\V1\\DomainConfigInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DomainConfigInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\DomainConfigList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DomainConfigList.php',
    'Twilio\\Rest\\Messaging\\V1\\DomainConfigOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DomainConfigOptions.php',
    'Twilio\\Rest\\Messaging\\V1\\DomainConfigPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/DomainConfigPage.php',
    'Twilio\\Rest\\Messaging\\V1\\ExternalCampaignInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/ExternalCampaignInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\ExternalCampaignList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/ExternalCampaignList.php',
    'Twilio\\Rest\\Messaging\\V1\\ExternalCampaignPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/ExternalCampaignPage.php',
    'Twilio\\Rest\\Messaging\\V1\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/ServiceContext.php',
    'Twilio\\Rest\\Messaging\\V1\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/ServiceInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/ServiceList.php',
    'Twilio\\Rest\\Messaging\\V1\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/ServiceOptions.php',
    'Twilio\\Rest\\Messaging\\V1\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/ServicePage.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\AlphaSenderContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/AlphaSenderContext.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\AlphaSenderInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/AlphaSenderInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\AlphaSenderList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/AlphaSenderList.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\AlphaSenderPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/AlphaSenderPage.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\PhoneNumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/PhoneNumberContext.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\PhoneNumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/PhoneNumberInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\PhoneNumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/PhoneNumberList.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\PhoneNumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/PhoneNumberPage.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\ShortCodeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/ShortCodeContext.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\ShortCodeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/ShortCodeInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\ShortCodeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/ShortCodeList.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\ShortCodePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/ShortCodePage.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\UsAppToPersonContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/UsAppToPersonContext.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\UsAppToPersonInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/UsAppToPersonInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\UsAppToPersonList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/UsAppToPersonList.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\UsAppToPersonOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/UsAppToPersonOptions.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\UsAppToPersonPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/UsAppToPersonPage.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\UsAppToPersonUsecaseInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/UsAppToPersonUsecaseInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\UsAppToPersonUsecaseList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/UsAppToPersonUsecaseList.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\UsAppToPersonUsecaseOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/UsAppToPersonUsecaseOptions.php',
    'Twilio\\Rest\\Messaging\\V1\\Service\\UsAppToPersonUsecasePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/Service/UsAppToPersonUsecasePage.php',
    'Twilio\\Rest\\Messaging\\V1\\TollfreeVerificationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/TollfreeVerificationContext.php',
    'Twilio\\Rest\\Messaging\\V1\\TollfreeVerificationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/TollfreeVerificationInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\TollfreeVerificationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/TollfreeVerificationList.php',
    'Twilio\\Rest\\Messaging\\V1\\TollfreeVerificationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/TollfreeVerificationOptions.php',
    'Twilio\\Rest\\Messaging\\V1\\TollfreeVerificationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/TollfreeVerificationPage.php',
    'Twilio\\Rest\\Messaging\\V1\\UsecaseInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/UsecaseInstance.php',
    'Twilio\\Rest\\Messaging\\V1\\UsecaseList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/UsecaseList.php',
    'Twilio\\Rest\\Messaging\\V1\\UsecasePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Messaging/V1/UsecasePage.php',
    'Twilio\\Rest\\Microvisor' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor.php',
    'Twilio\\Rest\\Microvisor\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1.php',
    'Twilio\\Rest\\Microvisor\\V1\\AccountConfigContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AccountConfigContext.php',
    'Twilio\\Rest\\Microvisor\\V1\\AccountConfigInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AccountConfigInstance.php',
    'Twilio\\Rest\\Microvisor\\V1\\AccountConfigList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AccountConfigList.php',
    'Twilio\\Rest\\Microvisor\\V1\\AccountConfigPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AccountConfigPage.php',
    'Twilio\\Rest\\Microvisor\\V1\\AccountSecretContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AccountSecretContext.php',
    'Twilio\\Rest\\Microvisor\\V1\\AccountSecretInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AccountSecretInstance.php',
    'Twilio\\Rest\\Microvisor\\V1\\AccountSecretList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AccountSecretList.php',
    'Twilio\\Rest\\Microvisor\\V1\\AccountSecretPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AccountSecretPage.php',
    'Twilio\\Rest\\Microvisor\\V1\\AppContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AppContext.php',
    'Twilio\\Rest\\Microvisor\\V1\\AppInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AppInstance.php',
    'Twilio\\Rest\\Microvisor\\V1\\AppList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AppList.php',
    'Twilio\\Rest\\Microvisor\\V1\\AppPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/AppPage.php',
    'Twilio\\Rest\\Microvisor\\V1\\DeviceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/DeviceContext.php',
    'Twilio\\Rest\\Microvisor\\V1\\DeviceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/DeviceInstance.php',
    'Twilio\\Rest\\Microvisor\\V1\\DeviceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/DeviceList.php',
    'Twilio\\Rest\\Microvisor\\V1\\DeviceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/DeviceOptions.php',
    'Twilio\\Rest\\Microvisor\\V1\\DevicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/DevicePage.php',
    'Twilio\\Rest\\Microvisor\\V1\\Device\\DeviceConfigContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/Device/DeviceConfigContext.php',
    'Twilio\\Rest\\Microvisor\\V1\\Device\\DeviceConfigInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/Device/DeviceConfigInstance.php',
    'Twilio\\Rest\\Microvisor\\V1\\Device\\DeviceConfigList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/Device/DeviceConfigList.php',
    'Twilio\\Rest\\Microvisor\\V1\\Device\\DeviceConfigPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/Device/DeviceConfigPage.php',
    'Twilio\\Rest\\Microvisor\\V1\\Device\\DeviceSecretContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/Device/DeviceSecretContext.php',
    'Twilio\\Rest\\Microvisor\\V1\\Device\\DeviceSecretInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/Device/DeviceSecretInstance.php',
    'Twilio\\Rest\\Microvisor\\V1\\Device\\DeviceSecretList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/Device/DeviceSecretList.php',
    'Twilio\\Rest\\Microvisor\\V1\\Device\\DeviceSecretPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Microvisor/V1/Device/DeviceSecretPage.php',
    'Twilio\\Rest\\Monitor' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor.php',
    'Twilio\\Rest\\Monitor\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1.php',
    'Twilio\\Rest\\Monitor\\V1\\AlertContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/AlertContext.php',
    'Twilio\\Rest\\Monitor\\V1\\AlertInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/AlertInstance.php',
    'Twilio\\Rest\\Monitor\\V1\\AlertList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/AlertList.php',
    'Twilio\\Rest\\Monitor\\V1\\AlertOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/AlertOptions.php',
    'Twilio\\Rest\\Monitor\\V1\\AlertPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/AlertPage.php',
    'Twilio\\Rest\\Monitor\\V1\\EventContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/EventContext.php',
    'Twilio\\Rest\\Monitor\\V1\\EventInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/EventInstance.php',
    'Twilio\\Rest\\Monitor\\V1\\EventList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/EventList.php',
    'Twilio\\Rest\\Monitor\\V1\\EventOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/EventOptions.php',
    'Twilio\\Rest\\Monitor\\V1\\EventPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Monitor/V1/EventPage.php',
    'Twilio\\Rest\\Notify' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify.php',
    'Twilio\\Rest\\Notify\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1.php',
    'Twilio\\Rest\\Notify\\V1\\CredentialContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/CredentialContext.php',
    'Twilio\\Rest\\Notify\\V1\\CredentialInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/CredentialInstance.php',
    'Twilio\\Rest\\Notify\\V1\\CredentialList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/CredentialList.php',
    'Twilio\\Rest\\Notify\\V1\\CredentialOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/CredentialOptions.php',
    'Twilio\\Rest\\Notify\\V1\\CredentialPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/CredentialPage.php',
    'Twilio\\Rest\\Notify\\V1\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/ServiceContext.php',
    'Twilio\\Rest\\Notify\\V1\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/ServiceInstance.php',
    'Twilio\\Rest\\Notify\\V1\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/ServiceList.php',
    'Twilio\\Rest\\Notify\\V1\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/ServiceOptions.php',
    'Twilio\\Rest\\Notify\\V1\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/ServicePage.php',
    'Twilio\\Rest\\Notify\\V1\\Service\\BindingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/Service/BindingContext.php',
    'Twilio\\Rest\\Notify\\V1\\Service\\BindingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/Service/BindingInstance.php',
    'Twilio\\Rest\\Notify\\V1\\Service\\BindingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/Service/BindingList.php',
    'Twilio\\Rest\\Notify\\V1\\Service\\BindingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/Service/BindingOptions.php',
    'Twilio\\Rest\\Notify\\V1\\Service\\BindingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/Service/BindingPage.php',
    'Twilio\\Rest\\Notify\\V1\\Service\\NotificationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/Service/NotificationInstance.php',
    'Twilio\\Rest\\Notify\\V1\\Service\\NotificationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/Service/NotificationList.php',
    'Twilio\\Rest\\Notify\\V1\\Service\\NotificationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/Service/NotificationOptions.php',
    'Twilio\\Rest\\Notify\\V1\\Service\\NotificationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Notify/V1/Service/NotificationPage.php',
    'Twilio\\Rest\\Numbers' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers.php',
    'Twilio\\Rest\\Numbers\\V2' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryComplianceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryComplianceInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryComplianceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryComplianceList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliancePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliancePage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\BundleContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/BundleContext.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\BundleInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/BundleInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\BundleList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/BundleList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\BundleOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/BundleOptions.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\BundlePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/BundlePage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\BundleCopyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/BundleCopyInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\BundleCopyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/BundleCopyList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\BundleCopyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/BundleCopyOptions.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\BundleCopyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/BundleCopyPage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\EvaluationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/EvaluationContext.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\EvaluationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/EvaluationInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\EvaluationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/EvaluationList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\EvaluationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/EvaluationPage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\ItemAssignmentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/ItemAssignmentContext.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\ItemAssignmentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/ItemAssignmentInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\ItemAssignmentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/ItemAssignmentList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\ItemAssignmentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/ItemAssignmentPage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\ReplaceItemsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/ReplaceItemsInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\ReplaceItemsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/ReplaceItemsList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\Bundle\\ReplaceItemsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/Bundle/ReplaceItemsPage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\EndUserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/EndUserContext.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\EndUserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/EndUserInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\EndUserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/EndUserList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\EndUserOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/EndUserOptions.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\EndUserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/EndUserPage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\EndUserTypeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/EndUserTypeContext.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\EndUserTypeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/EndUserTypeInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\EndUserTypeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/EndUserTypeList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\EndUserTypePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/EndUserTypePage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\RegulationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/RegulationContext.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\RegulationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/RegulationInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\RegulationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/RegulationList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\RegulationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/RegulationOptions.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\RegulationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/RegulationPage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\SupportingDocumentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/SupportingDocumentContext.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\SupportingDocumentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/SupportingDocumentInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\SupportingDocumentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/SupportingDocumentList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\SupportingDocumentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/SupportingDocumentOptions.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\SupportingDocumentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/SupportingDocumentPage.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\SupportingDocumentTypeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/SupportingDocumentTypeContext.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\SupportingDocumentTypeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/SupportingDocumentTypeInstance.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\SupportingDocumentTypeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/SupportingDocumentTypeList.php',
    'Twilio\\Rest\\Numbers\\V2\\RegulatoryCompliance\\SupportingDocumentTypePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Numbers/V2/RegulatoryCompliance/SupportingDocumentTypePage.php',
    'Twilio\\Rest\\Oauth' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth.php',
    'Twilio\\Rest\\Oauth\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1.php',
    'Twilio\\Rest\\Oauth\\V1\\DeviceCodeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/DeviceCodeInstance.php',
    'Twilio\\Rest\\Oauth\\V1\\DeviceCodeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/DeviceCodeList.php',
    'Twilio\\Rest\\Oauth\\V1\\DeviceCodeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/DeviceCodeOptions.php',
    'Twilio\\Rest\\Oauth\\V1\\DeviceCodePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/DeviceCodePage.php',
    'Twilio\\Rest\\Oauth\\V1\\OauthContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/OauthContext.php',
    'Twilio\\Rest\\Oauth\\V1\\OauthInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/OauthInstance.php',
    'Twilio\\Rest\\Oauth\\V1\\OauthList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/OauthList.php',
    'Twilio\\Rest\\Oauth\\V1\\OauthPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/OauthPage.php',
    'Twilio\\Rest\\Oauth\\V1\\OpenidDiscoveryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/OpenidDiscoveryContext.php',
    'Twilio\\Rest\\Oauth\\V1\\OpenidDiscoveryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/OpenidDiscoveryInstance.php',
    'Twilio\\Rest\\Oauth\\V1\\OpenidDiscoveryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/OpenidDiscoveryList.php',
    'Twilio\\Rest\\Oauth\\V1\\OpenidDiscoveryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/OpenidDiscoveryPage.php',
    'Twilio\\Rest\\Oauth\\V1\\TokenInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/TokenInstance.php',
    'Twilio\\Rest\\Oauth\\V1\\TokenList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/TokenList.php',
    'Twilio\\Rest\\Oauth\\V1\\TokenOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/TokenOptions.php',
    'Twilio\\Rest\\Oauth\\V1\\TokenPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/TokenPage.php',
    'Twilio\\Rest\\Oauth\\V1\\UserInfoContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/UserInfoContext.php',
    'Twilio\\Rest\\Oauth\\V1\\UserInfoInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/UserInfoInstance.php',
    'Twilio\\Rest\\Oauth\\V1\\UserInfoList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/UserInfoList.php',
    'Twilio\\Rest\\Oauth\\V1\\UserInfoPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Oauth/V1/UserInfoPage.php',
    'Twilio\\Rest\\Preview' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview.php',
    'Twilio\\Rest\\Preview\\DeployedDevices' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\FleetContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/FleetContext.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\FleetInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/FleetInstance.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\FleetList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/FleetList.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\FleetOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/FleetOptions.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\FleetPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/FleetPage.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\CertificateContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/CertificateContext.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\CertificateInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/CertificateInstance.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\CertificateList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/CertificateList.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\CertificateOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/CertificateOptions.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\CertificatePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/CertificatePage.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DeploymentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DeploymentContext.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DeploymentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DeploymentInstance.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DeploymentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DeploymentList.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DeploymentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DeploymentOptions.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DeploymentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DeploymentPage.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DeviceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DeviceContext.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DeviceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DeviceInstance.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DeviceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DeviceList.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DeviceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DeviceOptions.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\DevicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/DevicePage.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\KeyContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/KeyContext.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\KeyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/KeyInstance.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\KeyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/KeyList.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\KeyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/KeyOptions.php',
    'Twilio\\Rest\\Preview\\DeployedDevices\\Fleet\\KeyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/DeployedDevices/Fleet/KeyPage.php',
    'Twilio\\Rest\\Preview\\HostedNumbers' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\AuthorizationDocumentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/AuthorizationDocumentContext.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\AuthorizationDocumentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/AuthorizationDocumentInstance.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\AuthorizationDocumentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/AuthorizationDocumentList.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\AuthorizationDocumentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/AuthorizationDocumentOptions.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\AuthorizationDocumentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/AuthorizationDocumentPage.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\AuthorizationDocument\\DependentHostedNumberOrderInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/AuthorizationDocument/DependentHostedNumberOrderInstance.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\AuthorizationDocument\\DependentHostedNumberOrderList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/AuthorizationDocument/DependentHostedNumberOrderList.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\AuthorizationDocument\\DependentHostedNumberOrderOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/AuthorizationDocument/DependentHostedNumberOrderOptions.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\AuthorizationDocument\\DependentHostedNumberOrderPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/AuthorizationDocument/DependentHostedNumberOrderPage.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\HostedNumberOrderContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/HostedNumberOrderContext.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\HostedNumberOrderInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/HostedNumberOrderInstance.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\HostedNumberOrderList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/HostedNumberOrderList.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\HostedNumberOrderOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/HostedNumberOrderOptions.php',
    'Twilio\\Rest\\Preview\\HostedNumbers\\HostedNumberOrderPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/HostedNumbers/HostedNumberOrderPage.php',
    'Twilio\\Rest\\Preview\\Marketplace' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace.php',
    'Twilio\\Rest\\Preview\\Marketplace\\AvailableAddOnContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/AvailableAddOnContext.php',
    'Twilio\\Rest\\Preview\\Marketplace\\AvailableAddOnInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/AvailableAddOnInstance.php',
    'Twilio\\Rest\\Preview\\Marketplace\\AvailableAddOnList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/AvailableAddOnList.php',
    'Twilio\\Rest\\Preview\\Marketplace\\AvailableAddOnPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/AvailableAddOnPage.php',
    'Twilio\\Rest\\Preview\\Marketplace\\AvailableAddOn\\AvailableAddOnExtensionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/AvailableAddOn/AvailableAddOnExtensionContext.php',
    'Twilio\\Rest\\Preview\\Marketplace\\AvailableAddOn\\AvailableAddOnExtensionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/AvailableAddOn/AvailableAddOnExtensionInstance.php',
    'Twilio\\Rest\\Preview\\Marketplace\\AvailableAddOn\\AvailableAddOnExtensionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/AvailableAddOn/AvailableAddOnExtensionList.php',
    'Twilio\\Rest\\Preview\\Marketplace\\AvailableAddOn\\AvailableAddOnExtensionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/AvailableAddOn/AvailableAddOnExtensionPage.php',
    'Twilio\\Rest\\Preview\\Marketplace\\InstalledAddOnContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/InstalledAddOnContext.php',
    'Twilio\\Rest\\Preview\\Marketplace\\InstalledAddOnInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/InstalledAddOnInstance.php',
    'Twilio\\Rest\\Preview\\Marketplace\\InstalledAddOnList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/InstalledAddOnList.php',
    'Twilio\\Rest\\Preview\\Marketplace\\InstalledAddOnOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/InstalledAddOnOptions.php',
    'Twilio\\Rest\\Preview\\Marketplace\\InstalledAddOnPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/InstalledAddOnPage.php',
    'Twilio\\Rest\\Preview\\Marketplace\\InstalledAddOn\\InstalledAddOnExtensionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/InstalledAddOn/InstalledAddOnExtensionContext.php',
    'Twilio\\Rest\\Preview\\Marketplace\\InstalledAddOn\\InstalledAddOnExtensionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/InstalledAddOn/InstalledAddOnExtensionInstance.php',
    'Twilio\\Rest\\Preview\\Marketplace\\InstalledAddOn\\InstalledAddOnExtensionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/InstalledAddOn/InstalledAddOnExtensionList.php',
    'Twilio\\Rest\\Preview\\Marketplace\\InstalledAddOn\\InstalledAddOnExtensionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Marketplace/InstalledAddOn/InstalledAddOnExtensionPage.php',
    'Twilio\\Rest\\Preview\\Sync' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync.php',
    'Twilio\\Rest\\Preview\\Sync\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/ServiceContext.php',
    'Twilio\\Rest\\Preview\\Sync\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/ServiceInstance.php',
    'Twilio\\Rest\\Preview\\Sync\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/ServiceList.php',
    'Twilio\\Rest\\Preview\\Sync\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/ServiceOptions.php',
    'Twilio\\Rest\\Preview\\Sync\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/ServicePage.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\DocumentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/DocumentContext.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\DocumentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/DocumentInstance.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\DocumentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/DocumentList.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\DocumentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/DocumentOptions.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\DocumentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/DocumentPage.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\Document\\DocumentPermissionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/Document/DocumentPermissionContext.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\Document\\DocumentPermissionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/Document/DocumentPermissionInstance.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\Document\\DocumentPermissionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/Document/DocumentPermissionList.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\Document\\DocumentPermissionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/Document/DocumentPermissionPage.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncListContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncListContext.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncListInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncListInstance.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncListList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncListList.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncListOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncListOptions.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncListPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncListPage.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncList\\SyncListItemContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncList/SyncListItemContext.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncList\\SyncListItemInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncList/SyncListItemInstance.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncList\\SyncListItemList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncList/SyncListItemList.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncList\\SyncListItemOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncList/SyncListItemOptions.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncList\\SyncListItemPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncList/SyncListItemPage.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncList\\SyncListPermissionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncList/SyncListPermissionContext.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncList\\SyncListPermissionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncList/SyncListPermissionInstance.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncList\\SyncListPermissionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncList/SyncListPermissionList.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncList\\SyncListPermissionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncList/SyncListPermissionPage.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMapContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMapContext.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMapInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMapInstance.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMapList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMapList.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMapOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMapOptions.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMapPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMapPage.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMap\\SyncMapItemContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMap/SyncMapItemContext.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMap\\SyncMapItemInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMap/SyncMapItemInstance.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMap\\SyncMapItemList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMap/SyncMapItemList.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMap\\SyncMapItemOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMap/SyncMapItemOptions.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMap\\SyncMapItemPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMap/SyncMapItemPage.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMap\\SyncMapPermissionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMap/SyncMapPermissionContext.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMap\\SyncMapPermissionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMap/SyncMapPermissionInstance.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMap\\SyncMapPermissionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMap/SyncMapPermissionList.php',
    'Twilio\\Rest\\Preview\\Sync\\Service\\SyncMap\\SyncMapPermissionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Sync/Service/SyncMap/SyncMapPermissionPage.php',
    'Twilio\\Rest\\Preview\\Understand' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand.php',
    'Twilio\\Rest\\Preview\\Understand\\AssistantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/AssistantContext.php',
    'Twilio\\Rest\\Preview\\Understand\\AssistantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/AssistantInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\AssistantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/AssistantList.php',
    'Twilio\\Rest\\Preview\\Understand\\AssistantOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/AssistantOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\AssistantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/AssistantPage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantFallbackActionsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantFallbackActionsContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantFallbackActionsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantFallbackActionsInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantFallbackActionsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantFallbackActionsList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantFallbackActionsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantFallbackActionsOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantFallbackActionsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantFallbackActionsPage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantInitiationActionsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantInitiationActionsContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantInitiationActionsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantInitiationActionsInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantInitiationActionsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantInitiationActionsList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantInitiationActionsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantInitiationActionsOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\AssistantInitiationActionsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/AssistantInitiationActionsPage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\DialogueContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/DialogueContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\DialogueInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/DialogueInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\DialogueList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/DialogueList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\DialoguePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/DialoguePage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldTypeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldTypeContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldTypeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldTypeInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldTypeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldTypeList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldTypeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldTypeOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldTypePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldTypePage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldType\\FieldValueContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldType/FieldValueContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldType\\FieldValueInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldType/FieldValueInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldType\\FieldValueList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldType/FieldValueList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldType\\FieldValueOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldType/FieldValueOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\FieldType\\FieldValuePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/FieldType/FieldValuePage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\ModelBuildContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/ModelBuildContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\ModelBuildInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/ModelBuildInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\ModelBuildList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/ModelBuildList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\ModelBuildOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/ModelBuildOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\ModelBuildPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/ModelBuildPage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\QueryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/QueryContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\QueryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/QueryInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\QueryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/QueryList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\QueryOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/QueryOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\QueryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/QueryPage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\StyleSheetContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/StyleSheetContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\StyleSheetInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/StyleSheetInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\StyleSheetList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/StyleSheetList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\StyleSheetOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/StyleSheetOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\StyleSheetPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/StyleSheetPage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\TaskContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/TaskContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\TaskInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/TaskInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\TaskList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/TaskList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\TaskOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/TaskOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\TaskPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/TaskPage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\FieldContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/FieldContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\FieldInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/FieldInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\FieldList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/FieldList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\FieldPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/FieldPage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\SampleContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/SampleContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\SampleInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/SampleInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\SampleList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/SampleList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\SampleOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/SampleOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\SamplePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/SamplePage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\TaskActionsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/TaskActionsContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\TaskActionsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/TaskActionsInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\TaskActionsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/TaskActionsList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\TaskActionsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/TaskActionsOptions.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\TaskActionsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/TaskActionsPage.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\TaskStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/TaskStatisticsContext.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\TaskStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/TaskStatisticsInstance.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\TaskStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/TaskStatisticsList.php',
    'Twilio\\Rest\\Preview\\Understand\\Assistant\\Task\\TaskStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Understand/Assistant/Task/TaskStatisticsPage.php',
    'Twilio\\Rest\\Preview\\Wireless' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless.php',
    'Twilio\\Rest\\Preview\\Wireless\\CommandContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/CommandContext.php',
    'Twilio\\Rest\\Preview\\Wireless\\CommandInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/CommandInstance.php',
    'Twilio\\Rest\\Preview\\Wireless\\CommandList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/CommandList.php',
    'Twilio\\Rest\\Preview\\Wireless\\CommandOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/CommandOptions.php',
    'Twilio\\Rest\\Preview\\Wireless\\CommandPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/CommandPage.php',
    'Twilio\\Rest\\Preview\\Wireless\\RatePlanContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/RatePlanContext.php',
    'Twilio\\Rest\\Preview\\Wireless\\RatePlanInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/RatePlanInstance.php',
    'Twilio\\Rest\\Preview\\Wireless\\RatePlanList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/RatePlanList.php',
    'Twilio\\Rest\\Preview\\Wireless\\RatePlanOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/RatePlanOptions.php',
    'Twilio\\Rest\\Preview\\Wireless\\RatePlanPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/RatePlanPage.php',
    'Twilio\\Rest\\Preview\\Wireless\\SimContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/SimContext.php',
    'Twilio\\Rest\\Preview\\Wireless\\SimInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/SimInstance.php',
    'Twilio\\Rest\\Preview\\Wireless\\SimList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/SimList.php',
    'Twilio\\Rest\\Preview\\Wireless\\SimOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/SimOptions.php',
    'Twilio\\Rest\\Preview\\Wireless\\SimPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/SimPage.php',
    'Twilio\\Rest\\Preview\\Wireless\\Sim\\UsageContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/Sim/UsageContext.php',
    'Twilio\\Rest\\Preview\\Wireless\\Sim\\UsageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/Sim/UsageInstance.php',
    'Twilio\\Rest\\Preview\\Wireless\\Sim\\UsageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/Sim/UsageList.php',
    'Twilio\\Rest\\Preview\\Wireless\\Sim\\UsageOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/Sim/UsageOptions.php',
    'Twilio\\Rest\\Preview\\Wireless\\Sim\\UsagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Preview/Wireless/Sim/UsagePage.php',
    'Twilio\\Rest\\Pricing' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing.php',
    'Twilio\\Rest\\Pricing\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1.php',
    'Twilio\\Rest\\Pricing\\V1\\MessagingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/MessagingInstance.php',
    'Twilio\\Rest\\Pricing\\V1\\MessagingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/MessagingList.php',
    'Twilio\\Rest\\Pricing\\V1\\MessagingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/MessagingPage.php',
    'Twilio\\Rest\\Pricing\\V1\\Messaging\\CountryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Messaging/CountryContext.php',
    'Twilio\\Rest\\Pricing\\V1\\Messaging\\CountryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Messaging/CountryInstance.php',
    'Twilio\\Rest\\Pricing\\V1\\Messaging\\CountryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Messaging/CountryList.php',
    'Twilio\\Rest\\Pricing\\V1\\Messaging\\CountryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Messaging/CountryPage.php',
    'Twilio\\Rest\\Pricing\\V1\\PhoneNumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/PhoneNumberInstance.php',
    'Twilio\\Rest\\Pricing\\V1\\PhoneNumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/PhoneNumberList.php',
    'Twilio\\Rest\\Pricing\\V1\\PhoneNumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/PhoneNumberPage.php',
    'Twilio\\Rest\\Pricing\\V1\\PhoneNumber\\CountryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/PhoneNumber/CountryContext.php',
    'Twilio\\Rest\\Pricing\\V1\\PhoneNumber\\CountryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/PhoneNumber/CountryInstance.php',
    'Twilio\\Rest\\Pricing\\V1\\PhoneNumber\\CountryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/PhoneNumber/CountryList.php',
    'Twilio\\Rest\\Pricing\\V1\\PhoneNumber\\CountryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/PhoneNumber/CountryPage.php',
    'Twilio\\Rest\\Pricing\\V1\\VoiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/VoiceInstance.php',
    'Twilio\\Rest\\Pricing\\V1\\VoiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/VoiceList.php',
    'Twilio\\Rest\\Pricing\\V1\\VoicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/VoicePage.php',
    'Twilio\\Rest\\Pricing\\V1\\Voice\\CountryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Voice/CountryContext.php',
    'Twilio\\Rest\\Pricing\\V1\\Voice\\CountryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Voice/CountryInstance.php',
    'Twilio\\Rest\\Pricing\\V1\\Voice\\CountryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Voice/CountryList.php',
    'Twilio\\Rest\\Pricing\\V1\\Voice\\CountryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Voice/CountryPage.php',
    'Twilio\\Rest\\Pricing\\V1\\Voice\\NumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Voice/NumberContext.php',
    'Twilio\\Rest\\Pricing\\V1\\Voice\\NumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Voice/NumberInstance.php',
    'Twilio\\Rest\\Pricing\\V1\\Voice\\NumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Voice/NumberList.php',
    'Twilio\\Rest\\Pricing\\V1\\Voice\\NumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V1/Voice/NumberPage.php',
    'Twilio\\Rest\\Pricing\\V2' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2.php',
    'Twilio\\Rest\\Pricing\\V2\\CountryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/CountryContext.php',
    'Twilio\\Rest\\Pricing\\V2\\CountryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/CountryInstance.php',
    'Twilio\\Rest\\Pricing\\V2\\CountryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/CountryList.php',
    'Twilio\\Rest\\Pricing\\V2\\CountryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/CountryPage.php',
    'Twilio\\Rest\\Pricing\\V2\\NumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/NumberContext.php',
    'Twilio\\Rest\\Pricing\\V2\\NumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/NumberInstance.php',
    'Twilio\\Rest\\Pricing\\V2\\NumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/NumberList.php',
    'Twilio\\Rest\\Pricing\\V2\\NumberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/NumberOptions.php',
    'Twilio\\Rest\\Pricing\\V2\\NumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/NumberPage.php',
    'Twilio\\Rest\\Pricing\\V2\\VoiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/VoiceInstance.php',
    'Twilio\\Rest\\Pricing\\V2\\VoiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/VoiceList.php',
    'Twilio\\Rest\\Pricing\\V2\\VoicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/VoicePage.php',
    'Twilio\\Rest\\Pricing\\V2\\Voice\\CountryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/Voice/CountryContext.php',
    'Twilio\\Rest\\Pricing\\V2\\Voice\\CountryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/Voice/CountryInstance.php',
    'Twilio\\Rest\\Pricing\\V2\\Voice\\CountryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/Voice/CountryList.php',
    'Twilio\\Rest\\Pricing\\V2\\Voice\\CountryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/Voice/CountryPage.php',
    'Twilio\\Rest\\Pricing\\V2\\Voice\\NumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/Voice/NumberContext.php',
    'Twilio\\Rest\\Pricing\\V2\\Voice\\NumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/Voice/NumberInstance.php',
    'Twilio\\Rest\\Pricing\\V2\\Voice\\NumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/Voice/NumberList.php',
    'Twilio\\Rest\\Pricing\\V2\\Voice\\NumberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/Voice/NumberOptions.php',
    'Twilio\\Rest\\Pricing\\V2\\Voice\\NumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Pricing/V2/Voice/NumberPage.php',
    'Twilio\\Rest\\Proxy' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy.php',
    'Twilio\\Rest\\Proxy\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1.php',
    'Twilio\\Rest\\Proxy\\V1\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/ServiceContext.php',
    'Twilio\\Rest\\Proxy\\V1\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/ServiceInstance.php',
    'Twilio\\Rest\\Proxy\\V1\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/ServiceList.php',
    'Twilio\\Rest\\Proxy\\V1\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/ServiceOptions.php',
    'Twilio\\Rest\\Proxy\\V1\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/ServicePage.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\PhoneNumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/PhoneNumberContext.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\PhoneNumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/PhoneNumberInstance.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\PhoneNumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/PhoneNumberList.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\PhoneNumberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/PhoneNumberOptions.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\PhoneNumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/PhoneNumberPage.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\SessionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/SessionContext.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\SessionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/SessionInstance.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\SessionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/SessionList.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\SessionOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/SessionOptions.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\SessionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/SessionPage.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\InteractionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/InteractionContext.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\InteractionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/InteractionInstance.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\InteractionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/InteractionList.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\InteractionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/InteractionPage.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\ParticipantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/ParticipantContext.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\ParticipantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/ParticipantInstance.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\ParticipantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/ParticipantList.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\ParticipantOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/ParticipantOptions.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\ParticipantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/ParticipantPage.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\Participant\\MessageInteractionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/Participant/MessageInteractionContext.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\Participant\\MessageInteractionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/Participant/MessageInteractionInstance.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\Participant\\MessageInteractionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/Participant/MessageInteractionList.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\Participant\\MessageInteractionOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/Participant/MessageInteractionOptions.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\Session\\Participant\\MessageInteractionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/Session/Participant/MessageInteractionPage.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\ShortCodeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/ShortCodeContext.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\ShortCodeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/ShortCodeInstance.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\ShortCodeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/ShortCodeList.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\ShortCodeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/ShortCodeOptions.php',
    'Twilio\\Rest\\Proxy\\V1\\Service\\ShortCodePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Proxy/V1/Service/ShortCodePage.php',
    'Twilio\\Rest\\Routes' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes.php',
    'Twilio\\Rest\\Routes\\V2' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2.php',
    'Twilio\\Rest\\Routes\\V2\\PhoneNumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/PhoneNumberContext.php',
    'Twilio\\Rest\\Routes\\V2\\PhoneNumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/PhoneNumberInstance.php',
    'Twilio\\Rest\\Routes\\V2\\PhoneNumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/PhoneNumberList.php',
    'Twilio\\Rest\\Routes\\V2\\PhoneNumberOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/PhoneNumberOptions.php',
    'Twilio\\Rest\\Routes\\V2\\PhoneNumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/PhoneNumberPage.php',
    'Twilio\\Rest\\Routes\\V2\\SipDomainContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/SipDomainContext.php',
    'Twilio\\Rest\\Routes\\V2\\SipDomainInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/SipDomainInstance.php',
    'Twilio\\Rest\\Routes\\V2\\SipDomainList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/SipDomainList.php',
    'Twilio\\Rest\\Routes\\V2\\SipDomainOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/SipDomainOptions.php',
    'Twilio\\Rest\\Routes\\V2\\SipDomainPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/SipDomainPage.php',
    'Twilio\\Rest\\Routes\\V2\\TrunkContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/TrunkContext.php',
    'Twilio\\Rest\\Routes\\V2\\TrunkInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/TrunkInstance.php',
    'Twilio\\Rest\\Routes\\V2\\TrunkList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/TrunkList.php',
    'Twilio\\Rest\\Routes\\V2\\TrunkOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/TrunkOptions.php',
    'Twilio\\Rest\\Routes\\V2\\TrunkPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Routes/V2/TrunkPage.php',
    'Twilio\\Rest\\Serverless' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless.php',
    'Twilio\\Rest\\Serverless\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1.php',
    'Twilio\\Rest\\Serverless\\V1\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/ServiceContext.php',
    'Twilio\\Rest\\Serverless\\V1\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/ServiceInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/ServiceList.php',
    'Twilio\\Rest\\Serverless\\V1\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/ServiceOptions.php',
    'Twilio\\Rest\\Serverless\\V1\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/ServicePage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\AssetContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/AssetContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\AssetInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/AssetInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\AssetList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/AssetList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\AssetPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/AssetPage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Asset\\AssetVersionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Asset/AssetVersionContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Asset\\AssetVersionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Asset/AssetVersionInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Asset\\AssetVersionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Asset/AssetVersionList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Asset\\AssetVersionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Asset/AssetVersionPage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\BuildContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/BuildContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\BuildInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/BuildInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\BuildList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/BuildList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\BuildOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/BuildOptions.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\BuildPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/BuildPage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Build\\BuildStatusContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Build/BuildStatusContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Build\\BuildStatusInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Build/BuildStatusInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Build\\BuildStatusList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Build/BuildStatusList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Build\\BuildStatusPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Build/BuildStatusPage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\EnvironmentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/EnvironmentContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\EnvironmentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/EnvironmentInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\EnvironmentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/EnvironmentList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\EnvironmentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/EnvironmentOptions.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\EnvironmentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/EnvironmentPage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\DeploymentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/DeploymentContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\DeploymentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/DeploymentInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\DeploymentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/DeploymentList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\DeploymentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/DeploymentOptions.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\DeploymentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/DeploymentPage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\LogContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/LogContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\LogInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/LogInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\LogList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/LogList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\LogOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/LogOptions.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\LogPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/LogPage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\VariableContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/VariableContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\VariableInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/VariableInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\VariableList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/VariableList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\VariableOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/VariableOptions.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\Environment\\VariablePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/Environment/VariablePage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\FunctionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/FunctionContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\FunctionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/FunctionInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\FunctionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/FunctionList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\FunctionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/FunctionPage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\TwilioFunction\\FunctionVersionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/TwilioFunction/FunctionVersionContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\TwilioFunction\\FunctionVersionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/TwilioFunction/FunctionVersionInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\TwilioFunction\\FunctionVersionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/TwilioFunction/FunctionVersionList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\TwilioFunction\\FunctionVersionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/TwilioFunction/FunctionVersionPage.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\TwilioFunction\\FunctionVersion\\FunctionVersionContentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/TwilioFunction/FunctionVersion/FunctionVersionContentContext.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\TwilioFunction\\FunctionVersion\\FunctionVersionContentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/TwilioFunction/FunctionVersion/FunctionVersionContentInstance.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\TwilioFunction\\FunctionVersion\\FunctionVersionContentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/TwilioFunction/FunctionVersion/FunctionVersionContentList.php',
    'Twilio\\Rest\\Serverless\\V1\\Service\\TwilioFunction\\FunctionVersion\\FunctionVersionContentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Serverless/V1/Service/TwilioFunction/FunctionVersion/FunctionVersionContentPage.php',
    'Twilio\\Rest\\Studio' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio.php',
    'Twilio\\Rest\\Studio\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1.php',
    'Twilio\\Rest\\Studio\\V1\\FlowContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/FlowContext.php',
    'Twilio\\Rest\\Studio\\V1\\FlowInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/FlowInstance.php',
    'Twilio\\Rest\\Studio\\V1\\FlowList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/FlowList.php',
    'Twilio\\Rest\\Studio\\V1\\FlowPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/FlowPage.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\EngagementContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/EngagementContext.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\EngagementInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/EngagementInstance.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\EngagementList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/EngagementList.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\EngagementOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/EngagementOptions.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\EngagementPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/EngagementPage.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\EngagementContextContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/EngagementContextContext.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\EngagementContextInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/EngagementContextInstance.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\EngagementContextList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/EngagementContextList.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\EngagementContextPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/EngagementContextPage.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\StepContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/StepContext.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\StepInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/StepInstance.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\StepList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/StepList.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\StepPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/StepPage.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\Step\\StepContextContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/Step/StepContextContext.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\Step\\StepContextInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/Step/StepContextInstance.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\Step\\StepContextList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/Step/StepContextList.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Engagement\\Step\\StepContextPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Engagement/Step/StepContextPage.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\ExecutionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/ExecutionContext.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\ExecutionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/ExecutionInstance.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\ExecutionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/ExecutionList.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\ExecutionOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/ExecutionOptions.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\ExecutionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/ExecutionPage.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionContextContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionContextContext.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionContextInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionContextInstance.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionContextList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionContextList.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionContextPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionContextPage.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionStepContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionStepContext.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionStepInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionStepInstance.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionStepList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionStepList.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionStepPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionStepPage.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionStep\\ExecutionStepContextContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionStep/ExecutionStepContextContext.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionStep\\ExecutionStepContextInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionStep/ExecutionStepContextInstance.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionStep\\ExecutionStepContextList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionStep/ExecutionStepContextList.php',
    'Twilio\\Rest\\Studio\\V1\\Flow\\Execution\\ExecutionStep\\ExecutionStepContextPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V1/Flow/Execution/ExecutionStep/ExecutionStepContextPage.php',
    'Twilio\\Rest\\Studio\\V2' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2.php',
    'Twilio\\Rest\\Studio\\V2\\FlowContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/FlowContext.php',
    'Twilio\\Rest\\Studio\\V2\\FlowInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/FlowInstance.php',
    'Twilio\\Rest\\Studio\\V2\\FlowList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/FlowList.php',
    'Twilio\\Rest\\Studio\\V2\\FlowOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/FlowOptions.php',
    'Twilio\\Rest\\Studio\\V2\\FlowPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/FlowPage.php',
    'Twilio\\Rest\\Studio\\V2\\FlowValidateInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/FlowValidateInstance.php',
    'Twilio\\Rest\\Studio\\V2\\FlowValidateList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/FlowValidateList.php',
    'Twilio\\Rest\\Studio\\V2\\FlowValidateOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/FlowValidateOptions.php',
    'Twilio\\Rest\\Studio\\V2\\FlowValidatePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/FlowValidatePage.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\ExecutionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/ExecutionContext.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\ExecutionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/ExecutionInstance.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\ExecutionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/ExecutionList.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\ExecutionOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/ExecutionOptions.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\ExecutionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/ExecutionPage.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionContextContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionContextContext.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionContextInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionContextInstance.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionContextList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionContextList.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionContextPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionContextPage.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionStepContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionStepContext.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionStepInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionStepInstance.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionStepList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionStepList.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionStepPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionStepPage.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionStep\\ExecutionStepContextContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionStep/ExecutionStepContextContext.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionStep\\ExecutionStepContextInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionStep/ExecutionStepContextInstance.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionStep\\ExecutionStepContextList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionStep/ExecutionStepContextList.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\Execution\\ExecutionStep\\ExecutionStepContextPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/Execution/ExecutionStep/ExecutionStepContextPage.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\FlowRevisionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/FlowRevisionContext.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\FlowRevisionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/FlowRevisionInstance.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\FlowRevisionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/FlowRevisionList.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\FlowRevisionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/FlowRevisionPage.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\FlowTestUserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/FlowTestUserContext.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\FlowTestUserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/FlowTestUserInstance.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\FlowTestUserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/FlowTestUserList.php',
    'Twilio\\Rest\\Studio\\V2\\Flow\\FlowTestUserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Studio/V2/Flow/FlowTestUserPage.php',
    'Twilio\\Rest\\Supersim' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim.php',
    'Twilio\\Rest\\Supersim\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1.php',
    'Twilio\\Rest\\Supersim\\V1\\EsimProfileContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/EsimProfileContext.php',
    'Twilio\\Rest\\Supersim\\V1\\EsimProfileInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/EsimProfileInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\EsimProfileList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/EsimProfileList.php',
    'Twilio\\Rest\\Supersim\\V1\\EsimProfileOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/EsimProfileOptions.php',
    'Twilio\\Rest\\Supersim\\V1\\EsimProfilePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/EsimProfilePage.php',
    'Twilio\\Rest\\Supersim\\V1\\FleetContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/FleetContext.php',
    'Twilio\\Rest\\Supersim\\V1\\FleetInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/FleetInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\FleetList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/FleetList.php',
    'Twilio\\Rest\\Supersim\\V1\\FleetOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/FleetOptions.php',
    'Twilio\\Rest\\Supersim\\V1\\FleetPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/FleetPage.php',
    'Twilio\\Rest\\Supersim\\V1\\IpCommandContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/IpCommandContext.php',
    'Twilio\\Rest\\Supersim\\V1\\IpCommandInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/IpCommandInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\IpCommandList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/IpCommandList.php',
    'Twilio\\Rest\\Supersim\\V1\\IpCommandOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/IpCommandOptions.php',
    'Twilio\\Rest\\Supersim\\V1\\IpCommandPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/IpCommandPage.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkAccessProfileContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkAccessProfileContext.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkAccessProfileInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkAccessProfileInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkAccessProfileList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkAccessProfileList.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkAccessProfileOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkAccessProfileOptions.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkAccessProfilePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkAccessProfilePage.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkAccessProfile\\NetworkAccessProfileNetworkContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkAccessProfile/NetworkAccessProfileNetworkContext.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkAccessProfile\\NetworkAccessProfileNetworkInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkAccessProfile/NetworkAccessProfileNetworkInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkAccessProfile\\NetworkAccessProfileNetworkList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkAccessProfile/NetworkAccessProfileNetworkList.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkAccessProfile\\NetworkAccessProfileNetworkPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkAccessProfile/NetworkAccessProfileNetworkPage.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkContext.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkList.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkOptions.php',
    'Twilio\\Rest\\Supersim\\V1\\NetworkPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/NetworkPage.php',
    'Twilio\\Rest\\Supersim\\V1\\SettingsUpdateInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SettingsUpdateInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\SettingsUpdateList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SettingsUpdateList.php',
    'Twilio\\Rest\\Supersim\\V1\\SettingsUpdateOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SettingsUpdateOptions.php',
    'Twilio\\Rest\\Supersim\\V1\\SettingsUpdatePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SettingsUpdatePage.php',
    'Twilio\\Rest\\Supersim\\V1\\SimContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SimContext.php',
    'Twilio\\Rest\\Supersim\\V1\\SimInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SimInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\SimList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SimList.php',
    'Twilio\\Rest\\Supersim\\V1\\SimOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SimOptions.php',
    'Twilio\\Rest\\Supersim\\V1\\SimPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SimPage.php',
    'Twilio\\Rest\\Supersim\\V1\\Sim\\BillingPeriodInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/Sim/BillingPeriodInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\Sim\\BillingPeriodList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/Sim/BillingPeriodList.php',
    'Twilio\\Rest\\Supersim\\V1\\Sim\\BillingPeriodPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/Sim/BillingPeriodPage.php',
    'Twilio\\Rest\\Supersim\\V1\\Sim\\SimIpAddressInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/Sim/SimIpAddressInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\Sim\\SimIpAddressList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/Sim/SimIpAddressList.php',
    'Twilio\\Rest\\Supersim\\V1\\Sim\\SimIpAddressPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/Sim/SimIpAddressPage.php',
    'Twilio\\Rest\\Supersim\\V1\\SmsCommandContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SmsCommandContext.php',
    'Twilio\\Rest\\Supersim\\V1\\SmsCommandInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SmsCommandInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\SmsCommandList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SmsCommandList.php',
    'Twilio\\Rest\\Supersim\\V1\\SmsCommandOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SmsCommandOptions.php',
    'Twilio\\Rest\\Supersim\\V1\\SmsCommandPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/SmsCommandPage.php',
    'Twilio\\Rest\\Supersim\\V1\\UsageRecordInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/UsageRecordInstance.php',
    'Twilio\\Rest\\Supersim\\V1\\UsageRecordList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/UsageRecordList.php',
    'Twilio\\Rest\\Supersim\\V1\\UsageRecordOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/UsageRecordOptions.php',
    'Twilio\\Rest\\Supersim\\V1\\UsageRecordPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Supersim/V1/UsageRecordPage.php',
    'Twilio\\Rest\\Sync' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync.php',
    'Twilio\\Rest\\Sync\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1.php',
    'Twilio\\Rest\\Sync\\V1\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/ServiceContext.php',
    'Twilio\\Rest\\Sync\\V1\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/ServiceInstance.php',
    'Twilio\\Rest\\Sync\\V1\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/ServiceList.php',
    'Twilio\\Rest\\Sync\\V1\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/ServiceOptions.php',
    'Twilio\\Rest\\Sync\\V1\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/ServicePage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\DocumentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/DocumentContext.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\DocumentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/DocumentInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\DocumentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/DocumentList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\DocumentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/DocumentOptions.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\DocumentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/DocumentPage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\Document\\DocumentPermissionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/Document/DocumentPermissionContext.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\Document\\DocumentPermissionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/Document/DocumentPermissionInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\Document\\DocumentPermissionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/Document/DocumentPermissionList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\Document\\DocumentPermissionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/Document/DocumentPermissionPage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncListContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncListContext.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncListInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncListInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncListList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncListList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncListOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncListOptions.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncListPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncListPage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncList\\SyncListItemContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncList/SyncListItemContext.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncList\\SyncListItemInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncList/SyncListItemInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncList\\SyncListItemList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncList/SyncListItemList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncList\\SyncListItemOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncList/SyncListItemOptions.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncList\\SyncListItemPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncList/SyncListItemPage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncList\\SyncListPermissionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncList/SyncListPermissionContext.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncList\\SyncListPermissionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncList/SyncListPermissionInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncList\\SyncListPermissionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncList/SyncListPermissionList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncList\\SyncListPermissionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncList/SyncListPermissionPage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMapContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMapContext.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMapInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMapInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMapList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMapList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMapOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMapOptions.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMapPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMapPage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMap\\SyncMapItemContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMap/SyncMapItemContext.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMap\\SyncMapItemInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMap/SyncMapItemInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMap\\SyncMapItemList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMap/SyncMapItemList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMap\\SyncMapItemOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMap/SyncMapItemOptions.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMap\\SyncMapItemPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMap/SyncMapItemPage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMap\\SyncMapPermissionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMap/SyncMapPermissionContext.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMap\\SyncMapPermissionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMap/SyncMapPermissionInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMap\\SyncMapPermissionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMap/SyncMapPermissionList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncMap\\SyncMapPermissionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncMap/SyncMapPermissionPage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncStreamContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncStreamContext.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncStreamInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncStreamInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncStreamList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncStreamList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncStreamOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncStreamOptions.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncStreamPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncStreamPage.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncStream\\StreamMessageInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncStream/StreamMessageInstance.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncStream\\StreamMessageList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncStream/StreamMessageList.php',
    'Twilio\\Rest\\Sync\\V1\\Service\\SyncStream\\StreamMessagePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Sync/V1/Service/SyncStream/StreamMessagePage.php',
    'Twilio\\Rest\\Taskrouter' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter.php',
    'Twilio\\Rest\\Taskrouter\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1.php',
    'Twilio\\Rest\\Taskrouter\\V1\\WorkspaceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/WorkspaceContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\WorkspaceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/WorkspaceInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\WorkspaceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/WorkspaceList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\WorkspaceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/WorkspaceOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\WorkspacePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/WorkspacePage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\ActivityContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/ActivityContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\ActivityInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/ActivityInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\ActivityList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/ActivityList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\ActivityOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/ActivityOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\ActivityPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/ActivityPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\EventContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/EventContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\EventInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/EventInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\EventList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/EventList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\EventOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/EventOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\EventPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/EventPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskChannelContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskChannelInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskChannelList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskChannelOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskChannelPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueueContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueueContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueueInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueueInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueueList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueueList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueueOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueueOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueuePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueuePage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueCumulativeStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueCumulativeStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueCumulativeStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueCumulativeStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueCumulativeStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueCumulativeStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueCumulativeStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueCumulativeStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueCumulativeStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueCumulativeStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueRealTimeStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueRealTimeStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueRealTimeStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueRealTimeStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueRealTimeStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueRealTimeStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueRealTimeStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueRealTimeStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueRealTimeStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueRealTimeStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueueStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueueStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueuesStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueuesStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueuesStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueuesStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueuesStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueuesStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\TaskQueue\\TaskQueuesStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/TaskQueue/TaskQueuesStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Task\\ReservationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Task/ReservationContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Task\\ReservationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Task/ReservationInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Task\\ReservationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Task/ReservationList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Task\\ReservationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Task/ReservationOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Task\\ReservationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Task/ReservationPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkerContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkerContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkerInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkerInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkerList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkerList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkerOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkerOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkerPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkerPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\ReservationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/ReservationContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\ReservationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/ReservationInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\ReservationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/ReservationList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\ReservationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/ReservationOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\ReservationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/ReservationPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerChannelContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerChannelContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerChannelInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerChannelInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerChannelList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerChannelList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerChannelOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerChannelOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerChannelPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerChannelPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkerStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkerStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersCumulativeStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersCumulativeStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersCumulativeStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersCumulativeStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersCumulativeStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersCumulativeStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersCumulativeStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersCumulativeStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersCumulativeStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersCumulativeStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersRealTimeStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersRealTimeStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersRealTimeStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersRealTimeStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersRealTimeStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersRealTimeStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersRealTimeStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersRealTimeStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersRealTimeStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersRealTimeStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Worker\\WorkersStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Worker/WorkersStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkflowContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkflowContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkflowInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkflowInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkflowList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkflowList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkflowOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkflowOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkflowPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkflowPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowCumulativeStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowCumulativeStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowCumulativeStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowCumulativeStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowCumulativeStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowCumulativeStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowCumulativeStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowCumulativeStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowCumulativeStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowCumulativeStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowRealTimeStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowRealTimeStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowRealTimeStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowRealTimeStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowRealTimeStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowRealTimeStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowRealTimeStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowRealTimeStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowRealTimeStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowRealTimeStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\Workflow\\WorkflowStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/Workflow/WorkflowStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceCumulativeStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceCumulativeStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceCumulativeStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceCumulativeStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceCumulativeStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceCumulativeStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceCumulativeStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceCumulativeStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceCumulativeStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceCumulativeStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceRealTimeStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceRealTimeStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceRealTimeStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceRealTimeStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceRealTimeStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceRealTimeStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceRealTimeStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceRealTimeStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceRealTimeStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceRealTimeStatisticsPage.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceStatisticsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceStatisticsContext.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceStatisticsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceStatisticsInstance.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceStatisticsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceStatisticsList.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceStatisticsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceStatisticsOptions.php',
    'Twilio\\Rest\\Taskrouter\\V1\\Workspace\\WorkspaceStatisticsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Taskrouter/V1/Workspace/WorkspaceStatisticsPage.php',
    'Twilio\\Rest\\Trunking' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking.php',
    'Twilio\\Rest\\Trunking\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1.php',
    'Twilio\\Rest\\Trunking\\V1\\TrunkContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/TrunkContext.php',
    'Twilio\\Rest\\Trunking\\V1\\TrunkInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/TrunkInstance.php',
    'Twilio\\Rest\\Trunking\\V1\\TrunkList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/TrunkList.php',
    'Twilio\\Rest\\Trunking\\V1\\TrunkOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/TrunkOptions.php',
    'Twilio\\Rest\\Trunking\\V1\\TrunkPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/TrunkPage.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\CredentialListContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/CredentialListContext.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\CredentialListInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/CredentialListInstance.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\CredentialListList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/CredentialListList.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\CredentialListPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/CredentialListPage.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\IpAccessControlListContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/IpAccessControlListContext.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\IpAccessControlListInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/IpAccessControlListInstance.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\IpAccessControlListList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/IpAccessControlListList.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\IpAccessControlListPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/IpAccessControlListPage.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\OriginationUrlContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/OriginationUrlContext.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\OriginationUrlInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/OriginationUrlInstance.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\OriginationUrlList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/OriginationUrlList.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\OriginationUrlOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/OriginationUrlOptions.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\OriginationUrlPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/OriginationUrlPage.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\PhoneNumberContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/PhoneNumberContext.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\PhoneNumberInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/PhoneNumberInstance.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\PhoneNumberList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/PhoneNumberList.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\PhoneNumberPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/PhoneNumberPage.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\RecordingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/RecordingContext.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\RecordingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/RecordingInstance.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\RecordingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/RecordingList.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\RecordingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/RecordingOptions.php',
    'Twilio\\Rest\\Trunking\\V1\\Trunk\\RecordingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trunking/V1/Trunk/RecordingPage.php',
    'Twilio\\Rest\\Trusthub' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub.php',
    'Twilio\\Rest\\Trusthub\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfilesContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfilesContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfilesInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfilesInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfilesList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfilesList.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfilesOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfilesOptions.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfilesPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfilesPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesChannelEndpointAssignmentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesChannelEndpointAssignmentContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesChannelEndpointAssignmentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesChannelEndpointAssignmentInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesChannelEndpointAssignmentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesChannelEndpointAssignmentList.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesChannelEndpointAssignmentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesChannelEndpointAssignmentOptions.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesChannelEndpointAssignmentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesChannelEndpointAssignmentPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesEntityAssignmentsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesEntityAssignmentsContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesEntityAssignmentsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesEntityAssignmentsInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesEntityAssignmentsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesEntityAssignmentsList.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesEntityAssignmentsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesEntityAssignmentsPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesEvaluationsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesEvaluationsContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesEvaluationsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesEvaluationsInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesEvaluationsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesEvaluationsList.php',
    'Twilio\\Rest\\Trusthub\\V1\\CustomerProfiles\\CustomerProfilesEvaluationsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/CustomerProfiles/CustomerProfilesEvaluationsPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\EndUserContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/EndUserContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\EndUserInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/EndUserInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\EndUserList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/EndUserList.php',
    'Twilio\\Rest\\Trusthub\\V1\\EndUserOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/EndUserOptions.php',
    'Twilio\\Rest\\Trusthub\\V1\\EndUserPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/EndUserPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\EndUserTypeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/EndUserTypeContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\EndUserTypeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/EndUserTypeInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\EndUserTypeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/EndUserTypeList.php',
    'Twilio\\Rest\\Trusthub\\V1\\EndUserTypePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/EndUserTypePage.php',
    'Twilio\\Rest\\Trusthub\\V1\\PoliciesContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/PoliciesContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\PoliciesInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/PoliciesInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\PoliciesList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/PoliciesList.php',
    'Twilio\\Rest\\Trusthub\\V1\\PoliciesPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/PoliciesPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\SupportingDocumentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/SupportingDocumentContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\SupportingDocumentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/SupportingDocumentInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\SupportingDocumentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/SupportingDocumentList.php',
    'Twilio\\Rest\\Trusthub\\V1\\SupportingDocumentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/SupportingDocumentOptions.php',
    'Twilio\\Rest\\Trusthub\\V1\\SupportingDocumentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/SupportingDocumentPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\SupportingDocumentTypeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/SupportingDocumentTypeContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\SupportingDocumentTypeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/SupportingDocumentTypeInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\SupportingDocumentTypeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/SupportingDocumentTypeList.php',
    'Twilio\\Rest\\Trusthub\\V1\\SupportingDocumentTypePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/SupportingDocumentTypePage.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProductsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProductsContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProductsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProductsInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProductsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProductsList.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProductsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProductsOptions.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProductsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProductsPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsChannelEndpointAssignmentContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsChannelEndpointAssignmentContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsChannelEndpointAssignmentInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsChannelEndpointAssignmentInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsChannelEndpointAssignmentList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsChannelEndpointAssignmentList.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsChannelEndpointAssignmentOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsChannelEndpointAssignmentOptions.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsChannelEndpointAssignmentPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsChannelEndpointAssignmentPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsEntityAssignmentsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsEntityAssignmentsContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsEntityAssignmentsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsEntityAssignmentsInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsEntityAssignmentsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsEntityAssignmentsList.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsEntityAssignmentsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsEntityAssignmentsPage.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsEvaluationsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsEvaluationsContext.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsEvaluationsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsEvaluationsInstance.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsEvaluationsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsEvaluationsList.php',
    'Twilio\\Rest\\Trusthub\\V1\\TrustProducts\\TrustProductsEvaluationsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Trusthub/V1/TrustProducts/TrustProductsEvaluationsPage.php',
    'Twilio\\Rest\\Verify' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify.php',
    'Twilio\\Rest\\Verify\\V2' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2.php',
    'Twilio\\Rest\\Verify\\V2\\FormContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/FormContext.php',
    'Twilio\\Rest\\Verify\\V2\\FormInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/FormInstance.php',
    'Twilio\\Rest\\Verify\\V2\\FormList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/FormList.php',
    'Twilio\\Rest\\Verify\\V2\\FormPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/FormPage.php',
    'Twilio\\Rest\\Verify\\V2\\SafelistContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/SafelistContext.php',
    'Twilio\\Rest\\Verify\\V2\\SafelistInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/SafelistInstance.php',
    'Twilio\\Rest\\Verify\\V2\\SafelistList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/SafelistList.php',
    'Twilio\\Rest\\Verify\\V2\\SafelistPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/SafelistPage.php',
    'Twilio\\Rest\\Verify\\V2\\ServiceContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/ServiceContext.php',
    'Twilio\\Rest\\Verify\\V2\\ServiceInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/ServiceInstance.php',
    'Twilio\\Rest\\Verify\\V2\\ServiceList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/ServiceList.php',
    'Twilio\\Rest\\Verify\\V2\\ServiceOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/ServiceOptions.php',
    'Twilio\\Rest\\Verify\\V2\\ServicePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/ServicePage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\AccessTokenContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/AccessTokenContext.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\AccessTokenInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/AccessTokenInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\AccessTokenList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/AccessTokenList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\AccessTokenOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/AccessTokenOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\AccessTokenPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/AccessTokenPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\EntityContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/EntityContext.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\EntityInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/EntityInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\EntityList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/EntityList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\EntityPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/EntityPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\ChallengeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/ChallengeContext.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\ChallengeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/ChallengeInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\ChallengeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/ChallengeList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\ChallengeOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/ChallengeOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\ChallengePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/ChallengePage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\Challenge\\NotificationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/Challenge/NotificationInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\Challenge\\NotificationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/Challenge/NotificationList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\Challenge\\NotificationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/Challenge/NotificationOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\Challenge\\NotificationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/Challenge/NotificationPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\FactorContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/FactorContext.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\FactorInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/FactorInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\FactorList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/FactorList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\FactorOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/FactorOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\FactorPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/FactorPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\NewFactorInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/NewFactorInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\NewFactorList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/NewFactorList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\NewFactorOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/NewFactorOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\Entity\\NewFactorPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/Entity/NewFactorPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\MessagingConfigurationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/MessagingConfigurationContext.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\MessagingConfigurationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/MessagingConfigurationInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\MessagingConfigurationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/MessagingConfigurationList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\MessagingConfigurationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/MessagingConfigurationPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimitContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimitContext.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimitInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimitInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimitList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimitList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimitOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimitOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimitPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimitPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimit\\BucketContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimit/BucketContext.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimit\\BucketInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimit/BucketInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimit\\BucketList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimit/BucketList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimit\\BucketOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimit/BucketOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\RateLimit\\BucketPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/RateLimit/BucketPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\VerificationCheckInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/VerificationCheckInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\VerificationCheckList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/VerificationCheckList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\VerificationCheckOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/VerificationCheckOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\VerificationCheckPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/VerificationCheckPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\VerificationContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/VerificationContext.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\VerificationInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/VerificationInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\VerificationList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/VerificationList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\VerificationOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/VerificationOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\VerificationPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/VerificationPage.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\WebhookContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/WebhookContext.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\WebhookInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/WebhookInstance.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\WebhookList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/WebhookList.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\WebhookOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/WebhookOptions.php',
    'Twilio\\Rest\\Verify\\V2\\Service\\WebhookPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/Service/WebhookPage.php',
    'Twilio\\Rest\\Verify\\V2\\TemplateInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/TemplateInstance.php',
    'Twilio\\Rest\\Verify\\V2\\TemplateList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/TemplateList.php',
    'Twilio\\Rest\\Verify\\V2\\TemplateOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/TemplateOptions.php',
    'Twilio\\Rest\\Verify\\V2\\TemplatePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/TemplatePage.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptContext.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptInstance.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptList.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptOptions.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptPage.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptsSummaryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptsSummaryContext.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptsSummaryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptsSummaryInstance.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptsSummaryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptsSummaryList.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptsSummaryOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptsSummaryOptions.php',
    'Twilio\\Rest\\Verify\\V2\\VerificationAttemptsSummaryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Verify/V2/VerificationAttemptsSummaryPage.php',
    'Twilio\\Rest\\Video' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video.php',
    'Twilio\\Rest\\Video\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1.php',
    'Twilio\\Rest\\Video\\V1\\CompositionContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionContext.php',
    'Twilio\\Rest\\Video\\V1\\CompositionHookContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionHookContext.php',
    'Twilio\\Rest\\Video\\V1\\CompositionHookInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionHookInstance.php',
    'Twilio\\Rest\\Video\\V1\\CompositionHookList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionHookList.php',
    'Twilio\\Rest\\Video\\V1\\CompositionHookOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionHookOptions.php',
    'Twilio\\Rest\\Video\\V1\\CompositionHookPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionHookPage.php',
    'Twilio\\Rest\\Video\\V1\\CompositionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionInstance.php',
    'Twilio\\Rest\\Video\\V1\\CompositionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionList.php',
    'Twilio\\Rest\\Video\\V1\\CompositionOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionOptions.php',
    'Twilio\\Rest\\Video\\V1\\CompositionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionPage.php',
    'Twilio\\Rest\\Video\\V1\\CompositionSettingsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionSettingsContext.php',
    'Twilio\\Rest\\Video\\V1\\CompositionSettingsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionSettingsInstance.php',
    'Twilio\\Rest\\Video\\V1\\CompositionSettingsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionSettingsList.php',
    'Twilio\\Rest\\Video\\V1\\CompositionSettingsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionSettingsOptions.php',
    'Twilio\\Rest\\Video\\V1\\CompositionSettingsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/CompositionSettingsPage.php',
    'Twilio\\Rest\\Video\\V1\\RecordingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingContext.php',
    'Twilio\\Rest\\Video\\V1\\RecordingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingInstance.php',
    'Twilio\\Rest\\Video\\V1\\RecordingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingList.php',
    'Twilio\\Rest\\Video\\V1\\RecordingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingOptions.php',
    'Twilio\\Rest\\Video\\V1\\RecordingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingPage.php',
    'Twilio\\Rest\\Video\\V1\\RecordingSettingsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingSettingsContext.php',
    'Twilio\\Rest\\Video\\V1\\RecordingSettingsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingSettingsInstance.php',
    'Twilio\\Rest\\Video\\V1\\RecordingSettingsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingSettingsList.php',
    'Twilio\\Rest\\Video\\V1\\RecordingSettingsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingSettingsOptions.php',
    'Twilio\\Rest\\Video\\V1\\RecordingSettingsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RecordingSettingsPage.php',
    'Twilio\\Rest\\Video\\V1\\RoomContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RoomContext.php',
    'Twilio\\Rest\\Video\\V1\\RoomInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RoomInstance.php',
    'Twilio\\Rest\\Video\\V1\\RoomList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RoomList.php',
    'Twilio\\Rest\\Video\\V1\\RoomOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RoomOptions.php',
    'Twilio\\Rest\\Video\\V1\\RoomPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/RoomPage.php',
    'Twilio\\Rest\\Video\\V1\\Room\\ParticipantContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/ParticipantContext.php',
    'Twilio\\Rest\\Video\\V1\\Room\\ParticipantInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/ParticipantInstance.php',
    'Twilio\\Rest\\Video\\V1\\Room\\ParticipantList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/ParticipantList.php',
    'Twilio\\Rest\\Video\\V1\\Room\\ParticipantOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/ParticipantOptions.php',
    'Twilio\\Rest\\Video\\V1\\Room\\ParticipantPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/ParticipantPage.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\AnonymizeContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/AnonymizeContext.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\AnonymizeInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/AnonymizeInstance.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\AnonymizeList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/AnonymizeList.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\AnonymizePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/AnonymizePage.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\PublishedTrackContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/PublishedTrackContext.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\PublishedTrackInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/PublishedTrackInstance.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\PublishedTrackList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/PublishedTrackList.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\PublishedTrackPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/PublishedTrackPage.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\SubscribeRulesInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/SubscribeRulesInstance.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\SubscribeRulesList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/SubscribeRulesList.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\SubscribeRulesOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/SubscribeRulesOptions.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\SubscribeRulesPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/SubscribeRulesPage.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\SubscribedTrackContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/SubscribedTrackContext.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\SubscribedTrackInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/SubscribedTrackInstance.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\SubscribedTrackList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/SubscribedTrackList.php',
    'Twilio\\Rest\\Video\\V1\\Room\\Participant\\SubscribedTrackPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/Participant/SubscribedTrackPage.php',
    'Twilio\\Rest\\Video\\V1\\Room\\RecordingRulesInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/RecordingRulesInstance.php',
    'Twilio\\Rest\\Video\\V1\\Room\\RecordingRulesList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/RecordingRulesList.php',
    'Twilio\\Rest\\Video\\V1\\Room\\RecordingRulesOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/RecordingRulesOptions.php',
    'Twilio\\Rest\\Video\\V1\\Room\\RecordingRulesPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/RecordingRulesPage.php',
    'Twilio\\Rest\\Video\\V1\\Room\\RoomRecordingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/RoomRecordingContext.php',
    'Twilio\\Rest\\Video\\V1\\Room\\RoomRecordingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/RoomRecordingInstance.php',
    'Twilio\\Rest\\Video\\V1\\Room\\RoomRecordingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/RoomRecordingList.php',
    'Twilio\\Rest\\Video\\V1\\Room\\RoomRecordingOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/RoomRecordingOptions.php',
    'Twilio\\Rest\\Video\\V1\\Room\\RoomRecordingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Video/V1/Room/RoomRecordingPage.php',
    'Twilio\\Rest\\Voice' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice.php',
    'Twilio\\Rest\\Voice\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1.php',
    'Twilio\\Rest\\Voice\\V1\\ArchivedCallContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ArchivedCallContext.php',
    'Twilio\\Rest\\Voice\\V1\\ArchivedCallInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ArchivedCallInstance.php',
    'Twilio\\Rest\\Voice\\V1\\ArchivedCallList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ArchivedCallList.php',
    'Twilio\\Rest\\Voice\\V1\\ArchivedCallPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ArchivedCallPage.php',
    'Twilio\\Rest\\Voice\\V1\\ByocTrunkContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ByocTrunkContext.php',
    'Twilio\\Rest\\Voice\\V1\\ByocTrunkInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ByocTrunkInstance.php',
    'Twilio\\Rest\\Voice\\V1\\ByocTrunkList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ByocTrunkList.php',
    'Twilio\\Rest\\Voice\\V1\\ByocTrunkOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ByocTrunkOptions.php',
    'Twilio\\Rest\\Voice\\V1\\ByocTrunkPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ByocTrunkPage.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicyContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicyContext.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicyInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicyInstance.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicyList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicyList.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicyOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicyOptions.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicyPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicyPage.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicy\\ConnectionPolicyTargetContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicy/ConnectionPolicyTargetContext.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicy\\ConnectionPolicyTargetInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicy/ConnectionPolicyTargetInstance.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicy\\ConnectionPolicyTargetList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicy/ConnectionPolicyTargetList.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicy\\ConnectionPolicyTargetOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicy/ConnectionPolicyTargetOptions.php',
    'Twilio\\Rest\\Voice\\V1\\ConnectionPolicy\\ConnectionPolicyTargetPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/ConnectionPolicy/ConnectionPolicyTargetPage.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissionsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissionsInstance.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissionsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissionsList.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissionsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissionsPage.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\BulkCountryUpdateInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/BulkCountryUpdateInstance.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\BulkCountryUpdateList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/BulkCountryUpdateList.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\BulkCountryUpdatePage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/BulkCountryUpdatePage.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\CountryContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/CountryContext.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\CountryInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/CountryInstance.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\CountryList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/CountryList.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\CountryOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/CountryOptions.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\CountryPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/CountryPage.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\Country\\HighriskSpecialPrefixInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/Country/HighriskSpecialPrefixInstance.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\Country\\HighriskSpecialPrefixList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/Country/HighriskSpecialPrefixList.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\Country\\HighriskSpecialPrefixPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/Country/HighriskSpecialPrefixPage.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\SettingsContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/SettingsContext.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\SettingsInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/SettingsInstance.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\SettingsList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/SettingsList.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\SettingsOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/SettingsOptions.php',
    'Twilio\\Rest\\Voice\\V1\\DialingPermissions\\SettingsPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/DialingPermissions/SettingsPage.php',
    'Twilio\\Rest\\Voice\\V1\\IpRecordContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/IpRecordContext.php',
    'Twilio\\Rest\\Voice\\V1\\IpRecordInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/IpRecordInstance.php',
    'Twilio\\Rest\\Voice\\V1\\IpRecordList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/IpRecordList.php',
    'Twilio\\Rest\\Voice\\V1\\IpRecordOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/IpRecordOptions.php',
    'Twilio\\Rest\\Voice\\V1\\IpRecordPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/IpRecordPage.php',
    'Twilio\\Rest\\Voice\\V1\\SourceIpMappingContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/SourceIpMappingContext.php',
    'Twilio\\Rest\\Voice\\V1\\SourceIpMappingInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/SourceIpMappingInstance.php',
    'Twilio\\Rest\\Voice\\V1\\SourceIpMappingList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/SourceIpMappingList.php',
    'Twilio\\Rest\\Voice\\V1\\SourceIpMappingPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Voice/V1/SourceIpMappingPage.php',
    'Twilio\\Rest\\Wireless' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless.php',
    'Twilio\\Rest\\Wireless\\V1' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1.php',
    'Twilio\\Rest\\Wireless\\V1\\CommandContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/CommandContext.php',
    'Twilio\\Rest\\Wireless\\V1\\CommandInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/CommandInstance.php',
    'Twilio\\Rest\\Wireless\\V1\\CommandList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/CommandList.php',
    'Twilio\\Rest\\Wireless\\V1\\CommandOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/CommandOptions.php',
    'Twilio\\Rest\\Wireless\\V1\\CommandPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/CommandPage.php',
    'Twilio\\Rest\\Wireless\\V1\\RatePlanContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/RatePlanContext.php',
    'Twilio\\Rest\\Wireless\\V1\\RatePlanInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/RatePlanInstance.php',
    'Twilio\\Rest\\Wireless\\V1\\RatePlanList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/RatePlanList.php',
    'Twilio\\Rest\\Wireless\\V1\\RatePlanOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/RatePlanOptions.php',
    'Twilio\\Rest\\Wireless\\V1\\RatePlanPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/RatePlanPage.php',
    'Twilio\\Rest\\Wireless\\V1\\SimContext' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/SimContext.php',
    'Twilio\\Rest\\Wireless\\V1\\SimInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/SimInstance.php',
    'Twilio\\Rest\\Wireless\\V1\\SimList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/SimList.php',
    'Twilio\\Rest\\Wireless\\V1\\SimOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/SimOptions.php',
    'Twilio\\Rest\\Wireless\\V1\\SimPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/SimPage.php',
    'Twilio\\Rest\\Wireless\\V1\\Sim\\DataSessionInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/Sim/DataSessionInstance.php',
    'Twilio\\Rest\\Wireless\\V1\\Sim\\DataSessionList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/Sim/DataSessionList.php',
    'Twilio\\Rest\\Wireless\\V1\\Sim\\DataSessionPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/Sim/DataSessionPage.php',
    'Twilio\\Rest\\Wireless\\V1\\Sim\\UsageRecordInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/Sim/UsageRecordInstance.php',
    'Twilio\\Rest\\Wireless\\V1\\Sim\\UsageRecordList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/Sim/UsageRecordList.php',
    'Twilio\\Rest\\Wireless\\V1\\Sim\\UsageRecordOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/Sim/UsageRecordOptions.php',
    'Twilio\\Rest\\Wireless\\V1\\Sim\\UsageRecordPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/Sim/UsageRecordPage.php',
    'Twilio\\Rest\\Wireless\\V1\\UsageRecordInstance' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/UsageRecordInstance.php',
    'Twilio\\Rest\\Wireless\\V1\\UsageRecordList' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/UsageRecordList.php',
    'Twilio\\Rest\\Wireless\\V1\\UsageRecordOptions' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/UsageRecordOptions.php',
    'Twilio\\Rest\\Wireless\\V1\\UsageRecordPage' => $vendorDir . '/twilio/sdk/src/Twilio/Rest/Wireless/V1/UsageRecordPage.php',
    'Twilio\\Security\\RequestValidator' => $vendorDir . '/twilio/sdk/src/Twilio/Security/RequestValidator.php',
    'Twilio\\Serialize' => $vendorDir . '/twilio/sdk/src/Twilio/Serialize.php',
    'Twilio\\Stream' => $vendorDir . '/twilio/sdk/src/Twilio/Stream.php',
    'Twilio\\TaskRouter\\WorkflowConfiguration' => $vendorDir . '/twilio/sdk/src/Twilio/TaskRouter/WorkflowConfiguration.php',
    'Twilio\\TaskRouter\\WorkflowRule' => $vendorDir . '/twilio/sdk/src/Twilio/TaskRouter/WorkflowRule.php',
    'Twilio\\TaskRouter\\WorkflowRuleTarget' => $vendorDir . '/twilio/sdk/src/Twilio/TaskRouter/WorkflowRuleTarget.php',
    'Twilio\\TwiML\\FaxResponse' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/FaxResponse.php',
    'Twilio\\TwiML\\Fax\\Receive' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Fax/Receive.php',
    'Twilio\\TwiML\\GenericNode' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/GenericNode.php',
    'Twilio\\TwiML\\MessagingResponse' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/MessagingResponse.php',
    'Twilio\\TwiML\\Messaging\\Body' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Messaging/Body.php',
    'Twilio\\TwiML\\Messaging\\Media' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Messaging/Media.php',
    'Twilio\\TwiML\\Messaging\\Message' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Messaging/Message.php',
    'Twilio\\TwiML\\Messaging\\Redirect' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Messaging/Redirect.php',
    'Twilio\\TwiML\\TwiML' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/TwiML.php',
    'Twilio\\TwiML\\Video\\Room' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Video/Room.php',
    'Twilio\\TwiML\\VoiceResponse' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/VoiceResponse.php',
    'Twilio\\TwiML\\Voice\\Application' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Application.php',
    'Twilio\\TwiML\\Voice\\ApplicationSid' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/ApplicationSid.php',
    'Twilio\\TwiML\\Voice\\Autopilot' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Autopilot.php',
    'Twilio\\TwiML\\Voice\\Client' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Client.php',
    'Twilio\\TwiML\\Voice\\Conference' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Conference.php',
    'Twilio\\TwiML\\Voice\\Config' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Config.php',
    'Twilio\\TwiML\\Voice\\Connect' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Connect.php',
    'Twilio\\TwiML\\Voice\\Conversation' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Conversation.php',
    'Twilio\\TwiML\\Voice\\Dial' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Dial.php',
    'Twilio\\TwiML\\Voice\\Echo_' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Echo_.php',
    'Twilio\\TwiML\\Voice\\Enqueue' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Enqueue.php',
    'Twilio\\TwiML\\Voice\\Gather' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Gather.php',
    'Twilio\\TwiML\\Voice\\Hangup' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Hangup.php',
    'Twilio\\TwiML\\Voice\\Identity' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Identity.php',
    'Twilio\\TwiML\\Voice\\Leave' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Leave.php',
    'Twilio\\TwiML\\Voice\\Number' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Number.php',
    'Twilio\\TwiML\\Voice\\Parameter' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Parameter.php',
    'Twilio\\TwiML\\Voice\\Pause' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Pause.php',
    'Twilio\\TwiML\\Voice\\Pay' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Pay.php',
    'Twilio\\TwiML\\Voice\\Play' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Play.php',
    'Twilio\\TwiML\\Voice\\Prompt' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Prompt.php',
    'Twilio\\TwiML\\Voice\\Queue' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Queue.php',
    'Twilio\\TwiML\\Voice\\Record' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Record.php',
    'Twilio\\TwiML\\Voice\\Redirect' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Redirect.php',
    'Twilio\\TwiML\\Voice\\Refer' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Refer.php',
    'Twilio\\TwiML\\Voice\\ReferSip' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/ReferSip.php',
    'Twilio\\TwiML\\Voice\\Reject' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Reject.php',
    'Twilio\\TwiML\\Voice\\Room' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Room.php',
    'Twilio\\TwiML\\Voice\\Say' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Say.php',
    'Twilio\\TwiML\\Voice\\Sim' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Sim.php',
    'Twilio\\TwiML\\Voice\\Sip' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Sip.php',
    'Twilio\\TwiML\\Voice\\Siprec' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Siprec.php',
    'Twilio\\TwiML\\Voice\\Sms' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Sms.php',
    'Twilio\\TwiML\\Voice\\SsmlBreak' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlBreak.php',
    'Twilio\\TwiML\\Voice\\SsmlEmphasis' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlEmphasis.php',
    'Twilio\\TwiML\\Voice\\SsmlLang' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlLang.php',
    'Twilio\\TwiML\\Voice\\SsmlP' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlP.php',
    'Twilio\\TwiML\\Voice\\SsmlPhoneme' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlPhoneme.php',
    'Twilio\\TwiML\\Voice\\SsmlProsody' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlProsody.php',
    'Twilio\\TwiML\\Voice\\SsmlS' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlS.php',
    'Twilio\\TwiML\\Voice\\SsmlSayAs' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlSayAs.php',
    'Twilio\\TwiML\\Voice\\SsmlSub' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlSub.php',
    'Twilio\\TwiML\\Voice\\SsmlW' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/SsmlW.php',
    'Twilio\\TwiML\\Voice\\Start' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Start.php',
    'Twilio\\TwiML\\Voice\\Stop' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Stop.php',
    'Twilio\\TwiML\\Voice\\Stream' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Stream.php',
    'Twilio\\TwiML\\Voice\\Task' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/Task.php',
    'Twilio\\TwiML\\Voice\\VirtualAgent' => $vendorDir . '/twilio/sdk/src/Twilio/TwiML/Voice/VirtualAgent.php',
    'Twilio\\Values' => $vendorDir . '/twilio/sdk/src/Twilio/Values.php',
    'Twilio\\Version' => $vendorDir . '/twilio/sdk/src/Twilio/Version.php',
    'Twilio\\VersionInfo' => $vendorDir . '/twilio/sdk/src/Twilio/VersionInfo.php',
    'UnhandledMatchError' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/UnhandledMatchError.php',
    'ValueError' => $vendorDir . '/symfony/polyfill-php80/Resources/stubs/ValueError.php',
    'Vonage\\Account\\Balance' => $vendorDir . '/vonage/client-core/src/Account/Balance.php',
    'Vonage\\Account\\Client' => $vendorDir . '/vonage/client-core/src/Account/Client.php',
    'Vonage\\Account\\ClientFactory' => $vendorDir . '/vonage/client-core/src/Account/ClientFactory.php',
    'Vonage\\Account\\Config' => $vendorDir . '/vonage/client-core/src/Account/Config.php',
    'Vonage\\Account\\PrefixPrice' => $vendorDir . '/vonage/client-core/src/Account/PrefixPrice.php',
    'Vonage\\Account\\Price' => $vendorDir . '/vonage/client-core/src/Account/Price.php',
    'Vonage\\Account\\Secret' => $vendorDir . '/vonage/client-core/src/Account/Secret.php',
    'Vonage\\Account\\SecretCollection' => $vendorDir . '/vonage/client-core/src/Account/SecretCollection.php',
    'Vonage\\Account\\SmsPrice' => $vendorDir . '/vonage/client-core/src/Account/SmsPrice.php',
    'Vonage\\Account\\VoicePrice' => $vendorDir . '/vonage/client-core/src/Account/VoicePrice.php',
    'Vonage\\ApiErrorHandler' => $vendorDir . '/vonage/client-core/src/ApiErrorHandler.php',
    'Vonage\\Application\\Application' => $vendorDir . '/vonage/client-core/src/Application/Application.php',
    'Vonage\\Application\\ApplicationInterface' => $vendorDir . '/vonage/client-core/src/Application/ApplicationInterface.php',
    'Vonage\\Application\\Client' => $vendorDir . '/vonage/client-core/src/Application/Client.php',
    'Vonage\\Application\\ClientFactory' => $vendorDir . '/vonage/client-core/src/Application/ClientFactory.php',
    'Vonage\\Application\\Filter' => $vendorDir . '/vonage/client-core/src/Application/Filter.php',
    'Vonage\\Application\\Hydrator' => $vendorDir . '/vonage/client-core/src/Application/Hydrator.php',
    'Vonage\\Application\\MessagesConfig' => $vendorDir . '/vonage/client-core/src/Application/MessagesConfig.php',
    'Vonage\\Application\\RtcConfig' => $vendorDir . '/vonage/client-core/src/Application/RtcConfig.php',
    'Vonage\\Application\\VbcConfig' => $vendorDir . '/vonage/client-core/src/Application/VbcConfig.php',
    'Vonage\\Application\\VoiceConfig' => $vendorDir . '/vonage/client-core/src/Application/VoiceConfig.php',
    'Vonage\\Application\\Webhook' => $vendorDir . '/vonage/client-core/src/Application/Webhook.php',
    'Vonage\\Call\\Call' => $vendorDir . '/vonage/client-core/src/Call/Call.php',
    'Vonage\\Call\\Collection' => $vendorDir . '/vonage/client-core/src/Call/Collection.php',
    'Vonage\\Call\\Dtmf' => $vendorDir . '/vonage/client-core/src/Call/Dtmf.php',
    'Vonage\\Call\\Earmuff' => $vendorDir . '/vonage/client-core/src/Call/Earmuff.php',
    'Vonage\\Call\\Endpoint' => $vendorDir . '/vonage/client-core/src/Call/Endpoint.php',
    'Vonage\\Call\\Event' => $vendorDir . '/vonage/client-core/src/Call/Event.php',
    'Vonage\\Call\\Filter' => $vendorDir . '/vonage/client-core/src/Call/Filter.php',
    'Vonage\\Call\\Hangup' => $vendorDir . '/vonage/client-core/src/Call/Hangup.php',
    'Vonage\\Call\\Mute' => $vendorDir . '/vonage/client-core/src/Call/Mute.php',
    'Vonage\\Call\\Stream' => $vendorDir . '/vonage/client-core/src/Call/Stream.php',
    'Vonage\\Call\\Talk' => $vendorDir . '/vonage/client-core/src/Call/Talk.php',
    'Vonage\\Call\\Transfer' => $vendorDir . '/vonage/client-core/src/Call/Transfer.php',
    'Vonage\\Call\\Unearmuff' => $vendorDir . '/vonage/client-core/src/Call/Unearmuff.php',
    'Vonage\\Call\\Unmute' => $vendorDir . '/vonage/client-core/src/Call/Unmute.php',
    'Vonage\\Call\\Webhook' => $vendorDir . '/vonage/client-core/src/Call/Webhook.php',
    'Vonage\\Client' => $vendorDir . '/vonage/client-core/src/Client.php',
    'Vonage\\Client\\APIClient' => $vendorDir . '/vonage/client-core/src/Client/APIClient.php',
    'Vonage\\Client\\APIExceptionHandler' => $vendorDir . '/vonage/client-core/src/Client/APIExceptionHandler.php',
    'Vonage\\Client\\APIResource' => $vendorDir . '/vonage/client-core/src/Client/APIResource.php',
    'Vonage\\Client\\Callback\\Callback' => $vendorDir . '/vonage/client-core/src/Client/Callback/Callback.php',
    'Vonage\\Client\\Callback\\CallbackInterface' => $vendorDir . '/vonage/client-core/src/Client/Callback/CallbackInterface.php',
    'Vonage\\Client\\ClientAwareInterface' => $vendorDir . '/vonage/client-core/src/Client/ClientAwareInterface.php',
    'Vonage\\Client\\ClientAwareTrait' => $vendorDir . '/vonage/client-core/src/Client/ClientAwareTrait.php',
    'Vonage\\Client\\Credentials\\AbstractCredentials' => $vendorDir . '/vonage/client-core/src/Client/Credentials/AbstractCredentials.php',
    'Vonage\\Client\\Credentials\\Basic' => $vendorDir . '/vonage/client-core/src/Client/Credentials/Basic.php',
    'Vonage\\Client\\Credentials\\Container' => $vendorDir . '/vonage/client-core/src/Client/Credentials/Container.php',
    'Vonage\\Client\\Credentials\\CredentialsInterface' => $vendorDir . '/vonage/client-core/src/Client/Credentials/CredentialsInterface.php',
    'Vonage\\Client\\Credentials\\Keypair' => $vendorDir . '/vonage/client-core/src/Client/Credentials/Keypair.php',
    'Vonage\\Client\\Credentials\\OAuth' => $vendorDir . '/vonage/client-core/src/Client/Credentials/OAuth.php',
    'Vonage\\Client\\Credentials\\SignatureSecret' => $vendorDir . '/vonage/client-core/src/Client/Credentials/SignatureSecret.php',
    'Vonage\\Client\\Exception\\Exception' => $vendorDir . '/vonage/client-core/src/Client/Exception/Exception.php',
    'Vonage\\Client\\Exception\\Request' => $vendorDir . '/vonage/client-core/src/Client/Exception/Request.php',
    'Vonage\\Client\\Exception\\Server' => $vendorDir . '/vonage/client-core/src/Client/Exception/Server.php',
    'Vonage\\Client\\Exception\\ThrottleException' => $vendorDir . '/vonage/client-core/src/Client/Exception/ThrottleException.php',
    'Vonage\\Client\\Exception\\Transport' => $vendorDir . '/vonage/client-core/src/Client/Exception/Transport.php',
    'Vonage\\Client\\Exception\\Validation' => $vendorDir . '/vonage/client-core/src/Client/Exception/Validation.php',
    'Vonage\\Client\\Factory\\FactoryInterface' => $vendorDir . '/vonage/client-core/src/Client/Factory/FactoryInterface.php',
    'Vonage\\Client\\Factory\\MapFactory' => $vendorDir . '/vonage/client-core/src/Client/Factory/MapFactory.php',
    'Vonage\\Client\\Request\\AbstractRequest' => $vendorDir . '/vonage/client-core/src/Client/Request/AbstractRequest.php',
    'Vonage\\Client\\Request\\RequestInterface' => $vendorDir . '/vonage/client-core/src/Client/Request/RequestInterface.php',
    'Vonage\\Client\\Request\\WrapResponseInterface' => $vendorDir . '/vonage/client-core/src/Client/Request/WrapResponseInterface.php',
    'Vonage\\Client\\Response\\AbstractResponse' => $vendorDir . '/vonage/client-core/src/Client/Response/AbstractResponse.php',
    'Vonage\\Client\\Response\\Error' => $vendorDir . '/vonage/client-core/src/Client/Response/Error.php',
    'Vonage\\Client\\Response\\Response' => $vendorDir . '/vonage/client-core/src/Client/Response/Response.php',
    'Vonage\\Client\\Response\\ResponseInterface' => $vendorDir . '/vonage/client-core/src/Client/Response/ResponseInterface.php',
    'Vonage\\Client\\Signature' => $vendorDir . '/vonage/client-core/src/Client/Signature.php',
    'Vonage\\Conversations\\Collection' => $vendorDir . '/vonage/client-core/src/Conversations/Collection.php',
    'Vonage\\Conversations\\Conversation' => $vendorDir . '/vonage/client-core/src/Conversations/Conversation.php',
    'Vonage\\Conversion\\Client' => $vendorDir . '/vonage/client-core/src/Conversion/Client.php',
    'Vonage\\Conversion\\ClientFactory' => $vendorDir . '/vonage/client-core/src/Conversion/ClientFactory.php',
    'Vonage\\Entity\\ArrayAccessTrait' => $vendorDir . '/vonage/client-core/src/Entity/ArrayAccessTrait.php',
    'Vonage\\Entity\\CollectionAwareInterface' => $vendorDir . '/vonage/client-core/src/Entity/CollectionAwareInterface.php',
    'Vonage\\Entity\\CollectionAwareTrait' => $vendorDir . '/vonage/client-core/src/Entity/CollectionAwareTrait.php',
    'Vonage\\Entity\\CollectionInterface' => $vendorDir . '/vonage/client-core/src/Entity/CollectionInterface.php',
    'Vonage\\Entity\\CollectionTrait' => $vendorDir . '/vonage/client-core/src/Entity/CollectionTrait.php',
    'Vonage\\Entity\\EntityInterface' => $vendorDir . '/vonage/client-core/src/Entity/EntityInterface.php',
    'Vonage\\Entity\\Factory\\FactoryInterface' => $vendorDir . '/vonage/client-core/src/Entity/Factory/FactoryInterface.php',
    'Vonage\\Entity\\Filter\\DateFilter' => $vendorDir . '/vonage/client-core/src/Entity/Filter/DateFilter.php',
    'Vonage\\Entity\\Filter\\EmptyFilter' => $vendorDir . '/vonage/client-core/src/Entity/Filter/EmptyFilter.php',
    'Vonage\\Entity\\Filter\\FilterInterface' => $vendorDir . '/vonage/client-core/src/Entity/Filter/FilterInterface.php',
    'Vonage\\Entity\\Filter\\KeyValueFilter' => $vendorDir . '/vonage/client-core/src/Entity/Filter/KeyValueFilter.php',
    'Vonage\\Entity\\HasEntityTrait' => $vendorDir . '/vonage/client-core/src/Entity/HasEntityTrait.php',
    'Vonage\\Entity\\Hydrator\\ArrayHydrateInterface' => $vendorDir . '/vonage/client-core/src/Entity/Hydrator/ArrayHydrateInterface.php',
    'Vonage\\Entity\\Hydrator\\ArrayHydrator' => $vendorDir . '/vonage/client-core/src/Entity/Hydrator/ArrayHydrator.php',
    'Vonage\\Entity\\Hydrator\\HydratorInterface' => $vendorDir . '/vonage/client-core/src/Entity/Hydrator/HydratorInterface.php',
    'Vonage\\Entity\\IterableAPICollection' => $vendorDir . '/vonage/client-core/src/Entity/IterableAPICollection.php',
    'Vonage\\Entity\\IterableServiceShimTrait' => $vendorDir . '/vonage/client-core/src/Entity/IterableServiceShimTrait.php',
    'Vonage\\Entity\\JsonResponseTrait' => $vendorDir . '/vonage/client-core/src/Entity/JsonResponseTrait.php',
    'Vonage\\Entity\\JsonSerializableInterface' => $vendorDir . '/vonage/client-core/src/Entity/JsonSerializableInterface.php',
    'Vonage\\Entity\\JsonSerializableTrait' => $vendorDir . '/vonage/client-core/src/Entity/JsonSerializableTrait.php',
    'Vonage\\Entity\\JsonUnserializableInterface' => $vendorDir . '/vonage/client-core/src/Entity/JsonUnserializableInterface.php',
    'Vonage\\Entity\\ModernCollectionTrait' => $vendorDir . '/vonage/client-core/src/Entity/ModernCollectionTrait.php',
    'Vonage\\Entity\\NoRequestResponseTrait' => $vendorDir . '/vonage/client-core/src/Entity/NoRequestResponseTrait.php',
    'Vonage\\Entity\\Psr7Trait' => $vendorDir . '/vonage/client-core/src/Entity/Psr7Trait.php',
    'Vonage\\Entity\\RequestArrayTrait' => $vendorDir . '/vonage/client-core/src/Entity/RequestArrayTrait.php',
    'Vonage\\Insights\\Advanced' => $vendorDir . '/vonage/client-core/src/Insights/Advanced.php',
    'Vonage\\Insights\\AdvancedCnam' => $vendorDir . '/vonage/client-core/src/Insights/AdvancedCnam.php',
    'Vonage\\Insights\\Basic' => $vendorDir . '/vonage/client-core/src/Insights/Basic.php',
    'Vonage\\Insights\\Client' => $vendorDir . '/vonage/client-core/src/Insights/Client.php',
    'Vonage\\Insights\\ClientFactory' => $vendorDir . '/vonage/client-core/src/Insights/ClientFactory.php',
    'Vonage\\Insights\\CnamTrait' => $vendorDir . '/vonage/client-core/src/Insights/CnamTrait.php',
    'Vonage\\Insights\\Standard' => $vendorDir . '/vonage/client-core/src/Insights/Standard.php',
    'Vonage\\Insights\\StandardCnam' => $vendorDir . '/vonage/client-core/src/Insights/StandardCnam.php',
    'Vonage\\InvalidResponseException' => $vendorDir . '/vonage/client-core/src/InvalidResponseException.php',
    'Vonage\\Message\\AutoDetect' => $vendorDir . '/vonage/client-core/src/Message/AutoDetect.php',
    'Vonage\\Message\\Binary' => $vendorDir . '/vonage/client-core/src/Message/Binary.php',
    'Vonage\\Message\\Callback\\Receipt' => $vendorDir . '/vonage/client-core/src/Message/Callback/Receipt.php',
    'Vonage\\Message\\Client' => $vendorDir . '/vonage/client-core/src/Message/Client.php',
    'Vonage\\Message\\CollectionTrait' => $vendorDir . '/vonage/client-core/src/Message/CollectionTrait.php',
    'Vonage\\Message\\EncodingDetector' => $vendorDir . '/vonage/client-core/src/Message/EncodingDetector.php',
    'Vonage\\Message\\InboundMessage' => $vendorDir . '/vonage/client-core/src/Message/InboundMessage.php',
    'Vonage\\Message\\Message' => $vendorDir . '/vonage/client-core/src/Message/Message.php',
    'Vonage\\Message\\MessageInterface' => $vendorDir . '/vonage/client-core/src/Message/MessageInterface.php',
    'Vonage\\Message\\Query' => $vendorDir . '/vonage/client-core/src/Message/Query.php',
    'Vonage\\Message\\Response\\Collection' => $vendorDir . '/vonage/client-core/src/Message/Response/Collection.php',
    'Vonage\\Message\\Response\\Message' => $vendorDir . '/vonage/client-core/src/Message/Response/Message.php',
    'Vonage\\Message\\Shortcode' => $vendorDir . '/vonage/client-core/src/Message/Shortcode.php',
    'Vonage\\Message\\Shortcode\\Alert' => $vendorDir . '/vonage/client-core/src/Message/Shortcode/Alert.php',
    'Vonage\\Message\\Shortcode\\Marketing' => $vendorDir . '/vonage/client-core/src/Message/Shortcode/Marketing.php',
    'Vonage\\Message\\Shortcode\\TwoFactor' => $vendorDir . '/vonage/client-core/src/Message/Shortcode/TwoFactor.php',
    'Vonage\\Message\\Text' => $vendorDir . '/vonage/client-core/src/Message/Text.php',
    'Vonage\\Message\\Unicode' => $vendorDir . '/vonage/client-core/src/Message/Unicode.php',
    'Vonage\\Message\\Vcal' => $vendorDir . '/vonage/client-core/src/Message/Vcal.php',
    'Vonage\\Message\\Vcard' => $vendorDir . '/vonage/client-core/src/Message/Vcard.php',
    'Vonage\\Message\\Wap' => $vendorDir . '/vonage/client-core/src/Message/Wap.php',
    'Vonage\\Network' => $vendorDir . '/vonage/client-core/src/Network.php',
    'Vonage\\Network\\Number\\Callback' => $vendorDir . '/vonage/client-core/src/Network/Number/Callback.php',
    'Vonage\\Network\\Number\\Request' => $vendorDir . '/vonage/client-core/src/Network/Number/Request.php',
    'Vonage\\Network\\Number\\Response' => $vendorDir . '/vonage/client-core/src/Network/Number/Response.php',
    'Vonage\\NexmoBridge\\Autoloader' => $vendorDir . '/vonage/nexmo-bridge/src/Autoloader.php',
    'Vonage\\Numbers\\Client' => $vendorDir . '/vonage/client-core/src/Numbers/Client.php',
    'Vonage\\Numbers\\ClientFactory' => $vendorDir . '/vonage/client-core/src/Numbers/ClientFactory.php',
    'Vonage\\Numbers\\Filter\\AvailableNumbers' => $vendorDir . '/vonage/client-core/src/Numbers/Filter/AvailableNumbers.php',
    'Vonage\\Numbers\\Filter\\OwnedNumbers' => $vendorDir . '/vonage/client-core/src/Numbers/Filter/OwnedNumbers.php',
    'Vonage\\Numbers\\Hydrator' => $vendorDir . '/vonage/client-core/src/Numbers/Hydrator.php',
    'Vonage\\Numbers\\Number' => $vendorDir . '/vonage/client-core/src/Numbers/Number.php',
    'Vonage\\Redact\\Client' => $vendorDir . '/vonage/client-core/src/Redact/Client.php',
    'Vonage\\Redact\\ClientFactory' => $vendorDir . '/vonage/client-core/src/Redact/ClientFactory.php',
    'Vonage\\Response' => $vendorDir . '/vonage/client-core/src/Response.php',
    'Vonage\\Response\\Message' => $vendorDir . '/vonage/client-core/src/Response/Message.php',
    'Vonage\\SMS\\Client' => $vendorDir . '/vonage/client-core/src/SMS/Client.php',
    'Vonage\\SMS\\ClientFactory' => $vendorDir . '/vonage/client-core/src/SMS/ClientFactory.php',
    'Vonage\\SMS\\Collection' => $vendorDir . '/vonage/client-core/src/SMS/Collection.php',
    'Vonage\\SMS\\EncodingDetector' => $vendorDir . '/vonage/client-core/src/SMS/EncodingDetector.php',
    'Vonage\\SMS\\ExceptionErrorHandler' => $vendorDir . '/vonage/client-core/src/SMS/ExceptionErrorHandler.php',
    'Vonage\\SMS\\Message\\Binary' => $vendorDir . '/vonage/client-core/src/SMS/Message/Binary.php',
    'Vonage\\SMS\\Message\\Message' => $vendorDir . '/vonage/client-core/src/SMS/Message/Message.php',
    'Vonage\\SMS\\Message\\OutboundMessage' => $vendorDir . '/vonage/client-core/src/SMS/Message/OutboundMessage.php',
    'Vonage\\SMS\\Message\\SMS' => $vendorDir . '/vonage/client-core/src/SMS/Message/SMS.php',
    'Vonage\\SMS\\Message\\Vcal' => $vendorDir . '/vonage/client-core/src/SMS/Message/Vcal.php',
    'Vonage\\SMS\\Message\\Vcard' => $vendorDir . '/vonage/client-core/src/SMS/Message/Vcard.php',
    'Vonage\\SMS\\Message\\WAPPush' => $vendorDir . '/vonage/client-core/src/SMS/Message/WAPPush.php',
    'Vonage\\SMS\\SentSMS' => $vendorDir . '/vonage/client-core/src/SMS/SentSMS.php',
    'Vonage\\SMS\\Webhook\\DeliveryReceipt' => $vendorDir . '/vonage/client-core/src/SMS/Webhook/DeliveryReceipt.php',
    'Vonage\\SMS\\Webhook\\Factory' => $vendorDir . '/vonage/client-core/src/SMS/Webhook/Factory.php',
    'Vonage\\SMS\\Webhook\\InboundSMS' => $vendorDir . '/vonage/client-core/src/SMS/Webhook/InboundSMS.php',
    'Vonage\\User\\Collection' => $vendorDir . '/vonage/client-core/src/User/Collection.php',
    'Vonage\\User\\User' => $vendorDir . '/vonage/client-core/src/User/User.php',
    'Vonage\\Verify\\Check' => $vendorDir . '/vonage/client-core/src/Verify/Check.php',
    'Vonage\\Verify\\Client' => $vendorDir . '/vonage/client-core/src/Verify/Client.php',
    'Vonage\\Verify\\ClientFactory' => $vendorDir . '/vonage/client-core/src/Verify/ClientFactory.php',
    'Vonage\\Verify\\ExceptionErrorHandler' => $vendorDir . '/vonage/client-core/src/Verify/ExceptionErrorHandler.php',
    'Vonage\\Verify\\Request' => $vendorDir . '/vonage/client-core/src/Verify/Request.php',
    'Vonage\\Verify\\RequestPSD2' => $vendorDir . '/vonage/client-core/src/Verify/RequestPSD2.php',
    'Vonage\\Verify\\Verification' => $vendorDir . '/vonage/client-core/src/Verify/Verification.php',
    'Vonage\\Verify\\VerificationInterface' => $vendorDir . '/vonage/client-core/src/Verify/VerificationInterface.php',
    'Vonage\\Voice\\Call' => $vendorDir . '/vonage/client-core/src/Voice/Call.php',
    'Vonage\\Voice\\CallAction' => $vendorDir . '/vonage/client-core/src/Voice/CallAction.php',
    'Vonage\\Voice\\CallFactory' => $vendorDir . '/vonage/client-core/src/Voice/CallFactory.php',
    'Vonage\\Voice\\Call\\Call' => $vendorDir . '/vonage/client-core/src/Voice/Call/Call.php',
    'Vonage\\Voice\\Call\\Inbound' => $vendorDir . '/vonage/client-core/src/Voice/Call/Inbound.php',
    'Vonage\\Voice\\Client' => $vendorDir . '/vonage/client-core/src/Voice/Client.php',
    'Vonage\\Voice\\ClientFactory' => $vendorDir . '/vonage/client-core/src/Voice/ClientFactory.php',
    'Vonage\\Voice\\Endpoint\\App' => $vendorDir . '/vonage/client-core/src/Voice/Endpoint/App.php',
    'Vonage\\Voice\\Endpoint\\EndpointFactory' => $vendorDir . '/vonage/client-core/src/Voice/Endpoint/EndpointFactory.php',
    'Vonage\\Voice\\Endpoint\\EndpointInterface' => $vendorDir . '/vonage/client-core/src/Voice/Endpoint/EndpointInterface.php',
    'Vonage\\Voice\\Endpoint\\Phone' => $vendorDir . '/vonage/client-core/src/Voice/Endpoint/Phone.php',
    'Vonage\\Voice\\Endpoint\\SIP' => $vendorDir . '/vonage/client-core/src/Voice/Endpoint/SIP.php',
    'Vonage\\Voice\\Endpoint\\VBC' => $vendorDir . '/vonage/client-core/src/Voice/Endpoint/VBC.php',
    'Vonage\\Voice\\Endpoint\\Websocket' => $vendorDir . '/vonage/client-core/src/Voice/Endpoint/Websocket.php',
    'Vonage\\Voice\\Filter\\VoiceFilter' => $vendorDir . '/vonage/client-core/src/Voice/Filter/VoiceFilter.php',
    'Vonage\\Voice\\Message\\Callback' => $vendorDir . '/vonage/client-core/src/Voice/Message/Callback.php',
    'Vonage\\Voice\\Message\\Message' => $vendorDir . '/vonage/client-core/src/Voice/Message/Message.php',
    'Vonage\\Voice\\NCCO\\Action\\ActionInterface' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/Action/ActionInterface.php',
    'Vonage\\Voice\\NCCO\\Action\\Connect' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/Action/Connect.php',
    'Vonage\\Voice\\NCCO\\Action\\Conversation' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/Action/Conversation.php',
    'Vonage\\Voice\\NCCO\\Action\\Input' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/Action/Input.php',
    'Vonage\\Voice\\NCCO\\Action\\Notify' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/Action/Notify.php',
    'Vonage\\Voice\\NCCO\\Action\\Record' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/Action/Record.php',
    'Vonage\\Voice\\NCCO\\Action\\Stream' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/Action/Stream.php',
    'Vonage\\Voice\\NCCO\\Action\\Talk' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/Action/Talk.php',
    'Vonage\\Voice\\NCCO\\NCCO' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/NCCO.php',
    'Vonage\\Voice\\NCCO\\NCCOFactory' => $vendorDir . '/vonage/client-core/src/Voice/NCCO/NCCOFactory.php',
    'Vonage\\Voice\\OutboundCall' => $vendorDir . '/vonage/client-core/src/Voice/OutboundCall.php',
    'Vonage\\Voice\\Webhook' => $vendorDir . '/vonage/client-core/src/Voice/Webhook.php',
    'Vonage\\Voice\\Webhook\\Answer' => $vendorDir . '/vonage/client-core/src/Voice/Webhook/Answer.php',
    'Vonage\\Voice\\Webhook\\Error' => $vendorDir . '/vonage/client-core/src/Voice/Webhook/Error.php',
    'Vonage\\Voice\\Webhook\\Event' => $vendorDir . '/vonage/client-core/src/Voice/Webhook/Event.php',
    'Vonage\\Voice\\Webhook\\Factory' => $vendorDir . '/vonage/client-core/src/Voice/Webhook/Factory.php',
    'Vonage\\Voice\\Webhook\\Input' => $vendorDir . '/vonage/client-core/src/Voice/Webhook/Input.php',
    'Vonage\\Voice\\Webhook\\Notification' => $vendorDir . '/vonage/client-core/src/Voice/Webhook/Notification.php',
    'Vonage\\Voice\\Webhook\\Record' => $vendorDir . '/vonage/client-core/src/Voice/Webhook/Record.php',
    'Vonage\\Voice\\Webhook\\Transfer' => $vendorDir . '/vonage/client-core/src/Voice/Webhook/Transfer.php',
    'Vonage\\Webhook\\Factory' => $vendorDir . '/vonage/client-core/src/Webhook/Factory.php',
    'Webmozart\\Assert\\Assert' => $vendorDir . '/webmozart/assert/src/Assert.php',
    'Webmozart\\Assert\\InvalidArgumentException' => $vendorDir . '/webmozart/assert/src/InvalidArgumentException.php',
    'Webmozart\\Assert\\Mixin' => $vendorDir . '/webmozart/assert/src/Mixin.php',
    'Whoops\\Exception\\ErrorException' => $vendorDir . '/filp/whoops/src/Whoops/Exception/ErrorException.php',
    'Whoops\\Exception\\Formatter' => $vendorDir . '/filp/whoops/src/Whoops/Exception/Formatter.php',
    'Whoops\\Exception\\Frame' => $vendorDir . '/filp/whoops/src/Whoops/Exception/Frame.php',
    'Whoops\\Exception\\FrameCollection' => $vendorDir . '/filp/whoops/src/Whoops/Exception/FrameCollection.php',
    'Whoops\\Exception\\Inspector' => $vendorDir . '/filp/whoops/src/Whoops/Exception/Inspector.php',
    'Whoops\\Handler\\CallbackHandler' => $vendorDir . '/filp/whoops/src/Whoops/Handler/CallbackHandler.php',
    'Whoops\\Handler\\Handler' => $vendorDir . '/filp/whoops/src/Whoops/Handler/Handler.php',
    'Whoops\\Handler\\HandlerInterface' => $vendorDir . '/filp/whoops/src/Whoops/Handler/HandlerInterface.php',
    'Whoops\\Handler\\JsonResponseHandler' => $vendorDir . '/filp/whoops/src/Whoops/Handler/JsonResponseHandler.php',
    'Whoops\\Handler\\PlainTextHandler' => $vendorDir . '/filp/whoops/src/Whoops/Handler/PlainTextHandler.php',
    'Whoops\\Handler\\PrettyPageHandler' => $vendorDir . '/filp/whoops/src/Whoops/Handler/PrettyPageHandler.php',
    'Whoops\\Handler\\XmlResponseHandler' => $vendorDir . '/filp/whoops/src/Whoops/Handler/XmlResponseHandler.php',
    'Whoops\\Inspector\\InspectorFactory' => $vendorDir . '/filp/whoops/src/Whoops/Inspector/InspectorFactory.php',
    'Whoops\\Inspector\\InspectorFactoryInterface' => $vendorDir . '/filp/whoops/src/Whoops/Inspector/InspectorFactoryInterface.php',
    'Whoops\\Inspector\\InspectorInterface' => $vendorDir . '/filp/whoops/src/Whoops/Inspector/InspectorInterface.php',
    'Whoops\\Run' => $vendorDir . '/filp/whoops/src/Whoops/Run.php',
    'Whoops\\RunInterface' => $vendorDir . '/filp/whoops/src/Whoops/RunInterface.php',
    'Whoops\\Util\\HtmlDumperOutput' => $vendorDir . '/filp/whoops/src/Whoops/Util/HtmlDumperOutput.php',
    'Whoops\\Util\\Misc' => $vendorDir . '/filp/whoops/src/Whoops/Util/Misc.php',
    'Whoops\\Util\\SystemFacade' => $vendorDir . '/filp/whoops/src/Whoops/Util/SystemFacade.php',
    'Whoops\\Util\\TemplateHelper' => $vendorDir . '/filp/whoops/src/Whoops/Util/TemplateHelper.php',
    'WpOrg\\Requests\\Auth' => $vendorDir . '/rmccue/requests/src/Auth.php',
    'WpOrg\\Requests\\Auth\\Basic' => $vendorDir . '/rmccue/requests/src/Auth/Basic.php',
    'WpOrg\\Requests\\Autoload' => $vendorDir . '/rmccue/requests/src/Autoload.php',
    'WpOrg\\Requests\\Capability' => $vendorDir . '/rmccue/requests/src/Capability.php',
    'WpOrg\\Requests\\Cookie' => $vendorDir . '/rmccue/requests/src/Cookie.php',
    'WpOrg\\Requests\\Cookie\\Jar' => $vendorDir . '/rmccue/requests/src/Cookie/Jar.php',
    'WpOrg\\Requests\\Exception' => $vendorDir . '/rmccue/requests/src/Exception.php',
    'WpOrg\\Requests\\Exception\\ArgumentCount' => $vendorDir . '/rmccue/requests/src/Exception/ArgumentCount.php',
    'WpOrg\\Requests\\Exception\\Http' => $vendorDir . '/rmccue/requests/src/Exception/Http.php',
    'WpOrg\\Requests\\Exception\\Http\\Status304' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status304.php',
    'WpOrg\\Requests\\Exception\\Http\\Status305' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status305.php',
    'WpOrg\\Requests\\Exception\\Http\\Status306' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status306.php',
    'WpOrg\\Requests\\Exception\\Http\\Status400' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status400.php',
    'WpOrg\\Requests\\Exception\\Http\\Status401' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status401.php',
    'WpOrg\\Requests\\Exception\\Http\\Status402' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status402.php',
    'WpOrg\\Requests\\Exception\\Http\\Status403' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status403.php',
    'WpOrg\\Requests\\Exception\\Http\\Status404' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status404.php',
    'WpOrg\\Requests\\Exception\\Http\\Status405' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status405.php',
    'WpOrg\\Requests\\Exception\\Http\\Status406' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status406.php',
    'WpOrg\\Requests\\Exception\\Http\\Status407' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status407.php',
    'WpOrg\\Requests\\Exception\\Http\\Status408' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status408.php',
    'WpOrg\\Requests\\Exception\\Http\\Status409' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status409.php',
    'WpOrg\\Requests\\Exception\\Http\\Status410' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status410.php',
    'WpOrg\\Requests\\Exception\\Http\\Status411' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status411.php',
    'WpOrg\\Requests\\Exception\\Http\\Status412' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status412.php',
    'WpOrg\\Requests\\Exception\\Http\\Status413' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status413.php',
    'WpOrg\\Requests\\Exception\\Http\\Status414' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status414.php',
    'WpOrg\\Requests\\Exception\\Http\\Status415' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status415.php',
    'WpOrg\\Requests\\Exception\\Http\\Status416' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status416.php',
    'WpOrg\\Requests\\Exception\\Http\\Status417' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status417.php',
    'WpOrg\\Requests\\Exception\\Http\\Status418' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status418.php',
    'WpOrg\\Requests\\Exception\\Http\\Status428' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status428.php',
    'WpOrg\\Requests\\Exception\\Http\\Status429' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status429.php',
    'WpOrg\\Requests\\Exception\\Http\\Status431' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status431.php',
    'WpOrg\\Requests\\Exception\\Http\\Status500' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status500.php',
    'WpOrg\\Requests\\Exception\\Http\\Status501' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status501.php',
    'WpOrg\\Requests\\Exception\\Http\\Status502' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status502.php',
    'WpOrg\\Requests\\Exception\\Http\\Status503' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status503.php',
    'WpOrg\\Requests\\Exception\\Http\\Status504' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status504.php',
    'WpOrg\\Requests\\Exception\\Http\\Status505' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status505.php',
    'WpOrg\\Requests\\Exception\\Http\\Status511' => $vendorDir . '/rmccue/requests/src/Exception/Http/Status511.php',
    'WpOrg\\Requests\\Exception\\Http\\StatusUnknown' => $vendorDir . '/rmccue/requests/src/Exception/Http/StatusUnknown.php',
    'WpOrg\\Requests\\Exception\\InvalidArgument' => $vendorDir . '/rmccue/requests/src/Exception/InvalidArgument.php',
    'WpOrg\\Requests\\Exception\\Transport' => $vendorDir . '/rmccue/requests/src/Exception/Transport.php',
    'WpOrg\\Requests\\Exception\\Transport\\Curl' => $vendorDir . '/rmccue/requests/src/Exception/Transport/Curl.php',
    'WpOrg\\Requests\\HookManager' => $vendorDir . '/rmccue/requests/src/HookManager.php',
    'WpOrg\\Requests\\Hooks' => $vendorDir . '/rmccue/requests/src/Hooks.php',
    'WpOrg\\Requests\\IdnaEncoder' => $vendorDir . '/rmccue/requests/src/IdnaEncoder.php',
    'WpOrg\\Requests\\Ipv6' => $vendorDir . '/rmccue/requests/src/Ipv6.php',
    'WpOrg\\Requests\\Iri' => $vendorDir . '/rmccue/requests/src/Iri.php',
    'WpOrg\\Requests\\Port' => $vendorDir . '/rmccue/requests/src/Port.php',
    'WpOrg\\Requests\\Proxy' => $vendorDir . '/rmccue/requests/src/Proxy.php',
    'WpOrg\\Requests\\Proxy\\Http' => $vendorDir . '/rmccue/requests/src/Proxy/Http.php',
    'WpOrg\\Requests\\Requests' => $vendorDir . '/rmccue/requests/src/Requests.php',
    'WpOrg\\Requests\\Response' => $vendorDir . '/rmccue/requests/src/Response.php',
    'WpOrg\\Requests\\Response\\Headers' => $vendorDir . '/rmccue/requests/src/Response/Headers.php',
    'WpOrg\\Requests\\Session' => $vendorDir . '/rmccue/requests/src/Session.php',
    'WpOrg\\Requests\\Ssl' => $vendorDir . '/rmccue/requests/src/Ssl.php',
    'WpOrg\\Requests\\Transport' => $vendorDir . '/rmccue/requests/src/Transport.php',
    'WpOrg\\Requests\\Transport\\Curl' => $vendorDir . '/rmccue/requests/src/Transport/Curl.php',
    'WpOrg\\Requests\\Transport\\Fsockopen' => $vendorDir . '/rmccue/requests/src/Transport/Fsockopen.php',
    'WpOrg\\Requests\\Utility\\CaseInsensitiveDictionary' => $vendorDir . '/rmccue/requests/src/Utility/CaseInsensitiveDictionary.php',
    'WpOrg\\Requests\\Utility\\FilteredIterator' => $vendorDir . '/rmccue/requests/src/Utility/FilteredIterator.php',
    'WpOrg\\Requests\\Utility\\InputValidator' => $vendorDir . '/rmccue/requests/src/Utility/InputValidator.php',
    'ZipStream\\CentralDirectoryFileHeader' => $vendorDir . '/maennchen/zipstream-php/src/CentralDirectoryFileHeader.php',
    'ZipStream\\CompressionMethod' => $vendorDir . '/maennchen/zipstream-php/src/CompressionMethod.php',
    'ZipStream\\DataDescriptor' => $vendorDir . '/maennchen/zipstream-php/src/DataDescriptor.php',
    'ZipStream\\EndOfCentralDirectory' => $vendorDir . '/maennchen/zipstream-php/src/EndOfCentralDirectory.php',
    'ZipStream\\Exception' => $vendorDir . '/maennchen/zipstream-php/src/Exception.php',
    'ZipStream\\Exception\\DosTimeOverflowException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/DosTimeOverflowException.php',
    'ZipStream\\Exception\\FileNotFoundException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/FileNotFoundException.php',
    'ZipStream\\Exception\\FileNotReadableException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/FileNotReadableException.php',
    'ZipStream\\Exception\\FileSizeIncorrectException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/FileSizeIncorrectException.php',
    'ZipStream\\Exception\\OverflowException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/OverflowException.php',
    'ZipStream\\Exception\\ResourceActionException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/ResourceActionException.php',
    'ZipStream\\Exception\\SimulationFileUnknownException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/SimulationFileUnknownException.php',
    'ZipStream\\Exception\\StreamNotReadableException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/StreamNotReadableException.php',
    'ZipStream\\Exception\\StreamNotSeekableException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/StreamNotSeekableException.php',
    'ZipStream\\File' => $vendorDir . '/maennchen/zipstream-php/src/File.php',
    'ZipStream\\GeneralPurposeBitFlag' => $vendorDir . '/maennchen/zipstream-php/src/GeneralPurposeBitFlag.php',
    'ZipStream\\LocalFileHeader' => $vendorDir . '/maennchen/zipstream-php/src/LocalFileHeader.php',
    'ZipStream\\OperationMode' => $vendorDir . '/maennchen/zipstream-php/src/OperationMode.php',
    'ZipStream\\PackField' => $vendorDir . '/maennchen/zipstream-php/src/PackField.php',
    'ZipStream\\Time' => $vendorDir . '/maennchen/zipstream-php/src/Time.php',
    'ZipStream\\Version' => $vendorDir . '/maennchen/zipstream-php/src/Version.php',
    'ZipStream\\Zip64\\DataDescriptor' => $vendorDir . '/maennchen/zipstream-php/src/Zip64/DataDescriptor.php',
    'ZipStream\\Zip64\\EndOfCentralDirectory' => $vendorDir . '/maennchen/zipstream-php/src/Zip64/EndOfCentralDirectory.php',
    'ZipStream\\Zip64\\EndOfCentralDirectoryLocator' => $vendorDir . '/maennchen/zipstream-php/src/Zip64/EndOfCentralDirectoryLocator.php',
    'ZipStream\\Zip64\\ExtendedInformationExtraField' => $vendorDir . '/maennchen/zipstream-php/src/Zip64/ExtendedInformationExtraField.php',
    'ZipStream\\ZipStream' => $vendorDir . '/maennchen/zipstream-php/src/ZipStream.php',
    'ZipStream\\Zs\\ExtendedInformationExtraField' => $vendorDir . '/maennchen/zipstream-php/src/Zs/ExtendedInformationExtraField.php',
    'net\\authorize\\api\\constants\\ANetEnvironment' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/constants/ANetEnvironment.php',
    'net\\authorize\\api\\contract\\v1\\ANetApiRequestType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ANetApiRequestType.php',
    'net\\authorize\\api\\contract\\v1\\ANetApiResponseType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ANetApiResponseType.php',
    'net\\authorize\\api\\contract\\v1\\ARBCancelSubscriptionRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBCancelSubscriptionRequest.php',
    'net\\authorize\\api\\contract\\v1\\ARBCancelSubscriptionResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBCancelSubscriptionResponse.php',
    'net\\authorize\\api\\contract\\v1\\ARBCreateSubscriptionRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBCreateSubscriptionRequest.php',
    'net\\authorize\\api\\contract\\v1\\ARBCreateSubscriptionResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBCreateSubscriptionResponse.php',
    'net\\authorize\\api\\contract\\v1\\ARBGetSubscriptionListRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBGetSubscriptionListRequest.php',
    'net\\authorize\\api\\contract\\v1\\ARBGetSubscriptionListResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBGetSubscriptionListResponse.php',
    'net\\authorize\\api\\contract\\v1\\ARBGetSubscriptionListSortingType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBGetSubscriptionListSortingType.php',
    'net\\authorize\\api\\contract\\v1\\ARBGetSubscriptionRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBGetSubscriptionRequest.php',
    'net\\authorize\\api\\contract\\v1\\ARBGetSubscriptionResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBGetSubscriptionResponse.php',
    'net\\authorize\\api\\contract\\v1\\ARBGetSubscriptionStatusRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBGetSubscriptionStatusRequest.php',
    'net\\authorize\\api\\contract\\v1\\ARBGetSubscriptionStatusResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBGetSubscriptionStatusResponse.php',
    'net\\authorize\\api\\contract\\v1\\ARBSubscriptionMaskedType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBSubscriptionMaskedType.php',
    'net\\authorize\\api\\contract\\v1\\ARBSubscriptionType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBSubscriptionType.php',
    'net\\authorize\\api\\contract\\v1\\ARBUpdateSubscriptionRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBUpdateSubscriptionRequest.php',
    'net\\authorize\\api\\contract\\v1\\ARBUpdateSubscriptionResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ARBUpdateSubscriptionResponse.php',
    'net\\authorize\\api\\contract\\v1\\ArbTransactionType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ArbTransactionType.php',
    'net\\authorize\\api\\contract\\v1\\ArrayOfSettingType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ArrayOfSettingType.php',
    'net\\authorize\\api\\contract\\v1\\AuDeleteType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/AuDeleteType.php',
    'net\\authorize\\api\\contract\\v1\\AuDetailsType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/AuDetailsType.php',
    'net\\authorize\\api\\contract\\v1\\AuResponseType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/AuResponseType.php',
    'net\\authorize\\api\\contract\\v1\\AuUpdateType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/AuUpdateType.php',
    'net\\authorize\\api\\contract\\v1\\AuthenticateTestRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/AuthenticateTestRequest.php',
    'net\\authorize\\api\\contract\\v1\\AuthenticateTestResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/AuthenticateTestResponse.php',
    'net\\authorize\\api\\contract\\v1\\AuthorizationIndicatorType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/AuthorizationIndicatorType.php',
    'net\\authorize\\api\\contract\\v1\\BankAccountMaskedType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/BankAccountMaskedType.php',
    'net\\authorize\\api\\contract\\v1\\BankAccountType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/BankAccountType.php',
    'net\\authorize\\api\\contract\\v1\\BatchDetailsType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/BatchDetailsType.php',
    'net\\authorize\\api\\contract\\v1\\BatchStatisticType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/BatchStatisticType.php',
    'net\\authorize\\api\\contract\\v1\\CardArtType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CardArtType.php',
    'net\\authorize\\api\\contract\\v1\\CcAuthenticationType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CcAuthenticationType.php',
    'net\\authorize\\api\\contract\\v1\\ContactDetailType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ContactDetailType.php',
    'net\\authorize\\api\\contract\\v1\\CreateCustomerPaymentProfileRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateCustomerPaymentProfileRequest.php',
    'net\\authorize\\api\\contract\\v1\\CreateCustomerPaymentProfileResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateCustomerPaymentProfileResponse.php',
    'net\\authorize\\api\\contract\\v1\\CreateCustomerProfileFromTransactionRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateCustomerProfileFromTransactionRequest.php',
    'net\\authorize\\api\\contract\\v1\\CreateCustomerProfileRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateCustomerProfileRequest.php',
    'net\\authorize\\api\\contract\\v1\\CreateCustomerProfileResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateCustomerProfileResponse.php',
    'net\\authorize\\api\\contract\\v1\\CreateCustomerProfileTransactionRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateCustomerProfileTransactionRequest.php',
    'net\\authorize\\api\\contract\\v1\\CreateCustomerProfileTransactionResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateCustomerProfileTransactionResponse.php',
    'net\\authorize\\api\\contract\\v1\\CreateCustomerShippingAddressRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateCustomerShippingAddressRequest.php',
    'net\\authorize\\api\\contract\\v1\\CreateCustomerShippingAddressResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateCustomerShippingAddressResponse.php',
    'net\\authorize\\api\\contract\\v1\\CreateProfileResponseType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateProfileResponseType.php',
    'net\\authorize\\api\\contract\\v1\\CreateTransactionRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateTransactionRequest.php',
    'net\\authorize\\api\\contract\\v1\\CreateTransactionResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreateTransactionResponse.php',
    'net\\authorize\\api\\contract\\v1\\CreditCardMaskedType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreditCardMaskedType.php',
    'net\\authorize\\api\\contract\\v1\\CreditCardSimpleType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreditCardSimpleType.php',
    'net\\authorize\\api\\contract\\v1\\CreditCardTrackType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreditCardTrackType.php',
    'net\\authorize\\api\\contract\\v1\\CreditCardType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CreditCardType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerAddressExType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerAddressExType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerAddressType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerAddressType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerDataType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerDataType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerPaymentProfileBaseType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerPaymentProfileBaseType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerPaymentProfileExType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerPaymentProfileExType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerPaymentProfileListItemType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerPaymentProfileListItemType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerPaymentProfileMaskedType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerPaymentProfileMaskedType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerPaymentProfileSortingType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerPaymentProfileSortingType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerPaymentProfileType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerPaymentProfileType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerProfileBaseType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerProfileBaseType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerProfileExType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerProfileExType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerProfileIdType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerProfileIdType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerProfileInfoExType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerProfileInfoExType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerProfileMaskedType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerProfileMaskedType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerProfilePaymentType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerProfilePaymentType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerProfileSummaryType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerProfileSummaryType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerProfileType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerProfileType.php',
    'net\\authorize\\api\\contract\\v1\\CustomerType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/CustomerType.php',
    'net\\authorize\\api\\contract\\v1\\DecryptPaymentDataRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DecryptPaymentDataRequest.php',
    'net\\authorize\\api\\contract\\v1\\DecryptPaymentDataResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DecryptPaymentDataResponse.php',
    'net\\authorize\\api\\contract\\v1\\DeleteCustomerPaymentProfileRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DeleteCustomerPaymentProfileRequest.php',
    'net\\authorize\\api\\contract\\v1\\DeleteCustomerPaymentProfileResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DeleteCustomerPaymentProfileResponse.php',
    'net\\authorize\\api\\contract\\v1\\DeleteCustomerProfileRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DeleteCustomerProfileRequest.php',
    'net\\authorize\\api\\contract\\v1\\DeleteCustomerProfileResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DeleteCustomerProfileResponse.php',
    'net\\authorize\\api\\contract\\v1\\DeleteCustomerShippingAddressRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DeleteCustomerShippingAddressRequest.php',
    'net\\authorize\\api\\contract\\v1\\DeleteCustomerShippingAddressResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DeleteCustomerShippingAddressResponse.php',
    'net\\authorize\\api\\contract\\v1\\DriversLicenseMaskedType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DriversLicenseMaskedType.php',
    'net\\authorize\\api\\contract\\v1\\DriversLicenseType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/DriversLicenseType.php',
    'net\\authorize\\api\\contract\\v1\\EmailSettingsType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/EmailSettingsType.php',
    'net\\authorize\\api\\contract\\v1\\EmvTagType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/EmvTagType.php',
    'net\\authorize\\api\\contract\\v1\\EncryptedTrackDataType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/EncryptedTrackDataType.php',
    'net\\authorize\\api\\contract\\v1\\EnumCollection' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/EnumCollection.php',
    'net\\authorize\\api\\contract\\v1\\ErrorResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ErrorResponse.php',
    'net\\authorize\\api\\contract\\v1\\ExtendedAmountType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ExtendedAmountType.php',
    'net\\authorize\\api\\contract\\v1\\FDSFilterType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/FDSFilterType.php',
    'net\\authorize\\api\\contract\\v1\\FingerPrintType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/FingerPrintType.php',
    'net\\authorize\\api\\contract\\v1\\FraudInformationType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/FraudInformationType.php',
    'net\\authorize\\api\\contract\\v1\\GetAUJobDetailsRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetAUJobDetailsRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetAUJobDetailsResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetAUJobDetailsResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetAUJobSummaryRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetAUJobSummaryRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetAUJobSummaryResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetAUJobSummaryResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetBatchStatisticsRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetBatchStatisticsRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetBatchStatisticsResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetBatchStatisticsResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerPaymentProfileListRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerPaymentProfileListRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerPaymentProfileListResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerPaymentProfileListResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerPaymentProfileRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerPaymentProfileRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerPaymentProfileResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerPaymentProfileResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerProfileIdsRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerProfileIdsRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerProfileIdsResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerProfileIdsResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerProfileRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerProfileRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerProfileResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerProfileResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerShippingAddressRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerShippingAddressRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetCustomerShippingAddressResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetCustomerShippingAddressResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetHostedPaymentPageRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetHostedPaymentPageRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetHostedPaymentPageResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetHostedPaymentPageResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetHostedProfilePageRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetHostedProfilePageRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetHostedProfilePageResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetHostedProfilePageResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetMerchantDetailsRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetMerchantDetailsRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetMerchantDetailsResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetMerchantDetailsResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetSettledBatchListRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetSettledBatchListRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetSettledBatchListResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetSettledBatchListResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetTransactionDetailsRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetTransactionDetailsRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetTransactionDetailsResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetTransactionDetailsResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetTransactionListForCustomerRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetTransactionListForCustomerRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetTransactionListRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetTransactionListRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetTransactionListResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetTransactionListResponse.php',
    'net\\authorize\\api\\contract\\v1\\GetUnsettledTransactionListRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetUnsettledTransactionListRequest.php',
    'net\\authorize\\api\\contract\\v1\\GetUnsettledTransactionListResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/GetUnsettledTransactionListResponse.php',
    'net\\authorize\\api\\contract\\v1\\HeldTransactionRequestType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/HeldTransactionRequestType.php',
    'net\\authorize\\api\\contract\\v1\\ImpersonationAuthenticationType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ImpersonationAuthenticationType.php',
    'net\\authorize\\api\\contract\\v1\\IsAliveRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/IsAliveRequest.php',
    'net\\authorize\\api\\contract\\v1\\IsAliveResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/IsAliveResponse.php',
    'net\\authorize\\api\\contract\\v1\\KeyBlockType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/KeyBlockType.php',
    'net\\authorize\\api\\contract\\v1\\KeyManagementSchemeType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/KeyManagementSchemeType.php',
    'net\\authorize\\api\\contract\\v1\\KeyManagementSchemeType\\DUKPTAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/KeyManagementSchemeType/DUKPTAType.php',
    'net\\authorize\\api\\contract\\v1\\KeyManagementSchemeType\\DUKPTAType\\DeviceInfoAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/KeyManagementSchemeType/DUKPTAType/DeviceInfoAType.php',
    'net\\authorize\\api\\contract\\v1\\KeyManagementSchemeType\\DUKPTAType\\EncryptedDataAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/KeyManagementSchemeType/DUKPTAType/EncryptedDataAType.php',
    'net\\authorize\\api\\contract\\v1\\KeyManagementSchemeType\\DUKPTAType\\ModeAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/KeyManagementSchemeType/DUKPTAType/ModeAType.php',
    'net\\authorize\\api\\contract\\v1\\KeyValueType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/KeyValueType.php',
    'net\\authorize\\api\\contract\\v1\\LineItemType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/LineItemType.php',
    'net\\authorize\\api\\contract\\v1\\ListOfAUDetailsType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ListOfAUDetailsType.php',
    'net\\authorize\\api\\contract\\v1\\LogoutRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/LogoutRequest.php',
    'net\\authorize\\api\\contract\\v1\\LogoutResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/LogoutResponse.php',
    'net\\authorize\\api\\contract\\v1\\MerchantAuthenticationType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/MerchantAuthenticationType.php',
    'net\\authorize\\api\\contract\\v1\\MerchantContactType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/MerchantContactType.php',
    'net\\authorize\\api\\contract\\v1\\MessagesType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/MessagesType.php',
    'net\\authorize\\api\\contract\\v1\\MessagesType\\MessageAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/MessagesType/MessageAType.php',
    'net\\authorize\\api\\contract\\v1\\MobileDeviceLoginRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/MobileDeviceLoginRequest.php',
    'net\\authorize\\api\\contract\\v1\\MobileDeviceLoginResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/MobileDeviceLoginResponse.php',
    'net\\authorize\\api\\contract\\v1\\MobileDeviceRegistrationRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/MobileDeviceRegistrationRequest.php',
    'net\\authorize\\api\\contract\\v1\\MobileDeviceRegistrationResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/MobileDeviceRegistrationResponse.php',
    'net\\authorize\\api\\contract\\v1\\MobileDeviceType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/MobileDeviceType.php',
    'net\\authorize\\api\\contract\\v1\\NameAndAddressType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/NameAndAddressType.php',
    'net\\authorize\\api\\contract\\v1\\OpaqueDataType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/OpaqueDataType.php',
    'net\\authorize\\api\\contract\\v1\\OrderExType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/OrderExType.php',
    'net\\authorize\\api\\contract\\v1\\OrderType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/OrderType.php',
    'net\\authorize\\api\\contract\\v1\\OtherTaxType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/OtherTaxType.php',
    'net\\authorize\\api\\contract\\v1\\PagingType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PagingType.php',
    'net\\authorize\\api\\contract\\v1\\PayPalType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PayPalType.php',
    'net\\authorize\\api\\contract\\v1\\PaymentDetailsType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PaymentDetailsType.php',
    'net\\authorize\\api\\contract\\v1\\PaymentEmvType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PaymentEmvType.php',
    'net\\authorize\\api\\contract\\v1\\PaymentMaskedType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PaymentMaskedType.php',
    'net\\authorize\\api\\contract\\v1\\PaymentProfileType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PaymentProfileType.php',
    'net\\authorize\\api\\contract\\v1\\PaymentScheduleType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PaymentScheduleType.php',
    'net\\authorize\\api\\contract\\v1\\PaymentScheduleType\\IntervalAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PaymentScheduleType/IntervalAType.php',
    'net\\authorize\\api\\contract\\v1\\PaymentSimpleType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PaymentSimpleType.php',
    'net\\authorize\\api\\contract\\v1\\PaymentType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PaymentType.php',
    'net\\authorize\\api\\contract\\v1\\PermissionType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/PermissionType.php',
    'net\\authorize\\api\\contract\\v1\\ProcessingOptionsType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProcessingOptionsType.php',
    'net\\authorize\\api\\contract\\v1\\ProcessorType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProcessorType.php',
    'net\\authorize\\api\\contract\\v1\\ProfileTransAmountType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProfileTransAmountType.php',
    'net\\authorize\\api\\contract\\v1\\ProfileTransAuthCaptureType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProfileTransAuthCaptureType.php',
    'net\\authorize\\api\\contract\\v1\\ProfileTransAuthOnlyType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProfileTransAuthOnlyType.php',
    'net\\authorize\\api\\contract\\v1\\ProfileTransCaptureOnlyType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProfileTransCaptureOnlyType.php',
    'net\\authorize\\api\\contract\\v1\\ProfileTransOrderType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProfileTransOrderType.php',
    'net\\authorize\\api\\contract\\v1\\ProfileTransPriorAuthCaptureType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProfileTransPriorAuthCaptureType.php',
    'net\\authorize\\api\\contract\\v1\\ProfileTransRefundType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProfileTransRefundType.php',
    'net\\authorize\\api\\contract\\v1\\ProfileTransVoidType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProfileTransVoidType.php',
    'net\\authorize\\api\\contract\\v1\\ProfileTransactionType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ProfileTransactionType.php',
    'net\\authorize\\api\\contract\\v1\\ReturnedItemType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ReturnedItemType.php',
    'net\\authorize\\api\\contract\\v1\\SecurePaymentContainerErrorType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SecurePaymentContainerErrorType.php',
    'net\\authorize\\api\\contract\\v1\\SecurePaymentContainerRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SecurePaymentContainerRequest.php',
    'net\\authorize\\api\\contract\\v1\\SecurePaymentContainerResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SecurePaymentContainerResponse.php',
    'net\\authorize\\api\\contract\\v1\\SendCustomerTransactionReceiptRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SendCustomerTransactionReceiptRequest.php',
    'net\\authorize\\api\\contract\\v1\\SendCustomerTransactionReceiptResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SendCustomerTransactionReceiptResponse.php',
    'net\\authorize\\api\\contract\\v1\\SettingType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SettingType.php',
    'net\\authorize\\api\\contract\\v1\\SolutionType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SolutionType.php',
    'net\\authorize\\api\\contract\\v1\\SubMerchantType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SubMerchantType.php',
    'net\\authorize\\api\\contract\\v1\\SubscriptionCustomerProfileType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SubscriptionCustomerProfileType.php',
    'net\\authorize\\api\\contract\\v1\\SubscriptionDetailType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SubscriptionDetailType.php',
    'net\\authorize\\api\\contract\\v1\\SubscriptionPaymentType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SubscriptionPaymentType.php',
    'net\\authorize\\api\\contract\\v1\\SubsequentAuthInformationType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/SubsequentAuthInformationType.php',
    'net\\authorize\\api\\contract\\v1\\TokenMaskedType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TokenMaskedType.php',
    'net\\authorize\\api\\contract\\v1\\TransRetailInfoType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransRetailInfoType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionDetailsType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionDetailsType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionDetailsType\\EmvDetailsAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionDetailsType/EmvDetailsAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionDetailsType\\EmvDetailsAType\\TagAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionDetailsType/EmvDetailsAType/TagAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionListSortingType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionListSortingType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionRequestType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionRequestType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionRequestType\\UserFieldsAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionRequestType/UserFieldsAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\EmvResponseAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/EmvResponseAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\EmvResponseAType\\TagsAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/EmvResponseAType/TagsAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\ErrorsAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/ErrorsAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\ErrorsAType\\ErrorAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/ErrorsAType/ErrorAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\MessagesAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/MessagesAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\MessagesAType\\MessageAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/MessagesAType/MessageAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\PrePaidCardAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/PrePaidCardAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\SecureAcceptanceAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/SecureAcceptanceAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\SplitTenderPaymentsAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/SplitTenderPaymentsAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\SplitTenderPaymentsAType\\SplitTenderPaymentAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/SplitTenderPaymentsAType/SplitTenderPaymentAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionResponseType\\UserFieldsAType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionResponseType/UserFieldsAType.php',
    'net\\authorize\\api\\contract\\v1\\TransactionSummaryType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/TransactionSummaryType.php',
    'net\\authorize\\api\\contract\\v1\\UpdateCustomerPaymentProfileRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateCustomerPaymentProfileRequest.php',
    'net\\authorize\\api\\contract\\v1\\UpdateCustomerPaymentProfileResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateCustomerPaymentProfileResponse.php',
    'net\\authorize\\api\\contract\\v1\\UpdateCustomerProfileRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateCustomerProfileRequest.php',
    'net\\authorize\\api\\contract\\v1\\UpdateCustomerProfileResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateCustomerProfileResponse.php',
    'net\\authorize\\api\\contract\\v1\\UpdateCustomerShippingAddressRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateCustomerShippingAddressRequest.php',
    'net\\authorize\\api\\contract\\v1\\UpdateCustomerShippingAddressResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateCustomerShippingAddressResponse.php',
    'net\\authorize\\api\\contract\\v1\\UpdateHeldTransactionRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateHeldTransactionRequest.php',
    'net\\authorize\\api\\contract\\v1\\UpdateHeldTransactionResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateHeldTransactionResponse.php',
    'net\\authorize\\api\\contract\\v1\\UpdateMerchantDetailsRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateMerchantDetailsRequest.php',
    'net\\authorize\\api\\contract\\v1\\UpdateMerchantDetailsResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateMerchantDetailsResponse.php',
    'net\\authorize\\api\\contract\\v1\\UpdateSplitTenderGroupRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateSplitTenderGroupRequest.php',
    'net\\authorize\\api\\contract\\v1\\UpdateSplitTenderGroupResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UpdateSplitTenderGroupResponse.php',
    'net\\authorize\\api\\contract\\v1\\UserFieldType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/UserFieldType.php',
    'net\\authorize\\api\\contract\\v1\\ValidateCustomerPaymentProfileRequest' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ValidateCustomerPaymentProfileRequest.php',
    'net\\authorize\\api\\contract\\v1\\ValidateCustomerPaymentProfileResponse' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/ValidateCustomerPaymentProfileResponse.php',
    'net\\authorize\\api\\contract\\v1\\WebCheckOutDataType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/WebCheckOutDataType.php',
    'net\\authorize\\api\\contract\\v1\\WebCheckOutDataTypeTokenType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/contract/v1/WebCheckOutDataTypeTokenType.php',
    'net\\authorize\\api\\controller\\ARBCancelSubscriptionController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/ARBCancelSubscriptionController.php',
    'net\\authorize\\api\\controller\\ARBCreateSubscriptionController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/ARBCreateSubscriptionController.php',
    'net\\authorize\\api\\controller\\ARBGetSubscriptionController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/ARBGetSubscriptionController.php',
    'net\\authorize\\api\\controller\\ARBGetSubscriptionListController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/ARBGetSubscriptionListController.php',
    'net\\authorize\\api\\controller\\ARBGetSubscriptionStatusController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/ARBGetSubscriptionStatusController.php',
    'net\\authorize\\api\\controller\\ARBUpdateSubscriptionController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/ARBUpdateSubscriptionController.php',
    'net\\authorize\\api\\controller\\AuthenticateTestController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/AuthenticateTestController.php',
    'net\\authorize\\api\\controller\\CreateCustomerPaymentProfileController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/CreateCustomerPaymentProfileController.php',
    'net\\authorize\\api\\controller\\CreateCustomerProfileController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/CreateCustomerProfileController.php',
    'net\\authorize\\api\\controller\\CreateCustomerProfileFromTransactionController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/CreateCustomerProfileFromTransactionController.php',
    'net\\authorize\\api\\controller\\CreateCustomerProfileTransactionController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/CreateCustomerProfileTransactionController.php',
    'net\\authorize\\api\\controller\\CreateCustomerShippingAddressController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/CreateCustomerShippingAddressController.php',
    'net\\authorize\\api\\controller\\CreateTransactionController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/CreateTransactionController.php',
    'net\\authorize\\api\\controller\\DecryptPaymentDataController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/DecryptPaymentDataController.php',
    'net\\authorize\\api\\controller\\DeleteCustomerPaymentProfileController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/DeleteCustomerPaymentProfileController.php',
    'net\\authorize\\api\\controller\\DeleteCustomerProfileController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/DeleteCustomerProfileController.php',
    'net\\authorize\\api\\controller\\DeleteCustomerShippingAddressController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/DeleteCustomerShippingAddressController.php',
    'net\\authorize\\api\\controller\\GetAUJobDetailsController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetAUJobDetailsController.php',
    'net\\authorize\\api\\controller\\GetAUJobSummaryController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetAUJobSummaryController.php',
    'net\\authorize\\api\\controller\\GetBatchStatisticsController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetBatchStatisticsController.php',
    'net\\authorize\\api\\controller\\GetCustomerPaymentProfileController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetCustomerPaymentProfileController.php',
    'net\\authorize\\api\\controller\\GetCustomerPaymentProfileListController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetCustomerPaymentProfileListController.php',
    'net\\authorize\\api\\controller\\GetCustomerProfileController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetCustomerProfileController.php',
    'net\\authorize\\api\\controller\\GetCustomerProfileIdsController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetCustomerProfileIdsController.php',
    'net\\authorize\\api\\controller\\GetCustomerShippingAddressController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetCustomerShippingAddressController.php',
    'net\\authorize\\api\\controller\\GetHostedPaymentPageController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetHostedPaymentPageController.php',
    'net\\authorize\\api\\controller\\GetHostedProfilePageController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetHostedProfilePageController.php',
    'net\\authorize\\api\\controller\\GetMerchantDetailsController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetMerchantDetailsController.php',
    'net\\authorize\\api\\controller\\GetSettledBatchListController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetSettledBatchListController.php',
    'net\\authorize\\api\\controller\\GetTransactionDetailsController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetTransactionDetailsController.php',
    'net\\authorize\\api\\controller\\GetTransactionListController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetTransactionListController.php',
    'net\\authorize\\api\\controller\\GetTransactionListForCustomerController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetTransactionListForCustomerController.php',
    'net\\authorize\\api\\controller\\GetUnsettledTransactionListController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/GetUnsettledTransactionListController.php',
    'net\\authorize\\api\\controller\\IsAliveController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/IsAliveController.php',
    'net\\authorize\\api\\controller\\LogoutController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/LogoutController.php',
    'net\\authorize\\api\\controller\\MobileDeviceLoginController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/MobileDeviceLoginController.php',
    'net\\authorize\\api\\controller\\MobileDeviceRegistrationController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/MobileDeviceRegistrationController.php',
    'net\\authorize\\api\\controller\\SecurePaymentContainerController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/SecurePaymentContainerController.php',
    'net\\authorize\\api\\controller\\SendCustomerTransactionReceiptController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/SendCustomerTransactionReceiptController.php',
    'net\\authorize\\api\\controller\\UpdateCustomerPaymentProfileController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/UpdateCustomerPaymentProfileController.php',
    'net\\authorize\\api\\controller\\UpdateCustomerProfileController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/UpdateCustomerProfileController.php',
    'net\\authorize\\api\\controller\\UpdateCustomerShippingAddressController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/UpdateCustomerShippingAddressController.php',
    'net\\authorize\\api\\controller\\UpdateHeldTransactionController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/UpdateHeldTransactionController.php',
    'net\\authorize\\api\\controller\\UpdateMerchantDetailsController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/UpdateMerchantDetailsController.php',
    'net\\authorize\\api\\controller\\UpdateSplitTenderGroupController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/UpdateSplitTenderGroupController.php',
    'net\\authorize\\api\\controller\\ValidateCustomerPaymentProfileController' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/ValidateCustomerPaymentProfileController.php',
    'net\\authorize\\api\\controller\\base\\ApiOperationBase' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/base/ApiOperationBase.php',
    'net\\authorize\\api\\controller\\base\\IApiOperation' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/api/controller/base/IApiOperation.php',
    'net\\authorize\\util\\ANetSensitiveFields' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/util/ANetSensitiveFields.php',
    'net\\authorize\\util\\Helpers' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/util/Helpers.php',
    'net\\authorize\\util\\HttpClient' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/util/HttpClient.php',
    'net\\authorize\\util\\Log' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/util/Log.php',
    'net\\authorize\\util\\LogFactory' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/util/LogFactory.php',
    'net\\authorize\\util\\Mapper' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/util/Mapper.php',
    'net\\authorize\\util\\MapperObj' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/util/MapperObj.php',
    'net\\authorize\\util\\SensitiveDataConfigType' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/util/SensitiveDataConfigType.php',
    'net\\authorize\\util\\SensitiveTag' => $vendorDir . '/authorizenet/authorizenet/lib/net/authorize/util/SensitiveTag.php',
    'voku\\helper\\ASCII' => $vendorDir . '/voku/portable-ascii/src/voku/helper/ASCII.php',
);
